/*
 *  File Name	    : linux/arch/arm/mach-mp200/dma.c
 *  Function	    : dmac
 *  Release Version : Ver 1.01
 *  Release Date    : 2006/03/20
 *
 *  Copyright (C) NEC Electronics Corporation 2005-2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <asm/irq.h>
#include <asm/dma.h>
#include <linux/device.h>

#include "dma.h"

/****  Prototype definition  ****/

static int dma_probe(struct device *dev);

static int dma_suspend(struct device *dev, u32 state, u32 level);
static int dma_resume(struct device *dev, u32 level);

static int dma_setup_ARM(dma_regs_t * regs, dma_addr_t src_ptr, u_int size,
			 dma_addr_t dst_ptr, u_int32_t mode);
static int dma_setup_M2P(dma_regs_t * regs, dma_addr_t src_ptr, u_int size,
			 dma_addr_t dst_ptr, u_int32_t mode);
static int dma_setup_P2M(dma_regs_t * regs, dma_addr_t src_ptr, u_int size,
			 dma_addr_t dst_ptr, u_int32_t mode);

/****  Local data definition  ****/

#ifdef CONFIG_MP201
static dma_state_pch_t dma_state_pch[MP200_DMAC_MAX_P_CHANNELS];
static dma_state_lch_t dma_state_pch0_lch[MP200_DMAC_P0_MAX_L_CHANNELS];
static dma_state_lch_t dma_state_pch1_lch[MP200_DMAC_P1_MAX_L_CHANNELS];
static dma_state_lch_t dma_state_pch2_lch[MP200_DMAC_P2_MAX_L_CHANNELS];
#else
static dma_state_pch_t dma_state_pch[MP200_DMAC_MAX_P_CHANNELS];
static dma_state_lch_t dma_state_pch0_lch[MP200_DMAC_P0_MAX_L_CHANNELS];
static dma_state_lch_t dma_state_pch2_lch[MP200_DMAC_P2_MAX_L_CHANNELS];
static dma_state_lch_t dma_state_pch3_lch[MP200_DMAC_P3_MAX_L_CHANNELS];
#endif				/* CONFIG_MP201 */

/****  Structured data definition  ****/

static struct device_driver dma_driver = {
	.name = "dma",
	.bus = &platform_bus_type,
	.probe = dma_probe,
	.suspend = dma_suspend,
	.resume = dma_resume,
};

/* for physical channel #0 */
#ifdef CONFIG_MP201
mp200_dmal_t dma_pch0_lch[MP200_DMAC_P0_MAX_L_CHANNELS] = {
	/* *name, callback, *data, defmode, secure, in_use, peid */
	{.name = "ARM M->M0",.defmode = MP200_DMAC_DEFMODE_32BIT,.secure = 0},
	{.name = "ARM M->M1",.defmode = MP200_DMAC_DEFMODE_32BIT,.secure = 0},
	{.name = "ARM M->M2",.defmode = MP200_DMAC_DEFMODE_32BIT,.secure = 0},
	{.name = "ARM M->M3",.defmode = MP200_DMAC_DEFMODE_32BIT,.secure = 1},
};
#else
mp200_dmal_t dma_pch0_lch[MP200_DMAC_P0_MAX_L_CHANNELS] = {
	{.name = "ARM M->M0",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 1,},
	{.name = "ARM M->M1",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
	{.name = "ARM M->M2",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
};
#endif				/* CONFIG_MP201 */

#ifdef CONFIG_MP201
mp200_dmal_t dma_pch1_lch[MP200_DMAC_P1_MAX_L_CHANNELS] = {
	/* *name, callback, *data, defmode, secure, in_use, peid */
	{.name = "M->P0",.defmode = MP200_DMAC_DEFMODE_8BIT,.secure = 0},	/* 8,16,32: UART */
	{.name = "M->P1",.defmode = MP200_DMAC_DEFMODE_32BIT,.secure = 0},	/* 8,16,32: SPI0 */
	{.name = "M->P2",.defmode = MP200_DMAC_DEFMODE_32BIT,.secure = 0},	/* 8,16,32: SPI1/MSP */
	{.name = "M->P3",.defmode = MP200_DMAC_DEFMODE_32BIT,.secure = 0},	/* 8,16,32: MSP */
	{.name = "M->P4",.defmode = MP200_DMAC_DEFMODE_32BIT,.secure = 0},	/* 8,16,32: PCM0 */
	{.name = "M->P5",.defmode = MP200_DMAC_DEFMODE_32BIT,.secure = 0},	/* 8,16,32: PCM1 */
	{.name = "M->P6",.defmode = MP200_DMAC_DEFMODE_16BIT,.secure = 0},	/* 16     : ASYNC */
	{.name = "M->P7",.defmode = MP200_DMAC_DEFMODE_32BIT,.secure = 1},	/* 8,16,32: SD/MMM ch7 secure layer */
};
#else
/* dummy for physical channel #1
mp200_dmal_t dma_pch1_lch[MP200_DMAC_P1_MAX_L_CHANNELS] = {
	{.name = "K6SC M->M0",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 1,},
	{.name = "K6SC M->M1",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 1,},
	{.name = "K6SC M->M2",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 1,},
	{.name = "K6SC M->M3",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 1,},
	{.name = "K6SC M->M4",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 1,},
	{.name = "K6SC M->M5",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 1,},
}; */
#endif				/* CONFIG_MP201 */

/* for physical channel #2 */
#ifdef CONFIG_MP201
mp200_dmal_t dma_pch2_lch[MP200_DMAC_P2_MAX_L_CHANNELS] = {
	/* *name, callback, *data, defmode, secure, in_use, peid, regs */
	{.name = "P->M0",.defmode = MP200_DMAC_DEFMODE_8BIT,.secure = 0},	/* UART */
	{.name = "P->M1",.defmode = MP200_DMAC_DEFMODE_32BIT,.secure = 0},	/* SPI0 */
	{.name = "P->M2",.defmode = MP200_DMAC_DEFMODE_32BIT,.secure = 0},	/* SPI1/MSP */
	{.name = "P->M3",.defmode = MP200_DMAC_DEFMODE_32BIT,.secure = 0},	/* MSP */
	{.name = "P->M4",.defmode = MP200_DMAC_DEFMODE_32BIT,.secure = 0},	/* PCM0 */
	{.name = "P->M5",.defmode = MP200_DMAC_DEFMODE_32BIT,.secure = 0},	/* PCM1 */
	{.name = "P->M6",.defmode = MP200_DMAC_DEFMODE_16BIT,.secure = 0},	/* ASYNC */
	{.name = "P->M7",.defmode = MP200_DMAC_DEFMODE_32BIT,.secure = 1},	/* SD/MMM ch7 secure layer */
};
#else
mp200_dmal_t dma_pch2_lch[MP200_DMAC_P2_MAX_L_CHANNELS] = {
	{.name = "M->P0",.defmode = MP200_DMAC_AMODE_BIT8,.secure = 0,},
	{.name = "M->P1",.defmode = MP200_DMAC_AMODE_BIT8,.secure = 0,},
	{.name = "M->P2",.defmode = MP200_DMAC_AMODE_BIT8,.secure = 0,},
	{.name = "M->P3",.defmode = MP200_DMAC_AMODE_BIT8,.secure = 0,},
	{.name = "M->P4",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
	{.name = "M->P5",.defmode = MP200_DMAC_AMODE_BIT16,.secure = 1,},
	{.name = "M->P6",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
	{.name = "M->P7",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
	{.name = "M->P8",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
	{.name = "M->P9",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
	{.name = "M->P10",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
	{.name = "M->P11",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
	{.name = "M->P12",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
	{.name = "M->P13",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
	{.name = "M->P14",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
};
#endif				/* CONFIG_MP201 */

#ifndef CONFIG_MP201
/* for physical channel #3 */
mp200_dmal_t dma_pch3_lch[MP200_DMAC_P3_MAX_L_CHANNELS] = {
	{.name = "P->M0",.defmode = MP200_DMAC_AMODE_BIT8,.secure = 0,},
	{.name = "P->M1",.defmode = MP200_DMAC_AMODE_BIT8,.secure = 0,},
	{.name = "P->M2",.defmode = MP200_DMAC_AMODE_BIT8,.secure = 0,},
	{.name = "P->M3",.defmode = MP200_DMAC_AMODE_BIT8,.secure = 0,},
	{.name = "P->M4",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
	{.name = "P->M5",.defmode = MP200_DMAC_AMODE_BIT16,.secure = 1,},
	{.name = "P->M6",.defmode = MP200_DMAC_AMODE_BIT16,.secure = 0,},
	{.name = "P->M7",.defmode = MP200_DMAC_AMODE_BIT16,.secure = 0,},
	{.name = "P->M8",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
	{.name = "P->M9",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
	{.name = "P->M10",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
	{.name = "P->M11",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
	{.name = "P->M12",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
	{.name = "P->M13",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
	{.name = "P->M14",.defmode = MP200_DMAC_AMODE_BIT32,.secure = 0,},
};
#endif				/* CONFIG_MP201 */

/* for every physical channels (4ch) */
#ifdef CONFIG_MP201
mp200_dmap_t dma_pch[MP200_DMAC_MAX_P_CHANNELS] = {
	{
	 /* control : */
	 .control = {
		     (cntsts_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x8000),
		     (cntsts_t *) IO_ADDRESS(MP200_DMAC_S0_BASE + 0x8000),
		     },
	 /* intstat : */
	 .intstat = {
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x8200),	/* for intsel. regester code is */
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x8300),	/*   00:PE, 01:CPI, 02:reserved, 03:DSP */
		     NULL,
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x8500),
		     },
	 /* intsel : */
	 .intsel = (intdest_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x8100),
	 .parbase = {		/* parbase: for normal and secure space */
		     (dma_regs_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x0000),
		     (dma_regs_t *) IO_ADDRESS(MP200_DMAC_S0_BASE + 0x0000),
		     },
	 /* lch */
	 .lch = dma_pch0_lch,
	 /* lchno */
	 .lchno = MP200_DMAC_P0_MAX_L_CHANNELS,
	 /* setup_dma */
	 .setup_dma = dma_setup_ARM,
	 },
	{
	 /* control : for normal and secure space */
	 .control = {
		     (cntsts_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x9000),
		     (cntsts_t *) IO_ADDRESS(MP200_DMAC_S0_BASE + 0x9000),
		     },
	 /* intstat: <MP211> for PE0/CPI/(rsv)/DSP     */
	 .intstat = {
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x9200),
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x9300),
		     NULL,
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x9500),
		     },
	 /* intsel: */
	 .intsel = (intdest_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x9100),
	 /* parbase: for normal and secure space */
	 .parbase = {
		     (dma_regs_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x1000),	/* M2P.parameter base address */
		     (dma_regs_t *) IO_ADDRESS(MP200_DMAC_S0_BASE + 0x1000),
		     },
	 /* lch: */
	 .lch = dma_pch1_lch,
	 /* lchno: */
	 .lchno = MP200_DMAC_P1_MAX_L_CHANNELS,
	 /* setup_dma */
	 .setup_dma = dma_setup_M2P,
	 },
	{
	 /* control: for normal and secure space */
	 .control = {
		     (cntsts_t *) IO_ADDRESS(MP200_DMAC_BASE + 0xa000),
		     (cntsts_t *) IO_ADDRESS(MP200_DMAC_S0_BASE + 0xa000),
		     },
	 /* intstat: <MP211> for PE0/CPI/(rsv)/DSP     */
	 .intstat = {
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0xa200),
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0xa300),
		     NULL,
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0xa500),
		     },
	 /* intsel: */
	 .intsel = (intdest_t *) IO_ADDRESS(MP200_DMAC_BASE + 0xa100),
	 /* parbase: for normal and secure space */
	 .parbase = {
		     (dma_regs_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x2000),	/* P2M.paramter base address */
		     (dma_regs_t *) IO_ADDRESS(MP200_DMAC_S0_BASE + 0x2000),
		     },
	 /* lch: */
	 .lch = dma_pch2_lch,
	 /* lchno: */
	 .lchno = MP200_DMAC_P2_MAX_L_CHANNELS,
	 /* setup_dma */
	 .setup_dma = dma_setup_P2M,
	 },
};
#else
mp200_dmap_t dma_pch[MP200_DMAC_MAX_P_CHANNELS] = {
	{			/* PCH #0 */
	 .control = {		/* for normal and secure space */
		     (cntsts_t *) IO_ADDRESS(MP200_DMAC_BASE),
		     (cntsts_t *) IO_ADDRESS(MP200_DMAC_S0_BASE),
		     },
	 .intstat = {		/* for PE0/PE1/PE2/DSP */
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x0100),
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x0114),
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x0128),
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x013c),
		     },
	 .intstat2 = {		/* dummy for PE0/PE1/PE2/DSP (LCH12-14) */
		      (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x0100),
		      (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x0114),
		      (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x0128),
		      (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x013c),
		      },
	 .intsel = (intdest_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x0150),
	 .parbase = {		/* for normal and secure space */
		     (dma_regs_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x1000),
		     (dma_regs_t *) IO_ADDRESS(MP200_DMAC_S0_BASE + 0x1000),
		     },
	 .lch = dma_pch0_lch,
	 .lchno = MP200_DMAC_P0_MAX_L_CHANNELS,
	 .setup_dma = dma_setup_ARM,
	 },
	{			/* dummy entry for PCH #1 */
	 .control = {		/* for normal and secure space */
		     (cntsts_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x2000),
		     (cntsts_t *) IO_ADDRESS(MP200_DMAC_S0_BASE + 0x2000),
		     },
	 .intstat = {		/* for PE0/PE1/PE2/DSP */
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x2100),
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x2128),
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x2150),
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x2178),
		     },
	 .intstat2 = {		/* dummy for PE0/PE1/PE2/DSP (LCH12-14) */
		      (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x2100),
		      (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x2128),
		      (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x2150),
		      (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x2178),
		      },
	 .intsel = (intdest_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x2200),
	 .parbase = {		/* for normal and secure space */
		     (dma_regs_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x3000),
		     (dma_regs_t *) IO_ADDRESS(MP200_DMAC_S0_BASE + 0x3000),
		     },
	 .lch = NULL,
	 .lchno = 0,
	 .setup_dma = NULL,
	 },
	{			/* PCH #2 */
	 .control = {		/* for normal and secure space */
		     (cntsts_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x4000),
		     (cntsts_t *) IO_ADDRESS(MP200_DMAC_S0_BASE + 0x4000),
		     },
	 .intstat = {		/* for PE0/PE1/PE2/DSP */
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x4100),
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x413c),
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x4178),
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x41b4),
		     },
	 .intstat2 = {		/* for PE0/PE1/PE2/DSP (LCH12-14) */
		      (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x41f4),
		      (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x4208),
		      (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x421c),
		      (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x4230),
		      },
	 .intsel = (intdest_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x41f0),
	 .parbase = {		/* for normal and secure space */
		     (dma_regs_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x5000),
		     (dma_regs_t *) IO_ADDRESS(MP200_DMAC_S0_BASE + 0x5000),
		     },
	 .lch = dma_pch2_lch,
	 .lchno = MP200_DMAC_P2_MAX_L_CHANNELS,
	 .setup_dma = dma_setup_M2P,
	 },
	{			/* PCH #3 */
	 .control = {		/* for normal and secure space */
		     (cntsts_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x6000),
		     (cntsts_t *) IO_ADDRESS(MP200_DMAC_S0_BASE + 0x6000),
		     },
	 .intstat = {		/* for PE0/PE1/PE2/DSP */
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x6100),
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x613c),
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x6178),
		     (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x61b4),
		     },
	 .intstat2 = {		/* for PE0/PE1/PE2/DSP (LCH12-14) */
		      (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x61f4),
		      (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x6208),
		      (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x621c),
		      (intparm_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x6230),
		      },
	 .intsel = (intdest_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x61f0),
	 .parbase = {		/* for normal and secure space */
		     (dma_regs_t *) IO_ADDRESS(MP200_DMAC_BASE + 0x7000),
		     (dma_regs_t *) IO_ADDRESS(MP200_DMAC_S0_BASE + 0x7000),
		     },
	 .lch = dma_pch3_lch,
	 .lchno = MP200_DMAC_P3_MAX_L_CHANNELS,
	 .setup_dma = dma_setup_P2M,
	 },
};
#endif				/* CONFIG_MP201 */

/****  Internal functions  ****/

static void dma_save_state(void)
{
	dma_regs_t *regs;
	int i;
	int j;
	int k;
	intparm_t *intstat;

	for (i = 0; i < MP200_DMAC_MAX_P_CHANNELS; i++) {
#ifndef CONFIG_MP201
		if (i == 1) {
			continue;
		}
#endif				/* CONFIG_MP201 */
		for (j = 0; j < 3; j++) {
#ifdef CONFIG_MP201
			if (j == 2) {
				continue;
			}
#endif				/* CONFIG_MP201 */
			for (k = 0; k < ((dma_pch[i].lchno - 1) / 4 + 1); k++) {
#ifdef CONFIG_MP201
				intstat = dma_pch[i].intstat[j] + k;
#else
				if (k < (MP200_DMAC_SPLIT_CHNO / 4)) {
					intstat = dma_pch[i].intstat[j] + k;
				} else {
					intstat = dma_pch[i].intstat2[j] + k -
					    (MP200_DMAC_SPLIT_CHNO / 4);
				}
#endif				/* CONFIG_MP201 */
				dma_state_pch[i].enable[j][k] = intstat->enable;
			}
		}
		dma_state_pch[i].intdest = dma_pch[i].intsel->intdest;
		switch (i) {
		case 0:
			for (j = 0; j < MP200_DMAC_P0_MAX_L_CHANNELS; j++) {
				regs =
				    dma_pch[i].parbase[dma_pch0_lch[j].secure] +
				    j;
				dma_state_pch0_lch[j].aadd = regs->aadd;
				dma_state_pch0_lch[j].aoff = regs->aoff;
				dma_state_pch0_lch[j].asize_count =
				    regs->asize_count;
				dma_state_pch0_lch[j].badd = regs->badd;
				dma_state_pch0_lch[j].boff = regs->boff;
				dma_state_pch0_lch[j].bsize_count =
				    regs->bsize_count;
				dma_state_pch0_lch[j].leng = regs->leng;
				dma_state_pch0_lch[j].size = regs->size;
				dma_state_pch0_lch[j].mode = regs->mode;
			}
			break;
#ifdef CONFIG_MP201
		case 1:
			for (j = 0; j < MP200_DMAC_P1_MAX_L_CHANNELS; j++) {
				regs =
				    dma_pch[i].parbase[dma_pch1_lch[j].secure] +
				    j;
				dma_state_pch1_lch[j].aadd = regs->aadd;
				dma_state_pch1_lch[j].aoff = regs->aoff;
				dma_state_pch1_lch[j].asize = regs->asize;
				dma_state_pch1_lch[j].asize_count =
				    regs->asize_count;
				dma_state_pch1_lch[j].badd = regs->badd;
				dma_state_pch1_lch[j].leng = regs->leng;
				dma_state_pch1_lch[j].mode = regs->mode;
				dma_state_pch1_lch[j].time = regs->time;
			}
			break;
#endif				/* CONFIG_MP201 */
		case 2:
#ifdef CONFIG_MP201
			for (j = 0; j < MP200_DMAC_P2_MAX_L_CHANNELS; j++) {
				regs =
				    dma_pch[i].parbase[dma_pch2_lch[j].secure] +
				    j;
				dma_state_pch2_lch[j].aadd = regs->aadd;
				dma_state_pch2_lch[j].badd = regs->badd;
				dma_state_pch2_lch[j].boff = regs->boff;
				dma_state_pch2_lch[j].bsize = regs->bsize;
				dma_state_pch2_lch[j].bsize_count =
				    regs->bsize_count;
				dma_state_pch2_lch[j].leng = regs->leng;
				dma_state_pch2_lch[j].mode = regs->mode;
				dma_state_pch2_lch[j].time = regs->time;
			}
#else
			for (j = 0; j < MP200_DMAC_P2_MAX_L_CHANNELS; j++) {
				regs =
				    dma_pch[i].parbase[dma_pch2_lch[j].secure] +
				    j;
				dma_state_pch2_lch[j].aadd = regs->aadd;
				dma_state_pch2_lch[j].aoff = regs->aoff;
				dma_state_pch2_lch[j].asize = regs->asize;
				dma_state_pch2_lch[j].asize_count =
				    regs->asize_count;
				dma_state_pch2_lch[j].badd = regs->badd;
				dma_state_pch2_lch[j].leng = regs->leng;
				dma_state_pch2_lch[j].mode = regs->mode;
				dma_state_pch2_lch[j].time = regs->time;
			}
#endif				/* CONFIG_MP201 */
			break;
#ifndef CONFIG_MP201
		case 3:
			for (j = 0; j < MP200_DMAC_P3_MAX_L_CHANNELS; j++) {
				regs =
				    dma_pch[i].parbase[dma_pch3_lch[j].secure] +
				    j;
				dma_state_pch3_lch[j].aadd = regs->aadd;
				dma_state_pch3_lch[j].badd = regs->badd;
				dma_state_pch3_lch[j].boff = regs->boff;
				dma_state_pch3_lch[j].bsize = regs->bsize;
				dma_state_pch3_lch[j].bsize_count =
				    regs->bsize_count;
				dma_state_pch3_lch[j].leng = regs->leng;
				dma_state_pch3_lch[j].mode = regs->mode;
				dma_state_pch3_lch[j].time = regs->time;
			}
			break;
#endif				/* CONFIG_MP201 */
		default:
			break;
		}
	}
}

static void dma_restore_state(void)
{
	dma_regs_t *regs;
	int i;
	int j;
	int k;
	intparm_t *intstat;

	for (i = 0; i < MP200_DMAC_MAX_P_CHANNELS; i++) {
#ifndef CONFIG_MP201
		if (i == 1) {
			continue;
		}
#endif				/* CONFIG_MP201 */
		for (j = 0; j < 3; j++) {
#ifdef CONFIG_MP201
			if (j == 2) {
				continue;
			}
#endif				/* CONFIG_MP201 */
			for (k = 0; k < ((dma_pch[i].lchno - 1) / 4 + 1); k++) {
#ifdef CONFIG_MP201
				intstat = dma_pch[i].intstat[j] + k;
#else
				if (k < (MP200_DMAC_SPLIT_CHNO / 4)) {
					intstat = dma_pch[i].intstat[j] + k;
				} else {
					intstat = dma_pch[i].intstat2[j] + k -
					    (MP200_DMAC_SPLIT_CHNO / 4);
				}
#endif				/* CONFIG_MP201 */
				intstat->enable = dma_state_pch[i].enable[j][k];
			}
		}
		dma_pch[i].intsel->intdest = dma_state_pch[i].intdest;
		switch (i) {
		case 0:
			for (j = 0; j < MP200_DMAC_P0_MAX_L_CHANNELS; j++) {
				regs =
				    dma_pch[i].parbase[dma_pch0_lch[j].secure] +
				    j;
				regs->aadd = dma_state_pch0_lch[j].aadd;
				regs->aoff = dma_state_pch0_lch[j].aoff;
				regs->asize_count =
				    dma_state_pch0_lch[j].asize_count;
				regs->badd = dma_state_pch0_lch[j].badd;
				regs->boff = dma_state_pch0_lch[j].boff;
				regs->bsize_count =
				    dma_state_pch0_lch[j].bsize_count;
				regs->leng = dma_state_pch0_lch[j].leng;
				regs->size = dma_state_pch0_lch[j].size;
				regs->mode = dma_state_pch0_lch[j].mode;
			}
			break;
#ifdef CONFIG_MP201
		case 1:
			for (j = 0; j < MP200_DMAC_P1_MAX_L_CHANNELS; j++) {
				regs =
				    dma_pch[i].parbase[dma_pch1_lch[j].secure] +
				    j;
				regs->aadd = dma_state_pch1_lch[j].aadd;
				regs->aoff = dma_state_pch1_lch[j].aoff;
				regs->asize = dma_state_pch1_lch[j].asize;
				regs->asize_count =
				    dma_state_pch1_lch[j].asize_count;
				regs->badd = dma_state_pch1_lch[j].badd;
				regs->leng = dma_state_pch1_lch[j].leng;
				regs->mode = dma_state_pch1_lch[j].mode;
				regs->time = dma_state_pch1_lch[j].time;
			}
			break;
#endif				/* CONFIG_MP201 */
		case 2:
#ifdef CONFIG_MP201
			for (j = 0; j < MP200_DMAC_P2_MAX_L_CHANNELS; j++) {
				regs =
				    dma_pch[i].parbase[dma_pch2_lch[j].secure] +
				    j;
				regs->aadd = dma_state_pch2_lch[j].aadd;
				regs->badd = dma_state_pch2_lch[j].badd;
				regs->boff = dma_state_pch2_lch[j].boff;
				regs->bsize = dma_state_pch2_lch[j].bsize;
				regs->bsize_count =
				    dma_state_pch2_lch[j].bsize_count;
				regs->leng = dma_state_pch2_lch[j].leng;
				regs->mode = dma_state_pch2_lch[j].mode;
				regs->time = dma_state_pch2_lch[j].time;
			}
#else
			for (j = 0; j < MP200_DMAC_P2_MAX_L_CHANNELS; j++) {
				regs =
				    dma_pch[i].parbase[dma_pch2_lch[j].secure] +
				    j;
				regs->aadd = dma_state_pch2_lch[j].aadd;
				regs->aoff = dma_state_pch2_lch[j].aoff;
				regs->asize = dma_state_pch2_lch[j].asize;
				regs->asize_count =
				    dma_state_pch2_lch[j].asize_count;
				regs->badd = dma_state_pch2_lch[j].badd;
				regs->leng = dma_state_pch2_lch[j].leng;
				regs->mode = dma_state_pch2_lch[j].mode;
				regs->time = dma_state_pch2_lch[j].time;
			}
#endif				/* CONFIG_MP201 */
			break;
#ifndef CONFIG_MP201
		case 3:
			for (j = 0; j < MP200_DMAC_P3_MAX_L_CHANNELS; j++) {
				regs =
				    dma_pch[i].parbase[dma_pch3_lch[j].secure] +
				    j;
				regs->aadd = dma_state_pch3_lch[j].aadd;
				regs->badd = dma_state_pch3_lch[j].badd;
				regs->boff = dma_state_pch3_lch[j].boff;
				regs->bsize = dma_state_pch3_lch[j].bsize;
				regs->bsize_count =
				    dma_state_pch3_lch[j].bsize_count;
				regs->leng = dma_state_pch3_lch[j].leng;
				regs->mode = dma_state_pch3_lch[j].mode;
				regs->time = dma_state_pch3_lch[j].time;
			}
			break;
#endif				/* CONFIG_MP201 */
		default:
			break;
		}
	}
}

static int dma_suspend(struct device *dev, u32 state, u32 level)
{
#ifdef DMA_DEBUG
	printk(KERN_INFO "%s(): bus_id=%s state=%d level=%d\n",
	       __FUNCTION__, dev->bus_id, state, level);
#endif

	switch (level) {
	case SUSPEND_SAVE_STATE:
		dma_save_state();
		break;
	default:
		break;
	}

	return 0;
}

static int dma_resume(struct device *dev, u32 level)
{
	switch (level) {
	case RESUME_RESTORE_STATE:
		dma_restore_state();
		break;
	default:
		break;
	}

#ifdef DMA_DEBUG
	printk(KERN_INFO "%s(): bus_id=%s level=%d\n",
	       __FUNCTION__, dev->bus_id, level);
#endif

	return 0;
}

/*
 * Check channel number
 */
static inline int dma_check_validity_channel(int channel)
{
	unsigned int pchno;
	unsigned int lchno;

	pchno = MP200_DMAC_PCHNO(channel);
	lchno = MP200_DMAC_LCHNO(channel);

#ifdef CONFIG_MP201
	if ((channel & ~MP200_DMAC_MASK_CHNO) == 0) {
		if (((pchno == 0) && (lchno < MP200_DMAC_P0_MAX_L_CHANNELS))
		    || ((pchno == 1) && (lchno < MP200_DMAC_P1_MAX_L_CHANNELS))
		    || ((pchno == 2) && (lchno < MP200_DMAC_P2_MAX_L_CHANNELS))) {
			return 0;
		}
	}
#else
	if ((channel & ~MP200_DMAC_MASK_CHNO) == 0) {
		if (((pchno == 0) && (lchno < MP200_DMAC_P0_MAX_L_CHANNELS))
		    || ((pchno == 1) && (lchno < MP200_DMAC_P1_MAX_L_CHANNELS))
		    || ((pchno == 2) && (lchno < MP200_DMAC_P2_MAX_L_CHANNELS))
		    || ((pchno == 3) && (lchno < MP200_DMAC_P3_MAX_L_CHANNELS))) {
			return 0;
		}
	}
#endif				/* CONFIG_MP201 */

	return 1;
}

/*
 * Functions for default settings.
 *
 * Each function sets DMA up to appropriate default setting when
 * `size' is not 0. Skip parameter settings except source and destination
 * addresses when `size' is 0 so that client driver can use special mode
 * like repeat transfer, timeout, etc.
 */

/* PCH #0 */
static int dma_setup_ARM(dma_regs_t * regs, dma_addr_t src_ptr, u_int size,
			 dma_addr_t dst_ptr, u_int32_t mode)
{
	if (((src_ptr & ~MP200_DMAC_MASK_ADDRESS) != 0)
	    || ((dst_ptr & ~MP200_DMAC_MASK_ADDRESS) != 0)) {
		return 1;
	}
	if (((size & ~MP200_DMAC_MASK_BLKSIZE) != 0)
	    || ((size & ~MP200_DMAC_MASK_LENGTH) != 0)) {
		return 1;
	}
	regs->aadd = src_ptr;
	regs->badd = dst_ptr;
	if (size != 0) {
		regs->aoff = 0;
		regs->boff = 0;
		regs->asize_count = 0;
		regs->bsize_count = 0;
		regs->leng = size;
		regs->size = size;
		regs->mode = mode;
	}

	return 0;
}

/* PCH #2 */
static int dma_setup_M2P(dma_regs_t * regs, dma_addr_t src_ptr, u_int size,
			 dma_addr_t dst_ptr, u_int32_t mode)
{
	if (((src_ptr & ~MP200_DMAC_MASK_ADDRESS) != 0)
	    || ((dst_ptr & ~MP200_DMAC_MASK_ADDRESS) != 0)) {
		return 1;
	}
	if (((size & ~MP200_DMAC_MASK_BLKSIZE) != 0)
	    || ((size & ~MP200_DMAC_MASK_LENGTH) != 0)) {
		return 1;
	}
	regs->aadd = src_ptr;
	regs->badd = dst_ptr;
	if (size != 0) {
		regs->aoff = 0;
		regs->asize_count = 0;
		regs->asize = size;
		regs->leng = size;
		regs->mode = mode;
	}

	return 0;
}

/* PCH #3 */
static int dma_setup_P2M(dma_regs_t * regs, dma_addr_t src_ptr, u_int size,
			 dma_addr_t dst_ptr, u_int32_t mode)
{
	if (((src_ptr & ~MP200_DMAC_MASK_ADDRESS) != 0)
	    || ((dst_ptr & ~MP200_DMAC_MASK_ADDRESS) != 0)) {
		return 1;
	}
	if (((size & ~MP200_DMAC_MASK_BLKSIZE) != 0)
	    || ((size & ~MP200_DMAC_MASK_LENGTH) != 0)) {
		return 1;
	}
	regs->aadd = src_ptr;
	regs->badd = dst_ptr;
	if (size != 0) {
		regs->boff = 0;
		regs->bsize_count = 0;
		regs->bsize = size;
		regs->leng = size;
		regs->mode = mode;
	}

	return 0;
}

/*
 * Interrupt Handler
 */
static irqreturn_t
dma_irq_handler(int irq, void *dev_id, struct pt_regs *intregs)
{
	mp200_dmap_t *pch;
	mp200_dmal_t *lch;
	dma_regs_t *regs = dev_id;
	int chno;
	int stshift;
	intparm_t *intcnt;
	unsigned int sts;
	unsigned int rawsts;
	int err;

	/*
	 * We can compute channel number from address of control registers
	 */
	chno = MP200_DMAC_PARBASE2CHNO(regs);
	err = dma_check_validity_channel(chno);
	if (err != 0) {
		return IRQ_NONE;
	}
	pch = &dma_pch[MP200_DMAC_PCHNO(chno)];

	/*
	 * Check interrupt status.  If it's not for this channel then return.
	 *
	 * Statuses for 4 logical channels are packed into 1 interrupt
	 * status register. I compute shift count so that I can get
	 * status field for my channel easily.
	 */
#ifdef CONFIG_MP201
	intcnt = pch->intstat[0] + MP200_DMAC_LCHNO(chno) / 4;
#else
	if (MP200_DMAC_LCHNO(chno) < MP200_DMAC_SPLIT_CHNO) {
		intcnt = pch->intstat[0] + MP200_DMAC_LCHNO(chno) / 4;
	} else {
		intcnt = pch->intstat2[0];
		intcnt += (MP200_DMAC_LCHNO(chno) - MP200_DMAC_SPLIT_CHNO) / 4;
	}
#endif				/* CONFIG_MP201 */
	stshift = (MP200_DMAC_LCHNO(chno) % 4) * 8;
	sts = (intcnt->stat >> stshift) & 0xff;
	if (sts == 0) {
		return IRQ_NONE;	/* there is no interrupt */
	}

	rawsts = (intcnt->raw_stat >> stshift) & 0xff;

	/* clear interrupt status */
	intcnt->clear = sts << stshift;

	/* call client driver's interrupt handling routine */
	lch = pch->lch + MP200_DMAC_LCHNO(chno);
	lch->callback(lch->data, sts, rawsts);

	return IRQ_HANDLED;
}

/****  External functions  ****/

/*
 * Acquire DMA channel
 *   mp200_request_dma()
 */
int
mp200_request_dma(int channel, const char *device_id,
		  dma_callback_t callback, void *data, dma_regs_t ** dma_regs)
{
	mp200_dmap_t *pch;
	mp200_dmal_t *lch;
	dma_regs_t *regs;
	unsigned int intdest;
	int err;

	err = dma_check_validity_channel(channel);
	if (err != 0) {
		return -ENODEV;
	}
	if (dma_regs == NULL) {
		return -EINVAL;
	}

	pch = &dma_pch[MP200_DMAC_PCHNO(channel)];
	if (pch->lch == NULL) {
		return -ENODEV;
	}
	lch = pch->lch + MP200_DMAC_LCHNO(channel);
	if (lch->in_use != DMA_NO_USE) {
		return -EBUSY;
	}
	regs = pch->parbase[lch->secure] + MP200_DMAC_LCHNO(channel);

	/*
	 * Stop desired DMA channel to make it sure before
	 * interrupt related settings.
	 */
	mp200_clear_dma(channel);

	/*
	 * Register interrupt handler for appropriate INTNO that is
	 * determined by processor element number (PE No.) which executing
	 * this routine.
	 */
#ifdef CONFIG_MP201
	err = request_irq(INT_DMACPU, dma_irq_handler,
			  (SA_SHIRQ | SA_INTERRUPT), device_id, regs);
	if (err != 0) {
		printk(KERN_INFO
		       "%s(): unable to request IRQ %d for DMA channel (%s)\n",
		       __FUNCTION__, INT_DMACPU, device_id);
		return err;
	}
#else
	err = request_irq(INT_DMAPE0, dma_irq_handler,
			  (SA_SHIRQ | SA_INTERRUPT), device_id, regs);
	if (err != 0) {
		printk(KERN_INFO
		       "%s(): unable to request IRQ %d for DMA channel (%s)\n",
		       __FUNCTION__, INT_DMAPE0, device_id);
		return err;
	}
#endif				/* CONFIG_MP201 */

	/*
	 * Set interrupt output line to the PE which executing this routine.
	 * XXX needs mutual exclusion between PE's
	 */
	intdest = pch->intsel->intdest;
	intdest &= ~(3 << (MP200_DMAC_LCHNO(channel) * 2));
	pch->intsel->intdest = intdest;

	/*
	 * RESET does not initialize some DMAC registers.
	 * Set DMA channel to known state (for single block transfer) here
	 * so that stabilizing phenomena caused by client code
	 * lacks register setting (bug).
	 * Use cold start address where most likely boot ROM exists.
	 */
	pch->setup_dma(regs, 0x0, 0x1000, 0x0, lch->defmode);

	/*
	 * Hold information for further DMAC operations.
	 */
	lch->in_use = DMA_USE;
	lch->callback = callback;
	lch->data = data;

	/*
	 * return pointer to control registers.
	 */
	*dma_regs = regs;

	return 0;
}

/*
 * Release DMA channel
 */
void mp200_free_dma(int channel)
{
	mp200_dmap_t *pch;
	mp200_dmal_t *lch;
	dma_regs_t *regs;
	int err;

	err = dma_check_validity_channel(channel);
	if (err != 0) {
		return;
	}

	pch = &dma_pch[MP200_DMAC_PCHNO(channel)];
	if (pch->lch == NULL) {
		return;
	}
	lch = pch->lch + MP200_DMAC_LCHNO(channel);
	if (lch->in_use != DMA_USE) {
		return;
	}

	/*
	 * Stop DMA and release interrupt
	 */
	mp200_clear_dma(channel);
	regs = pch->parbase[lch->secure] + MP200_DMAC_LCHNO(channel);
#ifdef CONFIG_MP201
	free_irq(INT_DMACPU, regs);
#else
	free_irq(INT_DMAPE0, regs);
#endif				/* CONFIG_MP201 */

	lch->in_use = DMA_NO_USE;
	lch->callback = NULL;
	lch->data = NULL;

	return;
}

/*
 * Setup and enable DMA.
 */
int mp200_start_dma(int channel, dma_addr_t src_ptr, u_int size,
		    dma_addr_t dst_ptr, int intmask)
{
	mp200_dmap_t *pch;
	mp200_dmal_t *lch;
	dma_regs_t *regs;
	int stshift;
	intparm_t *intcnt;
	cntsts_t *base;
	int err;

	err = dma_check_validity_channel(channel);
	if (err != 0) {
		return -ENODEV;
	}
	if ((intmask & ~MP200_DMAC_MASK_INT) != 0) {
		return -EINVAL;
	}

	pch = &dma_pch[MP200_DMAC_PCHNO(channel)];
	if (pch->lch == NULL) {
		return -ENODEV;
	}
	lch = pch->lch + MP200_DMAC_LCHNO(channel);
	if (lch->in_use != DMA_USE) {
		return -ENXIO;
	}

	if ((intmask != 0) && (lch->callback == NULL)) {
		return -EINVAL;
	}

	/*
	 * Set DMA channel up to appropriate default setting.
	 */
	regs = pch->parbase[lch->secure] + MP200_DMAC_LCHNO(channel);
	err = pch->setup_dma(regs, src_ptr, size, dst_ptr, lch->defmode);
	if (err != 0) {
		return -EINVAL;
	}

	/*
	 * Enable interrupt.
	 * At first, disable all interrupt, then enable desired interrupt.
	 */
#ifdef CONFIG_MP201
	intcnt = pch->intstat[0] + MP200_DMAC_LCHNO(channel) / 4;
#else
	if (MP200_DMAC_LCHNO(channel) < MP200_DMAC_SPLIT_CHNO) {
		intcnt = pch->intstat[0] + MP200_DMAC_LCHNO(channel) / 4;
	} else {
		intcnt = pch->intstat2[0];
		intcnt +=
		    (MP200_DMAC_LCHNO(channel) - MP200_DMAC_SPLIT_CHNO) / 4;
	}
#endif				/* CONFIG_MP201 */
	stshift = (MP200_DMAC_LCHNO(channel) % 4) * 8;
	intcnt->disable = 0xff << stshift;
	intcnt->clear = 0xff << stshift;
	intcnt->enable = (unsigned char)intmask << stshift;

	/*
	 * Enable DMA.
	 */
	base = pch->control[lch->secure];
	base->start = 1 << MP200_DMAC_LCHNO(channel);

	return 0;
}

/*
 * Acquire DMA channel status.
 */
int mp200_dma_status(int channel)
{
	mp200_dmap_t *pch;
	mp200_dmal_t *lch;
	int stat;
	cntsts_t *base;
	int err;

	err = dma_check_validity_channel(channel);
	if (err != 0) {
		return -ENODEV;
	}

	pch = &dma_pch[MP200_DMAC_PCHNO(channel)];
	if (pch->lch == NULL) {
		return -ENODEV;
	}
	lch = pch->lch + MP200_DMAC_LCHNO(channel);
	/* does not check `in_use'. can read status of any DMA channel */

	/*
	 * Read status register, return corresponding bit.
	 */
	base = pch->control[lch->secure];
	stat = (base->status >> MP200_DMAC_LCHNO(channel)) & 1;

	return stat;
}

/*
 * Acquire DMA channel status (register writable flag).
 */
int mp200_dma_writable_flag(int channel)
{
	mp200_dmap_t *pch;
	mp200_dmal_t *lch;
	int stat;
	cntsts_t *base;
	int err;

	err = dma_check_validity_channel(channel);
	if (err != 0) {
		return -ENODEV;
	}
#ifdef CONFIG_MP201
	if (MP200_DMAC_PCHNO(channel) == 0) {
		return -ENODEV;
	}
#else
	if ((MP200_DMAC_PCHNO(channel) == 0)
	    || (MP200_DMAC_PCHNO(channel) == 1)) {
		return -ENODEV;
	}
#endif				/* CONFIG_MP201 */

	pch = &dma_pch[MP200_DMAC_PCHNO(channel)];
	if (pch->lch == NULL) {
		return -ENODEV;
	}
	lch = pch->lch + MP200_DMAC_LCHNO(channel);
	/* does not check `in_use'. can read status of any DMA channel */

	/*
	 * Read status register, return corresponding bit.
	 */
	base = pch->control[lch->secure];
	stat = (base->status >> (16 + MP200_DMAC_LCHNO(channel))) & 1;

	return stat;
}

/*
 * Acquire all DMA channel status.
 * check all DMA channels (except channels for K6SC),
 * return 1 if there is enabled channel.
 * Don't start DMA channel while calling this function.
 */
int mp200_dma_busy(void)
{
	int p;
	int l;
	cntsts_t *base;
	mp200_dmal_t *lch;

	for (p = 0; p < MP200_DMAC_MAX_P_CHANNELS; p++) {
#ifndef CONFIG_MP201
		if (p == 1) {	/* skip PCH #1 */
			continue;
		}
#endif				/* CONFIG_MP201 */
		for (l = 0; l < dma_pch[p].lchno; l++) {
			lch = dma_pch[p].lch + l;
			if (lch->in_use == DMA_USE) {
				base = dma_pch[p].control[lch->secure];
				if (((base->status >> l) & 1) != 0) {
					return 1;
				}
			}
		}
	}

	return 0;
}

/*
 * Stop DMA channel.
 */
void mp200_clear_dma(int channel)
{
	mp200_dmap_t *pch;
	mp200_dmal_t *lch;
	cntsts_t *base;
	intparm_t *intcnt;
	int stshift;
	int err;

	err = dma_check_validity_channel(channel);
	if (err != 0) {
		return;
	}

	pch = &dma_pch[MP200_DMAC_PCHNO(channel)];
	if (pch->lch == NULL) {
		return;
	}
	lch = pch->lch + MP200_DMAC_LCHNO(channel);
	if (lch->in_use != DMA_USE) {
		return;
	}

	/*
	 * disable all interrupt
	 */
#ifdef CONFIG_MP201
	intcnt = pch->intstat[0] + MP200_DMAC_LCHNO(channel) / 4;
#else
	if (MP200_DMAC_LCHNO(channel) < MP200_DMAC_SPLIT_CHNO) {
		intcnt = pch->intstat[0] + MP200_DMAC_LCHNO(channel) / 4;
	} else {
		intcnt = pch->intstat2[0];
		intcnt +=
		    (MP200_DMAC_LCHNO(channel) - MP200_DMAC_SPLIT_CHNO) / 4;
	}
#endif				/* CONFIG_MP201 */
	stshift = (MP200_DMAC_LCHNO(channel) % 4) * 8;
	intcnt->disable = 0xff << stshift;
	intcnt->clear = 0xff << stshift;

	/*
	 * stop DMA
	 */
	base = pch->control[lch->secure];
	base->stop = 1 << MP200_DMAC_LCHNO(channel);

	return;
}

/*
 * Stop DMA channel too.
 */
void mp200_reset_dma(int channel)
{
	mp200_clear_dma(channel);

	return;
}

/*
 * Retrieve current DMA position.
 */
dma_addr_t mp200_get_dma_pos(int channel)
{
	mp200_dmap_t *pch;
	mp200_dmal_t *lch;
	dma_regs_t *regs;
	int err;

	err = dma_check_validity_channel(channel);
	if (err != 0) {
		return 0;
	}

	pch = &dma_pch[MP200_DMAC_PCHNO(channel)];
	if (pch->lch == NULL) {
		return 0;
	}
	lch = pch->lch + MP200_DMAC_LCHNO(channel);
	/* does not check `in_use'. can read position of any DMA channel */

	regs = pch->parbase[lch->secure] + MP200_DMAC_LCHNO(channel);

	switch (MP200_DMAC_PCHNO(channel)) {
	case 0:
#ifdef CONFIG_MP201
	case 1:
		return regs->aadp;
		break;
	case 2:
		return regs->badp;
		break;
#else
	case 2:
		return regs->aadp;
		break;
	case 3:
		return regs->badp;
		break;
#endif				/* CONFIG_MP201 */
	default:
		break;
	}

	return 0;
}

/*
 * Stop DMA.
 * We'd like to suspend DMA instead of stopping, if we can.
 * Unfortunately, DMAC on MP200 can't suspend, we stop DMA at this moment.
 * (can't resume)
 */
void mp200_stop_dma(int channel)
{
	mp200_dmap_t *pch;
	mp200_dmal_t *lch;
	cntsts_t *base;
	intparm_t *intcnt;
	int stshift;
	int err;

	err = dma_check_validity_channel(channel);
	if (err != 0) {
		return;
	}

	pch = &dma_pch[MP200_DMAC_PCHNO(channel)];
	if (pch->lch == NULL) {
		return;
	}
	lch = pch->lch + MP200_DMAC_LCHNO(channel);
	if (lch->in_use != DMA_USE) {
		return;
	}

	/*
	 * disable all interrupt
	 */
#ifdef CONFIG_MP201
	intcnt = pch->intstat[0] + MP200_DMAC_LCHNO(channel) / 4;
#else
	if (MP200_DMAC_LCHNO(channel) < MP200_DMAC_SPLIT_CHNO) {
		intcnt = pch->intstat[0] + MP200_DMAC_LCHNO(channel) / 4;
	} else {
		intcnt = pch->intstat2[0];
		intcnt +=
		    (MP200_DMAC_LCHNO(channel) - MP200_DMAC_SPLIT_CHNO) / 4;
	}
#endif				/* CONFIG_MP201 */
	stshift = (MP200_DMAC_LCHNO(channel) % 4) * 8;
	intcnt->disable = 0xff << stshift;
	intcnt->clear = 0xff << stshift;

	/*
	 * stop DMA
	 */
	base = pch->control[lch->secure];
	base->stop = 1 << MP200_DMAC_LCHNO(channel);

	return;
}

/*
 * Probe
 */
static int dma_probe(struct device *dev)
{
	int i;

	for (i = 0; i < MP200_DMAC_P0_MAX_L_CHANNELS; i++) {
		dma_pch0_lch[i].callback = NULL;
		dma_pch0_lch[i].data = NULL;
		dma_pch0_lch[i].in_use = DMA_NO_USE;
	}
#ifdef CONFIG_MP201
	for (i = 0; i < MP200_DMAC_P1_MAX_L_CHANNELS; i++) {
		dma_pch1_lch[i].callback = NULL;
		dma_pch1_lch[i].data = NULL;
		dma_pch1_lch[i].in_use = DMA_NO_USE;
	}
#endif				/* CONFIG_MP201 */
	for (i = 0; i < MP200_DMAC_P2_MAX_L_CHANNELS; i++) {
		dma_pch2_lch[i].callback = NULL;
		dma_pch2_lch[i].data = NULL;
		dma_pch2_lch[i].in_use = DMA_NO_USE;
	}
#ifndef CONFIG_MP201
	for (i = 0; i < MP200_DMAC_P3_MAX_L_CHANNELS; i++) {
		dma_pch3_lch[i].callback = NULL;
		dma_pch3_lch[i].data = NULL;
		dma_pch3_lch[i].in_use = DMA_NO_USE;
	}
#endif				/* CONFIG_MP201 */

	return 0;
}

/*
 * Initialize
 */
static int __init mp200_dma_init(void)
{
	return driver_register(&dma_driver);
}

arch_initcall(mp200_dma_init);

EXPORT_SYMBOL(mp200_request_dma);
EXPORT_SYMBOL(mp200_free_dma);
EXPORT_SYMBOL(mp200_start_dma);
EXPORT_SYMBOL(mp200_dma_status);
EXPORT_SYMBOL(mp200_dma_busy);
EXPORT_SYMBOL(mp200_clear_dma);
EXPORT_SYMBOL(mp200_reset_dma);
EXPORT_SYMBOL(mp200_get_dma_pos);
EXPORT_SYMBOL(mp200_stop_dma);
EXPORT_SYMBOL(mp200_dma_writable_flag);
