/*
 *  File Name	    : linux/arch/arm/mach-mp200/dma.h
 *  Function	    : dmac
 *  Release Version : Ver 1.01
 *  Release Date    : 2006/02/01
 *
 *  Copyright (C) NEC Electronics Corporation 2005-2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#ifndef __ARCH_ARM_MACH_MP200_DMA_H
#define __ARCH_ARM_MACH_MP200_DMA_H

/****  Macro definition  ****/

#ifdef CONFIG_MP201
#define MP200_DMAC_MASK_CHNO		0x00000037
#define MP200_DMAC_MASK_ADDRESS 	0xffffffff
#define MP200_DMAC_MASK_BLKSIZE 	0x0000ffff
#define MP200_DMAC_MASK_LENGTH		0x00ffffff
#else
#define MP200_DMAC_MASK_CHNO		0x000000ef
#define MP200_DMAC_MASK_ADDRESS 	0xfffffffe
#define MP200_DMAC_MASK_BLKSIZE 	0x0000fffe
#define MP200_DMAC_MASK_LENGTH		0x00fffffe
#endif				/* CONFIG_MP201 */

#define MP200_DMAC_MASK_INT		( MP200_DMAC_INT_TIME_EN \
					| MP200_DMAC_INT_ERROR_EN \
					| MP200_DMAC_INT_BLOCK_EN \
					| MP200_DMAC_INT_LENG_EN )

#ifndef CONFIG_MP201
#define MP200_DMAC_SPLIT_CHNO		12
#endif				/* CONFIG_MP201 */

/****  Macro function  ****/

#ifdef CONFIG_MP201
#define MP200_DMAC_PCHNO(x)		(((x) & 0x30) >> 4)
#define MP200_DMAC_LCHNO(x)		((x) & 0x07)
#else
#define MP200_DMAC_PCHNO(x)		(((x) & 0xe0) >> 5)
#define MP200_DMAC_LCHNO(x)		((x) & 0x0f)
#endif				/* CONFIG_MP201 */

#define MP200_DMAC_PARBASE2CHNO(x)	((((long)(x)) >> 8) & MP200_DMAC_MASK_CHNO)

/****  Structure definition  ****/

/*
 * MP200 DMAC register arrangement
 */

/* control registers */
typedef struct {
	volatile u_int32_t start;	/* +0x00 DMA start control */
	volatile u_int32_t status;	/* +0x04 DMA status */
	volatile u_int32_t stop;	/* +0x08 DMA stop control */
} cntsts_t;

/*
 * interrupt related parameter registers
 * Every physical DMA channel has array of this structure for
 * every logical channel and every PE's
 */
typedef struct {
	volatile u_int32_t stat;	/* +0x00 interrupt status */
	volatile u_int32_t raw_stat;	/* +0x04 interrupt raw status */
	volatile u_int32_t enable;	/* +0x08 interrupt enable */
	volatile u_int32_t disable;	/* +0x0c interrupt disable */
	volatile u_int32_t clear;	/* +0x10 interrupt clear */
#ifdef CONFIG_MP201
	volatile u_int32_t reserved[3];	/* +0x14 - +0x1c reserved */
#endif				/* CONFIG_MP201 */
} intparm_t;

/*
 * interrupt destination setting register
 */
typedef struct {
	volatile u_int32_t intdest;	/* +0x00 interrupt destination */
} intdest_t;

/*
 * Data for managing logical DMA channels
 */
typedef struct {
	const char *name;	/* irq name of L-channel. */
	dma_callback_t callback;	/* call-back function from interrupt. */
	void *data;		/* argument of call-back function. */
	u_int32_t defmode;	/* default transfer mode. */
	u_int8_t secure;	/* secure channel (1) or not (0) */
	u_int8_t in_use;	/* DMA in use flag. */
} mp200_dmal_t;

/* for `in_use' setting */
#define DMA_NO_USE	0
#define DMA_USE 	1

/*
 * Data for managing physical DMA channels
 */
typedef struct {
	cntsts_t *control[2];	/* control/status regs. (normal and secure) */
	intparm_t *intstat[4];	/* interrupt control regs. (LCH0-11) (each PE) */
#ifndef CONFIG_MP201
	intparm_t *intstat2[4];	/* interrupt control regs. (LCH12-14) (each PE) */
#endif				/* CONFIG_MP201 */
	intdest_t *intsel;	/* interrupt destination setting reg. */
	dma_regs_t *parbase[2];	/* base of parameter registers array. (normal and secure) */
	mp200_dmal_t *lch;	/* control structures for logical CH. */
	u_int32_t lchno;	/* No. of logical channel. */
	int (*setup_dma) (dma_regs_t *, dma_addr_t, u_int, dma_addr_t,
			  u_int32_t);
	/* setup function. (for default) */
} mp200_dmap_t;

typedef struct {
	u_int32_t enable[4][4];
	u_int32_t intdest;
} dma_state_pch_t;

typedef struct {
	u32 aadd;
	u32 aoff;
	u32 asize;
	u32 asize_count;
	u32 badd;
	u32 boff;
	u32 bsize;
	u32 bsize_count;
	u32 leng;
	u32 size;
	u32 mode;
	u32 time;
} dma_state_lch_t;

#endif				/* __ARCH_ARM_MACH_MP200_DMA_H */
