/*
 *  File Name       : linux/arch/arm/mach-mp200/generic.c
 *  Function        : generic
 *  Release Version : Ver 1.04
 *  Release Date    : 2006/03/29
 *
 *  Copyright (C) NEC Electronics Corporation 2005-2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

/*
 * Copyright 2006,2007 Sony Corporation. 
 */

#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/device.h>
#include <linux/serial.h>
#include <linux/serial_core.h>
#include <linux/kgdb.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/mach/map.h>

#include <asm/arch/pmu.h>

#include "generic.h"

/*
 * Common MP200 I/O Mapping
 * 
 * Logical      Physical
 * e4000000	40000000	APB0
 * e5000000	50000000	ASYNC1
 * e6000000	60000000	AHB Slave
 * e8000000	80000000	Local
 * ec000000	c0000000	APB1
 */

#ifdef CONFIG_MP201
static struct map_desc standard_io_desc[] __initdata = {
	/* BANK 4 */
	MP200_MAP_DESC(MP200_TI1_BASE, MP200_TI1_SIZE),
	MP200_MAP_DESC(MP200_TI2_BASE, MP200_TI2_SIZE),
	MP200_MAP_DESC(MP200_TI3_BASE, MP200_TI3_SIZE),
	MP200_MAP_DESC(MP200_TI4_BASE, MP200_TI4_SIZE),
	MP200_MAP_DESC(MP200_TI5_BASE, MP200_TI5_SIZE),
	MP200_MAP_DESC(MP200_WDT1_BASE, MP200_WDT1_SIZE),
	MP200_MAP_DESC(MP200_DCV_BASE, MP200_DCV_SIZE),
	MP200_MAP_DESC(MP200_FIFO_BASE, MP200_FIFO_SIZE),
	MP200_MAP_DESC(MP200_ROTATOR_BASE, MP200_ROTATOR_SIZE),
	MP200_MAP_DESC(MP200_DMAC_BASE, MP200_DMAC_SIZE),
	MP200_MAP_DESC(MP200_IMAGEPROC_BASE, MP200_IMAGEPROC_SIZE),
	MP200_MAP_DESC(MP200_CAMERA_BASE, MP200_CAMERA_SIZE),
	MP200_MAP_DESC(MP200_PCM0_BASE, MP200_PCM0_SIZE),
	MP200_MAP_DESC(MP200_PCM1_BASE, MP200_PCM1_SIZE),
	MP200_MAP_DESC(MP200_NTSC_BASE, MP200_NTSC_SIZE),
	MP200_MAP_DESC(MP200_PB0_BASE, MP200_PB0_SIZE),
	MP200_MAP_DESC(MP200_SWT_BASE, MP200_SWT_SIZE),
	MP200_MAP_DESC(MP200_SP0_BASE, MP200_SP0_SIZE),
	MP200_MAP_DESC(MP200_SP1_BASE, MP200_SP1_SIZE),
	MP200_MAP_DESC(MP200_DTV_BASE, MP200_DTV_SIZE),
	MP200_MAP_DESC(MP200_NAND_BASE, MP200_NAND_SIZE),
	MP200_MAP_DESC(MP200_STM_BASE, MP200_STM_SIZE),
	MP200_MAP_DESC(MP200_DMAC_S0_BASE, MP200_DMAC_S0_SIZE),
	MP200_MAP_DESC(MP200_FIFO_S1_BASE, MP200_FIFO_S1_SIZE),

	/* BANK5 */
	MP200_MAP_DESC(MP200_UART0_BASE, MP200_UART0_SIZE),
	MP200_MAP_DESC(MP200_I2C_BASE, MP200_I2C_SIZE),
	MP200_MAP_DESC(MP200_AB1_BASE, MP200_AB1_SIZE),
	MP200_MAP_DESC(MP200_SDM_S0_BASE, MP200_SDM_S0_SIZE),

	/* BANK 8 */
	MP200_MAP_DESC(MP200_PEID_BASE, MP200_PEID_SIZE),

	/* BANK 10 */
	MP200_MAP_DESC(MP200_SSRAM_BASE, MP200_SSRAM_SIZE),

	/* BANK 11 */
	MP200_MAP_DESC(MP200_USB_BASE, MP200_USB_SIZE),
	MP200_MAP_DESC(MP200_ATA_BASE, MP200_ATA_SIZE),
	MP200_MAP_DESC(MP200_MSP_BASE, MP200_MSP_SIZE),
	MP200_MAP_DESC(MP200_MMM_BASE, MP200_MMM_SIZE),

	/* BANK 12 */
	MP200_MAP_DESC(MP200_TI0_BASE, MP200_TI0_SIZE),
	MP200_MAP_DESC(MP200_WDT0_BASE, MP200_WDT0_SIZE),
	MP200_MAP_DESC(MP200_INTC_BASE, MP200_INTC_SIZE),
	MP200_MAP_DESC(MP200_TFD_BASE, MP200_TFD_SIZE),
	MP200_MAP_DESC(MP200_LCDC_BASE, MP200_LCDC_SIZE),
	MP200_MAP_DESC(MP200_GPIO_BASE, MP200_GPIO_SIZE),
	MP200_MAP_DESC(MP200_PWM_BASE, MP200_PWM_SIZE),
	MP200_MAP_DESC(MP200_PB1_BASE, MP200_PB1_SIZE),
	MP200_MAP_DESC(MP200_PMU_BASE, MP200_PMU_SIZE),
	MP200_MAP_DESC(MP200_SMU_S0_BASE, MP200_SMU_S0_SIZE),
	MP200_MAP_DESC(MP200_uWIRE_BASE, MP200_uWIRE_SIZE),
	MP200_MAP_DESC(MP200_SMU_S1_BASE, MP200_SMU_S1_SIZE),
	MP200_MAP_DESC(MP200_INTC_S1_BASE, MP200_INTC_S1_SIZE)
};
#else
static struct map_desc standard_io_desc[] __initdata = {
	/* BANK 4 */
	MP200_MAP_DESC(MP200_TIMER1_BASE, MP200_TIMER1_SIZE),
	MP200_MAP_DESC(MP200_TIMER2_BASE, MP200_TIMER2_SIZE),
	MP200_MAP_DESC(MP200_TIMER3_BASE, MP200_TIMER3_SIZE),
	MP200_MAP_DESC(MP200_WDT1_BASE, MP200_WDT1_SIZE),
	MP200_MAP_DESC(MP200_WDT2_BASE, MP200_WDT2_SIZE),
	MP200_MAP_DESC(MP200_WDT3_BASE, MP200_WDT3_SIZE),
	MP200_MAP_DESC(MP200_3DACC_BASE, MP200_3DACC_SIZE),
	MP200_MAP_DESC(MP200_FIFO_BASE, MP200_FIFO_SIZE),
	MP200_MAP_DESC(MP200_ROTATOR_BASE, MP200_ROTATOR_SIZE),
	MP200_MAP_DESC(MP200_DMAC_BASE, MP200_DMAC_SIZE),
	MP200_MAP_DESC(MP200_IMAGEPROC_BASE, MP200_IMAGEPROC_SIZE),
	MP200_MAP_DESC(MP200_CAMERA_BASE, MP200_CAMERA_SIZE),
	MP200_MAP_DESC(MP200_PCM0_BASE, MP200_PCM0_SIZE),
	MP200_MAP_DESC(MP200_PCM1_BASE, MP200_PCM1_SIZE),
	MP200_MAP_DESC(MP200_PCM2_BASE, MP200_PCM2_SIZE),
	MP200_MAP_DESC(MP200_MOD1_BASE, MP200_MOD1_SIZE),
	MP200_MAP_DESC(MP200_NTSC_BASE, MP200_NTSC_SIZE),
	MP200_MAP_DESC(MP200_PB0_BASE, MP200_PB0_SIZE),
	MP200_MAP_DESC(MP200_SWT_BASE, MP200_SWT_SIZE),
	MP200_MAP_DESC(MP200_SP1_BASE, MP200_SP1_SIZE),
	MP200_MAP_DESC(MP200_SP2_BASE, MP200_SP2_SIZE),
	MP200_MAP_DESC(MP200_DTV_BASE, MP200_DTV_SIZE),
	MP200_MAP_DESC(MP200_CCP_BASE, MP200_CCP_SIZE),
	MP200_MAP_DESC(MP200_RMT_BASE, MP200_RMT_SIZE),
	MP200_MAP_DESC(MP200_SP0_BASE, MP200_SP0_SIZE),
	MP200_MAP_DESC(MP200_TG0_BASE, MP200_TG0_SIZE),
	MP200_MAP_DESC(MP200_TG1_BASE, MP200_TG1_SIZE),
	MP200_MAP_DESC(MP200_TG2_BASE, MP200_TG2_SIZE),
	MP200_MAP_DESC(MP200_TG3_BASE, MP200_TG3_SIZE),
	MP200_MAP_DESC(MP200_TG4_BASE, MP200_TG4_SIZE),
	MP200_MAP_DESC(MP200_TG5_BASE, MP200_TG5_SIZE),
	MP200_MAP_DESC(MP200_MMM_S0_BASE, MP200_MMM_S0_SIZE),
	MP200_MAP_DESC(MP200_DMAC_S0_BASE, MP200_DMAC_S0_SIZE),
	MP200_MAP_DESC(MP200_FIFO_S1_BASE, MP200_FIFO_S1_SIZE),

	/* BANK5 */
	MP200_MAP_DESC(MP200_UART0_BASE, MP200_UART0_SIZE),
	MP200_MAP_DESC(MP200_UART1_BASE, MP200_UART1_SIZE),
	MP200_MAP_DESC(MP200_UART2_BASE, MP200_UART2_SIZE),
	MP200_MAP_DESC(MP200_I2C_BASE, MP200_I2C_SIZE),
	MP200_MAP_DESC(MP200_ASYNC1_BASE, MP200_ASYNC1_SIZE),
	MP200_MAP_DESC(MP200_SDIP_S0_BASE, MP200_SDIP_S0_SIZE),
	MP200_MAP_DESC(MP200_UART3_S1_BASE, MP200_UART3_S1_SIZE),

	/* BANK 6 */
	MP200_MAP_DESC(MP200_USB_BASE, MP200_USB_SIZE),

	/* BANK 8 */
	MP200_MAP_DESC(MP200_PEID_BASE, MP200_PEID_SIZE),

	/* BANK 10 */
	MP200_MAP_DESC(MP200_SSRAM_BASE, MP200_SSRAM_SIZE),

	/* BANK 12 */
	MP200_MAP_DESC(MP200_TIMER0_BASE, MP200_TIMER0_SIZE),
	MP200_MAP_DESC(MP200_WDT0_BASE, MP200_WDT0_SIZE),
	MP200_MAP_DESC(MP200_INTC_BASE, MP200_INTC_SIZE),
	MP200_MAP_DESC(MP200_LCDC_BASE, MP200_LCDC_SIZE),
	MP200_MAP_DESC(MP200_GPIO_BASE, MP200_GPIO_SIZE),
	MP200_MAP_DESC(MP200_SIO0_BASE, MP200_SIO0_SIZE),
	MP200_MAP_DESC(MP200_SIO1_BASE, MP200_SIO1_SIZE),
	MP200_MAP_DESC(MP200_PB1_BASE, MP200_PB1_SIZE),
	MP200_MAP_DESC(MP200_PMU_BASE, MP200_PMU_SIZE),
	MP200_MAP_DESC(MP200_SMU_S0_BASE, MP200_SMU_S0_SIZE),
	MP200_MAP_DESC(MP200_uWIRE_BASE, MP200_uWIRE_SIZE),
	MP200_MAP_DESC(MP200_SMU_S1_BASE, MP200_SMU_S1_SIZE),
	MP200_MAP_DESC(MP200_INTC_S1_BASE, MP200_INTC_S1_SIZE)
};
#endif				/* CONFIG_MP201 */

static struct plat_serial8250_port serial_platform_data[] = {
	{
	 .membase = (char *)IO_ADDRESS(MP200_UART0_BASE),
	 .mapbase = (unsigned long)MP200_UART0_BASE,
	 .irq = INT_UART0,
	 .flags = UPF_SKIP_TEST,
	 .iotype = UPIO_MEM,
	 .regshift = 0,
	 .uartclk = MP200_BASE_BAUD,
	 },
#ifndef CONFIG_MP201
	{
	 .membase = (char *)IO_ADDRESS(MP200_UART1_BASE),
	 .mapbase = (unsigned long)MP200_UART1_BASE,
	 .irq = INT_UART1,
	 .flags = UPF_SKIP_TEST,
	 .iotype = UPIO_MEM,
	 .regshift = 0,
	 .uartclk = MP200_BASE_BAUD,
	 },
	{
	 .membase = (char *)IO_ADDRESS(MP200_UART2_BASE),
	 .mapbase = (unsigned long)MP200_UART2_BASE,
	 .irq = INT_UART2,
	 .flags = UPF_SKIP_TEST,
	 .iotype = UPIO_MEM,
	 .regshift = 0,
	 .uartclk = MP200_BASE_BAUD,
	 },
	{
	 .membase = (char *)IO_ADDRESS(MP200_UART3_S1_BASE),
	 .mapbase = (unsigned long)MP200_UART3_S1_BASE,
	 .irq = INT_UART3,
	 .flags = UPF_SKIP_TEST,
	 .iotype = UPIO_MEM,
	 .regshift = 0,
	 .uartclk = MP200_BASE_BAUD,
	 },
#endif				/* CONFIG_MP201 */
	{
	 /* terminate */
	 },
};

static struct platform_device dma_device = {
	.name = "dma",
	.id = -1,
};

static struct platform_device i2c_device = {
	.name = "i2c",
	.id = -1,
};

/* 2006.10.06 has */
#ifdef CONFIG_SONY_ICX1087
static struct platform_device spi_device = {
	.name = "spi",
	.id = 0,
};
#else
#ifdef CONFIG_MP201
static struct platform_device spi0_device = {
	.name = "spi",
	.id = 0,
};
#endif				/* CONFIG_MP201 */
static struct platform_device spi1_device = {
	.name = "spi",
	.id = 1,
};

#ifndef CONFIG_MP201
static struct platform_device spi2_device = {
	.name = "spi",
	.id = 2,
};
#endif
#endif

static struct platform_device serial8250_device = {
	.name = "serial8250",
	.id = 0,
	.dev = {
		.platform_data = serial_platform_data,
		},
};

static struct platform_device pcm_device = {
	.name = "pcm",
	.id = -1,
};

static struct platform_device mp200_fb_device = {
	.name = "mp200_fb",
	.id = -1,
};

static struct platform_device mp200_image_device = {
	.name = "mp200_image",
	.id = -1,
};

#ifdef CONFIG_MP201
static struct platform_device mp200_rotator_device = {
	.name = "mp200_rotator",
	.id = -1,
};
#endif				/* CONFIG_MP201 */

static struct platform_device mp200_camera_device = {
	.name = "mp200_camera",
	.id = -1,
};

static struct platform_device mp200_dsp_device = {
	.name = "mp200_dsp",
	.id = -1,
};

static struct platform_device mp200_ntsc_device = {
	.name = "mp200_ntsc",
	.id = -1,
};

static struct platform_device mp200_dtv_device = {
	.name = "mp200_dtv",
	.id = -1,
};

static struct platform_device mp200_usb_otg_device = {
	.name = "mp200_usb_otg",
	.id = -1,
};

static struct platform_device *platform_devs[] __initdata = {
	&dma_device,
	&i2c_device,
/* 2006.10.05 has */
#ifdef CONFIG_SONY_ICX1087
	&spi_device,
#else
#ifdef CONFIG_MP201
	&spi0_device,
#endif				/* CONFIG_MP201 */
	&spi1_device,
#ifndef CONFIG_MP201
	&spi2_device,
#endif				/* CONFIG_MP201 */
#endif
	&serial8250_device,
	&mp200_fb_device,
	&mp200_image_device,
#ifdef CONFIG_MP201
	&mp200_rotator_device,
#endif				/* CONFIG_MP201 */
	&mp200_camera_device,
	&mp200_dsp_device,
	&mp200_ntsc_device,
	&mp200_dtv_device,
	&pcm_device,
	&mp200_usb_otg_device,
};

void __init mp200_map_io(void)
{
	iotable_init(standard_io_desc, ARRAY_SIZE(standard_io_desc));
}

#ifdef CONFIG_MP201
void __init mp200_serial_init(void)
{
	/* reset U70 */
	mp200_pmu_reset_device(MP200_RESETDEVICE_U70);
	mp200_pmu_unreset_device(MP200_RESETDEVICE_U70);
}
#else
void __init mp200_serial_init(int *ports)
{
	int i;

	for (i = 0; i < 4; i++) {
		if (ports[i] == 0) {
			serial_platform_data[i].membase = 0;
			serial_platform_data[i].mapbase = 0;
			continue;
		}
		/* reset U7x */
		mp200_pmu_reset_device(MP200_RESETDEVICE_U70 + i);
		mp200_pmu_unreset_device(MP200_RESETDEVICE_U70 + i);

#ifdef CONFIG_KGDB_8250
		kgdb8250_add_platform_port(i, &serial_platform_data[i]);
#endif
	}
}
#endif				/* CONFIG_MP201 */

static int __init mp200_init(void)
{
	return platform_add_devices(platform_devs, ARRAY_SIZE(platform_devs));
}

arch_initcall(mp200_init);
