/*
 *  File Name       : linux/arch/arm/mach-mp200/gio.c
 *  Function        : gpio
 *  Release Version : Ver 1.00
 *  Release Date    : 2005/04/28
 *
 *  Copyright (C) NEC Electronics Corporation 2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#include <linux/interrupt.h>
#include <linux/module.h>
#include <linux/delay.h>

#include <asm/mach/irq.h>
#include <asm/arch/gpiocore.h>

#ifdef CONFIG_MP201
#define PIN_MASK(X)	(1U << ((X) & 0xf))
#else
#define PIN_MASK(X)	(1U << ((X) & 0x1f))	/* IIA/IEN/IDS/PLS/... */
#define PIN_INDEX(X)	((X) >> 5)
#endif				/* CONFIG_MP201 */

spinlock_t mp200_gio_lock = SPIN_LOCK_UNLOCKED;

static void mp200_gio_irq_ack(unsigned int irq);
static void mp200_gio_irq_mask(unsigned int irq);
static void mp200_gio_irq_unmask(unsigned int irq);
static int mp200_gio_set_irq_type(unsigned int irq, unsigned int type);

static struct irqchip mp200_gio_chip_ack = {
	.ack = mp200_gio_irq_ack,
	.mask = mp200_gio_irq_mask,
	.unmask = mp200_gio_irq_unmask,
	.type = mp200_gio_set_irq_type,
};

static struct irqchip mp200_gio_chip = {
	.ack = mp200_gio_irq_mask,
	.mask = mp200_gio_irq_mask,
	.unmask = mp200_gio_irq_unmask,
	.type = mp200_gio_set_irq_type,
};

#ifdef CONFIG_MP201
static uint32_t accept_bits[4] = { 0, };
#else
static uint32_t accept_bits[2] = { 0, };
#endif				/* CONFIG_MP201 */

/* called from set_irq_type() */
static int mp200_gio_set_irq_type(unsigned int irq, unsigned int type)
{
	unsigned int pin = irq - INT_GPIO_BASE;
	unsigned int mode = 0;
	unsigned int flag;
	unsigned int oiia;
	unsigned int x;
	unsigned int mask;
	unsigned int dtshift;
#ifndef CONFIG_MP201
	unsigned int pls32k = 0;
#endif				/* CONFIG_MP201 */

	if ((irq < INT_GPIO_BASE) || (INT_GPIO_LAST < irq)) {
		return -EINVAL;
	}
#ifdef CONFIG_MP201
	if ((type & __IRQT_ASYNC) != 0) {
		if ((type & (IRQT_HIGH | IRQT_LOW)) != 0) {
			return -EINVAL;
		}
	}
#else
	if ((type & __IRQT_32KPLS) != 0) {
		if ((type & (IRQT_HIGH | IRQT_LOW)) != 0) {
			return -EINVAL;
		}
		pls32k = 1;
		type &= ~(__IRQT_32KPLS);
	}
#endif				/* CONFIG_MP201 */

#ifdef CONFIG_MP201
	switch (type & ~(__IRQT_ASYNC)) {
#else
	switch (type) {
#endif				/* CONFIG_MP201 */
	case IRQT_RISING:
		mode = 0x00;
		break;
	case IRQT_FALLING:
		mode = 0x01;
		break;
	case IRQT_HIGH:
		mode = 0x02;
		break;
	case IRQT_LOW:
		mode = 0x03;
		break;
#ifdef CONFIG_MP201
	case IRQT_BOTHEDGE:
		mode = 0x04;
		break;
#endif				/* CONFIG_MP201 */
	default:
		return -EINVAL;
	}

	mask = PIN_MASK(pin);
#ifdef CONFIG_MP201
	dtshift = (pin & 0x7) << 2;	/* IDT[HL] shift bit */
#else
	dtshift = (pin & 0xf) << 1;	/* DT[12][HL] shift bit */
#endif				/* CONFIG_MP201 */

#ifdef CONFIG_MP201
	if ((mode == 0x00) || (mode == 0x01) || (mode == 0x04)) {
#else
	if ((mode == 0x00) || (mode == 0x01)) {
#endif				/* CONFIG_MP201 */
		set_irq_chip(irq, &mp200_gio_chip_ack);
		set_irq_handler(irq, do_edge_IRQ);
	} else {
		set_irq_chip(irq, &mp200_gio_chip);
		set_irq_handler(irq, do_level_IRQ);
	}

	MP200_GIO_SPINLOCK(0, flag);

#ifdef CONFIG_MP201
	if ((type & __IRQT_ASYNC) != 0) {
		mode |= 0x08;
	}

	if (pin < 16) {
		/* IA enable -> disable */
		oiia = __raw_readl(VA_GIO + GPIO0_IA);
		if ((oiia & mask) != 0) {
			__raw_writel((oiia & ~(mask)), (VA_GIO + GPIO0_IA));
		}

		/* set IDT */
		if (pin < 8) {
			x = __raw_readl(VA_GIO + GPIO0_IDTL);
			if ((x & (0xfU << dtshift)) != (mode << dtshift)) {
				x &= ~(0xfU << dtshift);
				x |= (mode << dtshift);
				__raw_writel(x, (VA_GIO + GPIO0_IDTL));
			}
		} else {
			x = __raw_readl(VA_GIO + GPIO0_IDTH);
			if ((x & (0xfU << dtshift)) != (mode << dtshift)) {
				x &= ~(0xfU << dtshift);
				x |= (mode << dtshift);
				__raw_writel(x, (VA_GIO + GPIO0_IDTH));
			}
		}
		__raw_writel(mask, (VA_GIO + GPIO0_IR));

		/* Restore if changed */
		if ((oiia & mask) != 0) {
			__raw_writel(oiia, (VA_GIO + GPIO0_IA));
		}
	} else if (pin < 32) {
		/* IA enable -> disable */
		oiia = __raw_readl(VA_GIO + GPIO1_IA);
		if ((oiia & mask) != 0) {
			__raw_writel((oiia & ~(mask)), (VA_GIO + GPIO1_IA));
		}

		/* set IDT */
		if (pin < 24) {
			x = __raw_readl(VA_GIO + GPIO1_IDTL);
			if ((x & (0xfU << dtshift)) != (mode << dtshift)) {
				x &= ~(0xfU << dtshift);
				x |= (mode << dtshift);
				__raw_writel(x, (VA_GIO + GPIO1_IDTL));
			}
		} else {
			x = __raw_readl(VA_GIO + GPIO1_IDTH);
			if ((x & (0xfU << dtshift)) != (mode << dtshift)) {
				x &= ~(0xfU << dtshift);
				x |= (mode << dtshift);
				__raw_writel(x, (VA_GIO + GPIO1_IDTH));
			}
		}
		__raw_writel(mask, (VA_GIO + GPIO1_IR));

		/* Restore if changed */
		if ((oiia & mask) != 0) {
			__raw_writel(oiia, (VA_GIO + GPIO1_IA));
		}
	} else if (pin < 48) {
		/* IA enable -> disable */
		oiia = __raw_readl(VA_GIO + GPIO2_IA);
		if ((oiia & mask) != 0) {
			__raw_writel((oiia & ~(mask)), (VA_GIO + GPIO2_IA));
		}

		/* set IDT */
		if (pin < 40) {
			x = __raw_readl(VA_GIO + GPIO2_IDTL);
			if ((x & (0xfU << dtshift)) != (mode << dtshift)) {
				x &= ~(0xfU << dtshift);
				x |= (mode << dtshift);
				__raw_writel(x, (VA_GIO + GPIO2_IDTL));
			}
		} else {
			x = __raw_readl(VA_GIO + GPIO2_IDTH);
			if ((x & (0xfU << dtshift)) != (mode << dtshift)) {
				x &= ~(0xfU << dtshift);
				x |= (mode << dtshift);
				__raw_writel(x, (VA_GIO + GPIO2_IDTH));
			}
		}
		__raw_writel(mask, (VA_GIO + GPIO2_IR));

		/* Restore if changed */
		if ((oiia & mask) != 0) {
			__raw_writel(oiia, (VA_GIO + GPIO2_IA));
		}
	} else {
		/* IA enable -> disable */
		oiia = __raw_readl(VA_GIO + GPIO3_IA);
		if ((oiia & mask) != 0) {
			__raw_writel((oiia & ~(mask)), (VA_GIO + GPIO3_IA));
		}

		/* set IDT */
		if (pin < 56) {
			x = __raw_readl(VA_GIO + GPIO3_IDTL);
			if ((x & (0xfU << dtshift)) != (mode << dtshift)) {
				x &= ~(0xfU << dtshift);
				x |= (mode << dtshift);
				__raw_writel(x, (VA_GIO + GPIO3_IDTL));
			}
		} else {
			x = __raw_readl(VA_GIO + GPIO3_IDTH);
			if ((x & (0xfU << dtshift)) != (mode << dtshift)) {
				x &= ~(0xfU << dtshift);
				x |= (mode << dtshift);
				__raw_writel(x, (VA_GIO + GPIO3_IDTH));
			}
		}
		__raw_writel(mask, (VA_GIO + GPIO3_IR));

		/* Restore if changed */
		if ((oiia & mask) != 0) {
			__raw_writel(oiia, (VA_GIO + GPIO3_IA));
		}
	}
#else
	if (pin < 32) {
		/* IIA enable -> disable */
		oiia = __raw_readl(VA_GIO + GP_IIAL);
		if ((oiia & mask) != 0) {
			__raw_writel((oiia & ~(mask)), (VA_GIO + GP_IIAL));
		}
		__raw_writel(mask, (VA_GIO + GP_IIRL));

		/* set 32KHz Synchronous/Asynchronous mode */
		x = __raw_readl(VA_GIO + GIO_PLSL);
		if ((pls32k != 0) && ((x & mask) == 0)) {
			__raw_writel((x | mask), (VA_GIO + GIO_PLSL));
		} else if ((pls32k == 0) && ((x & mask) != 0)) {
			__raw_writel((x & ~(mask)), (VA_GIO + GIO_PLSL));
		}

		/* set DT */
		if (pin < 16) {
			x = __raw_readl(VA_GIO + GP_DT1L);
			if ((x & (0x3U << dtshift)) != (mode << dtshift)) {
				x &= ~(0x3U << dtshift);
				x |= (mode << dtshift);
				__raw_writel(x, (VA_GIO + GP_DT1L));
			}
		} else {
			x = __raw_readl(VA_GIO + GP_DT2L);
			if ((x & (0x3U << dtshift)) != (mode << dtshift)) {
				x &= ~(0x3U << dtshift);
				x |= (mode << dtshift);
				__raw_writel(x, (VA_GIO + GP_DT2L));
			}
		}
		udelay(32);

		/* Restore if changed */
		if ((oiia & mask) != 0) {
			__raw_writel(oiia, (VA_GIO + GP_IIAL));
		}
	} else {
		/* IIA enable -> disable */
		oiia = __raw_readl(VA_GIO + GP_IIAH);
		if ((oiia & mask) != 0) {
			__raw_writel((oiia & ~(mask)), (VA_GIO + GP_IIAH));
		}
		__raw_writel(mask, (VA_GIO + GP_IIRH));

		/* set 32KHz Synchronous/Asynchronous mode */
		x = __raw_readl(VA_GIO + GIO_PLSH);
		if ((pls32k != 0) && ((x & mask) == 0)) {
			__raw_writel((x | mask), (VA_GIO + GIO_PLSH));
		} else if ((pls32k == 0) && ((x & mask) != 0)) {
			__raw_writel((x & ~(mask)), (VA_GIO + GIO_PLSH));
		}

		/* set DT */
		if (pin < 48) {
			x = __raw_readl(VA_GIO + GP_DT1H);
			if ((x & (0x3U << dtshift)) != (mode << dtshift)) {
				x &= ~(0x3U << dtshift);
				x |= (mode << dtshift);
				__raw_writel(x, (VA_GIO + GP_DT1H));
			}
		} else {
			x = __raw_readl(VA_GIO + GP_DT2H);
			if ((x & (0x3U << dtshift)) != (mode << dtshift)) {
				x &= ~(0x3U << dtshift);
				x |= (mode << dtshift);
				__raw_writel(x, (VA_GIO + GP_DT2H));
			}
		}
		udelay(32);

		/* Restore if changed */
		if ((oiia & mask) != 0) {
			__raw_writel(oiia, (VA_GIO + GP_IIAH));
		}
	}
#endif				/* CONFIG_MP201 */

	MP200_GIO_SPINUNLOCK(0, flag);

	return 0;
}

static void mp200_gio_irq_handler(unsigned int irq, struct irqdesc *desc,
				  struct pt_regs *regs)
{
	struct irqdesc *d;
	uint32_t ist, gio_irq;

	if ((irq < INT_GIO0) || (INT_GIO3 < irq)) {
		return;
	}

	desc->chip->ack(irq);

	do {
#ifdef CONFIG_MP201
		if (irq == INT_GIO0) {
			gio_irq = INT_GPIO_0;
			ist = __raw_readl(VA_GIO + GPIO0_MST) & 0xffffU;
		} else if (irq == INT_GIO1) {
			gio_irq = INT_GPIO_16;
			ist = __raw_readl(VA_GIO + GPIO1_MST) & 0xffffU;
		} else if (irq == INT_GIO2) {
			gio_irq = INT_GPIO_32;
			ist = __raw_readl(VA_GIO + GPIO2_MST) & 0xffffU;
		} else {
			gio_irq = INT_GPIO_48;
			ist = __raw_readl(VA_GIO + GPIO3_MST) & 0xffffU;
		}
#else
		if (irq == INT_GIO0) {
			gio_irq = INT_GPIO_0;
			ist = __raw_readl(VA_GIO + GP_MSTL) & 0xffffU;
		} else if (irq == INT_GIO1) {
			gio_irq = INT_GPIO_16;
			ist = __raw_readl(VA_GIO + GP_MSTL) >> 16;
		} else if (irq == INT_GIO2) {
			gio_irq = INT_GPIO_32;
			ist = __raw_readl(VA_GIO + GP_MSTH) & 0xffffU;
		} else {
			gio_irq = INT_GPIO_48;
			ist = __raw_readl(VA_GIO + GP_MSTH) >> 16;
		}
#endif				/* CONFIG_MP201 */

		if (ist == 0)
			break;

		while (ist) {
			if ((ist & 1) != 0) {
				d = irq_desc + gio_irq;
				d->handle(gio_irq, d, regs);
			}
			ist >>= 1;
			gio_irq++;
		}
	} while (1);

	desc->chip->unmask(irq);
}

static void mp200_gio_irq_ack(unsigned int irq)
{
	unsigned int pin = irq - INT_GPIO_BASE;
	unsigned int mask = PIN_MASK(pin);

	if ((irq < INT_GPIO_BASE) || (INT_GPIO_LAST < irq)) {
		return;
	}
#ifdef CONFIG_MP201
	if (pin < 16) {
		__raw_writel(mask, (VA_GIO + GPIO0_IR));
	} else if (pin < 32) {
		__raw_writel(mask, (VA_GIO + GPIO1_IR));
	} else if (pin < 48) {
		__raw_writel(mask, (VA_GIO + GPIO2_IR));
	} else {
		__raw_writel(mask, (VA_GIO + GPIO3_IR));
	}
#else
	if (pin < 32) {
		__raw_writel(mask, (VA_GIO + GP_IIRL));
	} else {
		__raw_writel(mask, (VA_GIO + GP_IIRH));
	}
#endif				/* CONFIG_MP201 */
}

static void mp200_gio_irq_mask(unsigned int irq)
{
	unsigned int pin = irq - INT_GPIO_BASE;
	unsigned int mask = PIN_MASK(pin);

	if ((irq < INT_GPIO_BASE) || (INT_GPIO_LAST < irq)) {
		return;
	}

	/* disable */
#ifdef CONFIG_MP201
	if (pin < 16) {
		__raw_writel(mask, (VA_GIO + GPIO0_IME));
	} else if (pin < 32) {
		__raw_writel(mask, (VA_GIO + GPIO1_IME));
	} else if (pin < 48) {
		__raw_writel(mask, (VA_GIO + GPIO2_IME));
	} else {
		__raw_writel(mask, (VA_GIO + GPIO3_IME));
	}
#else
	if (pin < 32) {
		__raw_writel(mask, (VA_GIO + GP_IDSL));
	} else {
		__raw_writel(mask, (VA_GIO + GP_IDSH));
	}
#endif				/* CONFIG_MP201 */
}

static void mp200_gio_irq_unmask(unsigned int irq)
{
	unsigned int iia;
	unsigned int pin = irq - INT_GPIO_BASE;
	unsigned int mask = PIN_MASK(pin);
	unsigned int flag;

	if ((irq < INT_GPIO_BASE) || (INT_GPIO_LAST < irq)) {
		return;
	}

	/* IRQ enable */
#ifdef CONFIG_MP201
	if (pin < 16) {
		MP200_GIO_SPINLOCK(0, flag);
		if ((accept_bits[0] & mask) == 0) {
			iia = __raw_readl(VA_GIO + GPIO0_IA);
			__raw_writel((iia | mask), (VA_GIO + GPIO0_IA));
			accept_bits[0] |= mask;
		}
		MP200_GIO_SPINUNLOCK(0, flag);
		__raw_writel(mask, (VA_GIO + GPIO0_IMN));
	} else if (pin < 32) {
		MP200_GIO_SPINLOCK(0, flag);
		if ((accept_bits[1] & mask) == 0) {
			iia = __raw_readl(VA_GIO + GPIO1_IA);
			__raw_writel((iia | mask), (VA_GIO + GPIO1_IA));
			accept_bits[1] |= mask;
		}
		MP200_GIO_SPINUNLOCK(0, flag);
		__raw_writel(mask, (VA_GIO + GPIO1_IMN));
	} else if (pin < 48) {
		MP200_GIO_SPINLOCK(0, flag);
		if ((accept_bits[2] & mask) == 0) {
			iia = __raw_readl(VA_GIO + GPIO2_IA);
			__raw_writel((iia | mask), (VA_GIO + GPIO2_IA));
			accept_bits[2] |= mask;
		}
		MP200_GIO_SPINUNLOCK(0, flag);
		__raw_writel(mask, (VA_GIO + GPIO2_IMN));
	} else {
		MP200_GIO_SPINLOCK(0, flag);
		if ((accept_bits[3] & mask) == 0) {
			iia = __raw_readl(VA_GIO + GPIO3_IA);
			__raw_writel((iia | mask), (VA_GIO + GPIO3_IA));
			accept_bits[3] |= mask;
		}
		MP200_GIO_SPINUNLOCK(0, flag);
		__raw_writel(mask, (VA_GIO + GPIO3_IMN));
	}
#else
	if (pin < 32) {
		MP200_GIO_SPINLOCK(0, flag);
		if ((accept_bits[0] & mask) == 0) {
			iia = __raw_readl(VA_GIO + GP_IIAL);
			__raw_writel((iia | mask), (VA_GIO + GP_IIAL));
			accept_bits[0] |= mask;
		}
		MP200_GIO_SPINUNLOCK(0, flag);
		__raw_writel(mask, (VA_GIO + GP_IENL));
	} else {
		MP200_GIO_SPINLOCK(0, flag);
		if ((accept_bits[1] & mask) == 0) {
			iia = __raw_readl(VA_GIO + GP_IIAH);
			__raw_writel((iia | mask), (VA_GIO + GP_IIAH));
			accept_bits[1] |= mask;
		}
		MP200_GIO_SPINUNLOCK(0, flag);
		__raw_writel(mask, (VA_GIO + GP_IENH));
	}
#endif				/* CONFIG_MP201 */
}

static int __init mp200_gio_init(void)
{
	int i;

#ifdef CONFIG_MP201
	/* GIO Interrupt disable */
	__raw_writel(0xffff, (VA_GIO + GPIO0_IME));
	__raw_writel(0xffff, (VA_GIO + GPIO1_IME));
	__raw_writel(0xffff, (VA_GIO + GPIO2_IME));
	__raw_writel(0xffff, (VA_GIO + GPIO3_IME));
#endif				/* CONFIG_MP201 */

	/* setup default GIO Interrupt modes */
	for (i = INT_GPIO_BASE; i <= INT_GPIO_LAST; i++) {
		set_irq_chip(i, &mp200_gio_chip_ack);
		set_irq_handler(i, do_edge_IRQ);
		set_irq_flags(i, IRQF_VALID);
	}

	set_irq_chained_handler(INT_GIO0, mp200_gio_irq_handler);
	set_irq_chained_handler(INT_GIO1, mp200_gio_irq_handler);
	set_irq_chained_handler(INT_GIO2, mp200_gio_irq_handler);
	set_irq_chained_handler(INT_GIO3, mp200_gio_irq_handler);

	return 0;
}

arch_initcall(mp200_gio_init);

EXPORT_SYMBOL(mp200_gio_lock);
