/*
 *  File Name       : linux/arch/arm/mach-mp200/mp200_board.c
 *  Function        : mp200_board
 *  Release Version : Ver 1.01
 *  Release Date    : 2006/05/25
 *
 *  Copyright (C) NEC Electronics Corporation 2005-2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

/*
 * Copyright 2006,2007 Sony Corporation. 
 */

#include <linux/device.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/types.h>
#include <linux/mtd/nand.h>
#include <linux/mtd/partitions.h>

#include <asm/hardware.h>
#include <asm/io.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/flash.h>

#include <asm/arch/gio.h>
#include <asm/arch/gpiocore.h>

/* 2007.03.02 has */
#ifdef CONFIG_SONY_ICX1087
#include <asm/arch/smu.h>
#endif

#include "generic.h"
#include "time.h"

#ifdef CONFIG_MP201
/* 2006.10.13 has */
#ifdef CONFIG_SONY_ICX1087
/* GPIO is set by U-Boot at Boot.                */
/* so, at here, set bit to change for kernel.    */
/* Now, the INPUT/OUTPUT value were set to zero. */
/* because, there are no change for kernel,      */
#define GPIO_OUTPORT 0x0000000000000000ULL	/* GPIO output */
#define GPIO_INPORT  0x0000000000000000ULL	/* GPIO input */
#else
#define GPIO_INPORT	0xFFFFFFDFFFFFFE7FULL	/* GPIO input */
#endif
#else
#define GPIO_INPORT	0x8FFFFFFF4D3F7CC0ULL	/* GPIO input */
#endif				/* CONFIG_MP201 */
#ifndef CONFIG_MP201
static int __initdata mp200_serial_ports[] = { 1, 0, 1, 1 };
#endif				/* CONFIG_MP201 */

/* 2007.03.02 has */
#ifdef CONFIG_SONY_ICX1087
#define SMU_PULLCTL_PCM0_VALUE 0x00000007
#define SMU_PULLCTL_PCM0_MASK  0x00010007
#endif

static struct map_desc mp200_board_io_desc[] __initdata = {
	/* BANK2 */
/* 2006.10.19 has */
#ifndef CONFIG_SONY_ICX1087
	MP200_MAP_DESC(MP200_EXTDEVICE_BASE, MP200_EXTDEVICE_SIZE),
#endif
	MP200_MAP_DESC(MP200_ASYNC0_BASE, MP200_ASYNC0_SIZE),
};

static struct resource smc91x_resources[] = {
	[0] = {
	       .start = MP200_ETHER_BASE,
	       .end = MP200_ETHER_BASE + MP200_ETHER_SIZE - 1,
	       .flags = IORESOURCE_MEM,
	       },
	[1] = {
	       .start = INT_GPIO_ETHER_INT,
	       .end = INT_GPIO_ETHER_INT,
	       .flags = IORESOURCE_IRQ,
	       },
};
static struct platform_device smc91x_device = {
	.name = "smc91x",
	.id = -1,
	.num_resources = ARRAY_SIZE(smc91x_resources),
	.resource = smc91x_resources,
};

/* 2006.10.19 has */
#ifdef CONFIG_SONY_ICX1087
static struct resource s1r72v17_resources[] = {
	[0] = {
	       .start = MP200_EXTUSB_BASE,
	       .end = MP200_EXTUSB_BASE + MP200_EXTUSB_SIZE - 1,
	       .flags = IORESOURCE_MEM,
	       },
	[1] = {
	       .start = INT_GPIO_EXTUSB_INT,
	       .end = INT_GPIO_EXTUSB_INT,
	       .flags = IORESOURCE_IRQ,
	       },
};
static struct platform_device s1r72v17_device = {
	.name = "s1r72v17",
	.id = -1,
	.num_resources = ARRAY_SIZE(s1r72v17_resources),
	.resource = s1r72v17_resources,
};
#endif

#ifdef CONFIG_MP201
static struct mtd_partition mp200_nand_partition[] = {
#ifdef CONFIG_SONY_ICX1087 /* 2006.05.18 yam */
	{
	.name = "NAND FS0",
	.offset = 0,
	.size = 0x10000000
	},
	{
	.name = "NAND FS1",
	.offset = MTDPART_OFS_APPEND,
 	.size = MTDPART_SIZ_FULL
	}
#else
	{
	 .name = "nand device",
	 .offset = 0,
	 .size = MTDPART_SIZ_FULL
	 },
#endif
};
static struct flash_platform_data mp200_nand_data = {
	.map_name = "mp200_nand",
	.parts = mp200_nand_partition,
	.nr_parts = ARRAY_SIZE(mp200_nand_partition),
};
static struct platform_device mp200_nand_device = {
	.name = "mp200_nand",
	.id = -1,
	.dev = {
		.platform_data = &mp200_nand_data,
		},
};
#else
static struct mtd_partition mp200_nand_partition[] = {
	{
	 .name = "nand device",
	 .offset = 0,
	 .size = (64 * 1024 * 1024)
	 },
};
static struct flash_platform_data mp200_nand_data = {
	.map_name = "mp200_nand",
	.width = NAND_BUSWIDTH_16,
	.parts = mp200_nand_partition,
	.nr_parts = ARRAY_SIZE(mp200_nand_partition),
};
static struct resource mp200_nand_resource = {
	.start = MP200_NAND_IO_BASE,
	.end = MP200_NAND_IO_BASE + MP200_NAND_IO_SIZE - 1,
	.flags = IORESOURCE_MEM,
};
static struct platform_device mp200_nand_device = {
	.name = "mp200_nand",
	.id = -1,
	.dev = {
		.platform_data = &mp200_nand_data,
		},
	.num_resources = 1,
	.resource = &mp200_nand_resource,
};
#endif				/* CONFIG_MP201 */

/* 2006.05.12 has */
#ifdef CONFIG_SONY_ICX1087
static struct mtd_partition mp200_flash0_partitions[] = {
	{
	 .name = "Boot",
	 .offset = 0,
	 .size = 0x00100000,
	 .mask_flags = MTD_WRITEABLE,	/* force read-only */
	 },
	{
	 .name = "Kernel",
	 .offset = MTDPART_OFS_APPEND,
	 .size = 0x00300000,
	 .mask_flags = MTD_WRITEABLE,	/* force read-only */
	 },
	{
	 .name = "Root",
	 .offset = MTDPART_OFS_APPEND,
	 .size = 0x00C00000,
	 .mask_flags = MTD_WRITEABLE,	/* force read-only */
	 },
};
#else
static struct mtd_partition mp200_flash0_partitions[] = {
	{
	 .name = "Boot",
	 .offset = 0,
	 .size = 0x00200000,
	 .mask_flags = MTD_WRITEABLE,	/* force read-only */
	 },
	{
	 .name = "Kernel",
	 .offset = MTDPART_OFS_APPEND,
	 .size = 0x00400000,
	 .mask_flags = MTD_WRITEABLE,	/* force read-only */
	 },
	{
	 .name = "Root",
	 .offset = MTDPART_OFS_APPEND,
	 .size = 0x01a00000,
	 .mask_flags = MTD_WRITEABLE,	/* force read-only */
	 },
};
#endif
static struct flash_platform_data mp200_flash0_data = {
	.map_name = "cfi_probe",
	.width = 2,
	.parts = mp200_flash0_partitions,
	.nr_parts = ARRAY_SIZE(mp200_flash0_partitions),
};
static struct resource mp200_flash0_resource = {
	.start = MP200_FLASH_BASE,
/* 2006.05.12 has */
#ifdef CONFIG_SONY_ICX1087
	.end = MP200_FLASH_BASE + SZ_16M - 1,
#else
	.end = MP200_FLASH_BASE + SZ_32M - 1,
#endif
	.flags = IORESOURCE_MEM,
};
static struct platform_device mp200_flash0_device = {
	.name = "mp200_flash",
	.id = 0,
	.dev = {
		.platform_data = &mp200_flash0_data,
		},
	.num_resources = 1,
	.resource = &mp200_flash0_resource,
};

#ifndef CONFIG_MP201
static struct mtd_partition mp200_flash1_partitions[] = {
	{
	 .name = "JFFS2",
	 .offset = 0,
	 .size = MTDPART_SIZ_FULL,
	 .mask_flags = 0},
};
static struct flash_platform_data mp200_flash1_data = {
	.map_name = "cfi_probe",
	.width = 2,
	.parts = mp200_flash1_partitions,
	.nr_parts = ARRAY_SIZE(mp200_flash1_partitions),
};
static struct resource mp200_flash1_resource = {
	.start = (MP200_FLASH_BASE + SZ_32M),
	.end = (MP200_FLASH_BASE + SZ_32M) + SZ_32M - 1,
	.flags = IORESOURCE_MEM,
};
static struct platform_device mp200_flash1_device = {
	.name = "mp200_flash",
	.id = 1,
	.dev = {
		.platform_data = &mp200_flash1_data,
		},
	.num_resources = 1,
	.resource = &mp200_flash1_resource,
};

static struct platform_device mp200_touchscreen_device = {
	.name = "mp200_touchscreen",
	.id = -1,
};
#endif				/* CONFIG_MP201 */

#ifdef CONFIG_MP201
static struct platform_device *devs[] __initdata = {
	&smc91x_device,
	&mp200_nand_device,
	&mp200_flash0_device,
/* 2006.10.19 has */
#ifdef CONFIG_SONY_ICX1087
	&s1r72v17_device,
#endif
};
#else
static struct platform_device *devs[] __initdata = {
	&smc91x_device,
	&mp200_nand_device,
	&mp200_flash0_device,
	&mp200_flash1_device,
	&mp200_touchscreen_device,
};
#endif				/* CONFIG_MP201 */

static void __init fixup_mp200_board(struct machine_desc *desc,
				     struct tag *tags, char **cmdline,
				     struct meminfo *mi)
{
	/* none */
}

static void __init mp200_board_map_io(void)
{
	mp200_map_io();
	iotable_init(mp200_board_io_desc, ARRAY_SIZE(mp200_board_io_desc));
#ifdef CONFIG_MP201
	mp200_serial_init();
#else
	mp200_serial_init(mp200_serial_ports);
#endif
}

static void __init mp200_board_init(void)
{
	/* initialize GPIO register */
	/* GPIO set output port  */
/* 2006.10.13 has */
#ifdef CONFIG_SONY_ICX1087
	mp200_gio_set_mode_output(GPIO_OUTPORT);
#else
	mp200_gio_set_mode_output(~GPIO_INPORT);
#endif

	/* GPIO set input port */
	mp200_gio_set_mode_input(GPIO_INPORT);

/* 2007.03.02 has */
#ifdef CONFIG_SONY_ICX1087
	/* modify pcm0 pull control.(U-Boot bug) */
	mp200_pmu_write_reg(SMU_PULLCTL_PCM,SMU_PULLCTL_PCM0_VALUE,SMU_PULLCTL_PCM0_MASK);
#endif
}

static int __init mp200_board_device_init(void)
{
	return platform_add_devices(devs, ARRAY_SIZE(devs));
}

arch_initcall(mp200_board_device_init);

MACHINE_START(MP200, "MP200")
    MAINTAINER("NEC Electronics Corp.")
    BOOT_MEM(PHYS_OFFSET, MP200_BANK5_BASE, IO_ADDRESS(MP200_BANK5_BASE))
    BOOT_PARAMS(PHYS_OFFSET + 0x100)
    FIXUP(fixup_mp200_board)
    MAPIO(mp200_board_map_io)
    INITIRQ(mp200_init_irq)
    INIT_MACHINE(mp200_board_init)
    .timer = &mp200_timer, MACHINE_END
