/*
  mp201regs.c -- Register monitor for MP201

  Copyright 2006 Sony Corporation.

	This code is based on SA1110's register moniter

 Usage:
 # insmod mp201regs.ko
 # cd /proc/mp201regs
 use "cat" or "echo" command.
 # cd /
 # rmmod mp201regs
*/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/config.h>
#include <linux/kernel.h>		/* for printk(), pr_debug() */
#include <linux/module.h>		/* for all module */
#include <linux/init.h>			/* for module initialize/cleanup macro */
#include <linux/proc_fs.h>		/* for /proc file system */
#include <linux/errno.h>		/* for error number */
#include <asm/uaccess.h>		/* for copy_to_user(), copy_from_user() */
#include <asm/hardware.h>		/* for IO_ADDRESS() macro */
#include <asm/arch/pwc.h>		/* for pwc_read(), pwc_write() */

#define MY_MODULE_VER	"v0.6"
#define MY_MODULE_NAME	"mp201regs"

#define PrDbg(fmt, arg...)	printk(KERN_DEBUG MY_MODULE_NAME ": " fmt,##arg)
#define PrInfo(fmt, arg...)	printk(KERN_INFO MY_MODULE_NAME ": " fmt,##arg)
#define PrWarn(fmt, arg...)	printk(KERN_WARNING MY_MODULE_NAME ": " fmt,##arg)
#define PrErr(fmt, arg...)	printk(KERN_ERR MY_MODULE_NAME ": " fmt,##arg)

#define REG_DIRNAME "mp201regs"

#define RO	(S_IRUSR | S_IRGRP)	/* read only */
#define WO	(S_IWUSR | S_IWGRP)	/* write only */
#define RW	(RO|WO)				/* read/write */

typedef struct regs_entry {
	uint32_t	addr;		/* physical address */
	unsigned	size;		/* register size (1, 2 or 4) */
	mode_t		rw;			/* access type (RO, WO or RW) */
	const char	*name;		/* register name */
	const char	*desc;		/* description (optional) */
	unsigned 	low_ino;	/* i-node of /proc entry */
} regs_entry_t;

#define NUM_REG_ENTRY	(sizeof(Regs) / sizeof(Regs[0]))

static regs_entry_t Regs[] = {
/* addr,    size r/w  name,              description */
 /* Timer1-5 */
 {0x40000000, 4, RW, "TIMER1_TM_OP",	"default=0x00000000"},
 {0x40000004, 4, WO, "TIMER1_TM_CLR",	"default=0x00000000"},
 {0x40000008, 4, RW, "TIMER1_TM_SET",	"default=0x00000000"},
 {0x4000000c, 4, RO, "TIMER1_RCR",		"default=0x00000000"},
 {0x40000014, 4, RO, "TIMER1_TM_SCLR",	"default=0x00000000"},
 {0x40010000, 4, RW, "TIMER2_TM_OP",	"default=0x00000000"},
 {0x40010004, 4, WO, "TIMER2_TM_CLR",	"default=0x00000000"},
 {0x40010008, 4, RW, "TIMER2_TM_SET",	"default=0x00000000"},
 {0x4001000c, 4, RO, "TIMER2_RCR",		"default=0x00000000"},
 {0x40010014, 4, RO, "TIMER2_TM_SCLR",	"default=0x00000000"},
 {0x40020000, 4, RW, "TIMER3_TM_OP",	"default=0x00000000"},
 {0x40020004, 4, WO, "TIMER3_TM_CLR",	"default=0x00000000"},
 {0x40020008, 4, RW, "TIMER3_TM_SET",	"default=0x00000000"},
 {0x4002000c, 4, RO, "TIMER3_RCR",		"default=0x00000000"},
 {0x40020014, 4, RO, "TIMER3_TM_SCLR",	"default=0x00000000"},
 {0x40030000, 4, RW, "TIMER4_TM_OP",	"default=0x00000000"},
 {0x40030004, 4, WO, "TIMER4_TM_CLR",	"default=0x00000000"},
 {0x40030008, 4, RW, "TIMER4_TM_SET",	"default=0x00000000"},
 {0x4003000c, 4, RO, "TIMER4_RCR",		"default=0x00000000"},
 {0x40030014, 4, RO, "TIMER4_TM_SCLR",	"default=0x00000000"},
 {0x40040000, 4, RW, "TIMER5_TM_OP",	"default=0x00000000"},
 {0x40040004, 4, WO, "TIMER5_TM_CLR",	"default=0x00000000"},
 {0x40040008, 4, RW, "TIMER5_TM_SET",	"default=0x00000000"},
 {0x4004000c, 4, RO, "TIMER5_RCR",		"default=0x00000000"},
 {0x40040014, 4, RO, "TIMER5_TM_SCLR",	"default=0x00000000"},
 /* Watchdog Timer1 */
 {0x40050000, 4, RW, "WATCHDOG1_TM_OP",		"default=0x00000000"},
 {0x40050004, 4, WO, "WATCHDOG1_TM_CLR",	"default=0x00000000"},
 {0x40050008, 4, RW, "WATCHDOG1_TM_SET",	"default=0x00000000"},
 {0x4005000c, 4, RO, "WATCHDOG1_RCR",		"default=0x00000000"},
 {0x40050014, 4, RO, "WATCHDOG1_TM_SCLR",	"default=0x00000000"},

 /* DMA M2P LCH4 setting */
 {0x40091400, 4, RW, "DMA_M2P_LCH4_AADD",					"Source Start Addr"},
 {0x40091404, 4, RO, "DMA_M2P_LCH4_AADP",					"Source Addr Ptr"},
 {0x40091408, 4, RW, "DMA_M2P_LCH4_AOFF",					"Source Addr Offset"},
 {0x4009140c, 4, RW, "DMA_M2P_LCH4_ASIZE",					"Source Block Size"},
 {0x40091410, 4, RW, "DMA_M2P_LCH4_ASIZE_COUNT",			"Source Block Count"},
 {0x40091420, 4, RW, "DMA_M2P_LCH4_BADD",					"Destination Addr"},
 {0x40091440, 4, RW, "DMA_M2P_LCH4_LENG",					"Length"},
 {0x40091444, 4, RO, "DMA_M2P_LCH4_LENG_RCOUNT",			"Read Length Count"},
 {0x40091448, 4, RO, "DMA_M2P_LCH4_LENG_WCOUNT",			"Write Length Count"},
 {0x40091450, 4, RW, "DMA_M2P_LCH4_MODE",					"0x00000000"},
 /* DMA control */
 {0x40098000, 4, WO, "DMA_ARM_CONT",						"0x00000000"},
 {0x40098004, 4, RO, "DMA_ARM_CONTSTATUS",					"0x00000000"},
 {0x40098008, 4, WO, "DMA_ARM_END",							"0x00000000"},
 /* DMA interrupt */
 {0x40098100, 4, RW, "DMA_ARM_LCHx_INT_SEL",				"0x00000000"},
 {0x40098200, 4, RO, "DMA_ARM_PE0_LCH0LCH3_INT_CONT",		"0x00000000"},
 {0x40098204, 4, RO, "DMA_ARM_PE0_LCH0LCH3_INT_RAW",		"0x00000000"},
 {0x40098208, 4, RW, "DMA_ARM_PE0_LCH0LCH3_INT_ENABLE",		"0x00000000"},
 {0x4009820c, 4, RW, "DMA_ARM_PE0_LCH0LCH3_INT_ENABLE_CL",	"0x00000000"},
 {0x40098210, 4, WO, "DMA_ARM_PE0_LCH0LCH3_INT_REQ_CL",		"0x00000000"},
 /* DMA control for M2P (Memory to Peripheral) */
 {0x40099000, 4, WO, "DMA_M2P_CONT",						"0x00000000"},
 {0x40099004, 4, RO, "DMA_M2P_CONTSTATUS",					"0x00000000"},
 {0x40099008, 4, WO, "DMA_M2P_END",							"0x00000000"},
 /* DMA interrupt for M2P */
 {0x40099100, 4, RW, "DMA_M2P_LCHx_INT_SEL",				"0x00000000"},
 {0x40099200, 4, RO, "DMA_M2P_PE0_LCH0LCH3_INT_CONT",		"0x00000000"},
 {0x40099204, 4, RO, "DMA_M2P_PE0_LCH0LCH3_INT_RAW",		"0x00000000"},
 {0x40099208, 4, RW, "DMA_M2P_PE0_LCH0LCH3_INT_ENABLE",		"0x00000000"},
 {0x4009920c, 4, RW, "DMA_M2P_PE0_LCH0LCH3_INT_ENABLE_CL",	"0x00000000"},
 {0x40099210, 4, WO, "DMA_M2P_PE0_LCH0LCH3_INT_REQ_CL",		"0x00000000"},
 {0x40099220, 4, RO, "DMA_M2P_PE0_LCH4LCH7_INT_CONT",		"0x00000000"},
 {0x40099224, 4, RO, "DMA_M2P_PE0_LCH4LCH7_INT_RAW",		"0x00000000"},
 {0x40099228, 4, RW, "DMA_M2P_PE0_LCH4LCH7_INT_ENABLE",		"0x00000000"},
 {0x4009922c, 4, RW, "DMA_M2P_PE0_LCH4LCH7_INT_ENABLE_CL",	"0x00000000"},
 {0x40099230, 4, WO, "DMA_M2P_PE0_LCH4LCH7_INT_REQ_CL",		"0x00000000"},
 /* DMA control (secure) */
 {0x48028000, 4, WO, "DMA_ARM_CONT_SEC",					"0x00000000"},
 {0x48028004, 4, RO, "DMA_ARM_CONTSTATUS_SEC",				"0x00000000"},
 {0x48028008, 4, WO, "DMA_ARM_END_SEC",						"0x00000000"},
 /* DMA control for M2P (Memory to Peripheral) secure */
 {0x48029000, 4, WO, "DMA_M2P_CONT_SEC",					"0x00000000"},
 {0x48029004, 4, RO, "DMA_M2P_CONTSTATUS_SEC",				"0x00000000"},
 {0x48029008, 4, WO, "DMA_M2P_END_SEC",						"0x00000000"},

 /* PCM0 */
 {0x400c0000, 4, RW, "PM0_FUNC_SEL",	"0x00000000"},
 {0x400c0004, 4, WO, "PM0_TXRX_EN",		""},
 {0x400c0004, 4, RO, "PM0_TXRX_MON",	"0x00000000"},
 {0x400c0008, 4, WO, "PM0_TXRX_DIS",	""},
 {0x400c000c, 4, RW, "PM0_CYCLE",		"0x00000000"},
 {0x400c0010, 4, RO, "PM0_RAW",			"0x00000000"},
 {0x400c0014, 4, RO, "PM0_STATUS",		"0x00000000"},
 {0x400c0018, 4, WO, "PM0_ENSET",		""},
 {0x400c0018, 4, RO, "PM0_ENMON",		"0x00000000"},
 {0x400c001c, 4, WO, "PM0_ENCLR",		""},
 {0x400c0020, 4, WO, "PM0_CLEAR",		""},
 {0x400c0024, 4, RW, "PM0_TXQ",			"0x00000000"},
 {0x400c0028, 4, RO, "PM0_RXQ",			"0x00000000"},
 {0x400c0030, 4, RW, "PM0_CYCLE2",		"0x00000000"},

 /* SPI0 */
 {0x40130000, 4, RW, "SP0_MODE",		"0x00000000"},
 {0x40130004, 4, RW, "SP0_POL",			"0x00007000"},
 {0x40130008, 4, RW, "SP0_CONTROL",		"0x00000040"},
 {0x4013000c, 4, RW, "SP0_TX_TIM",		"0x00000000"},
 {0x40130010, 4, WO, "SP0_TX_DATA",		"0x00000000"},
 {0x40130014, 4, RO, "SP0_RX_DATA",		""},
 {0x40130018, 4, RO, "SP0_STATUS",		"0x00000000"},
 {0x4013001c, 4, RO, "SP0_RAW_STATUS",	"0x00000000"},
 {0x40130020, 4, RW, "SP0_ENSET",		"0x00000000"},
 {0x40130024, 4, WO, "SP0_ENCLR",		"0x00000000"},
 {0x40130028, 4, WO, "SP0_FFCLR",		"0x00000000"},

 /* Timer0 */
 {0xc0000000, 4, RW, "TIMER0_TM_OP",	"default=0x00000000"},
 {0xc0000004, 4, WO, "TIMER0_TM_CLR",	"default=0x00000000"},
 {0xc0000008, 4, RW, "TIMER0_TM_SET",	"default=0x00000000"},
 {0xc000000c, 4, RO, "TIMER0_RCR",		"default=0x00000000"},
 {0xc0000014, 4, RO, "TIMER0_TM_SCLR",	"default=0x00000000"},
 /* Watchdog Timer0 */
 {0xc0010000, 4, RW, "WATCHDOG0_TM_OP",		"default=0x00000000"},
 {0xc0010004, 4, WO, "WATCHDOG0_TM_CLR",	"default=0x00000000"},
 {0xc0010008, 4, RW, "WATCHDOG0_TM_SET",	"default=0x00000000"},
 {0xc001000c, 4, RO, "WATCHDOG0_RCR",		"default=0x00000000"},
 {0xc0010014, 4, RO, "WATCHDOG0_TM_SCLR",	"default=0x00000000"},

 /* LCD */
 {0xc0040000, 4, RW, "LCD_CONTROL",		"0x00000000"},
 {0xc0040008, 4, RW, "LCD_LCDOUT",		"0x00000000"},
 {0xc004000c, 4, RO, "LCD_STATUS",		"0x00000000"},
 {0xc0040010, 4, RW, "LCD_AREAADR_A",	"0x00000000"},
 {0xc0040014, 4, RW, "LCD_AREAADR_B",	"0x00000000"},
 {0xc0040018, 4, RW, "LCD_AREAADR_C",	"0x00000000"},
 {0xc004001c, 4, RW, "LCD_HOFFSET",		"0x00000000"},
 {0xc0040020, 4, RW, "LCD_HTOTAL",		"0x00000000"},
 {0xc0040024, 4, RW, "LCD_HAREA",		"0x00000000"},
 {0xc0040028, 4, RW, "LCD_HEDGE1",		"0x00000000"},
 {0xc004002c, 4, RW, "LCD_HEDGE2",		"0x00000000"},
 {0xc0040030, 4, RW, "LCD_VTOTAL",		"0x00000000"},
 {0xc0040034, 4, RW, "LCD_VAREA",		"0x00000000"},
 {0xc0040038, 4, RW, "LCD_VEDGE1",		"0x00000000"},
 {0xc004003c, 4, RW, "LCD_VEDGE2",		"0x00000000"},
 {0xc0040040, 4, RW, "LCD_BUSSEL",		"0x00000000"},
 {0xc0040044, 4, RW, "LCD_FRAMESEL",	"0x00000001"},
 {0xc0040048, 4, RW, "LCD_GAMMACTRL",	"0x00000000"},
 {0xc004004c, 4, RW, "LCD_GAMMAADR",	"0x00000000"},
 {0xc0040050, 4, RW, "LCD_GAMMADATA",	"0x00000000"},
 {0xc0040054, 4, RW, "LCD_REVERSE",		"0x00000000"},
 {0xc0040058, 4, RW, "LCD_DATAREQ",		"0x00000000"},
 {0xc0040060, 4, RO, "LCD_INTSTATUS",	"0x00000000"},
 {0xc0040064, 4, RO, "LCD_INTRAWSTATUS","0x00000000"},
 {0xc0040068, 4, RW, "LCD_INTENSET",	"0x00000000"},
 {0xc004006c, 4, WO, "LCD_INTENCLR",	"0x00000000"},
 {0xc0040070, 4, WO, "LCD_INTFFCLR",	"0x00000000"},
 {0xc0040074, 4, RW, "LCD_ERRORADR",	"0x00000000"},

 /* GPIO */
 {0xc0050000, 4, RW, "GPIO0_EO",		"0x00000000, 16bit"},
 {0xc0050004, 4, RW, "GPIO0_EI",		"0x00000000, 16bit"},
 {0xc0050008, 4, RW, "GPIO0_DO0",		"0x00000000, 32bit"},
 {0xc005000c, 4, RO, "GPIO0_DI",		"16bit"},
 {0xc0050010, 4, RW, "GPIO0_IA",		"0x00000000, 16bit"},
 {0xc0050014, 4, RW, "GPIO0_IMN",		"0x00000000, 16bit"},
 {0xc0050018, 4, RW, "GPIO0_IME",		"0x00000000, 16bit"},
 {0xc005001c, 4, RO, "GPIO0_RAW",		"0x00000000, 16bit"},
 {0xc0050020, 4, RO, "GPIO0_MST",		"0x00000000, 16bit"},
 {0xc0050024, 4, WO, "GPIO0_IR",		"0x00000000, 16bit"},
 {0xc0050028, 4, RW, "GPIO0_ISE",		"0x00000000, 16bit"},
 {0xc005002c, 4, RW, "GPIO0_IDTL",		"0x00000000, 32bit"},
 {0xc0050030, 4, RW, "GPIO0_IDTH",		"0x00000000, 32bit"},
 {0xc0050100, 4, RW, "GPIO1_EO",		"0x00000000, 16bit"},
 {0xc0050104, 4, RW, "GPIO1_EI",		"0x00000000, 16bit"},
 {0xc0050108, 4, RW, "GPIO1_DO0",		"0x00000000, 32bit"},
 {0xc005010c, 4, RO, "GPIO1_DI",		"16bit"},
 {0xc0050110, 4, RW, "GPIO1_IA",		"0x00000000, 16bit"},
 {0xc0050114, 4, RW, "GPIO1_IMN",		"0x00000000, 16bit"},
 {0xc0050118, 4, RW, "GPIO1_IME",		"0x00000000, 16bit"},
 {0xc005011c, 4, RO, "GPIO1_RAW",		"0x00000000, 16bit"},
 {0xc0050120, 4, RO, "GPIO1_MST",		"0x00000000, 16bit"},
 {0xc0050124, 4, WO, "GPIO1_IR",		"0x00000000, 16bit"},
 {0xc0050128, 4, RW, "GPIO1_ISE",		"0x00000000, 16bit"},
 {0xc005012c, 4, RW, "GPIO1_IDTL",		"0x00000000, 32bit"},
 {0xc0050130, 4, RW, "GPIO1_IDTH",		"0x00000000, 32bit"},
 {0xc0050200, 4, RW, "GPIO2_EO",		"0x00000000, 16bit"},
 {0xc0050204, 4, RW, "GPIO2_EI",		"0x00000000, 16bit"},
 {0xc0050208, 4, RW, "GPIO2_DO0",		"0x00000000, 32bit"},
 {0xc005020c, 4, RO, "GPIO2_DI",		"16bit"},
 {0xc0050210, 4, RW, "GPIO2_IA",		"0x00000000, 16bit"},
 {0xc0050214, 4, RW, "GPIO2_IMN",		"0x00000000, 16bit"},
 {0xc0050218, 4, RW, "GPIO2_IME",		"0x00000000, 16bit"},
 {0xc005021c, 4, RO, "GPIO2_RAW",		"0x00000000, 16bit"},
 {0xc0050220, 4, RO, "GPIO2_MST",		"0x00000000, 16bit"},
 {0xc0050224, 4, WO, "GPIO2_IR",		"0x00000000, 16bit"},
 {0xc0050228, 4, RW, "GPIO2_ISE",		"0x00000000, 16bit"},
 {0xc005022c, 4, RW, "GPIO2_IDTL",		"0x00000000, 32bit"},
 {0xc0050230, 4, RW, "GPIO2_IDTH",		"0x00000000, 32bit"},
 {0xc0050300, 4, RW, "GPIO3_EO",		"0x00000000, 16bit"},
 {0xc0050304, 4, RW, "GPIO3_EI",		"0x00000000, 16bit"},
 {0xc0050308, 4, RW, "GPIO3_DO0",		"0x00000000, 32bit"},
 {0xc005030c, 4, RO, "GPIO3_DI",		"16bit"},
 {0xc0050310, 4, RW, "GPIO3_IA",		"0x00000000, 16bit"},
 {0xc0050314, 4, RW, "GPIO3_IMN",		"0x00000000, 16bit"},
 {0xc0050318, 4, RW, "GPIO3_IME",		"0x00000000, 16bit"},
 {0xc005031c, 4, RO, "GPIO3_RAW",		"0x00000000, 16bit"},
 {0xc0050320, 4, RO, "GPIO3_MST",		"0x00000000, 16bit"},
 {0xc0050324, 4, WO, "GPIO3_IR",		"0x00000000, 16bit"},
 {0xc0050328, 4, RW, "GPIO3_ISE",		"0x00000000, 16bit"},
 {0xc005032c, 4, RW, "GPIO3_IDTL",		"0x00000000, 32bit"},
 {0xc0050330, 4, RW, "GPIO3_IDTH",		"0x00000000, 32bit"},

 /* SMU reset control */
 {0xc8010000, 4, RW, "RESETREQ0",		"0x000000e0, 10bit"},
 {0xc8010004, 4, RW, "RESETREQ1A",		"0x00000269, 19bit"},
 {0xc8010008, 4, RW, "RESETREQ1B",		"0x00000000, 17bit"},
 {0xc801000c, 4, RW, "RESETREQDSP",		"0x00000000, 2bit"},
 {0xc8010010, 4, RW, "RESETREQ0_ENA",	"0x00000000, 10bit"},
 {0xc8010014, 4, RW, "RESETREQ1A_ENA",	"0x00000000, 19bit"},
 {0xc8010018, 4, RW, "RESETREQ1B_ENA",	"0x00000000, 17bit"},
 {0xc801001c, 4, RW, "RESETREQDSP_ENA",	"0x00000000, 2bit"},
 {0xc8010020, 4, RW, "WDT_INT_RESET",	"0x00000000, 6bit"},
 /* SMU clock control */
 {0xc8010040, 4, RW, "PLLSEL",				"0:OSC, 1:RTC, 2:PLL enable"},
 {0xc8010044, 4, RW, "PLLSEL2",				"0x00000001, 3bit"},
 {0xc8010048, 4, RW, "CLK_DOMAIN_DIV",		"0x00000002, 2bit"},
 {0xc801004c, 4, RW, "OSC_CTRL",			"0x00000005, 3bit"},
 {0xc8010050, 4, RW, "LPLL_CTRL1",			"0x00000000, 4bit"},
 {0xc8010054, 4, RW, "LPLL_CTRL2",			"LPLL 0:stop, 1:active"},
 {0xc8010058, 4, RW, "HPLL_CTRL1",			"0x0000007c, 7bit"},
 {0xc801005c, 4, RW, "HPLL_CTRL2",			"0x00000005, 5bit"},
 {0xc8010060, 4, RW, "HPLL_CTRL3",			"0x00000000, 1bit"},
 {0xc8010064, 4, RW, "HPLL_CTRL4",			"HPLL 0:stop, 1:active"},
 {0xc8010068, 4, RW, "PLLLOCK_CTRL",		"0x00000000, 2bit"},
 /* SMU gated clock control */
 {0xc8010080, 4, RW, "GCLKCTRL0",			"0x07ffffff, 27bit"},
 {0xc8010084, 4, RW, "GCLKCTRL1",			"0x3ffffffb, 30bit"},
 {0xc8010088, 4, RW, "GCLKCTRL2",			"0x1f0779ff, 29bit"},
 {0xc801008c, 4, RW, "GCLKCTRL0_ENA",		"0x00000000, 27bit"},
 {0xc8010090, 4, RW, "GCLKCTRL1_ENA",		"0x00000000, 30bit"},
 {0xc8010094, 4, RW, "GCLKCTRL2_ENA",		"0x00000000, 29bit"},
 {0xc8010098, 4, RW, "GCLKEN_POL",			"0x00000031, 6bit"},
 {0xc80100a0, 4, RW, "AHBCLKCTRL0",			"0x00000000, 14bit"},
 {0xc80100a4, 4, RW, "AHBCLKCTRL1",			"0x00000000, 9bit"},
 {0xc80100a8, 4, RW, "APBCLKCTRL",			"0x00000000, 21bit"},
 {0xc80100ac, 4, RW, "CLKCTRL",				"0x00000000, 3bit"},
 {0xc80100b0, 4, RW, "AUTO_FRQ_CHANGE",		"0x00070000, 28bit"},
 {0xc80100b4, 4, RW, "PLL_DOWN_EN",			"0x00000000, 1bit"},
 /* SMU async clock control */
 {0xc8010100, 4, RW, "LCD_LCLK_SEL",		"0:OSC, 1:LPLL, 2:HPLL"},
 {0xc8010104, 4, RW, "CAM_SCLK_SEL",		"0:OSC, 1:LPLL, 2:HPLL"},
 {0xc8010108, 4, RW, "ATA_DCLK_SEL",		"0:OSC, 1:LPLL, 2:HPLL"},
 {0xc801010c, 4, RW, "U70_SCLK_SEL",		"0:OSC, 1:LPLL, 2:HPLL"},
 {0xc8010110, 4, RW, "PM0_SCLK_SEL",		"0:OSC, 2:HPLL, 6:EXTPCMCLK"},
 {0xc8010114, 4, RW, "PM1_SCLK_SEL",		"0:OSC, 2:HPLL, 6:EXTPCMCLK"},
 {0xc8010118, 4, RW, "GIO_INT_CLK_SEL",		"0:OSC, 4:RTC"},
 {0xc801011c, 4, RW, "STM_SCLK_SEL",		"0:OSC, 5:EXTCLK"},
 {0xc8010120, 4, RW, "TI0_TIN_SEL",			"0:OSC, 4:RTC, 5:EXTCLK"},
 {0xc8010124, 4, RW, "TI1_TIN_SEL",			"0:OSC, 4:RTC, 5:EXTCLK"},
 {0xc8010128, 4, RW, "PWM_SCLK_SEL",		"0:OSC, 4:RTC, 5:EXTCLK"},
 {0xc801012c, 4, RW, "REFCLKO_SEL",			"0:OSC, 1:LPLL, 2:HPLL"},
 {0xc8010130, 4, RW, "LCD_LCLK_DIV",		"0x00000114, 9bit"},
 {0xc8010134, 4, RW, "TFD_LCLK_DIV",		"0x00000006, 5bit"},
 {0xc8010138, 4, RW, "CAM_SCLK_DIV",		"0x00000001, 5bit"},
 {0xc801013c, 4, RW, "ATA_DCLK_DIV",		"0x00000005, 3bit"},
 {0xc8010140, 4, RW, "SPI0_SCLK_DIV",		"0x00000003, 8bit"},
 {0xc8010144, 4, RW, "SPI1_SCLK_DIV",		"0x00000003, 8bit"},
 {0xc8010148, 4, RW, "U70_SCLK_DIV",		"0x00000003, 6bit"},
 {0xc801014c, 4, RW, "IIC_SCLK_DIV",		"0x00000000, 1bit"},
 {0xc8010150, 4, RW, "PM0_SCLK_DIV",		"0x00000009, 6bit"},
 {0xc8010154, 4, RW, "PM1_SCLK_DIV",		"0x00000009, 6bit"},
 {0xc8010158, 4, RW, "MWI_SCLK_DIV",		"0x00000013, 8bit"},
 {0xc801015c, 4, RW, "SDM_SCLK_DIV",		"0x00000004, 6bit"},
 {0xc8010160, 4, RW, "MSP_SCLK_DIV",		"0x00000005, 4bit"},
 {0xc8010164, 4, RW, "REFCLKO_DIV",			"0x00000005, 8bit"},
 /* SMU clock source change interrupt control */
 {0xc8010190, 4, RO, "SMU_INT_STATUS",		"0x00000000, 1bit"},
 {0xc8010194, 4, RO, "SMU_INT_RAW_STATUS",	"0x00000000, 1bit"},
 {0xc8010198, 4, WO, "SMU_INT_ENSET",		"1bit"},
 {0xc801019c, 4, WO, "SMU_INT_ENCLR",		"1bit"},
 {0xc80101a0, 4, RO, "SMU_INT_ENMON",		"0x00000000, 1bit"},
 {0xc80101a4, 4, WO, "SMU_INT_CLEAR",		"1bit"},
 /* SMU delay control */
 {0xc80101c0, 4, RW, "FLASHCLK_DLY",		"0x0100003f, 12bit"},
 {0xc80101c4, 4, RW, "SDR_CLK_DLY",			"0x00000000, 6bit"},
 {0xc80101c8, 4, RW, "SDR_UDCLK_DLY",		"0x0000003f, 12bit"},
 {0xc80101cc, 4, RW, "SDR_LDCLK_DLY",		"0x0000003f, 12bit"},
 /* SMU pin function control */
 {0xc8010200, 4, RW, "SMU_PCTL_ASYNC",		"0x0000003c, 6bit"},
 {0xc8010204, 4, RW, "SMU_PCTL_SDR",		"0x00000000, 2bit"},
 {0xc8010208, 4, RW, "SMU_PCTL_NAND",		"0x00000000, 9bit"},
 {0xc801020c, 4, RW, "SMU_PCTL_C1",			"0x00000000, 4bit"},
 {0xc8010210, 4, RW, "SMU_PCTL_DJTAG",		"0x00000001, 2bit"},
 {0xc8010214, 4, RW, "SMU_PCTL_CAM",		"0x00000000, 5bit"},
 {0xc8010218, 4, RW, "SMU_GIOCTL_PM1",		"0:PM1, 1:GPO"},
 {0xc801021c, 4, RW, "SMU_PCTL_SPI",		"0x00000006, 3bit"},
 {0xc8010220, 4, RW, "SMU_PCTL_C2",			"0x00000130, 17bit"},
 {0xc8010224, 4, RW, "SMU_PULLCTL_ASYNC",	"Master:0x010000cd, Slave:0x011000ef, 25bit"},
 {0xc8010228, 4, RW, "SMU_PULLCTL_SDR",		"0x0007ffff, 19bit"},
 {0xc801022c, 4, RW, "SMU_PULLCTL_NAND",	"0x00001fff, 29bit"},
 {0xc8010230, 4, RW, "SMU_PULLCTL_P1",		"0x00001f0f, 13bit"},
 {0xc8010234, 4, RW, "SMU_PULLCTL_CPUDEBUG","0x000c0003, 22bit"},
 {0xc8010238, 4, RW, "SMU_PULLCTL_DSPDEBUG","0x0000001f, 21bit"},
 {0xc801023c, 4, RW, "SMU_PULLCTL_CAM",		"0x000001ff, 13bit"},
 {0xc8010240, 4, RW, "SMU_PULLCTL_PCM",		"0x00000707, 25bit"},
 {0xc8010244, 4, RW, "SMU_PULLCTL_SPI",		"0x00000f0f, 28bit"},
 {0xc8010248, 4, RW, "SMU_PULLCTL_P2",		"0x0000207f, 29bit"},
 {0xc801024c, 4, RW, "SMU_PULLCTL_TEST",	"0x0000001f, 17bit"},
 {0xc8010250, 4, RW, "SMU_DRV_LCD",			"0x00005515, 16bit"},
 {0xc8010254, 4, RW, "SMU_DRV_SDRAM",		"0x00000000, 8bit"},
 {0xc8010258, 4, RW, "SMU_DRV_D1",			"0x01555555, 26bit"},
 {0xc801025c, 4, RW, "SMU_DRV_D2",			"0x05555555, 28bit"},
 /* SMU other control */
 {0xc8010280, 4, RW, "SMU_RESET_STATE",		"0x00000000, 8bit"},
 {0xc8010284, 4, RO, "MODE_ST",				"0x00000000, 5bit"},
 {0xc8010288, 4, RW, "SMU_GPO",				"0x00000000, 32bit"},
 /* SMU registers for AB0 */
	 /* omission */
 /* SMU registers for AB1 */
	 /* omission */
 /* SMU registers for CPI */
	 /* omission */
 /* SMU register for MSP */
 {0xc80103c0, 4, RW, "MSP_SYSC_OPT0",		"0x00000000, 5bit"},

 /* PowerIC power block */
 {0x00000002, 1, RW, "PWC_REGA",		"addr=0x02, default=0x03"},
 {0x00000004, 1, RW, "PWC_PSCNT1",		"addr=0x04, default=0x00"},
 {0x00000006, 1, RW, "PWC_PSCNT2",		"addr=0x06, default=0x00"},
 {0x00000008, 1, RW, "PWC_MODESET1",	"addr=0x08, default=0x00"},
 {0x0000000a, 1, RW, "PWC_VOUTSET1",	"addr=0x0a, default=0xac"},
 {0x0000000c, 1, RW, "PWC_VOUTSET2",	"addr=0x0c, default=0xa0"},
 {0x0000000e, 1, RW, "PWC_VOUTSET3",	"addr=0x0e, default=0x00"},
 /* PowerIC GPIO block */
 {0x00000010, 1, RW, "PWC_IOSEL1",		"addr=0x10, default=0x00"},
 {0x00000012, 1, RW, "PWC_IOSEL2",		"addr=0x12, default=0x00"},
 {0x00000014, 1, RW, "PWC_IOOUT1",		"addr=0x14, default=0x00"},
 {0x00000015, 1, RO, "PWC_IOIN1",		"addr=0x15"},
 {0x00000016, 1, RW, "PWC_IOOUT2",		"addr=0x16, default=0x00"},
 {0x00000017, 1, RO, "PWC_IOIN2",		"addr=0x17"},
 {0x00000018, 1, RW, "PWC_GPPUPD1",		"addr=0x18, default=0x00"},
 {0x0000001a, 1, RW, "PWC_GPPUPD2",		"addr=0x1a, default=0x00"},
 {0x0000001c, 1, RW, "PWC_GPPUPD3",		"addr=0x1c, default=0x55"},
 {0x0000001e, 1, RW, "PWC_GPPUPD4",		"addr=0x1e, default=0x55"},
 /* PowerIC  INTC block */
 {0x00000020, 1, RW, "PWC_FMASK1",		"addr=0x20, default=0x00"},
 {0x00000021, 1, RW, "PWC_FACTOR1",		"addr=0x21, default=0x00"},
 {0x00000022, 1, RW, "PWC_FMASK2",		"addr=0x22, default=0x00"},
 {0x00000023, 1, RW, "PWC_FACTOR2",		"addr=0x23, default=0x00"},
 {0x00000024, 1, RW, "PWC_FMASK3",		"addr=0x24, default=0x00"},
 {0x00000025, 1, RW, "PWC_FACTOR3",		"addr=0x25, default=0x00"},
 {0x00000026, 1, RW, "PWC_STATE1",		"addr=0x26, default=0x00"},
 {0x00000028, 1, RW, "PWC_STATE2",		"addr=0x28, default=0x00"},
 {0x0000002A, 1, RW, "PWC_STATE3",		"addr=0x2A, default=0x00"},
 {0x0000002C, 1, RW, "PWC_CLRFACT1",	"addr=0x2C, default=0x00"},
 {0x0000002E, 1, RW, "PWC_CLRFACT2",	"addr=0x2E, default=0x00"},
 {0x00000030, 1, RW, "PWC_CLRFACT3",	"addr=0x30, default=0x00"},
 {0x00000032, 1, RW, "PWC_DETMOD1L",	"addr=0x32, default=0x00"},
 {0x00000034, 1, RW, "PWC_DETMOD1H",	"addr=0x34, default=0x00"},
 {0x00000036, 1, RW, "PWC_DETMOD2L",	"addr=0x36, default=0x00"},
 {0x00000038, 1, RW, "PWC_DETMOD2H",	"addr=0x38, default=0x00"},
 {0x0000003A, 1, RW, "PWC_DETMOD3L",	"addr=0x3A, default=0x00"},
 {0x0000003C, 1, RW, "PWC_DETMOD3H",	"addr=0x3C, default=0x00"},
 {0x0000003E, 1, RW, "PWC_BMASK1",		"addr=0x3E, default=0x00"},
 {0x00000040, 1, RW, "PWC_BMASK2",		"addr=0x40, default=0x00"},
 {0x00000042, 1, RW, "PWC_BMASK3",		"addr=0x42, default=0x00"},
 {0x00000044, 1, RW, "PWC_REGRSTMSK1",	"addr=0x44, default=0x00"},
 {0x00000046, 1, RW, "PWC_REGRSTMSK2",	"addr=0x46, default=0x00"},
 {0x00000048, 1, RW, "PWC_SYSRSTMSK1",	"addr=0x48, default=0x00"},
 {0x0000004A, 1, RW, "PWC_SYSRSTMSK2",	"addr=0x4A, default=0x00"},
 {0x0000004C, 1, RW, "PWC_REGPWONMSK",	"addr=0x4C, default=0x00"},
 {0x0000006D, 1, RW, "PWC_WRSTFCT",		"addr=0x6D, default=0x00"},
 {0x0000006E, 1, RW, "PWC_WRSTCL",		"addr=0x6E, default=0x00"},
 {0x00000092, 1, RW, "PWC_WRSTMSK",		"addr=0x92, default=0x00"},
 {0x00000094, 1, RW, "PWC_WAKECONT",	"addr=0x94, default=0x00"},
 /* LSI ID & Version block */
 {0x0000006f, 1, RO, "PWC_LSIVER",		"addr=0x6f, default=0x??"},
};

#define IO_READ8(addr)	(*(volatile uint8_t *)IO_ADDRESS(addr))
#define IO_READ16(addr)	(*(volatile uint16_t *)IO_ADDRESS(addr))
#define IO_READ32(addr)	(*(volatile uint32_t *)IO_ADDRESS(addr))
#define	IO_WRITE8(val, addr) \
	do { *(volatile uint8_t *)IO_ADDRESS(addr) = (uint8_t)(val); } while (0)
#define	IO_WRITE16(val, addr) \
	do { *(volatile uint16_t *)IO_ADDRESS(addr) = (uint16_t)(val); } while (0)
#define	IO_WRITE32(val, addr) \
	do { *(volatile uint32_t *)IO_ADDRESS(addr) = (uint32_t)(val); } while (0)

static ssize_t
do_read_reg(struct file *file, char *userbuf, size_t nbytes, loff_t *ppos)
{
	unsigned	i_ino = file->f_dentry->d_inode->i_ino;
	unsigned	value;
	char	workbuf[160];
	int		count, i;

	if (*ppos > 0) { /* Assume reading completed in previous read */
		return 0; /* EOF */
	}

	/* search same i-node entry */
	for (i = 0; i < NUM_REG_ENTRY; i++) {
		if (Regs[i].low_ino == i_ino) {
			break;
		}
	}
	if (i == NUM_REG_ENTRY) {
		return -EINVAL;		/* entry not found */
	}

	if (Regs[i].rw == WO) {
		return -EACCES;
	}

PrDbg("p=%x, v=%x\n", Regs[i].addr, IO_ADDRESS(Regs[i].addr));

	if (Regs[i].addr <= PWC_MAX) {
		pwc_read(Regs[i].addr, &value);
	} else {
		switch (Regs[i].size) {
		case 1:	value = IO_READ8(Regs[i].addr); break;
		case 2:	value = IO_READ16(Regs[i].addr); break;
		case 4:	value = IO_READ32(Regs[i].addr); break;
		default: PrWarn("driver bug. invalid size\n"); return -EIO;
		}
	}

	count = sprintf(workbuf, "%-34s: 0x%08x (%s)\n",
				Regs[i].name, value, Regs[i].desc);
	*ppos += count;

	if (count > nbytes) { /* Assume output can be read at one time */
		return -EINVAL;
	}

	if (copy_to_user(userbuf, workbuf, count)) {
		return -EFAULT;
	}
	return count;
}

static ssize_t
do_write_reg(struct file *file, const char *userbuf, size_t count, loff_t *ppos)
{
	unsigned	i_ino = file->f_dentry->d_inode->i_ino;
	char		workbuf[32];
	int			i;
	uint32_t	new_value;

	if (count >= sizeof(workbuf)) {
		return -EINVAL;
	}
	if (copy_from_user(workbuf, userbuf, count)) {
		return -EFAULT;
	}
	workbuf[count] = '\0';

	for (i = 0; i < NUM_REG_ENTRY; i++) {
		if (Regs[i].low_ino == i_ino) {
			break;
		}
	}
	if (i == NUM_REG_ENTRY) {
		return -EINVAL;
	}

	if (Regs[i].rw == RO) {
		return -EACCES;
	}

	new_value = simple_strtoul(workbuf, NULL, 0);

	if (Regs[i].addr <= PWC_MAX) {
		pwc_write(Regs[i].addr, new_value, 0xff);
	} else {
		switch (Regs[i].size) {
		case 1:	IO_WRITE8(new_value, Regs[i].addr); break;
		case 2:	IO_WRITE16(new_value, Regs[i].addr); break;
		case 4:	IO_WRITE32(new_value, Regs[i].addr); break;
		default: PrWarn("driver bug. invalid size\n"); return -EIO;
		}
	}

	return count;
}

static struct file_operations do_reg_operations = {
	read:	do_read_reg,
	write:	do_write_reg
};

static struct proc_dir_entry *reg_dir;

static int __init init_reg_monitor(void)
{
	struct proc_dir_entry *entry;
	int i;

	PrInfo(" " MY_MODULE_VER " started.\n");

	reg_dir = proc_mkdir(REG_DIRNAME, NULL);
	if (reg_dir == NULL) {
		PrErr("can't create /proc/" REG_DIRNAME "\n");
		return -ENOMEM;
	}

	for (i = 0; i < NUM_REG_ENTRY; i++) {
		entry = create_proc_entry(Regs[i].name, Regs[i].rw, reg_dir);
		if (entry == NULL) {
			PrErr("can't create /proc/" REG_DIRNAME "/%s\n",
				Regs[i].name);
			break;
		}
		Regs[i].low_ino = entry->low_ino;
		entry->proc_fops = &do_reg_operations;
	}

	/* error occurred ? */
	if (i < NUM_REG_ENTRY) {
		while (i) {
			remove_proc_entry(Regs[--i].name, reg_dir);
		}
		remove_proc_entry(REG_DIRNAME, NULL);
		return -ENOMEM;
	}

	return 0;
}

static void __exit cleanup_reg_monitor(void)
{
	int i;

	for (i = 0; i < NUM_REG_ENTRY; i++) {
		remove_proc_entry(Regs[i].name, reg_dir);
	}
	remove_proc_entry(REG_DIRNAME, NULL);
}

module_init(init_reg_monitor);
module_exit(cleanup_reg_monitor);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Sony Corporation.");
MODULE_DESCRIPTION("MP201 register monitor");
