/*
 *  File Name       : linux/arch/arm/mach-mp200/time.c
 *  Function        : time
 *  Release Version : Ver 1.01
 *  Release Date    : 2006/05/12
 *
 *  Copyright (C) NEC Electronics Corporation 2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/delay.h>

#include <asm/io.h>
#include <asm/mach/time.h>

#include <asm/arch/smu.h>
#include <asm/arch/pmu.h>
#include <asm/arch/preempt.h>

#include "time.h"

static struct tm_param_t tm_param[] = {
#ifdef CONFIG_MP201
	/* TIMER_TGn */
	{.usecs = TIMER_DEFAULT_TIME,.tm_op = 0,},
	{.usecs = DSP_COUNT_VALUE,.tm_op = 0,},
	{.usecs = TIMER_DEFAULT_TIME,.tm_op = 0,},
	{.usecs = TIMER_DEFAULT_TIME,.tm_op = 0,},
	/* TIMER_TIn */
	{},
	{},
	/* TIMER_TWn */
	{.usecs = TIMER_DEFAULT_TIME,},
	{.usecs = TIMER_DEFAULT_TIME,},
#else
	/* TIMER_TGn */
	{.usecs = TIMER_DEFAULT_TIME,.tm_op = 0,},
	{.usecs = TIMER_DEFAULT_TIME,.tm_op = 0,},
	{.usecs = TIMER_DEFAULT_TIME,.tm_op = 0,},
	{.usecs = TIMER_DEFAULT_TIME,.tm_op = 0,},
	{.usecs = TIMER_DEFAULT_TIME,.tm_op = 0,},
	{.usecs = TIMER_DEFAULT_TIME,.tm_op = 0,},
	/* TIMER_TIn */
	{},
	{},
	{},
	/* TIMER_TWn */
	{.usecs = TIMER_DEFAULT_TIME,},
	{.usecs = TIMER_DEFAULT_TIME,},
	{.usecs = TIMER_DEFAULT_TIME,},
#endif				/* CONFIG_MP201 */
};

static const struct tm_reg_t tm_reg[] = {
#ifdef CONFIG_MP201
	/* TIMER_TGn */
	{
	 .irq = INT_TI2,
	 .tm_op = IO_ADDRESS(MP200_TI2_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TI2_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TI2_BASE) + 0x0008,
	 .rcr = IO_ADDRESS(MP200_TI2_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TI1_TIN,
	 .rstdev = MP200_RESETDEVICE_TI2,
	 },
	{
	 .irq = INT_TI3,
	 .tm_op = IO_ADDRESS(MP200_TI3_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TI3_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TI3_BASE) + 0x0008,
	 .rcr = IO_ADDRESS(MP200_TI3_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TI1_TIN,
	 .rstdev = MP200_RESETDEVICE_TI3,
	 },
	{
	 .irq = INT_TI4,
	 .tm_op = IO_ADDRESS(MP200_TI4_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TI4_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TI4_BASE) + 0x0008,
	 .rcr = IO_ADDRESS(MP200_TI4_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TI1_TIN,
	 .rstdev = MP200_RESETDEVICE_TI4,
	 },
	{
	 .irq = INT_TI5,
	 .tm_op = IO_ADDRESS(MP200_TI5_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TI5_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TI5_BASE) + 0x0008,
	 .rcr = IO_ADDRESS(MP200_TI5_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TI1_TIN,
	 .rstdev = MP200_RESETDEVICE_TI5,
	 },
	/* TIMER_TIn */
	{
	 .irq = INT_TI0,
	 .tm_op = IO_ADDRESS(MP200_TI0_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TI0_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TI0_BASE) + 0x0008,
	 .rcr = IO_ADDRESS(MP200_TI0_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TI0_TIN,
	 .rstdev = MP200_RESETDEVICE_TI0,
	 .tin_sel = SMU_TI0_TIN_SEL,
	 },
	{
	 .irq = INT_TI1,
	 .tm_op = IO_ADDRESS(MP200_TI1_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TI1_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TI1_BASE) + 0x0008,
	 .rcr = IO_ADDRESS(MP200_TI1_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TI1_TIN,
	 .rstdev = MP200_RESETDEVICE_TI1,
	 .tin_sel = SMU_TI1_TIN_SEL,
	 },
	/* TIMER_TWn */
	{
	 .tm_op = IO_ADDRESS(MP200_WDT0_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_WDT0_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_WDT0_BASE) + 0x0008,
	 .rstdev = MP200_RESETDEVICE_TW0,
	 .clkctrldev = MP200_CLKCTRL_TI0PCLK,
	 .tin_sel = SMU_TI0_TIN_SEL,
	 },
	{
	 .tm_op = IO_ADDRESS(MP200_WDT1_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_WDT1_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_WDT1_BASE) + 0x0008,
	 .rstdev = MP200_RESETDEVICE_TW1,
	 .clkctrldev = MP200_CLKCTRL_TI1PCLK,
	 .tin_sel = SMU_TI1_TIN_SEL,
	 },
#else
	/* TIMER_TGn */
	{
	 .irq = INT_TG0,
	 .tm_op = IO_ADDRESS(MP200_TG0_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TG0_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TG0_BASE) + 0x0008,
	 .rcr = IO_ADDRESS(MP200_TG0_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TG0_TIN,
	 .rstdev = MP200_RESETDEVICE_TG0,
	 },
	{
	 .irq = INT_TG1,
	 .tm_op = IO_ADDRESS(MP200_TG1_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TG1_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TG1_BASE) + 0x0008,
	 .rcr = IO_ADDRESS(MP200_TG1_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TG1_TIN,
	 .rstdev = MP200_RESETDEVICE_TG1,
	 },
	{
	 .irq = INT_TG2,
	 .tm_op = IO_ADDRESS(MP200_TG2_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TG2_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TG2_BASE) + 0x0008,
	 .rcr = IO_ADDRESS(MP200_TG2_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TG2_TIN,
	 .rstdev = MP200_RESETDEVICE_TG2,
	 },
	{
	 .irq = INT_TG3,
	 .tm_op = IO_ADDRESS(MP200_TG3_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TG3_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TG3_BASE) + 0x0008,
	 .rcr = IO_ADDRESS(MP200_TG3_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TG3_TIN,
	 .rstdev = MP200_RESETDEVICE_TG3,
	 },
	{
	 .irq = INT_TG4,
	 .tm_op = IO_ADDRESS(MP200_TG4_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TG4_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TG4_BASE) + 0x0008,
	 .rcr = IO_ADDRESS(MP200_TG4_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TG4_TIN,
	 .rstdev = MP200_RESETDEVICE_TG4,
	 },
	{
	 .irq = INT_TG5,
	 .tm_op = IO_ADDRESS(MP200_TG5_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TG5_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TG5_BASE) + 0x0008,
	 .rcr = IO_ADDRESS(MP200_TG5_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TG5_TIN,
	 .rstdev = MP200_RESETDEVICE_TG5,
	 },
	/* TIMER_TIn */
	{
	 .irq = INT_TIMER0,
	 .tm_op = IO_ADDRESS(MP200_TIMER0_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TIMER0_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TIMER0_BASE) + 0x0008,
	 .rcr = IO_ADDRESS(MP200_TIMER0_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TI0_TIN,
	 .rstdev = MP200_RESETDEVICE_TI0,
	 .tin_sel = SMU_TI0TIN_SEL,
	 },
	{
	 .irq = INT_TIMER1,
	 .tm_op = IO_ADDRESS(MP200_TIMER1_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TIMER1_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TIMER1_BASE) + 0x0008,
	 .rcr = IO_ADDRESS(MP200_TIMER1_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TI1_TIN,
	 .rstdev = MP200_RESETDEVICE_TI1,
	 .tin_sel = SMU_TI1TIN_SEL,
	 },
	{
	 .irq = INT_TIMER2,
	 .tm_op = IO_ADDRESS(MP200_TIMER2_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_TIMER2_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_TIMER2_BASE) + 0x0008,
	 .rcr = IO_ADDRESS(MP200_TIMER2_BASE) + 0x000c,
	 .clkdev = MP200_CLOCKGATE_TI2_TIN,
	 .rstdev = MP200_RESETDEVICE_TI2,
	 .tin_sel = SMU_TI2TIN_SEL,
	 },
	/* TIMER_TWn */
	{
	 .tm_op = IO_ADDRESS(MP200_WDT0_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_WDT0_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_WDT0_BASE) + 0x0008,
	 .rstdev = MP200_RESETDEVICE_TW0,
	 .clkctrldev = MP200_CLKCTRL_TI0PCLK,
	 .tin_sel = SMU_TI0TIN_SEL,
	 },
	{
	 .tm_op = IO_ADDRESS(MP200_WDT1_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_WDT1_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_WDT1_BASE) + 0x0008,
	 .rstdev = MP200_RESETDEVICE_TW1,
	 .clkctrldev = MP200_CLKCTRL_TI1PCLK,
	 .tin_sel = SMU_TI1TIN_SEL,
	 },
	{
	 .tm_op = IO_ADDRESS(MP200_WDT2_BASE) + 0x0000,
	 .tm_clr = IO_ADDRESS(MP200_WDT2_BASE) + 0x0004,
	 .tm_set = IO_ADDRESS(MP200_WDT2_BASE) + 0x0008,
	 .rstdev = MP200_RESETDEVICE_TW2,
	 .clkctrldev = MP200_CLKCTRL_TI1PCLK,
	 .tin_sel = SMU_TI2TIN_SEL,
	 },
#endif				/* CONFIG_MP201 */
};

#ifdef CONFIG_MP201
static const unsigned int intc_raw0 = VA_INTC + MP200_INTC_IT0_RAW0;
#else
static const unsigned int intc_raw1 = VA_INTC + MP200_INTC_IT0_RAW1;
#endif				/* CONFIG_MP201 */

static spinlock_t timer_spinlock = SPIN_LOCK_UNLOCKED;

unsigned long __noinstrument mp200_system_timer_read(int nr)
{
#ifndef CONFIG_MP201
	inl(tm_reg[nr].rcr);
#endif				/* CONFIG_MP201 */
	return inl(tm_reg[nr].rcr);
}

EXPORT_SYMBOL(mp200_system_timer_read);

void mp200_system_timer_start(int nr, unsigned int mode)
{
	outl((TSTART | TM_EN | mode), tm_reg[nr].tm_op);
}

EXPORT_SYMBOL(mp200_system_timer_start);

static unsigned long timer_interval = TIMER_INTERVAL_12M;

unsigned long __noinstrument mp200_timer_ticks_to_usecs(unsigned long nr_ticks)
{
	if (timer_interval == TIMER_INTERVAL_32K) {
		return COUNT_TO_USEC_32K(nr_ticks);
	} else {
		return COUNT_TO_USEC_12M(nr_ticks);
	}
}

#if defined(CONFIG_KFI)
static int mp200_system_timer_started = 0;

unsigned long __noinstrument machinecycles_to_usecs(unsigned long nr_ticks)
{
	return mp200_timer_ticks_to_usecs(nr_ticks);
}

unsigned long __noinstrument do_getmachinecycles(void)
{
	return (mp200_system_timer_started != 0 ?
		mp200_system_timer_read(TIMER_KFI) : 0);
}
#endif

/*
 * Returns number of ms since last clock interrupt.
 * Note that interrupts will have been disabled by do_gettimeoffset().
 */
static unsigned long mp200_gettimeoffset(void)
{
	unsigned long ticks;
	unsigned long status1;
	unsigned long status2;

	/* This function will be called with interrupt disabled.
	 * Check interrupt RAW register before and after reading counter.
	 * If counter is saturated and interrupt is not occurred,
	 * wait a few PCLK after reading counter to ensure
	 * whether interrupt handling is or is not done.
	 * (Determine we are called whether before or after interrupt
	 *  handler execution.)
	 */

#ifdef CONFIG_MP201
	status1 = inl(intc_raw0) & (1 << tm_reg[TIMER_SYSTEM].irq);
#else
	status1 = inl(intc_raw1) & (1 << (tm_reg[TIMER_SYSTEM].irq - 32));
#endif				/* CONFIG_MP201 */

	ticks = mp200_system_timer_read(TIMER_SYSTEM);

	/* counter saturated, but not interrupt */
	if ((ticks == 0) && (status1 == 0)) {
		/* wait for about 3 PCLK */
		__asm__ __volatile__("nop");
		__asm__ __volatile__("nop");
		__asm__ __volatile__("nop");
		__asm__ __volatile__("nop");
		__asm__ __volatile__("nop");
		__asm__ __volatile__("nop");
	}
#ifdef CONFIG_MP201
	status2 = inl(intc_raw0) & (1 << tm_reg[TIMER_SYSTEM].irq);
#else
	status2 = inl(intc_raw1) & (1 << (tm_reg[TIMER_SYSTEM].irq - 32));
#endif				/* CONFIG_MP201 */

	/* If interrupt is occurred, we need to add 1 tick because
	 * jiffies is not updated yet.
	 */
	if (status2 != 0) {
		if (ticks < (timer_interval / 2)) {
			ticks += (timer_interval + 1);
		}
	}

	return mp200_timer_ticks_to_usecs(ticks);
}

/*
 * IRQ handler for the timer
 */
static irqreturn_t
mp200_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	write_seqlock(&xtime_lock);

	timer_tick(regs);

	write_sequnlock(&xtime_lock);

	return IRQ_HANDLED;
}

static struct irqaction mp200_tick_count_timer_irq = {
	.name = "mp200_timer",
	.flags = (SA_NODELAY | SA_INTERRUPT),
	.handler = mp200_timer_interrupt
};

/*
 * Set up timer interrupt, and return the current time in seconds.
 */
static void __init mp200_init_timer(void)
{
	timer_set_clock_parm(TIMER_FULLSPEED);
	timer_set_clock(TIMER_INIT);

#if (defined(CONFIG_KFI) || defined(CONFIG_ILATENCY) || defined(CONFIG_DPM_STATS))
	mp200_system_timer_start(TIMER_KFI, 0);
#endif
#if defined(CONFIG_KFI)
	mp200_system_timer_started = 1;
#endif

	setup_irq(tm_reg[TIMER_SYSTEM].irq, &mp200_tick_count_timer_irq);
	mp200_timer.offset = mp200_gettimeoffset;

	mp200_system_timer_start(TIMER_SYSTEM, TO_EN);
}

struct sys_timer mp200_timer = {
	.init = mp200_init_timer,
	.offset = NULL,		/* Initialized later */
};

/*
 * Generic time functions
 */

static inline int check_validity_channel(unsigned int timer)
{
	if ((TIMER_TG_MIN <= timer) && (timer < TIMER_TG_MAX_NUM)
	    && (timer != TIMER_DSP)) {
		return 0;
	} else {
		return -ENODEV;
	}
}

static inline unsigned int get_count_value(unsigned timer, unsigned int usecs)
{
	unsigned int counts;

	if (usecs == TIMER_MAX_COUNT) {
		return TIMER_MAX_COUNT;
	}
#ifdef CONFIG_MP201
	if ((inl(SMU_TI1_TIN_SEL) & MASK_TIN_SEL) == TIN_SEL_32K) {
#else
	if (inl(SMU_TGNTIN_SEL) & (1 << (timer * 4))) {
#endif				/* CONFIG_MP201 */
		/* 32KHz */
		counts = USEC_TO_COUNT_32K(usecs);
	} else {
		/* 12MHz */
		counts = USEC_TO_COUNT_12M(usecs);
	}

	return counts;
}

static inline unsigned int
get_tw_count_value(unsigned timer, unsigned int usecs)
{
	unsigned int counts;

#ifdef CONFIG_MP201
	if ((inl(tm_reg[TIMER_TW0 + timer].tin_sel) & MASK_TIN_SEL) ==
	    TIN_SEL_32K) {
#else
	if ((inl(tm_reg[TIMER_TW0 + timer].tin_sel) & 1) != 0) {
#endif				/* CONFIG_MP201 */
		/* 32KHz */
		counts = USEC_TO_COUNT_32K(usecs);
	} else {
		/* 12MHz */
		counts = USEC_TO_COUNT_12M(usecs);
	}

	return counts;
}

static inline unsigned int get_usec_value(unsigned timer, unsigned int counts)
{
	unsigned int usecs;

#ifdef CONFIG_MP201
	if ((inl(SMU_TI1_TIN_SEL) & MASK_TIN_SEL) == TIN_SEL_32K) {
#else
	if (inl(SMU_TGNTIN_SEL) & (1 << (timer * 4))) {
#endif				/* CONFIG_MP201 */
		/* 32KHz */
		usecs = COUNT_TO_USEC_32K(counts);

	} else {
		/* 12MHz */
		usecs = COUNT_TO_USEC_12M(counts);
	}

	return usecs;
}

/*
 * Change internal timer parameter
 */
void timer_set_clock_parm(unsigned int mode)
{
	switch (mode) {
	case TIMER_RESUME:
	case TIMER_FULLSPEED:
		timer_interval = TIMER_INTERVAL;
#ifdef CONFIG_ILATENCY
		ticks_per_usec = TICKS_PER_USEC;
#endif
		break;
	case TIMER_SUSPEND:
	case TIMER_ECONOMY:
		timer_interval = TIMER_INTERVAL_32K;
#ifdef CONFIG_ILATENCY
		ticks_per_usec = TICKS_PER_USEC_32K;
#endif
		break;
	default:
		break;
	}
}

void timer_set_clock(unsigned int mode)
{
#ifdef CONFIG_MP201
	static unsigned int ti_tm_op[] = { 0, 0 };
	static unsigned int tw_tm_op[] = { 0, 0 };
	static unsigned int tg_tm_op[] = { 0, 0, 0, 0 };
#else
	static unsigned int ti_tm_op[] = { 0, 0, 0 };
	static unsigned int tw_tm_op[] = { 0, 0, 0 };
	static unsigned int tg_tm_op[] = { 0, 0, 0, 0, 0, 0 };
#endif				/* CONFIG_MP201 */
	int i;
	unsigned int count;

	switch (mode) {
	case TIMER_SUSPEND:
		/* get tm_op and stop timer */
		for (i = 0; i < TIMER_MAX_NUM; i++) {
			/* get tm_op */
			ti_tm_op[i] = inl(tm_reg[TIMER_TI0 + i].tm_op);
			tw_tm_op[i] = inl(tm_reg[TIMER_TW0 + i].tm_op);
			/* timer stop */
			outl(0, tm_reg[TIMER_TI0 + i].tm_op);
			outl(0, tm_reg[TIMER_TW0 + i].tm_op);
		}
		for (i = 0; i < TIMER_TG_MAX_NUM; i++) {
			/* get tm_op */
			tg_tm_op[i] = inl(tm_reg[TIMER_TG0 + i].tm_op);
			/* timer stop */
			outl(0, tm_reg[TIMER_TG0 + i].tm_op);
		}

		/* wait */
		udelay(TIMER_SET_DELAY);

		/* select TIN clock */
#ifdef CONFIG_MP201
		outl(TIN_SEL_32K, SMU_TI0_TIN_SEL);
		outl(TIN_SEL_32K, SMU_TI1_TIN_SEL);
#else
		outl(TIN_SEL_32K, SMU_TI0TIN_SEL);
		outl(TIN_SEL_32K, SMU_TI1TIN_SEL);
		outl(TIN_SEL_32K, SMU_TI2TIN_SEL);
		outl(TGNTIN_SEL_32K, SMU_TGNTIN_SEL);
#endif				/* CONFIG_MP201 */

		/* Auto clock OFF */
		mp200_pmu_clkctrl_off(MP200_CLKCTRL_TI0PCLK);
		mp200_pmu_clkctrl_off(MP200_CLKCTRL_TI1PCLK);

		/* reset device and close clockgate w/o TW0 */
		for (i = 0; i < TIMER_MAX_NUM; i++) {
			/* reset device */
			mp200_pmu_reset_device(tm_reg[TIMER_TI0 + i].rstdev);
			if (i == 0) {
				/* set TW0 counter */
				count = get_tw_count_value(0,
							   tm_param[TIMER_TW0].
							   usecs);
				outl(count, tm_reg[TIMER_TW0].tm_set);
				/* start TW0 */
				outl(tw_tm_op[0], tm_reg[TIMER_TW0].tm_op);
			} else {
				/* reset device */
				mp200_pmu_reset_device(tm_reg[TIMER_TW0 + i].
						       rstdev);
				/* close clockgate */
				mp200_pmu_close_clockgate(tm_reg[TIMER_TI0 + i].
							  clkdev);
			}
		}
		for (i = 0; i < TIMER_TG_MAX_NUM; i++) {
			/* reset device */
			mp200_pmu_reset_device(tm_reg[TIMER_TG0 + i].rstdev);
			/* close clockgate */
			mp200_pmu_close_clockgate(tm_reg[TIMER_TG0 + i].clkdev);
		}

#ifdef CONFIG_MP201
		mp200_pmu_close_clockgate(MP200_CLOCKGATE_TI1_TIN);
#endif				/* CONFIG_MP201 */
		mp200_pmu_close_clockgate(MP200_CLOCKGATE_TI0_PCLK);
		mp200_pmu_close_clockgate(MP200_CLOCKGATE_TI1_PCLK);

		break;

	case TIMER_RESUME:
	case TIMER_INIT:
		/* Auto clock OFF */
		mp200_pmu_clkctrl_off(MP200_CLKCTRL_TI0PCLK);
		mp200_pmu_clkctrl_off(MP200_CLKCTRL_TI1PCLK);

		mp200_pmu_open_clockgate(MP200_CLOCKGATE_TI0_PCLK);
		mp200_pmu_open_clockgate(MP200_CLOCKGATE_TI1_PCLK);

		/* open clockgate and unreset device and stop timer */
#ifdef CONFIG_MP201
		mp200_pmu_open_clockgate(MP200_CLOCKGATE_TI1_TIN);
#endif				/* CONFIG_MP201 */
		for (i = 0; i < TIMER_MAX_NUM; i++) {
			/* open clockgate */
			mp200_pmu_open_clockgate(tm_reg[TIMER_TI0 + i].clkdev);
			/* unreset device */
			mp200_pmu_unreset_device(tm_reg[TIMER_TI0 + i].rstdev);
			mp200_pmu_unreset_device(tm_reg[TIMER_TW0 + i].rstdev);
			/* timer stop */
			outl(0, tm_reg[TIMER_TI0 + i].tm_op);
			outl(0, tm_reg[TIMER_TW0 + i].tm_op);
		}
		for (i = 0; i < TIMER_TG_MAX_NUM; i++) {
			/* open clockgate */
			mp200_pmu_open_clockgate(tm_reg[TIMER_TG0 + i].clkdev);
			/* unreset device */
			mp200_pmu_unreset_device(tm_reg[TIMER_TG0 + i].rstdev);
			/* timer stop */
			outl(0, tm_reg[TIMER_TG0 + i].tm_op);
		}

		/* wait */
		udelay(TIMER_SET_DELAY);

		/* select TIN clock */
#ifdef CONFIG_MP201
		outl(TIN_SEL_VAL, SMU_TI0_TIN_SEL);
		outl(TIN_SEL_VAL, SMU_TI1_TIN_SEL);
#else
		outl(TIN_SEL_VAL, SMU_TI0TIN_SEL);
		outl(TIN_SEL_VAL, SMU_TI1TIN_SEL);
		outl(TIN_SEL_VAL, SMU_TI2TIN_SEL);
		outl(TGNTIN_SEL_VAL, SMU_TGNTIN_SEL);
#endif				/* CONFIG_MP201 */

		/* set counter */
		for (i = 0; i < TIMER_MAX_NUM; i++) {
			outl(COUNTER_MAXVAL, tm_reg[TIMER_TI0 + i].tm_set);
			count = get_tw_count_value(i, tm_param[TIMER_TW0 + i].
						   usecs);
			outl(count, tm_reg[TIMER_TW0 + i].tm_set);
		}
		for (i = 0; i < TIMER_TG_MAX_NUM; i++) {
			count = get_count_value(i, tm_param[TIMER_TG0 + i].
						usecs);
			outl(count, tm_reg[TIMER_TG0 + i].tm_set);
		}

		/* Auto clock ON */
		mp200_pmu_clkctrl_on(MP200_CLKCTRL_TI0PCLK);
		mp200_pmu_clkctrl_on(MP200_CLKCTRL_TI1PCLK);

		/* timer start */
		for (i = 0; i < TIMER_MAX_NUM; i++) {
			outl(ti_tm_op[i], tm_reg[TIMER_TI0 + i].tm_op);
			outl(tw_tm_op[i], tm_reg[TIMER_TW0 + i].tm_op);
		}
		for (i = 0; i < TIMER_TG_MAX_NUM; i++) {
			outl(tg_tm_op[i], tm_reg[TIMER_TG0 + i].tm_op);
		}

		break;

	case TIMER_FULLSPEED:
	case TIMER_ECONOMY:
		for (i = 0; i < TIMER_MAX_NUM; i++) {
			/* get tm_op */
			ti_tm_op[i] = inl(tm_reg[TIMER_TI0 + i].tm_op);
			tw_tm_op[i] = inl(tm_reg[TIMER_TW0 + i].tm_op);
			/* timer stop */
			outl(0, tm_reg[TIMER_TI0 + i].tm_op);
			outl(0, tm_reg[TIMER_TW0 + i].tm_op);
		}

		for (i = TIMER_TG_MIN; i < TIMER_TG_MAX_NUM; i++) {
			/* get tm_op */
			tg_tm_op[i] = inl(tm_reg[TIMER_TG0 + i].tm_op);
			/* timer stop */
			outl(0, tm_reg[TIMER_TG0 + i].tm_op);
		}

		/* wait */
		udelay(TIMER_SET_DELAY);

		/* select TIN clock */
#ifdef CONFIG_MP201
		if (mode == TIMER_FULLSPEED) {
			outl(TIN_SEL_VAL, SMU_TI0_TIN_SEL);
			outl(TIN_SEL_VAL, SMU_TI1_TIN_SEL);
		} else {
			outl(TIN_SEL_32K, SMU_TI0_TIN_SEL);
			outl(TIN_SEL_32K, SMU_TI1_TIN_SEL);
		}
#else
		if (mode == TIMER_FULLSPEED) {
			outl(TIN_SEL_VAL, SMU_TI0TIN_SEL);
			outl(TIN_SEL_VAL, SMU_TI1TIN_SEL);
			outl(TIN_SEL_VAL, SMU_TI2TIN_SEL);
			outl(TGNTIN_SEL_VAL, SMU_TGNTIN_SEL);
		} else {
			outl(TIN_SEL_32K, SMU_TI0TIN_SEL);
			outl(TIN_SEL_32K, SMU_TI1TIN_SEL);
			outl(TIN_SEL_32K, SMU_TI2TIN_SEL);
			outl(TGNTIN_SEL_32K, SMU_TGNTIN_SEL);
		}
#endif				/* CONFIG_MP201 */

		/* Auto clock OFF */
		mp200_pmu_clkctrl_off(MP200_CLKCTRL_TI0PCLK);
		mp200_pmu_clkctrl_off(MP200_CLKCTRL_TI1PCLK);

		for (i = 0; i < TIMER_MAX_NUM; i++) {
			/* set counter */
			if (mode == TIMER_FULLSPEED) {
				count = COUNTER_MAXVAL;
			} else {
				count = TIMER_INTERVAL_32K;
			}
			outl(count, tm_reg[TIMER_TI0 + i].tm_set);
			count = get_tw_count_value(i, tm_param[TIMER_TW0 + i].
						   usecs);
			outl(count, tm_reg[TIMER_TW0 + i].tm_set);
		}
		for (i = TIMER_TG_MIN; i < TIMER_TG_MAX_NUM; i++) {
			count = get_count_value(i, tm_param[TIMER_TG0 + i].
						usecs);
			outl(count, tm_reg[TIMER_TG0 + i].tm_set);
		}

		/* Auto clock ON */
		mp200_pmu_clkctrl_on(MP200_CLKCTRL_TI0PCLK);
		mp200_pmu_clkctrl_on(MP200_CLKCTRL_TI1PCLK);

		/* timer start */
		for (i = 0; i < TIMER_MAX_NUM; i++) {
			outl(ti_tm_op[i], tm_reg[TIMER_TI0 + i].tm_op);
			outl(tw_tm_op[i], tm_reg[TIMER_TW0 + i].tm_op);
		}
		for (i = TIMER_TG_MIN; i < TIMER_TG_MAX_NUM; i++) {
			outl(tg_tm_op[i], tm_reg[TIMER_TG0 + i].tm_op);
		}

		break;

	default:
		printk(KERN_INFO "%s(): set clock error mode = %d.\n",
		       __FUNCTION__, mode);
		break;
	}
}

int mp200_timer_start(unsigned int timer)
{
	int ret;

	ret = check_validity_channel(timer);
	if (ret != 0) {
		return ret;
	}

	/* Timer start */
	tm_param[TIMER_TG0 + timer].tm_op |= TSTART;
	outl(TM_EN | tm_param[TIMER_TG0 + timer].tm_op,
	     tm_reg[TIMER_TG0 + timer].tm_op);

	return ret;
}

int mp200_timer_stop(unsigned int timer)
{
	int ret;

	ret = check_validity_channel(timer);
	if (ret != 0) {
		return ret;
	}

	/* Timer stop */
	tm_param[TIMER_TG0 + timer].tm_op &= ~(TSTART);
	outl(0, tm_reg[TIMER_TG0 + timer].tm_op);

	return ret;
}

int mp200_timer_set_period(unsigned int timer, unsigned int usecs)
{
	int ret = 0, val_tm_op, regval;

	ret = check_validity_channel(timer);
	if (ret != 0) {
		return ret;
	}

	if (((usecs < TIMER_MIN_USECS) || (TIMER_MAX_USECS < usecs)) &&
	    (usecs != TIMER_MAX_COUNT)) {
		return -EINVAL;
	}

	spin_lock(&timer_spinlock);

	tm_param[TIMER_TG0 + timer].usecs = usecs;

	/* get tm_op */
	val_tm_op = inl(tm_reg[TIMER_TG0 + timer].tm_op);

	/* timer stop */
	outl(0, tm_reg[TIMER_TG0 + timer].tm_op);

	/* wait */
	udelay(TIMER_SET_DELAY);

	/* Auto clock OFF */
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_TI1PCLK);

	/* set counter */
	regval = get_count_value(timer, usecs);
	outl(regval, tm_reg[TIMER_TG0 + timer].tm_set);

	/* timer start (restore tm_op) */
	outl(val_tm_op, tm_reg[TIMER_TG0 + timer].tm_op);

	/* Auto clock ON */
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_TI1PCLK);

	spin_unlock(&timer_spinlock);

	return ret;
}

int mp200_timer_get_period(unsigned int timer, unsigned int *usecs)
{
	int ret;

	ret = check_validity_channel(timer);
	if (ret != 0) {
		return ret;
	}

	if (usecs == NULL) {
		return -EINVAL;
	}

	*usecs = tm_param[TIMER_TG0 + timer].usecs;

	return ret;
}

int mp200_timer_get_usecs(unsigned int timer, unsigned int *usecs)
{
	unsigned int regval;
	int ret;

	ret = check_validity_channel(timer);
	if (ret != 0) {
		return ret;
	}

	if (usecs == NULL) {
		return -EINVAL;
	}

	(void)inl(tm_reg[TIMER_TG0 + timer].rcr);
	regval = inl(tm_reg[TIMER_TG0 + timer].rcr);
	*usecs = get_usec_value(timer, regval);

	return ret;
}

int mp200_timer_get_counts(unsigned int timer, unsigned int *counts)
{
	int ret;

	ret = check_validity_channel(timer);
	if (ret != 0) {
		return ret;
	}

	if (counts == NULL) {
		return -EINVAL;
	}

	(void)inl(tm_reg[TIMER_TG0 + timer].rcr);
	*counts = inl(tm_reg[TIMER_TG0 + timer].rcr);

	return ret;
}

int
mp200_timer_register_cb(unsigned int timer, char *devname,
			irqreturn_t(*cb) (int, void *, struct pt_regs *),
			void *dev_id)
{
	unsigned int irq;
	unsigned int irq_flags;
	int ret;

	ret = check_validity_channel(timer);
	if (ret != 0) {
		return ret;
	}

	irq = tm_reg[TIMER_TG0 + timer].irq;
	irq_flags = SA_SHIRQ | SA_INTERRUPT;
	ret = request_irq(irq, cb, irq_flags, devname, dev_id);
	if (ret != 0) {
		return ret;
	}

	tm_param[TIMER_TG0 + timer].tm_op |= TO_EN;

	return ret;
}

int mp200_timer_unregister_cb(unsigned int timer, void *dev_id)
{
	unsigned int irq;
	int ret;

	ret = check_validity_channel(timer);
	if (ret != 0) {
		return ret;
	}

	irq = tm_reg[TIMER_TG0 + timer].irq;
	free_irq(irq, dev_id);

	return 0;
}

int mp200_wdt_set_timeout(unsigned int timer, unsigned int usecs)
{
	int ret = 0, val_tm_op, regval;

	if ((timer < 0) || (timer >= TIMER_MAX_NUM)) {
		return -ENODEV;
	}

	if ((usecs < TIMER_MIN_USECS) || (TIMER_MAX_USECS < usecs)) {
		return -EINVAL;
	}

	spin_lock(&timer_spinlock);

	tm_param[TIMER_TW0 + timer].usecs = usecs;

	/* get tm_op */
	val_tm_op = inl(tm_reg[TIMER_TW0 + timer].tm_op);

	/* timer stop */
	outl(0, tm_reg[TIMER_TW0 + timer].tm_op);

	/* wait */
	udelay(TIMER_SET_DELAY);

	/* Auto clock OFF */
	mp200_pmu_clkctrl_off(tm_reg[TIMER_TW0 + timer].clkctrldev);

	/* set counter */
	regval = get_tw_count_value(timer, usecs);
	outl(regval, tm_reg[TIMER_TW0 + timer].tm_set);

	/* timer start (restore tm_op) */
	outl(val_tm_op, tm_reg[TIMER_TW0 + timer].tm_op);

	/* Auto clock ON */
	mp200_pmu_clkctrl_on(tm_reg[TIMER_TW0 + timer].clkctrldev);

	spin_unlock(&timer_spinlock);

	return ret;
}

int mp200_wdt_ping(unsigned int timer)
{
	int ret = 0;

	if ((timer < 0) || (timer >= TIMER_MAX_NUM)) {
		return -ENODEV;
	}

	spin_lock(&timer_spinlock);

	/* Auto clock OFF */
	mp200_pmu_clkctrl_off(tm_reg[TIMER_TW0 + timer].clkctrldev);

	/* timer clear */
	outl(TCR_CLR, tm_reg[TIMER_TW0 + timer].tm_clr);

	/* wait */
	udelay(TIMER_SET_DELAY);

	/* Auto clock ON */
	mp200_pmu_clkctrl_on(tm_reg[TIMER_TW0 + timer].clkctrldev);

	spin_unlock(&timer_spinlock);

	return ret;
}

EXPORT_SYMBOL(mp200_timer_start);
EXPORT_SYMBOL(mp200_timer_stop);
EXPORT_SYMBOL(mp200_timer_set_period);
EXPORT_SYMBOL(mp200_timer_get_period);
EXPORT_SYMBOL(mp200_timer_get_usecs);
EXPORT_SYMBOL(mp200_timer_get_counts);
EXPORT_SYMBOL(mp200_timer_register_cb);
EXPORT_SYMBOL(mp200_timer_unregister_cb);
EXPORT_SYMBOL(mp200_wdt_set_timeout);
EXPORT_SYMBOL(mp200_wdt_ping);
