/*
 *  File Name       : linux/arch/arm/mach-mp200/uwire.c
 *  Function        : uwire
 *  Release Version : Ver 1.01
 *  Release Date    : 2006/01/31
 *
 *  Copyright (C) NEC Electronics Corporation 2005-2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/init.h>
#include <linux/module.h>
#include <asm/hardware.h>
#include <asm/arch/uwire.h>
#include <asm/arch/smu.h>
#include <asm/arch/pmu.h>
#include <asm/io.h>
#include "uwire.h"

/*
 * Solution Board
 */
static int uwire_cs0_active_level = 1;
static int uwire_cs1_active_level = 0;

static spinlock_t lck = SPIN_LOCK_UNLOCKED;

static int uwire_clk_source = -1;
static int uwire_pclk_pll = 1;

static int uwire_init_flag = 1;

/*
 * SMU Control
 */

static inline void uwire_pclock_start(void)
{
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_MWI_PCLK);
}

static inline void uwire_pclock_stop(void)
{
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_MWI_PCLK);
}

static inline void uwire_sclock_start(void)
{
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_MWI_SCLK);
}

static inline void uwire_sclock_stop(void)
{
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_MWI_SCLK);
}

static inline void uwire_clock_start(void)
{
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_MWI_PCLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_MWI_SCLK);
}

static inline void uwire_clock_stop(void)
{
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_MWI_PCLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_MWI_SCLK);
}

#ifdef CONFIG_MP201
static inline void uwire_dev_unreset(void)
{
	mp200_pmu_unreset_device(MP200_RESETDEVICE_MWI);
}
static inline void uwire_clkctrl_on(void)
{
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_MWIPCLK);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_MWISCLKLP);
}

static inline void uwire_clkctrl_off(void)
{
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_MWIPCLK);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_MWISCLKLP);
}
#endif

/*
   setting frequency devider function
   argument is positive integer 1-256
*/

static inline void uwire_set_divisor(u_int div)
{
	if ((0 < div) && (div < 257)) {
#ifdef CONFIG_MP201
		outl((div - 1), SMU_MWI_SCLK_DIV);	/* CHG MP201 */
#else
		outl((div - 1), SMU_DIVMWISCLK);
#endif				/* CONFIG_MP201 */
	}
}

/*
 * cs control
 */
static inline u_int sl_board_cs_fixup(int cs)
{
	if (cs == UWIRE_CS0) {
		return ((CS0_EN | (uwire_cs0_active_level ? CS0 : 0))
			| (CS1_EN | (uwire_cs1_active_level ? 0 : CS1)));
	}
	if (cs == UWIRE_CS1) {
		return ((CS0_EN | (uwire_cs0_active_level ? 0 : CS0))
			| (CS1_EN | (uwire_cs1_active_level ? CS1 : 0)));
	}
	return ((CS0_EN | (uwire_cs0_active_level ? 0 : CS0))
		| (CS1_EN | (uwire_cs1_active_level ? 0 : CS1)));
}

int uwire_set_active_level(int cs, int high)
{
	int org_level;

	/* uwire initialize error check */
	if (uwire_init_flag) {
		printk(KERN_INFO "uwire_set_active_level(): "
		       "uwire initialize error\n");
		return -EPERM;
	}

	if (cs == UWIRE_CS0) {
		org_level = uwire_cs0_active_level;
		uwire_cs0_active_level = high;
	} else if (cs == UWIRE_CS1) {
		org_level = uwire_cs1_active_level;
		uwire_cs1_active_level = high;
	} else {
		return -EINVAL;
	}
	return org_level;
}

int uwire_get_div_sclk(int sclk)
{
	int div_sclk = -1;

	if (sclk != UWIRE_SCLOCK_ANY) {
		switch (sclk) {
		case UWIRE_SCLOCK_1500KHz:
			div_sclk = uwire_clk_source / 1500 - 1;
			break;
		case UWIRE_SCLOCK_3MHz:
			div_sclk = uwire_clk_source / 3000 - 1;
			break;
		case UWIRE_SCLOCK_6MHz:
			div_sclk = uwire_clk_source / 6000 - 1;
			break;
		case UWIRE_SCLOCK_12MHz:
			div_sclk = uwire_clk_source / 12000 - 1;
			break;
		case UWIRE_SCLOCK_24MHz:
			div_sclk = uwire_clk_source / 24000 - 1;
			break;
		case UWIRE_SCLOCK_48MHz:
			div_sclk = uwire_clk_source / 48000 - 1;
			break;
		default:
			div_sclk = uwire_clk_source / (96000 / (sclk + 1)) - 1;
			break;
		}
		if (div_sclk < 0) {
			div_sclk = 0;
		}
	}

	return div_sclk;
}

/*
 * uWire I/O
 */

int uwire_vl_read(int cs, int sclk, u_char addr, u_int len, u_long * data)
{
	u_int uwire_tick;
	u_long cpu_flags;
	u_long mask;
	u_int org_sclk = ~0U;
	int div_sclk = -1;

	/* uwire initialize error check */
	if (uwire_init_flag) {
		printk(KERN_INFO "uwire_vl_read(): "
		       "uwire initialize error\n");
		return -EPERM;
	}
#ifdef UWIRE_DEBUG
	printk("uwire_vl_read(): CS%d read.\n", cs);
	printk("uwire_vl_read(): addr=0x%x\n", addr);
#endif

	/* Check CS Number */
	if ((UWIRE_CS0 != cs) && (UWIRE_CS1 != cs)) {
		printk(KERN_INFO "uwire_vl_read(): #CS(%d) is invalid number\n",
		       cs);
		return -EINVAL;
	}

	if ((len < 1) || (UWIRE_MAX_BIT_LEN < len)) {
		printk(KERN_INFO "uwire_vl_read(): data length invalid(%u)\n",
		       len);
		return -EINVAL;
	} else if (len == UWIRE_MAX_BIT_LEN) {
		mask = ~0UL;
	} else {
		mask = (1UL << len) - 1;
	}

	div_sclk = uwire_get_div_sclk(sclk);

	spin_lock_irqsave(&lck, cpu_flags);

	if (div_sclk != -1) {
#ifdef CONFIG_MP201
		org_sclk = inl(SMU_MWI_SCLK_DIV);	/* CHG MP201 */
		outl(div_sclk, SMU_MWI_SCLK_DIV);	/* CHG MP201 */
#else
		org_sclk = inl(SMU_DIVMWISCLK);
		outl(div_sclk, SMU_DIVMWISCLK);
#endif				/* CONFIG_MP201 */
	}

	uwire_sclock_start();	/* SCLK start */

	/* clear */
	outl(MWI_FFCLR, MWI_INTFFCLR);	/* MWI_STATUS clear */

	/* int clear check */
	while (inl(MWI_INTRAW) & MWI_RAW) {
	}

	outl(sl_board_cs_fixup(-1), MWI_CONT);

	/* start transmit&receive */
	if (UWIRE_CS0 == cs) {
		outl(MWI_EDGE, MWI_CS0);
	} else {
		outl(MWI_EDGE, MWI_CS1);
	}

	outl(MWI_SEQUENTIAL_MODE | ((len - 1) << UWIRE_BIT_RX_SHIFT) |
	     sl_board_cs_fixup(cs), MWI_CONT);

	outl((addr << 24), MWI_TXQA);	/* MSB shift (31-24 <-- 7-0) */
	outl(MWI_START_FLAG, MWI_START);

	/* wait end transmit */
	uwire_tick = 0;
	while ((inl(MWI_INTRAW) & MWI_RAW) == 0) {
		if (++uwire_tick > UWIRE_MAX_WAIT) {
#ifdef UWIRE_DEBUG
			printk("uwire_vl_read(): raw=%x\n", inl(MWI_INTRAW));
#endif
			uwire_sclock_stop();
			outl(MWI_FFCLR, MWI_INTFFCLR);	/* MWI_STATUS clear */

			/* Select CS Inactive : CSx_EN | CSx(0) x=0or1 */
			outl(sl_board_cs_fixup(-1), MWI_CONT);

			if (org_sclk != ~0U) {
#ifdef CONFIG_MP201
				outl(org_sclk, SMU_MWI_SCLK_DIV);	/* CHG MP201 */
#else
				outl(org_sclk, SMU_DIVMWISCLK);
#endif
			}

			spin_unlock_irqrestore(&lck, cpu_flags);
			printk(KERN_INFO "uwire_vl_read(): busy\n");

			return -EBUSY;
		}
	}

	*data = inl(MWI_RXQ) & mask;

	outl(MWI_FFCLR, MWI_INTFFCLR);	/* MWI_STATUS clear */
	uwire_sclock_stop();	/* SCLK stop */

	if (org_sclk != ~0U) {
#ifdef CONFIG_MP201
		outl(org_sclk, SMU_MWI_SCLK_DIV);	/* CHG MP201 */
#else
		outl(org_sclk, SMU_DIVMWISCLK);
#endif				/* CONFIG_MP201 */
	}

	/* int clear check */
	while (inl(MWI_INTRAW) & MWI_RAW) {
	}

	/* Select CS Inactive : CSx_EN | CSx(0) x=0or1 */
	outl(sl_board_cs_fixup(-1), MWI_CONT);

	spin_unlock_irqrestore(&lck, cpu_flags);

#ifdef UWIRE_DEBUG
	printk("uwire_vl_read(): CS%d read data=%x\n", cs, *data);
#endif
	return 0;
}

int uwire_read(int cs, u_char addr, u_char * data)
{
	u_long r_data;
	int ret;

	ret = uwire_vl_read(cs, UWIRE_SCLOCK_ANY, addr, 8, &r_data);
	if (ret == 0) {
		*data = (u_char) r_data;
	}
	return ret;
}

int uwire_vl_write(int cs, int sclk, u_char addr, u_int len, u_long data)
{
	u_int uwire_tick;
	u_long cpu_flags;
	u_int org_sclk = ~0U;
	int div_sclk = -1;

	/* uwire initialize error check */
	if (uwire_init_flag) {
		printk(KERN_INFO "uwire_vl_write(): "
		       "uwire initialize error\n");
		return -EPERM;
	}
#ifdef UWIRE_DEBUG
	printk("uwire_vl_write(): CS%d write.\n", cs);
	printk("uwire_vl_write(): addr=0x%x\n", addr);
#endif

	/* Check CS Number */
	if ((UWIRE_CS0 != cs) && (UWIRE_CS1 != cs)) {
		printk(KERN_INFO
		       "uwire_vl_write(): #CS(%d) is invalid number\n", cs);
		return -EINVAL;
	}

	if ((len < 1) || (UWIRE_MAX_BIT_LEN < len)) {
		printk(KERN_INFO "uwire_vl_write(): data length invalid(%u)\n",
		       len);
		return -EINVAL;
	}

	div_sclk = uwire_get_div_sclk(sclk);

	spin_lock_irqsave(&lck, cpu_flags);

	if (div_sclk != -1) {
#ifdef CONFIG_MP201
		org_sclk = inl(SMU_MWI_SCLK_DIV);	/* CHG MP201 */
		outl(div_sclk, SMU_MWI_SCLK_DIV);	/* CHG MP201 */
#else
		org_sclk = inl(SMU_DIVMWISCLK);
		outl(div_sclk, SMU_DIVMWISCLK);
#endif				/* CONFIG_MP201 */
	}

	uwire_sclock_start();	/* SCLK start */

	/* clear */
	outl(MWI_FFCLR, MWI_INTFFCLR);	/* MWI_STATUS clear */

	/* int clear check */
	while (inl(MWI_INTRAW) & MWI_RAW) {
	}

	outl(sl_board_cs_fixup(-1), MWI_CONT);

	/* start transmit */
	if (UWIRE_CS0 == cs) {
		outl(MWI_EDGE, MWI_CS0);
	} else {
		outl(MWI_EDGE, MWI_CS1);
	}
	outl(MWI_PACKAGE_MODE | ((len - 1) << UWIRE_BIT_DTX_SHIFT) |
	     sl_board_cs_fixup(cs), MWI_CONT);
	outl((addr << 24), MWI_TXQA);	/* MSB shift (31-24 <-- 7-0) */
	outl((data << (32 - len)), MWI_TXQ);	/* MSB shift (31-24 <-- 7-0) */
	outl(MWI_START_FLAG, MWI_START);

	/* wait end transmit */
	uwire_tick = 0;
	while ((inl(MWI_INTRAW) & MWI_RAW) == 0) {
		if (++uwire_tick > UWIRE_MAX_WAIT) {
#ifdef UWIRE_DEBUG
			printk("uwire_vl_write(): raw=%x\n", inl(MWI_INTRAW));
#endif
			outl(MWI_FFCLR, MWI_INTFFCLR);	/* MWI_STATUS clear */
			uwire_sclock_stop();

			/* Select CS Inactive : CSx_EN | CSx(0) x=0or1 */
			outl(sl_board_cs_fixup(-1), MWI_CONT);

			if (org_sclk != ~0U) {
#ifdef CONFIG_MP201
				outl(org_sclk, SMU_MWI_SCLK_DIV);	/* CHG MP201 */
#else
				outl(org_sclk, SMU_DIVMWISCLK);
#endif				/* CONFIG_MP201 */
			}

			spin_unlock_irqrestore(&lck, cpu_flags);
			printk(KERN_INFO "uwire_vl_write(): busy\n");

			return -EBUSY;
		}
	}

	outl(MWI_FFCLR, MWI_INTFFCLR);	/* MWI_STATUS clear */
	uwire_sclock_stop();	/* SCLK stop */

	if (org_sclk != ~0U) {
#ifdef CONFIG_MP201
		outl(org_sclk, SMU_MWI_SCLK_DIV);	/* CHG MP201 */
#else
		outl(org_sclk, SMU_DIVMWISCLK);
#endif				/* CONFIG_MP201 */
	}

	/* int clear check */
	while (inl(MWI_INTRAW) & MWI_RAW) {
	}

	/* Select CS Inactive : CSx_EN | CSx(0) x=0or1 */
	outl(sl_board_cs_fixup(-1), MWI_CONT);

	spin_unlock_irqrestore(&lck, cpu_flags);

	return 0;
}

int uwire_write(int cs, u_char addr, u_char data)
{
	int ret;

	ret = uwire_vl_write(cs, UWIRE_SCLOCK_ANY, addr, 8, data);

	return ret;
}

static int __init cs0_setup(char *sel)
{
	if (*sel == '0') {
		uwire_cs0_active_level = 0;
	} else {
		uwire_cs0_active_level = 1;
	}
	return 0;
}

static int __init cs1_setup(char *sel)
{
	if (*sel == '0') {
		uwire_cs1_active_level = 0;
	} else {
		uwire_cs1_active_level = 1;
	}
	return 0;
}

__setup("uwire_cs0=", cs0_setup);
__setup("uwire_cs1=", cs1_setup);

#ifdef CONFIG_MP201
static int __init uwire_init(void)
{
	u_int flags;
	u_long plln, pllm, pllp;
	u_long pllclk, nrm_div;
	static const u_long smu_table_lplln[] = {
		122880, 126976, 131072, 135168, 139264, 143360,
		147456, 151552, 155648, 159744, 163840
	};

	printk(KERN_INFO "Starting uwire.\n");

	spin_lock_irqsave(&lck, flags);
	uwire_clock_stop();	/* PCLK/SCLK stop */

/*
    determin clock and set a devisor at valid value
*/
	switch (inl(SMU_PLLSEL) & MASK_SMU_PLLSEL) {
	case SMU_PLLSEL_OSC:	/* CHG MP201 */
		uwire_clk_source = 12000;
		uwire_set_divisor(DIV_12M);
		uwire_pclk_pll = 0;
		break;

	case SMU_PLLSEL_RTC:	/* CHG MP201 */
		uwire_clk_source = 32;
		uwire_set_divisor(1);	/* XXX */
		uwire_pclk_pll = 0;
		break;

	case SMU_PLLSEL_PLL:
		/* PLL1 */
		/* CHG MP201 */
		if (inl(SMU_PLLSEL2) & SMU_NRM_CK_SEL) {
			/* LPLL */
			plln = inl(SMU_LPLL_CTRL1);
			pllclk = smu_table_lplln[plln];
		} else {
			/* HPLL */
			plln = (inl(SMU_HPLL_CTRL1) & MASK_HPLL_N) + 1;
			pllm = (inl(SMU_HPLL_CTRL2) & MASK_HPLL_M) + 1;
			pllp = (inl(SMU_HPLL_CTRL3) & MASK_HPLL_P);
			pllclk = (12000 * plln) / pllm;
			if (pllp != 0) {
				pllclk >>= 1;
			}
		}
		nrm_div = (inl(SMU_AUTO_FRQ_CHANGE) & 0xF000000) >> 24;
		uwire_clk_source = pllclk / (nrm_div + 1);

		uwire_set_divisor(uwire_clk_source / 3000);	/* 3MHz */
		uwire_pclk_pll = 1;

#ifdef UWIRE_DEBUG
		printk(KERN_INFO "uwire: sclk src=%lu.%3lu, divisor=%lu\n", uwire_clk_source / 1000, uwire_clk_source % 1000, uwire_clk_source / 3000);	/* CHG MP201 */
#endif
		break;
	default:
		printk(KERN_INFO "uwire: Unknown SCLK Source\n");
		uwire_clk_source = 12000;
		uwire_set_divisor(DIV_12M);
		uwire_pclk_pll = 0;
		break;
	}

	/* PCLK stop is possible, but we get going that just in case */
	uwire_clock_start();	/* CLK start */
	uwire_clkctrl_off();	/* AUTO CLK OFF */
	uwire_dev_unreset();	/* UNRESET */
	uwire_clkctrl_on();	/* AUTO CLK ON */
	spin_unlock_irqrestore(&lck, flags);

	uwire_init_flag = 0;

#ifdef UWIRE_DEBUG
	{
		u_int cs0, cs1, non;
		cs0 = sl_board_cs_fixup(UWIRE_CS0);
		cs1 = sl_board_cs_fixup(UWIRE_CS1);
		non = sl_board_cs_fixup(-1);
		printk(KERN_INFO "uWire: CS0=0x%02x, CS1=0x%02x, INI=0x%02x\n",
		       cs0, cs1, non);
	}
#endif

	return 0;
}

#else
static int __init uwire_init(void)
{
	u_long flags;
	unsigned long pll1_p_val, pll1_m_val, pll1_n_val;
	unsigned long pll1ctrl0;
	unsigned long pll1ctrl1;
	unsigned long pll1;

	printk(KERN_INFO "Starting uwire.\n");

	spin_lock_irqsave(&lck, flags);
	uwire_clock_stop();	/* PCLK/SCLK stop */

/*
    determin clock and set a devisor at valid value
*/
	switch (inl(SMU_PLLSEL) & MASK_SMU_PLLSEL) {
	case SMU_PLLSEL_12M:
		uwire_clk_source = 12000;
		uwire_set_divisor(DIV_12M);
		uwire_pclk_pll = 0;
		break;

	case SMU_PLLSEL_32K:
		uwire_clk_source = 32;
		uwire_set_divisor(1);	/* XXX */
		uwire_pclk_pll = 0;
		break;

	case SMU_PLLSEL_PLL:
		/* PLL1 */
		pll1ctrl0 = inl(SMU_PLL1CTRL0);
		pll1ctrl1 = inl(SMU_PLL1CTRL1);

		pll1_n_val = (pll1ctrl0 & MASK_SMU_PLL1_N_VAL) + 1;
		if (pll1ctrl0 & MASK_SMU_PLL1_MPFIX) {
			/* fix mode */
			pll1_m_val = 2;
			pll1_p_val = 0;
		} else {
			pll1_m_val = (pll1ctrl1 & MASK_SMU_PLL1_M_VAL) + 1;
			pll1_p_val = (pll1ctrl1 & MASK_SMU_PLL1_P_VAL);
		}

		pll1 = (12000 * pll1_n_val) / pll1_m_val;	/* KHz */
		if (pll1_p_val) {
			pll1 >>= 1;
		}

		uwire_clk_source = pll1;
		uwire_set_divisor(pll1 / 3000);	/* 3MHz */
		uwire_pclk_pll = 1;

#ifdef UWIRE_DEBUG
		printk(KERN_INFO "uwire: sclk src=%lu.%3lu, divisor=%lu\n",
		       pll1 / 1000, pll1 % 1000, pll1 / 3000);
#endif
		break;
	default:
		printk(KERN_INFO "uwire: Unknown SCLK Source\n");
		uwire_clk_source = 12000;
		uwire_set_divisor(DIV_12M);
		uwire_pclk_pll = 0;
		break;
	}

	/* PCLK stop is possible, but we get going that just in case */
	uwire_pclock_start();	/* PCLK start */
	spin_unlock_irqrestore(&lck, flags);

	uwire_init_flag = 0;

#ifdef UWIRE_DEBUG
	{
		u_int cs0, cs1, non;
		cs0 = sl_board_cs_fixup(UWIRE_CS0);
		cs1 = sl_board_cs_fixup(UWIRE_CS1);
		non = sl_board_cs_fixup(-1);
		printk(KERN_INFO "uWire: CS0=0x%02x, CS1=0x%02x, INI=0x%02x\n",
		       cs0, cs1, non);
	}
#endif

	return 0;
}
#endif				/* CONFIG_MP201 */

static void __exit uwire_exit(void)
{
	u_long flags;

	spin_lock_irqsave(&lck, flags);
	uwire_clock_stop();
	spin_unlock_irqrestore(&lck, flags);

	uwire_init_flag = 1;

	return;
}

arch_initcall(uwire_init);
module_exit(uwire_exit);

EXPORT_SYMBOL(uwire_read);
EXPORT_SYMBOL(uwire_write);
EXPORT_SYMBOL(uwire_set_active_level);
EXPORT_SYMBOL(uwire_vl_read);
EXPORT_SYMBOL(uwire_vl_write);
