/*
 *  linux/arch/arm/mach-mx3/irq.c
 *
 * Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/device.h>
#include <linux/errno.h>
#include <asm/hardware.h>
#include <asm/irq.h>
#include <asm/io.h>
#include <asm/mach/irq.h>
#include <asm/arch/platform.h>

/*!
 * @file irq.c
 * 
 * @brief This file contains the interrupt enable/disable functions. 
 * 
 * @ingroup Interrupt
 */

/*
 * We simply use the ENABLE DISABLE registers inside the mx31
 * to turn on/off specific interrupts.  
 */

/*!
 * Disable interrupt number "irq" in the AVIC
 *
 * @param  irq          interrupt source number
 */
static void mx31_mask_irq(unsigned int irq)
{
        *_reg_AVIC_INTDISNUM = irq;
}

/*!
 * Enable interrupt number "irq" in the AVIC
 *
 * @param  irq          interrupt source number
 */
static void mx31_unmask_irq(unsigned int irq)
{
        *_reg_AVIC_INTENNUM = irq;
}

static struct irqchip mx31_avic_chip = {
        .ack    = mx31_mask_irq,
        .mask   = mx31_mask_irq,
        .unmask = mx31_unmask_irq,
};

/*!
 * This function initializes the AVIC hardware and disables all the 
 * interrupts. It registers the interrupt enable and disable functions 
 * to the kernel for each interrupt source.
 */
void __init mx31_init_irq(void)
{
        unsigned int irq = 0;
        
        /* put the AVIC into the reset value with all interrupts disabled */
        *_reg_AVIC_INTCNTL    = 0;
        *_reg_AVIC_NIMASK     = 0x1F;
        *_reg_AVIC_INTENABLEH = 0;      /* disable all interrupts */
        *_reg_AVIC_INTENABLEL = 0;
        *_reg_AVIC_INTTYPEH   = 0;      /* all IRQ no FIQ */
        *_reg_AVIC_INTTYPEL   = 0;
        
        for (irq = 0; irq < NR_IRQS; irq++) {
                set_irq_chip(irq, &mx31_avic_chip);
                set_irq_handler(irq, do_level_IRQ);
                set_irq_flags(irq, IRQF_VALID);
        }
}
