/*
 * linux/arch/arm/mach-mx3/mx31ads_gpio.c
 *
 * Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/errno.h>
#include <asm/arch/gpio.h>
#include <asm/io.h>
#include <asm/arch/board.h>
#include <asm/arch/clock.h>

/*!
 * @file mx31ads_gpio.c
 * 
 * @brief This file contains all the GPIO setup functions for the board. 
 * 
 * @ingroup GPIO
 */

/*!
 * This system-wise GPIO function initializes the pins during system startup. 
 * All the statically linked device drivers should put the proper GPIO initialization 
 * code inside this function. It is called by \b fixup_mx31ads() during
 * system startup. This function is board specific. 
 */
void mx31ads_gpio_init(void) 
{
        /* config CS4 */
        iomux_config_mux(MX31_PIN_CS4, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
}

/*!
 * Setup GPIO for a UART port to be active
 *
 * @param  port         a UART port
 * @param  no_irda      indicates if the port is used for SIR
 */
void gpio_uart_active(int port, int no_irda)
{
        /*
         * Configure the IOMUX control registers for the UART signals 
         */
        switch (port) {
        /* UART 1 IOMUX Configs */
        case 0:
                iomux_config_mux(MX31_PIN_RXD1, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_TXD1, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_RTS1, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CTS1, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_DTR_DTE1, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_DSR_DTE1, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_RI_DTE1, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_DCD_DTE1, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                break;
        /* UART 2 IOMUX Configs */
        case 1:
                iomux_config_mux(MX31_PIN_TXD2, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_RTS2, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CTS2, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_DTR_DCE2, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_RXD2, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                break;
        /* UART 3 IOMUX Configs */
        case 2:
                iomux_config_mux(MX31_PIN_CSPI1_MOSI, OUTPUTCONFIG_ALT2, INPUTCONFIG_ALT2);
                iomux_config_mux(MX31_PIN_CSPI1_MISO, OUTPUTCONFIG_ALT2, INPUTCONFIG_ALT2);
                iomux_config_mux(MX31_PIN_CSPI1_SS0, OUTPUTCONFIG_ALT2, INPUTCONFIG_ALT2);
                iomux_config_mux(MX31_PIN_CSPI1_SS2, OUTPUTCONFIG_ALT2, INPUTCONFIG_ALT2);
                break;
        /* UART 4 IOMUX Configs */
        case 3:
                iomux_config_mux(MX31_PIN_USBOTG_DATA0, OUTPUTCONFIG_ALT2, INPUTCONFIG_ALT2);
                iomux_config_mux(MX31_PIN_USBOTG_DATA3, OUTPUTCONFIG_ALT2, INPUTCONFIG_ALT2);
                iomux_config_mux(MX31_PIN_USBOTG_DATA4, OUTPUTCONFIG_ALT2, INPUTCONFIG_ALT2);
                iomux_config_mux(MX31_PIN_USBOTG_DATA5, OUTPUTCONFIG_ALT2, INPUTCONFIG_ALT2);
                break;
        /* UART 5 IOMUX Configs */
        case 4:
                iomux_config_mux(MX31_PIN_USBH2_CLK, OUTPUTCONFIG_ALT1, INPUTCONFIG_ALT1);
                iomux_config_mux(MX31_PIN_USBH2_DIR, OUTPUTCONFIG_ALT1, INPUTCONFIG_ALT1);
                iomux_config_mux(MX31_PIN_USBH2_STP, OUTPUTCONFIG_ALT1, INPUTCONFIG_ALT1);
                iomux_config_mux(MX31_PIN_USBH2_NXT, OUTPUTCONFIG_ALT1, INPUTCONFIG_ALT1);
                break;
        default:
                break;
        }
        /*
         * TODO: Enable the UART Transceiver by configuring the GPIO pins 
         */
        
        /*
         * TODO: Configure the Pad registers for the UART pins
         */
}

/*!
 * Setup GPIO for a UART port to be inactive
 *
 * @param  port         a UART port
 * @param  no_irda      indicates if the port is used for SIR
 */
void gpio_uart_inactive(int port, int no_irda)
{
        /*
         * TODO: Disable the UART Transceiver by configuring the GPIO pin
         */
        switch (port) {
        case 0:
                iomux_config_mux(MX31_PIN_RXD1, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_TXD1, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_RTS1, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CTS1, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_DTR_DTE1, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_DSR_DTE1, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_RI_DTE1, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_DCD_DTE1, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                break;
        case 1:
                iomux_config_mux(MX31_PIN_TXD2, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_RTS2, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CTS2, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_DTR_DCE2, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_RXD2, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                break;
        case 2:
                iomux_config_mux(MX31_PIN_CSPI1_MOSI, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI1_MISO, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI1_SS0, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI1_SS2, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                break;
        case 3:
                iomux_config_mux(MX31_PIN_USBOTG_DATA0, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_USBOTG_DATA3, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_USBOTG_DATA4, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_USBOTG_DATA5, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                break;
        case 4:
                iomux_config_mux(MX31_PIN_USBH2_CLK, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_USBH2_DIR, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_USBH2_STP, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_USBH2_NXT, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                break;
        default:
                break;
        }
}

/*!
 * Configure the IOMUX GPR register to receive shared SDMA UART events
 *
 * @param  port         a UART port
 */
void config_uartdma_event(int port) 
{
        switch (port) {
        case 1:
                /* Configure to receive UART 2 SDMA events */
                iomux_config_gpr(MUX_PGP_FIRI, false);
                break;
        case 2:
                /* Configure to receive UART 3 SDMA events */
                iomux_config_gpr(MUX_CSPI1_UART3, true);
                break;
        case 4:
                /* Configure to receive UART 5 SDMA events */
                iomux_config_gpr(MUX_CSPI3_UART5_SEL, true);
                break;
        default:
                break;
        }
}

/*!
 * Setup GPIO for Keypad  to be active
 * 
 */
void gpio_keypad_active(void)
{
        /*
         * Configure the IOMUX control register for keypad signals.
         */
        iomux_config_mux(MX31_PIN_KEY_COL0, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
        iomux_config_mux(MX31_PIN_KEY_COL1, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
        iomux_config_mux(MX31_PIN_KEY_COL2, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
        iomux_config_mux(MX31_PIN_KEY_COL3, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
        iomux_config_mux(MX31_PIN_KEY_COL4, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
        iomux_config_mux(MX31_PIN_KEY_COL5, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
        iomux_config_mux(MX31_PIN_KEY_COL6, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
        iomux_config_mux(MX31_PIN_KEY_COL7, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
        iomux_config_mux(MX31_PIN_KEY_ROW0, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
        iomux_config_mux(MX31_PIN_KEY_ROW1, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
        iomux_config_mux(MX31_PIN_KEY_ROW2, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
        iomux_config_mux(MX31_PIN_KEY_ROW3, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
        iomux_config_mux(MX31_PIN_KEY_ROW4, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
        iomux_config_mux(MX31_PIN_KEY_ROW5, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
        iomux_config_mux(MX31_PIN_KEY_ROW6, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
        iomux_config_mux(MX31_PIN_KEY_ROW7, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
}

/*!
 * Setup GPIO for Keypad to be inactive
 *
 */
void gpio_keypad_inactive(void)
{
        iomux_config_mux(MX31_PIN_KEY_COL0, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
        iomux_config_mux(MX31_PIN_KEY_COL1, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
        iomux_config_mux(MX31_PIN_KEY_COL2, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
        iomux_config_mux(MX31_PIN_KEY_COL3, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
        iomux_config_mux(MX31_PIN_KEY_COL4, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
        iomux_config_mux(MX31_PIN_KEY_COL5, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
        iomux_config_mux(MX31_PIN_KEY_COL6, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
        iomux_config_mux(MX31_PIN_KEY_COL7, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
        iomux_config_mux(MX31_PIN_KEY_ROW0, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
        iomux_config_mux(MX31_PIN_KEY_ROW1, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
        iomux_config_mux(MX31_PIN_KEY_ROW2, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
        iomux_config_mux(MX31_PIN_KEY_ROW3, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
        iomux_config_mux(MX31_PIN_KEY_ROW4, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
        iomux_config_mux(MX31_PIN_KEY_ROW5, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
        iomux_config_mux(MX31_PIN_KEY_ROW6, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
        iomux_config_mux(MX31_PIN_KEY_ROW7, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
}

/*!
 * Setup GPIO for a CSPI device to be active
 *
 * @param  cspi_mod         an CSPI device
 */
void gpio_spi_active(int cspi_mod)
{
        switch (cspi_mod) {
        case 0:
                /* SPI1 */      
                iomux_config_mux(MX31_PIN_CSPI1_MISO, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI1_MOSI, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI1_SCLK, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI1_SPI_RDY, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI1_SS0, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI1_SS1, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI1_SS2, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                break;
        case 1:
                /* SPI2 */
                iomux_config_mux(MX31_PIN_CSPI2_MISO, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI2_MOSI, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI2_SCLK, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI2_SPI_RDY, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI2_SS0, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI2_SS1, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI2_SS2, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                break;
        case 2:
                /* SPI3 */
                /*
                iomux_config_mux(MX31_PIN_CSPI2_MISO, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI2_MOSI, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI2_SCLK, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI2_SPI_RDY, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI2_SS0, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI2_SS1, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_CSPI2_SS2, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                */
                break;
        default:
                break;
        }
}

/*!
 * Setup GPIO for a CSPI device to be inactive
 *
 * @param  cspi_mod         a CSPI device
 */
void gpio_spi_inactive(int cspi_mod)
{
        switch (cspi_mod) {
        case 0:
                /* SPI1 */      
                iomux_config_mux(MX31_PIN_CSPI1_MISO, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI1_MOSI, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI1_SCLK, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI1_SPI_RDY, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI1_SS0, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI1_SS1, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI1_SS2, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                break;
        case 1:
                /* SPI2 */
                iomux_config_mux(MX31_PIN_CSPI2_MISO, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI2_MOSI, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI2_SCLK, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI2_SPI_RDY, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI2_SS0, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI2_SS1, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI2_SS2, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                break;
        case 2:
                /* SPI3 */
                /*
                iomux_config_mux(MX31_PIN_CSPI2_MISO, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI2_MOSI, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI2_SCLK, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI2_SPI_RDY, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI2_SS0, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI2_SS1, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                iomux_config_mux(MX31_PIN_CSPI2_SS2, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
                */
                break;
        default:
                break;
        }
}

/*!
 * Setup GPIO for an I2C device to be active
 *
 * @param  i2c_num         an I2C device
 */
void gpio_i2c_active(int i2c_num) 
{
        switch (i2c_num) {
        case 0:
                iomux_config_mux(MX31_PIN_I2C_CLK, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_I2C_DAT, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                break;
        case 1:
                iomux_config_mux(MX31_PIN_CSPI2_MOSI, OUTPUTCONFIG_ALT1, INPUTCONFIG_ALT1);
                iomux_config_mux(MX31_PIN_CSPI2_MISO, OUTPUTCONFIG_ALT1, INPUTCONFIG_ALT1);
                break;
        case 2:
                iomux_config_mux(MX31_PIN_CSPI2_SS2, OUTPUTCONFIG_ALT1, INPUTCONFIG_ALT1);
                iomux_config_mux(MX31_PIN_CSPI2_SCLK, OUTPUTCONFIG_ALT1, INPUTCONFIG_ALT1);
                break;
        default:
                break;
        }
                
}

/*!
 * Setup GPIO for an I2C device to be inactive
 *
 * @param  i2c_num         an I2C device
 */
void gpio_i2c_inactive(int i2c_num) 
{
        switch (i2c_num) {
        case 0:
                iomux_config_mux(MX31_PIN_I2C_CLK, OUTPUTCONFIG_GPIO, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_I2C_DAT, OUTPUTCONFIG_GPIO, INPUTCONFIG_FUNC);
                break;
        case 1:
                iomux_config_mux(MX31_PIN_CSPI2_MOSI, OUTPUTCONFIG_GPIO, INPUTCONFIG_ALT1);
                iomux_config_mux(MX31_PIN_CSPI2_MISO, OUTPUTCONFIG_GPIO, INPUTCONFIG_ALT1);
                break;
        case 2:
                iomux_config_mux(MX31_PIN_CSPI2_SS2, OUTPUTCONFIG_GPIO, INPUTCONFIG_ALT1);
                iomux_config_mux(MX31_PIN_CSPI2_SCLK, OUTPUTCONFIG_GPIO, INPUTCONFIG_ALT1);
                break;
        default:
                break;
        }
}

/*!
 * This function configures the IOMux block for Atlas standard operations.
 *
 */
void gpio_atlas_active(void) 
{
        iomux_config_mux(MX31_PIN_GPIO1_3, OUTPUTCONFIG_GPIO, 
                		INPUTCONFIG_GPIO);
        gpio_config(0, GPIO_INT3, 0, GPIO_INT_RISE_EDGE);
}

/*!
 * This function clears the Atlas intrrupt.
 *
 */
void gpio_atlas_clear_int(void) 
{
	gpio_clear_int(0, GPIO_INT3);
}

/*!
 * This function return the SPI connected to Atlas.
 *
 */
int gpio_atlas_get_spi(void) 
{
        return 1;
}

/*!
 * This function return the SPI smave select for Atlas.
 *
 */
int gpio_atlas_get_ss(void) 
{
        return 0;
}

/*!
 * Setup GPIO for SDHC to be active
 *
 * @param module SDHC module number
 */
void gpio_sdhc_active(int module) 
{
        switch(module){
        case 0:
                iomux_config_mux(MX31_PIN_SD1_CLK, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_SD1_CMD, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_SD1_DATA0, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_SD1_DATA1, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_SD1_DATA2, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                iomux_config_mux(MX31_PIN_SD1_DATA3, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
                break;
        case 1:
                /* TODO:what are the pins for SDHC2? */
               break;
        default:
                break;
        }
}

/*!
 * Setup GPIO for SDHC1 to be inactive
 *
 * @param module SDHC module number
 */
void gpio_sdhc_inactive(int module) 
{
        switch(module){
        case 0:
                iomux_config_mux(MX31_PIN_SD1_CLK, OUTPUTCONFIG_GPIO, INPUTCONFIG_NONE);      
                iomux_config_mux(MX31_PIN_SD1_CMD, OUTPUTCONFIG_GPIO, INPUTCONFIG_NONE);      
                iomux_config_mux(MX31_PIN_SD1_DATA0, OUTPUTCONFIG_GPIO, INPUTCONFIG_NONE);
                iomux_config_mux(MX31_PIN_SD1_DATA1, OUTPUTCONFIG_GPIO, INPUTCONFIG_NONE); 
                iomux_config_mux(MX31_PIN_SD1_DATA2, OUTPUTCONFIG_GPIO, INPUTCONFIG_NONE);
                iomux_config_mux(MX31_PIN_SD1_DATA3, OUTPUTCONFIG_GPIO, INPUTCONFIG_NONE);      
                break;
        case 1:
                /* TODO:what are the pins for SDHC2? */
                break;
        default:
                break;
        }
}

/*!
 * Setup the IOMUX/GPIO for SDHC1 SD1_DET.
 * 
 * @param  host Pointer to MMC/SD host structure.
 * @param  handler      GPIO ISR function pointer for the GPIO signal.
 * @return The function returns 0 on success and -1 on failure.
 *
 **/
int sdhc_intr_setup(void *host, gpio_irq_handler handler)
{
        return 0;
}

/*!
 * Clear the IOMUX/GPIO for SDHC1 SD1_DET.
 * 
 * @param flag Flag represents whether the card is inserted/removed.
 *
 **/
void sdhc_intr_clear(int *flag)
{

}

/*!
 * Setup GPIO for LCD to be active
 *
 */
void gpio_lcd_active(void)
{
        u16 temp;

        iomux_config_mux(MX31_PIN_LD0, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC); 
        iomux_config_mux(MX31_PIN_LD1, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC); 
        iomux_config_mux(MX31_PIN_LD2, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC); 
        iomux_config_mux(MX31_PIN_LD3, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC); 
        iomux_config_mux(MX31_PIN_LD4, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC); 
        iomux_config_mux(MX31_PIN_LD5, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC); 
        iomux_config_mux(MX31_PIN_LD6, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC); 
        iomux_config_mux(MX31_PIN_LD7, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC); 
        iomux_config_mux(MX31_PIN_LD8, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC); 
        iomux_config_mux(MX31_PIN_LD9, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC); 
        iomux_config_mux(MX31_PIN_LD10, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC); 
        iomux_config_mux(MX31_PIN_LD11, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC); 
        iomux_config_mux(MX31_PIN_LD12, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC); 
        iomux_config_mux(MX31_PIN_LD13, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC); 
        iomux_config_mux(MX31_PIN_LD14, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC); 
        iomux_config_mux(MX31_PIN_LD15, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC); 
        iomux_config_mux(MX31_PIN_LD16, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);        // LD16
        iomux_config_mux(MX31_PIN_LD17, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);     // LD17
        iomux_config_mux(MX31_PIN_VSYNC3, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);      // VSYNC
        iomux_config_mux(MX31_PIN_HSYNC, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);      // HSYNC
        iomux_config_mux(MX31_PIN_FPSHIFT, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);      // CLK
        iomux_config_mux(MX31_PIN_DRDY0, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);        // DRDY
        iomux_config_mux(MX31_PIN_D3_REV, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);         // REV
        iomux_config_mux(MX31_PIN_CONTRAST, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);   // CONTR
        iomux_config_mux(MX31_PIN_D3_SPL, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);         // SPL
        iomux_config_mux(MX31_PIN_D3_CLS, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);         // CLS

        temp = PBC_BCTRL1_LCDON; 
        writew(temp, PBC_BASE_ADDRESS + PBC_BCTRL1_SET); 
}

/*!
 * Setup GPIO for LCD to be inactive
 *
 */
void gpio_lcd_inactive(void)
{
        u16 pbc_bctrl1_set = 0;

        pbc_bctrl1_set = (u16) PBC_BCTRL1_LCDON; 
        writew(pbc_bctrl1_set, PBC_BASE_ADDRESS + PBC_BCTRL1_SET + 2); 
}

/*!
 * Switch to the specified sensor - MX31 ADS has two
 *
 */
void gpio_sensor_select(int sensor)
{
        u16 temp;

	switch(sensor) {
	case 0:
		temp = PBC_BCTRL1_SENSOR2_ON;
		writew(temp, PBC_BASE_ADDRESS + PBC_BCTRL1_CLEAR);
		temp = PBC_BCTRL1_SENSOR1_ON;
		writew(temp, PBC_BASE_ADDRESS + PBC_BCTRL1_SET); 
		break;  
	case 1:
		temp = PBC_BCTRL1_SENSOR1_ON;
		writew(temp, PBC_BASE_ADDRESS + PBC_BCTRL1_CLEAR);
		temp = PBC_BCTRL1_SENSOR2_ON;
		writew(temp, PBC_BASE_ADDRESS + PBC_BCTRL1_SET); 
		break;  
	default:
		break;
	}
}

/*!
 * Setup GPIO for sensor to be active
 *
 */
void gpio_sensor_active(void)
{
        u16 temp;
     
	gpio_sensor_select(0);	// TODO - need method to select the camera

	/* 
	 * Configure the iomuxen for the CSI.
	 */

	iomux_config_mux(MX31_PIN_CSI_D4, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
	iomux_config_mux(MX31_PIN_CSI_D5, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
	iomux_config_mux(MX31_PIN_CSI_D6, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
	iomux_config_mux(MX31_PIN_CSI_D7, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
	iomux_config_mux(MX31_PIN_CSI_D8, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
	iomux_config_mux(MX31_PIN_CSI_D9, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
	iomux_config_mux(MX31_PIN_CSI_D10, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
	iomux_config_mux(MX31_PIN_CSI_D11, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
	iomux_config_mux(MX31_PIN_CSI_D12, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
	iomux_config_mux(MX31_PIN_CSI_D13, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
	iomux_config_mux(MX31_PIN_CSI_D14, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
	iomux_config_mux(MX31_PIN_CSI_D15, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
	iomux_config_mux(MX31_PIN_CSI_HSYNC, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
	iomux_config_mux(MX31_PIN_CSI_MCLK, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
	iomux_config_mux(MX31_PIN_CSI_PIXCLK, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);
	iomux_config_mux(MX31_PIN_CSI_VSYNC, OUTPUTCONFIG_FUNC, INPUTCONFIG_FUNC);

#ifdef CONFIG_MXC_IPU_CAMERA_16BIT
	/* 
	 * The other 4 data bits are multiplexed on MX31.
	 */
	iomux_config_mux(MX31_PIN_ATA_CS0, OUTPUTCONFIG_ALT2, INPUTCONFIG_ALT2);
	iomux_config_mux(MX31_PIN_ATA_CS1, OUTPUTCONFIG_ALT2, INPUTCONFIG_ALT2);
	iomux_config_mux(MX31_PIN_ATA_DIOR, OUTPUTCONFIG_ALT2, INPUTCONFIG_ALT2);
	iomux_config_mux(MX31_PIN_ATA_DIOW, OUTPUTCONFIG_ALT2, INPUTCONFIG_ALT2);
#endif

	/* 
	 * Now enable the CSI buffers
	 */

	temp = PBC_BCTRL2_CSI_EN;
	writew(temp, PBC_BASE_ADDRESS + PBC_BCTRL2_CLEAR);

#ifdef CONFIG_MXC_IPU_CAMERA_16BIT
	/* 
	 * Enable the other buffer for the additional 4 data bits.
	 */
	temp = PBC_BCTRL4_CSI_MSB_EN;
	writew(temp, PBC_BASE_ADDRESS + PBC_BCTRL4_CLEAR);
#endif
}

/*!
 * Setup EDIO/IOMUX for external UART.
 *
 * @param port UART port
 * @return    The function returns 0 if the interrupt pin is connected to 
 *            EDIO or 1 if the pin is connected to GPIO. For GPIO pin the return
 *            value is used to make gpio_request_irq() instead of request_irq().
 */
int extuart_intr_setup(int port)
{
        return 0;
}

/*!
 * Get the EDIO interrupt, clear if set. 
 *
 * @param port UART port
 */
void extuart_intr_clear(int port)
{
}

/*!
 * Do IOMUX configs required to put the
 * pin back in low power mode.
 *
 * @param  port UART port
 * @return  The function returns 0 if the interrupt pin is connected to 
 *          EDIO or 1 if the pin is connected to GPIO. For GPIO pin the 
 *          return value is used to make gpio_free_irq() instead of free_irq().
 */
int extuart_intr_cleanup(int port)
{
        return 0;
}

/*
 * Find the minimum clock for SDHC.
 *
 * @param clk SDHC module number.
 * @return Returns the minimum SDHC clock.
 */
unsigned int sdhc_get_min_clock(enum mxc_clocks clk)
{
        return 150000;
}

/*
 * Find the maximum clock for SDHC.
 * @param clk SDHC module number.
 * @return Returns the maximum SDHC clock.
 */
unsigned int sdhc_get_max_clock(enum mxc_clocks clk)
{
        return 21000000;
}

/*
 * Probe for the card. If present the GPIO data would be set.
 */
int sdhc_find_card(void)
{
        return 0;
}

int enet_free_irq(unsigned int irq, void *dev_id)
{
        writew(0x0100, PBC_BASE_ADDRESS + PBC_INTMASK_CLEAR); 
        gpio_free_irq(0, 4, GPIO_HIGH_PRIO);
        return 0;
}

int enet_request_irq(unsigned int irq, 
        irqreturn_t (*handler)(int, void *, struct pt_regs *),
        unsigned long irq_flags, const char * devname, void *dev_id)
{
    /*
     * Configure INT line as GPIO input
     */
    iomux_config_mux(MX31_PIN_GPIO1_4, OUTPUTCONFIG_GPIO, INPUTCONFIG_GPIO);
        
    gpio_config(0, 4, false, GPIO_INT_HIGH_LEV);
    
    if (gpio_request_irq(0, 4, GPIO_HIGH_PRIO, handler, 
            irq_flags, devname, dev_id) != 0) {
        enet_free_irq(irq, dev_id);
        return -EBUSY;
    }
    
    writew(0x0100, PBC_BASE_ADDRESS + PBC_INTMASK_SET); 
    return 0;
}
