/*
 *  linux/arch/arm/mach-scma11/scma11bb.c
 *
 *  Copyright (C) 2000 Deep Blue Solutions Ltd
 *  Copyright (C) 2002 Shane Nay (shane@minirl.com)
 *  Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/config.h>
#include <linux/types.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/init.h>
#include <asm/hardware.h>
#include <asm/irq.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/arch/memory.h>

#include "dsp_bringup.h"

/*!
 * @file scma11bb.c
 * 
 * @brief This file contains the board specific initialization routines. 
 * 
 * @ingroup System
 */

extern void scma11_map_io(void);
extern void scma11_init_irq(void);
extern void scma11_cpu_init(void) __init;
extern void scma11bb_gpio_init(void) __init;
extern struct sys_timer mxc_timer;

static char command_line[COMMAND_LINE_SIZE];

/*!
 * SCM-A11 EVB specific fixup function. It is called by \b setup_arch() in 
 * setup.c file very early on during kernel starts. It allows the user to 
 * statically fill in the proper values for the passed-in parameters. None of 
 * the parameters is used currently.
 * 
 * @param  desc         pointer to \b struct \b machine_desc
 * @param  tags         pointer to \b struct \b tag
 * @param  cmdline      pointer to the command line
 * @param  mi           pointer to \b struct \meminfo
 */
static void __init fixup_scma11bb(struct machine_desc *desc, 
                                      struct tag *tags, char **cmdline, 
                                      struct meminfo *mi)
{
        scma11_cpu_init();
        
        /*store command line for use on mxc_board_init*/
        strcpy(command_line, *cmdline);
}

/*!
 * Board specific initialization.
 */
static void __init mxc_board_init(void)
{
        scma11bb_gpio_init();
        
        /*try to get SPI control back to start Atlas driver properly*/
        dsp_memwrite_request((unsigned long)0xFFFDC004, (unsigned long)0x30303010);
        dsp_memwrite_request((unsigned long)0xFFFDC008, (unsigned long)0x10101030);

        /*search for dsp specific parameters from kernel's command line*/
        if(dsp_parse_cmdline((const char*)command_line) != 0)
        dsp_startapp_request();
						
}

/*
 * The following uses standard kernel macros define in arch.h in order to 
 * initialize __mach_desc_SCMA11BB data structure.
 */
MACHINE_START(SCMA11BB, "Freescale SCM-A11 BB")
        MAINTAINER("Freescale Semiconductor")
        /*       physical memory    physical IO        virtual IO     */
        BOOT_MEM(PHYS_OFFSET_ASM, AIPS1_BASE_ADDR, AIPS1_BASE_ADDR_VIRT)
	BOOT_PARAMS(PHYS_OFFSET_ASM + 0x100)
        FIXUP(fixup_scma11bb)
        MAPIO(scma11_map_io)
        INITIRQ(scma11_init_irq)
        INIT_MACHINE(mxc_board_init)
        .timer		= &mxc_timer,
MACHINE_END

