/*
 * linux/arch/arm/mach-scma11/scma11bb_gpio.c
 *
 * Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* TODO: Need to complete GPIO configuration for SCM-A11 */

#include <asm/arch/gpio.h>
#include <asm/io.h>
#include <asm/arch/board.h>

/*!
 * @file scma11bb_gpio.c
 *
 * @brief This file contains all the GPIO setup functions for the board.
 *
 * @ingroup GPIO
 */

/*!
 * This system-wise GPIO function initializes the pins during system startup.
 * All the statically linked device drivers should put the proper GPIO initialization
 * code inside this function. It is called by \b fixup_scma11bb() during
 * system startup. This function is board specific.
 */
void scma11bb_gpio_init(void)
{
    /* TODO: Need to configure IOMUX for SCM-A11 */
        /* config CS5 */
        //return;
        iomux_config_mux(AP_GPIO_AP_C15, OUTPUTCONFIG_FUNC4, INPUTCONFIG_NONE);
}

/*!
 * Setup GPIO for a UART port to be active
 *
 * @param  port         a UART port
 * @param  no_irda      indicates if the port is used for SIR
 */
void gpio_uart_active(int port, int no_irda)
{
        unsigned int pbc_bctrl1_set = 0;

        /*
         * Configure the IOMUX control registers for the UART signals
         * and enable the UART transceivers
         */
        switch (port) {
        /* UART 1 IOMUX Configs */
        case 0:
                iomux_config_mux(AP_U1_TXD, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(AP_U1_CTS_B, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(AP_U1_RXD, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(AP_U1_RTS_B, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                /*
                 * Enable the UART 1 Transceiver
                 */
                pbc_bctrl1_set |= PBC_BCTRL1_UENA;
                break;
        /* UART 2 IOMUX Configs */
        case 1:
        	iomux_config_mux(SP_USB_SE0_VM, OUTPUTCONFIG_FUNC2, INPUTCONFIG_FUNC2); /* TXD */
                iomux_config_mux(SP_USB_DAT_VP, OUTPUTCONFIG_FUNC2, INPUTCONFIG_FUNC2); /* RXD */
                iomux_config_mux(SP_USB_RXD, OUTPUTCONFIG_FUNC2, INPUTCONFIG_FUNC2); /* RTS */
                iomux_config_mux(AP_U2_CTS_B, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1); /* CTS */
                iomux_config_mux(AP_U2_RI_B, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1); /* RI */
                iomux_config_mux(AP_U2_DTR_B, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1); /* DTR */
                iomux_config_mux(AP_U2_DSR_B, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1); /* DSR */
                /*
                 * Enable the UART 2 Transceiver
                 */
                pbc_bctrl1_set |= PBC_BCTRL1_UENCE;
                break;
        /* UART 3 IOMUX Configs */
        case 2:
                iomux_config_mux(SP_U3_TXD, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_U3_CTS_B, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_U3_RXD, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_U3_RTS_B, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                /*
                 * Enable the UART 3 Transceiver
                 * Note: Check this, currently modelled this way in Virtio
                 */
                pbc_bctrl1_set |= PBC_BCTRL1_UENB;
                break;
        default:
                break;
        }
        writew(pbc_bctrl1_set, PBC_BASE_ADDRESS + PBC_BCTRL1_SET);
        /*
         * TODO: Configure the Pad registers for the UART pins
         */
}

/*!
 * Setup GPIO for a UART port to be inactive
 *
 * @param  port         a UART port
 * @param  no_irda      indicates if the port is used for SIR
 */
void gpio_uart_inactive(int port, int no_irda)
{
        unsigned int pbc_bctrl1_clr = 0;

        /*
         * Disable the UART Transceiver by configuring the GPIO pin
         */
        switch (port) {
        case 0:
                /*
                 * Disable the UART 1 Transceiver
                 */
                pbc_bctrl1_clr |= PBC_BCTRL1_UENA;
                break;
        case 1:
                /*
                 * Disable the UART 2 Transceiver
                 * Note: Check this, currently modelled this way in Virtio
                 */
                pbc_bctrl1_clr |= PBC_BCTRL1_UENCE;
                break;
        case 2:
                /*
                 * Disable the UART 3 Transceiver
                 * Note: Check this, currently modelled this way in Virtio
                 */
                pbc_bctrl1_clr |= PBC_BCTRL1_UENB;
                break;
        default:
                break;
        }
        writew(pbc_bctrl1_clr, PBC_BASE_ADDRESS + PBC_BCTRL1_CLEAR);
}

/*
 * The function is stubbed out and does not apply for SCM-A11
 */
void config_uartdma_event(int port)
{
        return;
}

/*!
 *  Setup GPIO for keypad to be active
 */
void gpio_keypad_active(void)
{
        iomux_config_mux(AP_KPROW0, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPROW1, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPROW2, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPROW3, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPROW4, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPROW5, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_GPIO_AP_B17, OUTPUTCONFIG_FUNC1,
                         INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_GPIO_AP_B18, OUTPUTCONFIG_FUNC1,
                         INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPCOL0, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPCOL1, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPCOL2, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPCOL3, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPCOL4, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPCOL5, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_GPIO_AP_B22, OUTPUTCONFIG_FUNC1,
                         INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_GPIO_AP_B23, OUTPUTCONFIG_FUNC1,
                         INPUTCONFIG_FUNC1);
}

/*!
 * Setup GPIO for keypad to be inactive
 */
void gpio_keypad_inactive(void)
{
        iomux_config_mux(AP_KPROW0, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPROW1, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPROW2, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPROW3, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPROW4, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPROW5, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_GPIO_AP_B17, OUTPUTCONFIG_DEFAULT,
                         INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_GPIO_AP_B18, OUTPUTCONFIG_DEFAULT,
                         INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPCOL0, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPCOL1, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPCOL2, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPCOL3, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPCOL4, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPCOL5, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_GPIO_AP_B22, OUTPUTCONFIG_DEFAULT,
                         INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_GPIO_AP_B23, OUTPUTCONFIG_DEFAULT,
                         INPUTCONFIG_DEFAULT);
}

/*!
 * Setup GPIO for a CSPI device to be active
 *
 * @param  cspi_mod         an CSPI device
 */
void gpio_spi_active(int cspi_mod)
{
        switch (cspi_mod) {
        case 0:
                /* SPI1_SS0 IOMux configuration */
                iomux_config_mux(SP_SPI1_SS0, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI1 MISO IOMux configuration */
                iomux_config_mux(SP_SPI1_MISO, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI1 MOSI IOMux configuration */
                iomux_config_mux(SP_SPI1_MOSI, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI1_SS1 IOMux configuration */
                iomux_config_mux(SP_SPI1_SS1, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI1 CLK IOMux configuration */
                iomux_config_mux(SP_SPI1_CLK, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                break;
        case 1:
                break;
        default:
                break;
        }
}

/*!
 * Setup GPIO for a CSPI device to be inactive
 *
 * @param  cspi_mod         a CSPI device
 */
void gpio_spi_inactive(int cspi_mod)
{
        switch (cspi_mod) {
        case 0:
                /* SPI1_SS0 IOMux configuration */
                iomux_config_mux(SP_SPI1_SS0, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI1 MISO IOMux configuration */
                iomux_config_mux(SP_SPI1_MISO, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI1 MOSI IOMux configuration */
                iomux_config_mux(SP_SPI1_MOSI, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI1_SS1 IOMux configuration */
                iomux_config_mux(SP_SPI1_SS1, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI1 CLK IOMux configuration */
                iomux_config_mux(SP_SPI1_CLK, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                break;
        case 1:
                break;
        default:
                break;
        }
}

/*!
 * Setup GPIO for an I2C device to be active
 *
 * @param  i2c_num         an I2C device
 */
void gpio_i2c_active(int i2c_num)
{
        switch (i2c_num) {
        case 0:
                iomux_config_mux(AP_I2CLK, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(AP_I2DAT, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        default:
                break;
        }
}

/*!
 * Setup GPIO for an I2C device to be inactive
 *
 * @param  i2c_num         an I2C device
 */
void gpio_i2c_inactive(int i2c_num)
{
        switch (i2c_num) {
        case 0:
                iomux_config_mux(AP_I2CLK, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(AP_I2DAT, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                break;
        default:
                break;
        }
}

/*!
 * This function configures the IOMux block for Atlas standard operations.
 *
 */
void gpio_atlas_active(void) 
{
        iomux_config_mux(AP_ED_INT5, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        edio_config(ED_INT5, 0,EDIO_INT_RISE_EDGE);
}

/*!
 * This function clears the Atlas intrrupt.
 *
 */
void gpio_atlas_clear_int(void) 
{
        edio_clear_int(ED_INT5);
        edio_get_int(ED_INT5);
}

/*!
 * This function return the SPI connected to Atlas.
 *
 */
int gpio_atlas_get_spi(void) 
{
        /* SCMA11 -> SPI2 = 1 */
        return 1;
}

/*!
 * This function return the SPI smave select for Atlas.
 *
 */
int gpio_atlas_get_ss(void) 
{
        /* SCMA11BB -> SS = 2 */
        return 2;
}

/*!
 * Setup GPIO for SDHC to be active
 *
 * @param module SDHC module number
 */
void gpio_sdhc_active(int module) 
{
        switch(module){
        case 1:
                iomux_config_mux(SP_SD1_CMD, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD1_CLK, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD1_DAT0, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD1_DAT1, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD1_DAT2, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD1_DAT3, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                break;
        case 2:
                iomux_config_mux(SP_SD2_CMD, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD2_CLK, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD2_DAT0, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD2_DAT1, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD2_DAT2, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD2_DAT3, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
               break;
        default:
                break;
        }
}

/*!
 * Setup GPIO for SDHC1 to be inactive
 *
 * @param module SDHC module number
 */
void gpio_sdhc_inactive(int module) 
{
        switch(module){
        case 1:
                iomux_config_mux(SP_SD1_CMD, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD1_CLK, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD1_DAT0, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD1_DAT1, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD1_DAT2, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD1_DAT3, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                break;
        case 2:
                iomux_config_mux(SP_SD2_CMD, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD2_CLK, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD2_DAT0, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD2_DAT1, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD2_DAT2, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD2_DAT3, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                break;
        default:
                break;
        }
}

/*!
 * Setup GPIO for LCD to be active
 *
 */
void gpio_lcd_active(void)
{
        u16 pbc_bctrl1_set = 0;

        /* RTS */
        iomux_config_mux(AP_IPU_D2_CS, OUTPUTCONFIG_FUNC2, INPUTCONFIG_FUNC2); 

        pbc_bctrl1_set = (u16) PBC_BCTRL1_LCDON;
        writew(pbc_bctrl1_set, PBC_BASE_ADDRESS + PBC_BCTRL1_SET); 
}

/*!
 * Setup GPIO for LCD to be inactive
 *
 */
void gpio_lcd_inactive(void)
{
        u16 pbc_bctrl1_set = 0;

        /* RTS */
        iomux_config_mux(AP_IPU_D2_CS, OUTPUTCONFIG_FUNC2, INPUTCONFIG_FUNC2); 

        pbc_bctrl1_set = (u16) PBC_BCTRL1_LCDON;
        writew(pbc_bctrl1_set, PBC_BASE_ADDRESS + PBC_BCTRL1_SET + 2); 
}

/*!
 * Setup GPIO for sensor to be active
 *
 */
void gpio_sensor_active(void)
{
        u16 temp;
     
        temp = PBC_BCTRL1_SENSORON; 

        writew(temp, PBC_BASE_ADDRESS + PBC_BCTRL1_SET); 
}

