/*
 * linux/arch/arm/mach-scma11/scma11evb_gpio.c
 *
 * Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* TODO: Need to complete GPIO configuration for SCM-A11 */

#include <asm/arch/gpio.h>
#include <asm/io.h>
#include <asm/arch/board.h>
#include <asm/arch/clock.h>
#include <linux/delay.h>
/*!
 * @file scma11evb_gpio.c
 *
 * @brief This file contains all the GPIO setup functions for the board.
 *
 * @ingroup GPIO
 */

/*!
 * This system-wise GPIO function initializes the pins during system startup.
 * All the statically linked device drivers should put the proper GPIO initialization
 * code inside this function. It is called by \b fixup_scma11evb() during
 * system startup. This function is board specific.
 */
void scma11evb_gpio_init(void)
{
    /* TODO: Need to configure IOMUX for SCM-A11 */
        /* config CS5 */

        iomux_config_mux(AP_GPIO_AP_C15, OUTPUTCONFIG_FUNC4, INPUTCONFIG_NONE);
}

/*!
 * Setup GPIO for a UART port to be active
 *
 * @param  port         a UART port
 * @param  no_irda      indicates if the port is used for SIR
 */
void gpio_uart_active(int port, int no_irda)
{
        unsigned int pbc_bctrl1_set = 0;

        /*
         * Configure the IOMUX control registers for the UART signals
         * and enable the UART transceivers
         */
        switch (port) {
        /* UART 1 IOMUX Configs */
        case 0:
                iomux_config_mux(AP_U1_TXD, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(AP_U1_CTS_B, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(AP_U1_RXD, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(AP_U1_RTS_B, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                /*
                 * Enable the UART 1 Transceiver or Irda Transmitter
                 */
                if (no_irda == 1) {
                        pbc_bctrl1_set |= PBC_BCTRL1_UENA;
                } else {
                        iomux_config_mux(AP_GPIO_AP_C17, OUTPUTCONFIG_DEFAULT, 
                                         INPUTCONFIG_DEFAULT);
                        gpio_config(2, 17, true, GPIO_INT_NONE);
                        /* Clear the SD/Mode signal */
                        gpio_set_data(2, 17, 0);
                        pbc_bctrl1_set |= PBC_BCTRL1_IREN;
                }
                break;
        /* UART 2 IOMUX Configs */
        case 1:
  
                iomux_config_mux(SP_USB_SE0_VM, OUTPUTCONFIG_FUNC2, INPUTCONFIG_FUNC2); /* TXD */
                iomux_config_mux(SP_USB_DAT_VP, OUTPUTCONFIG_FUNC2, INPUTCONFIG_FUNC2); /* RXD */
                iomux_config_mux(SP_USB_RXD, OUTPUTCONFIG_FUNC2, INPUTCONFIG_FUNC2); /* RTS */
                iomux_config_mux(AP_U2_CTS_B, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1); /* CTS */
                iomux_config_mux(AP_U2_RI_B, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1); /* RI */
                iomux_config_mux(AP_U2_DTR_B, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1); /* DTR */
                iomux_config_mux(AP_U2_DSR_B, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1); /* DSR */
                /*
                 * Enable the UART 2 Transceiver
                 */
                pbc_bctrl1_set |= PBC_BCTRL1_UENCE;
                break;
        /* UART 3 IOMUX Configs */
        case 2:
                iomux_config_mux(SP_U3_TXD, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_U3_CTS_B, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_U3_RXD, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_U3_RTS_B, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                /*
                 * Enable the UART 3 Transceiver or Irda Transmitter
                 */
                if (no_irda == 1) {
                        pbc_bctrl1_set |= PBC_BCTRL1_UENB;
                } else {
                        pbc_bctrl1_set |= PBC_BCTRL1_IREN;
                }
                break;
        default:
                break;
        }
        writew(pbc_bctrl1_set, PBC_BASE_ADDRESS + PBC_BCTRL1_SET);
        /*
         * TODO: Configure the Pad registers for the UART pins
         */
}

/*!
 * Setup GPIO for a UART port to be inactive
 *
 * @param  port         a UART port
 * @param  no_irda      indicates if the port is used for SIR
 */
void gpio_uart_inactive(int port, int no_irda)
{
        unsigned int pbc_bctrl1_clr = 0;

        /*
         * Disable the UART Transceiver by configuring the GPIO pin
         */
        switch (port) {
        case 0:
                /*
                 * Disable the UART 1 Transceiver
                 */
                if (no_irda == 1) {
                        pbc_bctrl1_clr |= PBC_BCTRL1_UENA;
                } else {
                        pbc_bctrl1_clr |= PBC_BCTRL1_IREN;
                }
                break;
        case 1:
                /*
                 * Disable the UART 2 Transceiver
                 */
                pbc_bctrl1_clr |= PBC_BCTRL1_UENCE;
                break;
        case 2:
                /*
                 * Disable the UART 3 Transceiver
                 */
                if (no_irda == 1) {
                        pbc_bctrl1_clr |= PBC_BCTRL1_UENB;
                } else {
                        pbc_bctrl1_clr |= PBC_BCTRL1_IREN;
                }
                break;
        default:
                break;
        }
        writew(pbc_bctrl1_clr, PBC_BASE_ADDRESS + PBC_BCTRL1_CLEAR);
}

/*
 * The function is stubbed out and does not apply for SCM-A11
 */
void config_uartdma_event(int port)
{
        return;
}

/*!
 *  Setup GPIO for keypad to be active
 */
void gpio_keypad_active(void)
{
        iomux_config_mux(AP_KPROW0, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPROW1, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPROW2, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPROW3, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPROW4, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPROW5, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_GPIO_AP_B17, OUTPUTCONFIG_FUNC1,
                         INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_GPIO_AP_B18, OUTPUTCONFIG_FUNC1,
                         INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPCOL0, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPCOL1, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPCOL2, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPCOL3, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPCOL4, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_KPCOL5, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_GPIO_AP_B22, OUTPUTCONFIG_FUNC1,
                         INPUTCONFIG_FUNC1);
        iomux_config_mux(AP_GPIO_AP_B23, OUTPUTCONFIG_FUNC1,
                         INPUTCONFIG_FUNC1);
}

/*!
 * Setup GPIO for keypad to be inactive
 */
void gpio_keypad_inactive(void)
{
        iomux_config_mux(AP_KPROW0, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPROW1, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPROW2, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPROW3, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPROW4, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPROW5, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_GPIO_AP_B17, OUTPUTCONFIG_DEFAULT,
                         INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_GPIO_AP_B18, OUTPUTCONFIG_DEFAULT,
                         INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPCOL0, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPCOL1, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPCOL2, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPCOL3, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPCOL4, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_KPCOL5, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_GPIO_AP_B22, OUTPUTCONFIG_DEFAULT,
                         INPUTCONFIG_DEFAULT);
        iomux_config_mux(AP_GPIO_AP_B23, OUTPUTCONFIG_DEFAULT,
                         INPUTCONFIG_DEFAULT);
}

/*!
 * Setup GPIO for a CSPI device to be active
 *
 * @param  cspi_mod         an CSPI device
 */
void gpio_spi_active(int cspi_mod)
{
        switch (cspi_mod) {
        case 0:
                /* SPI1_SS0 IOMux configuration */
                iomux_config_mux(SP_SPI1_SS0, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI1 MISO IOMux configuration */
                iomux_config_mux(SP_SPI1_MISO, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI1 MOSI IOMux configuration */
                iomux_config_mux(SP_SPI1_MOSI, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI1_SS1 IOMux configuration */
                iomux_config_mux(SP_SPI1_SS1, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI1 CLK IOMux configuration */
                iomux_config_mux(SP_SPI1_CLK, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                break;
        case 1:
                /* SPI2 MOSI IOMux configuration */
                iomux_config_mux(SP_SD2_CMD, OUTPUTCONFIG_FUNC2, INPUTCONFIG_FUNC2);
                /* SPI2 SPI2_CLK IOMux configuration */
                iomux_config_mux(SP_SD2_CLK, OUTPUTCONFIG_FUNC2, INPUTCONFIG_FUNC2);
                /* SPI2 SPI2_MISO IOMux configuration */
                iomux_config_mux(SP_SD2_DAT0, OUTPUTCONFIG_FUNC2, INPUTCONFIG_FUNC2);
                /* SPI2 SPI2_SS3 IOMux configuration */
                iomux_config_mux(SP_SD2_DAT3, OUTPUTCONFIG_FUNC2, INPUTCONFIG_FUNC2);
                break;
        default:
                break;
        }
}

/*!
 * Setup GPIO for a CSPI device to be inactive
 *
 * @param  cspi_mod         a CSPI device
 */
void gpio_spi_inactive(int cspi_mod)
{
        switch (cspi_mod) {
        case 0:
                /* SPI1_SS0 IOMux configuration */
                iomux_config_mux(SP_SPI1_SS0, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI1 MISO IOMux configuration */
                iomux_config_mux(SP_SPI1_MISO, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI1 MOSI IOMux configuration */
                iomux_config_mux(SP_SPI1_MOSI, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI1_SS1 IOMux configuration */
                iomux_config_mux(SP_SPI1_SS1, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI1 CLK IOMux configuration */
                iomux_config_mux(SP_SPI1_CLK, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                break;
        case 1:
                /* SPI2 MOSI IOMux configuration */
                iomux_config_mux(SP_SD2_CMD, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI2 SPI2_CLK IOMux configuration */
                iomux_config_mux(SP_SD2_CLK, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI2 SPI2_MISO IOMux configuration */
                iomux_config_mux(SP_SD2_DAT0, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                /* SPI2 SPI2_SS3 IOMux configuration */
                iomux_config_mux(SP_SD2_DAT3, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                break;
        default:
                break;
        }
}

/*!
 * Setup GPIO for an I2C device to be active
 *
 * @param  i2c_num         an I2C device
 */
void gpio_i2c_active(int i2c_num)
{
        switch (i2c_num) {
        case 0:
                iomux_config_mux(AP_I2CLK, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(AP_I2DAT, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        default:
                break;
        }
}

/*!
 * Setup GPIO for an I2C device to be inactive
 *
 * @param  i2c_num         an I2C device
 */
void gpio_i2c_inactive(int i2c_num)
{
        switch (i2c_num) {
        case 0:
                iomux_config_mux(AP_I2CLK, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(AP_I2DAT, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                break;
        default:
                break;
        }
}

#ifdef CONFIG_ARCH_SCMA11EVB
#define SCMA11_EVB_PASS1 1
#define SCMA11_EVB_PASS1_SSI1 1
#define SCMA11_EVB_PASS1_SSI2 1
/* See dam.c to complete the workaround for SSI scma11 pass1 */
#endif

/*!
 * This function configures the IOMux block for Atlas standard operations.
 *
 */
void gpio_atlas_active(void) 
{
        iomux_config_mux(AP_ED_INT5, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
        edio_config(ED_INT5, 0,EDIO_INT_RISE_EDGE);

#ifdef SCMA11_EVB_PASS1
#ifdef SCMA11_EVB_PASS1_SSI1
        iomux_config_mux(AP_GPIO_AP_C8, OUTPUTCONFIG_FUNC2, INPUTCONFIG_FUNC2);
#endif
#ifdef SCMA11_EVB_PASS1_SSI2
        iomux_config_mux(AP_GPIO_AP_C16, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
#endif
#endif
}

/*!
 * This function clears the Atlas intrrupt.
 *
 */
void gpio_atlas_clear_int(void) 
{
        edio_clear_int(ED_INT5);
        edio_get_int(ED_INT5);
}

/*!
 * This function return the SPI connected to Atlas.
 *
 */
int gpio_atlas_get_spi(void) 
{
        /* SCMA11 -> SPI2 = 1 */
        return 1;
}

/*!
 * This function return the SPI smave select for Atlas.
 *
 */
int gpio_atlas_get_ss(void) 
{
        /* SCMA11EVB -> SS = 3 */
        return 3;
}

/*!
 * Setup GPIO for SDHC to be active
 *
 * @param module SDHC module number
 */
void gpio_sdhc_active(int module) 
{
        unsigned int pbc_bctrl1_set = 0;
        switch(module){
        case 0:
                iomux_config_mux(SP_SD1_CMD, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD1_CLK, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD1_DAT0, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD1_DAT1, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD1_DAT2, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD1_DAT3, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                /*
                 * select 2.53 - 3.32v voltage for the MMC1
                 * interface.
                 */
                pbc_bctrl1_set |= PBC_BCTRL1_MCP1;
                break;
        case 1:
                iomux_config_mux(SP_SD2_CMD, OUTPUTCONFIG_FUNC1, INPUTCONFIG_NONE);
                iomux_config_mux(SP_SD2_CLK, OUTPUTCONFIG_FUNC1, INPUTCONFIG_NONE);
                iomux_config_mux(SP_SD2_DAT0, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD2_DAT1, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD2_DAT2, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                iomux_config_mux(SP_SD2_DAT3, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                /*
                 * select 2.53 - 3.32v voltage for the MMC2
                 * interface.
                 */
                pbc_bctrl1_set |= PBC_BCTRL1_MCP2;
                break;
        default:
                break;
        }
        pbc_bctrl1_set |= PBC_BCTRL1_MCP2;
        writew(pbc_bctrl1_set, PBC_BASE_ADDRESS + PBC_BCTRL1_SET);
}

/*!
 * Setup GPIO for SDHC1 to be inactive
 *
 * @param module SDHC module number
 */
void gpio_sdhc_inactive(int module) 
{
        unsigned int pbc_bctrl1_set = 0;
        switch(module){
        case 0:
                iomux_config_mux(SP_SD1_CMD, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD1_CLK, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD1_DAT0, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD1_DAT1, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD1_DAT2, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD1_DAT3, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                pbc_bctrl1_set &= PBC_BCTRL1_MCP1;
                break;
        case 1:
                iomux_config_mux(SP_SD2_CMD, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD2_CLK, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD2_DAT0, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD2_DAT1, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD2_DAT2, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                iomux_config_mux(SP_SD2_DAT3, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                pbc_bctrl1_set &= PBC_BCTRL1_MCP2;
                break;
        default:
                break;
        }
        writew(pbc_bctrl1_set, PBC_BASE_ADDRESS + PBC_BCTRL1_SET);
}

/*!
 * Setup the IOMUX/GPIO for SDHC1 SD1_DET.
 * 
 * @param  host Pointer to MMC/SD host structure.
 * @param  handler      GPIO ISR function pointer for the GPIO signal.
 * @return The function returns 0 on success and -1 on failure.
 *
 **/
int sdhc_intr_setup(void *host, gpio_irq_handler handler)
{
        int ret;

        /* use SD2_DAT1 as GPIO for SD1_DET */
        iomux_config_mux(SP_SD2_DAT1, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
        gpio_config(3, 23, false, GPIO_INT_FALL_EDGE);
        ret = gpio_request_irq(3, 23, GPIO_HIGH_PRIO, handler,
                        0, "MXCMMC", host);
        if (ret != 0) {
                gpio_free_irq(3, 23, GPIO_HIGH_PRIO);
        }
        return ret;
}

/*!
 * Clear the IOMUX/GPIO for SDHC1 SD1_DET.
 * 
 * @param flag Flag represents whether the card is inserted/removed.
 *             Using this sensitive level of GPIO signal is changed.
 *
 **/
void sdhc_intr_clear(int *flag)
{
        if(*flag) {
                gpio_config(3, 23, false, GPIO_INT_FALL_EDGE);
                *flag = 0;
        } else {
                gpio_config(3, 23, false, GPIO_INT_RISE_EDGE);
                *flag = 1;
        }
}

/*
 * Find the minimum clock for SDHC.
 *
 * @param clk SDHC module number.
 * @return Returns the minimum SDHC clock.
 */
unsigned int sdhc_get_min_clock(enum mxc_clocks clk)
{
        return (mxc_get_clocks(SDHC1_CLK) / 9) / 32 ;
}

/*
 * Find the maximum clock for SDHC.
 * @param clk SDHC module number.
 * @return Returns the maximum SDHC clock.
 */
unsigned int sdhc_get_max_clock(enum mxc_clocks clk)
{
        return mxc_get_clocks(SDHC1_CLK) / 2;
}

/*
 * Probe for the card. If present the GPIO data would be set.
 */
int sdhc_find_card(void)
{
        return gpio_get_data(3, 23);
}

/*!
 * Setup GPIO for LCD to be active
 *
 */
void gpio_lcd_active(void)
{
        u16 pbc_bctrl1_set = 0;

        /* RTS */
        iomux_config_mux(AP_IPU_D2_CS, OUTPUTCONFIG_FUNC2, INPUTCONFIG_FUNC2); 

        pbc_bctrl1_set = (u16) PBC_BCTRL1_LCDON;
        writew(pbc_bctrl1_set, PBC_BASE_ADDRESS + PBC_BCTRL1_SET); 
}
       
/*!
 * Setup GPIO for LCD to be inactive
 *
 */
void gpio_lcd_inactive(void)
{
        u16 pbc_bctrl1_set = 0;

        /* RTS */
        iomux_config_mux(AP_IPU_D2_CS, OUTPUTCONFIG_FUNC2, INPUTCONFIG_FUNC2); 

        pbc_bctrl1_set = (u16) PBC_BCTRL1_LCDON;
        writew(pbc_bctrl1_set, PBC_BASE_ADDRESS + PBC_BCTRL1_SET + 2); 
}

/*!
 * Setup GPIO for sensor to be active
 *
 */
void gpio_sensor_active(void)
{
        u16 temp;
     
        temp = PBC_BCTRL1_SENSORON; 

        writew(temp, PBC_BASE_ADDRESS + PBC_BCTRL1_SET); 
}

/*!
 * Setup EDIO/IOMUX for external UART.
 *
 * @param port UART port
 * @return    The function returns 0 if the interrupt pin is connected to 
 *            EDIO or 1 if the pin is connected to GPIO. For GPIO pin the return
 *            value is used to make gpio_request_irq() instead of request_irq().
 */
int extuart_intr_setup(int port)
{
        /*
         * 16C652 UART-A and UART-B interrupt pins are connected to
         * e_GPIO24 and e_GPIO25. They are routed to ED_INT6 and GPIO2_30
         * by configuring the IOMUX, EDIO and GPIO.
         */

        if (port == 0) {
                iomux_config_mux(AP_ED_INT6, OUTPUTCONFIG_FUNC1, INPUTCONFIG_FUNC1);
                edio_config(ED_INT6, 0, EDIO_INT_RISE_EDGE);
                return 0;
        } else if (port == 1) {
                /* 
                 * Configure the GPIO pin
                 */
                iomux_config_mux(SP_SIM1_RST_B, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                gpio_config(2, 30, false, GPIO_INT_RISE_EDGE);
                return 1;

        } else {
                printk("%s():Invalid port:%d\n", __FUNCTION__, port);
                return 0;
        }

}

/*!
 * Get the EDIO interrupt, clear if set. 
 *
 * @param port UART port
 */
void extuart_intr_clear(int port)
{
        /* Clear any Interrupt bits. */
        if (port == 0) {
                if (edio_get_int(ED_INT6)) {
                        edio_clear_int(ED_INT6);
                }
        } else if (port == 1) {
                /*
                 * Clears the bit in the GPIO register.
                 */ 
                gpio_clear_int(2, 30);
        } else {
                printk("%s():Invalid port:%d\n", __FUNCTION__, port);
        }
}

/*!
 * Do IOMUX configs required to put the
 * pin back in low power mode.
 *
 * @param  port UART port
 * @return  The function returns 0 if the interrupt pin is connected to 
 *          EDIO or 1 if the pin is connected to GPIO. For GPIO pin the 
 *          return value is used to make gpio_free_irq() instead of free_irq().
 */
int extuart_intr_cleanup(int port)
{
        if (port == 0) {
                iomux_config_mux(AP_ED_INT6, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                edio_config(ED_INT6, 0, EDIO_INT_NONE);
                return 0;
        } else if (port == 1) {
                /* 
                 * Configure the GPIO pin.
                 */
                iomux_config_mux(SP_SIM1_RST_B, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);
                gpio_config(2, 30, false, GPIO_INT_NONE);
                return 1;

        } else {
                printk("%s():Invalid port:%d\n", __FUNCTION__, port);
                return 0;
        }
}

int enet_request_irq(unsigned int irq, 
        irqreturn_t (*handler)(int, void *, struct pt_regs *),
        unsigned long irq_flags, const char * devname, void *dev_id)
{
    int ret = 0;
    u16 pbc_ver;
    
    /*
     * Configure INT line as EDIO input
     */
    pbc_ver = (*(volatile unsigned short*)PBC_BASE_ADDRESS) >> 8;
    edio_set_data(ED_INT3, 0);
    edio_config(ED_INT3, 0, EDIO_INT_LOW_LEV);

    ret = request_irq(irq, handler, irq_flags, devname, dev_id);
    return ret;
}

int enet_free_irq(unsigned int irq, void *dev_id)
{
    edio_config(ED_INT3, 0, EDIO_INT_NONE);
    edio_clear_int(ED_INT3);
    free_irq(irq, dev_id);
    return 0;
}

void slcd_gpio_config(void)
{
        gpio_config( 2, 22, true, GPIO_INT_NONE);
        gpio_set_data(2, 22, 1);
        iomux_config_mux(AP_ED_INT4, OUTPUTCONFIG_DEFAULT, INPUTCONFIG_DEFAULT);

        gpio_set_data(2, 22, 0);
        msleep(100);
        gpio_set_data(2, 22, 1);
        msleep(100);
}
