/*
 *  linux/arch/arm/mach-zeus/zeusevb.c
 *
 *  Copyright (C) 2000 Deep Blue Solutions Ltd
 *  Copyright (C) 2002 Shane Nay (shane@minirl.com)
 *  Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/config.h>
#include <linux/types.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/init.h>
#include <linux/serial_8250.h>
#include <linux/kgdb.h>

#include <asm/hardware.h>
#include <asm/irq.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/arch/memory.h>
#include <asm/arch/gpio.h>
#include <asm/arch/clock.h>
#include <asm/arch/board.h>

/*!
 * @file zeusevb.c
 * 
 * @brief This file contains the board specific initialization routines. 
 * 
 * @ingroup System
 */

extern void zeus_map_io(void);
extern void zeus_init_irq(void);
extern void zeus_cpu_init(void) __init;
extern void zeusevb_gpio_init(void) __init;
extern struct sys_timer mxc_timer;

/*!
 * The serial port definition structure. The fields contain:
 * {UART, CLK, PORT, IRQ, FLAGS}
 */
static struct plat_serial8250_port serial_platform_data[] = {
	{
		.iobase	= (unsigned long)(PBC_BASE_ADDRESS + PBC_SC16C652_UARTA),
		.irq		= INT_RESV1,    /* DUMMY value, shared GPIO3 IRQ */
		.uartclk	= 14745600,
		.regshift	= 0,
		.iotype		= UPIO_PORT,
		.flags		= UPF_BOOT_AUTOCONF | UPF_SKIP_TEST,
	}, {
		.iobase	= (unsigned long)(PBC_BASE_ADDRESS + PBC_SC16C652_UARTB),
		.irq		= INT_RESV2,    /* DUMMY value, shared GPIO3 IRQ */
		.uartclk	= 14745600,
		.regshift	= 0,
		.iotype		= UPIO_PORT,
		.flags		= UPF_BOOT_AUTOCONF | UPF_SKIP_TEST,
	},
};

/*!
 * REVISIT: document me
 */
static struct platform_device serial_device = {
	.name			= "serial8250",
	.id			= 0,
	.dev			= {
		.platform_data	= serial_platform_data,
	},
};

/*!
 * REVISIT: document me
 */
static int __init zeusevb_init(void)
{
	return platform_device_register(&serial_device);
}

arch_initcall(zeusevb_init);

/*!
 * ZEUS EVB specific fixup function. It is called by \b setup_arch() in 
 * setup.c file very early on during kernel starts. It allows the user to 
 * statically fill in the proper values for the passed-in parameters. None of 
 * the parameters is used currently.
 * 
 * @param  desc         pointer to \b struct \b machine_desc
 * @param  tags         pointer to \b struct \b tag
 * @param  cmdline      pointer to the command line
 * @param  mi           pointer to \b struct \b meminfo
 */
void __init fixup_zeusevb(struct machine_desc *desc, struct tag *tags,
        char **cmdline, struct meminfo *mi) 
{
#ifdef CONFIG_KGDB_8250
	int i;
	for (i=0;
	     i < (sizeof(serial_platform_data)/sizeof(serial_platform_data[0]));
	     i += 1)
		kgdb8250_add_platform_port(i, &serial_platform_data[i]);
#endif

        zeus_cpu_init();
}

/*!
 * Board specific initialization.
 */
static void __init mxc_board_init(void)
{        
        zeusevb_gpio_init();

        printk("\n------------------\n");
        printk("Main system clocks\n");
        printk("------------------\n");        
        printk("CPU_CLK = %lu Hz\n", mxc_get_clocks(CPU_CLK));
        printk("AHB_CLK = %lu Hz\n", mxc_get_clocks(AHB_CLK));
        printk("IPG_CLK = %lu Hz\n", mxc_get_clocks(IPG_CLK));
        printk("NFC_CLK = %lu Hz\n", mxc_get_clocks(NFC_CLK));
        printk("USB_CLK = %lu Hz\n", mxc_get_clocks(USB_CLK));
        printk("CKIH_CLK = %lu Hz\n", mxc_get_clocks(CKIH_CLK));
        printk("CKIL_CLK = %lu Hz\n", mxc_get_clocks(CKIL_CLK));
        printk("DIGRF_CLK = %lu Hz\n", mxc_get_clocks(DIGRF_CLK));
        
        printk("------------------\n");
        printk("Peripheral clocks\n");
        printk("------------------\n");        
        printk("UART1_BAUD = %lu Hz\n", mxc_get_clocks(UART1_BAUD));
        printk("UART2_BAUD = %lu Hz\n", mxc_get_clocks(UART2_BAUD));
        printk("UART3_BAUD = %lu Hz\n", mxc_get_clocks(UART3_BAUD));
        printk("SIM_BAUD = %lu Hz\n", mxc_get_clocks(SIM1_CLK));
        printk("SSI1_BAUD = %lu Hz\n", mxc_get_clocks(SSI1_BAUD));
        printk("SSI2_BAUD = %lu Hz\n", mxc_get_clocks(SSI2_BAUD));
        printk("CSI_BAUD = %lu Hz\n", mxc_get_clocks(CSI_BAUD));
        printk("SDHC1_CLK = %lu Hz\n", mxc_get_clocks(SDHC1_CLK));
        printk("SDHC2_CLK = %lu Hz\n", mxc_get_clocks(SDHC2_CLK));
        printk("FIRI_BAUD = %lu Hz\n", mxc_get_clocks(FIRI_BAUD));
        printk("IPU_CLK = %lu Hz\n", mxc_get_clocks(IPU_CLK));
}

/*
 * The following uses standard kernel macros define in arch.h in order to 
 * initialize __mach_desc_ZEUSEVB data structure.
 */
MACHINE_START(ZEUSEVB, "Freescale ZEUS EVB")
        MAINTAINER("Freescale Semiconductor")
#ifdef CONFIG_SERIAL_8250_CONSOLE
        /*       physical memory    physical IO        virtual IO     */
        BOOT_MEM(PHYS_OFFSET_ASM, CS4_BASE_ADDR, CS4_BASE_ADDR_VIRT)
#else
        /*       physical memory    physical IO        virtual IO     */
        BOOT_MEM(PHYS_OFFSET_ASM, AIPS1_BASE_ADDR, AIPS1_BASE_ADDR_VIRT)
#endif
        BOOT_PARAMS(PHYS_OFFSET_ASM + 0x100)
        FIXUP(fixup_zeusevb)
        MAPIO(zeus_map_io)
        INITIRQ(zeus_init_irq)
        INIT_MACHINE(mxc_board_init)
        .timer		= &mxc_timer,
MACHINE_END

