/*
 *  File Name       : drivers/char/mp200-led.c
 *  Function        : LED Driver
 *  Release Version : Ver 1.00
 *  Release Date    : 2006/03/23
 *
 *  Copyright (C) NEC Electronics Corporation 2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/device.h>
#include <linux/kdev_t.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>
#include <linux/cdev.h>
#include <asm/uaccess.h>
#include <asm/arch/gpiocore.h>
#include <asm/arch/led.h>

static int mp200_leddrv_open(struct inode *inode, struct file *filp);
static int mp200_leddrv_close(struct inode *inode, struct file *filp);
static int mp200_leddrv_ioctl(struct inode *inode, struct file *filp,
			      unsigned int cmd, unsigned long arg);

static int __init mp200_leddrv_init(void);
static void __exit mp200_leddrv_exit(void);

#define DEVICE_NAME	"led"

static struct file_operations mp200_leddrv_fops = {
	.owner = THIS_MODULE,
	.open = mp200_leddrv_open,
	.release = mp200_leddrv_close,
	.ioctl = mp200_leddrv_ioctl,
	.read = NULL,
	.write = NULL,
	.llseek = NULL,
};

static struct miscdevice mp200_leddrv_miscdev = {
	.minor = MISC_DYNAMIC_MINOR,
	.name = DEVICE_NAME,
	.fops = &mp200_leddrv_fops,
};

/***************************************************************************
** MODULE   : mp200_leddrv_open
** FUNCTION : OPEN Function
** ARG      : inode     : inode structure pointer
**          : filp      : file pointer
** RETURN   : 0         : success
**          : negative  : fail
****************************************************************************/
static int mp200_leddrv_open(struct inode *inode, struct file *filp)
{
	return (0);
}

/***************************************************************************
** MODULE   : mp200_leddrv_close
** FUNCTION : CLOSE Function
** ARG      : inode     : inode structure pointer
**          : filp      : file pointer
** RETURN   : 0         : success
**          : negative  : fail
****************************************************************************/
static int mp200_leddrv_close(struct inode *inode, struct file *filp)
{
	return (0);
}

/***************************************************************************
** MODULE   : mp200_leddrv_ioctl
** FUNCTION : IOCTL Function
** ARG      : inode     : inode structure pointer
**          : filp      : file pointer
**          : cmd       : command number
**          : arg       : command argument
** RETURN   : 0         : success
**          : negative  : fail
****************************************************************************/
static int mp200_leddrv_ioctl(struct inode *inode, struct file *filp,
			      unsigned int cmd, unsigned long arg)
{
	unsigned char data;
	unsigned int led_write_data;
	unsigned int led_read_data;
	int ret;
	int err = 0;

	if (_IOC_TYPE(cmd) > MP200_LED_MAGIC) {
		return (-ENOTTY);
	}

	if (_IOC_NR(cmd) > MP200_LED_IOC_NR) {
		return (-ENOTTY);
	}

	if (_IOC_DIR(cmd) & _IOC_READ) {
		err =
		    !access_ok(VERIFY_WRITE, (void __user *)arg,
			       _IOC_SIZE(cmd));
	} else if (_IOC_DIR(cmd) & _IOC_WRITE) {
		err =
		    !access_ok(VERIFY_READ, (void __user *)arg, _IOC_SIZE(cmd));
	}

	if (err) {
		return (-EFAULT);
	}

	switch (cmd) {
	case SET_LED_DATA:
		__get_user(data, (unsigned char *)arg);

		led_write_data =
		    (unsigned int)(data &
				   (MP200_BIT_LED5 | MP200_BIT_LED4 |
				    MP200_BIT_LED3));
		led_write_data <<= 4;
		led_write_data = ~led_write_data;
		ret = gpiocore_pwc_write(GPIOCORE_PWC_IOOUT2, led_write_data, (unsigned int)0x00000070);	/* Set LED(LED05,LED04,LED03) Data */
		if (ret != 0) {
			return (-ret);
		}

		led_write_data = (unsigned int)(data & MP200_BIT_LED6);
		led_write_data >>= 3;
		led_write_data = ~led_write_data;
		ret = gpiocore_pwc_write(GPIOCORE_PWC_IOOUTSEL, led_write_data, (unsigned int)0x00000001);	/* Set LED(LED06) Data */
		if (ret != 0) {
			return (-ret);
		}

		break;
	case GET_LED_DATA:
		ret = gpiocore_pwc_read(PWC_IOOUT2, &led_read_data);	/* Get LED(LED05,LED04,LED03) Data */
		if (ret != 0) {
			return (-ret);
		}

		led_read_data = ~led_read_data;
		data = (unsigned char)(led_read_data >> 4);
		data &= (MP200_BIT_LED5 | MP200_BIT_LED4 | MP200_BIT_LED3);

		ret = gpiocore_pwc_read(PWC_IOOUTSEL, &led_read_data);	/* Get LED(LED06) Data */
		if (ret != 0) {
			return (-ret);
		}

		led_read_data = ~led_read_data;
		led_read_data <<= 3;
		led_read_data &= ((unsigned int)MP200_BIT_LED6);

		data |= ((unsigned char)led_read_data);

		__put_user(data, (unsigned char *)arg);

		break;
	default:
		return (-EINVAL);
	}

	return (0);
}

/***************************************************************************
** MODULE   : mp200_leddrv_init
** FUNCTION : initialize module
** ARG      : none
** RETURN   : 0         : success
**          : negative  : fail
****************************************************************************/
static int __init mp200_leddrv_init(void)
{
	int ret;

	printk(KERN_INFO "Starting LED.\n");

	ret = gpiocore_pwc_write(GPIOCORE_PWC_IOSEL2, (unsigned int)0x00000000, (unsigned int)0x00000070);	/* INIT LED_PORT */
	if (ret != 0) {
		return (-ret);
	}

	ret = gpiocore_pwc_write(GPIOCORE_PWC_IOOUT2, (unsigned int)0x00000070, (unsigned int)0x00000070);	/* LED05...LED03 : OFF */
	if (ret != 0) {
		return (-ret);
	}

	ret = gpiocore_pwc_write(GPIOCORE_PWC_IOOUTSEL, (unsigned int)0x00000001, (unsigned int)0x00000001);	/* LED06 : OFF */
	if (ret != 0) {
		return (-ret);
	}

	ret = misc_register(&mp200_leddrv_miscdev);
	if (ret != 0) {
		return (ret);
	}

	return (0);
}

/***************************************************************************
** MODULE   : mp200_leddrv_exit
** FUNCTION : cleanup module
** ARG      : none
** RETURN   : none
****************************************************************************/
static void __exit mp200_leddrv_exit(void)
{
	int ret;

	ret = misc_deregister(&mp200_leddrv_miscdev);
	if (ret != 0) {
		return;
	}

	ret = gpiocore_pwc_write(GPIOCORE_PWC_IOSEL2, (unsigned int)0x00000000, (unsigned int)0x00000070);	/* INIT LED_PORT */
	if (ret != 0) {
		return;
	}

	ret = gpiocore_pwc_write(GPIOCORE_PWC_IOOUT2, (unsigned int)0x00000070, (unsigned int)0x00000070);	/* LED05...LED03 : OFF */
	if (ret != 0) {
		return;
	}

	ret = gpiocore_pwc_write(GPIOCORE_PWC_IOOUTSEL, (unsigned int)0x00000001, (unsigned int)0x00000001);	/* LED06 : OFF */
	if (ret != 0) {
		return;
	}

	ret = gpiocore_pwc_write(GPIOCORE_PWC_IOSEL2, (unsigned int)0x00000070, (unsigned int)0x00000070);	/* INIT LED_PORT */
	if (ret != 0) {
		return;
	}
}

module_init(mp200_leddrv_init);
module_exit(mp200_leddrv_exit);

MODULE_DESCRIPTION("MP200 LED Driver");
MODULE_LICENSE("GPL");

/* end of mp200-led.c */
