/*
 *
 * Copyright 2006 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MEReqChannelANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include<linux/init.h>
#include<linux/module.h>
#include<linux/errno.h>
#include<linux/moduleparam.h>
#include<linux/init.h>
#include<linux/kernel.h>
#include<linux/interrupt.h>
#include<linux/slab.h>
#include<linux/fs.h>
#include<linux/errno.h>
#include<linux/types.h>
#include<linux/proc_fs.h>
#include<linux/fcntl.h>
#include<linux/seq_file.h>
#include<linux/device.h>
#include<linux/cdev.h>
#include<linux/devfs_fs.h>
#include<linux/devfs_fs_kernel.h>

#include<asm/system.h>
#include<asm/uaccess.h>
#include<asm/delay.h>

#include<asm/arch/gpiocore.h>
#include<asm/arch/mp200_ad.h>

/*******************/
/*@ Define & Macro */
/*******************/

#define MAJOR_NUMBER 0 /* 0 = dynamic */

#define AD_TIMEOUT 1000

#define INIF_REG_MJR 0x00000001
#define INIF_ADD_CDV 0x00000002
#define INIF_REG_DRV 0x00000004
#define INIF_REG_DEV 0x00000008
#define INIF_MAK_DFS 0x00000010
#define INIF_ENB_REG 0x00000020
#define INIF_REQ_IRQ 0x00000040

#define AD_DATA_NONE  0
#define AD_DATA_GOOD  1
#define AD_DATA_FAIL -1

#define maximum(_a,_b) ((_a)>(_b)?(_a):(_b))
#define minimum(_a,_b) ((_a)<(_b)?(_a):(_b))

#ifdef AD_DEBUG
#define dbgprint(fmt, args...) printk(fmt, ##args)
#else
#define dbgprint(fmt, args...)
#endif

typedef unsigned long  word;
typedef unsigned short half;
typedef unsigned char  byte;

/**************/
/*@ Structure */
/**************/

/**************/
/*@ Prototype */
/**************/

int mp200_ad_init(void);
void mp200_ad_exit(void);

static void mp200_ad_shutdown(
	struct device *dev
);
static int mp200_ad_suspend(
	struct device *dev,
	u32 state,
	u32 level
);
static int mp200_ad_resume(
	struct device *dev,
	u32 level
);

static int mp200_ad_open(
	struct inode *inode,
	struct file *file
);
static int mp200_ad_release(
	struct inode *inode,
	struct file *file
);
static int mp200_ad_ioctl(
	struct inode *inode,
	struct file *file,
	unsigned int cmd,
	unsigned long arg
);

static irqreturn_t mp200_ad_interrupt(
	int irq,
	void *dev_id,
	struct pt_regs *fp
);

/********************/
/*@ Global Variable */
/********************/

struct platform_device MP200_AD_Device = {
	.name = "mp200_ad",
	.id  =  0,
};

struct device_driver MP200_AD_Driver = {
	.name =     "mp200_ad",
	.bus =      &platform_bus_type,
	.shutdown = mp200_ad_shutdown,
	.suspend =  mp200_ad_suspend,
	.resume =   mp200_ad_resume
};

struct file_operations MP200_AD_Fops = {
	.owner =    THIS_MODULE,
	.ioctl =    mp200_ad_ioctl,
	.open =     mp200_ad_open,
	.release =  mp200_ad_release,
};

static int  MajorNumber=MAJOR_NUMBER;
static word InitFlag=0;
static int  DirverReady=0;

static int ReqChannel; /* request ch 0-7 */
static int DataStatus; /* data status    */
static int DataValue;  /* data value     */

static spinlock_t        Spinlock;  /* device spinlock     */
static struct semaphore  Semaphore; /* device semaphore    */
static wait_queue_head_t Waitque;   /* data ready wait que */
static struct cdev       CDev;

/************************/
/*@ User Entry Routines */
/************************/

int mp200_ad_init(void)
{
	dev_t dev;
	int rv;

	printk(KERN_INFO "Starting A/D.\n");

	/***************************/
	/* Private Data Initialize */
	/***************************/

	ReqChannel=-1;
	DataStatus=AD_DATA_NONE;
	DataValue=0;
	spin_lock_init(&Spinlock);
	init_MUTEX(&Semaphore);
	init_waitqueue_head(&Waitque);

	/*******************/
	/* Register Driver */
	/*******************/

	/* fixed major */
	if(MajorNumber!=0){
		dev=MKDEV(MajorNumber,0);
		rv=register_chrdev_region(dev,1,"mp200_ad");
	}

	/* dynamic major */
	else{
		rv=alloc_chrdev_region(&dev,0,1,"mp200_ad");
		MajorNumber=MAJOR(dev);
	}

	if(rv<0){
		printk(KERN_ERR "mp200_ad: can't get major. %d\n",MajorNumber);
		mp200_ad_exit();
		return(rv);
	}
	InitFlag=InitFlag|INIF_REG_MJR;

	/* initialize cdev */
	cdev_init(&CDev,&MP200_AD_Fops);
	CDev.owner=THIS_MODULE;

	/* add driver */
	rv=cdev_add(&CDev,dev,1);
	if(rv){
		printk(KERN_ERR "mp200_ad: cdev_add error %d\n",rv);
		mp200_ad_exit();
		return(rv);
	}
	InitFlag=InitFlag|INIF_ADD_CDV;

	/* register driver */
	rv=driver_register(&MP200_AD_Driver);
	if(rv){
		printk(KERN_ERR "mp200_ad: driver_register error %d\n",rv);
		mp200_ad_exit();
		return(rv);
	}
	InitFlag=InitFlag|INIF_REG_DRV;

	/* register device */
	rv=platform_device_register(&MP200_AD_Device);
	if(rv){
		printk(KERN_ERR "mp200_ad: palatform_device_register error %d\n",rv);
		mp200_ad_exit();
		return(rv);
	}
	InitFlag=InitFlag|INIF_REG_DEV;

#ifdef CONFIG_DEVFS_FS
	/* create devfs */
	rv=devfs_mk_cdev(dev,S_IFCHR|S_IRUSR|S_IWUSR|S_IRGRP,"mp200_ad");
	if(rv){
		printk(KERN_ERR "mp200_ad: devfs_mk_cdev error %d\n",rv);
		mp200_ad_exit();
		return(rv);
	}
	InitFlag=InitFlag|INIF_MAK_DFS;
#endif

	/*******************/
	/* Hardware Config */
	/*******************/

	/* enable regulator */
	/* rv=pwc_regulator_on(PWC_PSCNTHP); */
	/* if(rv){ */
		/* printk(KERN_ERR "mp200_ad: pwc_regulator_on error %d\n",rv); */
		/* mp200_ad_exit(); */
		/* return(rv); */
	/* } */
	/* InitFlag=InitFlag|INIF_ENB_REG; */

	/* setup interrupt det type 0:level 1:rise 2:fall*/
	gpiocore_pwc_write(GPIOCORE_PWC_DETMOD3L,0x01,0x03);

	/* clear factor */
	gpiocore_pwc_write(GPIOCORE_PWC_CLRFACT3,0x01,0x01);

	/* register interrupt handler */
	request_irq(
		INT_PWC_ADINT,
		mp200_ad_interrupt,
		0,
		"mp200_ad",
		NULL
	);
	InitFlag=InitFlag|INIF_REQ_IRQ;

	/* set driver ready */
	DirverReady=1;

	return(0);
}

void mp200_ad_exit(void)
{
	/* disable interrupt */
	if(InitFlag&INIF_REQ_IRQ)
		disable_irq(INT_PWC_ADINT);

	/* clear factor */
	if(InitFlag&INIF_REQ_IRQ)
		gpiocore_pwc_write(GPIOCORE_PWC_CLRFACT3,0x01,0x01);

	/* unregister interrupt handler */
	if(InitFlag&INIF_REQ_IRQ)
		free_irq(INT_PWC_ADINT,NULL);

	/* if(InitFlag&INIF_ENB_REG) */
		/* pwc_regulator_off(PWC_PSCNTHP); */

#ifdef CONFIG_DEVFS_FS
	/* delete devfs */
	if(InitFlag&INIF_MAK_DFS)
		devfs_remove("mp201_ad");
#endif

	/* unregister driver */
	if(InitFlag&INIF_REG_DRV)
		driver_unregister(&MP200_AD_Driver);

	/* unregister device */
	if(InitFlag&INIF_REG_DEV)
		platform_device_unregister(&MP200_AD_Device);

	/* del driver */
	if(InitFlag&INIF_ADD_CDV)
		cdev_del(&CDev);

	/* release major */
	if(InitFlag&INIF_REG_MJR)
		unregister_chrdev_region(MKDEV(MajorNumber,0),1);

	return;
}

static void mp200_ad_shutdown(struct device *dev)
{
	/* int rv; */
	/* int i; */

	/* *(volatile unsigned long *)0xEC050200=0x00000008; */
	/* *(volatile unsigned long *)0xEC050208=0x00080000; */

	/* rv=pwc_regulator_off(PWC_PSCNTHP); */
	/* if(rv<0){ */
		/* printk(KERN_ERR "mp200_ad: pwc_regulator_off error %d\n",rv); */
		/* return; */
	/* } */

	/* for(i=0;i<500;i++) */
		/* udelay(1000); */

	/* gpiocore_pwc_write(GPIOCORE_PWC_CLRFACT1,0x03,0x03); */

	return;
}

static int mp200_ad_suspend(struct device *dev, u32 state, u32 level)
{
	/* int rv; */
	/* int i; */

	if(level==SUSPEND_POWER_DOWN){

		/* *(volatile unsigned long *)0xEC050200=0x00000008; */
		/* *(volatile unsigned long *)0xEC050208=0x00080000; */

		/* rv=pwc_regulator_off(PWC_PSCNTHP); */
		/* if(rv<0){ */
			/* printk(KERN_ERR "mp200_ad: pwc_regulator_off error %d\n",rv); */
			/* return(rv); */
		/* } */

		/* for(i=0;i<200;i++) */
			/* udelay(1000); */
	}

	return(0);
}

static int mp200_ad_resume(struct device *dev, u32 level)
{
	/* int rv; */

	/* if(level==RESUME_POWER_ON){ */
		/* rv=pwc_regulator_on(PWC_PSCNTHP); */
		/* if(rv<0){ */
			/* printk(KERN_ERR "mp200_ad: pwc_regulator_on error %d\n",rv); */
			/* return(rv); */
		/* } */
	/* } */

	return(0);
}

static int mp200_ad_open(struct inode *inode, struct file *file)
{
	/* read only check */
	if((file->f_flags&O_ACCMODE) != O_RDONLY){
		printk(KERN_WARNING "mp200_ad: read only\n");
		return(-EINVAL);
	}

	return(0);
}

static int mp200_ad_release(struct inode *inode, struct file *file)
{
	return(0);
}

static int mp200_ad_ioctl(
	struct inode *inode,
	struct file *file,
	unsigned int cmd,
	unsigned long arg
)
{
	int rv;
	unsigned int ui;
	unsigned long ul;

	/* check parameter */
	if(_IOC_TYPE(cmd) != MP200_AD_IOC_MAGIC){
		printk(KERN_WARNING "mp200_ad: illegal request\n");
		return(-ENOTTY);
	}

	/* check parameter */
	if(_IOC_NR(cmd) > MP200_AD_IOC_MAXNR){
		printk(KERN_WARNING "mp200_ad: illegal request\n");
		return(-ENOTTY);
	}

	/* check parameter */
	switch(cmd) {
		case MP200_AD_GETBAT:
		case MP200_AD_GETADP:
		case MP200_AD_GETAD3:
		case MP200_AD_GETAD4:
		case MP200_AD_GETAD5:
		case MP200_AD_GETAD6:
		case MP200_AD_GETAD7:
		case MP200_AD_GETAD8:
			break;
		default:
			printk(KERN_WARNING "mp200_ad: illegal request\n");
			return(-ENOTTY);
	}

	/* get sem for device occupation */
	if(file->f_flags & O_NONBLOCK){
		if(down_trylock(&Semaphore))
			return(-EAGAIN);
	}
	else{
		if(down_interruptible(&Semaphore))
			return(-ERESTARTSYS);
	}

	/* clear data status */
	DataStatus=AD_DATA_NONE;

	/* clear interrupt fuctor */
	gpiocore_pwc_read(GPIOCORE_PWC_FACTOR3,&ui);

	/* exclusive control for interrupt */
	spin_lock_irqsave(&Spinlock,ul);

	/* save request channel */
	switch(cmd){
		case MP200_AD_GETBAT:ReqChannel=0;break;
		case MP200_AD_GETADP:ReqChannel=1;break;
		case MP200_AD_GETAD3:ReqChannel=2;break;
		case MP200_AD_GETAD4:ReqChannel=3;break;
		case MP200_AD_GETAD5:ReqChannel=4;break;
		case MP200_AD_GETAD6:ReqChannel=5;break;
		case MP200_AD_GETAD7:ReqChannel=6;break;
		case MP200_AD_GETAD8:ReqChannel=7;break;
	}

	/* request A/D */
	gpiocore_pwc_write(GPIOCORE_PWC_ADMODE,0x10|(ReqChannel&0x7),0xff);

	/* exclusive control for interrupt */
	spin_unlock_irqrestore(&Spinlock,ul);

	/* wait for ad done */
	if(wait_event_interruptible(Waitque,(DataStatus!=AD_DATA_NONE))){
		up(&Semaphore);
		return(-ERESTARTSYS);
	}

	/* check data status */
	if(DataStatus!=AD_DATA_GOOD){
		up(&Semaphore);
		printk(KERN_ERR "mp200_ad: ad error\n");
		return(-EIO);
	}

	/* copy result to user */
	rv=copy_to_user(
		(void *)arg,
		(void *)&DataValue,
		_IOC_SIZE(cmd)
	);
	if(rv!=0){
		printk(KERN_ERR "mp200_ad: copy_to_user error %d\n",rv);
		up(&Semaphore);
		return(rv);
	}

	/* free sem for device occupation */
	up(&Semaphore);

	return(0);
}

static irqreturn_t mp200_ad_interrupt(int irq, void *dev_id, struct pt_regs *fp)
{
	int rv;
	unsigned int ui;
	unsigned long ul;

	/* clear factor */
	/* already clear in pwc_irq_demux() */
	/* gpiocore_pwc_write(GPIOCORE_PWC_CLRFACT3,0x01,0x01); */

	/* exclusive control for interrupt */
	spin_lock_irqsave(&Spinlock,ul);

	if(ReqChannel==-1){
		spin_unlock_irqrestore(&Spinlock,ul);
		return(IRQ_HANDLED);
	}

	/* read ad data */
	rv=gpiocore_pwc_read(GPIOCORE_PWC_ADDATA1+(ReqChannel*2),&ui);

	/* invalidate channel */
	ReqChannel=-1;

	/* exclusive control for interrupt */
	spin_unlock_irqrestore(&Spinlock,ul);

	if(rv<0){
		DataStatus=AD_DATA_FAIL;
		DataValue=0;
	}
	else{
		DataStatus=AD_DATA_GOOD;
		DataValue=(int)(ui&0xFF);
	}

	/* wakeup user context */
	wake_up_interruptible(&Waitque);

	return(IRQ_HANDLED);
}

int mp200_ad_get_val(int ch)
{
	int count;
	int val;
	int rv;
	unsigned int mask1;
	unsigned int mask3;
	unsigned long ul;

	if(ch<0 || ch>7)
		return(-1);

	if(DirverReady==0)
		return(-1);

	/* exclusive control for interrupt */
	/* spin_lock_irqsave(&Spinlock,ul); */
	/* provisional */
	spin_lock_bh(&Spinlock);
	spin_lock_irqsave(&Spinlock,ul);
	gpiocore_pwc_read(GPIOCORE_PWC_BMASK1,&mask1);
	gpiocore_pwc_write(GPIOCORE_PWC_BMASK1,0x00,0xF3);
	gpiocore_pwc_read(GPIOCORE_PWC_BMASK3,&mask3);
	gpiocore_pwc_write(GPIOCORE_PWC_BMASK3,0x00,0xD1);
	udelay(65);
	spin_unlock_irqrestore(&Spinlock,ul);

	/* disable interrupt */
	gpiocore_pwc_write(GPIOCORE_PWC_BMASK3,0x00,0x01);
	udelay(65);

	if(ReqChannel!=-1){
		/* stop request A/D */
		gpiocore_pwc_write(GPIOCORE_PWC_ADMODE,0x00|(ReqChannel&0x7),0xff);
	}

	/* clear factor */
	gpiocore_pwc_write(GPIOCORE_PWC_CLRFACT3,0x01,0x01);

	/* reset mask for factor bit */
	gpiocore_pwc_write(GPIOCORE_PWC_FMASK3,0x01,0x1);

	/* request A/D */
	gpiocore_pwc_write(GPIOCORE_PWC_ADMODE,0x10|(ch&0x7),0xff);

	/* polling A/D done */
	udelay(800);
	count=AD_TIMEOUT;
	while(count){
		rv=gpiocore_pwc_read(GPIOCORE_PWC_FACTOR3,(unsigned int *)&val);
		if(rv==0 && (val&0x01))
			break;
		count--;
	}

	val=-1;
	if(count!=0){
		/* read ad data */
		rv=gpiocore_pwc_read(GPIOCORE_PWC_ADDATA1+(ch*2),(unsigned int *)&val);
		if(rv>=0)
			val=val&0xFF;
	}

	/* stop request A/D */
	gpiocore_pwc_write(GPIOCORE_PWC_ADMODE,0x00|(ch&0x7),0xff);

	/* clear factor */
	gpiocore_pwc_write(GPIOCORE_PWC_CLRFACT3,0x01,0x01);

	/* enable interrupt */
	gpiocore_pwc_write(GPIOCORE_PWC_BMASK3,0x01,0x1);
	udelay(65);

	if(ReqChannel!=-1){
		/* request user A/D */
		gpiocore_pwc_write(GPIOCORE_PWC_ADMODE,0x10|(ReqChannel&0x7),0xff);
	}

	/* exclusive control for interrupt */
	/* spin_unlock_irqrestore(&Spinlock,ul); */
	/* provisional */
	spin_lock_irqsave(&Spinlock,ul);
	gpiocore_pwc_write(GPIOCORE_PWC_BMASK1,mask1,0xF3);
	gpiocore_pwc_write(GPIOCORE_PWC_BMASK3,mask3,0xD1);
	udelay(65);
	spin_unlock_irqrestore(&Spinlock,ul);
	spin_unlock_bh(&Spinlock);

	if(val==-1)
		printk(KERN_ERR "mp200_ad: A/D(ch:%d) timeout\n",ch);

	return(val);
}

EXPORT_SYMBOL(mp200_ad_get_val);
arch_initcall(mp200_ad_init);
module_exit(mp200_ad_exit);
MODULE_AUTHOR("Sony Corporation");
MODULE_DESCRIPTION("Sony ICX1087 A/D device driver");
MODULE_LICENSE("GPL");
