/*
 *  File Name       : mp200_ntsc.h
 *  Function        : NTSC I/F Driver
 *  Release Version : Ver 1.00
 *  Release Date    : 2005/07/15
 *
 *  Copyright (C) NEC Electronics Corporation 2005
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#ifndef MP200_NTSC_H
#define MP200_NTSC_H

#define MP200_REG(ADDR,OFF)	\
	(*(unsigned int volatile *)((unsigned char *)(ADDR) + OFF))

/* SMU_IOCTRL Masks */
#define SMU_IOCTRL_MASK		    0x000007f
#define SMU_IOCTRL_CAM_NTSCH	    0x0000040
#define SMU_IOCTRL_CHG_GIOL_CAMSTRH 0x0000001

/*** NTSC Registers ***/

#define NTSC_ADDR	IO_ADDRESS (MP200_NTSC_BASE)
#define NTSC_IOSIZE	(0x7c)

#define REG_NTS_CONTROL		MP200_REG(NTSC_ADDR,0x00)
#define REG_NTS_OUT		MP200_REG(NTSC_ADDR,0x04)
#define REG_NTS_STATUS		MP200_REG(NTSC_ADDR,0x08)
#define REG_NTS_YAREAAD_A	MP200_REG(NTSC_ADDR,0x0c)
#define REG_NTS_YAREAAD_B	MP200_REG(NTSC_ADDR,0x10)
#define REG_NTS_YAREAAD_C	MP200_REG(NTSC_ADDR,0x14)
#define REG_NTS_UVAREAAD_A	MP200_REG(NTSC_ADDR,0x18)
#define REG_NTS_UVAREAAD_B	MP200_REG(NTSC_ADDR,0x1c)
#define REG_NTS_UVAREAAD_C	MP200_REG(NTSC_ADDR,0x20)
#define REG_NTS_HOFFSET		MP200_REG(NTSC_ADDR,0x24)
#define REG_NTS_FRAMESEL	MP200_REG(NTSC_ADDR,0x28)
#define REG_NTS_INTSTATUS	MP200_REG(NTSC_ADDR,0x60)
#define REG_NTS_INTRAWSTATUS	MP200_REG(NTSC_ADDR,0x64)
#define REG_NTS_INTENSET	MP200_REG(NTSC_ADDR,0x68)
#define REG_NTS_INTENCLR	MP200_REG(NTSC_ADDR,0x6c)
#define REG_NTS_INTFFCLR	MP200_REG(NTSC_ADDR,0x70)
#define REG_NTS_ERRORADR	MP200_REG(NTSC_ADDR,0x74)
#define REG_NTS_SWRESET		MP200_REG(NTSC_ADDR,0x78)

#define NTS_CONTROL_MASK    0x000000f
#define NTS_CONTROL_UPSCALE 0x0000008
#define NTS_CONTROL_OUTMODE 0x0000004
#define NTS_CONTROL_CLKPOL  0x0000002
#define NTS_CONTROL_ENDIAN  0x0000001

/* NTS_OUT Masks */
#define NTS_OUT_MASK    0x0000003
#define NTS_OUT_NTSOUT	NTS_OUT_MASK

#define NTS_OUT_NTSOUT_OFF	0x00000000
#define NTS_OUT_NTSOUT_BLACK	0x00000001
#define NTS_OUT_NTSOUT_BLUE	0x00000002
#define NTS_OUT_NTSOUT_ON	0x00000003

/* NTS_STATUS Masks */
#define NTS_STATUS_MASK    0x0000003
#define NTS_STATUS	NTS_STATUS_MASK

#define NTS_STATUS_OFF		0x00000000
#define NTS_STATUS_BLACK	0x00000001
#define NTS_STATUS_BLUE		0x00000002
#define NTS_STATUS_ON		0x00000003

/* NTS_YAREAAD_x Masks */
#define NTS_YAREAAD_MASK     0xfffffffc

/* NTS_UVAREAAD_x Masks */
#define NTS_UVAREAAD_MASK    0xfffffffc

/* NTS_HOFFSET Masks */
#define NTS_HOFFSET_MASK     0x00001ffc

/* NTS_FRAMESEL Masks */
#define NTS_FRAMESEL_MASK	0x0000000f
#define NTS_FRAMESEL_AREASTATUS	0x0000000c

#define NTS_AREASTATUS_A	0x00000004
#define NTS_AREASTATUS_B	0x00000008
#define NTS_AREASTATUS_C	0x0000000c

#define NTS_FRAMESEL_AREASEL	0x00000003

#define NTS_FRAMESEL_A	0x00000001
#define NTS_FRAMESEL_B	0x00000002
#define NTS_FRAMESEL_C	0x00000003

/* NTS_INTSTATUS
 * NTS_INTRAWSTATUS
 * NTS_INTENSET Masks
 * NTS_INTENCLR Masks
 * NTS_INTFFCLR Masks
 */
#define NTS_INT_MASK		0x0000000f

#define NTS_INT_UNDERRUN	0x00000008
#define NTS_INT_DMASTOP		0x00000004
#define NTS_INT_DMAERR		0x00000002
#define NTS_INT_NTSVS		0x00000001

/* NTS_ERRORADR Masks */
#define NTS_ERRORADR_MASK	0xfffffffd
#define NTS_ERRORADR		0xfffffffc
#define NTS_ERRORADR_LOCK	0x00000001

/* NTS_SWRESET Masks */
#define NTS_SWRESET_MASK	0x00000001
#define NTS_SWRESET		NTS_SWRESET_MASK

enum {
	NTSC_BUF_UNUSED,	/* Unused */
	NTSC_BUF_READY,		/* Ready to output into */
	NTSC_BUF_OUTPUT,	/* In the process of being output into */
	NTSC_BUF_DONE,		/* Finished output, but not been synced yet */
};

typedef struct encoder_reg {
	char name[32];
	int (*hw_init)(void *);
	void (*hw_shutdown)(void *);
	int (*hw_command)(unsigned int cmd, void* arg);
	void* private_data;
} ENCODER_REG;

/* Maximum number of buffers */
#define NTSC_MAX_BUFNBRS	(0x1 << 5)	/* 32 power-of-two */

/* queues containing the buffer indices */
#define NTSC_QUEUE_SIZE	NTSC_MAX_BUFNBRS

typedef struct mp200_ntsc_queue {
	unsigned int head;		/* queue head */
	unsigned int tail;		/* queue tail */
	unsigned int len;		/* queue length */
	spinlock_t s_lock;		/* spinlock protecting the queue */
	int buf[NTSC_QUEUE_SIZE];	/* queue contents */
} MP200_NTSC_QUEUE;

typedef struct mp200_ntsc_frame {
	char *vadr;		/* points to actual buffer */
	unsigned long padr;	/* physical buffer address */
	volatile int state;
	wait_queue_head_t proc_list;	/* wait queue */
	int redisplay;
} MP200_NTSC_FRAME;

#define NTSC_FRAME_MAX		(3)
#define NTSC_FRAME_EMPTY	(0)

typedef struct ntsc_data {
	int open_count;		/* of process that have ntsc open */

	struct semaphore busy_lock;
				/* guard against SMP multithreading */

        struct mp200_ntsc_frame* frame;

	int outputframe;
	int selectframe;
	int frame_index;		/* the current frame to output into */
	int timeout;		/* time out frame count */

	unsigned int volatile ntscstatus;	/* ntsc irq states */

	struct mp200_ntsc_queue outputq;
				/* queue for buffers to be output */

	int norm;		/* Returned: VIDEO_MODE_PAL or VIDEO_MODE_NTSC */
	ENCODER_REG*	encoder;
	int width;
	int height;
	int depthbyte;

	int nbfrags;
	int clkpol;
	int endian;
	unsigned long fragsize;

	int background:1;	/* 0: brackback 1: buleback */
	int upscale:1;		/* 0: normal 1: upscale */
	int mapped:1;		/* mmap()'ed */
	int endless:1;
	int active:1;		/* actually in progressi */
	int videoout:1;
} NTSC_DATA;

struct ntsc_data *mp200_ntsc_register(ENCODER_REG* encoder);
void mp200_ntsc_unregister(struct ntsc_data *ntsc);

#endif /* MP200_NTSC_H */
