/*
 * mp200_pm_ext.c -- MP200 PM extention interface.
 *
 * Copyright 2006 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


#include <linux/init.h>
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/moduleparam.h>
#include <linux/init.h>

#include <linux/kernel.h>       /* printk() */
#include <linux/errno.h>        /* error codes */
#include <linux/types.h>        /* size_t */
#include <linux/fcntl.h>        /* O_ACCMODE */
#include <linux/seq_file.h>
#include <linux/cdev.h>
#include <linux/device.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/poll.h>

#include <asm/arch/mp200_pm_ext.h>


/**
 *
 */
struct mp200_pm_ext_dev {
    int    users;
    struct semaphore sem;
    struct cdev cdev;
    wait_queue_head_t wqh;
    spinlock_t        lock;
    int    nStatus;
    int    wake_up;
};

int mp200_pm_ext_init(void);
void mp200_pm_ext_exit(void);
int mp200_pm_ext_open(struct inode *inode, struct file *filp);
int mp200_pm_ext_release(struct inode *inode, struct file *filp);
int mp200_pm_ext_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg);
unsigned int mp200_pm_ext_poll(struct file *file, poll_table *wait);

int mp200_pm_ext_major = 0;

struct mp200_pm_ext_dev _mp200_pm_ext_devices;

struct file_operations mp200_pm_ext_fops = {
    .owner =    THIS_MODULE,
    .ioctl =    mp200_pm_ext_ioctl,
    .poll  =    mp200_pm_ext_poll,
    .open =     mp200_pm_ext_open,
    .release =  mp200_pm_ext_release,
};


static struct class_simple *pm_class;
static char  *dev_file  = "pm";



/**
 *
 */
int mp200_pm_ext_init(void)
{
    dev_t dev=0;
    int rv;
    
    printk(KERN_INFO "Starting mp200_pm_ext. \n");

    rv = alloc_chrdev_region(&dev,
                             0,
                             1,
                             "mp200_pm_ext");
    
    mp200_pm_ext_major = MAJOR(dev);


    if (rv<0) {
        printk(KERN_ERR "%s()  can't get major. %d\n",
               __FUNCTION__, mp200_pm_ext_major);
        return(rv);
    } else {

#if 0        
        printk(KERN_INFO "%s()  major=%d \n",
               __FUNCTION__,
               mp200_pm_ext_major);
#endif        
    }
    

    /**********************/
    /* Device Info Struct */
    /**********************/

    /* clear */
    memset(&_mp200_pm_ext_devices,
           0,
           sizeof(struct mp200_pm_ext_dev));

    /* initialize */
    _mp200_pm_ext_devices.users = 0;

    init_MUTEX(&(_mp200_pm_ext_devices.sem));
    spin_lock_init(&(_mp200_pm_ext_devices.lock));
    init_waitqueue_head(&(_mp200_pm_ext_devices.wqh));

    _mp200_pm_ext_devices.wake_up=0;

    cdev_init(&_mp200_pm_ext_devices.cdev, &mp200_pm_ext_fops);

    _mp200_pm_ext_devices.cdev.owner=THIS_MODULE;
    _mp200_pm_ext_devices.cdev.ops=&mp200_pm_ext_fops;
    
    rv = cdev_add(&_mp200_pm_ext_devices.cdev,
                  MKDEV(mp200_pm_ext_major, 0),
                  1);


	pm_class = class_simple_create(THIS_MODULE, dev_file);

	class_simple_device_add(pm_class, dev, NULL, dev_file);
	devfs_mk_cdev(dev, S_IFCHR | S_IRUGO | S_IWUGO, dev_file);

    
    if (rv) {
        printk(KERN_ERR "%s() error %d adding mp200_pm_ext\n",
               __FUNCTION__, rv);
    } else {
#if 0        
        printk(KERN_INFO "%s() normal end \n", __FUNCTION__);
#endif        
    }
    
    return (0);
}



/**
 *
 */
void mp200_pm_ext_exit(void)
{
    dev_t dev_no = MKDEV(mp200_pm_ext_major, 0);

    printk(KERN_ALERT "mp200_pm_ext: exit\n");

	devfs_remove(dev_file);
	class_simple_device_remove(dev_no);
	class_simple_destroy(pm_class);
    cdev_del(&_mp200_pm_ext_devices.cdev);
    unregister_chrdev_region(dev_no, 1);

    return;
}


/*
 *
 */
int mp200_pm_ext_open(struct inode *inode, struct file *filp)
{
    struct mp200_pm_ext_dev *dev;

#if 0    
    printk(KERN_INFO "%s() \n", __FUNCTION__);
#endif    

    dev = container_of(inode->i_cdev,struct mp200_pm_ext_dev,cdev);
    filp->private_data=dev;

    if(down_interruptible(&dev->sem)) {
        return(-ERESTARTSYS);
    }
    
    dev->users += 1;

#if 0    
    printk(KERN_INFO "%s() users=%d\n", __FUNCTION__, dev->users);
#endif
    
    up(&dev->sem);

    if ((filp->f_flags&O_ACCMODE) == O_WRONLY) {
        printk(KERN_ALERT "%s() write only\n", __FUNCTION__);
    }

    return(0);
}



/**
 *
 */
int mp200_pm_ext_release(struct inode *inode, struct file *filp)
{
    struct mp200_pm_ext_dev *dev;

#if 0    
    printk(KERN_INFO "%s() \n", __FUNCTION__);
#endif    

    /* dev = container_of(inode->i_cdev,struct mp200_pm_ext_dev,cdev); */
    /* filp->private_data=dev; */

    dev = filp->private_data;

    if (down_interruptible(&dev->sem)) {
        return(-ERESTARTSYS);
    }
    
    dev->users -= 1;
    
#if 0
    printk(KERN_INFO "%s() users=%d\n", __FUNCTION__, dev->users);
#endif    
    
    up(&dev->sem);

    return(0);
}


/**
 *
 */
int mp200_pm_ext_ioctl(struct inode *inode,
                       struct file *filp,
                       unsigned int cmd,
                       unsigned long arg)
{
    struct mp200_pm_ext_dev *dev = (struct mp200_pm_ext_dev *)filp->private_data;
    unsigned int *sts;
    int ret = 0;


	/* check parameter */
	if(_IOC_TYPE(cmd) != MP200_PM_EXT_IOC_MAGIC){
		printk(KERN_WARNING "%s() : illegal request\n", __FUNCTION__);
		return(-ENOTTY);
	}

    switch (cmd) {

    case MP200_PM_EXT_GET_FACTOR:
        sts = (unsigned int *)arg;
        *sts = dev->nStatus;
        printk(KERN_INFO "%s() factor=0x%04x \n", __FUNCTION__, dev->nStatus);
        break;

    default:
        printk(KERN_ERR "%s() OTHER:=0x%08x \n", __FUNCTION__, cmd);
        ret = -1;
        break;
    }
    
    return(ret);
}



/**
 *
 */
unsigned int mp200_pm_ext_poll(struct file *file, poll_table * wait)
{
    struct mp200_pm_ext_dev *dev = (struct mp200_pm_ext_dev *)file->private_data;
    int rv = 0;
    unsigned long flags;

    poll_wait(file,&dev->wqh,wait);

    spin_lock_irqsave(&dev->lock, flags);

    if(dev->wake_up != 0) {
        dev->wake_up = 0;
        rv=rv|POLLIN|POLLRDNORM;
    }
    
    spin_unlock_irqrestore(&dev->lock, flags);

    
#if 0    
    printk(KERN_INFO "%s() return(%d)\n", __FUNCTION__, rv);
#endif    
    return(rv);
    
}



/*
 *
 */
int mp200_pm_ext_notify(int nStatus)
{
    struct mp200_pm_ext_dev *dev = &_mp200_pm_ext_devices;
    unsigned long flags;

    dev->nStatus = nStatus;
    
    printk(KERN_INFO "%s() nStatus=0x%04x \n", __FUNCTION__, nStatus);
    
    spin_lock_irqsave(&dev->lock, flags);

    if (dev->wake_up == 0) {
        printk(KERN_INFO "%s() wake_up=0 \n", __FUNCTION__);
        dev->wake_up = 1;
        wake_up_interruptible(&dev->wqh);
    }
    spin_unlock_irqrestore(&dev->lock, flags);
    return(0);
}


MODULE_LICENSE("GPL");
EXPORT_SYMBOL(mp200_pm_ext_notify);

module_init(mp200_pm_ext_init);
module_exit(mp200_pm_ext_exit);

