/*
 *  File Name	    : drivers/char/mp200_pwm.c
 *  Function	    : Pulse Width Moduration(pwm).
 *  Release Version : Ver 1.00
 *  Release Date    : 2006/05/10
 *
 *  Copyright (C) NEC Electronics Corporation 2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

/*
 * Copyright 2006,2007 Sony Corporation. 
 */


#include <linux/version.h>
#include <linux/module.h>
#include <linux/init.h>
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
#include <linux/interrupt.h>
#endif
#include <linux/device.h>
#include <linux/spinlock.h>

#include <asm/io.h>
#include <asm/irq.h>
#include <asm/arch/pmu.h>
#include <asm/arch/smu.h>
#include <asm/arch/irqs.h>
#include <asm/arch/pwm.h>

#include "mp200_pwm.h"

#if	defined(MP200_PWM_DEBUG)
#define FNC_ENTRY \
	if (debug == 1 || debug >= 9) {	\
		printk(KERN_DEBUG "entry:%s\n", __FUNCTION__); \
	}
#define FNC_EXIT \
	if (debug == 1 || debug >= 9) {	\
		printk(KERN_DEBUG "exit:%s:%d\n", __FUNCTION__ , __LINE__); \
	}
#define d0b(fmt, args...)	\
		printk(KERN_DEBUG "%s:%d: " fmt , __FUNCTION__ , __LINE__ , \
		       ## args);
static int debug = 1;
#else
#define FNC_ENTRY
#define FNC_EXIT
#define d0b(fmt, args...)
#endif

static spinlock_t spin_lock = SPIN_LOCK_UNLOCKED;
static int pwm_init_flag = 1;

/* ------------------------------------------------------------------
   register address, state, callback default define
   ------------------------------------------------------------------ */
static mp200_pwm_cb_info_t g_pwm_cb_info[PWM_MAX_CHANNEL_NUM][PWM_MAX_COUNTER_NUM];

static pwm_info_t g_pwm_info[PWM_MAX_CHANNEL_NUM] = {
	{
		PWM_STATE_STOP,
		PWM_CH0_CTRL,
		PWM_CH0_MODE,
	},
	{
		PWM_STATE_STOP,
		PWM_CH1_CTRL,
		PWM_CH1_MODE,
	},
};


static mp200_pwm_cmpcnt_t g_pwm_cmpcnt[PWM_MAX_CHANNEL_NUM][PWM_MAX_COUNTER_NUM] = {
	{
		{
			MP200_PWM_COMPARE0,
			PWM_CH0_DELAY0,
			PWM_CH0_LEDGE0,
			PWM_CH0_TEDGE0,
			PWM_CH0_TOTAL0,
			PWM_CH0_LOOP0,
			NULL,
		},
		{
			MP200_PWM_COMPARE1,
			PWM_CH0_DELAY1,
			PWM_CH0_LEDGE1,
			PWM_CH0_TEDGE1,
			PWM_CH0_TOTAL1,
			PWM_CH0_LOOP1,
			NULL,
		},
		{
			MP200_PWM_COMPARE2,
			PWM_CH0_DELAY2,
			PWM_CH0_LEDGE2,
			PWM_CH0_TEDGE2,
			PWM_CH0_TOTAL2,
			PWM_CH0_LOOP2,
			NULL,
		},
	},
	{
		{
			MP200_PWM_COMPARE0,
			PWM_CH1_DELAY0,
			PWM_CH1_LEDGE0,
			PWM_CH1_TEDGE0,
			PWM_CH1_TOTAL0,
			PWM_CH1_LOOP0,
			NULL,
		},
		{
			MP200_PWM_COMPARE1,
			PWM_CH1_DELAY1,
			PWM_CH1_LEDGE1,
			PWM_CH1_TEDGE1,
			PWM_CH1_TOTAL1,
			PWM_CH1_LOOP1,
			NULL,
		},
		{
			MP200_PWM_COMPARE2,
			PWM_CH1_DELAY2,
			PWM_CH1_LEDGE2,
			PWM_CH1_TEDGE2,
			PWM_CH1_TOTAL2,
			PWM_CH1_LOOP2,
			NULL,
		},
	},
};

/* ------------------------------------------------------------------
   smu control.
   ------------------------------------------------------------------ */
static inline void pwm_sclock_start(void)
{
        mp200_pmu_open_clockgate(MP200_CLOCKGATE_PWM_SCLK);
}

static inline void pwm_sclock_stop(void)
{
        mp200_pmu_close_clockgate(MP200_CLOCKGATE_PWM_SCLK);
}

static inline void pwm_clock_start(void)
{
        mp200_pmu_open_clockgate(MP200_CLOCKGATE_PWM_CLK);
        mp200_pmu_open_clockgate(MP200_CLOCKGATE_PWM_SCLK);
}

static inline void pwm_clock_stop(void)
{
        mp200_pmu_close_clockgate(MP200_CLOCKGATE_PWM_CLK);
        mp200_pmu_close_clockgate(MP200_CLOCKGATE_PWM_SCLK);
}

static inline void pwm_dev_unreset(void)
{
        mp200_pmu_unreset_device(MP200_RESETDEVICE_PWM);
}

/* ------------------------------------------------------------------
   channel number check.
   ------------------------------------------------------------------ */
static inline int
check_validity_channel(unsigned char channel)
{
	FNC_ENTRY
	if( (channel == MP200_PWM_CH0) || (channel == MP200_PWM_CH1) ) {
		FNC_EXIT return( 0 );
	}
	FNC_EXIT return( 1 );
}


/* ------------------------------------------------------------------
   compare_counter number check.
   ------------------------------------------------------------------ */
static inline int
check_validity_compare_counter(unsigned char cmp_counter)
{
	FNC_ENTRY
	if( cmp_counter > MP200_PWM_COMPARE2 ) {
		FNC_EXIT return( 1 );
	}
	FNC_EXIT return( 0 );
}

/* --------------------------------------------------------------
   function : pwm_irq_handerl
   argument :
   return   : none
   comment  : dev_id ... is compare_registers address.
   -------------------------------------------------------------- */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
static irqreturn_t
#else
static void
#endif
pwm_irq_handler(int irq, void *dev_id, struct pt_regs *intregs)
{
	mp200_pwm_cb_info_t *info;
	unsigned int intsts, rawsts, sts, raw;
	unsigned int shift;
	unsigned int ch, cmp;
	FNC_ENTRY

	/* save interrupt status */
	intsts = inl(PWM_INTSTATUS);
	rawsts = inl(PWM_INTRAWSTATUS);
	/* clear interrupt status */
	outl(intsts, PWM_INTFFCLR);
	
	for(ch = 0; ch < PWM_MAX_CHANNEL_NUM; ch++) {
		if((intsts & (PWM_MASK_ENDSET_SETBIT << (ch << 3))) == 0) {
			continue;
		}
		for(cmp = 0; cmp < PWM_MAX_COUNTER_NUM; cmp++) {
			if((intsts & (0x3 << ((cmp << 1) + (ch << 3)))) == 0) {
				continue;
			}
			info = g_pwm_cmpcnt[ch][cmp].cb_info;
			if(NULL == info) {
				continue;
			}
			
			shift = ((cmp << 1) + (ch << 3));
			sts = (intsts >> shift) & 0x3;
			raw = (rawsts >> shift) & 0x3;
			/* check interrupt */
			if(sts & 0x1) {
				if(info->cb_count) {
					info->cb_count(
						info->cb_count_data, sts, raw);
				}
			}
			if(sts & 0x2) {
				if(info->cb_loop) {
					info->cb_loop(
						info->cb_loop_data, sts, raw);
				}
			}
		}
	}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
	FNC_EXIT return IRQ_HANDLED;
#else
	FNC_EXIT return;
#endif
}

/* --------------------------------------------------------------
   function : mp200_pwm_start
   argument : channel  . channel number
   -------------------------------------------------------------- */
int mp200_pwm_start( unsigned char channel )
{
	pwm_info_t *pwm_info_p;
	int err;
	unsigned int reg_value;
	unsigned long lock_flags;
	FNC_ENTRY

	if (pwm_init_flag) {
#ifdef PWM_DEBUG
		printk(KERN_INFO "%s : pwm initialize error\n", __FUNCTION__);
#endif
		FNC_EXIT return -EPERM;
	}

	/* check parameter */
	err = check_validity_channel(channel);
	if( err != 0 ) {
		FNC_EXIT return( -ENODEV );
	}
#ifdef PWM_DEBUG
	printk(KERN_INFO "%s : channle ok (ch%d)\n", __FUNCTION__, channel );
#endif

	pwm_info_p = &g_pwm_info[channel];

	spin_lock_irqsave(&spin_lock, lock_flags);

	/* check state */
	if( PWM_STATE_STOP != pwm_info_p->state ) {
		spin_unlock_irqrestore(&spin_lock, lock_flags);
		FNC_EXIT return( -EBUSY );
	}
#ifdef PWM_DEBUG
	printk(KERN_INFO "%s : state ok (0x%08X)\n", __FUNCTION__, pwm_info_p->state );
#endif

	/* regist irq_handler */
	/* 5,dev_id  field : use for regs */

	/* read use ch(read mode reg) */
	reg_value = inl( pwm_info_p->channel_mode );

	if( !(reg_value & (PWM_CMP_EN0_BIT | PWM_CMP_EN1_BIT | PWM_CMP_EN2_BIT)) ) {
		/* counter all not use */
#ifdef PWM_DEBUG
		printk(KERN_INFO "%s : compare counter no setting (reg=0x%08X)\n", __FUNCTION__, reg_value );
#endif
		spin_unlock_irqrestore(&spin_lock, lock_flags);
		FNC_EXIT return( -EINVAL );
	}

	if( PWM_STATE_STOP == g_pwm_info[(channel + 1) & 0x01].state ) {
		pwm_sclock_start();
		enable_irq( INT_PWM );
	}
	/* start countup */
	outl( PWM_START_BIT_ON, pwm_info_p->channel_control);

	/* change state */
	pwm_info_p->state = PWM_STATE_START;

	spin_unlock_irqrestore(&spin_lock, lock_flags);

	FNC_EXIT return( 0 );
}

/* --------------------------------------------------------------
   function : mp200_pwm_stop
   explain  : stop pwm
   argument : 	   : PWMư
	   : ch : ֹͥ
	 : BIOS_OK        : ｪλ
		 BIOS_ERR_PARAM : ѥ᡼顼
   -------------------------------------------------------------- */
int mp200_pwm_stop( unsigned char channel )
{
	int err;
	unsigned long lock_flags;
	FNC_ENTRY

	if (pwm_init_flag) {
#ifdef PWM_DEBUG
		printk(KERN_INFO "%s : pwm initialize error\n", __FUNCTION__);
#endif
		FNC_EXIT return -EPERM;
	}

	/* check parameter */
	err = check_validity_channel(channel);
	if( err != 0 ) {
		FNC_EXIT return( -ENODEV );
	}

	spin_lock_irqsave(&spin_lock, lock_flags);

	/* check state */
	if( PWM_STATE_STOP == g_pwm_info[channel].state ) {
		spin_unlock_irqrestore(&spin_lock, lock_flags);
		FNC_EXIT return( 0 );
	}

	/* stop countup */
	outl( PWM_START_BIT_OFF, g_pwm_info[channel].channel_control );

	/* change state */
	g_pwm_info[channel].state = PWM_STATE_STOP;
	if( PWM_STATE_STOP == g_pwm_info[(channel + 1) & 0x01].state ) {
		disable_irq( INT_PWM );
		pwm_sclock_stop();
	}
	
	spin_unlock_irqrestore(&spin_lock, lock_flags);

	FNC_EXIT return( 0 );
}

/* --------------------------------------------------------------
	̾   : pwmSetChConfig
	   : PWMͥư
	   : ch      : ֹͥ
		 pConfig : PWMͥ륳ե졼¤ΤؤΥݥ
	 : BIOS_OK         : ｪλ
		 BIOS_ERR_PARAM  : ѥ᡼顼
		 BIOS_ERR_CONFIG : ե졼Ƥ
		 BIOS_ERR_STATE  : ư
   -------------------------------------------------------------- */
int mp200_pwm_set_channel_config( unsigned char channel, mp200_pwm_ch_config_t *ch_config )
{
	int err;
	unsigned int mode_value;
	unsigned int set_value;
	unsigned int value;
	unsigned int reg_int_sft;
	pwm_info_t *pwm_info_p;
	unsigned long lock_flags;
	FNC_ENTRY

	if (pwm_init_flag) {
#ifdef PWM_DEBUG
		printk(KERN_INFO "%s : pwm initialize error\n", __FUNCTION__);
#endif
		FNC_EXIT return -EPERM;
	}

	/* check parameter */
	err = check_validity_channel(channel);
	if( err != 0 ) {
		FNC_EXIT return( -ENODEV );
	}
	if( NULL == ch_config ) {
		FNC_EXIT return( -EINVAL );
	}

	pwm_info_p = &g_pwm_info[channel];

	spin_lock_irqsave(&spin_lock, lock_flags);

	/* check state */
	if( PWM_STATE_STOP != pwm_info_p->state ) {
		spin_unlock_irqrestore(&spin_lock, lock_flags);
		FNC_EXIT return( -EBUSY );
	}

	/* check config argument value  */
	if( (~PWM_MASK_CODE_COUNTER & ch_config->use_cmp)
	      || (PWM_CODE_MODE_XOR < ch_config->mode)	/* max is MODE_XOR */
	      || (~PWM_MASK_CODE_INVERSE & ch_config->inverse)
	      || (~PWM_MASK_CODE_INTCOUNT & ch_config->interrupt_count)
	      || (~PWM_MASK_CODE_INTLOOP & ch_config->interrupt_loop) ) {
#ifdef PWM_DEBUG
		printk(KERN_INFO "%s : argument invalid.", __FUNCTION__ );
#endif
		spin_unlock_irqrestore(&spin_lock, lock_flags);
		FNC_EXIT return( -EINVAL );
	}

	/* set use counters */
	reg_int_sft = channel << 3;


	/* read mode reg */
	mode_value = inl( pwm_info_p->channel_mode ) & ~PWM_MASK_MODE_SETBIT;

	/* set mode */
	mode_value |= ch_config->mode << PWM_MODE_SFT;

	/* set use_cmp */
	value = ch_config->use_cmp;
	mode_value |= ((value & PWM_BIT_CMP2) << PWM_CMP_EN2_POS)
		| ((value & PWM_BIT_CMP1) << PWM_CMP_EN1_POS)
		| ((value & PWM_BIT_CMP0) << PWM_CMP_EN0_POS);

	/* set inverse */
	value = ch_config->inverse;
	mode_value |= ((value & PWM_BIT_CMP2) << PWM_CMP_INV2_POS)
		| ((value & PWM_BIT_CMP1) << PWM_CMP_INV1_POS)
		| ((value & PWM_BIT_CMP0) << PWM_CMP_INV0_POS);
	/* set mode reg */
	outl( mode_value, pwm_info_p->channel_mode );

	/* set interrupt count */
	value = ch_config->interrupt_count;
	set_value = ((value & PWM_BIT_CMP2) << PWM_CMP2_CENDSET_POS)
		| ((value & PWM_BIT_CMP1) << PWM_CMP1_CENDSET_POS)
		| ((value & PWM_BIT_CMP0) << PWM_CMP0_CENDSET_POS);

	/* interrupt_loop */
	value = ch_config->interrupt_loop;
	set_value |= ((value & PWM_BIT_CMP2) << PWM_CMP2_LENDSET_POS)
		| ((value & PWM_BIT_CMP1) << PWM_CMP1_LENDSET_POS)
		| ((value & PWM_BIT_CMP0) << PWM_CMP0_LENDSET_POS);
	
	/* set int disable reg */
	outl( (PWM_MASK_ENDSET_SETBIT << reg_int_sft), PWM_INTENCLR );

	/* set int enable reg */
	outl( (set_value << reg_int_sft), PWM_INTENSET );

	spin_unlock_irqrestore(&spin_lock, lock_flags);

	FNC_EXIT return( 0 );
}

/* --------------------------------------------------------------
   function : mp200_pwm_get_channel_config;
            : get channnel status
   argument : channel   ... ch number
              ch_config ... PWMͥ륳ե졼¤ΤؤΥݥ
   return   :
   -------------------------------------------------------------- */
int mp200_pwm_get_channel_config( unsigned char channel, mp200_pwm_ch_config_t *ch_config )
{
	int err;
	unsigned int reg_value;
	FNC_ENTRY

	if (pwm_init_flag) {
#ifdef PWM_DEBUG
		printk(KERN_INFO "%s : pwm initialize error\n", __FUNCTION__);
#endif
		FNC_EXIT return -EPERM;
	}

	/* check parameter */
	err = check_validity_channel(channel);
	if( err != 0 ) {
		FNC_EXIT return( -ENODEV );
	}
	if( (NULL == ch_config) ) {
		FNC_EXIT return( -EINVAL );
	}

	/* read mode reg */
	reg_value = inl( g_pwm_info[channel].channel_mode );

	/* mode */
	ch_config->mode = (reg_value & PWM_MASK_MODE) >> PWM_MODE_SFT;

	/* use_cmp */
	ch_config->use_cmp = ((reg_value & PWM_CMP_EN0_BIT) >> PWM_CMP_EN0_POS)
		| ((reg_value & PWM_CMP_EN1_BIT) >> PWM_CMP_EN1_POS)
		| ((reg_value & PWM_CMP_EN2_BIT) >> PWM_CMP_EN2_POS);

	/* inverse */
	ch_config->inverse = ((reg_value & PWM_CMP_INV0_BIT) >> PWM_CMP_INV0_POS)
		| ((reg_value & PWM_CMP_INV1_BIT) >> PWM_CMP_INV1_POS)
		| ((reg_value & PWM_CMP_INV2_BIT) >> PWM_CMP_INV2_POS);


	/* read int enable reg */

	reg_value = inl( PWM_INTENSET ) >> (channel << 3);

	/* int_count */
	ch_config->interrupt_count =
		((reg_value & PWM_CMP0_CENDSET_BIT) >> PWM_CMP0_CENDSET_POS)
		| ((reg_value & PWM_CMP1_CENDSET_BIT) >> PWM_CMP1_CENDSET_POS)
		| ((reg_value & PWM_CMP2_CENDSET_BIT) >> PWM_CMP2_CENDSET_POS);

	/* interrupt_loop */
	ch_config->interrupt_loop =
		((reg_value & PWM_CMP0_LENDSET_BIT) >> PWM_CMP0_LENDSET_POS)
		| ((reg_value & PWM_CMP1_LENDSET_BIT) >> PWM_CMP1_LENDSET_POS)
		| ((reg_value & PWM_CMP2_LENDSET_BIT) >> PWM_CMP2_LENDSET_POS);

	FNC_EXIT return( 0 );
}

/* --------------------------------------------------------------
   function : pwm_set_compare_config
              regist irq_handler, and compare counter parameters.
   argument : channel
              <pwm_cmpcnt_state> : setting parameters.
                 .cmp
                 .delay
                 .lead_edge
                 .trail_edge
                 .total_cycel
                 .loop_count
                 .cb_info
   return :
   -------------------------------------------------------------- */
int mp200_pwm_set_compare_counter( unsigned char channel, mp200_pwm_cmpcnt_t *src_cmpcnt )

{
	unsigned int value;
	int err;
	unsigned char cmp;
	mp200_pwm_cmpcnt_t *dst_cmpcnt;
	mp200_pwm_cb_info_t *src_cb_info;
	mp200_pwm_cb_info_t *dst_cb_info;
	pwm_info_t *pwm_info_p;
	unsigned long lock_flags;
	FNC_ENTRY

	if (pwm_init_flag) {
#ifdef PWM_DEBUG
		printk(KERN_INFO "%s : pwm initialize error\n", __FUNCTION__);
#endif
		FNC_EXIT return -EPERM;
	}

	err = check_validity_channel(channel);
	if( err != 0 ) {
		FNC_EXIT return( -ENODEV );
	}
	/* check parameter */
	if( NULL == src_cmpcnt ) {
		FNC_EXIT return( -EINVAL );
	}
	err = check_validity_compare_counter(src_cmpcnt->cmp);
	if( err != 0 ) {
		FNC_EXIT return( -ENODEV );
	}

	cmp = src_cmpcnt->cmp;
	pwm_info_p = &g_pwm_info[channel];

	spin_lock_irqsave(&spin_lock, lock_flags);

	/* check state */
	if( PWM_STATE_STOP != pwm_info_p->state ) {
		spin_unlock_irqrestore(&spin_lock, lock_flags);
		FNC_EXIT return( -EBUSY );
	}
	dst_cmpcnt = &g_pwm_cmpcnt[channel][cmp];

	/* check config (lead_edge < trail_edge <= total_cycle) */
	if( (src_cmpcnt->lead_edge >= src_cmpcnt->trail_edge) 
	    || (src_cmpcnt->trail_edge > src_cmpcnt->total_cycle) ) {
		spin_unlock_irqrestore(&spin_lock, lock_flags);
		FNC_EXIT return( -EINVAL );
	}

	/* read mode reg */
	value = inl( pwm_info_p->channel_mode );
	/* set reg */
	outl( src_cmpcnt->delay, dst_cmpcnt->delay );
	outl( src_cmpcnt->lead_edge, dst_cmpcnt->lead_edge );
	outl( src_cmpcnt->trail_edge, dst_cmpcnt->trail_edge );
	outl( src_cmpcnt->total_cycle, dst_cmpcnt->total_cycle );
	if( src_cmpcnt->loop_count ) {
		outl( src_cmpcnt->loop_count, dst_cmpcnt->loop_count );
		value |= PWM_CMP_ATST0_BIT << (cmp<<2); /* fixed loop */
	} else {
		value &= ~(PWM_CMP_ATST0_BIT << (cmp<<2)); /* infinity loop */
	}
	/* set mode reg */
	outl( value, pwm_info_p->channel_mode );

	/* save callback function & data */
	if( src_cmpcnt->cb_info == NULL ) {
		d0b("\n");
		dst_cmpcnt->cb_info = NULL;
	}else{
		d0b("\n");
		src_cb_info = src_cmpcnt->cb_info;

		if( (src_cb_info->cb_count != NULL)
		    || (src_cb_info->cb_loop != NULL) ) {

			d0b("\n");
			/* set callback */
			dst_cmpcnt->cb_info = &g_pwm_cb_info[channel][cmp];
			dst_cb_info = dst_cmpcnt->cb_info;
			src_cb_info = src_cmpcnt->cb_info;

			if( src_cb_info->cb_count != NULL ) {
				d0b("\n");
				dst_cb_info->cb_count = src_cb_info->cb_count;
				dst_cb_info->cb_count_data 
				    = src_cb_info->cb_count_data;
			}else{
				d0b("\n");
				dst_cb_info->cb_count = NULL;
				dst_cb_info->cb_count_data = NULL;
			}
			if( src_cb_info->cb_loop != NULL ) {
				d0b("\n");
				dst_cb_info->cb_loop = src_cb_info->cb_loop;
				dst_cb_info->cb_loop_data 
				    = src_cb_info->cb_loop_data;
			}else{
				d0b("\n");
				dst_cb_info->cb_loop = NULL;
				dst_cb_info->cb_loop_data = NULL;
			}
		}else{
			d0b("\n");
			dst_cmpcnt->cb_info = NULL;
		}
	}
	spin_unlock_irqrestore(&spin_lock, lock_flags);

	FNC_EXIT return( 0 );
}

/* --------------------------------------------------------------
	̾   : pwmGetCmpConfig
	   : PWMͥ롦ڥưּ
	   : ch      : ֹͥ
		 cmp     : ڥֹ(ʣԲ)
		 pConfig : PWMڥ󥿥ե졼¤ΤؤΥݥ
	 : BIOS_OK         : ｪλ
			 BIOS_ERR_PARAM  : ѥ᡼顼
   -------------------------------------------------------------- */
int mp200_pwm_get_compare_counter( unsigned char channel, mp200_pwm_cmpcnt_t *pwm_cmpcnt )
{
	int err;
	mp200_pwm_cb_info_t *info;
	unsigned int mode_value;
	pwm_info_t *pwm_info_p;
	mp200_pwm_cmpcnt_t *cmpcnt_reg_p;
	unsigned char cmp;
	FNC_ENTRY

	if (pwm_init_flag) {
#ifdef PWM_DEBUG
		printk(KERN_INFO "%s : pwm initialize error\n", __FUNCTION__);
#endif
		FNC_EXIT return -EPERM;
	}
	
	/* check parameter */
	err = check_validity_channel(channel);
	if( err != 0 ) {
#ifdef PWM_DEBUG
		printk(KERN_INFO "%s : channel error(ch.%d)\n", __FUNCTION__, channel );
#endif
		FNC_EXIT return( -ENODEV );
	}
	if( pwm_cmpcnt == NULL ) {
#ifdef PWM_DEBUG
		printk(KERN_INFO "%s : pwm_cmcnt is NULL!\n", __FUNCTION__);
#endif
		FNC_EXIT return( -EINVAL );
	}
	cmp = pwm_cmpcnt->cmp;

	err = check_validity_compare_counter(cmp);
	if( err != 0 ) {
#ifdef PWM_DEBUG
		printk(KERN_INFO "%s : compare counter illegal(cmp=0x%04X)\n", __FUNCTION__, cmp);
#endif
		FNC_EXIT return( -ENODEV );
	}

	pwm_info_p = &g_pwm_info[channel];
	cmpcnt_reg_p = &g_pwm_cmpcnt[channel][cmp];

	/* read reg */
	pwm_cmpcnt->delay       = inl( cmpcnt_reg_p->delay );
	pwm_cmpcnt->total_cycle = inl( cmpcnt_reg_p->total_cycle );
	pwm_cmpcnt->lead_edge   = inl( cmpcnt_reg_p->lead_edge );
	pwm_cmpcnt->trail_edge  = inl( cmpcnt_reg_p->trail_edge );
	/* read mode reg */
	mode_value = inl( pwm_info_p->channel_mode );
	if( mode_value &  (PWM_CMP_ATST0_BIT <<(cmp<<2)) ) {
		pwm_cmpcnt->loop_count	= inl( cmpcnt_reg_p->loop_count );
	} else {
		pwm_cmpcnt->loop_count	= 0;
	}

	/* load callback function & data */
	if( NULL != pwm_cmpcnt->cb_info ) {
#ifdef PWM_DEBUG
		printk(KERN_INFO "%s : cb_info is not NULL.\n", __FUNCTION__ );
#endif

		if( NULL == cmpcnt_reg_p->cb_info ) {
			pwm_cmpcnt->cb_info->cb_count = NULL;
			pwm_cmpcnt->cb_info->cb_count_data = NULL;
			pwm_cmpcnt->cb_info->cb_loop = NULL;
			pwm_cmpcnt->cb_info->cb_loop_data = NULL;
		}else{
		/* regist irq_interrupt() */
			info = cmpcnt_reg_p->cb_info;
			if( NULL == info->cb_count ) {
				pwm_cmpcnt->cb_info->cb_count = NULL;
				pwm_cmpcnt->cb_info->cb_count_data = NULL;
			}else{
				pwm_cmpcnt->cb_info->cb_count = info->cb_count;
				if( NULL == info->cb_count_data ) {
					pwm_cmpcnt->cb_info->cb_count_data = NULL;
				}else{
					pwm_cmpcnt->cb_info->cb_count_data = info->cb_count_data;
				}
			}
			if( NULL == info->cb_loop ) {
				pwm_cmpcnt->cb_info->cb_loop = NULL;
				pwm_cmpcnt->cb_info->cb_loop_data = NULL;
			}else{
				pwm_cmpcnt->cb_info->cb_loop = info->cb_loop;
				if( NULL == info->cb_loop_data ) {
					pwm_cmpcnt->cb_info->cb_loop_data = NULL;
				}else{
					pwm_cmpcnt->cb_info->cb_loop_data = info->cb_loop_data;
				}
			}
		}
	}
#ifdef PWM_DEBUG
	else {
		printk(KERN_INFO "%s : cb_info is NULL!!\n", __FUNCTION__ );
	}
#endif


	FNC_EXIT return( 0 );
}

/*
 * init
 */
static int __init
pwm_init(void)
{
	int ret = 0;
	int i, j;

	FNC_ENTRY

	printk(KERN_INFO "Starting %s.\n", PWM_NAME);

	/* stop clock */
	pwm_clock_stop();

	/* pctrl change to pwm */
	spin_lock_irq(&spin_lock);
	outl(inl(SMU_PCTL_C2) & ~PWM_PCTL_MASK, SMU_PCTL_C2); /* BIT7,6: 0,PWM: */
	spin_unlock_irq(&spin_lock);
	
	for( i = 0; i < PWM_MAX_CHANNEL_NUM; i++ ) {
		for( j = 0; j < PWM_MAX_COUNTER_NUM; j++ ) {
			g_pwm_cb_info[i][j].cb_count = NULL;
			g_pwm_cb_info[i][j].cb_count_data = NULL;
			g_pwm_cb_info[i][j].cb_loop = NULL;
			g_pwm_cb_info[i][j].cb_loop_data = NULL;
		}
	}

	outl(PWM_INIT_CLOCK, SMU_PWM_SCLK_SEL );

	/* supply clock */
	pwm_clock_start();
	/* unreset PWM */
	pwm_dev_unreset();
	/* stop sclock */
	/*pwm_sclock_stop(); comment out by ita 20061127. To secceed to the setting of u-boot */

	/* regist irq handler */
	ret = request_irq( INT_PWM, pwm_irq_handler, 
			SA_INTERRUPT, PWM_NAME, 0 );
	if( ret != 0 ) {
		printk(KERN_INFO
			"%s(): unable to request IRQ %d for PWM channel \n",
			__FUNCTION__, INT_PWM );
		/* stop clock */
		pwm_clock_stop();
		FNC_EXIT return( ret );
	}
	
	disable_irq(INT_PWM);

	pwm_init_flag = 0;

	FNC_EXIT return( 0 );
}

/*
 * exit
 */
static void __exit pwm_exit(void)
{
	FNC_ENTRY

	pwm_init_flag = 1;

	free_irq(INT_PWM, 0);
	
	/* stop clock */
	pwm_clock_stop();

	FNC_EXIT return;
}


#ifdef MODULE
module_init(pwm_init);
module_exit(pwm_exit);
#else

#ifdef CONFIG_SONY_ICX1087
arch_initcall(pwm_init);
#else
__initcall(pwm_init);
#endif

__exitcall(pwm_exit);
#endif

#ifndef MODULE
EXPORT_SYMBOL(mp200_pwm_start);
EXPORT_SYMBOL(mp200_pwm_stop);
EXPORT_SYMBOL(mp200_pwm_set_channel_config);
EXPORT_SYMBOL(mp200_pwm_get_channel_config);
EXPORT_SYMBOL(mp200_pwm_set_compare_counter);
EXPORT_SYMBOL(mp200_pwm_get_compare_counter);
#endif

#ifdef MODULE
#ifdef MP200_DEBUG
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
module_param(debug,  int, 0644);
#endif
#endif
MODULE_AUTHOR("2006 NEC ELECTRONICS CORPORATION");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("'PWM' pulse width moduration device driver.");
MODULE_SUPPORTED_DEVICE("");
#endif
