/* 
 * linux/drivers/ide/arm/mxc_ide.c
 *
 * Copyright (c) 2003-2004 Simtec Electronics
 *  Ben Dooks <ben@simtec.co.uk>
 *
 * Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <linux/module.h>
#include <linux/errno.h>
#include <linux/ide.h>
#include <linux/init.h>

#include <asm/mach-types.h>

#include <asm/io.h>
#include <asm/irq.h>
#include <asm/arch/platform.h>
#include "mxc_ide.h"


/* List of registered interfaces */
static ide_hwif_t *ifs[1];

static void mxc_selectproc (ide_drive_t *drive)
{
        printk("mxc_selectproc()...\n");
}

/* function to tune to PIO mode */

static void mxc_tune (ide_drive_t *drive, u8 pio)
{
        printk("mxc_tune()...\n");
}


static int __init
mxcide_register(unsigned long base, unsigned int aux, int irq,
		 ide_hwif_t *hwif)
{
	int i=0;
	hw_regs_t hw;
        const int regsize = 4;
        memset (&hw, 0, sizeof(hw));
        hwif = &ide_hwifs[0];

	for (i = IDE_DATA_OFFSET; i <= IDE_STATUS_OFFSET; i++) {
		hw.io_ports[i] = (unsigned long)base;
		base += regsize;
	}
	hw.io_ports[IDE_CONTROL_OFFSET] = aux;
	hw.irq = irq;
        hwif->selectproc = &mxc_selectproc;
        hwif->tuneproc = &mxc_tune;
	ide_register_hw(&hw, &hwif);

        /* 
         * The IDE code will set hwif->present if we have devices attached,
         * if we don't, discard the interface reset the ports.         
         */
        if ( !hwif->present)
        {
                printk(KERN_INFO "ide%d: Bus empty, interface released.\n",
                        hwif->index);
                 default_hwif_iops(hwif);
                 for (i = IDE_DATA_OFFSET; i <= IDE_CONTROL_OFFSET; ++i)
                           hwif->io_ports[i] = 0;
                 hwif->chipset = ide_unknown;
                 hwif->noprobe = 1;
                 return -ENODEV;
        }

	return 0;
}

static int __init mxcide_init(void)
{
        int index = 0;
	printk("MXC: IDE driver, (c) 2004-2005 Freescale Semiconductor\n");

        index = mxcide_register(IDE_ARM_IO, IDE_ARM_CTL, IDE_ARM_IRQ, &ifs[0]);
        if ( index ==-1)
        {
                printk("MXC_IDE: Failed to register the MXC IDE driver\n");
                return -1;
                   
        }
	return 0;
}

module_init(mxcide_init);

MODULE_AUTHOR("Freescale Semiconductor");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("Freescale MX3 IDE (based on Simtec BAST IDE driver by Ben Dooks <ben@simtec.co.uk>)");

