/*
 *  File Name       : linux/drivers/input/touchscreen/mp200_ts.h
 *  Function        : ADS7846 touchscreen controller
 *  Release Version : Ver 1.01
 *  Release Date    : 2006/02/22
 *
 *  Copyright (C) NEC Electronics Corporation 2005-2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify itunder the terms of
 *  the GNU General Public License as published by Free Softwere Foundation;either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#ifndef __DRIVERS_INPUT_TOUCHSCREEN_MP200_TS_H
#define __DRIVERS_INPUT_TOUCHSCREEN_MP200_TS_H

/* default parameter */
#define GARD_TIME		20	/* 20ms */
#define SCAN_TIME		50	/* 50ms */
#define GEN_PEN_EVENT		1
#define ADJUST_VSYNC		1
#define VSYNC_COUNT		3
#define POST_JITTER_FILTER	7

#define IIR_SHIFT_BITS		2
#define FIR_LEN			4
#define FILTER_FIR		0
#define FILTER_IIR		1

#define MP200_TS_NAME		"mp200_touchscreen"

/* ADS7846 parameters */

/* Conversion bit length */
#define MEASURE_BIT		12	/* 
					 * 12:12bit Conversion mode
					 *  8: 8bit Conversion mode
					 */

/* Control Command bit */
#define ADS7846_START_BIT	0x80

#define ADS7846_MEASURE_X	0x50
#define ADS7846_MEASURE_Y	0x10
#define ADS7846_MEASURE_Z1	0x30
#define ADS7846_MEASURE_Z2	0x40

#define ADS7846_8BITS		0x08
#define ADS7846_12BITS		0x00

#define ADS7846_SER		0x04
#define ADS7846_DFR		0x00

#define ADS7846_PD		0x00
#define ADS7846_ADC_ON		0x01
#define ADS7846_REF_ON		0x02
#define ADS7846_REF_ADC_ON	0x03

#if MEASURE_BIT == 12
#define ADS7846_MEASURE_BIT	ADS7846_12BITS
#else
#define ADS7846_MEASURE_BIT	ADS7846_8BITS
#endif

/* Control Commands */
#define MEASURE_X \
    (ADS7846_START_BIT | ADS7846_MEASURE_X | ADS7846_MEASURE_BIT | ADS7846_DFR | ADS7846_PD)
#define MEASURE_Y \
    (ADS7846_START_BIT | ADS7846_MEASURE_Y | ADS7846_MEASURE_BIT | ADS7846_DFR | ADS7846_PD)
#define MEASURE_Z1 \
    (ADS7846_START_BIT | ADS7846_MEASURE_Z1 | ADS7846_MEASURE_BIT | ADS7846_DFR | ADS7846_PD)
#define MEASURE_Z2 \
    (ADS7846_START_BIT | ADS7846_MEASURE_Z2 | ADS7846_MEASURE_BIT | ADS7846_DFR | ADS7846_PD)

#define ADS7846_DLEN	(1 + MEASURE_BIT)	/* busy cycle + data cycle */

#define ADS7846_DATA_MASK ((1UL << MEASURE_BIT) - 1)

#define ADS7846_UWIRE_CS  UWIRE_CS_BOARD_B
#define ADS7846_UWIRE_CLK UWIRE_SCLOCK_1500KHz

#define ADS7846_STATUS_BIT (1 << PWC_INT_TS_INT)

struct mp200_ts_driver {
	int irq;

	int x_raw, y_raw, z1_raw, z2_raw;

	/* LCD VSYNC */
	unsigned int adjust_vsync;

	/* filter type: FILTER_IIR or FILTER_FIR or NONE */
	unsigned int filter_type;
	int nsample;

	/* FIR */
	int fir_x[FIR_LEN], fir_y[FIR_LEN];
	int fir_index;

	/* IIR: infinite impulse response low-pass filter */
	int x_iir, y_iir;
	int iir_shift;
	int iir_output_threshold;
	int ntotal;

	/* post jitter filter */
	unsigned int post_jitter_filter;
	int x_out, y_out;

	unsigned int generate_pen_event;

	/* pen state flag */
	int pendown;

	/* chattering gard timer */
	struct timer_list chatter_timer;
	unsigned int gard_time;

	/* scan timer */
	struct timer_list acq_timer;
	unsigned int scan_time;

	/* VSYNC scan */
	lcdc_vsync_action_t vsync_action;

	struct input_dev mp200_ts_dev;
};
#endif				/* __DRIVERS_INPUT_TOUCHSCREEN_MP200_TS_H */
