/*
 * File Name       : drivers/media/video/mp200/mp200_v4l2_core.c
 * Function        : Video for Linux driver for MP201 driver core
 * Release Version :
 * Release Date    :
 *
 * Copyright (C) NEC Electronics Corporation 2006
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

/*
 * Copyright 2006,2007 Sony Corporation. 
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/vmalloc.h>
#include <linux/slab.h>
#include <linux/proc_fs.h>
#include <linux/ctype.h>
#include <linux/pagemap.h>
#include <linux/mm.h>
#include <linux/device.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/videodev.h>
#include <linux/pci.h>
#include <asm/semaphore.h>
#include <asm/processor.h>
#include <linux/dma-mapping.h>
#include <linux/fb.h>

#include <asm/io.h>
#include <asm/byteorder.h>
#include <asm/irq.h>

#include <asm/arch/rot.h>
#include <asm/arch/mp201_mem.h>
#include <../drivers/video/mp200_v4l2/mp200_common.h>
#include "mp200_v4l2_core.h"

#include <asm/arch/icx1087_cycles.h>  /* 2006.08.15 ita*/
#define  PERIOD_IRQ 400                /*to measure period of irq disable.*/

/*===============================================================*/
/* debug parameters                                              */
/*===============================================================*/
#define _DPM_DBG   0 /* debug suspend/resume *//* 0: disable  1: enable */
#define _LCK_DBG   0 /* debug spinlock */      /* 0: disable  1: enable */
#define _WKQ_DBG   0 /* debug workqueue */     /* 0: disable  1: enable */
#define _TMR_DBG   0 /* debug timer */         /* 0: disable  1: enable */
#define _VBQ_DBG   0 /* debug videobuf_queue *//* 0: disable  1: enable */
#define _ROT_DBG   0 /* debug ROT process */   /* 0: disable  1: print  2: callback print */
#define _LCD_DBG   0 /* debug LCD process */   /* 0: disable  1: print  2: callback print */

#define dbg_printk(level, fmt, arg...) \
	if(level > 0) printk(KERN_DEBUG " @v4l2: " fmt, ## arg);

#define dbg_lock(lock, fmt) \
	if(_LCK_DBG > 0){   \
		if(spin_is_locked(lock)) printk(KERN_DEBUG " @v4l2: " fmt); \
	}

/*===============================================================*/
/* external function prottyped                                   */
/*===============================================================*/
extern int  mp200_lcd_set_v4l2_image_data(V4L2_IMAGE_INFO v4l2_iamge_info); /* LCD request function */
extern void mp200_rot_regist(struct mp200_rot_req *p);                      /* ROT request function */


/*===============================================================*/
/* callback function prottyped                                   */
/*===============================================================*/
void mp200_v4l2_core_rot_callback(struct mp200_rot_req *p, int flag);
void mp200_v4l2_core_lcd_callback(int err, FRAME_DATA frame_data);


/*===============================================================*/
/* kernel timer function                                         */
/*===============================================================*/
static void mp200_v4l2_init_timer(struct mp200_v4l2_device *dev);
static void mp200_v4l2_calc_waittime(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb);
static void mp200_v4l2_calc_adjusttime(struct mp200_v4l2_device *dev);
static int  mp200_v4l2_add_timer(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb);
static void mp200_v4l2_timeout_top_half(unsigned long data);
static int  mp200_v4l2_check_process(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb);


/*===============================================================*/
/* workqueue function                                            */
/*===============================================================*/
static void mp200_v4l2_timeout_bottom_half_do(void *num);
static void mp200_v4l2_reschedule_do(void *num);
static void mp200_v4l2_rot_callback_bottom_half_do(void *num);
static void mp200_v4l2_lcd_call_do(void *num);
static void mp200_v4l2_stop_do(void *num);


/*===============================================================*/
/* internal function prottyped                                   */
/*===============================================================*/
/* ------------------ rot_driver_ops -------------------------------------------- */
static int  mp200_v4l2_rot_call_func(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb);
static int  mp200_v4l2_make_rot_req(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, MP200_ROT_REQ_V4L2 *qROT_req);
/* ------------------ lcd_driver_ops -------------------------------------------- */
static int  mp200_v4l2_lcd_call_func(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, int call_flg);
/* ------------------ videobuf_queue_ops ---------------------------------------- */
static int  mp200_v4l2_core_vbq_setup(struct videobuf_queue *q, unsigned int *cnt, unsigned int *size);
static int  mp200_v4l2_core_vbq_prepare(struct videobuf_queue *q, struct videobuf_buffer *vb, enum v4l2_field field);
static int mp200_v4l2_core_vbq_sizecheck_pix(struct videobuf_buffer *vb);
static int mp200_v4l2_core_vbq_sizecheck_crop(struct videobuf_buffer *vb);
static int mp200_v4l2_core_vbq_sizecheck(struct videobuf_queue *q, struct videobuf_buffer *vb);
static int  mp200_v4l2_core_vbq_cancel(struct videobuf_queue *q, struct videobuf_buffer *vb);
static void mp200_v4l2_core_vbq_queue(struct videobuf_queue *q, struct videobuf_buffer *vb);
static void mp200_v4l2_core_vbq_complete(void *arg1, void *arg);
static void mp200_v4l2_core_vbq_release(struct videobuf_queue *q, struct videobuf_buffer *vb);
/* ------------------ system_call_ops ------------------------------------------- */
static int  mp200_v4l2_core_do_ioctl(struct inode *inode, struct file *file, unsigned int cmd, void *arg);
static int  mp200_v4l2_core_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg);
static unsigned int  mp200_v4l2_core_poll(struct file *file, struct poll_table_struct *wait);
static int  mp200_v4l2_core_release(struct inode *inode, struct file *file);
static int  mp200_v4l2_core_open(struct inode *inode, struct file *file);
#ifdef CONFIG_DPM
static int  mp200_v4l2_core_suspend(struct device *dev, u32 state, u32 level);
static int  mp200_v4l2_core_resume(struct device *dev, u32 level);
#endif	/* CONFIG_DPM */
void        mp200_v4l2_core_cleanup(void);
int __init  mp200_v4l2_core_init(void);


/*===============================================================*/
/* file_operations                                               */
/*===============================================================*/
static struct file_operations mp200_v4l2_core_fops = 
{
	.owner			= THIS_MODULE,
	.llseek			= no_llseek,
	.ioctl			= mp200_v4l2_core_ioctl,
	.poll			= mp200_v4l2_core_poll,
	.open			= mp200_v4l2_core_open,
	.release		= mp200_v4l2_core_release,
};


/*===============================================================*/
/* device_driver                                                 */
/*===============================================================*/
static struct device_driver mp200_v4l2_core_driver = {
	.name			= DEV_NAME,
	.bus			= &platform_bus_type,
	.probe			= NULL,
	.remove			= NULL,
#ifdef CONFIG_DPM
	.suspend		= mp200_v4l2_core_suspend,
	.resume			= mp200_v4l2_core_resume,
#endif	/* CONFIG_DPM */
	.shutdown		= NULL,
};


/*===============================================================*/
/* platform_device                                               */
/*===============================================================*/
static struct platform_device mp200_v4l2_core_device = {
	.name	= DEV_NAME,
	.dev	= {
			.release 	= NULL,
		  },
	.id	= 0,
};


/*===============================================================*/
/* module parameters                                             */
/*===============================================================*/
struct mp200_v4l2_device *mp200_dev;
static int video_nr = -1;	/* video device minor (-1 ==> auto assign) */




/* ------------------ timer_function ----------------------------------------- */
/*****************************************************************************
* MODULE   : mp200_v4l2_init_timer
* FUNCTION : wait to call LCD driver init function.
* RETURN   : -
* NOTE     : 
* CREATE   : 2006/06/16 M.Haguro
* UPDATE   : 
******************************************************************************/
static void mp200_v4l2_init_timer(struct mp200_v4l2_device *dev)
{
	unsigned long flags;
    int start, us;   /*2006.08.15 ita.to measure period of irq disable.*/

	dbg_lock(&dev->timer_lock, "init_timer: timer_lock\n");
	spin_lock_irqsave(&dev->timer_lock, flags);
    start = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
	dev->timer.adjust_time = DEFAULT_ADJUST;

	init_timer(&dev->timer.wait_timer);
	dev->timer.wait_timer.function = mp200_v4l2_timeout_top_half;
	dev->timer.wait_timer.data = 0;
    /*2006.08.15 ita.to measure period of irq disable.*/
    us = CalcMicroSec(GetCycles() - start);
    if (us > PERIOD_IRQ) {
        printk(KERN_INFO "V4L2(v4l2_core)10: long disable interrupt -- %dus\n",us);
    }
	spin_unlock_irqrestore(&dev->timer_lock, flags);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_calc_waittime
* FUNCTION : wait to call LCD driver function.
* RETURN   : -
* NOTE     : 
* CREATE   : 2006/06/16 M.Haguro
* UPDATE   : 
******************************************************************************/
static void mp200_v4l2_calc_waittime(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb)
{
	do_gettimeofday(&dev->timer.now_time);
	if(vb->ts.tv_sec > dev->timer.now_time.tv_sec){
		dev->timer.wait_time = (vb->ts.tv_sec - dev->timer.now_time.tv_sec) * 1000;
		if(vb->ts.tv_usec > dev->timer.now_time.tv_usec){
			dev->timer.wait_time += (vb->ts.tv_usec - dev->timer.now_time.tv_usec) / 1000;
		}else{
			dev->timer.wait_time -= (dev->timer.now_time.tv_usec - vb->ts.tv_usec) / 1000;
		}
	}else if(vb->ts.tv_sec == dev->timer.now_time.tv_sec){
		if(vb->ts.tv_usec > dev->timer.now_time.tv_usec){
			dev->timer.wait_time = (vb->ts.tv_usec - dev->timer.now_time.tv_usec) / 1000;
		}else{
			dev->timer.wait_time = -1;
		}
	}else{
		dev->timer.wait_time = -1;
	}
	dbg_printk(_TMR_DBG, "dev->timer.wait_time = %ld\n", dev->timer.wait_time);
	dev->timer.wait_time -= dev->timer.adjust_time;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_calc_adjusttime
* FUNCTION : wait to call LCD driver function.
* RETURN   : -
* NOTE     : 
* CREATE   : 2006/06/16 M.Haguro
* UPDATE   : 
******************************************************************************/
static void mp200_v4l2_calc_adjusttime(struct mp200_v4l2_device *dev)
{
	unsigned long flags;
    int start, us;   /*2006.08.15 ita.to measure period of irq disable.*/

	dbg_lock(&dev->timer_lock, "calc_adjusttime: timer_lock\n");
	spin_lock_irqsave(&dev->timer_lock, flags);
    start = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
	do_gettimeofday(&dev->timer.comp_time);
	if(dev->timer.comp_time.tv_sec > dev->timer.do_time.tv_sec){
		dev->timer.adjust_time = (dev->timer.comp_time.tv_sec - dev->timer.do_time.tv_sec) * 1000;
		if(dev->timer.comp_time.tv_usec > dev->timer.do_time.tv_usec){
			dev->timer.adjust_time += (dev->timer.comp_time.tv_usec - dev->timer.do_time.tv_usec) / 1000;
		}else{
			dev->timer.adjust_time -= (dev->timer.do_time.tv_usec - dev->timer.comp_time.tv_usec) / 1000;
		}
	}else if(dev->timer.comp_time.tv_sec == dev->timer.do_time.tv_sec){
		if(dev->timer.comp_time.tv_usec > dev->timer.do_time.tv_usec){
			dev->timer.adjust_time = (dev->timer.comp_time.tv_usec - dev->timer.do_time.tv_usec) / 1000;
		}else{
			dev->timer.adjust_time = 0;
		}
	}else{
		dev->timer.adjust_time = 0;
	}
	dbg_printk(_TMR_DBG, "dev->timer.adjust_time = %ld\n", dev->timer.adjust_time);
    us = CalcMicroSec(GetCycles() - start);
    if (us > PERIOD_IRQ) {
        printk(KERN_INFO "V4L2(v4l2_core)20: long disable interrupt -- %dus\n",us);
    }
	spin_unlock_irqrestore(&dev->timer_lock, flags);
    /*2006.08.15 ita.to measure period of irq disable.*/
}


/*****************************************************************************
* MODULE   : mp200_v4l2_add_timer
* FUNCTION : wait to call LCD driver function.
* RETURN   : -
* NOTE     : 
* CREATE   : 2006/06/16 M.Haguro
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_add_timer(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb)
{
	struct mp200_v4l2_fh *fh = dev->streaming;
	unsigned long flags;
    int start, us;   /*2006.08.15 ita.to measure period of irq disable.*/

	dbg_printk(_TMR_DBG, "add_timer() in.\n");

	dbg_lock(&dev->timer_lock, "add_timer: timer_lock\n");
	spin_lock_irqsave(&dev->timer_lock, flags);
    start = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
	dev->timer.vb = vb;
	mp200_v4l2_calc_waittime(dev, vb);

	if(dev->timer.wait_time > 10){
		dev->timer.wait_timer.expires = jiffies + dev->timer.wait_time / 10;
		dev->waiting = fh;

		dev->do_timer = STATE_RUN_TIMER;
		add_timer(&dev->timer.wait_timer);
	}
	else{
		dev->do_timer = STATE_STOP_TIMER;
		queue_work(dev->v4l2_workqueue, &dev->wk_timeout_bottom_half);
	}
    /*2006.08.15 ita.to measure period of irq disable.*/
    us = CalcMicroSec(GetCycles() - start);
    if (us > PERIOD_IRQ) {
        printk(KERN_INFO "V4L2(v4l2_core)30: long disable interrupt -- %dus\n",us);
    }
	spin_unlock_irqrestore(&dev->timer_lock, flags);
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_timeout_top_half
* FUNCTION : wait to call LCD driver function.
* RETURN   : -
* NOTE     : 
* CREATE   : 2006/06/16 M.Haguro
* UPDATE   : 
******************************************************************************/
static void mp200_v4l2_timeout_top_half(unsigned long data)
{
	struct mp200_v4l2_device *dev = mp200_dev;
	struct videobuf_buffer *vb    = dev->timer.vb;
	unsigned long flags;
    int start, us;   /*2006.08.15 ita.to measure period of irq disable.*/

	/* check process */
	dbg_lock(&dev->do_timer_lock, "timeout_top_half: do_timer_lock\n");
	spin_lock_irqsave(&dev->do_timer_lock, flags);
    start = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
	if(!mp200_v4l2_check_process(dev, vb)){
		dev->do_timer = STATE_STOP_TIMER;
		queue_work(dev->v4l2_workqueue, &dev->wk_timeout_bottom_half);
	}
    /*2006.08.15 ita.to measure period of irq disable.*/
    us = CalcMicroSec(GetCycles() - start);
    if (us > PERIOD_IRQ) {
        printk(KERN_INFO "V4L2(v4l2_core)50: long disable interrupt -- %dus\n",us);
    }
	spin_unlock_irqrestore(&dev->do_timer_lock, flags);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_check_process
* FUNCTION : 
* RETURN   :  0: success
*          : -1: stop timer
* NOTE     : 
* CREATE   : 
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_check_process(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb)
{
	unsigned long flags, flags2;
	int retval = 0;
    int start, us;   /*2006.08.15 ita.to measure period of irq disable.*/

	dbg_lock(&dev->img_lock, "check_process: img_lock\n");
	spin_lock_irqsave(&dev->img_lock, flags);
    start = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
	if(vb->ts.tv_sec != -1 && vb->ts.tv_usec != -1){
		if(dev->do_timer == STATE_STOP_TIMER){
			dbg_lock(&dev->timer_lock, "check_process: timer_lock\n");
			spin_lock_irqsave(&dev->timer_lock, flags2);
			dev->waiting = NULL;
			spin_unlock_irqrestore(&dev->timer_lock, flags2);

			dbg_lock(&dev->mix_lock, "check_process: mix_lock\n");
			spin_lock_irqsave(&dev->mix_lock, flags2);
			dev->mixing = NULL;
			spin_unlock_irqrestore(&dev->mix_lock, flags2);

			retval = -1;
		}
	}
    /*2006.08.15 ita.to measure period of irq disable.*/
    us = CalcMicroSec(GetCycles() - start);
    if (us > PERIOD_IRQ) {
        printk(KERN_INFO "V4L2(v4l2_core)70: long disable interrupt -- %dus\n",us);
    }
	spin_unlock_irqrestore(&dev->img_lock, flags);

	return retval;
}


/* ------------------ workqueue_function ------------------------------------------ */
/*****************************************************************************
* MODULE   : mp200_v4l2_timeout_bottom_half_do
* FUNCTION : wait to call LCD driver function.
* RETURN   : -
* NOTE     : 
* CREATE   : 2006/06/16 M.Haguro
* UPDATE   : 
******************************************************************************/
static void mp200_v4l2_timeout_bottom_half_do(void *num)
{
	struct mp200_v4l2_device *dev = mp200_dev;
	struct videobuf_buffer   *vb  = dev->timer.vb;

	dbg_printk(_WKQ_DBG, "timeout_bottom_half_do: lcd_call_func().\n");
	dbg_printk(_VBQ_DBG, "timeout_bottom_half_do: (%p) ->prev(%p) ->next(%p) ->state(%d)\n", vb, vb->stream.prev, vb->stream.next, vb->state);

	if(dev->streaming){
#if ENABLE_DELAY
		unsigned long flags;

		dbg_lock(&dev->timer_lock, "timeout_bottom_half_do: timer_lock\n");
		spin_lock_irqsave(&dev->timer_lock, flags);
		mp200_v4l2_calc_waittime(dev, vb);

		if(dev->timer.wait_time > 0){
			spin_unlock_irqrestore(&dev->timer_lock, flags);
			msleep((unsigned int)dev->timer.wait_time);
			spin_lock_irqsave(&dev->timer_lock, flags);
		}
		spin_unlock_irqrestore(&dev->timer_lock, flags);
#endif

		/* call lcd driver function. */
		if(mp200_v4l2_lcd_call_func(dev, vb, STREAM_PLAYBACK)){
			mp200_v4l2_core_vbq_complete(dev, vb);
			queue_work(dev->v4l2_workqueue, &dev->wk_reschedule);
		}
	}
}


/*****************************************************************************
* MODULE   : mp200_v4l2_reschedule_do
* FUNCTION : call LCD driver function, next queue.
* RETURN   : -
* NOTE     : 
* CREATE   : 2006/06/21 M.Haguro
* UPDATE   : 
******************************************************************************/
static void mp200_v4l2_reschedule_do(void *num)
{
	struct mp200_v4l2_device *dev = mp200_dev;
	struct videobuf_buffer   *vb  = dev->timer.vb;
	struct videobuf_queue    *q   = &dev->streaming->vbq;
	struct list_head *list;
	unsigned long flags, flags2, flags3;
    int start, us;   /*2006.08.15 ita.to measure period of irq disable.*/
    int start2, us2;   /*2006.08.15 ita.to measure period of irq disable.*/

	dbg_printk(_WKQ_DBG, "reschedule_do: call in.\n");

	dbg_lock(&dev->vbq_lock, "reschedule_do: vbq_lock\n");
	spin_lock_irqsave(&dev->vbq_lock, flags);
    start = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
	if(dev->streaming){
		list_for_each(list, &q->stream) {
			vb = list_entry(list, struct videobuf_buffer, stream);
			if (vb->state == STATE_PREPARED)
			{
				/* call lcd */
				vb->state = STATE_QUEUED;
				dbg_printk(_VBQ_DBG, "reschedule_do: (%p) ->prev(%p) ->next(%p) ->state(%d)\n", vb, vb->stream.prev, vb->stream.next, vb->state);

				if(vb->ts.tv_sec == -1 || vb->ts.tv_usec == -1){
					/* call lcd driver function. */
					dev->timer.vb = vb;
					dbg_printk(_WKQ_DBG, "reschedule_do: wk_lcd_call()\n");
					queue_work(dev->v4l2_workqueue, &dev->wk_lcd_call);
				}else{
					/* start timer */
					dbg_lock(&dev->do_timer_lock, "reschedule_do: do_timer_lock\n");
					spin_lock_irqsave(&dev->do_timer_lock, flags2);
                    start2 = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
					dbg_printk(_WKQ_DBG, "reschedule_do: add timer()\n");

					mp200_v4l2_add_timer(dev, vb);
                    /*2006.08.15 ita.to measure period of irq disable.*/
                    us2 = CalcMicroSec(GetCycles() - start2);
                    if (us2 > PERIOD_IRQ) {
                        printk(KERN_INFO "V4L2(v4l2_core)80: long disable interrupt -- %dus\n",us2);
                    }
					spin_unlock_irqrestore(&dev->do_timer_lock, flags2);
				}
                /*2006.08.15 ita.to measure period of irq disable.*/
                us = CalcMicroSec(GetCycles() - start);
                if (us > PERIOD_IRQ) {
                    printk(KERN_INFO "V4L2(v4l2_core)90: long disable interrupt -- %dus\n",us);
                }
				spin_unlock_irqrestore(&dev->vbq_lock, flags);
				return;
			}
		}
	}
	dbg_printk(_WKQ_DBG, "reschedule_do: stop mixing.\n");
	dbg_lock(&dev->timer_lock, "reschedule_do: timer_lock\n");
	dbg_lock(&dev->mix_lock, "reschedule_do: mix_lock\n");
	spin_lock_irqsave(&dev->timer_lock, flags2);
	spin_lock_irqsave(&dev->mix_lock, flags3);
	dev->mixing = NULL;
	spin_unlock_irqrestore(&dev->mix_lock, flags3);
	dev->waiting = NULL;
	spin_unlock_irqrestore(&dev->timer_lock, flags2);
    /*2006.08.15 ita.to measure period of irq disable.*/
    us = CalcMicroSec(GetCycles() - start);
    if (us > PERIOD_IRQ) {
        printk(KERN_INFO "V4L2(v4l2_core)100: long disable interrupt -- %dus\n",us);
    }
	spin_unlock_irqrestore(&dev->vbq_lock, flags);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_rot_callback_bottom_half_do
* FUNCTION : wait to call LCD driver function.
* RETURN   : -
* NOTE     : 
* CREATE   : 2006/06/16 M.Haguro
* UPDATE   : 
******************************************************************************/
static void mp200_v4l2_rot_callback_bottom_half_do(void *num)
{
	struct mp200_v4l2_device *dev = mp200_dev;
	struct videobuf_buffer   *vb  = dev->timer.vb;

	dbg_printk((_WKQ_DBG || (_ROT_DBG & 0x2)), "rot_callback_bottom_half_do: lcd_call_func().\n");

	if(dev->streaming){
		/* call lcd driver function. */
		if(mp200_v4l2_lcd_call_func(dev, vb, STREAM_PLAYBACK_ROT)){
			mp200_v4l2_core_vbq_complete(dev, vb);
			queue_work(dev->v4l2_workqueue, &dev->wk_reschedule);
		}
	}
}


/*****************************************************************************
* MODULE   : mp200_v4l2_lcd_call_do
* FUNCTION : wait to call LCD driver function.
* RETURN   : -
* NOTE     : 
* CREATE   : 2006/06/16 M.Haguro
* UPDATE   : 
******************************************************************************/
static void mp200_v4l2_lcd_call_do(void *num)
{
	struct mp200_v4l2_device *dev = mp200_dev;
	struct videobuf_buffer   *vb  = dev->timer.vb;

	dbg_printk((_WKQ_DBG || (_LCD_DBG & 0x2)), "lcd_call_do: lcd_call_func()\n");

	if(dev->streaming){
		/* call lcd driver function. */
		if(mp200_v4l2_lcd_call_func(dev, vb, STREAM_PLAYBACK)){
			mp200_v4l2_core_vbq_complete(dev, vb);
			queue_work(dev->v4l2_workqueue, &dev->wk_reschedule);
		}
	}
}


/*****************************************************************************
* MODULE   : mp200_v4l2_stop_do
* FUNCTION : wait to call LCD driver function.
* RETURN   : -
* NOTE     : 
* CREATE   : 2006/07/24 M.Haguro
* UPDATE   : 
******************************************************************************/
static void mp200_v4l2_stop_do(void *num)
{
	dbg_printk((_WKQ_DBG || (_LCD_DBG & 0x2)), "stop_do: lcd_call_func()\n");

	/* call lcd driver function. */
	mp200_v4l2_lcd_call_func(NULL, NULL, STREAM_STOP);
}


/* ------------------ callback_function ----------------------------------------- */
/*****************************************************************************
* MODULE   : mp200_v4l2_core_rot_callback
* FUNCTION : ROT driver callback function.
* RETURN   : -
* NOTE     : When the animation rotation is completed/error, 
*          : the ROT driver calls this function.
* CREATE   : 2006/06/06 M.Haguro
* UPDATE   : 2006/06/14 M.Haguro
******************************************************************************/
void mp200_v4l2_core_rot_callback(struct mp200_rot_req *p, int flag)
{
	struct mp200_v4l2_device *dev = (struct mp200_v4l2_device *)p->data;
	struct videobuf_buffer   *vb  = (struct videobuf_buffer *)dev->rot_data.vb;

	dbg_printk((_ROT_DBG & 0x2), "rot_callback: call in.\n");

	/* get pointer to device information structure */
	dev->rot_data.rot_comp = flag;

	dev->timer.vb = vb;
	queue_work(dev->v4l2_workqueue, &dev->wk_rot_callback_bottom_half);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_lcd_callback
* FUNCTION : LCD driver callback function.
* RETURN   : -
* NOTE     : callback from LCD driver.
*          : When the movie buffer & frame buffer synthesis is completed/error,
*          : the LCD driver calls this function.
* CREATE   : 2006/05/31 M.Haguro
* UPDATE   : 2006/06/21 M.haguro
******************************************************************************/
void mp200_v4l2_core_lcd_callback(int err, FRAME_DATA frame_data)
{
	struct mp200_v4l2_device *dev = (struct mp200_v4l2_device *)frame_data.dev;
	struct videobuf_buffer   *vb  = (struct videobuf_buffer *)frame_data.buf;

	dbg_printk((_LCD_DBG & 0x2), "lcd_callback: call in.\n");

	if(err >= 0){
		if(vb->save_effect.movie_angle != 0){
			/* change next destination buffer */
			if(dev->rot_data.next_buffer == ROT_BUFFER_A)
				dev->rot_data.next_buffer = ROT_BUFFER_B;
			else    dev->rot_data.next_buffer = ROT_BUFFER_A;
		}
	}

	mp200_v4l2_core_vbq_complete(dev, vb);
	queue_work(dev->v4l2_workqueue, &dev->wk_reschedule);
}


/* ------------------ rot_driver_ops -------------------------------------------- */
/*****************************************************************************
* MODULE   : mp200_v4l2_rot_call_func
* FUNCTION : request to ROT driver function.
* RETURN   :  0: success
*          : -1: error
* NOTE     : This function the movie rotation demand to the ROT driver.
* CREATE   : 2006/06/06 M.Haguro
* UPDATE   : 2006/06/14 M.Haguro
******************************************************************************/
static int mp200_v4l2_rot_call_func(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb)
{
	unsigned long flags;

	if(mp200_v4l2_make_rot_req(dev, vb, &dev->rot_data.qROT_req)){
		printk("\n @v4l2: ROT request data makeing error.\n\n");
		return -1;
	}

	dbg_lock(&dev->rot_lock, "rot_call_func: rot_lock\n");
	spin_lock_irqsave(&dev->rot_lock, flags);

	/* set pointer to callback function from rot driver */
	dev->rot_data.qROT_req.callback = (void *)mp200_v4l2_core_rot_callback;
	/* set pointer to Device information into dev->req.data */
	dev->rot_data.qROT_req.data     = (unsigned long)dev;
	dev->rot_data.vb  = (void *)vb;

	spin_unlock_irqrestore(&dev->rot_lock, flags);

	/* call rot driver */
	mp200_rot_regist((struct mp200_rot_req *)(&dev->rot_data.qROT_req));

	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_make_rot_req
* FUNCTION : make rotate request data to ROT driver.
* RETURN   :  0: success
*          : -1: error
* NOTE     : The register information table make, requested to the ROT driver.
* CREATE   : 2006/06/06 M.Haguro
* UPDATE   : 2006/06/14 M.Haguro
******************************************************************************/
static int mp200_v4l2_make_rot_req(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, MP200_ROT_REQ_V4L2 *qROT_req)
{
	unsigned long flags;
	unsigned long base_addr;
	int index_req = 0;
	int rot_mode, uv_bpp = 0;
    int start, us;   /*2006.08.15 ita.to measure period of irq disable.*/

	/*******************************
	    make RO_MODE
	********************************/
	rot_mode  = (RO_MODE_SRCENDIAN_LITTLE | RO_MODE_DSTENDIAN_LITTLE);

	switch(vb->pixelformat)
	{
		case V4L2_PIX_FMT_NV12:  /* YUV420 */
			rot_mode |= RO_MODE_SRCYUV_420;
			uv_bpp = 2;
			break;
		case V4L2_PIX_FMT_NV422: /* YUV422 */
			rot_mode |= RO_MODE_SRCYUV_422;
			uv_bpp = 1;
			break;
		default:
			return -1;
	}

	switch(vb->save_effect.movie_angle)
	{
		default:
		case 0: return -1;                              /*   0 deg */
		case 1: rot_mode |= RO_MODE_ROT_MOD_90;  break; /*  90 deg */
		case 2: rot_mode |= RO_MODE_ROT_MOD_180; break; /* 180 deg */
		case 3: rot_mode |= RO_MODE_ROT_MOD_270; break; /* 270 deg */
	}

	dbg_lock(&dev->rot_lock, "make_rot_req: rot_lock\n");
	spin_lock_irqsave(&dev->rot_lock, flags);
    start = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/

	/*******************************
	    set ROT request data
	********************************/
	dev->rot_data.src_add.PhysAddr_Y   = vb->PhysAddr_Y  + vb->save_effect.source_crop.left + vb->save_effect.source_crop.top * vb->bytesperline;
	dev->rot_data.src_add.PhysAddr_UV  = vb->PhysAddr_UV + vb->save_effect.source_crop.left + vb->save_effect.source_crop.top * vb->bytesperline / uv_bpp;
	/* check next destination buffer */
	if(dev->rot_data.next_buffer == ROT_BUFFER_A) base_addr = ROT_BUFFER_ADDR_A;
	else                                          base_addr = ROT_BUFFER_ADDR_B;
	dev->rot_data.rot_add.PhysAddr_Y   = base_addr;
	dev->rot_data.rot_add.PhysAddr_UV  = base_addr + vb->save_effect.source_crop.width * vb->save_effect.source_crop.height;

	if(vb->save_effect.movie_angle == RO_MODE_ROT_MOD_180){
		dev->rot_data.width        = vb->save_effect.source_crop.width;
		dev->rot_data.height       = vb->save_effect.source_crop.height;
		dev->rot_data.bytesperline = vb->save_effect.source_crop.width;
	}
	else{
		dev->rot_data.width        = vb->save_effect.source_crop.height;
		dev->rot_data.height       = vb->save_effect.source_crop.width;
		dev->rot_data.bytesperline = vb->save_effect.source_crop.height;
	}

	/*******************************
	    set ROT register
	********************************/
	/*==============================
	    RO_MODE
	================================*/
	RotRequest(rot_mode,                           RO_MODE);
	RotRequest(0x05,                               RO_FRAME);

	/*==============================
	    source image
	================================*/
	RotRequest(dev->rot_data.src_add.PhysAddr_Y,   RO_SRCYADR_A);
	RotRequest(dev->rot_data.src_add.PhysAddr_UV,  RO_SRCUVADR_A);
	RotRequest(vb->bytesperline,                   RO_SRCSIZE);
	RotRequest(vb->save_effect.source_crop.width,  RO_SRCHSIZE);
	RotRequest(vb->save_effect.source_crop.height, RO_SRCVSIZE);

	/*==============================
	    destination image
	================================*/
	RotRequest(dev->rot_data.rot_add.PhysAddr_Y,   RO_DSTYADR_A);
	RotRequest(dev->rot_data.rot_add.PhysAddr_UV,  RO_DSTUVADR_A);
	RotRequest(dev->rot_data.width,                RO_DSTSIZE);

    /*2006.08.15 ita.to measure period of irq disable.*/
    us = CalcMicroSec(GetCycles() - start);
    if (us > PERIOD_IRQ) {
        printk(KERN_INFO "V4L2(v4l2_core)110: long disable interrupt -- %dus\n",us);
    }
	spin_unlock_irqrestore(&dev->rot_lock, flags);

	/*==============================
	    Start ROT
	================================*/
	RotRequest(RO_REQ_BIT,                         RO_REQ);

	/*==============================
	    end mark
	================================*/
	RotRequest(-1, -1);

	dbg_printk(_ROT_DBG, "=== mp200_rot_regist() =======================================\n");
	dbg_printk(_ROT_DBG, " RO_MODE          > 0x%08x\n", rot_mode);
	switch(vb->save_effect.movie_angle)
	{
		case 1: dbg_printk(_ROT_DBG, "    rotation = 90deg\n");  break;
		case 2: dbg_printk(_ROT_DBG, "    rotation = 180deg\n"); break;
		case 3: dbg_printk(_ROT_DBG, "    rotation = 270deg\n"); break;
	}
	if(uv_bpp == 1){
		dbg_printk(_ROT_DBG, "    YUV422 -> YUV420\n");
	}else{  dbg_printk(_ROT_DBG, "    YUV420 -> YUV420\n");
	}
	dbg_printk(_ROT_DBG, "    Src   > w = %3d  h = %3d\n", vb->save_effect.source_crop.width, vb->save_effect.source_crop.height);
	dbg_printk(_ROT_DBG, "    Dest  > w = %3d\n\n", dev->rot_data.width);
	dbg_printk(_ROT_DBG, "    SrcAddrY    = 0x%08lx      DestAddrY   = 0x%08lx\n", dev->rot_data.src_add.PhysAddr_Y, dev->rot_data.rot_add.PhysAddr_Y);
	dbg_printk(_ROT_DBG, "    SrcAddrUV   = 0x%08lx      DestAddrUV  = 0x%08lx\n", dev->rot_data.src_add.PhysAddr_UV, dev->rot_data.rot_add.PhysAddr_UV);
	dbg_printk(_ROT_DBG, "\n");
	dbg_printk(_ROT_DBG, " RO_FRAME         > 0x05\n");
	dbg_printk(_ROT_DBG, "\n");
	dbg_printk(_ROT_DBG, " RO_SRCYADR_A     > 0x%08lx\n", dev->rot_data.src_add.PhysAddr_Y);
	dbg_printk(_ROT_DBG, " RO_SRCUVADR_A    > 0x%08lx\n", dev->rot_data.src_add.PhysAddr_UV);
	dbg_printk(_ROT_DBG, " RO_SRCSIZE       > %d\n",      vb->bytesperline);
	dbg_printk(_ROT_DBG, " RO_SRCHSIZE      > %d\n",      vb->save_effect.source_crop.width);
	dbg_printk(_ROT_DBG, " RO_SRCVSIZE      > %d\n",      vb->save_effect.source_crop.height);
	dbg_printk(_ROT_DBG, "\n");
	dbg_printk(_ROT_DBG, " RO_DSTYADR_A     > 0x%08lx\n", dev->rot_data.rot_add.PhysAddr_Y);
	dbg_printk(_ROT_DBG, " RO_DSTUVADR_A    > 0x%08lx\n", dev->rot_data.rot_add.PhysAddr_UV);
	dbg_printk(_ROT_DBG, " RO_DSTSIZE       > %d\n",      dev->rot_data.width);
	dbg_printk(_ROT_DBG, "\n");
	dbg_printk(_ROT_DBG, " RO_REQ           > 0x%08x\n", RO_REQ_BIT);
	dbg_printk(_ROT_DBG, "==============================================================\n\n");

	return 0;
}


/* ------------------ lcd_driver_ops -------------------------------------------- */
/*****************************************************************************
* MODULE   : mp200_v4l2_lcd_call_func
* FUNCTION : request to LCD driver function.
* RETURN   :  0: success
*          : -1: error
* NOTE     : This function the drawing demand to the LCD driver.
* CREATE   : 2006/06/01 Y.Hara
* UPDATE   : 2006/06/14 M.Haguro
******************************************************************************/
static int mp200_v4l2_lcd_call_func(struct mp200_v4l2_device *dev, struct videobuf_buffer *vb, int call_flg)
{
	V4L2_IMAGE_INFO	v4l2_image_info;
	unsigned long flags, flags2;
	int retval = 0;

	switch(call_flg)
	{
		case STREAM_PLAYBACK:
		{
			dbg_printk(_VBQ_DBG, "lcd_calli_func: (%p) ->prev(%p)  ->next(%p) ->state(%d)\n", &vb->stream, vb->stream.prev, vb->stream.next, vb->state);
			do_gettimeofday(&dev->timer.do_time);

			/* movie reproduction processing */
			if(vb->save_effect.movie_angle){
				dbg_printk((_LCD_DBG | _ROT_DBG), "lcd_call_func: ROT processing.\n");

				/* YUV image rotation processing */
				retval = mp200_v4l2_rot_call_func(dev, vb);
				return retval;
			}
			else{
				dbg_printk((_LCD_DBG | _ROT_DBG), "lcd_call_func: IMG processing.\n");
				dbg_lock(&dev->img_lock, "lcd_call_func: img_lock\n");

				/* LCD driver's request information making */
				spin_lock_irqsave(&dev->img_lock, flags);
				v4l2_image_info.src_image_data.size     = vb->bytesperline;
				v4l2_image_info.src_image_data.yrgbaddr = vb->PhysAddr_Y;
				v4l2_image_info.src_image_data.uvaddr   = vb->PhysAddr_UV;
				v4l2_image_info.src_image_data.x        = vb->save_effect.source_crop.left;
				v4l2_image_info.src_image_data.y        = vb->save_effect.source_crop.top;
				v4l2_image_info.src_image_data.hsize    = vb->save_effect.source_crop.width;
				v4l2_image_info.src_image_data.vsize    = vb->save_effect.source_crop.height;

				if(vb->pixelformat == V4L2_PIX_FMT_NV12)
					v4l2_image_info.yuvfmt          = V4L2_YUV420_FORMAT;
				else if(vb->pixelformat == V4L2_PIX_FMT_NV422)
					v4l2_image_info.yuvfmt          = V4L2_YUV422_FORMAT;
				else return -1;

				v4l2_image_info.dst_image_hsize         = vb->save_effect.screen.width;
				v4l2_image_info.dst_image_vsize         = vb->save_effect.screen.height;
				v4l2_image_info.offsetx                 = vb->save_effect.screen.left;
				v4l2_image_info.offsety                 = vb->save_effect.screen.top;
				v4l2_image_info.endian                  = V4L2_LITTLE_ENDIAN;

				v4l2_image_info.src_frame_data.dev = (void *)dev;
				v4l2_image_info.src_frame_data.buf = (void *)vb;
				spin_unlock_irqrestore(&dev->img_lock, flags);
			}
			break;
		}

		case STREAM_PLAYBACK_ROT:
		{
			dbg_printk((_LCD_DBG | _ROT_DBG), "lcd_call_func: ROT-IMG processing.\n");
			dbg_lock(&dev->img_lock, "lcd_call_func: img_lock\n");
			dbg_lock(&dev->rot_lock, "lcd_call_func: rot_lock\n");

			/* LCD driver's request information making */
			spin_lock_irqsave(&dev->img_lock, flags);
			spin_lock_irqsave(&dev->rot_lock, flags2);
			v4l2_image_info.src_image_data.size     = dev->rot_data.bytesperline;
			v4l2_image_info.src_image_data.yrgbaddr = dev->rot_data.rot_add.PhysAddr_Y;
			v4l2_image_info.src_image_data.uvaddr   = dev->rot_data.rot_add.PhysAddr_UV;
			v4l2_image_info.src_image_data.x        = 0;
			v4l2_image_info.src_image_data.y        = 0;
			v4l2_image_info.src_image_data.hsize    = dev->rot_data.width;
			v4l2_image_info.src_image_data.vsize    = dev->rot_data.height;
			spin_unlock_irqrestore(&dev->rot_lock, flags2);

			v4l2_image_info.yuvfmt                  = V4L2_YUV420_FORMAT;
			v4l2_image_info.dst_image_hsize         = vb->save_effect.screen.width;
			v4l2_image_info.dst_image_vsize         = vb->save_effect.screen.height;
			v4l2_image_info.offsetx                 = vb->save_effect.screen.left;
			v4l2_image_info.offsety                 = vb->save_effect.screen.top;
			v4l2_image_info.endian                  = V4L2_LITTLE_ENDIAN;

			v4l2_image_info.src_frame_data.dev = (void *)dev;
			v4l2_image_info.src_frame_data.buf = (void *)vb;
			spin_unlock_irqrestore(&dev->img_lock, flags);
			break;
		}

		case STREAM_STOP:
		{
			dbg_printk((_LCD_DBG | _ROT_DBG), "lcd_call_func: STOP processing.\n");

			/* movie stop processing */
			v4l2_image_info.src_image_data.size     = 0;
			v4l2_image_info.src_image_data.yrgbaddr = 0;
			v4l2_image_info.src_image_data.uvaddr   = 0;
			v4l2_image_info.src_image_data.x        = 0;
			v4l2_image_info.src_image_data.y        = 0;
			v4l2_image_info.src_image_data.hsize    = 0;
			v4l2_image_info.src_image_data.vsize    = 0;
			v4l2_image_info.yuvfmt                  = 0;
			v4l2_image_info.dst_image_hsize         = 0;
			v4l2_image_info.dst_image_vsize         = 0;
			v4l2_image_info.offsetx                 = 0;
			v4l2_image_info.offsety                 = 0;
			v4l2_image_info.endian                  = 0;

			v4l2_image_info.src_frame_data.dev = 0;
			v4l2_image_info.src_frame_data.buf = 0;
			break;
		}

		default:
		{
			return -1;
		}
	}

	dbg_printk((_LCD_DBG | _ROT_DBG), "=== mp200_lcd_set_v4l2_image_data() ==========================\n");
	dbg_printk((_LCD_DBG | _ROT_DBG), " src: address Y   = 0x%lx\n", v4l2_image_info.src_image_data.yrgbaddr);
	dbg_printk((_LCD_DBG | _ROT_DBG), "      address UV  = 0x%lx\n", v4l2_image_info.src_image_data.uvaddr);
	dbg_printk((_LCD_DBG | _ROT_DBG), "      ScreenWidth = %d\n", v4l2_image_info.src_image_data.size);
	dbg_printk((_LCD_DBG | _ROT_DBG), "      x = %d,  y = %d\n", v4l2_image_info.src_image_data.x, v4l2_image_info.src_image_data.y);
	dbg_printk((_LCD_DBG | _ROT_DBG), "      w = %d,  h = %d\n", v4l2_image_info.src_image_data.hsize, v4l2_image_info.src_image_data.vsize);
	dbg_printk((_LCD_DBG | _ROT_DBG), " dst: x = %ld,  y = %ld\n", v4l2_image_info.offsetx, v4l2_image_info.offsety);
	dbg_printk((_LCD_DBG | _ROT_DBG), "      w = %d,  h = %d\n", v4l2_image_info.dst_image_hsize, v4l2_image_info.dst_image_vsize);
	dbg_printk((_LCD_DBG | _ROT_DBG), "      endi = %d, fmt = %d\n", v4l2_image_info.endian, v4l2_image_info.yuvfmt);
	dbg_printk((_LCD_DBG | _ROT_DBG), "==============================================================\n\n");

	/* LCD driver call */
	dbg_printk((_LCD_DBG | _ROT_DBG), "lcd_call_func: LCD driver call.\n");
	retval = mp200_lcd_set_v4l2_image_data(v4l2_image_info);
	if ( retval != 0 ) {
		printk("\n @v4l2: mp200_lcd_set_v4l2_image_data: return error.\n\n");
	}

	return retval;
}


/* ------------------ videobuf_queue_ops ---------------------------------------- */
/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_setup
* FUNCTION : 
* RETURN   : 0: success
* NOTE     : Limit the number of available kernel image capture buffers based
*          : on the number requested, the currently selected image size, 
*          : and the maximum amount of memory permitted for kernel buffers.
* CREATE   : 
* UPDATE   : 
******************************************************************************/
static int
mp200_v4l2_core_vbq_setup(struct videobuf_queue *q, unsigned int *cnt, unsigned int *size)
{
	struct mp200_v4l2_device *dev = q->priv_data;
	unsigned long flags;

	if (*cnt <= 0)
		*cnt = VIDEO_MAX_FRAME; /* supply a default number of buffers */

	if (*cnt > VIDEO_MAX_FRAME)
		*cnt = VIDEO_MAX_FRAME;

	dbg_lock(&dev->img_lock, "vbq_setup: img_lock\n");
	spin_lock_irqsave(&dev->img_lock, flags);
	*size = dev->pix.sizeimage;
	spin_unlock_irqrestore(&dev->img_lock, flags);

	if(dev->pix.sizeimage <= 0) return -1;

	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_prepare
* FUNCTION : 
* RETURN   :       0: success
*          : -EINVAL: error
* NOTE     : This function registers information to video buffer.
* CREATE   : 
* UPDATE   : 06/06/12 M.Haguro
******************************************************************************/
static int
mp200_v4l2_core_vbq_prepare(struct videobuf_queue *q, struct videobuf_buffer *vb,
        enum v4l2_field field)
{
	struct mp200_v4l2_device *dev = q->priv_data;
	unsigned long flags;
	unsigned int retval = 0;
    int start, us;   /*2006.08.15 ita.to measure period of irq disable.*/

	dbg_lock(&dev->img_lock, "vbq_prepare: img_lock\n");
	spin_lock_irqsave(&dev->img_lock, flags);
    start = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
	if (dev->pix.sizeimage > vb->bsize) {
        /*2006.08.15 ita.to measure period of irq disable.*/
        us = CalcMicroSec(GetCycles() - start);
        if (us > PERIOD_IRQ) {
            printk(KERN_INFO "V4L2(v4l2_core)120: long disable interrupt -- %dus\n",us);
        }
		spin_unlock_irqrestore(&dev->img_lock, flags);
		return -EINVAL;
	}
	vb->field  = field;

	vb->save_effect.source_crop.left   = dev->efct.source_crop.left;
	vb->save_effect.source_crop.top    = dev->efct.source_crop.top;
	vb->save_effect.source_crop.width  = dev->efct.source_crop.width;
	vb->save_effect.source_crop.height = dev->efct.source_crop.height;
	vb->save_effect.movie_angle        = dev->efct.movie_angle;
	vb->save_effect.screen.left        = dev->efct.screen.left;
	vb->save_effect.screen.top         = dev->efct.screen.top;
	vb->save_effect.screen.width       = dev->efct.screen.width;
	vb->save_effect.screen.height      = dev->efct.screen.height;

	vb->pixelformat                    = dev->pix.pixelformat;
	if((vb->pixelformat != V4L2_PIX_FMT_NV12) && (vb->pixelformat != V4L2_PIX_FMT_NV422)){
		spin_unlock_irqrestore(&dev->img_lock, flags);
		printk("\n @v4l2: error! pixelformat is incorrect.\n\n");
		return -EINVAL;
	}

	/* call size check function */
	if(mp200_v4l2_core_vbq_sizecheck(q, vb)){
        /*2006.08.15 ita.to measure period of irq disable.*/
        us = CalcMicroSec(GetCycles() - start);
        if (us > PERIOD_IRQ) {
            printk(KERN_INFO "V4L2(v4l2_core)130: long disable interrupt -- %dus\n",us);
        }
		spin_unlock_irqrestore(&dev->img_lock, flags);
		return -EINVAL;
	}

	dbg_printk(_VBQ_DBG, "=== vbq_queue() ==============================================\n");
	dbg_printk(_VBQ_DBG, " src :      PhysAddrY(0x%lx)\n", vb->PhysAddr_Y);
	dbg_printk(_VBQ_DBG, "            PhysAddrUV(0x%lx)\n", vb->PhysAddr_UV);
	dbg_printk(_VBQ_DBG, "            width(%d), height(%d), pitch(%d)\n", vb->width, vb->height, vb->bytesperline);
	dbg_printk(_VBQ_DBG, " timestamp: sec(%ld), usec(%ld)\n", vb->ts.tv_sec, vb->ts.tv_usec);
	dbg_printk(_VBQ_DBG, " crop:      left(%d),   top(%d)\n", vb->save_effect.source_crop.top, vb->save_effect.source_crop.left);
	dbg_printk(_VBQ_DBG, "            width(%d), height(%d)\n", vb->save_effect.source_crop.width, vb->save_effect.source_crop.height);
	switch(vb->save_effect.movie_angle){
	case 0: dbg_printk(_VBQ_DBG, "            angle(0deg)\n");   break;
	case 1: dbg_printk(_VBQ_DBG, "            angle(90deg)\n");  break;
	case 2: dbg_printk(_VBQ_DBG, "            angle(180deg)\n"); break;
	case 3: dbg_printk(_VBQ_DBG, "            angle(270deg)\n"); break;
	}
	dbg_printk(_VBQ_DBG, " screen:    left(%d),   top(%d)\n", vb->save_effect.screen.top, vb->save_effect.screen.left);
	dbg_printk(_VBQ_DBG, "            width(%d), height(%d)\n", vb->save_effect.screen.width, vb->save_effect.screen.height);
	dbg_printk(_VBQ_DBG, "==============================================================\n");

	if(vb->ts.tv_sec == -1 || vb->ts.tv_usec == -1)
		retval = mp200_v4l2_core_vbq_cancel(q, vb);

	q->sequence ++;
	if(q->sequence == 0) q->sequence++;
	dev->efct.sequence = q->sequence;

    /*2006.08.15 ita.to measure period of irq disable.*/
    us = CalcMicroSec(GetCycles() - start);
    if (us > PERIOD_IRQ) {
        printk(KERN_INFO "V4L2(v4l2_core)140: long disable interrupt -- %dus\n",us);
    }
	spin_unlock_irqrestore(&dev->img_lock, flags);

	vb->state = STATE_PREPARED;
	return retval;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_sizecheck_pix
* FUNCTION : 
* RETURN   :  0: success
*          : -1: error
* NOTE     : This function registers information to video buffer.
* CREATE   : 06/07/20 H.Haguro
* UPDATE   : 
******************************************************************************/
static int
mp200_v4l2_core_vbq_sizecheck_pix(struct videobuf_buffer *vb)
{
	if((vb->width < IMAGE_WIDTH_MIN) ||
	   (vb->width > IMAGE_WIDTH_MAX) ||
	   (vb->width & IMAGE_WIDTH_STEP) != 0){
		printk("\n @v4l2: error! width(%3d) is incorrect.\n\n", vb->width);
		return -1;
	}
	if((vb->height < IMAGE_HEIGHT_MIN) ||
	   (vb->height > IMAGE_HEIGHT_MAX) ||
	   (vb->height & IMAGE_HEIGHT_STEP) != 0){
		printk("\n @v4l2: error! height(%3d) is incorrect.\n\n", vb->height);
		return -1;
	}
	if((vb->bytesperline < vb->width) ||
	   (vb->bytesperline > IMAGE_WIDTH_MAX)){
		printk("\n @v4l2: error! bytesperline(%3d) is incorrect.\n\n", vb->bytesperline);
		return -1;
	}
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_sizecheck_crop
* FUNCTION : 
* RETURN   :  0: success
*          : -1: error
* NOTE     : This function registers information to video buffer.
* CREATE   : 06/07/20 H.Haguro
* UPDATE   : 
******************************************************************************/
static int
mp200_v4l2_core_vbq_sizecheck_crop(struct videobuf_buffer *vb)
{
	if((vb->save_effect.source_crop.left < IMAGE_LEFT_MIN) ||
	   (vb->save_effect.source_crop.left > vb->width)      ||
	   (vb->save_effect.source_crop.left & IMAGE_LEFT_STEP) != 0){
		printk("\n @v4l2: error! source_crop.left(%3d) is incorrect.\n\n", vb->save_effect.source_crop.left);
		return -1;
	}
	if((vb->save_effect.source_crop.top < IMAGE_TOP_MIN) ||
	   (vb->save_effect.source_crop.top > vb->height)    ||
	   (vb->save_effect.source_crop.top & IMAGE_TOP_STEP) != 0){
		printk("\n @v4l2: error! source_crop.top(%3d) is incorrect.\n\n", vb->save_effect.source_crop.top);
		return -1;
	}

	if((vb->save_effect.source_crop.width < IMAGE_WIDTH_MIN) ||
	   (vb->save_effect.source_crop.width > vb->width)       ||
	   (vb->save_effect.source_crop.width & IMAGE_WIDTH_STEP) != 0){
		printk("\n @v4l2: error! source_crop.width(%3d) is incorrect.\n\n", vb->save_effect.source_crop.width);
		return -1;
	}

	if((vb->save_effect.source_crop.height < IMAGE_HEIGHT_MIN) ||
	   (vb->save_effect.source_crop.height > vb->height)       ||
	   (vb->save_effect.source_crop.height & IMAGE_HEIGHT_STEP) != 0){
		printk("\n @v4l2: error! source_crop.height(%3d) is incorrect.\n\n", vb->save_effect.source_crop.height);
		return -1;
	}

	if((vb->save_effect.source_crop.left + vb->save_effect.source_crop.width) > vb->width){
		printk("\n @v4l2: error! source_crop.left+width(%3d) is incorrect.\n\n", vb->save_effect.source_crop.left + vb->save_effect.source_crop.width);
		return -1;
	}
	if((vb->save_effect.source_crop.top + vb->save_effect.source_crop.height) > vb->height){
		printk("\n @v4l2: error! source_crop.top+height(%3d) is incorrect.\n\n", vb->save_effect.source_crop.top + vb->save_effect.source_crop.height);
		return -1;
	}
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_sizecheck
* FUNCTION : 
* RETURN   :  0: success
*          : -1: error
* NOTE     : This function registers information to video buffer.
* CREATE   : 06/06/16 H.Nagashima
* UPDATE   : 
******************************************************************************/
static int
mp200_v4l2_core_vbq_sizecheck(struct videobuf_queue *q, struct videobuf_buffer *vb)
{
	unsigned int src_size, dst_size, dst_pos;
	unsigned int src_org_size_w, src_org_size_h;

	/* pix size check */
	if(mp200_v4l2_core_vbq_sizecheck_pix(vb)){
		return -1;
	}

	/* crop size check */
	if(mp200_v4l2_core_vbq_sizecheck_crop(vb)){
		return -1;
	}

	/* angle check */
	if(vb->save_effect.movie_angle < 0 || vb->save_effect.movie_angle > 3){
		printk("\n @v4l2: error! movie_angle(%d) is incorrect.\n\n", vb->save_effect.movie_angle);
		return -1;
	}
	
	/* address check */
	if((vb->PhysAddr_Y & IMAGE_PADDR_STEP) != 0  || vb->PhysAddr_Y == IMAGE_PADDR_NULL){
		printk("\n @v4l2: error! PhysAddrY(0x%lx) is incorrect.\n\n", vb->PhysAddr_Y);
		return -1;
	}
	if((vb->PhysAddr_UV & IMAGE_PADDR_STEP) != 0 || vb->PhysAddr_UV == IMAGE_PADDR_NULL){
		printk("\n @v4l2: error! PhysAddrUV(0x%lx) is incorrect.\n\n", vb->PhysAddr_UV);
		return -1;
	}
	
	/* revise LCD output size */
	if(vb->save_effect.screen.left > FRONT_WIDTH_V || vb->save_effect.screen.top > FRONT_HEIGHT_V ){
		vb->save_effect.screen.width  = 0;
		vb->save_effect.screen.height = 0;
		return 0;
	}
	
	src_org_size_w = vb->save_effect.source_crop.left + vb->save_effect.source_crop.width;
	src_org_size_h = vb->save_effect.source_crop.top  + vb->save_effect.source_crop.height;
	
	if(vb->save_effect.screen.left + vb->save_effect.screen.width  > FRONT_WIDTH_V ){
		dst_pos     = vb->save_effect.screen.left;
		dst_size    = vb->save_effect.screen.width;
		if(vb->save_effect.movie_angle == RO_MODE_ROT_MOD_0 || vb->save_effect.movie_angle == RO_MODE_ROT_MOD_180)
		{
			src_size    = vb->save_effect.source_crop.width;
			vb->save_effect.screen.width      = FRONT_WIDTH_V - dst_pos;
			vb->save_effect.source_crop.width = vb->save_effect.screen.width * src_size / dst_size;
			if((vb->save_effect.source_crop.width      & IMAGE_WIDTH_STEP) == 0x1) vb->save_effect.source_crop.width -= 1;
			else if((vb->save_effect.source_crop.width & IMAGE_WIDTH_STEP) == 0x2) vb->save_effect.source_crop.width -= 2;
			else if((vb->save_effect.source_crop.width & IMAGE_WIDTH_STEP) == 0x3) vb->save_effect.source_crop.width -= 3;
			if(vb->save_effect.movie_angle == RO_MODE_ROT_MOD_180){
				vb->save_effect.source_crop.left = src_org_size_w - vb->save_effect.source_crop.width;
			}
		}else{
			src_size    = vb->save_effect.source_crop.height;
			vb->save_effect.screen.width       = FRONT_WIDTH_V - dst_pos;
			vb->save_effect.source_crop.height = vb->save_effect.screen.width * src_size / dst_size;
			if((vb->save_effect.source_crop.height      & IMAGE_WIDTH_STEP) == 0x1) vb->save_effect.source_crop.height -= 1;
			else if((vb->save_effect.source_crop.height & IMAGE_WIDTH_STEP) == 0x2) vb->save_effect.source_crop.height -= 2;
			else if((vb->save_effect.source_crop.height & IMAGE_WIDTH_STEP) == 0x3) vb->save_effect.source_crop.height -= 3;
			if(vb->save_effect.movie_angle == RO_MODE_ROT_MOD_90){
				vb->save_effect.source_crop.top = src_org_size_h - vb->save_effect.source_crop.height;
			}
		}
	}
	if(vb->save_effect.screen.top + vb->save_effect.screen.height  > FRONT_HEIGHT_V){
		dst_pos     = vb->save_effect.screen.top;
		dst_size    = vb->save_effect.screen.height;
		if(vb->save_effect.movie_angle == RO_MODE_ROT_MOD_0 || vb->save_effect.movie_angle == RO_MODE_ROT_MOD_180)
		{
			src_size    = vb->save_effect.source_crop.height;
			vb->save_effect.screen.height      = FRONT_HEIGHT_V - dst_pos;
			vb->save_effect.source_crop.height = vb->save_effect.screen.height * src_size / dst_size;
			if((vb->save_effect.source_crop.height      & IMAGE_WIDTH_STEP) == 0x1) vb->save_effect.source_crop.height -= 1;
			else if((vb->save_effect.source_crop.height & IMAGE_WIDTH_STEP) == 0x2) vb->save_effect.source_crop.height -= 2;
			else if((vb->save_effect.source_crop.height & IMAGE_WIDTH_STEP) == 0x3) vb->save_effect.source_crop.height -= 3;
			if(vb->save_effect.movie_angle == RO_MODE_ROT_MOD_180){
				vb->save_effect.source_crop.top = src_org_size_h - vb->save_effect.source_crop.height;
			}
		}else{
			src_size    = vb->save_effect.source_crop.width;
			vb->save_effect.screen.height       = FRONT_HEIGHT_V - dst_pos;
			vb->save_effect.source_crop.width   = vb->save_effect.screen.height * src_size / dst_size;
			if((vb->save_effect.source_crop.width      & IMAGE_WIDTH_STEP) == 0x1) vb->save_effect.source_crop.width -= 1;
			else if((vb->save_effect.source_crop.width & IMAGE_WIDTH_STEP) == 0x2) vb->save_effect.source_crop.width -= 2;
			else if((vb->save_effect.source_crop.width & IMAGE_WIDTH_STEP) == 0x3) vb->save_effect.source_crop.width -= 3;
			if(vb->save_effect.movie_angle == RO_MODE_ROT_MOD_270){
				vb->save_effect.source_crop.left = src_org_size_w - vb->save_effect.source_crop.width;
			}
		}
	}
	
	/* IMG not support dest size check */
	if(vb->save_effect.screen.width < DEST_WIDTH_MIN || vb->save_effect.screen.height < DEST_HEIGHT_MIN ){
		vb->save_effect.screen.width  = 0;
		vb->save_effect.screen.height = 0;
	}
	else if(vb->save_effect.source_crop.height < IMAGE_HEIGHT_MIN || vb->save_effect.source_crop.width < IMAGE_WIDTH_MIN ){
		printk("\n @v4l2: error! source_crop.width(%3d) or source_crop.height(%3d) is incorrect.\n\n",
		        vb->save_effect.source_crop.width, vb->save_effect.source_crop.height);
		return -1;
	}
	else
	{
		if((vb->save_effect.screen.width  & DEST_IMAGE_ALIGN) != 0) vb->save_effect.screen.width -= 1;
		if((vb->save_effect.screen.height & DEST_IMAGE_ALIGN) != 0) vb->save_effect.screen.height -= 1;
	}
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_cancel
* FUNCTION : 
* RETURN   : 0: success
*          : 1: stop timer
* NOTE     : This function registers information to video buffer.
* CREATE   : 
* UPDATE   : 06/07/03 M.Haguro
******************************************************************************/
static int
mp200_v4l2_core_vbq_cancel(struct videobuf_queue *q, struct videobuf_buffer *vb)
{
	struct mp200_v4l2_device *dev = q->priv_data;
	unsigned long flags, flags2;
	unsigned int retval = 0;
    int start, us;   /*2006.08.15 ita.to measure period of irq disable.*/
    int start2, us2;   /*2006.08.15 ita.to measure period of irq disable.*/

	dbg_lock(&dev->timer_lock, "vbq_cancel: timer_lock\n");
	spin_lock_irqsave(&dev->timer_lock, flags);
    start = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
	if(dev->waiting){
		dbg_lock(&dev->do_timer_lock, "vbq_cancel: do_timer_lock\n");
		spin_lock_irqsave(&dev->do_timer_lock, flags2);
        start2 = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
		if(dev->do_timer == STATE_RUN_TIMER){
			del_timer_sync(&dev->timer.wait_timer);
			dev->mixing = NULL;
			dev->waiting = NULL;
			dev->do_timer = STATE_STOP_TIMER;
			retval = 1;
		}
        /*2006.08.15 ita.to measure period of irq disable.*/
        us2 = CalcMicroSec(GetCycles() - start2);
        if (us2 > PERIOD_IRQ) {
            printk(KERN_INFO "V4L2(v4l2_core)150: long disable interrupt -- %dus\n",us2);
        }
		spin_unlock_irqrestore(&dev->do_timer_lock, flags2);
	}
    /*2006.08.15 ita.to measure period of irq disable.*/
    us = CalcMicroSec(GetCycles() - start);
    if (us > PERIOD_IRQ) {
        printk(KERN_INFO "V4L2(v4l2_core)160: long disable interrupt -- %dus\n",us);
    }
	spin_unlock_irqrestore(&dev->timer_lock, flags);

	return retval;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_queue
* FUNCTION : 
* RETURN   : -
* NOTE     : This function executes video buffer information.
* CREATE   : 
* UPDATE   : 06/06/16 M.Haguro
******************************************************************************/
static void
mp200_v4l2_core_vbq_queue(struct videobuf_queue *q, struct videobuf_buffer *vb)
{
	struct mp200_v4l2_device *dev = q->priv_data;
	unsigned long flags, flags2;
    int start2, us2;   /*2006.08.15 ita.to measure period of irq disable.*/
    int start, us;   /*2006.08.15 ita.to measure period of irq disable.*/

	dbg_lock(&dev->timer_lock, "vbq_queue: timer_lock\n");
	dbg_lock(&dev->mix_lock, "vbq_queue: mix_lock\n");
	spin_lock_irqsave(&dev->timer_lock, flags);
    start = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
	spin_lock_irqsave(&dev->mix_lock, flags2);
    start2 = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
	
	if(dev->waiting == NULL && dev->mixing == NULL && dev->suspend == 0){
		vb->state = STATE_QUEUED;
		dbg_printk(_VBQ_DBG, "vbq_queue: (%p) ->prev(%p) ->next(%p) ->state(%d)\n", vb, vb->stream.prev, vb->stream.next, vb->state);

		dev->mixing = dev->streaming;

		if(vb->ts.tv_sec == -1 || vb->ts.tv_usec == -1){
			/* call lcd driver function. */
			dbg_printk(_WKQ_DBG, "vbq_queue: mk_call_lcd()\n");
			dev->timer.vb = vb;
			queue_work(dev->v4l2_workqueue, &dev->wk_lcd_call);
		}else{
			/* start timer */
			dbg_printk(_WKQ_DBG, "vbq_queue: add_timer()\n");
			dbg_lock(&dev->do_timer_lock, "vbq_queue: do_timer_lock\n");
			spin_lock_irqsave(&dev->do_timer_lock, flags);
            start = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
			mp200_v4l2_add_timer(dev, vb);
            /*2006.08.15 ita.to measure period of irq disable.*/
            us = CalcMicroSec(GetCycles() - start);
            if (us > PERIOD_IRQ) {
                printk(KERN_INFO "V4L2(v4l2_core)170: long disable interrupt -- %dus\n",us);
            }
			spin_unlock_irqrestore(&dev->do_timer_lock, flags);
		}
	}
    /*2006.08.15 ita.to measure period of irq disable.*/
    us2 = CalcMicroSec(GetCycles() - start2);
    if (us2 > PERIOD_IRQ) {
        printk(KERN_INFO "V4L2(v4l2_core)180: long disable interrupt -- %dus\n",us2);
    }
	spin_unlock_irqrestore(&dev->mix_lock, flags2);
    /*2006.08.15 ita.to measure period of irq disable.*/
    us = CalcMicroSec(GetCycles() - start);
    if (us > PERIOD_IRQ) {
        printk(KERN_INFO "V4L2(v4l2_core)190: long disable interrupt -- %dus\n",us);
    }
	spin_unlock_irqrestore(&dev->timer_lock, flags);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_complete
* FUNCTION : 
* RETURN   : -
* NOTE     : This function is called from interrupt context 
*          : when a IMG transferof a videobuf_buffer completes.
* CREATE   : 
* UPDATE   : 
******************************************************************************/
static void
mp200_v4l2_core_vbq_complete(void *arg1, void *arg)
{
	struct mp200_v4l2_device *dev = (struct mp200_v4l2_device *)arg1;
	struct videobuf_buffer *vb = (struct videobuf_buffer *)arg;

	dbg_lock(&dev->vbq_lock, "vbq_complete: vbq_lock\n");
	spin_lock(&dev->vbq_lock);

	vb->field_count = dev->field_count;
	dev->field_count += 2;

	mp200_v4l2_calc_adjusttime(dev);

	vb->state = STATE_DONE;
	if(dev->vb_old != 0){
		dev->vb_old->state = STATE_DQBUF_PERMIT;
		wake_up(&dev->vb_old->done);
		dbg_printk(_VBQ_DBG, "vbq_complete: (%p) ->prev(%p) ->next(%p) ->state(%d) :old (%p) ->state(%d) ->done(%p)\n",
					&vb->stream, vb->stream.prev, vb->stream.next, vb->state, dev->vb_old, dev->vb_old->state, &dev->vb_old->done);
	}
	else dbg_printk(_VBQ_DBG, "vbq_complete: (%p) ->prev(%p) ->next(%p) ->state(%d)\n", &vb->stream, vb->stream.prev, vb->stream.next, vb->state);
	wake_up(&vb->clear_done);
	dev->vb_old = vb;

	spin_unlock(&dev->vbq_lock);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_vbq_release
* FUNCTION : 
* RETURN   : -
* NOTE     : This function is called a videobuf_buffer release.
* CREATE   : 
* UPDATE   : 
******************************************************************************/
static void
mp200_v4l2_core_vbq_release(struct videobuf_queue *q, struct videobuf_buffer *vb)
{
	videobuf_waiton(vb, 0, 0);
	vb->state = STATE_NEEDS_INIT;
}


/* ------------------ system_call_ops ------------------------------------------- */
/*****************************************************************************
* MODULE   : mp200_v4l2_core_do_ioctl
* FUNCTION : 
* RETURN   :       0: success
*          : -EINVAL: Argument error
* NOTE     : this function is substance of the ioctl system call.
*          : @@ supported ioctl command
*          :    VIDIOC_S_FMT,       VIDIOC_G_FMT,       VIDIOC_S_CROP,
*          :    VIDIOC_G_CROP,      VIDIOC_S_CTRL,      VIDIOC_G_CTRL,
*          :    VIDIOC_REQBUFS,     VIDIOC_S_EFFECT,    VIDIOC_G_EFFECT,
*          :    VIDIOC_STREAMON,    VIDIOC_STREAMOFF,   VIDIOC_QBUF,
*          :    VIDIOC_DQBUF
*          : @@ not supported ioctl command
*          :    VIDIOC_ENUMINPUT,   VIDIOC_G_INPUT,     VIDIOC_S_INPUT,
*          :    VIDIOC_ENUM_FMT,    VIDIOC_TRY_FMT,     VIDIOC_QUERYCTRL,
*          :    VIDIOC_G_FBUF,      VIDIOC_S_FBUF,      VIDIOC_OVERLAY,
*          :    VIDIOC_ENUMSTD,     VIDIOC_G_STD,       VIDIOC_S_STD,
*          :    VIDIOC_QUERYSTD,    VIDIOC_G_AUDIO,     VIDIOC_S_AUDIO
*          :    VIDIOC_G_AUDOUT,    VIDIOC_S_AUDOUT,    VIDIOC_G_JPEGCOMP,
*          :    VIDIOC_S_JPEGCOMP,  VIDIOC_G_TUNER,     VIDIOC_S_TUNER,
*          :    VIDIOC_G_MODULATOR, VIDIOC_S_MODULATOR, VIDIOC_G_FREQUENCY,
*          :    VIDIOC_S_FREQUENCY, VIDIOC_QUERYCAP,    VIDIOC_ENUMOUTPUT,
*          :    VIDIOC_G_OUTPUT,    VIDIOC_S_OUTPUT,    VIDIOC_QUERYBUF
* CREATE   : 
* UPDATE   : 2006/06/16 M.Haguro
******************************************************************************/
static int
mp200_v4l2_core_do_ioctl(struct inode *inode, struct file *file, unsigned int cmd, 
		     void *arg)
{
	struct mp200_v4l2_fh *fh  = file->private_data;
	struct mp200_v4l2_device *dev = fh->dev;
	unsigned long flags;
	int err;

    int start, us;   /*2006.08.15 ita.to measure period of irq disable.*/

	switch (cmd) {
		case VIDIOC_ENUMINPUT:
		case VIDIOC_G_INPUT:
		case VIDIOC_S_INPUT:
		case VIDIOC_ENUM_FMT:
		case VIDIOC_TRY_FMT:
		case VIDIOC_QUERYCTRL:
		{
			return -EINVAL;
		}

		case VIDIOC_G_FMT:
		{
			struct v4l2_format *fmt = arg;

			/* get the current format */
			dbg_lock(&dev->img_lock, "do_ioctl: img_lock\n");
			spin_lock_irqsave(&dev->img_lock, flags);
			if(fmt->type == V4L2_BUF_TYPE_VIDEO_OUTPUT){
				fmt->fmt.pix = dev->pix;
			}
			else if(fmt->type == V4L2_BUF_TYPE_VIDEO_OVERLAY){
				fmt->fmt.win.w = dev->efct.screen;
			}
			else{
				spin_unlock_irqrestore(&dev->img_lock, flags);
				return -EINVAL;
			}
			spin_unlock_irqrestore(&dev->img_lock, flags);
			return 0;
		}

		case VIDIOC_S_FMT:
		{
			struct v4l2_format *fmt = arg;

			/* set image info */
			dbg_lock(&dev->img_lock, "do_ioctl: img_lock\n");
			spin_lock_irqsave(&dev->img_lock, flags);
			if(fmt->type == V4L2_BUF_TYPE_VIDEO_OUTPUT){
//				dev->pix = fmt->pix;
				dev->pix.sizeimage    = sizeof(struct videobuf_buffer);
				dev->pix.width        = fmt->fmt.pix.width;
				dev->pix.height       = fmt->fmt.pix.height;
				dev->pix.pixelformat  = fmt->fmt.pix.pixelformat;
				dev->pix.bytesperline = fmt->fmt.pix.bytesperline;
			}
			/* set overlay info */
			else if(fmt->type == V4L2_BUF_TYPE_VIDEO_OVERLAY){
//				dev->efct.screen = fmt->fmt.win.w;
				dev->efct.screen.left   = fmt->fmt.win.w.left;
				dev->efct.screen.top    = fmt->fmt.win.w.top;
				dev->efct.screen.width  = fmt->fmt.win.w.width;
				dev->efct.screen.height = fmt->fmt.win.w.height;
			}
			else{
				spin_unlock_irqrestore(&dev->img_lock, flags);
				return -EINVAL;
			}
			spin_unlock_irqrestore(&dev->img_lock, flags);
			return 0;
		}

		case VIDIOC_G_CROP:
		{
			struct v4l2_crop *vc = arg;

			dbg_lock(&dev->img_lock, "do_ioctl: img_lock\n");
			spin_lock_irqsave(&dev->img_lock, flags);
			if(vc->type != V4L2_BUF_TYPE_VIDEO_OVERLAY){
				spin_unlock_irqrestore(&dev->img_lock, flags);
				return -EINVAL;
			}
			/* get crop data of original movie */
			vc->c.left   = dev->efct.source_crop.left;
			vc->c.top    = dev->efct.source_crop.top;
			vc->c.width  = dev->efct.source_crop.width;
			vc->c.height = dev->efct.source_crop.height;
			spin_unlock_irqrestore(&dev->img_lock, flags);
			return 0;
		}

		case VIDIOC_S_CROP:
		{
			struct v4l2_crop *vc = arg;

			/* set crop data of original movie */
			dbg_lock(&dev->img_lock, "do_ioctl: img_lock\n");
			spin_lock_irqsave(&dev->img_lock, flags);
			if(vc->type != V4L2_BUF_TYPE_VIDEO_OVERLAY){
				spin_unlock_irqrestore(&dev->img_lock, flags);
				return -EINVAL;
			}
			dev->efct.source_crop.left   = vc->c.left;
			dev->efct.source_crop.top    = vc->c.top;
			dev->efct.source_crop.width  = vc->c.width;
			dev->efct.source_crop.height = vc->c.height;
			spin_unlock_irqrestore(&dev->img_lock, flags);
			return 0;
		}

		case VIDIOC_G_CTRL:
		{
			struct v4l2_control *vc = arg;

			/* get current rotation angle */
			dbg_lock(&dev->img_lock, "do_ioctl: img_lock\n");
			spin_lock_irqsave(&dev->img_lock, flags);
			if(vc->id != V4L2_MOVIE_DISPLAY_ANGLE){
				spin_unlock_irqrestore(&dev->img_lock, flags);
				return -EINVAL;
			}
			vc->value = dev->efct.movie_angle;
			spin_unlock_irqrestore(&dev->img_lock, flags);
			return 0;
		}

		case VIDIOC_S_CTRL:
		{
			struct v4l2_control *vc = arg;

			/* set rotation angle */
			dbg_lock(&dev->img_lock, "do_ioctl: img_lock\n");
			spin_lock_irqsave(&dev->img_lock, flags);
			if(vc->id != V4L2_MOVIE_DISPLAY_ANGLE){
				spin_unlock_irqrestore(&dev->img_lock, flags);
				return -EINVAL;
			}
			dev->efct.movie_angle = vc->value;
			spin_unlock_irqrestore(&dev->img_lock, flags);
			return 0;
		}

		case VIDIOC_G_EFFECT:
		{
			struct v4l2_effect *vef = arg;

			/* get current effect infomation */
			dbg_lock(&dev->img_lock, "do_ioctl: img_lock\n");
			spin_lock_irqsave(&dev->img_lock, flags);
			vef->sequence           = dev->efct.sequence;
			vef->source_crop.left   = dev->efct.source_crop.left;
			vef->source_crop.top    = dev->efct.source_crop.top;
			vef->source_crop.width  = dev->efct.source_crop.width;
			vef->source_crop.height = dev->efct.source_crop.height;
			vef->movie_angle        = dev->efct.movie_angle;
			vef->screen.left        = dev->efct.screen.left;
			vef->screen.top         = dev->efct.screen.top;
			vef->screen.width       = dev->efct.screen.width;
			vef->screen.height      = dev->efct.screen.height;
			spin_unlock_irqrestore(&dev->img_lock, flags);
			return 0;
		}
		case VIDIOC_S_EFFECT:
		{
			struct v4l2_effect *vef = arg;

			/* set effect infomation */
			dbg_lock(&dev->img_lock, "do_ioctl: img_lock\n");
			spin_lock_irqsave(&dev->img_lock, flags);
			dev->efct.sequence           = 0;
			dev->efct.source_crop.left   = vef->source_crop.left;
			dev->efct.source_crop.top    = vef->source_crop.top;
			dev->efct.source_crop.width  = vef->source_crop.width;
			dev->efct.source_crop.height = vef->source_crop.height;
			dev->efct.movie_angle        = vef->movie_angle;
			dev->efct.screen.left        = vef->screen.left;
			dev->efct.screen.top         = vef->screen.top;
			dev->efct.screen.width       = vef->screen.width;
			dev->efct.screen.height      = vef->screen.height;
			spin_unlock_irqrestore(&dev->img_lock, flags);
			return 0;
		}

		case VIDIOC_QUERYCAP:
		case VIDIOC_G_FBUF: /* Get the frame buffer parameters */
		case VIDIOC_S_FBUF: /* set the frame buffer parameters */
		case VIDIOC_OVERLAY:
		{
			/* not supported */
			return -EINVAL;
		}

		case VIDIOC_REQBUFS:
		{
			return videobuf_reqbufs(&fh->vbq, arg);
		}

		case VIDIOC_QUERYBUF:
		{
			/* not supported */
			return -EINVAL;
		}

		case VIDIOC_QBUF:
		{
			return videobuf_qbuf(&fh->vbq, arg);
		}

		case VIDIOC_DQBUF:
		{
			return videobuf_dqbuf(&fh->vbq, arg, file->f_flags & O_NONBLOCK);
		}

		case VIDIOC_STREAMON:
		{
			dbg_lock(&dev->img_lock, "do_ioctl: img_lock\n");
			spin_lock_irqsave(&dev->img_lock, flags);
			if (dev->streaming) {
				spin_unlock_irqrestore(&dev->img_lock, flags);
				return -EBUSY;
			}
			else {
				dev->streaming = fh;
			}
			dev->vb_old = 0;

			/* video-buffer-queue sequence nomber initialize */
			fh->vbq.sequence = 0;
			spin_unlock_irqrestore(&dev->img_lock, flags);

			/* init timer */
			mp200_v4l2_init_timer(dev);

			return videobuf_streamon(&fh->vbq);
		}

		case VIDIOC_STREAMOFF:
		{
			dbg_lock(&dev->img_lock, "do_ioctl: img_lock\n");
			spin_lock_irqsave(&dev->img_lock, flags);
			dev->vb_old = NULL;
			spin_unlock_irqrestore(&dev->img_lock, flags);

			/* streamoff */
			err = videobuf_streamoff(&fh->vbq);
			if (err < 0)
				return err;

			/* stop timer */
			dbg_lock(&dev->timer_lock, "do_ioctl: timer_lock\n");
			spin_lock_irqsave(&dev->timer_lock, flags);
            start = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
			if (dev->waiting == fh) {
				if(dev->do_timer == STATE_RUN_TIMER)
					del_timer_sync(&dev->timer.wait_timer);
				dev->do_timer = STATE_STOP_TIMER;
				dev->waiting = NULL;
			}
			dev->vb_old = NULL;
            /*2006.08.15 ita.to measure period of irq disable.*/
            us = CalcMicroSec(GetCycles() - start);
            if (us > PERIOD_IRQ) {
                printk(KERN_INFO "V4L2(v4l2_core)200: long disable interrupt -- %dus\n",us);
            }
			spin_unlock_irqrestore(&dev->timer_lock, flags);

			dbg_lock(&dev->mix_lock, "do_ioctl: mix_lock\n");
			spin_lock_irqsave(&dev->mix_lock, flags);
			dev->mixing = NULL;
			spin_unlock_irqrestore(&dev->mix_lock, flags);

			dbg_lock(&dev->img_lock, "do_ioctl: img_lock\n");
			spin_lock_irqsave(&dev->img_lock, flags);
            start = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
			if (dev->streaming == fh) {
				/* call lcd driver function. */
				queue_work(dev->v4l2_workqueue, &dev->wk_stop);
				dev->streaming = NULL;
				dev->vb_old = NULL;
			}
            /*2006.08.15 ita.to measure period of irq disable.*/
            us = CalcMicroSec(GetCycles() - start);
            if (us > PERIOD_IRQ) {
                printk(KERN_INFO "V4L2(v4l2_core)210: long disable interrupt -- %dus\n",us);
            }
			spin_unlock_irqrestore(&dev->img_lock, flags);
			return 0;
		}

		case VIDIOC_ENUMSTD:
		case VIDIOC_G_STD:
		case VIDIOC_S_STD:
		case VIDIOC_QUERYSTD:
		{
			/* we don't have an analog video standard, 
			 * so we don't need to implement these ioctls.
			 */
			 return -EINVAL;
		}
		case VIDIOC_G_AUDIO:
		case VIDIOC_S_AUDIO:
		case VIDIOC_G_AUDOUT:
		case VIDIOC_S_AUDOUT:
		{
			/* we don't have any audio inputs or outputs */
			return -EINVAL;
		}

		case VIDIOC_G_JPEGCOMP:
		case VIDIOC_S_JPEGCOMP:
		{
			/* JPEG compression is not supported */
			return -EINVAL;
		}

		case VIDIOC_G_TUNER:
		case VIDIOC_S_TUNER:
		case VIDIOC_G_MODULATOR:
		case VIDIOC_S_MODULATOR:
		case VIDIOC_G_FREQUENCY:
		case VIDIOC_S_FREQUENCY:
		{
			/* we don't have a tuner or modulator */
			return -EINVAL;
		}

		case VIDIOC_ENUMOUTPUT:
		case VIDIOC_G_OUTPUT:
		case VIDIOC_S_OUTPUT:
		{
			/* not supported
			 * output device is LCD only
			 */
			return -EINVAL;
		}

		default:
		{
			/* unrecognized ioctl */
			return -ENOIOCTLCMD;
		}
	}
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_ioctl
* FUNCTION : a stub function of ioctl
* RETURN   :       0: success
*          : -EINVAL: Argument error
* NOTE     : 
* CREATE   : 
* UPDATE   : 
******************************************************************************/
static int
mp200_v4l2_core_ioctl(struct inode *inode, struct file *file, unsigned int cmd, 
		  unsigned long arg)
{
	return video_usercopy(inode, file, cmd, arg, mp200_v4l2_core_do_ioctl);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_poll
* FUNCTION : a system call function of poll & select
* RETURN   :       0: success
*          : -EINVAL: Argument error
* NOTE     : 
* CREATE   : 06/06/09 Y.Hara
* UPDATE   : 06/06/13 M.Haguro
******************************************************************************/
static unsigned int
mp200_v4l2_core_poll(struct file *file, struct poll_table_struct *wait)
{
	struct mp200_v4l2_fh *fh = file->private_data;

	return videobuf_poll_stream(file, &fh->vbq, wait);
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_release
* FUNCTION : v4l2 driver's close processing function.
* RETURN   : 0: success
* NOTE     : 
* CREATE   : 
* UPDATE   : 06/06/14 M.Haguro
******************************************************************************/
static int
mp200_v4l2_core_release(struct inode *inode, struct file *file)
{
	struct mp200_v4l2_fh *fh = file->private_data;
	struct mp200_v4l2_device *dev = fh->dev;
    int start, us;   /*2006.08.15 ita.to measure period of irq disable.*/
	unsigned long flags;
	int err;

	dbg_lock(&dev->img_lock, "release: img_lock\n");
	spin_lock_irqsave(&dev->img_lock, flags);
    start = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
	if (dev->streaming == fh) {
		dev->vb_old = NULL;
        /*2006.08.15 ita.to measure period of irq disable.*/
        us = CalcMicroSec(GetCycles() - start);
        if (us > PERIOD_IRQ) {
            printk(KERN_INFO "V4L2(v4l2_core)220: long disable interrupt -- %dus\n",us);
        }

		spin_unlock_irqrestore(&dev->img_lock, flags);

		/* streamoff */
		err = videobuf_streamoff(&fh->vbq);
		if (err < 0)
			return err;

		/* call lcd driver function. */
		queue_work(dev->v4l2_workqueue, &dev->wk_stop);

		dbg_lock(&dev->img_lock, "release: img_lock\n");
		spin_lock_irqsave(&dev->img_lock, flags);
        start = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
	}
    /*2006.08.15 ita.to measure period of irq disable.*/
    us = CalcMicroSec(GetCycles() - start);
    if (us > PERIOD_IRQ) {
        printk(KERN_INFO "V4L2(v4l2_core)230: long disable interrupt -- %dus\n",us);
    }
	spin_unlock_irqrestore(&dev->img_lock, flags);

	/* stop timer */
	dbg_lock(&dev->timer_lock, "release: timer_lock\n");
	spin_lock_irqsave(&dev->timer_lock, flags);
    start = GetCycles();  /*2006.08.15 ita.to measure period of irq disable.*/
	if (dev->waiting == fh) {
		if(dev->do_timer == STATE_RUN_TIMER)
			del_timer_sync(&dev->timer.wait_timer);
		dev->do_timer = STATE_STOP_TIMER;
		dev->waiting = NULL;
	}
    /*2006.08.15 ita.to measure period of irq disable.*/
    us = CalcMicroSec(GetCycles() - start);
    if (us > PERIOD_IRQ) {
        printk(KERN_INFO "V4L2(v4l2_core)240: long disable interrupt -- %dus\n",us);
    }
	spin_unlock_irqrestore(&dev->timer_lock, flags);

	dbg_lock(&dev->mix_lock, "release: mix_lock\n");
	spin_lock_irqsave(&dev->mix_lock, flags);
	dev->mixing = NULL;
	spin_unlock_irqrestore(&dev->mix_lock, flags);

	dbg_lock(&dev->img_lock, "release: img_lock\n");
	spin_lock_irqsave(&dev->img_lock, flags);
	if (dev->streaming == fh) {
		dev->vb_old = NULL;
		dev->streaming = NULL;
	}
	spin_unlock_irqrestore(&dev->img_lock, flags);

	if (fh->vbq.read_buf) {
		mp200_v4l2_core_vbq_release(&fh->vbq, fh->vbq.read_buf);
		kfree(fh->vbq.read_buf);
	}

	err = videobuf_mmap_free(&fh->vbq);
	if(err) printk("\n @v4l2: release: videobuf_free failed.\n\n");

	file->private_data = NULL;
	kfree(fh);

	/* workqueue uninitialize */
	destroy_workqueue(dev->v4l2_workqueue);

	dev->vb_old = NULL;
	memset(&dev->pix,      0, sizeof(dev->pix));
	memset(&dev->efct,     0, sizeof(dev->efct));
	memset(&dev->rot_data, 0, sizeof(dev->rot_data));

	dev->active = 0;
	return 0;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_open
* FUNCTION : v4l2 driver's open processing function.
* RETURN   :       0: success
*          : -ENODEV: corresponding device doesn't exist.
*          : -ENOMEM: memory that can be used for the kernel is insufficient.
*          : -EPERM : no privilege in the call origin.
* NOTE     : 
* CREATE   : 
* UPDATE   : 06/06/14 M.Haguro
******************************************************************************/
static int
mp200_v4l2_core_open(struct inode *inode, struct file *file)
{
	int minor = iminor(inode);
	struct mp200_v4l2_device *dev = mp200_dev;
	struct mp200_v4l2_fh *fh;
	unsigned long flags;

	if (!dev || !dev->vfd || (dev->vfd->minor != minor))
		return -ENODEV;

	/* allocate per-filehandle data */
	fh = kmalloc(sizeof(*fh), GFP_KERNEL);

	if (NULL == fh)
		return -ENOMEM;
	file->private_data = fh;
	fh->dev = dev;
	fh->type = V4L2_BUF_TYPE_VIDEO_OUTPUT;

	dbg_lock(&dev->img_lock, "open: img_lock\n");
	spin_lock_irqsave(&dev->img_lock, flags);
	if (dev->active == 1) {
		printk (KERN_ERR DEV_NAME ": v4l2 device Active\n");
		spin_unlock_irqrestore(&dev->img_lock, flags);
		kfree(fh);
		return -EPERM;
	}
	dev->active = 1;
	spin_unlock_irqrestore(&dev->img_lock, flags);

	videobuf_queue_init(&fh->vbq, &dev->vbq_ops, NULL, &dev->vbq_lock,
		fh->type, V4L2_FIELD_NONE, sizeof(struct videobuf_buffer), fh->dev);

	/* workqueue initialize */
	dev->v4l2_workqueue = create_singlethread_workqueue("v4l2");
	INIT_WORK(&dev->wk_timeout_bottom_half, mp200_v4l2_timeout_bottom_half_do, NULL);
	INIT_WORK(&dev->wk_reschedule, mp200_v4l2_reschedule_do, NULL);
	INIT_WORK(&dev->wk_rot_callback_bottom_half, mp200_v4l2_rot_callback_bottom_half_do, NULL);
	INIT_WORK(&dev->wk_lcd_call, mp200_v4l2_lcd_call_do, NULL);
	INIT_WORK(&dev->wk_stop, mp200_v4l2_stop_do, NULL);

	/* video-buffer-queue index & sequence nomber initialize */
	fh->vbq.index_max = 0;
	fh->vbq.sequence  = 0;

	return 0;
}

#ifdef CONFIG_DPM
/*****************************************************************************
* MODULE   : mp200_v4l2_core_suspend
* FUNCTION : callback function when to suspend.
* RETURN   : 
* NOTE     : unsupport
* CREATE   : 
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_core_suspend(struct device *dev, u32 state, u32 level)
{
	struct mp200_v4l2_device *vdev = dev_get_drvdata(dev);
	unsigned long flags;
	int ret = 0;

	dbg_lock(&vdev->img_lock, "suspend: img_lock\n");
	spin_lock_irqsave(&vdev->img_lock, flags);
	switch (level) {
		case SUSPEND_DISABLE:
		{
			if(vdev->streaming){
				if(vdev->waiting){
					if(vdev->do_timer == STATE_RUN_TIMER){
						dbg_printk(_DPM_DBG, "suspend: del_timer\n");
						del_timer_sync(&vdev->timer.wait_timer);
					}
				}
			}
			vdev->suspend = 1;
			break;
		}
		case SUSPEND_SAVE_STATE:
		case SUSPEND_POWER_DOWN:
		default:
			break;
	}

	spin_unlock_irqrestore(&vdev->img_lock, flags);
	return ret;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_resume
* FUNCTION : callback function when to resume.
* RETURN   : 
* NOTE     : unsupport
* CREATE   : 
* UPDATE   : 
******************************************************************************/
static int mp200_v4l2_core_resume(struct device *dev, u32 level)
{
	struct mp200_v4l2_device *vdev = dev_get_drvdata(dev);
	unsigned long flags;
	int ret = 0;

	dbg_lock(&vdev->img_lock, "resume: img_lock\n");
	spin_lock_irqsave(&vdev->img_lock, flags);
	switch (level) {
		case RESUME_POWER_ON:
			break;
		case RESUME_RESTORE_STATE:
			if(!vdev->suspend)
				break;
		case RESUME_ENABLE:
		{
			if(vdev->streaming){
				if(vdev->waiting){
					if(vdev->do_timer == STATE_RUN_TIMER){
						dbg_printk(_DPM_DBG, "resume: add_timer\n");
						add_timer(&vdev->timer.wait_timer);
					}
				}
			}
			vdev->suspend = 0;
			break;
		}
		default:
			break;
	}

	spin_unlock_irqrestore(&vdev->img_lock, flags);
	return ret;
}
#endif	/* CONFIG_DPM */


/*****************************************************************************
* MODULE   : mp200_v4l2_core_cleanup
* FUNCTION : cleanup module
* RETURN   :       0: success
*          : -ENODEV: corresponding device doesn't exist.
* NOTE     : v4l2 driver cleanup function, when to driver ends.
* CREATE   : 
* UPDATE   : 
******************************************************************************/
void
mp200_v4l2_core_cleanup(void)
{
	struct mp200_v4l2_device *dev = mp200_dev;
	struct video_device *vfd;

	if (!dev)
		return;

	vfd = dev->vfd;
	if (vfd) {
		if (vfd->minor == -1) {
			/* The device never got registered, so release the 
			** video_device struct directly
			*/
			video_device_release(vfd);
		} else {
			/* The unregister function will release the video_device
			** struct as well as unregistering it.
			*/
			video_unregister_device(vfd);
			driver_unregister(&mp200_v4l2_core_driver);
			platform_device_unregister(&mp200_v4l2_core_device);
		}
		dev->vfd = NULL;
	}
	kfree(dev);
	mp200_dev = NULL;

	return;
}


/*****************************************************************************
* MODULE   : mp200_v4l2_core_init
* FUNCTION : initialize module
* RETURN   :       0: success
*          : -ENODEV: corresponding device doesn't exist.
* NOTE     : v4l2 driver initialize function, when to driver starts.
* CREATE   : 
* UPDATE   : 
******************************************************************************/
int __init 
mp200_v4l2_core_init(void)
{
	struct mp200_v4l2_device *dev;
	struct video_device *vfd;

	dev = kmalloc(sizeof(struct mp200_v4l2_device), GFP_KERNEL);
	if (!dev) {
		printk(KERN_ERR DEV_NAME ": could not allocate memory\n");
		goto init_error;
	}
	memset(dev, 0, sizeof(struct mp200_v4l2_device));
	
	/* Save the pointer to device in a global variable */
	mp200_dev = dev;

	/* initialize the video_device struct */
	vfd = dev->vfd = video_device_alloc();
	if (!vfd) {
		printk(KERN_ERR DEV_NAME 
			": could not allocate video device struct\n");
		goto init_error;
	}
	
 	vfd->release = video_device_release;

 	strlcpy(vfd->name, DEV_NAME, sizeof(vfd->name));
 	vfd->type = VID_TYPE_CAPTURE | VID_TYPE_OVERLAY | VID_TYPE_CHROMAKEY;
 	
 	/* need to register for a VID_HARDWARE_* ID in videodev.h */
 	vfd->hardware = 0;
 	vfd->fops     = &mp200_v4l2_core_fops;
 	video_set_drvdata(vfd, dev);
 	vfd->minor    = -1;

	/* initialize the spinlock used to serialize access to the image 
	 * parameters
	 */
	spin_lock_init(&dev->img_lock);
	spin_lock_init(&dev->mix_lock);
	spin_lock_init(&dev->rot_lock);
	spin_lock_init(&dev->timer_lock);
	spin_lock_init(&dev->do_timer_lock);

	/* initialize the kernel timer */
	mp200_v4l2_init_timer(dev);

	/* initialize the videobuf queue ops */
	dev->vbq_ops.buf_setup   = mp200_v4l2_core_vbq_setup;
	dev->vbq_ops.buf_prepare = mp200_v4l2_core_vbq_prepare;
	dev->vbq_ops.buf_queue   = mp200_v4l2_core_vbq_queue;
	dev->vbq_ops.buf_release = mp200_v4l2_core_vbq_release;
	dev->vbq_ops.buf_cancel  = mp200_v4l2_core_vbq_cancel;

	dev_set_drvdata(&mp200_v4l2_core_device.dev, (void *)dev);
	if (platform_device_register(&mp200_v4l2_core_device) < 0) {
		printk(KERN_ERR DEV_NAME
			": could not register platform_device\n");
		goto init_error;
	}

	if (driver_register(&mp200_v4l2_core_driver) < 0) {
		printk(KERN_ERR DEV_NAME
			": could not register driver\n");
		platform_device_unregister(&mp200_v4l2_core_device);
		goto init_error;
        }
	if (video_register_device(vfd, VFL_TYPE_GRABBER, video_nr) < 0) {
		printk(KERN_ERR DEV_NAME 
			": could not register Video for Linux device\n");
		platform_device_unregister(&mp200_v4l2_core_device);
		driver_unregister(&mp200_v4l2_core_driver);
		goto init_error;
	}

	printk(KERN_INFO DEV_NAME 
		": registered device video%d [v4l2]\n", vfd->minor);
	return 0;

init_error:
	mp200_v4l2_core_cleanup();
	return -ENODEV;
}


MODULE_AUTHOR("NEC Electronics Corporation.");
MODULE_DESCRIPTION("MP201 Video for Linux2 driver");
MODULE_LICENSE("GPL");
module_param(video_nr, int, 0);
MODULE_PARM_DESC(video_nr, "Minor number for video device (-1 ==> auto assign)");

module_init(mp200_v4l2_core_init);
module_exit(mp200_v4l2_core_cleanup);

EXPORT_SYMBOL_GPL(mp200_v4l2_core_rot_callback);
EXPORT_SYMBOL_GPL(mp200_v4l2_core_lcd_callback);


