/*
 * File Name       : drivers/media/video/omap/mp200_v4l2_core.h
 * Function        : Video for Linux driver for MP201 driver core
 * Release Version :
 * Release Date    :
 *
 * Copyright (C) NEC Electronics Corporation 2006
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef MP200_V4L2_CORE__H
#define MP200_V4L2_CORE__H


#include <media/video-buf.h>
#include <asm/scatterlist.h>

#include "mp200_v4l2_rot.h"
#include "mp200_v4l2_time.h"


struct mp200_v4l2_fh;
struct mp200_v4l2_device;


/*===============================================================*/
/* per-device data structure                                     */
/*===============================================================*/
struct mp200_v4l2_device {
	struct device             dev;
	struct video_device      *vfd;
	
	spinlock_t                vbq_lock;     /* spinlock for videobuf queues */
	struct videobuf_queue_ops vbq_ops;      /* videobuf queue operations    */
	unsigned long             field_count;  /* field counter for videobuf_buffer */

	struct videobuf_buffer   *vb_old;

	/* v4l2_workqueue defines the v4l2 driver to LCD driver or ROT driver
         * working task.
	 */
	struct workqueue_struct  *v4l2_workqueue;
	struct work_struct        wk_timeout_bottom_half;
	struct work_struct        wk_reschedule;
	struct work_struct        wk_rot_callback_bottom_half;
	struct work_struct        wk_lcd_call;
	struct work_struct        wk_stop;

	/* The img_lock is used to serialize access to the image parameters for 
	 * overlay and capture.  Need to use spin_lock_irq when writing to the 
	 * reading, streaming, and previewing parameters.  A regular spin_lock 
	 * will suffice for all other cases.
	 */
	spinlock_t                img_lock;

 	/* We allow reading from at most one filehandle at a time.
 	 * non-NULL means reading is in progress.
 	 */
 	struct mp200_v4l2_fh     *reading;

 	/* We allow streaming from at most one filehandle at a time.
 	 * non-NULL means streaming is in progress.
 	 */
	struct mp200_v4l2_fh     *streaming;

 	/* We allow waiting from at most one filehandle at a time.
 	 * non-NULL means waiting is in progress.
 	 */
 	struct mp200_v4l2_fh     *waiting;

	/* revise time data */
	spinlock_t		  timer_lock;
	struct mp200_v4l2_time    timer;

	spinlock_t		  do_timer_lock;
	int                       do_timer;

	/* do mixing image */
	spinlock_t                mix_lock;
 	struct mp200_v4l2_fh     *mixing;

	/* temporaly rotation data */
	spinlock_t                rot_lock; 
	struct mp200_v4l2_rot     rot_data;

	/* pix defines the size and pixel format of the image captured by the
	 * sensor.  This also defines the size of the framebuffers.  The
	 * same pool of framebuffers is used for video capture and video
	 * overlay.  These parameters are set/queried by the
	 * VIDIOC_S_FMT/VIDIOC_G_FMT ioctls with a CAPTURE buffer type.
	 */
	struct v4l2_pix_format    pix;

	/* saving effet data */
	struct v4l2_effect        efct;

	char                      suspend;
	char                      active;
};


/*===============================================================*/
/* per-filehandle data structure                                 */
/*===============================================================*/
struct mp200_v4l2_fh {
	struct mp200_v4l2_device *dev;
	enum   v4l2_buf_type      type;
	struct videobuf_queue     vbq;
};


/*===============================================================*/
/* V4L2 streaming state flag. (need to LCD request)              */
/*===============================================================*/
#define STREAM_STOP         0
#define STREAM_PLAYBACK     1
#define STREAM_PLAYBACK_ROT 2


/*===============================================================*/
/* use image sizes check.                                        */
/*===============================================================*/
#define IMAGE_WIDTH_MIN                      8
#define IMAGE_WIDTH_MAX                    480 // 4092
#define IMAGE_WIDTH_STEP                   0x3

#define IMAGE_HEIGHT_MIN                     8
#define IMAGE_HEIGHT_MAX                   480 // 4092
#define IMAGE_HEIGHT_STEP                  0x3

#define IMAGE_LEFT_MIN                       0
#define IMAGE_LEFT_MAX         IMAGE_WIDTH_MAX
#define IMAGE_LEFT_STEP                    0x1
#define IMAGE_TOP_MIN                        0
#define IMAGE_TOP_MAX         IMAGE_HEIGHT_MAX
#define IMAGE_TOP_STEP                     0x1

#define IMAGE_PADDR_NULL                     0
#define IMAGE_PADDR_STEP                   0x3

#define DEST_WIDTH_MIN                      16
#define DEST_HEIGHT_MIN                     16
#define DEST_IMAGE_ALIGN                   0x1


/*===============================================================*/
/* device name definetion                                        */
/*===============================================================*/
#define DEV_NAME "mp200_v4l2"


#endif /* MP200_V4L2_CORE__H */
