/*
 *  File Name       : mp200_cam.c
 *  Function        : CAMERA I/F Driver
 *  Release Version : Ver 1.00
 *  Release Date    : 2006/03/24
 *
 *  Copyright (C) NEC Electronics Corporation 2005
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */
#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/vmalloc.h>
#include <linux/slab.h>
#include <linux/proc_fs.h>
#include <linux/ctype.h>
#include <linux/pagemap.h>
#include <linux/videodev.h>
#include <linux/delay.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/moduleparam.h>
#include <linux/cdev.h>
#include <asm/irq.h>
#include <asm/io.h>
#include <asm/hardware.h>
#include <asm/semaphore.h>
#include <asm/uaccess.h>
#include <asm/arch/gpiocore.h>
#include <asm/arch/pmu.h>
#include <asm/arch/mp200_cam.h>
#include "mp200_cam.h"

/* --------------------------------------------- */
/* #undef	MP200_CAM_DEBUG */
#if defined(MP200_CAM_DEBUG)
#define FNC_ENTRY \
	if (debug == 1 || debug >= 9) {	\
		printk(KERN_DEBUG "entry:%s\n", __FUNCTION__); \
	}
#define FNC_EXIT \
	if (debug == 1 || debug >= 9) {	\
		printk(KERN_DEBUG "exit:%s:%d\n", __FUNCTION__ , __LINE__); \
	}
#define d0b(fmt, args...)	\
		printk(KERN_DEBUG "%s:%d: " fmt , __FUNCTION__ , __LINE__ , \
		       ## args);
static int debug = 0;
static int clockoff	= 1;
#else
#define FNC_ENTRY
#define FNC_EXIT
#define d0b(fmt, args...)
#endif
#define err(format, arg...) printk(KERN_ERR format , ## arg)
#define info(format, arg...) printk(KERN_INFO format , ## arg)
#define warn(format, arg...) printk(KERN_WARNING format , ## arg)
#define emerg(format, arg...) printk(KERN_EMERG format , ## arg)
#define MP211_VER30_ERATTA_NO6 1
#define MP200_CAM_NAME "mp200_camera"
#define MP200_CAM_SIZE_LIST_MAX	(4)
#define MP200_CAM_FRAME_MAX	(3)
#define LIMIT(x) ((((x)>0xffffff)?0xff0000:(((x)<=0xffff)?0:(x)&0xff0000))>>16)

typedef struct mp200_cam_portratio_size {
	int height;
	int width;
} MP200_CAM_PORTRATIO_SIZE;
static const MP200_CAM_SIZE mp200_cam_size_list[MP200_CAM_SIZE_LIST_MAX] = {
	{ 128,  96}, /*    12,288 :Sub-QCIF */
	{ 160, 120}, /*    19,200 :QQVGA (Quarter Quarter VGA) */
	{ 320, 240}, /*    76,800 :QVGA (Quarter VGA QVGA) */
	{ 640, 480}, /*   307,200 :VGA (Video Graphics Array) */
	/* { 800, 600}, *//*   480,000 :SVGA (Super VGA) */
	/* {1024, 768}, *//*   786,432 :XGA (eXtended Graphics Array) */
	/* {1152, 864}, *//*   995,328 :XGA+ (eXtended Graphics Array Plus) */
	/* {1280, 960}, *//* 1,228,800 :Quad-VGA (Quad-Video Graphics Array) */
	/* {1400,1050}, *//* 1,470,000 :SXGA+ (Super XGA Plus) */
	/* {1600,1200}, *//* 1,920,000 :UXGA (Ultra XGA) */
	/* {2048,1536}, *//* 3,145,728 :QXGA (Quad XGA) */
	/* {3200,2400}, *//* 7,680,000 :QUXGA (Quad Ultra XGA) */
};
/* --------------------------------------------- */
void mp200_img_preferred_regist(struct mp200_img_req*);
int  mp200_img_cancel(struct mp200_img_req*, int);
static int mp200_cam_open(struct inode*, struct file*);
static int mp200_cam_close(struct inode*, struct file*);
static ssize_t mp200_cam_read(struct file*, char*, size_t, loff_t*);
static int mp200_cam_mmap(struct file*, struct vm_area_struct*);
static int mp200_cam_ioctl(
	struct inode*, struct file*, unsigned int, unsigned long);
static int mp200_cam_suspend(struct device*, u32 state, u32);
static int mp200_cam_resume(struct device*, u32);
static int __init mp200_cam_probe(struct device*);
/* --------------------------------------------- */
static int nframes	= 2;
static int maxdepth	= 16;
static int video_nr	= -1;
static int irq		= INT_CAM;
static struct device *mp200_cam_device;
static struct device_driver mp200_cam_driver = {
	.name		= MP200_CAM_NAME,
	.bus		= &platform_bus_type,
	.probe		= mp200_cam_probe,
	.remove		= NULL,
	.suspend	= mp200_cam_suspend,
	.resume		= mp200_cam_resume,
	.shutdown	= NULL,
};
static struct file_operations mp200_cam_fops = {
	.owner		= THIS_MODULE,
	.open		= mp200_cam_open,
	.release	= mp200_cam_close,
	.read		= mp200_cam_read,
	.mmap		= mp200_cam_mmap,
	.ioctl		= mp200_cam_ioctl,
	.llseek 	= no_llseek,
};
static MP200_CAM_DATA mp200_cam_priv;
static MP200_CAM_FRAME mp200_cam_frame_priv[MP200_CAM_FRAME_MAX];
static struct mp200_img_req_cam mp200_cam_img_req_priv[MP200_CAM_FRAME_MAX];
/* ------------------------ Queue routines --------------------- */
/* Inits the queue */
static inline void mp200_cam_initq(struct mp200_cam_queue* queue)
{
	FNC_ENTRY

	queue->head = queue->tail = 0;
	queue->len = 0;
	spin_lock_init(&queue->s_lock);

	FNC_EXIT
}
/* Pulls an element from the queue */
static inline int mp200_cam_pullq(struct mp200_cam_queue* queue)
{
	int result;
	unsigned long flags;
	FNC_ENTRY

	spin_lock_irqsave(&queue->s_lock, flags);
	if (!queue->len) {
		spin_unlock_irqrestore(&queue->s_lock, flags);
		FNC_EXIT return -1;
	}
	result = queue->buf[queue->head];
	queue->head++;
	queue->head &= (MP200_CAM_QUEUE_SIZE - 1);
	queue->len--;
	spin_unlock_irqrestore(&queue->s_lock, flags);

	FNC_EXIT return result;
}
/* Pushes an element into the queue */
static inline void mp200_cam_pushq(struct mp200_cam_queue* queue, int element)
{
	unsigned long flags;
	FNC_ENTRY

	spin_lock_irqsave(&queue->s_lock, flags);
	if (queue->len == MP200_CAM_QUEUE_SIZE) {
		/* remove the first element */
		queue->head++;
		queue->head &= (MP200_CAM_QUEUE_SIZE - 1);
		queue->len--;
	}
	queue->buf[queue->tail] = element;
	queue->tail++;
	queue->tail &= (MP200_CAM_QUEUE_SIZE - 1);
	queue->len++;
	spin_unlock_irqrestore(&queue->s_lock, flags);

	FNC_EXIT
}
/* Tests if the queue is empty */
static inline int mp200_cam_emptyq(struct mp200_cam_queue* queue, int* elem)
{
	int result;
	unsigned long flags;
	FNC_ENTRY

	spin_lock_irqsave(&queue->s_lock, flags);
	result = (queue->len == 0);
	if (!result && elem) *elem = queue->buf[queue->head];
	spin_unlock_irqrestore(&queue->s_lock, flags);

	FNC_EXIT return result;
}

/* ------------------------ Sizes routines --------------------- */
static inline int mp200_cam_depth(int palette)
{
	int res = -EINVAL;
	FNC_ENTRY

	switch(palette) {
	case VIDEO_PALETTE_RGB24:
		res = 24;
		break;
	case VIDEO_PALETTE_RGB565:
		res = 16;
	}

	FNC_EXIT return res;
}
static inline int mp200_cam_depth_byte(int palette)
{
	int res;
	FNC_ENTRY

	res = mp200_cam_depth(palette);
	if (0 < res) res /= 8;

	FNC_EXIT return res;
}
static inline int mp200_cam_get_palette_count(int size_no, int palette)
{
	int res;
	MP200_CAM_SIZE* size;
	FNC_ENTRY

	size = (MP200_CAM_SIZE*)mp200_cam_size_list;
	size += size_no;
	res = size->width * size->height * mp200_cam_depth_byte(palette);

	FNC_EXIT return res;
}
static inline void mp200_cam_get_truth_size(
	MP200_CAM_DATA* cam, int size_no, MP200_CAM_SIZE* size)
{
	FNC_ENTRY

	if (cam->portrait) {
		MP200_CAM_PORTRATIO_SIZE* port;
		port = (MP200_CAM_PORTRATIO_SIZE*)mp200_cam_size_list;
		port += size_no;
		size->width  = port->width;
		size->height = port->height;
	}
	else {
		*size = mp200_cam_size_list[size_no];
	}

	FNC_EXIT return;
}
static int mp200_cam_chk_size(
	MP200_CAM_DATA* cam, int width, int height, int* size_no, int mode)
{
	int i = MP200_CAM_SIZE_LIST_MAX;
	MP200_CAM_SIZE chk;
	FNC_ENTRY

	if(mode) {
		if (height < cam->min.height ||  height > cam->max.height) {
			 FNC_EXIT return -EINVAL;
		}
		if (width < cam->min.width ||  width > cam->max.width) {
			FNC_EXIT return -EINVAL;
		}
	}
	if (cam->portrait) i--;
	while (i) {
		i--;
		*size_no = i;
		mp200_cam_get_truth_size(cam, i, &chk);
		if (width >= chk.width && height >= chk.height) break;
	}
	if (width == chk.width && height == chk.height) {
		 FNC_EXIT return 0;
	}

	FNC_EXIT return 1;
}
static inline int mp200_cam_yuv422_plane_size(MP200_CAM_FRAME* cpframe)
{
	MP200_CAM_SIZE* size;
	FNC_ENTRY

	size = (MP200_CAM_SIZE*)mp200_cam_size_list;
	size += cpframe->size_no;

	FNC_EXIT return size->width * size->height;
}

/* ------------------------ Palette convert routines --------------------- */

static ssize_t mp200_cam_yuv2rgb(
	int palette, int* ysrcp, int* uvsrcp, unsigned char* dirp, size_t count)
{
	int cnt = 0;
	int dircnt = 0;
	FNC_ENTRY

	while (count > cnt) {
		int i = 0;
		int y, u, v, r, g, b, y1;

		/* data is little endian & 32bit set */
		while (sizeof(int) > i) {
			y  = (*ysrcp >> (8 * i)) & 0x000000ff;
			u = (*uvsrcp >> (8 * i)) & 0x000000ff;
			i++;
			y1 = (*ysrcp >> (8 * i)) & 0x000000ff;
			v = (*uvsrcp >> (8 * i)) & 0x000000ff;
			i++;
			/* YCbCr(CCIR601)
			 * R = Y                 + 1.371 * Cr(V)
			 * G = Y - 0.336 * Cb(U) - 0.698 * Cr(V)
			 * B = Y + 1.732 * Cb(U)
			 */
			y = y * 65535;
			u = u - 128;
			y1 = y1 * 65535;
			v  =  v - 128;
			r =  89849 * v;
			g = -22020 * u + -45744 * v;
			b = 113508 * u;
			switch(palette) {
			case VIDEO_PALETTE_RGB565:
				/* data type shout little endian */
				*dirp++ = ((LIMIT(g+y) & 0xfc) << 3) |
							(LIMIT(b+y) >> 3);
				*dirp++ = (LIMIT(r+y) & 0xf8) |
							(LIMIT(g+y) >> 5);
				*dirp++ = ((LIMIT(g+y1) & 0xfc) << 3) |
							(LIMIT(b+y1) >> 3);
				*dirp++ = (LIMIT(r+y1) & 0xf8) |
							(LIMIT(g+y1) >> 5);
				cnt+=4;
				break;
			case VIDEO_PALETTE_RGB24:
				*dirp++ = LIMIT(r+y);
				*dirp++ = LIMIT(g+y);
				*dirp++ = LIMIT(b+y);
				*dirp++ = LIMIT(r+y1);
				*dirp++ = LIMIT(g+y1);
				*dirp++ = LIMIT(b+y1);
				cnt+=6;
				break;
			}
		}
		ysrcp++;
		uvsrcp++;
		dircnt++;
	}

	FNC_EXIT return dircnt;
}
static ssize_t mp200_cam_cpuconvert(
	char* buf, MP200_CAM_FRAME* cpframe, MP200_CAM_DATA* cam )
{
	int count;
	int* ysrcp;
	int* uvsrcp;
	FNC_ENTRY

	ysrcp = (int*)cpframe->buf[0].vadr;
	uvsrcp = (int*)
		(cpframe->buf[0].vadr + mp200_cam_yuv422_plane_size(cpframe));
	count = mp200_cam_get_palette_count(
		cpframe->size_no, cpframe->palette);

	if (cam->mapped) {
		mp200_cam_yuv2rgb(cpframe->palette, ysrcp, uvsrcp, buf, count);
	}
	else {
		int cnt   = 0;
		int plane = 0;
		int lineb;
		void* line;
		MP200_CAM_SIZE size;

		mp200_cam_get_truth_size(cam, cpframe->size_no, &size);
		lineb = size.width * mp200_cam_depth_byte(cpframe->palette);
		line = kmalloc(lineb, GFP_KERNEL);
		if (!line) {
			FNC_EXIT return -ENOMEM;
		}
		while (count > cnt) {
			if (count < (cnt + lineb)) lineb = count - cnt;
			plane = mp200_cam_yuv2rgb(
				cpframe->palette, ysrcp, uvsrcp, line, lineb);
			if (copy_to_user(buf + cnt, line, lineb)) {
				kfree(line);
				FNC_EXIT return -EFAULT;
			}
			ysrcp  += plane;
			uvsrcp += plane;
			cnt += lineb;
		}
		kfree(line);
	}

	FNC_EXIT return count;
}

/* ------------------------ Memory routines --------------------- */

static void* mp200_cam_dma_alloc(size_t size, dma_addr_t * dma_handle)
{
	int i = 0;
	int sz;
	void *ret;
	unsigned long adr;
	int gfp = GFP_KERNEL;
	FNC_ENTRY

	sz = get_order(size);
	ret = (void *) __get_free_pages(gfp, sz);
	size = PAGE_SIZE * (1 << sz);

	if (ret != NULL) {
		memset(ret, 0, size);
		*dma_handle = virt_to_phys(ret);
		adr = (unsigned long) ret;
		while (size > 0) {
			i++;
			SetPageReserved(virt_to_page((void *)adr));
			adr += PAGE_SIZE;
			size -= PAGE_SIZE;
		}
	}

	FNC_EXIT return ret;
}
static void mp200_cam_dma_free(void *vaddr, size_t size)
{
	int i = 0;
	int sz;
	unsigned long adr;
	FNC_ENTRY

	sz = get_order(size);
	size = PAGE_SIZE * (1 << sz);

	if (vaddr) {
		adr = (unsigned long) vaddr;
		while ((long) size > 0) {
			i++;
			ClearPageReserved(virt_to_page((void *)adr));
			adr += PAGE_SIZE;
			size -= PAGE_SIZE;
		}
		free_pages((unsigned long) vaddr, sz);
	}

	FNC_EXIT
}
static int mp200_cam_free_buff(MP200_CAM_DATA* cam, int setmode)
{
	int i = 0;
	int doubling;
	size_t size;
	FNC_ENTRY

	if (!cam->frame) {
		FNC_EXIT return -EFAULT;
	}
	doubling = cam->doubling;

	switch(setmode) {
	case MP200_CAM_MODE_INIT :
		break;
	case MP200_CAM_MODE_IMGPR : /* IMG processor mode */
		switch(cam->convmode) {
		case MP200_CAM_MODE_INIT :
		case MP200_CAM_MODE_YUVPL :
		case MP200_CAM_MODE_CPUCV :
		case MP200_CAM_MODE_IMGPR :
			FNC_EXIT return 0;
		default :
			FNC_EXIT return -EINVAL;
		}
		break;
	case MP200_CAM_MODE_CPUCV : /* CPU convert mode */
		switch(cam->convmode) {
		case MP200_CAM_MODE_INIT :
			FNC_EXIT return 0;
		case MP200_CAM_MODE_IMGPR :
			if (!cam->mapped) {
				doubling = 0;
				break;
			}
		case MP200_CAM_MODE_YUVPL :
		case MP200_CAM_MODE_CPUCV :
			FNC_EXIT return 0;
		default :
			FNC_EXIT return -EINVAL;
		}
		break;
	case MP200_CAM_MODE_YUVPL : /* MP200 YUV plane mode */
		switch(cam->convmode) {
		case MP200_CAM_MODE_INIT :
			FNC_EXIT return 0;
		case MP200_CAM_MODE_IMGPR :
			doubling = 0;
			break;
		case MP200_CAM_MODE_CPUCV :
			if (cam->mapped) {
				doubling = 0;
				break;
			}
		case MP200_CAM_MODE_YUVPL :
			FNC_EXIT return 0;
		default :
			FNC_EXIT return -EINVAL;
		}
		break;
	default :
		FNC_EXIT return -EINVAL;
	}
	size = cam->fragsize * cam->nframes;

	while (cam->frame[0].buf[doubling].vadr) {
		mp200_cam_dma_free(cam->frame[0].buf[doubling].vadr, size);
		cam->frame[0].buf[doubling].vadr = NULL;
		cam->frame[0].buf[doubling].padr = 0;
		if (doubling) doubling--;
		else break;
	}
	if (MP200_CAM_MODE_IMGPR == cam->convmode
	|| (MP200_CAM_MODE_CPUCV == cam->convmode && cam->mapped)) {
		i = 0;
		doubling = cam->doubling;
		for (; i < cam->nframes; i++) {
			cam->frame[i].buf[0].vadr =
					cam->frame[i].buf[doubling].vadr;
			cam->frame[i].buf[0].padr =
					cam->frame[i].buf[doubling].padr;
			cam->frame[i].buf[doubling].vadr = NULL;
			cam->frame[i].buf[doubling].padr = 0;
		}
	}
	cam->convmode = setmode;
	cam->doubling = 0;

	FNC_EXIT return 0;
}
static int mp200_cam_allocate_buff(
	MP200_CAM_DATA* cam, int setmode, int mapped)
{
	int i = 0;
	int doubling = 0;
	char* vadr;
	unsigned long padr;
	size_t size;
	FNC_ENTRY

	if (!cam->frame) {
		FNC_EXIT return -EFAULT;
	}
	doubling = cam->doubling;
	switch(setmode) {
	case MP200_CAM_MODE_INIT :
		if (cam->frame[0].buf[0].vadr) {
			FNC_EXIT return 0;
		}
		mapped = 0;
		break;
	case MP200_CAM_MODE_IMGPR : /* IMG processor mode */
		switch(cam->convmode) {
		case MP200_CAM_MODE_INIT :
		case MP200_CAM_MODE_YUVPL :
			doubling = 1;
			break;
		case MP200_CAM_MODE_CPUCV :
			if (!cam->mapped) {
				doubling = 1;
				break;
			}
		case MP200_CAM_MODE_IMGPR :
			FNC_EXIT return 0;
		default :
			FNC_EXIT return -EINVAL;
		}
		break;
	case MP200_CAM_MODE_CPUCV : /* CPU convert mode */
		switch(cam->convmode) {
		case MP200_CAM_MODE_INIT :
		case MP200_CAM_MODE_YUVPL :
			if (mapped) doubling = 1;
			else {
				cam->convmode = MP200_CAM_MODE_CPUCV;
				FNC_EXIT return 0;
			}
			break;
		case MP200_CAM_MODE_CPUCV :
			if (!cam->mapped && mapped) {
				doubling = 1;
				break;
			}
		case MP200_CAM_MODE_IMGPR :
			cam->convmode = MP200_CAM_MODE_CPUCV;
			FNC_EXIT return 0;
		default :
			FNC_EXIT return -EINVAL;
		}
		break;
	case MP200_CAM_MODE_YUVPL : /* MP200 YUV plane mode */
		switch(cam->convmode) {
		case MP200_CAM_MODE_INIT :
		case MP200_CAM_MODE_IMGPR :
		case MP200_CAM_MODE_CPUCV :
		case MP200_CAM_MODE_YUVPL :
			cam->convmode = MP200_CAM_MODE_YUVPL;
			FNC_EXIT return 0;
		default :
			FNC_EXIT return -EINVAL;
		}
		break;
	default :
		FNC_EXIT return -EINVAL;
	}
	size = cam->fragsize * cam->nframes;

	vadr  = mp200_cam_dma_alloc(size, (dma_addr_t*)&padr);
	if (!vadr) goto err;
	memzero(vadr, size);

	cam->frame[i].buf[doubling].vadr = vadr;
	cam->frame[i].buf[doubling].padr = padr;
	i++;
	for (; i < cam->nframes; i++) {
		cam->frame[i].buf[doubling].vadr =
		cam->frame[0].buf[doubling].vadr + i * cam->fragsize;
		cam->frame[i].buf[doubling].padr =
		cam->frame[0].buf[doubling].padr + i * cam->fragsize;
	}
	if (MP200_CAM_MODE_YUVPL == cam->convmode && cam->mapped && mapped) {
		i = 0;
		for (; i < cam->nframes; i++) {
			vadr = cam->frame[i].buf[0].vadr;
			padr = cam->frame[i].buf[0].padr;
			cam->frame[i].buf[0].vadr =
					cam->frame[i].buf[doubling].vadr;
			cam->frame[i].buf[0].padr =
					cam->frame[i].buf[doubling].padr;
			cam->frame[i].buf[doubling].vadr = vadr;
			cam->frame[i].buf[doubling].padr = padr;
		}
	}
	cam->convmode = setmode;
	cam->doubling = doubling;

	FNC_EXIT return 0;
err:
	printk(MP200_CAM_NAME ": unable to allocate memory\n ");
	FNC_EXIT return -ENOMEM;
}

/* ------------------------ Register routines --------------------- */

static int mp200_gain_update(MP200_CAM_DATA* cam)
{
	unsigned long flags;
	FNC_ENTRY

	/*
	 *** Update CA_UPDATE ***
	 *
	 * CA_BNZR
	 * CA_BNGR
	 * CA_CBZR
	 * CA_CBGR
	 * CA_CRZR
	 * CA_CRGR
	 *
	 * REG_CA_MIRROR
	 *
	 */
	local_irq_save(flags);
	REG_CA_BNGR = cam->gain.y_gain;
	REG_CA_CBGR = cam->gain.u_gain;
	REG_CA_CRGR = cam->gain.v_gain;
	REG_CA_BNZR = cam->gain.y_offset;
	REG_CA_CBZR = cam->gain.u_offset;
	REG_CA_CRZR = cam->gain.v_offset;
	local_irq_restore(flags);
	REG_CA_MIRROR = cam->setmirror << 2;

	/*** Update CA_UPDATE ***/
	REG_CA_UPDATE = CA_UPDATE_UPDATE_MASK;

	FNC_EXIT return 0;
}
static int mp200_camif_update(int frameno, MP200_CAM_DATA* cam)
{
	int adjust_clk = 0;
	int start_x;
	int start_y;
	int cut_size_no;
	int base_size_no;
	MP200_CAM_SIZE* base_size;
	MP200_CAM_SIZE* sizep;
	MP200_CAM_SIZE size;
	MP200_CAM_SIZE cut_size;
	MP200_CAM_SIZE* max;
	MP200_CAM_FRAME* camframe;
	FNC_ENTRY

	max = &cam->max;
	camframe = &cam->frame[frameno];
	/*
	 *** Update CA_UPDATE ***
	 *
	 * CA_X1R
	 * CA_X2R
	 * CA_Y1R
	 * CA_Y2R
	 *
	 * CA_LINESIZE_MAIN
	 * CA_XRATIO_MAIN
	 * CA_YRATIO_MAIN
	 * CA_DMAX_MAIN
	 * CA_DMAY_MAIN
	 *
	 */
	mp200_cam_chk_size(cam, max->width, max->height, &cut_size_no, 0);
	mp200_cam_get_truth_size(cam, cut_size_no, &cut_size);
	start_x = (max->width  - cut_size.width)  / 2;
	start_y = (max->height - cut_size.height) / 2;
#if MP211_VER30_ERATTA_NO6
	/* MP211 Ver3.0 Eratta No.6 VS_DET HS_DET DATA_DET */
	/* YUV data detection for half edge */
	if (cam->yuvtrail) adjust_clk = 1;
#endif
	REG_CA_X1R = (cam->hblankval + start_x)
			* cam->edge_sample - (3 + adjust_clk);
	REG_CA_X2R = (cam->hblankval + start_x + cut_size.width)
			* cam->edge_sample - (3 + adjust_clk);
	REG_CA_Y1R = cam->vblankval + start_y;
	REG_CA_Y2R = cam->vblankval + start_y + cut_size.height;

	mp200_cam_get_truth_size(cam, camframe->size_no, &size);
	REG_CA_LINESIZE_MAIN = size.width;
	REG_CA_DMAX_MAIN     = size.width;
	REG_CA_DMAY_MAIN     = size.height;

	mp200_cam_chk_size(cam, max->width, max->height, &base_size_no, 0);
	base_size = (MP200_CAM_SIZE*)mp200_cam_size_list;
	base_size += base_size_no;
	sizep = (MP200_CAM_SIZE*)mp200_cam_size_list;
	sizep += camframe->size_no;
	/* DMAX =< (X2R - X1R)/cam->edge_sample * 64 / (64 + XRATIO) */
	REG_CA_XRATIO_MAIN =
		((base_size->width - sizep->width) * 64) / sizep->width;
	/* DMAY =< (Y2R - Y1R) * 64 / (64 + YRATIO) */
	REG_CA_YRATIO_MAIN =
		((base_size->height - sizep->height) * 64) / sizep->height;
	/*** Update CA_UPDATE ***/
	REG_CA_UPDATE = CA_UPDATE_UPDATE_MASK;

	FNC_EXIT return 0;
}
static int mp200_camif_adjust_plane_addr(int frameno, MP200_CAM_DATA* cam)
{
	int plane_size;
	unsigned long y_plane;
	MP200_CAM_FRAME* camframe;
	FNC_ENTRY

	camframe = &cam->frame[frameno];
	y_plane = camframe->buf[0].padr;
	plane_size = mp200_cam_yuv422_plane_size(camframe);

	switch (frameno) {
	case 0 :
		REG_CA_YPLANE_A  = y_plane;
		REG_CA_UVPLANE_A = y_plane + plane_size;
		break;
	case 1 :
		REG_CA_YPLANE_B  = y_plane;
		REG_CA_UVPLANE_B = y_plane + plane_size;
		break;
	case 2 :
		REG_CA_YPLANE_C  = y_plane;
		REG_CA_UVPLANE_C = y_plane + plane_size;
		break;
	default :
		FNC_EXIT return -EINVAL;
	}

	FNC_EXIT return 0;
}

/* ------------------------ prosses routines --------------------- */

static int mp200_cam_fetch_frame(int frameno, MP200_CAM_DATA* cam)
{
	MP200_CAM_FRAME* camframe;
	FNC_ENTRY

	if (cam->errstatus) {
		FNC_EXIT return -EFAULT;
	}
	if (cam->stopped || CA_DMAREQ_MAINREQ_MASK == REG_CA_DMAREQ) {
		FNC_EXIT return 0;
	}
	camframe = &cam->frame[frameno];

	if (MP200_CAM_BUF_READY != camframe->state) {
		if (MP200_CAM_BUF_UNUSED != camframe->state) {
			FNC_EXIT return 0;
		}
		mp200_cam_pullq(&cam->grabq);
	}
	if (camframe->adjust) {
		mp200_camif_update(frameno, cam);
		mp200_camif_adjust_plane_addr(frameno, cam);
		camframe->adjust = 0;
	}
	cam->cur_frameno = frameno;
	camframe->state = MP200_CAM_BUF_GRABBING;
	REG_CA_FRAME = frameno + 1;
	REG_CA_DMAREQ = CA_DMAREQ_MAINREQ_MASK;

	FNC_EXIT return 0;

}
static int mp200_cam_capture_frame(int frameno, MP200_CAM_DATA* cam)
{
	unsigned long flags;
	MP200_CAM_FRAME* camframe;
	FNC_ENTRY

	if (cam->errstatus) {
		FNC_EXIT return -EFAULT;
	}
	camframe = &cam->frame[frameno];

	local_irq_save(flags);
	if (cam->stopped || CA_DMAREQ_MAINREQ_MASK == REG_CA_DMAREQ) {
		local_irq_restore(flags);
		mp200_cam_pushq(&cam->grabq, frameno);
	}
	else {
		camframe->state = MP200_CAM_BUF_READY;
		mp200_cam_fetch_frame(frameno, cam);
		local_irq_restore(flags);
	}

	FNC_EXIT return 0;
}
static int mp200_cam_frame_sync(
	int frameno, MP200_CAM_DATA* cam, unsigned int f_flags)
{
	int res = 0;
	MP200_CAM_FRAME* cpframe;
	DECLARE_WAITQUEUE(wait, current);
	FNC_ENTRY

	cpframe = &cam->frame[frameno];

	switch (cpframe->state) {
	case MP200_CAM_BUF_UNUSED:
		res = -EINVAL;
		break;
	case MP200_CAM_BUF_GRABBING:
		add_wait_queue(&cpframe->proc_list, &wait);
		do {
			set_current_state(TASK_INTERRUPTIBLE);
			if (cam->errstatus) {
				res = -EFAULT;
				err("%s hardware fault.\n", MP200_CAM_NAME);
				break;
			}
			if (f_flags & (O_NONBLOCK|O_NDELAY)) {
				res = -EAGAIN;
				break;
			}
			if (MP200_CAM_BUF_DONE == cpframe->state)
				break;

			if (signal_pending (current)) {
				res = -ERESTARTSYS;
				break;
			}
			schedule();
		} while (1);
		remove_wait_queue (&cpframe->proc_list, &wait);
		set_current_state(TASK_RUNNING);
		/* fall through */
		if (res) break;
	case MP200_CAM_BUF_DONE:
		break;
	}

	FNC_EXIT return res;
}

/* ------------------------ Interrupt routines --------------------- */

static inline void mp200_cam_wake_up_interruptible(MP200_CAM_DATA* cam)
{
	int i;
	FNC_ENTRY

	for (i=0; i < cam->nframes; i++) {
		if (MP200_CAM_BUF_GRABBING == cam->frame[i].state)
			wake_up_interruptible(&cam->frame[i].proc_list);
	}

	FNC_EXIT
}
void mp200_cam_img_irq(struct mp200_img_req* p, int flag)
{
	MP200_CAM_FRAME* imgframe = (MP200_CAM_FRAME*)p->data;
	MP200_CAM_DATA* cam = imgframe->cam;
	FNC_ENTRY

	if (0 > flag) {
		if (flag == IMG_CALLBACK_FAIL) {
			cam->errstatus |= MP200_ERR_IMGFAIL;
		}
		if (flag == IMG_CALLBACK_IMG_ERROR) {
			cam->errstatus |= MP200_ERR_IMGERR;
		}
		if (flag == IMG_CALLBACK_IMG_STOP) {
			cam->errstatus |= MP200_ERR_IMGSTOP;
		}
		mp200_cam_wake_up_interruptible(cam);

		FNC_EXIT return;
	}
	imgframe->state = MP200_CAM_BUF_DONE;

	wake_up_interruptible(&imgframe->proc_list);

	FNC_EXIT
}

/* ------------------------ IMG routines --------------------- */

static int mp200_cam_img_request(int frameno, MP200_CAM_DATA* cam)
{
	MP200_CAM_FRAME* imgframe;
	MP200_CAM_SIZE size;
	struct mp200_img_req* img_irq;
	struct mp200_img_reg_info* img_reg_info;
	FNC_ENTRY

	if (cam->errstatus) {
		FNC_EXIT return 0;
	}
	imgframe = &cam->frame[frameno];

	mp200_cam_get_truth_size(cam, imgframe->size_no, &size);

	img_irq = &cam->img[frameno].cam_irq;
	img_reg_info = img_irq->img_reg_info;

	img_reg_info[0].reg   = IMG_SRCSIZE_B;
	img_reg_info[0].data  = size.width;
	img_reg_info[1].reg   = IMG_DSTSIZE;
	img_reg_info[1].data  =
		size.width * mp200_cam_depth_byte(imgframe->palette);
	img_reg_info[2].reg   = IMG_SRCHSIZE_B;
	img_reg_info[2].data  = size.width;
	img_reg_info[3].reg   = IMG_SRCVSIZE_B;
	img_reg_info[3].data  = size.height;
	img_reg_info[4].reg   = IMG_PEL_ENDIAN;
	img_reg_info[4].data  = IMG_PEL_ENDIAN_BIT;
	img_reg_info[5].reg   = IMG_SRCYRGBADR_F;
	img_reg_info[5].data  = 0;
	img_reg_info[6].reg   = IMG_SRCUVADR_F;
	img_reg_info[6].data  = 0;
	img_reg_info[7].reg   = IMG_SRCYRGBADR_B;
	img_reg_info[7].data  = imgframe->buf[0].padr;
	img_reg_info[8].reg   = IMG_SRCUVADR_B;
	img_reg_info[8].data  =
		imgframe->buf[0].padr + mp200_cam_yuv422_plane_size(imgframe);
	img_reg_info[9].reg   = IMG_DSTYRGBADR;
	img_reg_info[9].data  = imgframe->buf[cam->doubling].padr;
	img_reg_info[10].reg  = IMG_DSTUVADR;
	img_reg_info[10].data = 0;
	img_reg_info[11].reg  = IMG_MODE;
	img_reg_info[11].data = IMG_OP_MODE_07_BIT;

	if (VIDEO_PALETTE_RGB565 == imgframe->palette ) {
		img_reg_info[11].data |= IMG_DSTFMT_BIT;
	}
	img_reg_info[12].reg  = IMG_REQ;
	img_reg_info[12].data = 1;

	img_irq->callback = mp200_cam_img_irq;
	img_irq->data = (unsigned long)imgframe;

	mp200_img_preferred_regist(img_irq);

	FNC_EXIT return 0;
}

/* ---------------------- Interrupt handling ------------------- */

static irqreturn_t mp200_cam_callback(
	int irq, void* dev_id, struct pt_regs* regs)
{
	int frameno;
	int status;
	MP200_CAM_DATA* cam = (MP200_CAM_DATA *)dev_id;
	MP200_CAM_FRAME* camframe;
	FNC_ENTRY

	status = REG_CA_STATUS;

	camframe = &cam->frame[cam->cur_frameno];

	if (status & (CA_INT_MAINORRAW_MASK | CA_INT_DMAERR_MASK)) {
		if (status & CA_INT_MAINORRAW_MASK) {
			cam->errstatus |= MP200_ERR_CAMOVERRUN;
		}
		if (status & CA_INT_DMAERR_MASK) {
			cam->errstatus |= MP200_ERR_CAMDMAERR;
		}
		REG_CA_ENCLR = 0x000037f;
		REG_CA_FFCLR = 0x000037f;

		mp200_cam_wake_up_interruptible(cam);

		FNC_EXIT return IRQ_HANDLED;
	}

	REG_CA_FFCLR = status;

	camframe->count = mp200_cam_get_palette_count(
		camframe->size_no, camframe->palette);

	if (MP200_CAM_MODE_IMGPR == cam->convmode) {
		mp200_cam_img_request(cam->cur_frameno, cam);
	}
	else {
		camframe->state = MP200_CAM_BUF_DONE;

		wake_up_interruptible(&camframe->proc_list);
	}
	if (!mp200_cam_emptyq(&cam->grabq, &frameno)) {
		mp200_cam_fetch_frame(frameno, cam);
	}

	FNC_EXIT return IRQ_HANDLED;
}

/* ------------------------ init routines --------------------- */

static inline void mp200_cam_frame_init(int frameno, MP200_CAM_DATA* cam)
{
	MP200_CAM_FRAME* camframe;
	FNC_ENTRY

	camframe = &cam->frame[frameno];
	camframe->count   = 0;
	camframe->palette = cam->setpalette;
	camframe->size_no = cam->setsize_no;
	camframe->state   = MP200_CAM_BUF_UNUSED;
	camframe->cam = (void*)cam;
	camframe->adjust = 0;
	init_waitqueue_head(&camframe->proc_list);

	FNC_EXIT return;
}
static int mp200_camif_init(MP200_CAM_DATA* cam)
{
	int i;
	unsigned int csr_reg;
	MP200_CAM_FRAME* camframe;
	FNC_ENTRY

	camframe = cam->frame;

	/* output endian little */
	csr_reg = CA_CSR_DATA_OD_MASK;
	/* input y->u->y->v */
	if (cam->yuyv) csr_reg |= CA_CSR_DATA_ID_MASK;
	/* 8bit full */
	csr_reg |= CA_CSR_LIMITSEL_MASK;
	/*
	 * VSYNC HSYNC
	 *** pulse ***
	 *
	 *   <--------CR_X2R+3-------->
	 * H  _1
	 * 0_| |_______________________  V
	 *    <-CA_X1R+3->               |_
	 *                             +  _| +CA_Y2R
	 *                      CA_Y1R | |   |
	 *                             + |   |
	 *                +-----------+  |   |
	 *                |///////////|  |   |
	 *                |///////////|  |   |
	 *                |///////////|  |   |
	 *                |///////////|  |   |
	 *                |///////////|  |   |
	 *                +-----------+  |   +
	 *                               |_
	 *                             +  _| +CA_Y2R
	 *                      CA_Y1R | |   |
	 *                             + |   |
	 *                +-----------+  |   |
	 *                |///////////|  |   |
	 *                |///////////|  |   |
	 *
	 *** valid ***
	 *
	 *   <--------CR_X2R+3--------> V
	 * H_            1____________  |
	 *   |___________|<-CA_X1R+3->| |_0
	 * 0 (H 1line skip)               |         |
	 *   +-----------+                |         |CA_Y2R
	 *   |///////////|                |         |
	 *   |///////////|                |         |
	 *   |///////////|                |         |
	 *   |///////////|                |         |
	 *   |///////////|                |         |
	 *   +-----------+              1_|         |
	 *                              |   |       |
	 *                              |   |CA_Y1R |
	 *                              |_  |       |
	 *   (H 1line skip)               |
	 *   +-----------+                |
	 *   |///////////|                |
	 */
	if (!cam->syncvalid) {
		if (cam->vsynclow) csr_reg |= CA_CSR_VS_DET_MASK;
		if (cam->hsynclow) csr_reg |= CA_CSR_HS_DET_MASK;
	}
	else {
		csr_reg |= CA_CSR_VS_POL_MASK;
		csr_reg |= CA_CSR_HS_POL_MASK;
		if (!cam->vsynclow) csr_reg |= CA_CSR_VS_DET_MASK;
		if (!cam->hsynclow) csr_reg |= CA_CSR_HS_DET_MASK;
	}
	/* YUV data double edge */
	if (1 == cam->edge_sample) csr_reg |= CA_CSR_CLK_EDGE_MASK;
	/* YUV data detection for half edge */
	if (cam->yuvtrail) csr_reg |= CA_CSR_DATA_DET_MASK;

	REG_CA_CSR = csr_reg;
	REG_CA_DMACNT = CA_DMACNT_MNRESIZE_MASK;

	for (i=0; cam->nframes > i; i++) {
		mp200_camif_adjust_plane_addr(i, cam);
	}
	mp200_gain_update(cam);
	mp200_camif_update(0, cam);

	FNC_EXIT return 0;
}
static int mp200_cam_hardware_release(MP200_CAM_DATA* cam)
{
	int i;
	int res = 0;
	unsigned long flags;
	FNC_ENTRY

	cam->stopped = 1;

	i = 8;
	/* Hynix clock 12MHz 640*480 -> 1 flame time 69msec */
	while (i && REG_CA_DMAREQ) {
		set_current_state(TASK_UNINTERRUPTIBLE);
		/* 10ms */
		schedule_timeout(HZ / 100);
		set_current_state(TASK_RUNNING);
		i--;
	}
	if (MP200_CAM_MODE_IMGPR == cam->convmode) {
		for (i=0; i<cam->nframes; i++) {
			switch(cam->img[i].cam_irq.status) {
			case IMG_REQ_STATUS_REGISTERED:
			case IMG_REQ_STATUS_RUNNING:
				mp200_img_cancel(
					&cam->img[i].cam_irq, IMG_CANCEL_SYNC);
			}
		}
	}
	REG_CA_ENCLR = 0x000037f;
	REG_CA_FFCLR = 0x000037f;
	free_irq(irq, (void*)cam);

	if (cam->hw_shutdown) {
		int dummy;
		cam->hw_shutdown(&dummy);
	}
	/* SMU */
	local_irq_save(flags);
	/* stop CLK */
	REG_GCLKCTRL0ENA = SMU_CAM_CHGL1_SCLK_GCK_ENA;
	REG_GCLKCTRL0 = 0;
	REG_DIVCAMCHGL1SCLK = 0x0;
	REG_CAM_PLLSEL = 0x3;
	/* start CLK */
	REG_GCLKCTRL0ENA = SMU_CAM_CHGL1_SCLK_GCK_ENA;
	REG_GCLKCTRL0 = SMU_CAM_CHGL1_SCLK_GCK_ENA;
	REG_SMU_IOCTRL =
		(REG_SMU_IOCTRL | SMU_IOCTRL_CAML_DTVH) &
		~SMU_IOCTRL_CHG_GIOL_CAMSTRH;
	local_irq_restore(flags);

#if MP200_CAM_DEBUG
	if (clockoff) {
		mp200_pmu_clkctrl_off(MP200_CLKCTRL_CAM);
		mp200_pmu_clkctrl_off(MP200_CLKCTRL_CAMPCLK);
		mp200_pmu_reset_device(MP200_RESETDEVICE_CAM);
		mp200_pmu_close_clockgate(MP200_CLOCKGATE_CAM_CLK);
		mp200_pmu_close_clockgate(MP200_CLOCKGATE_CAM_SCLK);
		mp200_pmu_close_clockgate(MP200_CLOCKGATE_CAM_PCLK);
	}
#else
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_CAM);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_CAMPCLK);
	mp200_pmu_reset_device(MP200_RESETDEVICE_CAM);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_CAM_CLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_CAM_SCLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_CAM_PCLK);
#endif
	REG_CA_MODULECONT = 0;

	FNC_EXIT return res;
}
static int mp200_cam_hardware_init(MP200_CAM_DATA* cam)
{
	int res;
	unsigned long flags;
	FNC_ENTRY

	REG_CA_MODULECONT = 1;
	REG_CA_ENCLR = 0x000037f;
	REG_CA_FFCLR = 0x000037f;

	if ((res =
		request_irq(
		irq, mp200_cam_callback, SA_INTERRUPT, "Camera", (void*)cam))
		 < 0) {
		FNC_EXIT return -ENODEV;
	}
	REG_CA_ENSET = CA_INT_MASKS;

	local_irq_save(flags);
	/* stop CLK */
	REG_GCLKCTRL0ENA = SMU_CAM_CHGL1_SCLK_GCK_ENA;
	REG_GCLKCTRL0 = 0;
	/* CAM PLL1 AHB domain 96MHz div 4 -> 24MHz */
	REG_DIVCAMCHGL1SCLK = 7;
	REG_CAM_PLLSEL = 0x0;
	/* start CLK */
	REG_GCLKCTRL0ENA = SMU_CAM_CHGL1_SCLK_GCK_ENA;
	REG_GCLKCTRL0 = SMU_CAM_CHGL1_SCLK_GCK_ENA;
	REG_SMU_IOCTRL =
		(REG_SMU_IOCTRL | SMU_IOCTRL_CHG_GIOL_CAMSTRH) &
		~SMU_IOCTRL_CAML_DTVH;
	local_irq_restore(flags);

	/* CAM MACRO unreset  */
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_CAM);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_CAMPCLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_CAM_CLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_CAM_SCLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_CAM_PCLK);
	mp200_pmu_unreset_device(MP200_RESETDEVICE_CAM);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_CAM);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_CAMPCLK);

	if (cam->hw_init) {
		int dummy;
		cam->hw_init(&dummy);
	}

	FNC_EXIT return 0;
}

/* ------------------------- V4L interface --------------------- */

static int mp200_cam_open(struct inode* inode, struct file* file)
{
	int i,res = 0;
        MP200_CAM_DATA* cam = &mp200_cam_priv;
	FNC_ENTRY

	if (cam->open_count > 0) {
		FNC_EXIT return -EBUSY;
	}
	if (!cam->frame) {
		FNC_EXIT return -ENODEV;
	}

	down(&cam->busy_lock);
	if (1 > nframes) nframes = 1;
	if (3 < nframes) nframes = 3;
	if (24 != maxdepth) maxdepth = 16;

	cam->nframes = nframes;
	cam->maxdepth = maxdepth;
	cam->cur_frameno = 0;
	mp200_cam_initq(&cam->grabq);

	for (i=0; cam->nframes > i; i++) {
		mp200_cam_frame_init(i, cam);
	}
	cam->mapped    = 0;
	cam->stopped   = 0;
	cam->errstatus = 0;

	res = mp200_cam_hardware_init(cam);
	if (res) {
		up(&cam->busy_lock);
		mp200_cam_hardware_release(cam);
		FNC_EXIT return res;
	}

	res = mp200_cam_allocate_buff(cam, MP200_CAM_MODE_INIT, 0);
	if (res) {
		up(&cam->busy_lock);
		mp200_cam_hardware_release(cam);
		FNC_EXIT return res;
	}
	mp200_camif_init(cam);

	cam->open_count = 1;
	file->private_data = cam;

	up(&cam->busy_lock);

	FNC_EXIT return 0;
}
static int mp200_cam_close(struct inode* inode, struct file* file)
{
        MP200_CAM_DATA* cam = file->private_data;
	FNC_ENTRY

	if (down_interruptible(&cam->busy_lock)) {
		FNC_EXIT return -EINTR;
	}

	mp200_cam_hardware_release(cam);

	cam->open_count = 0;

	/* clean up capture-buffers */
	mp200_cam_free_buff(cam, MP200_CAM_MODE_INIT);

	up(&cam->busy_lock);

	FNC_EXIT return 0;
}
static ssize_t mp200_cam_read(
	struct file* file, char* buf, size_t count, loff_t* ppos)
{
        MP200_CAM_DATA* cam = file->private_data;
	MP200_CAM_FRAME* cpframe;
	int done = 0;
	int cur_frameno;
	int nxt_frameno;
	int res = 0;
	unsigned long flags;
	FNC_ENTRY

	/* make this _really_ smp and multithread-safe */
	if (down_interruptible(&cam->busy_lock)) {
		FNC_EXIT return -EINTR;
	}
	if (cam->errstatus) {
		up(&cam->busy_lock);
		FNC_EXIT return -EFAULT;
	}
	if (!buf) {
		up(&cam->busy_lock);
		FNC_EXIT return -EINVAL;
	}
	if (!count) {
		up(&cam->busy_lock);
		FNC_EXIT return 0;
	}
	if (cam->mapped) {
		up(&cam->busy_lock);
		FNC_EXIT return -ENXIO;
	}

	if (count < mp200_cam_get_palette_count(
		cam->setsize_no, cam->setpalette)) {
		up(&cam->busy_lock);
		FNC_EXIT return -EFAULT;
	}
	if (MP200_CAM_MODE_INIT == cam->convmode) {
		res = mp200_cam_allocate_buff(cam, cam->setconvmode, 0);
		if (res) {
			up(&cam->busy_lock);
			FNC_EXIT return res;
		}
	}

	cur_frameno = cam->cur_frameno;
	cpframe = &cam->frame[cur_frameno];

	switch (cpframe->state) {
	case MP200_CAM_BUF_UNUSED:
	case MP200_CAM_BUF_DONE:
		done = 1;
	}
	if(cam->sub_adjust) {
		cpframe->adjust = 1;
		cam->sub_adjust = 0;
	}
	if (cam->setsize_no != cpframe->size_no) {
		local_irq_save(flags);
		cpframe->adjust = 1;
		cpframe->size_no = cam->setsize_no;
		local_irq_restore(flags);
	}
	if (cam->setpalette != cpframe->palette) {
		cpframe->palette = cam->setpalette;
	}
	if (done) {
		mp200_cam_capture_frame(cur_frameno, cam);
	}

	/* next frame push */
	nxt_frameno = cur_frameno + 1;
	if (nxt_frameno >= cam->nframes) {
		nxt_frameno = 0;
	}
	cpframe = &cam->frame[nxt_frameno];

	if (cam->setsize_no != cpframe->size_no) {
		local_irq_save(flags);
		cpframe->adjust = 1;
		cpframe->size_no = cam->setsize_no;
		local_irq_restore(flags);
	}
	if (cam->setpalette != cpframe->palette) {
		cpframe->palette = cam->setpalette;
	}

	mp200_cam_capture_frame(nxt_frameno, cam);

	res = mp200_cam_frame_sync(cur_frameno, cam, file->f_flags);
	if (res) {
		up(&cam->busy_lock);
		FNC_EXIT return res;
	}

	cpframe = &cam->frame[cur_frameno];

	if (MP200_CAM_MODE_CPUCV == cam->convmode) {
		count = mp200_cam_cpuconvert(buf, cpframe, cam);
	}
	else {
		count = cpframe->count;
		if (copy_to_user(
			buf, cpframe->buf[cam->doubling].vadr, count)) {
			up(&cam->busy_lock);
			FNC_EXIT return -EFAULT;
		}
	}
	cpframe->state = MP200_CAM_BUF_UNUSED;

	if (!mp200_cam_emptyq(&cam->grabq, &cur_frameno)) {
		local_irq_save(flags);
		mp200_cam_fetch_frame(cur_frameno, cam);
		local_irq_restore(flags);
	}

	up(&cam->busy_lock);

	FNC_EXIT return count;
}
static int mp200_cam_ioctl(
	struct inode* inode, struct file* file, unsigned int cmd,
	unsigned long arg)
{
        MP200_CAM_DATA* cam = file->private_data;
	unsigned long flags;
	int res = 0;
	int restart = 0;
	int sync = 0;
	FNC_ENTRY

	if (down_interruptible(&cam->busy_lock)) {
		FNC_EXIT return -EINTR;
	}
	switch (cmd) {
	case VIDIOCGCAP:
	{
		struct video_capability	vc;	/* v4l capability query  */

		vc.type		= VID_TYPE_CAPTURE|VID_TYPE_SCALES;
		vc.channels	= 1;
		vc.audios	= 0;
		vc.maxwidth	= cam->max.width;
		vc.maxheight	= cam->max.height;
		vc.minwidth	= cam->min.width;
		vc.minheight	= cam->min.height;
		if (copy_to_user(
			(void*)arg, &vc, sizeof(struct video_capability))) {
			res = -EFAULT;
		}
		break;
	}
	case VIDIOCGCHAN:
	{
		struct video_channel v;

		if (copy_from_user(&v, (void*)arg, sizeof(v))) {
			res = -EFAULT;
			break;
		}
		if (v.channel!=0) {
			res = -EINVAL;
			break;
		}
		v.flags		= 0;
		v.tuners	= 0;
		v.type		= VIDEO_TYPE_CAMERA;
		strcpy(v.name, "mp200-camera");
		if (copy_to_user((void*)arg, &v, sizeof(v))) {
			res = -EFAULT;
		}
		break;
	}
	case VIDIOCSCHAN:
	{
		int v;

		if (copy_from_user(&v, (void*)arg,sizeof(v))) {
			res = -EFAULT;
			break;
		}
		if (v!=0) res = -EINVAL;
		break;
	}
	case VIDIOCGPICT:
	{
		struct video_picture	vp;	/* v4l camera settings */

		vp.contrast	= 0x8000;
		vp.whiteness	= 0x8000;
		vp.colour	= 0x8000;
		vp.hue		= 0x8000;
		vp.brightness	= 0x8000;
		vp.depth	= mp200_cam_depth(cam->setpalette);
		vp.palette	= cam->setpalette;
		if (copy_to_user(
			(void*)arg, &vp, sizeof(struct video_picture))) {
			res = -EFAULT;
		}
		break;
	}
	case VIDIOCSPICT:
	{
		int val;
		struct video_picture	vp;	/* v4l camera settings */

		if (copy_from_user(&vp, (void*)arg, sizeof(vp))) {
			res = -EFAULT;
			break;
		}
		if (vp.depth > maxdepth) {
			res = -EINVAL;
			break;
		}
		val = mp200_cam_depth(vp.palette);
		if (0 > val) {
			res = val;
			break;
		}
		if (vp.depth != val) {
			res = -EINVAL;
			break;
		}
		/* Now load the camera. */
		if (vp.palette != cam->setpalette) {
			cam->setpalette = vp.palette;
			sync = 1;
		}
		break;
	}
	case VIDIOCGWIN:
	{
		MP200_CAM_SIZE size;

		mp200_cam_get_truth_size(cam, cam->setsize_no, &size);
		cam->vw.width  = size.width;
		cam->vw.height = size.height;
		if (copy_to_user(
			(void*)arg, &cam->vw, sizeof(struct video_window))) {
			res = -EFAULT;
		}
		break;
	}
	case VIDIOCSWIN:
	{
		int val;
		int size_no;
		MP200_CAM_SIZE size;
		struct video_window vw;

		if (copy_from_user(&vw, (void*)arg, sizeof(vw))) {
			res = -EFAULT;
			break;
		}
		if (vw.flags) {
			res = -EINVAL;
			break;
		}
		if (vw.clipcount) {
			res = -EINVAL;
			break;
		}
		val = mp200_cam_chk_size(
			cam, vw.width, vw.height, &size_no, 1);
		if (0 > val) {
			res = val;
			break;
		}
		if (0 != val) sync = 1;
		mp200_cam_get_truth_size(cam, size_no, &size);
		vw.width  = size.width;
		vw.height = size.height;
		cam->vw = vw;
		if (copy_to_user(
			(void*)arg, &cam->vw, sizeof(struct video_window))) {
			res = -EFAULT;
			break;
		}
		if (size_no != cam->setsize_no) {
			cam->setsize_no = size_no;
			sync = 1;
		}
		break;
	}
	case VIDIOCGMBUF:
	{
		struct video_mbuf vm;
		int i;

		memset(&vm, 0 , sizeof(vm));
		vm.size = cam->fragsize * cam->nframes;
		vm.frames = cam->nframes;
		for (i = 0; i < cam->nframes; i++) {
			vm.offsets[i] = i * cam->fragsize;
		}
		if (copy_to_user((void*)arg, &vm, sizeof(vm))) {
			res = -EFAULT;
		}
		break;
	}
	case VIDIOCSYNC:
	{
		int i;
		MP200_CAM_FRAME* cpframe;

		if (get_user(i, (int *)arg)) {
			res = -EFAULT;
			break;
		}
		if (i < 0 || i >= cam->nframes) {
			res = -EINVAL;
			break;
		}
		res = mp200_cam_frame_sync(i, cam, file->f_flags);
		if (res) break;
		cpframe = &cam->frame[i];
		if (MP200_CAM_MODE_CPUCV == cam->convmode) {
			char* buf = 0;
			buf = cam->frame[i].buf[cam->doubling].vadr;
			mp200_cam_cpuconvert(buf, cpframe, cam);
		}
		cpframe->state = MP200_CAM_BUF_UNUSED;

		break;
	}
	case VIDIOCMCAPTURE:
	{
		int depth;
		int size_no;
		MP200_CAM_FRAME* cpframe;
		struct video_mmap vm;

		if (cam->errstatus) {
			res = -EFAULT;
			break;
		}
		if (copy_from_user(&vm, (void*)arg,sizeof(vm))) {
			res = -EFAULT;
			break;
		}
		if (vm.frame >= cam->nframes || vm.frame < 0) {
			res = -EINVAL;
			break;
		}
		cpframe = &cam->frame[vm.frame];
		if (cpframe->state != MP200_CAM_BUF_UNUSED) {
			res = -EBUSY;
			break;
		}
		res = mp200_cam_chk_size(
			cam, vm.width, vm.height, &size_no, 1);
		if (res) {
			res = -EINVAL;
			break;
		}
		depth = mp200_cam_depth(vm.format);
		if (0 > depth) {
			res = -EINVAL;
			break;
		}
		if (mp200_cam_get_palette_count(
			size_no, vm.format) > cam->fragsize) {
			res = -EINVAL;
			break;
		}
		local_irq_save(flags);
		if ((size_no != cpframe->size_no) || (cam->sub_adjust)) {
			 cpframe->adjust = 1;
			 cam->sub_adjust = 0;
		}
		cpframe->size_no = size_no;
		cpframe->palette = vm.format;
		local_irq_restore(flags);
		res = mp200_cam_capture_frame(vm.frame, cam);

		break;
	}
	case VIDIOCKEY:
		break;
	case VIDIOCGTUNER:
	case VIDIOCSTUNER:
	case VIDIOCCAPTURE:
	case VIDIOCGFBUF:
	case VIDIOCSFBUF:
	case VIDIOCGFREQ:
	case VIDIOCSFREQ:
	case VIDIOCGAUDIO:
	case VIDIOCSAUDIO:
		res = -EINVAL;
		break;
	case MP200_CAM_S_GAIN:
	{
		MP200_CAM_GAIN gain;

		if (copy_from_user(&gain, (void*)arg,sizeof(gain))) {
			res = -EFAULT;
		}
		if ((cam->gain.y_gain != gain.y_gain)
		 || (cam->gain.u_gain != gain.u_gain)
		 || (cam->gain.v_gain != gain.v_gain)
		 || (cam->gain.y_offset != gain.y_offset)
		 || (cam->gain.u_offset != gain.u_offset)
		 || (cam->gain.v_offset != gain.v_offset)) {
			cam->gain = gain;
			mp200_gain_update(cam);
		}
		break;
	}
	case MP200_CAM_G_GAIN:
		if (copy_to_user(
			(void*)arg, &cam->gain, sizeof(MP200_CAM_GAIN)))
			res = -EFAULT;
		break;
	case MP200_CAM_G_CTRL:
		res = put_user(cam->setmirror, (int*)arg);
		break;
	case MP200_CAM_S_CTRL:
	{
		int setmirror;

		res = get_user(setmirror, (int*)arg);
		setmirror &= 0x00000003;
		if (cam->setmirror != setmirror) {
			cam->setmirror = setmirror;
			mp200_gain_update(cam);
		}
		break;
	}
	case MP200_CAM_GET_MODE:
		res = put_user(cam->setconvmode, (int*)arg);
		break;
	case MP200_CAM_GET_ROTATION:
		res = put_user(cam->setportrait, (int*)arg);
		break;
	case MP200_CAM_MODES_IMGPR:
		restart = 1;
		sync = 1;
		cam->setconvmode = MP200_CAM_MODE_IMGPR;
		break;
	case MP200_CAM_MODES_CPUCV:
		restart = 1;
		sync = 1;
		cam->setconvmode = MP200_CAM_MODE_CPUCV;
		break;
	case MP200_CAM_MODES_MP200YUV:
		restart = 1;
		sync = 1;
		cam->setconvmode = MP200_CAM_MODE_YUVPL;
		break;
	case MP200_CAM_ROTATION_PORTRAIT:
		sync = 1;
		cam->setportrait = 1;
		break;
	case MP200_CAM_ROTATION_LANDSCAPE:
		sync = 1;
		cam->setportrait = 0;
		break;
	default:
		if (cam->client_ioctl) {
			res = cam->client_ioctl(inode, file, cmd, arg);
		}
		else {
			res = -ENOIOCTLCMD;
		}
	} /* switch */
	if (res) {
		up(&cam->busy_lock);
		FNC_EXIT return res;
	}
	if (restart) {
		cam->stopped = 1;
	}
	if (sync) {
		int i;
		for (i=0; cam->nframes > i;i++) {
			mp200_cam_frame_sync(i, cam, 0);
		}
	}
	if (cam->setportrait != cam->portrait) {
		int size_no;
		MP200_CAM_SIZE chk;
		mp200_cam_get_truth_size(cam,cam->setsize_no,&chk);
		local_irq_save(flags);
		cam->sub_adjust = 1;
		cam->portrait = cam->setportrait;
		local_irq_restore(flags);
		mp200_cam_chk_size(cam, chk.height, chk.width, &size_no, 0);
		cam->setsize_no = size_no;
	}
	if (cam->setconvmode != cam->convmode) {
		int i;

		res = mp200_cam_free_buff(cam, cam->setconvmode);
		if (res) {
			up(&cam->busy_lock);
			FNC_EXIT return res;
		}
		res = mp200_cam_allocate_buff(
			cam, cam->setconvmode, cam->mapped);
		if (res) {
			up(&cam->busy_lock);
			FNC_EXIT return res;
		}
		for (i=0;cam->nframes > i;i++) {
			mp200_camif_adjust_plane_addr(i, cam);
		}
	}

	if (restart) {
		int i;
		cam->stopped = 0;
		if (!mp200_cam_emptyq(&cam->grabq, &i)) {
			local_irq_save(flags);
			mp200_cam_fetch_frame(i, cam);
			local_irq_restore(flags);
		}
	}
	up(&cam->busy_lock);

	FNC_EXIT return 0;
}
static int mp200_cam_mmap(struct file* file, struct vm_area_struct* vma)
{
        MP200_CAM_DATA* cam = file->private_data;
	unsigned long start = vma->vm_start;
	unsigned long size  = vma->vm_end - vma->vm_start;
	unsigned long pos;
	int res;
	FNC_ENTRY

	if (size > cam->fragsize * cam->nframes) {
		FNC_EXIT return -EINVAL;
	}
	/* make this _really_ smp-safe */
	if (down_interruptible(&cam->busy_lock)) {
		FNC_EXIT return -EINTR;
	}
	if (!cam->mapped) {
		res = mp200_cam_allocate_buff(cam, cam->setconvmode, 1);
		if (res) {
			up(&cam->busy_lock);
			FNC_EXIT return res;
		}
	}

	pos = (unsigned long)(cam->frame[0].buf[cam->doubling].padr);
	res = remap_pfn_range(
		vma, start, pos >> PAGE_SHIFT, size, vma->vm_page_prot);
	if (res) {
		up(&cam->busy_lock);
		FNC_EXIT return res;
	}
	cam->mapped = 1;

	up(&cam->busy_lock);

	FNC_EXIT return 0;
}

/* ------------------------ register interface --------------------- */

static void mp200_cam_init(
	MP200_CAM_DATA* cam, MP200_CAM_PRELIMINARY* pre )
{
	int i;
	FNC_ENTRY

	cam->open_count = 0;

	init_MUTEX(&cam->busy_lock);
	mp200_cam_initq(&cam->grabq);
	cam->vblankval = pre->vblankval;
	cam->hblankval = pre->hblankval;
	if (pre->doubledge) cam->edge_sample = 1;
	else cam->edge_sample = 2;

	cam->gain.y_gain = 0x80;
	cam->gain.u_gain = 0x80;
	cam->gain.v_gain = 0x80;
	cam->gain.y_offset = 0;
	cam->gain.u_offset = 0;
	cam->gain.v_offset = 0;

	cam->max.width		= pre->maxwidth;
	cam->max.height		= pre->maxheight;
	cam->min.width		= pre->minwidth;
	cam->min.height		= pre->minheight;
	cam->hw_init		= pre->hw_init;
	cam->hw_shutdown	= pre->hw_shutdown;
	cam->client_ioctl	= pre->client_ioctl;

	cam->convmode		= MP200_CAM_MODE_INIT;
	cam->doubling		= 0;
	cam->portrait		= 0;
	cam->sub_adjust		= 0;
	cam->syncvalid 		= pre->syncvalid;
	cam->yuyv		= pre->yuyv;
	cam->yuvtrail		= pre->yuvtrail;
	cam->vsynclow		= pre->vsynclow;
	cam->hsynclow		= pre->hsynclow;
	cam->mapped		= 0;
	cam->stopped		= 0;

	mp200_cam_chk_size(
		cam, cam->max.width, cam->max.height, &cam->setsize_no, 0);

	cam->setmirror		= 0;
	cam->setpalette		= VIDEO_PALETTE_RGB565;
	cam->setconvmode	= MP200_CAM_MODE_IMGPR;
	cam->cur_frameno = 0;
	cam->fragsize = PAGE_ALIGN(
		cam->max.width * cam->max.height * (maxdepth/8));

	for (i=0; cam->nframes > i; i++) mp200_cam_frame_init(i, cam);

	FNC_EXIT return;
}
MP200_CAM_DATA* mp200_cam_register(MP200_CAM_PRELIMINARY* pre)
{
        MP200_CAM_DATA* cam = &mp200_cam_priv;
	FNC_ENTRY

	cam->img = mp200_cam_img_req_priv;
	cam->frame = mp200_cam_frame_priv;

	mp200_cam_init( cam, pre );

	info("V4L: Registered \"%s\" on %s\n", pre->name, MP200_CAM_NAME);

	FNC_EXIT return cam;
}
void mp200_cam_unregister(MP200_CAM_DATA* cam)
{
	FNC_ENTRY

	cam->frame = NULL;

	FNC_EXIT
}
static int __init mp200_cam_probe(struct device *dev)
{
	FNC_ENTRY

	mp200_cam_device = dev;

	FNC_EXIT return 0;
}
static int mp200_cam_suspend(struct device * dev, u32 state, u32 level)
{
        MP200_CAM_DATA* cam = &mp200_cam_priv;
	FNC_ENTRY

	if (cam->open_count > 0) { FNC_EXIT return -EBUSY;}

	FNC_EXIT return 0;
}
static int mp200_cam_resume(struct device * dev, u32 level)
{
	FNC_ENTRY
	FNC_EXIT return 0;
}

/* ------------------------ module init --------------------- */

static void __exit mp200_exit_cam(void)
{
	MP200_CAM_DATA* cam = &mp200_cam_priv;
	struct video_device *vfd;
	FNC_ENTRY

	vfd = cam->vdev;

	if (vfd) {
		if (vfd->minor == -1) {
			/* The device never got registered, so release the
			** video_device struct directly
			*/
			video_device_release(vfd);
		} else {
			/* The unregister function will release the video_device
			** struct as well as unregistering it.
			*/
			video_unregister_device(vfd);
			driver_unregister(&mp200_cam_driver);
		}
		cam->vdev = NULL;
	}

	FNC_EXIT return;
}
static int __init mp200_init_cam(void)
{
	MP200_CAM_DATA* cam = &mp200_cam_priv;
	struct video_device *vfd;
	FNC_ENTRY

	info("Starting %s.\n", MP200_CAM_NAME);

	memset(cam, 0, sizeof(MP200_CAM_DATA));

	/* initialize the video_device struct */
	vfd = cam->vdev = video_device_alloc();
	if (!vfd) {
		printk(KERN_ERR MP200_CAM_NAME
			": could not allocate video device struct\n");
		goto init_error;
	}

 	vfd->release = video_device_release;

 	strlcpy(vfd->name, MP200_CAM_NAME, sizeof(vfd->name));
 	vfd->type = VID_TYPE_CAPTURE;

 	/* need to register for a VID_HARDWARE_* ID in videodev.h */
 	vfd->hardware = 0;
 	vfd->fops = &mp200_cam_fops;
 	video_set_drvdata(vfd, cam);
 	vfd->minor = -1;

	if (driver_register(&mp200_cam_driver) < 0) {
		printk(KERN_ERR MP200_CAM_NAME
			": could not register driver\n");
		goto init_error;
        }

	vfd->dev = mp200_cam_device;

	if (video_register_device(vfd, VFL_TYPE_GRABBER, video_nr) < 0) {
		printk(KERN_ERR MP200_CAM_NAME
			": could not register Video for Linux device\n");
		driver_unregister(&mp200_cam_driver);
		goto init_error;
	}

	FNC_EXIT return 0;

init_error:
	mp200_exit_cam();
	FNC_EXIT return -ENODEV;
}

module_init(mp200_init_cam);
module_exit(mp200_exit_cam);

EXPORT_SYMBOL(mp200_cam_register);
EXPORT_SYMBOL(mp200_cam_unregister);

#if MP200_CAM_DEBUG
module_param(debug,  int, 0644);
module_param(clockoff, int, 0644);
#endif

#ifdef MODULE
module_param(nframes,  int, 0644);
module_param(maxdepth,  int, 0644);
MODULE_AUTHOR("2006 NEC ELECTRONICS CORPORATION");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION(MP200_CAM_NAME);
MODULE_SUPPORTED_DEVICE("");
#endif /* MODULE */

