/*
 *  File Name       : mp200_cam.h
 *  Function        : CAMERA I/F Driver
 *  Release Version : Ver 1.00
 *  Release Date    : 2005/07/15
 *
 *  Copyright (C) NEC Electronics Corporation 2005
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */
#ifndef MP200_CAM_H
#define MP200_CAM_H

#include <asm/arch/img.h>

#define MP200_REG(ADDR,OFF)	\
	(*(unsigned int volatile *)((unsigned char *)(ADDR) + OFF))

/*** SMU Registers ***/
#define SMU_ADDR	IO_ADDRESS (MP200_SMU_S0_BASE)

/* SMU_IOCTRL resistor physical:0xc081_0218 */
#define REG_SMU_IOCTRL		MP200_REG(SMU_ADDR,0x0218)

/* CAM PLL1 AHB domain 96MHz div 4 -> 24MHz */
/* CAM_PLLSEL 0xC801_00BC physical:0xc081_00bc
 *    bit   1  0
 *          0  0    PLL1
 *          0  1    PLL2
 *          1  0    PLL3
 *          1  1    12M  (default)
 */
#define REG_CAM_PLLSEL		   MP200_REG(SMU_ADDR,0x00bc)
/* DIVCAMCHGL1SCLK0XC801_0130
   DIVCAMSCLK(4:0) = 0x3 */
#define REG_DIVCAMCHGL1SCLK	   MP200_REG(SMU_ADDR,0x0130)
/* GCLKCTRL0 0xC801_0198*/
#define REG_GCLKCTRL0		   MP200_REG(SMU_ADDR,0x0198)
/* GCLKCTRL0ENA0xC801_019C*/
#define REG_GCLKCTRL0ENA	   MP200_REG(SMU_ADDR,0x019c)

#define SMU_CAM_CHGL1_SCLK_GCK_ENA 0x04000000

/* SMU_IOCTRL Masks */
#define SMU_IOCTRL_MASK		   0x000007f
#define SMU_IOCTRL_CAML_DTVH	   0x0000040 /* CAM/NTSC select (R/W)
					       * 0:CAM 1:NTSC
					       */
#define SMU_IOCTRL_CHG_GIOL_CAMSTRH 0x0000001 /* GIO/CAM select (R/W)
						* 0:GIO 1:CAM
						*/

/*** CAMERA I/F Registers ***/

#define CAM_ADDR	IO_ADDRESS (MP200_CAMERA_BASE)
#define CAM_IOSIZE	(0x0234)

/* CA_STATUS			R	default:0 */
#define REG_CA_STATUS		MP200_REG(CAM_ADDR,0x0000)
/* CA_RAWSTATUS			R	default:0 */
#define REG_CA_RAWSTATUS	MP200_REG(CAM_ADDR,0x0004)
/* CA_ENSET			R/W	default:0 */
#define REG_CA_ENSET		MP200_REG(CAM_ADDR,0x0008)
/* CA_ENCLR			W	default:0 */
#define REG_CA_ENCLR		MP200_REG(CAM_ADDR,0x000C)
/* CA_FFCLR			W	default:0 */
#define REG_CA_FFCLR		MP200_REG(CAM_ADDR,0x0010)
/* CA_ERRORADR			R/W	default:00000000H */
#define REG_CA_ERRORADR		MP200_REG(CAM_ADDR,0x0014)
/* CA_CSR			R/W	default:0 */
#define REG_CA_CSR		MP200_REG(CAM_ADDR,0x0020)
/* CA_X1R			R/W	default:0 */
#define REG_CA_X1R		MP200_REG(CAM_ADDR,0x0030)
/* CA_X2R			R/W	default:0 */
#define REG_CA_X2R		MP200_REG(CAM_ADDR,0x0034)
/* CA_Y1R			R/W	default:0 */
#define REG_CA_Y1R		MP200_REG(CAM_ADDR,0x0038)
/* CA_Y2R			R/W	default:0 */
#define REG_CA_Y2R		MP200_REG(CAM_ADDR,0x003C)
/* CA_BNZR			R/W	default:0 */
#define REG_CA_BNZR		MP200_REG(CAM_ADDR,0x0040)
/* CA_BNGR			R/W	default:80H */
#define REG_CA_BNGR		MP200_REG(CAM_ADDR,0x0044)
/* CA_CBZR			R/W	default:0 */
#define REG_CA_CBZR		MP200_REG(CAM_ADDR,0x0048)
/* CA_CBGR			R/W	default:80H */
#define REG_CA_CBGR		MP200_REG(CAM_ADDR,0x004C)
/* CA_CRZR			R/W	default:0 */
#define REG_CA_CRZR		MP200_REG(CAM_ADDR,0x0050)
/* CA_CRGR			R/W	default:80H */
#define REG_CA_CRGR		MP200_REG(CAM_ADDR,0x0054)
/* CA_DMACNT			R/W	default:0C000H */
#define REG_CA_DMACNT		MP200_REG(CAM_ADDR,0x0080)
/* CA_FRAME			R/W	default:0 */
#define REG_CA_FRAME		MP200_REG(CAM_ADDR,0x0084)
/* CA_DMAREQ			R/W	default:0 */
#define REG_CA_DMAREQ		MP200_REG(CAM_ADDR,0x0088)
/* CA_DMASTOP			W	default:0 */
#define REG_CA_DMASTOP		MP200_REG(CAM_ADDR,0x008C)
/* CA_LINESIZE_MAIN		R/W	default:0 */
#define REG_CA_LINESIZE_MAIN	MP200_REG(CAM_ADDR,0x0100)
/* CA_XRATIO_MAIN		R/W	default:0 */
#define REG_CA_XRATIO_MAIN	MP200_REG(CAM_ADDR,0x0104)
/* CA_YRATIO_MAIN		R/W	default:0 */
#define REG_CA_YRATIO_MAIN	MP200_REG(CAM_ADDR,0x0108)
/* CA_DMAX_MAIN			R/W	default:0 */
#define REG_CA_DMAX_MAIN	MP200_REG(CAM_ADDR,0x010C)
/* CA_DMAY_MAIN			R/W	default:0 */
#define REG_CA_DMAY_MAIN	MP200_REG(CAM_ADDR,0x0110)
/* CA_YPLANE_A			R/W	default:0 */
#define REG_CA_YPLANE_A		MP200_REG(CAM_ADDR,0x0114)
/* CA_UVPLANE_A			R/W	default:0 */
#define REG_CA_UVPLANE_A	MP200_REG(CAM_ADDR,0x0118)
/* CA_YPLANE_B			R/W	default:0 */
#define REG_CA_YPLANE_B		MP200_REG(CAM_ADDR,0x011C)
/* CA_UVPLANE_B			R/W	default:0 */
#define REG_CA_UVPLANE_B	MP200_REG(CAM_ADDR,0x0120)
/* CA_YPLANE_C			R/W	default:0 */
#define REG_CA_YPLANE_C		MP200_REG(CAM_ADDR,0x0124)
/* CA_UVPLANE_C			R/W	default:0 */
#define REG_CA_UVPLANE_C	MP200_REG(CAM_ADDR,0x0128)
/* CA_MODULECONT		R/W	default:0 */
#define REG_CA_MODULECONT	MP200_REG(CAM_ADDR,0x022C)
/* CA_UPDATE			R/W	default:0 */
#define REG_CA_UPDATE		MP200_REG(CAM_ADDR,0x0230)
/* CA_MIRROR			R/W	default:0 */
#define REG_CA_MIRROR		MP200_REG(CAM_ADDR,0x0234)


/* CA_STATUS
 * CA_RAWSTATUS
 * CA_ENSET
 * CA_ENCLR
 * CA_FFCLR Masks */

#define CA_INT_MAINORRAW_MASK	0x0000008
#define CA_INT_MAINTCRAW_MASK	0x0000004
#define CA_INT_DMAERR_MASK	0x0000002
#define CA_INT_CAMVS_MASK	0x0000001
#define CA_INT_MASKS	      ( CA_INT_MAINORRAW_MASK \
			      | CA_INT_MAINTCRAW_MASK \
			      | CA_INT_DMAERR_MASK )

/* CA_ERRORADR Masks */
#define CA_ERRORADR_LOCK_MASK	0x0000001
/* CA_CSR Masks */
#define CA_CSR_DATA_OD_MASK	0x0000400
			/*
			 *		    bit32            bit0
			 *	0 Y Plane | Y2 | Y3 | Y0 | Y1 |
			 *	   UV Plane | U1 | V1 | U0 | V0 |
			 *	1 Y Plane | Y3 | Y2 | Y1 | Y0 |
			 *	   UV Plane | V1 | U1 | V0 | U0 |
			 */
#define CA_CSR_DATA_ID_MASK	0x0000200	/* 0 UYVY
						 * 1 YUYV
						 */
#define CA_CSR_LD_TMG_MASK	0x0000100
#define CA_CSR_VS_DET_MASK	0x0000080
#define CA_CSR_HS_DET_MASK	0x0000040
#define CA_CSR_LIMITSEL_MASK	0x0000020	/* 0ITU Rec656
						 *    (Y:16-235 U,V:16-240)
						 * 18bit Full
						 */
#define CA_CSR_SYNCMODE_MASK	0x0000010	/* 0 CAMHS,CAMVS
						 * 1 ITU Rec656
						 */
#define CA_CSR_CLK_EDGE_MASK	0x0000008
#define CA_CSR_DATA_DET_MASK	0x0000004
#define CA_CSR_VS_POL_MASK	0x0000002
#define CA_CSR_HS_POL_MASK	0x0000001

/* CA_DMACNT Masks */
#define CA_DMACNT_DMAMODE_MASK	0x0020000	/* 0Camera DMA
						 * 1DTV DMA
						 */
#define CA_DMACNT_MAINYUV_MASK	0x0001000	/* 0YUV422
						 * 1YUV420
						 */
#define CA_DMACNT_MAINREC_MASK	0x0000300
#define CA_DMACNT_MAINMODE_MASK	0x0000030
#define CA_DMACNT_MNRESIZE_MASK	0x0000004
#define CA_DMACNT_PCULLR_MASK	0x0000003

/* CA_FRAME Masks */
#define CA_FRAME_MAINFRM_MASK	   0x0000003
/* CA_DMAREQ Masks */
#define CA_DMAREQ_MAINREQ_MASK	   0x0000001
/* CA_DMASTOP Masks */
#define CA_DMASTOP_MAINSTOP_MASK   0x0000001
/* CA_MODULECONT Masks */
#define CA_MODULECONT_HW_RSTZ_MASK 0x0000001
/* CA_UPDATE Masks */
#define CA_UPDATE_UPDATE_MASK 0x0000001

/* make a preliminary investigation */
typedef struct mp200_cam_preliminary
{
	char	name[32];

	int maxwidth; 		/* max capture width  value (pixel) */
	int maxheight;		/* max capture height value (pixel) */
	int minwidth;		/* min capture width  value (pixel) */
	int minheight;		/* min capture height value (pixel) */

	int vblankval;		/* v blank value (clock) */
	int hblankval;		/* h blank value (clock) */
				/*** blank ***  _   _   _   _   _   _
				 *  clock     _| |_| |_| |_| |_| |_| |_
				 *                  _
				 *  sync      _____| |_________________
				 *  data       x x x x x x o o o o o o
				 *                 <-blk->
				 */
	int maxHzval;		/* max Hz value exp:24Mhz:24000000 */

	int yuv8bit:1;		/*
				 * 0ITU Rec656 (Y:16-235 U,V:16-240)
				 * 18bit Full  (Y,U,V 0-255)
				 ** Camera I/F stream input YUV422 only
				 */
	int synciturec656:1;	/* sync mode
				 * 0 V/H sync
				 * 1 ITU Rec656
				 */
	int syncvalid:1;	/* V/H sync type
				 * 0 : pulse
				 * 1 : valid
				 ** pulse ** _
				 *  sync ___| |________________
				 *  data xxxooooooooooooooooxxx
				 ** valid ** ______________
				 *  sync ___|              |___
				 *  data xxxooooooooooooooooxxx
				 */

	int vsynctrail:1;	/* v sync of a clock pulse
				 * 0 leading edge of the clock pulse
				 * 1 trailing edge of a clock pulse
				 ** leading **  _   _   _   _   _   _
				 *  clock     _| |_| |_| |_| |_| |_| |_
				 *                  _
				 *  v sync    _____| |_________________
				 ** trailing **   _   _   _   _   _   _
				 *  clock      |_| |_| |_| |_| |_| |_|
				 *                  _
				 *  v sync    _____| |_________________
				 */
	int hsynctrail:1;	/* h sync of a clock pulse
				 * 0 leading edge
				 * 1 trailing edge
				 ***   same as above v sync
				 */
	int vsynclow:1;		/* v sync
				 * 0 high
				 * 1 low
				 ** high **       _
				 *  v sync    ___| |___________________
				 ** low **    ___   ___________________
				 *  v sync       |_|
				 */
	int hsynclow:1;		/* h sync
				 * 0 high
				 * 1 low
				 ***   same as above v sync
				 */
	int yuyv:1;		/* stream data sequence for half edge
				 * 0 : U(Cb)YV(Cr)Y
				 * 1 : YU(Cb)YV(Cr)
				 */
	int doubledge:1;	/* YUV data
				 * 0 half edge
				 * 1 double edge
				 ** half **      _   _   _   _   _   _
				 *  clock     |_| |_| |_| |_| |_| |_| |_
				 *            +---+---+---+---+---+---+-
				 *  stream    | Y | U | Y | V | Y | U |
				 *            +---+---+---+---+---+---+-
				 ** double **   _   _   _   _   _   _
				 *  clock     _| |_| |_| |_| |_| |_| |_
				 *            +-+-+-+-+-+-+-+-+-+-+-+-+
				 *  stream    |Y|U|Y|V|Y|U|Y|V|Y|U|Y|V|
				 *            +-+-+-+-+-+-+-+-+-+-+-+-+
				 */
	int yuvtrail:1;		/* YUV data detection for half edge
				 * 0 leading edge
				 * 1 trailing edge
				 ** leading **   _   _   _   _   _   _
				 *  clock     |_| |_| |_| |_| |_| |_| |_
				 *            +---+---+---+---+---+---+-
				 *  stream    | Y | U | Y | V | Y | U |
				 *            +---+---+---+---+---+---+-
				 ** trailing **_   _   _   _   _   _   _
				 *  clock     | |_| |_| |_| |_| |_| |_|
				 *            +---+---+---+---+---+---+-
				 *  stream    | Y | U | Y | V | Y | U |
				 *            +---+---+---+---+---+---+-
				 */

	void (*hw_init)(void *);
	void (*hw_shutdown)(void *);
	int (*client_ioctl)(struct inode *, struct file *, uint, ulong);

} MP200_CAM_PRELIMINARY;

/*
 * transbuf & frame <- physical buffer
 *
 * --- mmap transfer buffer control --- mapped:1
 *
 *  ** IMG processor mode **	convmode:MP200_CAM_MODE_IMGPR
 *    Camera      transbuf       frame
 *     +---+        +---+        +---+
 *     | @ | ----=> |///| _      |   |
 *     |===|        |///|  |_    |   |
 *     +---+ CamI/F +---+    |   +---+
 *            (DMA) |   |    +=> |///|
 *                  |   | IMG    |///|
 *                  +---+  (DMA) +---+
 *                               (maping)
 *
 *  ** CPU convert mode **	convmode:MP200_CAM_MODE_CPUCV
 *    Camera      transbuf             frame
 *     +---+        +---+              +---+
 *     | @ | ----=> |///| _            |   |
 *     |===|        |///|  |_______    |   |
 *     +---+ CamI/F +---+          |   +---+
 *            (DMA) |   |          +=> |///|
 *                  |   | VIDIOCSYNC   |///|
 *                  +---+  cpu convert +---+
 *                                     (maping)
 *
 *  ** MP200 YUV plane mode **	convmode:MP200_CAM_MODE_YUVPL
 *    Camera        frame
 *     +---+        +---+
 *     | @ | ----=> |///|
 *     |===|        |///|
 *     +---+ CamI/F +---+
 *            (DMA) |   |
 *                  |   |
 *                  +---+
 *                  (maping)
 *
 * --- read transfer buffer control --- mapped:0
 *
 *  ** IMG processor mode **	convmode:MP200_CAM_MODE_IMGPR
 *    Camera      transbuf       frame
 *     +---+        +---+        +---+
 *     | @ | ----=> |///| _      |   |
 *     |===|        |///|  |_    |   |        user buffer
 *     +---+ CamI/F +---+    |   +---+        +---+
 *            (DMA) |   |    +=> |///| ----=> |///|
 *                  |   | IMG    |///| mcopy  |///|
 *                  +---+  (DMA) +---+        +---+
 *
 *  ** CPU convert mode **	convmode:MP200_CAM_MODE_CPUCV
 *    Camera        frame
 *     +---+        +---+
 *     | @ | ----=> |///| _
 *     |===|        |///|  |_______    user buffer
 *     +---+ CamI/F +---+          |   +---+
 *            (DMA) |   |          +=> |///|
 *                  |   | VIDIOCSYNC   |///|
 *                  +---+  cpu convert +---+
 *
 *  ** MP200 YUV plane mode **	convmode:MP200_CAM_MODE_YUVPL
 *    Camera        frame        user buffer
 *     +---+        +---+        +---+
 *     | @ | ----=> |///| ----=> |///|
 *     |===|        |///| mcopy  |///|
 *     +---+ CamI/F +---+        +---+
 *            (DMA) |   |
 *                  |   |
 *                  +---+
 * doubling:<-1
 *   mapped:1  convmode:MP200_CAM_MODE_IMGPR
 *             convmode:MP200_CAM_MODE_CPUCV
 *   mapped:0  convmode:MP200_CAM_MODE_IMGPR
 *
 * doubling:<-0
 *   mapped:1  convmode:MP200_CAM_MODE_YUVPL
 *   mapped:0  convmode:MP200_CAM_MODE_CPUCV
 *             convmode:MP200_CAM_MODE_YUVPL
 *
 * frame    => frame[cam->doubling][x]
 * transbuf => frame[0][x]
 *
 * IRQ transbuf & frame
 *
 * --- mmap IRQ control --- mapped:1
 *  ** grabq **
 *   VIDIOCMCAPTURE   -> mp200_cam_pushq(grabq, frame)
 *   VIDIOCSYNC       -> wait_event_interruptible(grabq.proc_list)
 *   -- Interrupt handling --
 *   CamI/F DMA irq   -> mp200_cam_pullq(grabq) : fetch_frame(frame)
 *
 *     * MP200_CAM_MODE_CPUCV & MP200_CAM_MODE_YUVPL *
 *     CamI/F DMA irq -> wake_up_interruptible(grabq.proc_list)
 *     * MP200_CAM_MODE_IMGPR *
 *     CamI/F DMA irq -> mp200_img_request()
 *     IMG irq        -> wake_up_interruptible(grabq.proc_list)
 *
 * --- read IRQ control --- mapped:0
 *   ** midstream & curframe **
 *   midstream : 0 -> fetch_frame(curframe) + mp200_cam_pushq(grabq, next frame)
 *             : 1 -> wait_event_interruptible(grabq[curframe].proc_list)
 *   -- Interrupt handling --
 *   same as above
 *
 */
enum {
	MP200_CAM_BUF_READY,	/* Ready to grab into */
	MP200_CAM_BUF_GRABBING,	/* In the process of being grabbed into */
	MP200_CAM_BUF_DONE,		/* Finished grabbing, but not been synced yet */
	MP200_CAM_BUF_UNUSED,	/* Unused (no MCAPTURE) */
};

#define MP200_ERR_CAMOVERRUN 	0x01
#define MP200_ERR_CAMDMAERR	0x02
#define MP200_ERR_IMGFAIL	0x04
#define MP200_ERR_IMGERR	0x08
#define MP200_ERR_IMGSTOP	0x10

/* Maximum number of buffers */
#define MP200_CAM_MAX_BUFNBRS	32

/* queues containing the buffer indices */
#define MP200_CAM_QUEUE_SIZE	MP200_CAM_MAX_BUFNBRS

typedef struct mp200_cam_queue {
	unsigned int head;		/* queue head */
	unsigned int tail;		/* queue tail */
	unsigned int len;		/* queue length */
	spinlock_t s_lock;		/* spinlock protecting the queue */
	int buf[MP200_CAM_QUEUE_SIZE];	/* queue contents */
} MP200_CAM_QUEUE;

typedef struct mp200_cam_buf {
	char *vadr;		/* points to actual buffer */
	unsigned long padr;	/* physical buffer address */
} MP200_CAM_BUF;

typedef struct mp200_cam_size {
	int width;
	int height;
} MP200_CAM_SIZE;

typedef struct mp200_cam_frame {
	MP200_CAM_BUF buf[2];
	int count;
	int palette;
	int size_no;
	volatile int state;
	wait_queue_head_t proc_list;		/* wait queue */
	void* cam;
	int adjust:1;
} MP200_CAM_FRAME;

enum {
	MP200_CAM_MODE_INIT = 0,	/* init */
	MP200_CAM_MODE_IMGPR,	/* IMG processor mode */
	MP200_CAM_MODE_CPUCV,	/* CPU convert mode */
	MP200_CAM_MODE_YUVPL,	/* MP200 YUV plane mode */
};

typedef struct mp200_img_req_cam {
	struct mp200_img_req cam_irq;
	struct mp200_img_reg_info img_reg_info[12];
} MP200_IMG_REQ_CAM;

typedef struct mp200_cam_data {
	int open_count;		/* of process that have camera open */
	struct semaphore busy_lock; /* guard against SMP multithreading */

        struct mp200_cam_frame* frame;
	unsigned long fragsize; /* FRAME_NUM-buffering, need a array */

	int cur_frameno;	/* the current frame to grab into */

	unsigned int volatile errstatus;	/* ctrl states */

	struct mp200_cam_queue grabq;	/* queue for buffers to be grabbed */
	struct mp200_img_req_cam* img;

	int vblankval;		/* camera v blank value (clock) */
	int hblankval;		/* camera h blank value (clock) */
	int edge_sample;

	MP200_CAM_GAIN gain;
	MP200_CAM_SIZE max;
	MP200_CAM_SIZE min;

	struct video_device*	vdev;	/* v4l videodev */
	struct video_window	vw;	/* v4l capture area * data custody */

	int nframes;
	int maxdepth;

	void (*hw_init)(void *);
	void (*hw_shutdown)(void *);
	int (*client_ioctl)(struct inode *, struct file *, uint, ulong);
	int convmode;		/* convert mode
				 *  MP200_CAM_MODE_IMGPR : IMG processor mode
				 *  MP200_CAM_MODE_CPUCV : CPU convert mode
				 *  MP200_CAM_MODE_YUVPL : MP200 YUV plane mode
				 */
	int doubling;		/* doubling buffers */

	int setconvmode;	/* user set convert mode */
	int setsize_no;		/* user set read size */
	int setpalette;		/* user set palette */
	int setmirror;		/* user set mirror */
	int setportrait:1;	/* user set portrait monitor */
	int sub_adjust:1;

	int portrait:1;		/* portrait monitor */
	int syncvalid:1;	/* V/H sync type */
	int yuyv:1;		/* stream data sequence for half edge */
	int yuvtrail:1;		/* YUV data detection for half edge */
	int vsynclow:1;		/* v sync */
	int hsynclow:1;		/* h sync */

	int mapped:1;		/* mmap()'ed */
	int stopped:1;		/* might be active but stopped */
} MP200_CAM_DATA;

struct mp200_cam_data *mp200_cam_register(MP200_CAM_PRELIMINARY *pre);
void mp200_cam_unregister(struct mp200_cam_data *cam);

#endif /* MP200_CAM_H */
