/*
 * Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  
 * 02111-1307, USA
 *
 */

#ifndef __ATLAS_CONFIG_H__
#define __ATLAS_CONFIG_H__
 
 /*!
 * @file atlas_config.h
 * @brief This file contains configuration define used by Atlas protocol. 
 *
 * @ingroup ATLAS
 */ 

/*
 * Includes
 */
#include <linux/kernel.h>		/* printk() */
#include <linux/module.h>		/* modules */
#include <linux/init.h>			/* module_{init,exit}() */
#include <linux/devfs_fs_kernel.h>	/* devfs support */
#include <linux/slab.h>			/* kmalloc()/kfree() */
#include <asm/uaccess.h>	        /* copy_{from,to}_user() */

#include "component_spec.h"	        
#include "atlas_external.h"	        

typedef struct timer_list osi_timer;
/*
#define __ALL_TRACES__ 
*/
#define __TEST_CODE_ENABLE__

#define	_A(a)		"Atlas : "a" \n"
#define _K_A(a)         _A(a)
#define _K_D(a)         KERN_DEBUG _A(a)
#define _K_I(a)         KERN_INFO _A(a)
#define _K_W(a)         KERN_WARNING _A(a)

#ifdef          __INIT_TAB_DEF

/*!
 * This structure contains initialized value of register.
 */
static unsigned int tab_init_reg[][2] = {
        {REG_INTERRUPT_MASK_0,   MASK_USED_BITS},		 /*clear all IT mask    */
        {REG_INTERRUPT_STATUS_0, MASK_USED_BITS},		 /*clear all IT status  */
        {REG_INTERRUPT_MASK_1,   MASK_USED_BITS},		 /*clear all IT mask 1  */
        {REG_INTERRUPT_STATUS_1, MASK_USED_BITS},		 /*clear all IT status 1*/
        {-1, -1},
};
#endif

#define         __ATLAS_2_0_

/*!
 * This enumeration define all error code used in Atlas drivers.
 */

enum error_code {
        /*!
         * none error
         */
        ERROR_NONE = 0,
        /*!
         * error process in use
         */
        ERROR_INUSE = -1,
        /*!
         * error in SPI initialize driver
         */
        ERROR_SPIINIT = -2,
        /*!
         * error in initialize section
         */
        ERROR_INIT = -3,
        /*!
         * error number of client overflow
         */
        ERROR_CLIENT_NBOVERFLOW = -4,
        /*!
         * error in malloc function
         */
        ERROR_MALLOC = -5,
        /*!
         * error in un-subscribe event
         */
        ERROR_UNSUBSCRIBE = -6,
        /*!
         * error event occur and not subscribed
         */
        ERROR_EVENT_NOT_SUBSCRIBED = -7,
        /*!
         * error bad call back 
         */
        ERROR_EVENT_CALL_BACK = -8,
        /*!
         * error input arg is wrong. 
         */
        ERROR_BAD_INPUT = -9,
};

#define CHECK_ERROR(a) do { \
        int ret = (a); \
        if(ret != ERROR_NONE) \
return ret; \
} while (0);

/* NULL pointer */
#if !defined(NULL)
#define NULL    ((void *) 0)
#endif

#ifdef __ALL_TRACES__
#define TRACEMSG(fmt,args...) printk(fmt,##args)
#else  /* __ALL_TRACES__ */
#define TRACEMSG(fmt,args...)
#endif /* __ALL_TRACES__ */

#define TRACEMSG_ALL_TIME(fmt,args...)  printk(fmt,##args)

#endif /* __ATLAS_CONFIG_H__ */
