/*
 * Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  
 * 02111-1307, USA
 *
 */

/*!
 * @file atlas_adc_defs.h
 * @brief This header contains all defines. 
 *
 * @ingroup ATLAS_ADC
 */

#ifndef _ATLAS_ADC__DEFS_H
#define _ATLAS_ADC__DEFS_H

#define ATLAS_ADC_DEVICE "/dev/atlas_adc"

#define ADC_TEST_CODE   

#ifndef  __ATLAS_2_0_
#define __TSI_FAILED
#endif 

/*
#define __ALL_TRACES__ 
*/
#ifdef __ALL_TRACES__
#define TRACEMSG_ADC(fmt,args...)  printk(fmt,##args)
#else  /* __ALL_TRACES__ */
#define TRACEMSG_ADC(fmt,args...)
#endif /* __ALL_TRACES__ */

#define ADC_NB_AVAILABLE        2

#define MAX_CHANNEL             7

#define TS_X_MAX                1000
#define TS_Y_MAX                1000

#define TS_X_MIN                80
#define TS_Y_MIN                80

/*
 * ADC 1
 */
 
#define ADC_EN                  0x000001
#define ADC_SGL_CH              0x000002
#define ADC_ADSEL               0x000008
#define ADC_CH_0_POS            5
#define ADC_CH_0_MASK           0x0000E0
#define ADC_CH_1_POS            8
#define ADC_CH_1_MASK           0x000700
#define ADC_DELAY_POS           11
#define ADC_DELAY_MASK          0x07F800
#define ADC_ATO                 0x080000
#define ASC_ADC                 0x100000
#define ADC_CHRGRAW_D5          0x008000

/*
 * ADC 2 - 4
 */
#define ADD1_RESULT_MASK        0x00000FFC	
#define ADD2_RESULT_MASK        0x00FFC000	
#define ADC_TS_MASK             0x00FFCFFC	

/*
 * ADC 3
 */
#define ADC_INC                 0x030000	
#define ADC_BIS                 0x800000	

/*
 * ADC 3
 */
#define ADC_NO_ADTRIG           0x200000
#define ADC_WCOMP               0x040000
#define ADC_WCOMP_H_POS         0
#define ADC_WCOMP_L_POS         9
#define ADC_WCOMP_H_MASK        0x00003F
#define ADC_WCOMP_L_MASK        0x007E00

#define ADC_MODE_MASK           0x00003F

/*!
 * Define state mode of ADC.
 */ 
typedef enum adc_state {
        /*!
         * Free.
         */ 
        ADC_FREE,
        /*!
         * Used.
         */ 
        ADC_USED,
        /*!
         * Monitoring
         */ 
        ADC_MONITORING,
} t_adc_state;

/*!
 * This function is used to update buffer of touch screen value in read mode.
 */ 
void update_buffer(void);

/*!
 * This function read the touch screen value.
 *
 * @param        ts_value    return value of touch screen
 * @param        wait_tsi    if true, this function is synchronous (wait in TSI event).
 * @param        request_adc if true, this function request the ADC before convert.
 *
 * @return       This function returns 0.
 */
int atlas_adc_read_ts(t_touch_screen *ts_value,int wait_tsi);

/*!
 * This function request a ADC.
 *
 * @return      This function returns index of ADC to be used (0 or 1) if successful.
                return -1 if error.            
 */ 
int atlas_adc_request(void);

/*!
 * This function release an ADC.
 *
 * @param        adc_index     index of ADC to be released.
 *
 * @return       This function returns 0 if successful.
 */ 
int atlas_adc_release(int adc_index);

/*!
 * This function enables the touch screen read interface.
 *
 * @return       This function returns 0 if successful.
 */ 
int atlas_adc_install_ts(void);

/*!
 * This function disables the touch screen read interface.
 *
 * @return       This function returns 0 if successful.
 */ 
int atlas_adc_remove_ts(void);

#endif /* _ATLAS_ADC__DEFS_H */
