/*
 * Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Driver for the Freescale Semiconductor MXC IPU
 *
 */

#include "asm/arch/ipu/ipu.h"
#include "asm/arch/ipu/mt9v143.h"
#include "asm/arch/mxc_i2c.h"

static mt9v143_conf mt9v143;
static int gMajor_mt9v143 = 0;
static u8 open_count = 0;
static u8 camera_power_count = 0;

/*
 * Function definitions
 */

static u16
endian_swap16(u16 data)
{
        u16 temp;

        temp = data;
        temp = ((data >> 8) & 0xff) | ((data << 8) & 0xff00);
        //printk(" temp %x\n", temp);

        return temp; 
}

/*!
 * Initialize mt9v143_ConfigLib
 * Sensor initialization to known states
 *
 * @param       coreReg       Core Registers
 * @param       ifpReg        IFP Register
 *
 * @return status
 */
static u8 
mt9v143_ConfigLib(mt9v143_coreReg *coreReg, mt9v143_IFPReg *ifpReg)
{
        u8 reg[1];
        u16 data;
        u8 error = 0;

        FUNC_START;

        /* 
         * setup to sensor core registers
         */
        reg[0] = MT9V143I_ADDR_SPACE_SEL;
        data = (u16) coreReg->addressSelect;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);
  	  
#ifdef CONFIG_VIRTIO_SUPPORT
#else
        //soft reset
        printk("mt9v143_ConfigLib: soft reset the sensor\n");
        reg[0] = MT9V143S_RESET;
        data = 1;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);
        msleep(10);
        data = 0;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);
#endif

        // switch to sensor registers
        reg[0] = MT9V143I_ADDR_SPACE_SEL;
        data = (u16) coreReg->addressSelect;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);
  	  
        // enable changes and put the Sync bit on
        reg[0] = MT9V143S_OUTPUT_CTRL;
        data = MT9V143S_OUTCTRL_SYNC | MT9V143S_OUTCTRL_CHIP_ENABLE;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

        // shutter width
        reg[0] = MT9V143S_SHUTTER_WIDTH;
        data = (u16) coreReg->shutterWidth;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

        // min PIXCLK - Default
        reg[0] = MT9V143S_PIXEL_CLOCK_SPEED;
        data = (u16) coreReg->pixelClockSpeed;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

        //Setup image flipping / Dark rows / row/column skip
        reg[0] = MT9V143S_READ_MODE;
        data = (u16) coreReg->readMode;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

        // changes take effect
        reg[0] = MT9V143S_OUTPUT_CTRL;
        data = MT9V143S_OUTCTRL_CHIP_ENABLE;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

        // Debug
        //mxc_i2c_read(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2); 
        //printk(" ************Sec MT9V143S_OUTPUT_CTRL %x.\n", data);

#ifdef CONFIG_VIRTIO_SUPPORT  
        //First Row
        reg[0] = MT9V143S_ROW_START;
        data = (u16) coreReg->rowStart;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

        //First column
        reg[0] = MT9V143S_COLUMN_START;
        data = (u16) coreReg->columnStart;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

    	//Window Height
        reg[0] = MT9V143S_WIN_HEIGHT;
        data = (u16) coreReg->windowHeight;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

     	//Window Width
        reg[0] = MT9V143S_WIN_WIDTH;
        data = (u16) coreReg->windowWidth;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

        // min H-blank - Default
        reg[0] = MT9V143S_HOR_BLANKING;
        data = (u16) coreReg->horizontalBlanking;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

        // min V-blank - Default
        reg[0] = MT9V143S_VER_BLANKING;
        data = (u16) coreReg->verticalBlanking;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

     	//zoom 0
        reg[0] = MT9V143S_DIGITAL_ZOOM;
        data = (u16) coreReg->digitalZoom;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);
#endif

        /* 
         * setup to IFP registers
         */
        //decimation control, select IFP register
        reg[0] = MT9V143I_ADDR_SPACE_SEL;
        data = (u16) ifpReg->addrSpaceSel;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

        // QQVGA
        reg[0] = MT9V143I_DECIMATION_CTRL;
        data = (u16) ifpReg->decimationControl;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

        reg[0] = MT9V143I_FORMAT_CONTROL;
        //mxc_i2c_read(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2); 
        //printk(" ************MT9V143_SetOutputFormat format read back %x.\n", data);
        data = (u16) ifpReg->formatControl;
        data = endian_swap16(data); 
        //printk(" ************MT9V143_SetOutputFormat format write to %x.\n", data);
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

        FUNC_END;

        return error;
}

/*!
 * mt9v143_PowerOn function
 * Turn Sensor power On/Off
 *
 * @param       cameraOn      true to turn camera on, otherwise shut down
 *
 * @return status
 */
u8 
mt9v143_PowerOn(bool cameraOn)
{
        u8 reg[1];
        u16 data;
        u8 error = 0;

        FUNC_START;

        if (cameraOn == true)
                camera_power_count++;
        else if (camera_power_count != 0)
                camera_power_count--;
                 
        // switch to sensor registers
        reg[0] = MT9V143I_ADDR_SPACE_SEL;
        data = (u16) MT9V143I_SEL_SCA;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

        reg[0] = MT9V143S_OUTPUT_CTRL;
        // changes take effect
        data = ((cameraOn == false) && (camera_power_count== 0)) ? 0 : MT9V143S_OUTCTRL_CHIP_ENABLE;
        data = endian_swap16(data); 
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

        FUNC_END;

        return error;
}

/*!
 * MT9V143 Sensor Configuration and Initialization
 * Initialize the CSI hardware interface to sensor 
 *
 * @return  Error code indicating success or failure
 */
static int 
mt9v143_sensorconfig(void)
{
        u8 err = 0;
        ipu_sens_conf *param;

        FUNC_START;

        param = (ipu_sens_conf *) 
                kmalloc(sizeof(ipu_sens_conf), GFP_KERNEL);
        gipu_param.sens = param;

        if (!gipu_param.sens) 
                return err;

        memset(param, 0, sizeof(ipu_sens_conf));
                        
        /*
         * Check if IPU General configuration has been done.
         */
        if (!gipu_param.ipu_genConfigured) {
                mxc_ipu_genconf();
        }

        /*
         * Micron MT9M143 Init 
         * pixel format :YUV422 ,8 bit per component, color order:UYVY 
         * BT.656 with synchronization codes. 
         */
#ifdef CONFIG_VIRTIO_SUPPORT
        param->sens_clk_src = 0; 
        param->sens_prtcl = 0x2; 
        param->sens_pix_clk_pol = 1; 
        param->sens_frm_width = 160;
        param->sens_frm_height = 120;
        param->div_ratio = 0; 
#else
        param->sens_clk_src = 1; //clock source input is IPP_IND_SENSB_SENS_CLK clock
        param->sens_prtcl = 0x0; //gated 
        param->sens_pix_clk_pol = 0; //pixel clock is inverted before applied to internal circuitry 
        param->sens_frm_width = 160 -1;//sensor frame column number minus 1
        param->sens_frm_height = 120 -1; //sensor frame rows number minus 1
        param->div_ratio = 2; 
#endif
        param->sens_data_format = 0x2; //yuv 422 
        param->data_witdh = 0x1; //8 bits per color 
        param->data_pol = 0x0; //data lines are directly applied to internal circuitry
        param->ccir_precom = 0xff0000;//sequence which comes before the CCIR command
        param->strt_fld0_blnk_1st = 0x6; //start of blanking line command) 
        param->end_fld0_actv = 0x4; //end of active line command mode
        param->strt_fld0_actv = 0x0;//start of active line command mode  
        /*
         * Call Sensor Configuration library
         */
        sensConfigLib(param); 
        FUNC_END;

        return err;
}

/*!
 * MT9V143 cleanup function
 *
 * @return  Error code indicating success or failure
 */
static void
mt9v143_sensorClean(void)
{
        FUNC_START;

        kfree(gipu_param.sens); 
        gipu_param.sens = 0;

        FUNC_END;
}

/*!
* Set sensor to test mode, which will generate chessboard.
*		 
* @return none
*/
void 
mt9v143_TestPattern(void)
{
        u8 reg[1];
        u16 data;

        FUNC_START;

        // set the chess box.
        // switch to sensor registers
        reg[0] = MT9V143I_ADDR_SPACE_SEL;
        data = MT9V143I_SEL_SCA;
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

        reg[0] = MT9V143S_TEST_DATA;
        data = 0;
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

        reg[0] = MT9V143S_OUTPUT_CTRL;
        data = 0x4a;
        mxc_i2c_write(I2C_BUS, MT9V143_I2C_ADDRESS, reg, 1, (u8 *) &data, 2);

        FUNC_END;
}

/*!
 * Entry point for the MT9V143 usecase
 *
 * @return  Error code indicating success or failure
 */
static int 
usecase_mt9v143(void)
{
        u8 err = 0;
        mt9v143_coreReg *coreReg;
        mt9v143_IFPReg *ifpReg;

        FUNC_START;

        coreReg = (mt9v143_coreReg *) 
                   kmalloc(sizeof(mt9v143_coreReg), GFP_KERNEL);
        mt9v143.coreReg = coreReg;

        if (!mt9v143.coreReg) 
                return err;

        memset(coreReg, 0, sizeof(mt9v143_coreReg)); 

        ifpReg = (mt9v143_IFPReg *) 
                          kmalloc(sizeof(mt9v143_IFPReg), GFP_KERNEL);
        mt9v143.ifpReg = ifpReg;

        if (!mt9v143.ifpReg) 
                return err;

        memset(ifpReg, 0, sizeof(mt9v143_IFPReg)); 

        /* 
         * setup default sensor core registers
         */
        coreReg->addressSelect = MT9V143I_SEL_SCA;  
        coreReg->rowStart = 188; 			 
        coreReg->columnStart = 254;         
        coreReg->windowHeight = 487;       
        coreReg->windowWidth = 647;        
        coreReg->horizontalBlanking = 0x09;  
        coreReg->verticalBlanking =	0x64;    
        coreReg->pixelClockSpeed = 0;   
        coreReg->digitalZoom = 0;      

#ifdef CONFIG_VIRTIO_SUPPORT
        coreReg->readMode = 0x1001;
        coreReg->shutterWidth = 0x1B9; 		 
#else
        coreReg->readMode = 0xd0a1;
        coreReg->shutterWidth = 0x100; 		 
#endif           
        /* 
         * setup to IFP registers
         */
        ifpReg->addrSpaceSel = MT9V143I_SEL_IFP;
        ifpReg->formatControl =	0xcd00;	      // CCIR
        ifpReg->decimationControl = MT9V143_OutputResolution_QQVGA;

        mt9v143_ConfigLib(coreReg, ifpReg);

        mt9v143_PowerOn(false);

        FUNC_END;

        return err;
}

/*!
 * Exit and cleanup for the MT9V143 usecase
 *
 * @return  Error code indicating success or failure
 */
static void __exit 
usecase_mt9v143Clean(void)
{
        FUNC_START;

        if (mt9v143.coreReg)
        { 
                kfree(mt9v143.coreReg); 
                mt9v143.coreReg = 0;
        }

        if (mt9v143.ifpReg)
        { 
                kfree(mt9v143.ifpReg); 
                mt9v143.ifpReg = 0;
        }

        FUNC_END;
}

static int 
mt9v143_open(struct inode *inode, struct file *filp)
{
        if (open_count++ == 0)
                {
                mt9v143_PowerOn(true);
                if (gipu_param.prp_vf_loaded == true)
                        {
                        PrpVF_Select(false, NULL);
                        }
                }
        return 0;
}

static int 
mt9v143_release(struct inode *inode, struct file *filp)
{
        if (--open_count == 0)
                {  
                if (gipu_param.prp_vf_loaded == true)
                        {
                        PrpVF_DeSelect();
                        }
                mt9v143_PowerOn(false);
                } 
        return 0;	
}

// This function is only for debug test purpose, do not use it for normal usage.
static ssize_t 
mt9v143_read(struct file *file, char __user *buffer, size_t count, loff_t *ppos)
{
        msleep(50000);
        return 0;
}

static int 
mt9v143_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg)
{
        switch (cmd) 
        {
                case MT9V143_IOCTL_POWER_ON:
                        mt9v143_PowerOn(true);
                        break;

                case MT9V143_IOCTL_POWER_OFF:
                        mt9v143_PowerOn(false);
                        break;

                default:
                        printk(" mt9v143_ioctl not supported ioctls\n");
                        return -1;
        }

        return 0;
}

static struct file_operations mt9v143_fops = {
        .owner =   THIS_MODULE,
        .open =    mt9v143_open,
        .release = mt9v143_release,
        .read =    mt9v143_read,
        .ioctl =   mt9v143_ioctl,
};

/*!
 * MT9V143 init function
 *
 * @return  Error code indicating success or failure
 */
static __init int 
mt9v143_init(void)
{
        u8 err = 0;
        FUNC_START;

        mt9v143_sensorconfig();
        usecase_mt9v143();

        if (gipu_param.use_v4l2 == true);
                camera_init();

        gMajor_mt9v143 = register_chrdev(0,"mt9v143", &mt9v143_fops);
        if (gMajor_mt9v143 < 0) {
                printk("mt9v143_init: unable to register driver\n");
                return gMajor_mt9v143;
        }

        devfs_mk_cdev(MKDEV(gMajor_mt9v143,0), S_IFCHR | S_IRUGO | S_IWUSR, "mt9v143");

        FUNC_END;
        return err;
}

/*!
 * MT9V143 cleanup function
 *
 * @return  Error code indicating success or failure
 */
static void __exit 
mt9v143_clean(void)
{
        FUNC_START;

        if (gipu_param.use_v4l2 == true);
                camera_exit();

        if(gMajor_mt9v143 > 0)
        {
                devfs_remove("mt9v143");
                unregister_chrdev(gMajor_mt9v143, "mt9v143");
        }
               
        mt9v143_sensorClean(); 
        usecase_mt9v143Clean();

        FUNC_END;
}

module_init(mt9v143_init);
module_exit(mt9v143_clean);

MODULE_AUTHOR("Freescale Semiconductor");
MODULE_DESCRIPTION("Mt9v143 Driver");
MODULE_LICENSE("GPL");

EXPORT_SYMBOL(mt9v143_TestPattern);
EXPORT_SYMBOL(mt9v143_PowerOn);
