/*
 * Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Driver for the Freescale Semiconductor
 *
 */

/*!
 * @file ipu_alloc.h
 * 
 * @brief simple ipu allocation driver for linux
 * 
 * @ingroup IPU
 */
#ifndef _IPU_ALLOC_H_
#define _IPU_ALLOC_H_

#define IPU_PAGE_ALIGN   ((u32) 0x00001000)

typedef struct _memDesc
{
        u32 start;
        u32 end;
        struct _memDesc *next;
} memDesc;

/*!
 * IPU_PoolInitialize
 *
 * @param       memPool         start address of the pool
 * @param       poolSize        memory pool size
 * @param       alignment       alignment for example page alignmnet will be 0x1000
 *
 * @return      0 for success  -1 for errors
 */
int IPU_PoolInitialize(u32 memPool, u32 poolSize, u32 alignment);

/*!
 * IPU_Malloc
 *
 * @param       size        memory pool size
 *
 * @return      physical address, 0 for error
 */
u32 IPU_Malloc(u32 size);

/*!
 * IPU_Free     
 *
 * @param       physical        physical address try to free
 *
 */
void IPU_Free(u32 physical);

#endif
