/*
 * Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Driver for the Freescale Semiconductor MXC IPU
 *
 */

/*!
 * @file ipu_prp_vf_adc.c
 * 
 * @brief IPU Use case for PRP-VF
 * 
 * @ingroup IPU
 */

#include "../drivers/media/video/mxc_v4l2_capture.h"
#include "../ipu.h"
#include "../ipu_prv.h"
#include "asm/arch/board.h"
#include "asm-arm/io.h"
#include "../ipu_regs.h"

/*
 * Function definitions
 */

/*!
 * prpvf_start - start the vf task
 *
 * @param private    cam_data * mxc v4l2 main structure
 *
 */
static void 
prpvf_start(void *private)
{
        cam_data *cam = (cam_data *) private;
        ipu_channel_params_t vf;
        ipu_channel_params_t params;
        u32 format;
        u32 size = 2;

        FUNC_START;

        if (!cam) {
                DPRINTK("prpvf_start private is NULL\n");
                return;
        }

        if (cam->overlay_active == true) { 
                DPRINTK("prpvf_start already start.\n");
                return;
        }

        memset(&vf, 0, sizeof(ipu_channel_params_t));
        vf.csi_prp_vf_adc.in_width = MXC_SENSOR_SCREEN_WIDTH;
        vf.csi_prp_vf_adc.in_height = MXC_SENSOR_SCREEN_HEIGHT;
        vf.csi_prp_vf_adc.in_pixel_fmt = IPU_PIX_FMT_YUYV;
        vf.csi_prp_vf_adc.out_width = cam->win.w.width;
        vf.csi_prp_vf_adc.out_height = cam->win.w.height;
        vf.csi_prp_vf_adc.graphics_combine_en = 0;
        vf.csi_prp_vf_adc.out_left = cam->win.w.left;
        vf.csi_prp_vf_adc.out_top = cam->win.w.top;

        if (cam->rotation >= IPU_ROTATE_VERT_FLIP) {
#ifdef CONFIG_FB_MXC_24BPP
                format = IPU_PIX_FMT_BGR24;
                size = 3;
                DPRINTK("prpvf_start BGR24\n");
#endif
#ifdef CONFIG_FB_MXC_16BPP
                format = IPU_PIX_FMT_RGB565;
                size = 2;
                DPRINTK("prpvf_start RGB565\n");
#endif
#ifdef CONFIG_FB_MXC_32BPP
                format = IPU_PIX_FMT_BGR32;
                size = 4;
                DPRINTK("prpvf_start BGR32\n");
#endif

                size = cam->win.w.width * cam->win.w.height	* size;
                vf.csi_prp_vf_adc.out_pixel_fmt = IPU_PIX_FMT_RGB565;
                ipu_uninit_channel(CSI_PRP_VF_MEM);
                ipu_init_channel(CSI_PRP_VF_MEM, &vf);
                if (cam->vf_bufs[0])
                        IPU_Free((u32)(cam->vf_bufs[0]));
                if (cam->vf_bufs[1])
                        IPU_Free((u32)(cam->vf_bufs[1]));
                cam->vf_bufs[0] = (void *)IPU_Malloc(PAGE_ALIGN(size));
                cam->vf_bufs[1] = (void *)IPU_Malloc(PAGE_ALIGN(size));
                if ((cam->vf_bufs[0] == NULL) || (cam->vf_bufs[1] == NULL)) {
                        DPRINTK("prpvf_start: Error to allocate vf buffer\n");
                        return;
                }
                ipu_init_channel_buffer(CSI_PRP_VF_MEM, IPU_OUTPUT_BUFFER,
                                        IPU_PIX_FMT_RGB565, vf.csi_prp_vf_mem.out_height, 
                                        vf.csi_prp_vf_mem.out_width, 
                                        vf.csi_prp_vf_mem.out_height, 
                                        IPU_ROTATE_NONE,
                                        cam->vf_bufs[0], cam->vf_bufs[1]);

                if (cam->rot_vf_bufs[0]) {
                        IPU_Free((u32)(cam->rot_vf_bufs[0]));
                }
                if (cam->rot_vf_bufs[1]) {
                        IPU_Free((u32)(cam->rot_vf_bufs[1]));
                }
                cam->rot_vf_bufs[0] = (void *) IPU_Malloc(PAGE_ALIGN(size));
                cam->rot_vf_bufs[1] = (void *) IPU_Malloc(PAGE_ALIGN(size));
                if ((cam->rot_vf_bufs[0] == NULL) || 
                    (cam->rot_vf_bufs[1] == NULL))	{
                        DPRINTK("prpvf_start: alloc rot_vf_bufs.\n");
                        return;
                }

                ipu_uninit_channel(MEM_ROT_VF_MEM);
                if (ipu_init_channel(MEM_ROT_VF_MEM, NULL) != 0) {
                        DPRINTK("prp_enc_set_format :Error "
                                "MEM_ROT_VF_MEM channel\n");
                        return;
                }

                if (ipu_init_channel_buffer(MEM_ROT_VF_MEM, IPU_INPUT_BUFFER, 
                                IPU_PIX_FMT_RGB565, vf.csi_prp_vf_mem.out_height, 
                                vf.csi_prp_vf_mem.out_width, 
                                vf.csi_prp_vf_mem.out_height, cam->rotation, 
                                cam->vf_bufs[0], cam->vf_bufs[1]) != 0) {
                        DPRINTK("prpvf_start: Error "
                                "MEM_ROT_VF_MEM input buffer\n");
                        return;
                }

                if (ipu_init_channel_buffer(MEM_ROT_VF_MEM, IPU_OUTPUT_BUFFER, 
                                IPU_PIX_FMT_RGB565, vf.csi_prp_vf_mem.out_width, 
                                vf.csi_prp_vf_mem.out_height, 
                                vf.csi_prp_vf_mem.out_width, IPU_ROTATE_NONE, 
                                cam->rot_vf_bufs[0], cam->rot_vf_bufs[1]) != 0){
                        DPRINTK("prpvf_start: Error "
                                "MEM_ROT_VF_MEM output buffer\n");
                        return;
                }

                if (ipu_link_channels(CSI_PRP_VF_MEM, MEM_ROT_VF_MEM) < 0) {
                        DPRINTK("prpvf_start: Error "
                                "linking CSI_PRP_VF_MEM-MEM_ROT_VF_MEM\n");
                        return;
                }

                ipu_disable_irq(IPU_IRQ_ADC_SYS2_EOF);
                ipu_disable_irq(IPU_IRQ_ADC_SYS1_EOF);
                ipu_uninit_channel(ADC_SYS1);
                ipu_disable_channel(ADC_SYS1,false);

                params.adc_sys1.disp = DISP0;
                params.adc_sys1.ch_mode = WriteTemplateNonSeq;
                params.adc_sys1.x_pos = cam->win.w.left;
                params.adc_sys1.y_pos = cam->win.w.top;
                ipu_init_channel(ADC_SYS1, &params);

                if (ipu_init_channel_buffer(ADC_SYS1, IPU_INPUT_BUFFER, 
                                IPU_PIX_FMT_RGB565, vf.csi_prp_vf_mem.out_width, 
                                vf.csi_prp_vf_mem.out_height, 
                                vf.csi_prp_vf_mem.out_width, IPU_ROTATE_NONE, 
                                cam->rot_vf_bufs[0], cam->rot_vf_bufs[1]) != 0){
                        DPRINTK("prpvf_start: Error "
                                "initializing ADC SYS1 buffer\n");
                        return;
                }

                if (ipu_link_channels(MEM_ROT_VF_MEM, ADC_SYS1) < 0) {
                        DPRINTK("prpvf_start: Error "
                                "linking MEM_ROT_VF_MEM-ADC_SYS1\n");
                        return;
                }

                ipu_enable_channel(CSI_PRP_VF_MEM);
                ipu_enable_channel(MEM_ROT_VF_MEM);
                ipu_enable_channel(ADC_SYS1);

                ipu_select_buffer(CSI_PRP_VF_MEM, IPU_OUTPUT_BUFFER, 0);
                ipu_select_buffer(MEM_ROT_VF_MEM, IPU_INPUT_BUFFER, 0);
                ipu_select_buffer(MEM_ROT_VF_MEM, IPU_OUTPUT_BUFFER, 0);
                ipu_select_buffer(ADC_SYS1, IPU_OUTPUT_BUFFER, 0);
                ipu_select_buffer(CSI_PRP_VF_MEM, IPU_OUTPUT_BUFFER, 1);
                ipu_select_buffer(MEM_ROT_VF_MEM, IPU_INPUT_BUFFER, 1);
                ipu_select_buffer(MEM_ROT_VF_MEM, IPU_OUTPUT_BUFFER, 1);
                ipu_select_buffer(ADC_SYS1, IPU_OUTPUT_BUFFER, 1);
        } 
        else {
                vf.csi_prp_vf_adc.out_pixel_fmt = IPU_PIX_FMT_BGR32;
                ipu_uninit_channel(CSI_PRP_VF_ADC);
                ipu_init_channel(CSI_PRP_VF_ADC, &vf);
                if (ipu_init_channel_buffer(CSI_PRP_VF_ADC, IPU_OUTPUT_BUFFER, 
                        IPU_PIX_FMT_BGR32, cam->win.w.width, cam->win.w.height,
                        cam->win.w.width, IPU_ROTATE_NONE,
                        NULL, NULL) != 0) {
                        DPRINTK("prpvf_start: Error "
                                "initializing CSI_PRP_VF_MEM\n");
                        return;
                }

                ipu_enable_channel(CSI_PRP_VF_ADC);
        }

        cam->overlay_active = true; 
        FUNC_END;
}

/*!
 * prpvf_stop - stop the vf task
 *
 * @param private    cam_data * mxc v4l2 main structure
 *
 */
static void 
prpvf_stop(void *private)
{
        cam_data *cam = (cam_data *) private;
        FUNC_START;

        if (cam->overlay_active == false) 
                return;

        if (cam->rotation >= IPU_ROTATE_VERT_FLIP) {
                ipu_unlink_channels(CSI_PRP_VF_MEM, MEM_ROT_VF_MEM);
                ipu_unlink_channels(MEM_ROT_VF_MEM, ADC_SYS1);

                ipu_uninit_channel(MEM_ROT_VF_MEM);
                ipu_uninit_channel(CSI_PRP_VF_MEM);
                ipu_uninit_channel(ADC_SYS1);

                ipu_disable_channel(MEM_ROT_VF_MEM, true);
                ipu_disable_channel(CSI_PRP_VF_MEM, true);
                ipu_disable_channel(ADC_SYS1, true);

                if (cam->vf_bufs[0]) {
                        IPU_Free((u32)(cam->vf_bufs[0]));
                        cam->vf_bufs[0] = NULL; 
                }  
                if (cam->vf_bufs[1]) {
                        IPU_Free((u32)(cam->vf_bufs[1]));
                        cam->vf_bufs[1] = NULL; 
                }
                if (cam->rot_vf_bufs[0]) {
                        IPU_Free((u32)(cam->rot_vf_bufs[0]));
                        cam->vf_bufs[0] = NULL; 
                }  
                if (cam->rot_vf_bufs[1]) {
                        IPU_Free((u32)(cam->rot_vf_bufs[1]));
                        cam->vf_bufs[1] = NULL; 
                }
                ipu_enable_irq(IPU_IRQ_ADC_SYS1_EOF);
                ipu_enable_irq(IPU_IRQ_ADC_SYS2_EOF);
        }
        else {
                ipu_uninit_channel(CSI_PRP_VF_ADC);
                ipu_disable_channel(CSI_PRP_VF_ADC, true);
        }
        cam->overlay_active = false; 
        FUNC_END;
}

/*!
 * function to select PRP-VF as the working path
 *
 * @param private    cam_data * mxc v4l2 main structure
 *
 * @return  None
 */
void 
prp_vf_adc_select(void *private)
{
        cam_data *cam;
        FUNC_START;
        if (private) {
                cam = (cam_data *) private;
                cam->output = 1;
                cam->vf_start_adc = prpvf_start;
                cam->vf_stop_adc = prpvf_stop;
                cam->overlay_active = false;  
        }
        FUNC_END;
}

/*!
 * function to de-select PRP-VF as the working path
 *
 * @param private    cam_data * mxc v4l2 main structure
 *
 * @return  None
 */
void 
prp_vf_adc_deselect(void *private)
{
        cam_data *cam;
        FUNC_START;
        prpvf_stop(private);

        if (private) {
                cam = (cam_data *) private;
                cam->vf_start_adc = NULL;
                cam->vf_stop_adc = NULL;
                cam->output = 0;
        }
        FUNC_END;
}

/*!
 * Init viewfinder task.
 *
 * @return  Error code indicating success or failure
 */
__init int 
prp_vf_adc_init(void)
{
        u8 err = 0;
        FUNC_START;

        FUNC_END;
        return err;
}

/*!
 * Deinit viewfinder task.
 *
 * @return  Error code indicating success or failure
 */
void __exit 
prp_vf_adc_exit(void)
{
        FUNC_START;
        FUNC_END;
}

module_init(prp_vf_adc_init);
module_exit(prp_vf_adc_exit);

EXPORT_SYMBOL(prp_vf_adc_select);
EXPORT_SYMBOL(prp_vf_adc_deselect);

MODULE_AUTHOR("Freescale Semiconductor");
MODULE_DESCRIPTION("IPU PRP VF Driver");
MODULE_LICENSE("GPL");
