/*
 * Copyright 2005 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*!
 * @file dvfs.h
 *
 * @brief MXC dvfs header file. 
 * 
 * @ingroup PM
 */ 
#ifndef __DVFS_H__
#define __DVFS_H__

#include "dvfs_dptc_struct.h"

/*
 * LTR0 register offsets
 */
#define DVFS_DIV3CK_OFFSET 1
#define DVFS_DIV3CK_MASK   0x00000003
#define DVFS_DNTHR_OFFSET  16
#define DVFS_DNTHR_MASK    0x003f0000
#define DVFS_UPTHR_OFFSET  22
#define DVFS_UPTHR_MASK    0x0fc00000

#define SET_DIV3CK(x) _reg_CCM_LTR0 = \
(( _reg_CCM_LTR0 & ~DVFS_DIV3CK_MASK ) | x<<DVFS_DIV3CK_OFFSET)

#define SET_DNTHR(x) _reg_CCM_LTR0 = \
(( _reg_CCM_LTR0 & ~DVFS_DNTHR_MASK ) | x<<DVFS_DNTHR_OFFSET)

#define SET_UPTHR(x) _reg_CCM_LTR0 = \
(( _reg_CCM_LTR0 & ~DVFS_UPTHR_MASK ) | x<<DVFS_UPTHR_OFFSET)

/*
 * LTR1 register offsets
 */
#define DVFS_PNCTHR_OFFSET 0
#define DVFS_PNCTHR_MASK   0x0000003f
#define DVFS_UPCNT_OFFSET  6
#define DVFS_UPCNT_MASK    0x00003fc0
#define DVFS_DNCNT_OFFSET  14
#define DVFS_DNCNT_MASK    0x003fc000
#define DVFS_LTBRSR_OFFSET 22
#define DVFS_LTBRSR_MASK   0x00400000
#define DVFS_LTBRSH_OFFSET 23
#define DVFS_LTBRSH_MASK   0x00800000

#define SET_PNCTHR(x) _reg_CCM_LTR1 = \
(( _reg_CCM_LTR1 & ~DVFS_PNCTHR_MASK ) | x<<DVFS_PNCTHR_OFFSET)
#define SET_UPCNT(x) _reg_CCM_LTR1 = \
(( _reg_CCM_LTR1 & ~DVFS_UPCNT_MASK ) | x<<DVFS_UPCNT_OFFSET)
#define SET_DNCNT(x) _reg_CCM_LTR1 = \
(( _reg_CCM_LTR1 & ~DVFS_DNCNT_MASK ) | x<<DVFS_DNCNT_OFFSET)
#define SET_LTBRSH() _reg_CCM_LTR1 |= DVFS_LTBRSH_MASK
#define SET_LTBRSR() _reg_CCM_LTR1 |= DVFS_LTBRSR_MASK

/*
 * LTR2 register offsets
 */
#define DVFS_EMAC_OFFSET 0
#define DVFS_EMAC_MASK   0x000001ff
#define DVFS_LTR2_WSW_OFFSET 11
#define DVFS_LTR2_WSW_MASK(x) (0x7<<((x-9)*3+DVFS_LTR2_WSW_OFFSET))
#define SET_EMAC(x) _reg_CCM_LTR2 = \
(( _reg_CCM_LTR2 & ~DVFS_EMAC_MASK ) | x<<DVFS_EMAC_OFFSET)
#define SET_LTR2_WSW(x,w) _reg_CCM_LTR2 = \
(( _reg_CCM_LTR2 & ~DVFS_LTR2_WSW_MASK(x)) | (w<<((x-9)*3+DVFS_LTR2_WSW_OFFSET)))

/*
 * LTR3 register offsets
 */
#define DVFS_LTR3_WSW_OFFSET 5
#define DVFS_LTR3_WSW_MASK(x) (0x7<<(x*3+DVFS_LTR3_WSW_OFFSET))
#define SET_LTR3_WSW(x,w) _reg_CCM_LTR3 = \
(( _reg_CCM_LTR3 & ~DVFS_LTR3_WSW_MASK(x)) | (w<<((x-9)*3+DVFS_LTR3_WSW_OFFSET)))

/*
 * PMCR0 register offsets
 */
#define DVFS_FSVAI_OFFSET 13
#define DVFS_LBCF_OFFSET  20
#define DVFS_LBFL_OFFSET  20
#define DVFS_VSCNT_OFFSET 24
#define DVFS_UDSC_OFFSET  27
#define DVFS_DFSUP_OFFSET 30

/*
 * PMCR1 register offsets
 */
#define DVFS_DVGP_OFFSET 0
#define SET_DVGP(x) _reg_CCM_PMCR1 = ((_reg_CCM_PMCR1 & ~(0xf<<DVFS_DVGP_OFFSET)) | (x<<DVFS_DVGP_OFFSET))

/*!
 * Unmask DVFS interrupt
 */
#define UNMASK_DVFS_INT()	_reg_CCM_PMCR0 &= ~FSVAIM 
/*!
 * Mask DVFS interrupt
 */
#define MASK_DVFS_INT()	        _reg_CCM_PMCR0 |= FSVAIM

/*!
 * Unmask DVFS log buffer interrupt
 */
#define UNMASK_DVFS_LB_INT()	_reg_CCM_PMCR0 &= ~LBMI
/*!
 * Mask DVFS log buffer interrupt
 */
#define MASK_DVFS_LB_INT()	_reg_CCM_PMCR0 |= LBMI

/*
 * MCU will get DVFS interrupt
 */
#define SET_DVFIS() _reg_CCM_PMCR0 |= DVFIS;

/*
 * Always give DVFS interrupt
 */
#define SET_DVFEV() _reg_CCM_PMCR0 |= DVFEV;

/*!
 * Enable DVFS hardware
 */
#define ENABLE_DVFS()		_reg_CCM_PMCR0 |= DVFEN

/*!
 * Disable DVFS hardware
 */
#define DISABLE_DVFS()		_reg_CCM_PMCR0 &= ~DVFEN

/*!
 * Read the FSVAI bits from the CCM
 */
#define GET_FSVAI()		(_reg_CCM_PMCR0 & FSVAI)>>DVFS_FSVAI_OFFSET

/*!
 * Read the DFSUP bits from the CCM
 */
#define GET_DFSUP()		(_reg_CCM_PMCR0 & DFSUP)>>DVFS_DFSUP_OFFSET

/*!
 * Read the LBFL bits from the CCM
 */
#define GET_LBFL(pmcr0)		(pmcr0 & LBFL)>>DVFS_LBFL_OFFSET

/*!
 * Read the LBCF bits from the CCM
 */
#define GET_LBCF(pmcr0)		(pmcr0 & LBCF)>>DVFS_LBFL_OFFSET

/*!
 * Set DVSUP bits
 */
#define SET_DVSUP(pmcr0, x)     pmcr0 = ((pmcr0 & ~DVSUP) | x<<DVFS_DVSUP_OFFSET)

/*!
 * Set DFSUP bits
 */
#define SET_DFSUP(pmcr0, x)     pmcr0 = ((pmcr0 & ~DFSUP) | x<<DVFS_DFSUP_OFFSET)

/*
 * Set UDSC (up or down)
 */
#define SET_UDSC(pmcr0, x)      pmcr0 = ((pmcr0 & ~UDSC) | x<<DVFS_UDSC_OFFSET)

#define SET_VSCNT(pmcr0, x)      pmcr0 = ((pmcr0 & ~VSCNT) | x<<DVFS_VSCNT_OFFSET)

/*!
 * If value equal to FSVAI bits indicates working point decrease
 */
#define DVFS_DECREASE		(unsigned long)0x2

/*!
 * If value equal to FSVAI bits indicates working point increase
 */
#define DVFS_INCREASE		(unsigned long)0x1

/*!
 * If value equal to FSVAI bits indicates working point increase to maximum
 */
#define DVFS_EMERG		(unsigned long)0x3

/*!
 * This function is called for module initialization.
 * It initializes the driver data structures and sets up the DVFS hardware.
 *
 * @return   0 to indicate success else returns a negative number. 
 *
 */   
int __init init_dvfs_controller(dvfs_dptc_params_s *params);

/*!
 * This function enables the DVFS module. this function updates the DVFS
 * thresholds, updates the Atlas, unmasks the DVFS interrupt and enables 
 * the DVFS module
 *
 * @return      0 if DVFS module was enabled else returns -EINVAL. 
 *
 */   
int start_dvfs(dvfs_dptc_params_s *params);

/*!
 * This function disables the DVFS module.
 * @return      0 if DVFS module was enabled else returns -EINVAL. 
 *
 */   
int stop_dvfs(dvfs_dptc_params_s *params);

/*!
 * This function sets SW general purpose bits
 * @return      0 on success (always) 
 */
int set_sw_gp(unsigned char arg);

/*!
 * This function sets DVFS to monitor WFI signal
 * @return      0 on success (always) 
 */
int set_wfi(unsigned char arg);

#ifndef CONFIG_MXC_DVFS_SDMA
/*!
 * This function is the DVFS Interrupt handler.
 */ 
void dvfs_irq(dvfs_dptc_params_s *params);
#endif

/*!
 * This define DVFS log buffer size
 */
#define DVFS_LB_SIZE 512

#endif
