/*
 * Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 */

/*
 * The code contained herein is licensed under the GNU General Public
 * License. You may obtain a copy of the GNU General Public License
 * Version 2 or later at the following locations:
 *
 * http://www.opensource.org/licenses/gpl-license.html
 * http://www.gnu.org/copyleft/gpl.html
 */

/*!
 * @file mxc_pm_scma11.h
 *
 * @brief This file provides all the Register Addresses, Bit definitions.
 *
 * @ingroup LPMD
 */

#ifndef MXC_PM_REG_H

#define MXC_PM_REG_H

#include <asm/arch/platform.h>

/*
 * Address offsets of the CRM_AP registers
 */
#define MXC_CRM_AP_ASCSR                        IO_ADDRESS(CRM_AP_BASE_ADDR + 0x00)
#define MXC_CRM_AP_ACDR                         IO_ADDRESS(CRM_AP_BASE_ADDR + 0x04)
#define MXC_CRM_AP_ACDER1                       IO_ADDRESS(CRM_AP_BASE_ADDR + 0x08)
#define MXC_CRM_AP_ACDER2                       IO_ADDRESS(CRM_AP_BASE_ADDR + 0x0c)
#define MXC_CRM_AP_ACGCR                        IO_ADDRESS(CRM_AP_BASE_ADDR + 0x10)
#define MXC_CRM_AP_ACCGCR                       IO_ADDRESS(CRM_AP_BASE_ADDR + 0x14)
#define MXC_CRM_AP_AMLPMRA                      IO_ADDRESS(CRM_AP_BASE_ADDR + 0x18)
#define MXC_CRM_AP_AMLPMRB                      IO_ADDRESS(CRM_AP_BASE_ADDR + 0x1c)
#define MXC_CRM_AP_AMLPMRC                      IO_ADDRESS(CRM_AP_BASE_ADDR + 0x20)
#define MXC_CRM_AP_ADCR                         IO_ADDRESS(CRM_AP_BASE_ADDR + 0x40)
#define MXC_CRM_AP_ACR                          IO_ADDRESS(CRM_AP_BASE_ADDR + 0x44)
#define MXC_CRM_AP_AMCR                         IO_ADDRESS(CRM_AP_BASE_ADDR + 0x48)

/*
 * Address offsets of the CRM_COM registers
 */
#define MXC_CRM_COM_CSCR                        IO_ADDRESS(CRM_COM_BASE_ADDR + 0x0c)

/*
 * Setting used for ap_delay - unit delay is 1/32768 seconds
 */
#define AP_DELAY_SETTING                        (0x00030000)


/*
 * Bit definitions of ASCSR, ACDR in CRM_AP and CSCR in CRM_COM
 */
//#define ASCSR_APSEL                             0x00000000
#define ASCSR_APSEL0                            0x00000008
#define ASCSR_APSEL1                            0x00000010
#define CSCR_CKOHSEL                            0x00060000
#define ACDR_ARMDIV                             0x00000800

/*
 * Bit definitions of ADCR in CRM_AP
 */
#define ADCR_ALT_PLL                            0x00000080
#define ADCR_DIV_BYP                            0x00000002
#define ADCR_DFS_DIV_EN                         0x00000020
#define ADCR_LFDF_LSB                           0x00000100
#define ADCR_LFDF_MSB                           0x00000200
#define ADCR_AP_DELAY                           0x00000000
#define ADCR_VSTAT                              0x00000008
#define ADCR_TSTAT                              0x00000010
#define ADCR_CLK_ON                             0x00000040

/*
 * Bit definitions of ACGCR in CRM_AP
 */
#define ACG0_STOP_WAIT                          0x00000001
#define ACG0_STOP                               0x00000003
#define ACG0_RUN                                0x00000007
#define ACG0_DISABLED                           0x00000000

#define ACG1_STOP_WAIT                          0x00000008
#define ACG1_STOP                               0x00000018
#define ACG1_RUN                                0x00000038
#define ACG1_DISABLED                           0x00000000

#define ACG2_STOP_WAIT                          0x00000040
#define ACG2_STOP                               0x000000C0
#define ACG2_RUN                                0x000001C0
#define ACG2_DISABLED                           0x00000000

#define ACG3_STOP_WAIT                          0x00000200
#define ACG3_STOP                               0x00000600
#define ACG3_RUN                                0x00000E00
#define ACG3_DISABLED                           0x00000000

#define ACG4_STOP_WAIT                          0x00001000
#define ACG4_STOP                               0x00003000
#define ACG4_RUN                                0x00007000
#define ACG4_DISABLED                           0x00000000

#define ACG5_STOP_WAIT                          0x00010000
#define ACG5_STOP                               0x00030000
#define ACG5_RUN                                0x00070000
#define ACG5_DISABLED                           0x00000000

#define ACG6_STOP_WAIT                          0x00080000
#define ACG6_STOP                               0x00180000
#define ACG6_RUN                                0x00380000
#define ACG6_DISABLED                           0x00000000

#define NUM_GATE_CTRL                           6

/*
 * Address offsets of DSM registers
 */
#define DSM_CRM_CONTROL                         IO_ADDRESS(DSM_BASE_ADDR + 0x50)
#define DSM_CONTROL1                            IO_ADDRESS(DSM_BASE_ADDR + 0x24)
#define DSM_CONTROL0                            IO_ADDRESS(DSM_BASE_ADDR + 0x20)
#define DSM_INT_STATUS                          IO_ADDRESS(DSM_BASE_ADDR + 0x34)
#define DSM_MEAS_TIME                           IO_ADDRESS(DSM_BASE_ADDR + 0x08)
#define DSM_SLEEP_TIME                          IO_ADDRESS(DSM_BASE_ADDR + 0x0c)
#define DSM_RESTART_TIME                        IO_ADDRESS(DSM_BASE_ADDR + 0x10)
#define DSM_WAKE_TIME                           IO_ADDRESS(DSM_BASE_ADDR + 0x14)
#define DSM_WARM_TIME                           IO_ADDRESS(DSM_BASE_ADDR + 0x18)
#define DSM_LOCK_TIME                           IO_ADDRESS(DSM_BASE_ADDR + 0x1c)
#define DSM_MASK_REG                            IO_ADDRESS(DSM_BASE_ADDR + 0x38)
#define DSM_CTREN                               IO_ADDRESS(DSM_BASE_ADDR + 0x28)

/*
 * Bit definitions of various registers in DSM
 */
#define CRM_CTRL_DVFS_BYP                       0x00000008
#define CRM_CTRL_DVFS_VCTRL                     0x00000004
#define CRM_CTRL_LPMD1                          0x00000002
#define CRM_CTRL_LPMD0                          0x00000001
#define CONTROL0_STBY_COMMIT_EN                 0x00000200
#define CONTROL0_MSTR_EN                        0x00000001
#define CONTROL0_RESTART                        0x00000010
#define CONTROL1_RST_CNT32_EN                   0x00000800
#define CONTROL1_SLEEP                          0x00000100
#define CONTROL1_WAKEUP_DISABLE                 0x00004000
#define INT_STATUS_RWK                          0x00000080
#define INT_STATUS_WKTIME_INT                   0x00000200
#define INT_STATUS_SLEEP_INT                    0x00000100
#define INT_STATUS_NGK_INT                      0x00000040
#define INT_STATUS_PGK_INT                      0x00000020
#define INT_STATUS_STPL1T_INT                   0x00000010
#define INT_STATUS_RSTRT_INT                    0x00000008
#define INT_STATUS_LOCK_INT                     0x00000004
#define INT_STATUS_WTIME_INT                    0x00000002
#define INT_STATUS_MDONE_INT                    0x00000001
#define DSM_CTREN_CNT32                         0x00000001

/*
 * Address offsets of DPLL registers
 */
#define MXC_PLL0_DP_CTL                         IO_ADDRESS(ADPLL_BASE_ADDR + 0x00)
#define MXC_PLL0_DP_CONFIG                      IO_ADDRESS(ADPLL_BASE_ADDR + 0x04)
#define MXC_PLL0_DP_OP                          IO_ADDRESS(ADPLL_BASE_ADDR + 0x08)
#define MXC_PLL0_DP_MFD                         IO_ADDRESS(ADPLL_BASE_ADDR + 0x0c)
#define MXC_PLL0_DP_MFN                         IO_ADDRESS(ADPLL_BASE_ADDR + 0x10)
#define MXC_PLL0_DP_HFSOP                       IO_ADDRESS(ADPLL_BASE_ADDR + 0x1c)
#define MXC_PLL0_DP_HFSMFD                      IO_ADDRESS(ADPLL_BASE_ADDR + 0x20)
#define MXC_PLL0_DP_HFSMFN                      IO_ADDRESS(ADPLL_BASE_ADDR + 0x24)

#define MXC_PLL1_DP_CTL                         IO_ADDRESS(UDPLL_BASE_ADDR + 0x00)
#define MXC_PLL1_DP_OP                          IO_ADDRESS(UDPLL_BASE_ADDR + 0x08)
#define MXC_PLL1_DP_MFD                         IO_ADDRESS(UDPLL_BASE_ADDR + 0x0c)
#define MXC_PLL1_DP_MFN                         IO_ADDRESS(UDPLL_BASE_ADDR + 0x10)
#define MXC_PLL1_DP_HFSOP                       IO_ADDRESS(UDPLL_BASE_ADDR + 0x1c)
#define MXC_PLL1_DP_HFSMFD                      IO_ADDRESS(UDPLL_BASE_ADDR + 0x20)
#define MXC_PLL1_DP_HFSMFN                      IO_ADDRESS(UDPLL_BASE_ADDR + 0x24)

#define MXC_PLL2_DP_CTL                         IO_ADDRESS(BDPLL_BASE_ADDR + 0x00)
#define MXC_PLL2_DP_OP                          IO_ADDRESS(BDPLL_BASE_ADDR + 0x08)
#define MXC_PLL2_DP_MFD                         IO_ADDRESS(BDPLL_BASE_ADDR + 0x0c)
#define MXC_PLL2_DP_MFN                         IO_ADDRESS(BDPLL_BASE_ADDR + 0x10)
#define MXC_PLL2_DP_HFSOP                       IO_ADDRESS(BDPLL_BASE_ADDR + 0x1c)
#define MXC_PLL2_DP_HFSMFD                      IO_ADDRESS(BDPLL_BASE_ADDR + 0x20)
#define MXC_PLL2_DP_HFSMFN                      IO_ADDRESS(BDPLL_BASE_ADDR + 0x24)

/*
 * Bit definitions of DPLL-IP
 */
#define DP_MFI                                  0x00000050
#define DP_PDF                                  0x00000001
#define DP_MFD                                  0x00000000
#define DP_MFN                                  0x00000000
#define DP_HFS_MFI                              0x00000050
#define DP_HFS_PDF                              0x00000001
#define DP_HFS_MFD                              0x00000000
#define DP_HFS_MFN                              0x00000000
#define DP_HFSM                                 0x00000080
#define DP_AREN                                 0x00000002

/*
 * Bit definitions of DP_CTL register
 */
/*
#define DP_CTL_DPDCK0_2_EN                      0x00001000
#define DP_CTL_ADE                              0x00000800
#define DP_CTL_REF_CLK_DIV                      0x00000400
#define DP_CTL_REF_CLK_SEL1                     0x00000200
#define DP_CTL_REF_CLK_SEL0                     0x00000100
#define DP_CTL_HFSM                             0x00000080
#define DP_CTL_PRE                              0x00000040
#define DP_CTL_UPEN                             0x00000020
#define DP_CTL_RST                              0x00000010
#define DP_CTL_RPC                              0x00000008
#define DP_CTL_PLM                              0x00000004
#define DP_CTL_BRMO                             0x00000002
#define DP_CTL_LRF                              0x00000001
*/

#endif

