/*
 * Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*!
 * @file mxc_hacc.h
 *
 * @brief The header file for Hash Acceleration (HAC) module.
 * This file contains all register defines and bit definition of HAC module.
 *
 * @ingroup MXC_Security
 */

#ifndef MXC_HAC_H
#define MXC_HAC_H

#include <asm/arch/mxc_security_api.h>
#include <asm/arch/platform.h>
#include <asm-generic/errno-base.h>

#ifdef CONFIG_HAC_TEST_DEBUG
#define HAC_DEBUG(fmt, args...) printk(fmt,## args)
#else
#define HAC_DEBUG(fmt, args...)
#endif /* HAC_TEST_DEBUG */

/*
 * HAC Control register
 */
#define  HAC_CTL         (*((volatile ulong *)IO_ADDRESS(HAC_BASE_ADDR + 0x00)))

/*
 * HAC Start Address Register
 */
#define  HAC_START_ADDR  (*((volatile ulong *)IO_ADDRESS(HAC_BASE_ADDR + 0x04)))

/*
 * HAC Block Count Register
 */
#define  HAC_BLK_CNT     (*((volatile ulong *)IO_ADDRESS(HAC_BASE_ADDR + 0x08)))

/*
 * HAC Hash Register 4 Register
 */
#define  HAC_HSH4        (*((volatile ulong *)IO_ADDRESS(HAC_BASE_ADDR + 0x0C)))

/*
 * HAC Hash Register 3 Register
 */
#define  HAC_HSH3        (*((volatile ulong *)IO_ADDRESS(HAC_BASE_ADDR + 0x10)))

/*
 * HAC Hash Register 2 Register
 */
#define  HAC_HSH2        (*((volatile ulong *)IO_ADDRESS(HAC_BASE_ADDR + 0x14)))

/*
 * HAC Hash Register 1 Register
 */
#define  HAC_HSH1        (*((volatile ulong *)IO_ADDRESS(HAC_BASE_ADDR + 0x18)))

/*
 * HAC Hash Register 0 Register
 */
#define  HAC_HSH0        (*((volatile ulong *)IO_ADDRESS(HAC_BASE_ADDR + 0x1C)))

/*
 * HAC Hash Done status
 */
#define         HAC_CTL_DONE            0x01

/*
 * HAC  Hash Error Status
 */
#define         HAC_CTL_ERROR           0x02

/*
 * HAC  Hash Accelerator Module Busy Status
 */
#define         HAC_CTL_BUSY            0x04

/*
 * HAC  HAC Interrupt Mask Bit.
 */
#define         HAC_CTL_IMSK            0x08

/*
 * HAC  Command to Stop hash processing
 */
#define         HAC_CTL_STOP            0x10

/*
 * HAC  Command for software reset
 */
#define         HAC_CTL_SWRST           0x20

/*
 * HAC  Command to start hash processing
 */
#define         HAC_CTL_START           0x40

/*
 * HAC  Command to continue hash processing
 */
#define         HAC_CTL_CONTINUE        0x80

/*
 * HAC  Command to do padding
 */
#define         HAC_CTL_PAD             0x100

/*
 * HAC  Command to do burst read
 */
#define         HAC_CTL_BURST_CONFIG    0x600

/*
 * HAC  Command to configure the burst mode
 */
#define         HAC_CTL_BURST_MODE      0x800

/*!
 * Maximum block length that can be configured to the HAC module.
 */
#define         HAC_MAX_BLOCK_LENGTH    0x7FFFFF

#endif   /* MXC_HACC_H */
