/*
 * Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*!
 * @file mxc_rnga.h
 *
 * @brief MXC RNGA header file.
 *
 * @ingroup MXC_Security
 */

#ifndef _MXC_RNGA_H_
#define _MXC_RNGA_H_

#include <asm/arch/platform.h>
#include <asm/arch/mxc_security_api.h>
#include <linux/config.h>

#ifdef CONFIG_RNGA_TEST_DEBUG
#include <linux/module.h>
#endif /* RNGA_TEST_DEBUG */

/*
 * Defined differently in RNGA platform.h file. To make it
 * compatible for all the platforms this is #defined in this way.
 */
#ifdef CONFIG_ARCH_ARGONPLUSEVB
#define  RNGA_BASE_ADDR   RNG_BASE_ADDR
#endif

#ifdef RNGA_TEST_DEBUG
#define RNGA_DEBUG(fmt, args...) printk(fmt,## args)
#else
#define RNGA_DEBUG(fmt, args...)
#endif /* RNGA_TEST_DEBUG */

/*
 * Configures Hi-assurance, Interrupt, Clears interrupt, goes to power down
 * mode and starts the random number generation
 */
#define   RNGA_CONTROL          (*((volatile ulong *)IO_ADDRESS(RNGA_BASE_ADDR\
                                  + 0x0000)))
/* Gives the various status of the RNGA module */
#define   RNGA_STATUS           (*(volatile ulong *)IO_ADDRESS(RNGA_BASE_ADDR \
                                  + 0x0004))
/* Has the initial seed used for the generation of the random numbers */
#define   RNGA_ENTROPY          (*((volatile ulong *)IO_ADDRESS(RNGA_BASE_ADDR\
                                  + 0x0008)))
/* Has the random number output */
#define   RNGA_FIFO             (*((volatile ulong *)IO_ADDRESS(RNGA_BASE_ADDR\
                                  + 0x000C)))
/* Sets the mode of operation of the RNGA module */
#define   RNGA_MODE             (*((volatile ulong *)IO_ADDRESS(RNGA_BASE_ADDR\
                                  + 0x0010)))
/* Used to configure for the verification mode */
#define   RNGA_VRFY_CTRL        (*((volatile ulong *)IO_ADDRESS(RNGA_BASE_ADDR\
                                  + 0x0014)))
/* Used to control the oscillator counter */
#define   RNGA_OSC_CTRL_CNT     (*((volatile ulong *)IO_ADDRESS(RNGA_BASE_ADDR\
                                  + 0x0018)))
/* Used to count the number of oscillator pulses */
#define   RNGA_OSC1_CNT         (*((volatile ulong *)IO_ADDRESS(RNGA_BASE_ADDR\
                                  + 0x001C)))
/* Used to count the number of oscillator pulses */
#define   RNGA_OSC2_CNT         (*((volatile ulong *)IO_ADDRESS(RNGA_BASE_ADDR\
                                  + 0x0020)))
/* Gives the status of the oscillators */
#define   RNGA_OSC_STATUS       (*((volatile ulong *)IO_ADDRESS(RNGA_BASE_ADDR\
                                  + 0x0024)))

/*!
 * Start the random number generation
 */
#define         RNGA_START_RAND         1

/*!
 * Clear RNGA interrupt
 */
#define         RNGA_CLR_INTR           (1 << 3)

/*!
 * Set the RNGA to verification mode
 */
#define         RNGA_VERIFICATION       1

/*!
 * Set the RNGA shift clocks to off
 */
#define         RNGA_SHIFT_CLOCK_OFF    1

/*!
 * Force the shift clock to be driven by system clock
 */
#define         RNGA_FORCE_SYS_CLK      (1 << 1)

/*!
 * Reset the RNGA shift register
 */
#define         RNGA_RESET_SHIFT_REG    (1 << 2)

/*!
 * Maximum oscillator counter that the counter can accept.
 */
#define         RNGA_MAX_OSC_COUNT      0x3FFF  

/*!
 * Read from an RNGA register
 */
#define         RNGA_READ               0

/*!
 * Read from an RNGA register
 */
#define         RNGA_WRITE              1

/*!
 * Generate random numbers
 */
#define         RNGA_GEN_RAND           1

/*!
 * This parameter used to Place RNGA in low power mode
 */
#define         RNGA_SLEEP              (1 << 4)

/*!
 * This parameter is used to mask the least significant byte.
 */
#define         RNGA_MSBYTE_2_LSBYTE       8 

#endif /* _MXC_RNGA_H_ */

