/*
 * Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*!
 * @file mxc_rtic.h
 *
 * @brief The header file for Run-Time Integrity Checker (RTIC) module.
 * This file contains all register defines and bit definition of RTIC module.
 *
 * @ingroup MXC_Security
 */

#ifndef MXC_RTIC_H
#define MXC_RTIC_H

#include <asm/arch/mxc_security_api.h>
#include <asm/arch/platform.h>
#include <asm-generic/errno-base.h>

#ifdef CONFIG_RTIC_TEST_DEBUG
#define RTIC_DEBUG(fmt, args...) printk(fmt,## args)
#else
#define RTIC_DEBUG(fmt, args...)
#endif /* RTIC_TEST_DEBUG */

/*
 * RTIC status register
 */
#define  RTIC_STATUS          (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR  \
                                + 0x00)))

/*
 * RTIC Command Register
 */
#define  RTIC_COMMAND          (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x04)))

/*
 * RTIC Control Register
 */
#define  RTIC_CONTROL          (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x08)))

/*
 * RTIC Delay Timer/DMA Throttle
 */
#define  RTIC_DMA              (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x0C)))

/*
 * RTIC delay between DMA Bus reads
 */
#define  RTIC_DMA_DELAY_RD      0x001f  
 
/*
 * RTIC Memory A Address 1
 */
#define  RTIC_MEMAADDR1        (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x10)))

/*
 * RTIC Memory A Length 1
 */
#define  RTIC_MEMALEN1         (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x14)))
 
/*
 * RTIC Memory A Address 2
 */
#define  RTIC_MEMAADDR2        (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x18)))
/*
 * RTIC Memory A Length 2
 */
#define  RTIC_MEMALEN2         (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x1C)))
 
/*
 * RTIC Memory B Address 1
 */
#define  RTIC_MEMBADDR1        (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x30)))

/*
 * RTIC Memory B Length 1
 */
#define  RTIC_MEMBLEN1         (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x34)))

/*
 * RTIC Memory B Address 2
 */
#define  RTIC_MEMBADDR2        (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x38)))

/*
 * RTIC Memory B Length 2
 */
#define  RTIC_MEMBLEN2         (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x3C)))

/*
 *  RTIC Memory C Address 1
 */
#define  RTIC_MEMCADDR1        (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x50)))  

/*
 *  RTIC Memory C Length 1
 */
#define  RTIC_MEMCLEN1         (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x54)))
 
/*
 *  RTIC Memery C Address 2   
 */
#define  RTIC_MEMCADDR2       (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR  \
                                + 0x58)))

/*
 * RTIC Memory C Length 2
 */
#define  RTIC_MEMCLEN2        (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                + 0x5C)))
 
/*
 * RTIC Memory D Address 1
 */
#define  RTIC_MEMDADDR1       (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                + 0x70)))

/*
 * RTIC Memory D Length 1
 */
#define  RTIC_MEMDLEN1        (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                + 0x74)))

/*
 * RTIC Memory D Address 2
 */
#define  RTIC_MEMDADDR2      (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                               + 0x78)))
   
/*
 * RTIC Memory D Length 2
 */
#define  RTIC_MEMDLEN2       (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                               + 0x7C)))

/*
 * RTIC Fault address Register.
 */
#define  RTIC_FAULTADDR      (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                               + 0x90))) 
  
/*
 * RTIC Watchdog Timeout
 */
#define  RTIC_WDTIMER        (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                               + 0x94)))

/*
 * watch dog timer time out
 */
#define  RTIC_WDTIMER_VALUE      0x001f  

/*
 * RTIC  Memory A Hash Result[159:128]
 */
#define  RTIC_MEMAHASHRES4     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0xA0)))
 
/*
 * RTIC  Memory A Hash Result[127:96]
 */
#define  RTIC_MEMAHASHRES3     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0xA4)))
 
/*
 * RTIC  Memory A Hash Result[95:64]
 */
#define  RTIC_MEMAHASHRES2     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0xA8))) 

/*
 * RTIC  Memory A Hash Result[63:32]
 */
#define  RTIC_MEMAHASHRES1     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0xAC)))

/*
 * RTIC  Memory A Hash Result[31:0]
 */
#define  RTIC_MEMAHASHRES0     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0xB0)))
 
/*
 * RTIC  Memory B Hash Result[159:128]
 */
#define  RTIC_MEMBHASHRES4     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0xC0)))

/*
 * RTIC  Memory B Hash Result[127:96]
 */
#define  RTIC_MEMBHASHRES3     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0xC4)))

/*
 * RTIC  Memory B Hash Result[95:64]
 */
#define  RTIC_MEMBHASHRES2     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0xC8)))

/*
 * RTIC  Memory B Hash Result[63:32]
 */
#define  RTIC_MEMBHASHRES1     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0xCC)))

/*
 * RTIC  Memory B Hash Result[31:0]
 */
#define  RTIC_MEMBHASHRES0     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0xD0)))
 
/*
 * RTIC  Memory C Hash Result[159:128]
 */
#define  RTIC_MEMCHASHRES4     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0xE0)))

/*
 * RTIC  Memory C Hash Result[127:96]
 */
#define  RTIC_MEMCHASHRES3     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0xE4)))

/*
 * RTIC  Memory C Hash Result[95:64]
 */
#define  RTIC_MEMCHASHRES2     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0xE8)))

/*
 * RTIC  Memory C Hash Result[63:32]
 */
#define  RTIC_MEMCHASHRES1     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0xEC)))

/*
 * RTIC  Memory C Hash Result[31:0]
 */
#define  RTIC_MEMCHASHRES0     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0xF0)))
           
/*
 * RTIC  Memory D Hash Result[159:128]
 */
#define  RTIC_MEMDHASHRES4     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x100)))

/*
 * RTIC  Memory D Hash Result[127:96]
 */
#define  RTIC_MEMDHASHRES3     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x104)))

/*
 * RTIC  Memory D Hash Result[95:64]
 */
#define  RTIC_MEMDHASHRES2     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x108)))

/*
 * RTIC  Memory D Hash Result[63:32]
 */
#define  RTIC_MEMDHASHRES1     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x10C)))

/*
 * RTIC  Memory D Hash Result[31:0]
 */
#define  RTIC_MEMDHASHRES0     (*((volatile ulong *)IO_ADDRESS(RTIC_BASE_ADDR \
                                 + 0x110)))
           
/*
 * RTIC  Memory Integrity Status
 */
#define  RTIC_STAT_MEMBLK       0x100
 
/*
 * RTIC  Address Error
 */
#define  RTIC_STAT_ADDR_ERR     0x200
 
/*
 * RTIC  Length Error
 */
#define  RTIC_STAT_LEN_ERR      0x400
 
/*
 * RTIC  Clear Interrupts command
 */
#define  RTIC_CMD_CLRIRQ        0x01
 
/*
 * RTIC  SW reset command
 */
#define  RTIC_CMD_SW_RESET      0x02
 
/*
 * RTIC  Hash Once command
 */
#define  RTIC_CMD_HASH_ONCE     0x04
 
/*
 * RTIC  Run Time Check Command
 */
#define  RTIC_CMD_RUN_TIME_CHK  0x08
 
/*
 * RTIC  IRQ EN
 */
#define  RTIC_CTL_IRQ_EN        0x01

/*
 * RTIC  Hash Once Memory block A enable
 */
#define  RTIC_CTL_HASHONCE_MEMA_BLK_EN   0x10
 
/*
 * RTIC  Hash Once Memory block B Enable
 */
#define  RTIC_CTL_HASHONCE_MEMB_BLK_EN   0x20
 
/*
 * RTIC  Hash Once Memory block C Enable
 */
#define  RTIC_CTL_HASHONCE_MEMC_BLK_EN   0x40
 
/*
 * RTIC  Hash Once Memory block D Enable
 */
#define  RTIC_CTL_HASHONCE_MEMD_BLK_EN   0x80
         
/*
 * RTIC  Run Time Memory blk A Enable
 */
#define  RTIC_CTL_RUNTIME_MEMA_BLK_EN   0x100
 
/*
 * RTIC  Run Time Memory blk B Enable
 */
#define  RTIC_CTL_RUNTIME_MEMB_BLK_EN   0x200
           
/*
 * RTIC  Run Time Memory blk C Enable
 */
#define  RTIC_CTL_RUNTIME_MEMC_BLK_EN   0x400

/*
 * RTIC  Run Time Memory blk D Enable
 */
#define  RTIC_CTL_RUNTIME_MEMD_BLK_EN   0x800

/*!
 * Maximum block length that can be configured to the RTIC module.
 */
#define  RTIC_MAX_BLOCK_LENGTH          0x7FFFFF 

#endif  /* MXC_RTIC_H */
