/*
 * Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 */

/*
 * The code contained herein is licensed under the GNU General Public
 * License. You may obtain a copy of the GNU General Public License
 * Version 2 or later at the following locations:
 *
 * http://www.opensource.org/licenses/gpl-license.html
 * http://www.gnu.org/copyleft/gpl.html
 */ 

 /*!
  * @file ../ssi/registers.h
  * @brief This header file contains SPI driver low level definition to access module registers.
  *
  * @ingroup SPI
  */

#ifndef REGISTERS_H_
#define REGISTERS_H_

/*!
 * This include to define bool type, false and true definitions.
 */
#include <asm/hardware.h>
#include <asm/arch/platform.h>
#include <asm/arch/spba.h>

#define ModifyRegister32(a,b,c)	((c) = ( ( (c)&(~(a)) ) | (b) ))

#if defined(CONFIG_ARCH_MX3) || \
    defined(CONFIG_ARCH_ARGONPLUS) || \
    defined(CONFIG_ARCH_SCMA11)

#define SPBA_CPU_SSI            0x07

#ifndef _reg_SSI2_SPBA
#define _reg_SSI2_SPBA      (*((volatile unsigned long *) \
                              (IO_ADDRESS(SPBA_CTRL_BASE_ADDR + SPBA_SSI2))))
#endif

#if defined(CONFIG_ARCH_SCMA11)
#ifndef _reg_CRM_AP_ASCSR
#define _reg_CRM_AP_ASCSR   (*((volatile unsigned long *) \
                              (IO_ADDRESS(CRM_AP_BASE_ADDR + 0x00))))
#endif
#ifndef _reg_CRM_AP_ACDER1
#define _reg_CRM_AP_ACDER1      (*((volatile unsigned long *) \
                              (IO_ADDRESS(CRM_AP_BASE_ADDR + 0x08))))
#endif
#endif

#if defined(CONFIG_ARCH_ARGONPLUS)
#ifndef _reg_MCU_MCR
#define _reg_MCU_MCR   (*((volatile unsigned long *) \
                              (IO_ADDRESS(CRM_MCU_BASE_ADDR + 0x00))))
#endif
#ifndef _reg_MCU_MPDR1
#define _reg_MCU_MPDR1      (*((volatile unsigned long *) \
                              (IO_ADDRESS(CRM_MCU_BASE_ADDR + 0x08))))
#endif
#endif

#ifndef _reg_SSI1_STX0
#define    _reg_SSI1_STX0   (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x00))))
#endif
#ifndef _reg_SSI1_STX1
#define    _reg_SSI1_STX1   (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x04))))
#endif
#ifndef _reg_SSI1_SRX0
#define    _reg_SSI1_SRX0   (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x08))))
#endif
#ifndef _reg_SSI1_SRX1
#define    _reg_SSI1_SRX1   (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x0C))))
#endif
#ifndef _reg_SSI1_SCR
#define    _reg_SSI1_SCR    (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x10))))
#endif
#ifndef _reg_SSI1_SISR
#define    _reg_SSI1_SISR   (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x14))))
#endif
#ifndef _reg_SSI1_SIER
#define    _reg_SSI1_SIER   (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x18))))
#endif
#ifndef _reg_SSI1_STCR
#define    _reg_SSI1_STCR   (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x1C))))
#endif
#ifndef _reg_SSI1_SRCR
#define    _reg_SSI1_SRCR   (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x20))))
#endif
#ifndef _reg_SSI1_STCCR
#define    _reg_SSI1_STCCR  (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x24))))
#endif
#ifndef _reg_SSI1_SRCCR
#define    _reg_SSI1_SRCCR  (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x28))))
#endif
#ifndef _reg_SSI1_SFCSR
#define    _reg_SSI1_SFCSR  (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x2C))))
#endif
#ifndef _reg_SSI1_STR
#define    _reg_SSI1_STR    (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x30))))
#endif
#ifndef _reg_SSI1_SOR
#define    _reg_SSI1_SOR    (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x34))))
#endif
#ifndef _reg_SSI1_SACNT
#define    _reg_SSI1_SACNT  (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x38))))
#endif
#ifndef _reg_SSI1_SACADD
#define    _reg_SSI1_SACADD (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x3C))))
#endif
#ifndef _reg_SSI1_SACDAT
#define    _reg_SSI1_SACDAT (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x40))))
#endif
#ifndef _reg_SSI1_SATAG
#define    _reg_SSI1_SATAG  (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x44))))
#endif
#ifndef _reg_SSI1_STMSK
#define    _reg_SSI1_STMSK  (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x48))))
#endif
#ifndef _reg_SSI1_SRMSK
#define    _reg_SSI1_SRMSK  (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI1_BASE_ADDR + 0x4C))))
#endif

#ifndef _reg_SSI2_STX0
#define    _reg_SSI2_STX0   (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x00))))
#endif
#ifndef _reg_SSI2_STX1
#define    _reg_SSI2_STX1   (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x04))))
#endif
#ifndef _reg_SSI2_SRX0
#define    _reg_SSI2_SRX0   (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x08))))
#endif
#ifndef _reg_SSI2_SRX1
#define    _reg_SSI2_SRX1   (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x0C))))
#endif
#ifndef _reg_SSI2_SCR
#define    _reg_SSI2_SCR    (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x10))))
#endif
#ifndef _reg_SSI2_SISR
#define    _reg_SSI2_SISR   (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x14))))
#endif
#ifndef _reg_SSI2_SIER
#define    _reg_SSI2_SIER   (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x18))))
#endif
#ifndef _reg_SSI2_STCR
#define    _reg_SSI2_STCR   (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x1C))))
#endif
#ifndef _reg_SSI2_SRCR
#define    _reg_SSI2_SRCR   (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x20))))
#endif
#ifndef _reg_SSI2_STCCR
#define    _reg_SSI2_STCCR  (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x24))))
#endif
#ifndef _reg_SSI2_SRCCR
#define    _reg_SSI2_SRCCR  (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x28))))
#endif
#ifndef _reg_SSI2_SFCSR
#define    _reg_SSI2_SFCSR  (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x2C))))
#endif
#ifndef _reg_SSI2_STR
#define    _reg_SSI2_STR    (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x30))))
#endif
#ifndef _reg_SSI2_SOR
#define    _reg_SSI2_SOR    (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x34))))
#endif
#ifndef _reg_SSI2_SACNT
#define    _reg_SSI2_SACNT  (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x38))))
#endif
#ifndef _reg_SSI2_SACADD
#define    _reg_SSI2_SACADD (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x3C))))
#endif
#ifndef _reg_SSI2_SACDAT
#define    _reg_SSI2_SACDAT (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x40))))
#endif
#ifndef _reg_SSI2_SATAG
#define    _reg_SSI2_SATAG  (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x44))))
#endif
#ifndef _reg_SSI2_STMSK
#define    _reg_SSI2_STMSK  (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x48))))
#endif
#ifndef _reg_SSI2_SRMSK
#define    _reg_SSI2_SRMSK  (*((volatile unsigned long *) \
                              (IO_ADDRESS(SSI2_BASE_ADDR + 0x4C))))
#endif

/*!
 * Register access macros
 */
#define DIFF_SSI2_SSI1 (SSI2_BASE_ADDR-SSI1_BASE_ADDR)
#define    _reg_SSI_STX0(a)   (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x00))))
#define    _reg_SSI_STX1(a)   (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x04))))
#define    _reg_SSI_SRX0(a)   (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x08))))
#define    _reg_SSI_SRX1(a)   (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x0C))))
#define    _reg_SSI_SCR(a)    (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x10))))
#define    _reg_SSI_SISR(a)   (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x14))))
#define    _reg_SSI_SIER(a)   (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x18))))
#define    _reg_SSI_STCR(a)   (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x1C))))
#define    _reg_SSI_SRCR(a)   (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x20))))
#define    _reg_SSI_STCCR(a)  (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x24))))
#define    _reg_SSI_SRCCR(a)  (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x28))))
#define    _reg_SSI_SFCSR(a)  (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x2C))))
#define    _reg_SSI_STR(a)    (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x30))))
#define    _reg_SSI_SOR(a)    (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x34))))
#define    _reg_SSI_SACNT(a)  (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x38))))
#define    _reg_SSI_SACADD(a) (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x3C))))
#define    _reg_SSI_SACDAT(a) (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x40))))
#define    _reg_SSI_SATAG(a)  (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x44))))
#define    _reg_SSI_STMSK(a)  (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x48))))
#define    _reg_SSI_SRMSK(a)  (*((volatile unsigned long *) \
                              (IO_ADDRESS( (SSI1_BASE_ADDR+ \
                              (a*DIFF_SSI2_SSI1)) + 0x4C))))


/*!
 * SCR Register bit shift definitions
 */
#define ssi_enable_shift            0
#define ssi_transmit_enable_shift   1
#define ssi_receive_enable_shift    2
#define ssi_network_mode_shift      3
#define ssi_synchronous_mode_shift  4
#define ssi_i2s_mode_shift          5
#define ssi_system_clock_shift      7
#define ssi_two_channel_shift       8
#define ssi_clock_idle_shift        9

/*!
 * STCR & SRCR Registers bit shift definitions
 */
#define ssi_early_frame_sync_shift    0
#define ssi_frame_sync_length_shift   1
#define ssi_frame_sync_invert_shift   2
#define ssi_clock_polarity_shift      3
#define ssi_shift_direction_shift     4
#define ssi_clock_direction_shift     5
#define ssi_frame_direction_shift     6
#define ssi_fifo_enable_0_shift       7
#define ssi_fifo_enable_1_shift       8
#define ssi_bit_0_shift               9

/*!
 * STCCR & SRCCR Registers bit shift definitions
 */
#define ssi_prescaler_modulus_shift          0
#define ssi_frame_rate_divider_shift         8
#define ssi_word_length_shift               13
#define ssi_prescaler_range_shift           17
#define ssi_divide_by_two_shift             18
#define ssi_frame_divider_mask            0x1F
#define ssi_min_frame_divider_ratio          1
#define ssi_max_frame_divider_ratio         32
#define ssi_prescaler_modulus_mask        0xFF
#define ssi_min_prescaler_modulus_ratio      1
#define ssi_max_prescaler_modulus_ratio    256
#define ssi_word_length_mask               0xF
/*!
 * SISR Register definition
 */
#define ssi_irq_status_number        19

/*!
 * SFCSR Register bit shift definitions
 */
#define ssi_rx_fifo_1_count_shift       28
#define ssi_tx_fifo_1_count_shift       24
#define ssi_rx_fifo_1_watermark_shift   20
#define ssi_tx_fifo_1_watermark_shift   16
#define ssi_rx_fifo_0_count_shift       12
#define ssi_tx_fifo_0_count_shift        8
#define ssi_rx_fifo_0_watermark_shift    4
#define ssi_tx_fifo_0_watermark_shift    0
#define ssi_min_fifo_watermark           0
#define ssi_max_fifo_watermark           8

/*!
 * SSI Option Register (SOR) bit shift definitions
 */
#define ssi_frame_syn_reset_shift        0
#define ssi_wait_shift                   1
#define ssi_init_shift                   3
#define ssi_transmitter_clear_shift      4
#define ssi_receiver_clear_shift         5
#define ssi_clock_off_shift              6
#define ssi_wait_state_mask            0x3

/*!
 * SSI AC97 Control Register (SACNT) bit shift definitions
 */
#define ac97_mode_enable_shift           0
#define ac97_variable_operation_shift    1
#define ac97_tag_in_fifo_shift           2
#define ac97_read_command_shift          3
#define ac97_write_command_shift         4
#define ac97_frame_rate_divider_shift    5
#define ac97_frame_rate_mask          0x3F

/*!
 * SSI Test Register (STR) bit shift definitions
 */
#define ssi_test_mode_shift               15
#define ssi_rck2tck_shift                 14
#define ssi_rfs2tfs_shift                 13
#define ssi_rxstate_shift                 8
#define ssi_txd2rxd_shift                 7 
#define ssi_tck2rck_shift                 6
#define ssi_tfs2rfs_shift                 5
#define ssi_txstate_shift                 0

#endif

#endif
