/*
 * Copyright 2004 Freescale Semiconductor, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  
 * 02111-1307, USA
 *
 */

/*!
 * @file ssi.c
 * @brief This file contains the implementation of the SSI driver main services
 *
 *
 * @ingroup SSI
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <asm/uaccess.h>
#include <asm/arch/spba.h>      /* SPBA configuration for SSI2 */
#include <asm/arch/clock.h>

#include "ssi.h"
#include "registers.h"

#ifdef SSI_TEST

#include <linux/devfs_fs_kernel.h>      /* dev fs interface */


/*!
 * SSI1 major
 */
static int major_ssi1;

/*!
 * SSI2 major
 */
static int major_ssi2;


/*!
 * This function dumps the content of the registers of the SSI module.
 *
 * @param        module        the module number
 */
void dump_registers(ssi_mod module)
{
        printk("- SCR    = 0x%8X \n", (unsigned int)_reg_SSI_SCR(module));
        printk("- SISR   = 0x%8X \n", (unsigned int)_reg_SSI_SISR(module));
        printk("- SIER   = 0x%8X \n", (unsigned int)_reg_SSI_SIER(module));
        printk("- STCR   = 0x%8X \n", (unsigned int)_reg_SSI_STCR(module));
        printk("- SRCR   = 0x%8X \n", (unsigned int)_reg_SSI_SRCR(module));
        printk("- STCCR  = 0x%8X \n", (unsigned int)_reg_SSI_STCCR(module));
        printk("- SRCCR  = 0x%8X \n", (unsigned int)_reg_SSI_SRCCR(module));
        printk("- SFCSR  = 0x%8X \n", (unsigned int)_reg_SSI_SFCSR(module));
        printk("- STR    = 0x%8X \n", (unsigned int)_reg_SSI_STR(module));
        printk("- SOR    = 0x%8X \n", (unsigned int)_reg_SSI_SOR(module));
        printk("- SACNT  = 0x%8X \n", (unsigned int)_reg_SSI_SACNT(module));
        printk("- SACADD = 0x%8X \n", (unsigned int)_reg_SSI_SACADD(module));
        printk("- SACDAT = 0x%8X \n", (unsigned int)_reg_SSI_SACDAT(module));
        printk("- SATAG  = 0x%8X \n", (unsigned int)_reg_SSI_SATAG(module));
        printk("- STMSK  = 0x%8X \n", (unsigned int)_reg_SSI_STMSK(module));
        printk("- SRMSK  = 0x%8X \n", (unsigned int)_reg_SSI_SRMSK(module));
}
#endif

EXPORT_SYMBOL(ssi_ac97_frame_rate_divider);
EXPORT_SYMBOL(ssi_ac97_get_command_address_register);
EXPORT_SYMBOL(ssi_ac97_get_command_data_register);
EXPORT_SYMBOL(ssi_ac97_get_tag_register);
EXPORT_SYMBOL(ssi_ac97_mode_enable);
EXPORT_SYMBOL(ssi_ac97_tag_in_fifo);
EXPORT_SYMBOL(ssi_ac97_read_command);
EXPORT_SYMBOL(ssi_ac97_set_command_address_register);
EXPORT_SYMBOL(ssi_ac97_set_command_data_register);
EXPORT_SYMBOL(ssi_ac97_set_tag_register);
EXPORT_SYMBOL(ssi_ac97_variable_mode);
EXPORT_SYMBOL(ssi_ac97_write_command);
EXPORT_SYMBOL(ssi_clock_idle_state);
EXPORT_SYMBOL(ssi_clock_off);
EXPORT_SYMBOL(ssi_enable);
EXPORT_SYMBOL(ssi_get_data);
EXPORT_SYMBOL(ssi_get_status);
EXPORT_SYMBOL(ssi_i2s_mode);
EXPORT_SYMBOL(ssi_interrupt_disable);
EXPORT_SYMBOL(ssi_interrupt_enable);
EXPORT_SYMBOL(ssi_network_mode);
EXPORT_SYMBOL(ssi_receive_enable);
EXPORT_SYMBOL(ssi_rx_bit0);
EXPORT_SYMBOL(ssi_rx_clock_direction);
EXPORT_SYMBOL(ssi_rx_clock_divide_by_two);
EXPORT_SYMBOL(ssi_rx_clock_polarity);
EXPORT_SYMBOL(ssi_rx_clock_prescaler);
EXPORT_SYMBOL(ssi_rx_early_frame_sync);
EXPORT_SYMBOL(ssi_rx_fifo_counter);
EXPORT_SYMBOL(ssi_rx_fifo_enable);
EXPORT_SYMBOL(ssi_rx_fifo_full_watermark);
EXPORT_SYMBOL(ssi_rx_flush_fifo);
EXPORT_SYMBOL(ssi_rx_frame_direction);
EXPORT_SYMBOL(ssi_rx_frame_rate);
EXPORT_SYMBOL(ssi_rx_frame_sync_active);
EXPORT_SYMBOL(ssi_rx_frame_sync_length);
EXPORT_SYMBOL(ssi_rx_mask_time_slot);
EXPORT_SYMBOL(ssi_rx_prescaler_modulus);
EXPORT_SYMBOL(ssi_rx_shift_direction);
EXPORT_SYMBOL(ssi_rx_word_length);
EXPORT_SYMBOL(ssi_set_data);
EXPORT_SYMBOL(ssi_set_wait_states);
EXPORT_SYMBOL(ssi_synchronous_mode);
EXPORT_SYMBOL(ssi_system_clock);
EXPORT_SYMBOL(ssi_transmit_enable);
EXPORT_SYMBOL(ssi_two_channel_mode);
EXPORT_SYMBOL(ssi_tx_bit0);
EXPORT_SYMBOL(ssi_tx_clock_direction);
EXPORT_SYMBOL(ssi_tx_clock_divide_by_two);
EXPORT_SYMBOL(ssi_tx_clock_polarity);
EXPORT_SYMBOL(ssi_tx_clock_prescaler);
EXPORT_SYMBOL(ssi_tx_early_frame_sync);
EXPORT_SYMBOL(ssi_tx_fifo_counter);
EXPORT_SYMBOL(ssi_tx_fifo_empty_watermark);
EXPORT_SYMBOL(ssi_tx_fifo_enable);
EXPORT_SYMBOL(ssi_tx_flush_fifo);
EXPORT_SYMBOL(ssi_tx_frame_direction);
EXPORT_SYMBOL(ssi_tx_frame_rate);
EXPORT_SYMBOL(ssi_tx_frame_sync_active);
EXPORT_SYMBOL(ssi_tx_frame_sync_length);
EXPORT_SYMBOL(ssi_tx_mask_time_slot);
EXPORT_SYMBOL(ssi_tx_prescaler_modulus);
EXPORT_SYMBOL(ssi_tx_shift_direction);
EXPORT_SYMBOL(ssi_tx_word_length);

/*!
 * This function controls the AC97 frame rate divider.
 *
 * @param        module               the module number
 * @param        frame_rate_divider   the AC97 frame rate divider
 */
void ssi_ac97_frame_rate_divider(ssi_mod module, \
                                 unsigned char frame_rate_divider)
{
        _reg_SSI_SACNT(module) |= ((frame_rate_divider & ac97_frame_rate_mask) \
                                    <<ac97_frame_rate_divider_shift);
}

/*!
 * This function gets the AC97 command address register.
 *
 * @param        module        the module number
 * @return       This function returns the command address slot information.
 */
unsigned int ssi_ac97_get_command_address_register(ssi_mod module)
{
        return _reg_SSI_SACADD(module);
}

/*!
 * This function gets the AC97 command data register.
 *
 * @param        module        the module number
 * @return       This function returns the command data slot information.
 */
unsigned int ssi_ac97_get_command_data_register(ssi_mod module)
{
        return _reg_SSI_SACDAT(module);
}

/*!
 * This function gets the AC97 tag register.
 *
 * @param        module        the module number
 * @return       This function returns the tag information.
 */
unsigned int ssi_ac97_get_tag_register(ssi_mod module)
{
        return _reg_SSI_SATAG(module);
}

/*!
 * This function controls the AC97 mode.
 *
 * @param        module        the module number
 * @param        state         the AC97 mode state (enabled or disabled)
 */
void ssi_ac97_mode_enable(ssi_mod module, bool state)
{
        if (state == true) {
                _reg_SSI_SACNT(module) |= (1<<ac97_mode_enable_shift);
        }

        if (state == false) {
                _reg_SSI_SACNT(module) &= ~(1<<ac97_mode_enable_shift);
        }
}

/*!
 * This function controls the AC97 tag in FIFO behavior.
 *
 * @param        module        the module number
 * @param        state         the tag in fifo behavior (Tag info stored in Rx FIFO 0 if true, 
 * Tag info stored in SATAG register otherwise)
 */
void ssi_ac97_tag_in_fifo(ssi_mod module, bool state)
{
        if (state == true) {
                _reg_SSI_SACNT(module) |= (1<<ac97_tag_in_fifo_shift);
        }

        if (state == false) {
                _reg_SSI_SACNT(module) &= ~(1<<ac97_tag_in_fifo_shift);
        }
}

/*!
 * This function controls the AC97 read command.
 *
 * @param        module        the module number
 * @param        state         the next AC97 command is a read command or not
 */
void ssi_ac97_read_command(ssi_mod module, bool state)
{
        if (state == true) {
                _reg_SSI_SACNT(module) |= (1<<ac97_read_command_shift);
        }

        if (state == false) {
                _reg_SSI_SACNT(module) &= ~(1<<ac97_read_command_shift);
        }
}

/*!
 * This function sets the AC97 command address register.
 *
 * @param        module        the module number
 * @param        address       the command address slot information
 */
void ssi_ac97_set_command_address_register(ssi_mod module, \
                                           unsigned int address)
{
        _reg_SSI_SACADD(module) = address;
}

/*!
 * This function sets the AC97 command data register.
 *
 * @param        module        the module number
 * @param        data          the command data slot information
 */
void ssi_ac97_set_command_data_register(ssi_mod module, unsigned int data)
{
        _reg_SSI_SACDAT(module) = data;
}

/*!
 * This function sets the AC97 tag register.
 *
 * @param        module        the module number
 * @param        tag           the tag information
 */
void ssi_ac97_set_tag_register(ssi_mod module, unsigned int tag)
{
        _reg_SSI_SATAG(module) = tag;
}

/*!
 * This function controls the AC97 variable mode.
 *
 * @param        module        the module number
 * @param        state         the AC97 variable mode state (enabled or disabled)
 */
void ssi_ac97_variable_mode(ssi_mod module, bool state)
{
        if (state == true) {
                _reg_SSI_SACNT(module) |= (1<<ac97_variable_operation_shift);
        }

        if (state == false) {
                _reg_SSI_SACNT(module) &= ~(1<<ac97_variable_operation_shift);
        }
}

/*!
 * This function controls the AC97 write command.
 *
 * @param        module        the module number
 * @param        state         the next AC97 command is a write command or not
 */
void ssi_ac97_write_command(ssi_mod module, bool state)
{
        if (state == true) {
                _reg_SSI_SACNT(module) |= (1<<ac97_write_command_shift);
        }

        if (state == false) {
                _reg_SSI_SACNT(module) &= ~(1<<ac97_write_command_shift);
        }
}

/*!
 * This function controls the idle state of the transmit clock port during SSI internal gated mode.
 *
 * @param        module        the module number
 * @param        state         the clock idle state
 */
void ssi_clock_idle_state(ssi_mod module, idle_state state)
{
        ModifyRegister32(1<<ssi_clock_idle_shift, \
                         state<<ssi_clock_idle_shift, _reg_SSI_SCR(module));
}

/*!
 * This function turns off/on the ccm_ssi_clk to reduce power consumption.
 *
 * @param        module        the module number
 * @param        state         the state for ccm_ssi_clk (true: turn off, else:turn on)
 */
void ssi_clock_off(ssi_mod module, bool state)
{
        ModifyRegister32(1<<ssi_clock_off_shift, \
                         state<<ssi_clock_off_shift, \
                         _reg_SSI_SOR(module));
}

/*!
 * This function enables/disables the SSI module.
 *
 * @param        module        the module number
 * @param        state         the state for SSI module
 */
void ssi_enable(ssi_mod module, bool state)
{
        ModifyRegister32(1<<ssi_enable_shift, \
                         state<<ssi_enable_shift, \
                         _reg_SSI_SCR(module));
}

/*!
 * This function gets the data word in the Receive FIFO of the SSI module.
 *
 * @param        module        the module number
 * @param        fifo          the Receive FIFO to read
 * @return       This function returns the read data.
 */
unsigned int ssi_get_data(ssi_mod module, fifo_nb fifo)
{
        unsigned int result;
        result = 0;

        if (ssi_fifo_0 == fifo)
                result = _reg_SSI_SRX0(module);

        if (ssi_fifo_1 == fifo)
                result = _reg_SSI_SRX1(module);
                
        return result;
}

/*!
 * This function returns the status of the SSI module (SISR register) as a combination of status.
 *
 * @param        module        the module number
 * @return       This function returns the status of the SSI module.
 */
ssi_status_enable_mask ssi_get_status(ssi_mod module)
{
        ssi_status_enable_mask result;

        result = (ssi_status_enable_mask) \
                 (_reg_SSI_SISR(module) & ((1<<ssi_irq_status_number)-1));

        return result;
}

/*!
 * This function selects the I2S mode of the SSI module.
 *
 * @param        module        the module number
 * @param        mode          the I2S mode 
 */
void ssi_i2s_mode(ssi_mod module, mode_i2s mode)
{
        ModifyRegister32(3<<ssi_i2s_mode_shift, \
                         mode<<ssi_i2s_mode_shift, \
                         _reg_SSI_SCR(module));
}

/*!
 * This function disables the interrupts of the SSI module.
 *
 * @param        module        the module number
 * @param        mask          the mask of the interrupts to disable 
 */
void ssi_interrupt_disable(ssi_mod module, ssi_status_enable_mask mask)
{
        ModifyRegister32(mask, 0, _reg_SSI_SIER(module));
}

/*!
 * This function enables the interrupts of the SSI module.
 *
 * @param        module        the module number
 * @param        mask          the mask of the interrupts to enable 
 */
void ssi_interrupt_enable(ssi_mod module, ssi_status_enable_mask mask)
{
        ModifyRegister32(0, mask, _reg_SSI_SIER(module));
}

/*!
 * This function enables/disables the network mode of the SSI module.
 *
 * @param        module        the module number
 * @param        state         the network mode state 
 */
void ssi_network_mode(ssi_mod module, bool state)
{
        ModifyRegister32(1<<ssi_network_mode_shift, \
                         state<<ssi_network_mode_shift, \
                         _reg_SSI_SCR(module));
}

/*!
 * This function enables/disables the receive section of the SSI module.
 *
 * @param        module        the module number
 * @param        state         the receive section state 
 */
void ssi_receive_enable(ssi_mod module, bool state)
{
        ModifyRegister32(1<<ssi_receive_enable_shift, \
                         state<<ssi_receive_enable_shift, \
                         _reg_SSI_SCR(module));
}

/*!
 * This function configures the SSI module to receive data word at bit position 0 or 23 in the Receive shift register.
 *
 * @param        module        the module number
 * @param        state         the state to receive at bit 0 
 */
void ssi_rx_bit0(ssi_mod module, bool state)
{
        ModifyRegister32(1<<ssi_bit_0_shift, \
                         state<<ssi_bit_0_shift, \
                         _reg_SSI_SRCR(module));
}

/*!
 * This function controls the source of the clock signal used to clock the Receive shift register.
 *
 * @param        module        the module number
 * @param        direction     the clock signal direction 
 */
void ssi_rx_clock_direction(ssi_mod module, ssi_tx_rx_direction direction)
{
        ModifyRegister32(1<<ssi_clock_direction_shift, \
                         direction<<ssi_clock_direction_shift, \
                         _reg_SSI_SRCR(module));
}

/*!
 * This function configures the divide-by-two divider of the SSI module for the receive section.
 *
 * @param        module        the module number
 * @param        state         the divider state 
 */
void ssi_rx_clock_divide_by_two(ssi_mod module, bool state)
{
        ModifyRegister32(1<<ssi_divide_by_two_shift, \
                         state<<ssi_divide_by_two_shift, \
                         _reg_SSI_SRCCR(module));
}

/*!
 * This function controls which bit clock edge is used to clock in data.
 *
 * @param        module        the module number
 * @param        polarity      the clock polarity 
 */
void ssi_rx_clock_polarity(ssi_mod module, ssi_tx_rx_clock_polarity polarity)
{
        ModifyRegister32(1<<ssi_clock_polarity_shift, \
                         polarity<<ssi_clock_polarity_shift, \
                         _reg_SSI_SRCR(module));
}

/*!
 * This function configures a fixed divide-by-eight clock prescaler divider of the SSI module in series with the variable prescaler for the receive section.
 *
 * @param        module        the module number
 * @param        state         the prescaler state 
 */
void ssi_rx_clock_prescaler(ssi_mod module, bool state)
{
        ModifyRegister32(1<<ssi_prescaler_range_shift, \
                         state<<ssi_prescaler_range_shift, \
                         _reg_SSI_SRCCR(module));
}

/*!
 * This function controls the early frame sync configuration.
 *
 * @param        module        the module number
 * @param        early         the early frame sync configuration 
 */
void ssi_rx_early_frame_sync(ssi_mod module, \
                             ssi_tx_rx_early_frame_sync early)
{
        ModifyRegister32(1<<ssi_early_frame_sync_shift, \
                         early<<ssi_early_frame_sync_shift, \
                         _reg_SSI_SRCR(module));
}

/*!
 * This function gets the number of data words in the Receive FIFO.
 *
 * @param        module        the module number
 * @param        fifo          the fifo 
 * @return       This function returns the number of words in the Rx FIFO.
 */
unsigned char ssi_rx_fifo_counter(ssi_mod module, fifo_nb fifo)
{
        unsigned char result;
        result = 0;
        
        if (ssi_fifo_0 == fifo)
                result = (_reg_SSI_SFCSR(module) & \
                           (0xF<<ssi_rx_fifo_0_count_shift)) \
                         >> ssi_rx_fifo_0_count_shift;

        if (ssi_fifo_1 == fifo)
                result = (_reg_SSI_SFCSR(module) & \
                          (0xF<<ssi_rx_fifo_1_count_shift)) \
                         >> ssi_rx_fifo_1_count_shift;
        
        return result;
}

/*!
 * This function enables the Receive FIFO.
 *
 * @param        module        the module number
 * @param        fifo          the fifo to enable
 * @param        enable        the state of the fifo, enabled or disabled
 */
void ssi_rx_fifo_enable(ssi_mod module, fifo_nb fifo, bool enable)
{
        if (enable == true) {
        _reg_SSI_SRCR(module) |= ((1+fifo)<<ssi_fifo_enable_0_shift);
        }

        if (enable == false) {
                _reg_SSI_SRCR(module) &= ~((1+fifo)<<ssi_fifo_enable_0_shift);
        }
}

/*!
 * This function controls the threshold at which the RFFx flag will be set.
 *
 * @param        module        the module number
 * @param        fifo          the fifo to enable
 * @param        watermark     the watermark
 * @return       This function returns the result of the operation (0 if successful, -1 otherwise).
 */
int ssi_rx_fifo_full_watermark(ssi_mod module, \
                               fifo_nb fifo, \
                               unsigned char watermark)
{
        int result;
        result = 0;

        if ((watermark > ssi_min_fifo_watermark) && \
            (watermark <= ssi_max_fifo_watermark))
        {
        if (ssi_fifo_0 == fifo)
                ModifyRegister32(0xf<<ssi_rx_fifo_0_watermark_shift, \
                                 watermark<<ssi_rx_fifo_0_watermark_shift, \
                                 _reg_SSI_SFCSR(module));

        if (ssi_fifo_1 == fifo)
                ModifyRegister32(0xf<<ssi_rx_fifo_1_watermark_shift, \
                                 watermark<<ssi_rx_fifo_1_watermark_shift, \
                                 _reg_SSI_SFCSR(module));
        }
        else
        {
                result = -1;
        }
        return result;
}

/*!
 * This function flushes the Receive FIFOs.
 *
 * @param        module        the module number
 */
void ssi_rx_flush_fifo(ssi_mod module)
{
        ModifyRegister32(0, 1<<ssi_receiver_clear_shift, _reg_SSI_SOR(module));
}

/*!
 * This function controls the direction of the Frame Sync signal for the receive section.
 *
 * @param        module        the module number
 * @param        direction     the Frame Sync signal direction
 */
void ssi_rx_frame_direction(ssi_mod module, ssi_tx_rx_direction direction)
{
        ModifyRegister32(1<<ssi_frame_direction_shift, \
                         direction<<ssi_frame_direction_shift, \
                         _reg_SSI_SRCR(module) );
}

/*!
 * This function configures the Receive frame rate divider for the receive section.
 *
 * @param        module        the module number
 * @param        ratio         the divide ratio from 1 to 32
 * @return       This function returns the result of the operation (0 if successful, -1 otherwise).
 */
int ssi_rx_frame_rate(ssi_mod module, unsigned char ratio)
{
        int result;
        result = 0;

        if ((ratio >= ssi_min_frame_divider_ratio) && \
            (ratio <= ssi_max_frame_divider_ratio))
        ModifyRegister32(ssi_frame_divider_mask<<ssi_frame_rate_divider_shift, \
                         (ratio-1)<<ssi_frame_rate_divider_shift, \
                         _reg_SSI_SRCCR(module));
        else
                result = -1;
                
        return result;
}

/*!
 * This function controls the Frame Sync active polarity for the receive section.
 *
 * @param        module        the module number
 * @param        active        the Frame Sync active polarity
 */
void ssi_rx_frame_sync_active(ssi_mod module, \
                              ssi_tx_rx_frame_sync_active active)
{
        ModifyRegister32(1<<ssi_frame_sync_invert_shift, \
                         active<<ssi_frame_sync_invert_shift, \
                         _reg_SSI_SRCR(module));
}

/*!
 * This function controls the Frame Sync length (one word or one bit long) for the receive section.
 *
 * @param        module        the module number
 * @param        length        the Frame Sync length
 */
void ssi_rx_frame_sync_length(ssi_mod module, \
                              ssi_tx_rx_frame_sync_length length)
{
        ModifyRegister32(1<<ssi_frame_sync_length_shift, \
                         length<<ssi_frame_sync_length_shift, \
                         _reg_SSI_SRCR(module));
}

/*!
 * This function configures the time slot(s) to mask for the receive section.
 *
 * @param        module        the module number
 * @param        mask          the mask to indicate the time slot(s) masked
 */
void ssi_rx_mask_time_slot(ssi_mod module, unsigned int mask)
{
        ModifyRegister32(0xFFFFFFFF, mask, _reg_SSI_SRMSK(module));
}

/*!
 * This function configures the Prescale divider for the receive section.
 *
 * @param        module        the module number
 * @param        divider       the divide ratio from 1 to  256
 * @return       This function returns the result of the operation (0 if successful, -1 otherwise).
 */
int ssi_rx_prescaler_modulus(ssi_mod module, unsigned int divider)
{
        int result;
        result = 0;
        
        if ((divider >= ssi_min_prescaler_modulus_ratio) && \
            (divider <= ssi_max_prescaler_modulus_ratio))
                ModifyRegister32(ssi_prescaler_modulus_mask<< \
                                 ssi_prescaler_modulus_shift, \
                                 (divider-1)<<ssi_prescaler_modulus_shift, \
                                 _reg_SSI_SRCCR(module) );
        else
                result = -1;
                
        return result;  
}

/*!
 * This function controls whether the MSB or LSB will be received first in a sample.
 *
 * @param        module        the module number
 * @param        direction     the shift direction
 */
void ssi_rx_shift_direction(ssi_mod module, ssi_tx_rx_shift_direction direction)
{
        ModifyRegister32(1<<ssi_shift_direction_shift, \
                         direction<<ssi_shift_direction_shift, \
                         _reg_SSI_SRCR(module));
}

/*!
 * This function configures the Receive word length.
 *
 * @param        module        the module number
 * @param        length        the word length
 */
void ssi_rx_word_length(ssi_mod module, ssi_word_length length)
{
        ModifyRegister32(ssi_word_length_mask<<ssi_word_length_shift, 
                         length<<ssi_word_length_shift, \
                         _reg_SSI_SRCCR(module));
}

/*!
 * This function sets the data word in the Transmit FIFO of the SSI module.
 *
 * @param        module        the module number
 * @param        fifo          the FIFO number
 * @param        data          the data to load in the FIFO
 */
void ssi_set_data(ssi_mod module, fifo_nb fifo, unsigned int data)
{
        if (ssi_fifo_0 == fifo)
                _reg_SSI_STX0(module) = data;

        if (ssi_fifo_1 == fifo)
                _reg_SSI_STX1(module) = data;
}

/*!
 * This function controls the number of wait states between the core and SSI.
 *
 * @param        module        the module number
 * @param        wait          the number of wait state(s)
 */
void ssi_set_wait_states(ssi_mod module, ssi_wait_states wait)
{
        ModifyRegister32(ssi_wait_state_mask<<ssi_wait_shift , \
                         wait<<ssi_wait_shift , \
                         _reg_SSI_SOR(module));
}

/*!
 * This function enables/disables the synchronous mode of the SSI module.
 *
 * @param        module        the module number
 * @param        state         the synchronous mode state
 */
void ssi_synchronous_mode(ssi_mod module, bool state)
{
        ModifyRegister32(1<<ssi_synchronous_mode_shift, \
                         state<<ssi_synchronous_mode_shift, \
                         _reg_SSI_SCR(module));
}

/*!
 * This function allows the SSI module to output the SYS_CLK at the SRCK port.
 *
 * @param        module        the module number
 * @param        state         the system clock state
 */
void ssi_system_clock(ssi_mod module, bool state)
{
        ModifyRegister32(1<<ssi_system_clock_shift, \
                         state<<ssi_system_clock_shift, \
                         _reg_SSI_SCR(module));
}

/*!
 * This function enables/disables the transmit section of the SSI module.
 *
 * @param        module        the module number
 * @param        state         the transmit section state
 */
void ssi_transmit_enable(ssi_mod module, bool state)
{
        ModifyRegister32(1<<ssi_transmit_enable_shift, \
                         state<<ssi_transmit_enable_shift, \
                         _reg_SSI_SCR(module));
}

/*!
 * This function allows the SSI module to operate in the two channel mode.
 *
 * @param        module        the module number
 * @param        state         the two channel mode state
 */
void ssi_two_channel_mode(ssi_mod module, bool state)
{
        ModifyRegister32(1<<ssi_two_channel_shift, \
                         state<<ssi_two_channel_shift, \
                         _reg_SSI_SCR(module));
}

/*!
 * This function configures the SSI module to transmit data word from bit position 0 or 23 in the Transmit shift register.
 *
 * @param        module        the module number
 * @param        state         the transmit from bit 0 state 
 */
void ssi_tx_bit0(ssi_mod module, bool state)
{
        ModifyRegister32(1<<ssi_bit_0_shift, \
                         state<<ssi_bit_0_shift, \
                         _reg_SSI_STCR(module) );
}

/*!
 * This function controls the direction of the clock signal used to clock the Transmit shift register.
 *
 * @param        module        the module number
 * @param        direction     the clock signal direction 
 */
void ssi_tx_clock_direction(ssi_mod module, ssi_tx_rx_direction direction)
{
        ModifyRegister32(1<<ssi_clock_direction_shift, \
                         direction<<ssi_clock_direction_shift, \
                         _reg_SSI_STCR(module));
}

/*!
 * This function configures the divide-by-two divider of the SSI module for the transmit section.
 *
 * @param        module        the module number
 * @param        state         the divider state 
 */
void ssi_tx_clock_divide_by_two(ssi_mod module, bool state)
{
        ModifyRegister32(1<<ssi_divide_by_two_shift, \
                         state<<ssi_divide_by_two_shift, \
                         _reg_SSI_STCCR(module));
}

/*!
 * This function controls which bit clock edge is used to clock out data.
 *
 * @param        module        the module number
 * @param        polarity      the clock polarity 
 */
void ssi_tx_clock_polarity(ssi_mod module, ssi_tx_rx_clock_polarity polarity)
{
        ModifyRegister32(1<<ssi_clock_polarity_shift, \
                         polarity<<ssi_clock_polarity_shift, \
                         _reg_SSI_STCR(module));
}

/*!
 * This function configures a fixed divide-by-eight clock prescaler divider of the SSI module in series with the variable prescaler for the transmit section.
 *
 * @param        module        the module number
 * @param        state         the prescaler state 
 */
void ssi_tx_clock_prescaler(ssi_mod module, bool state)
{
        ModifyRegister32(1<<ssi_prescaler_range_shift, \
                         state<<ssi_prescaler_range_shift, \
                         _reg_SSI_STCCR(module));
}

/*!
 * This function controls the early frame sync configuration for the transmit section.
 *
 * @param        module        the module number
 * @param        early         the early frame sync configuration 
 */
void ssi_tx_early_frame_sync(ssi_mod module, ssi_tx_rx_early_frame_sync early)
{
        ModifyRegister32(1<<ssi_early_frame_sync_shift, \
                         early<<ssi_early_frame_sync_shift, \
                         _reg_SSI_STCR(module));
}

/*!
 * This function gets the number of data words in the Transmit FIFO.
 *
 * @param        module        the module number
 * @param        fifo          the fifo 
 * @return       This function returns the number of words in the Tx FIFO.
 */
unsigned char ssi_tx_fifo_counter(ssi_mod module, fifo_nb fifo)
{
        unsigned char result;
        result = 0;
        
        if (ssi_fifo_0 == fifo)
                result = ( _reg_SSI_SFCSR(module) & \
                           (0xF<<ssi_tx_fifo_0_count_shift) ) \
                         >> ssi_tx_fifo_0_count_shift;

        if (ssi_fifo_1 == fifo)
                result = ( _reg_SSI_SFCSR(module) & \
                           (0xF<<ssi_tx_fifo_1_count_shift) ) \
                         >> ssi_tx_fifo_1_count_shift;
        
        return result;
}

/*!
 * This function controls the threshold at which the TFEx flag will be set.
 *
 * @param        module        the module number
 * @param        fifo          the fifo to enable
 * @param        watermark     the watermark
 * @return       This function returns the result of the operation (0 if successful, -1 otherwise).
 */
int ssi_tx_fifo_empty_watermark(ssi_mod module, \
                                fifo_nb fifo, \
                                unsigned char watermark)
{
        int result;
        result = 0;

        if ((watermark > ssi_min_fifo_watermark) && \
            (watermark <= ssi_max_fifo_watermark))
        {
        if (ssi_fifo_0 == fifo)
                ModifyRegister32(0xf<<ssi_tx_fifo_0_watermark_shift, \
                                 watermark<<ssi_tx_fifo_0_watermark_shift, \
                                 _reg_SSI_SFCSR(module) );

        if (ssi_fifo_1 == fifo)
                ModifyRegister32(0xf<<ssi_tx_fifo_1_watermark_shift, \
                                 watermark<<ssi_tx_fifo_1_watermark_shift, \
                                 _reg_SSI_SFCSR(module) );
        }
        else
        {
                result = -1;
        }
        return result;
}

/*!
 * This function enables the Transmit FIFO.
 *
 * @param        module        the module number
 * @param        fifo          the fifo to enable
 * @param        enable        the state of the fifo, enabled or disabled
 */
void ssi_tx_fifo_enable(ssi_mod module, fifo_nb fifo, bool enable)
{
        if (enable == true) {
                _reg_SSI_STCR(module) |= ((1+fifo)<<ssi_fifo_enable_0_shift);
        }
        if (enable == false) {
                _reg_SSI_STCR(module) &= ~((1+fifo)<<ssi_fifo_enable_0_shift);
        }
}

/*!
 * This function flushes the Transmit FIFOs.
 *
 * @param        module        the module number
 */
void ssi_tx_flush_fifo(ssi_mod module)
{
        ModifyRegister32(0, \
                         1<<ssi_transmitter_clear_shift, \
                         _reg_SSI_SOR(module));
}

/*!
 * This function controls the direction of the Frame Sync signal for the transmit section.
 *
 * @param        module        the module number
 * @param        direction     the Frame Sync signal direction
 */
void ssi_tx_frame_direction(ssi_mod module, ssi_tx_rx_direction direction)
{
        ModifyRegister32(1<<ssi_frame_direction_shift, \
                         direction<<ssi_frame_direction_shift, \
                         _reg_SSI_STCR(module));
}

/*!
 * This function configures the Transmit frame rate divider.
 *
 * @param        module        the module number
 * @param        ratio         the divide ratio from 1 to 32
 * @return       This function returns the result of the operation (0 if successful, -1 otherwise).
 */
int ssi_tx_frame_rate(ssi_mod module, unsigned char ratio)
{
        int result;
        result = 0;

        if ((ratio >= ssi_min_frame_divider_ratio) && \
            (ratio <= ssi_max_frame_divider_ratio))
                ModifyRegister32(ssi_frame_divider_mask<< \
                                 ssi_frame_rate_divider_shift, \
                                 (ratio-1)<<ssi_frame_rate_divider_shift, \
                                 _reg_SSI_STCCR(module) );
        else
                result = -1;
                
        return result;
}

/*!
 * This function controls the Frame Sync active polarity for the transmit section.
 *
 * @param        module        the module number
 * @param        active        the Frame Sync active polarity
 */
void ssi_tx_frame_sync_active(ssi_mod module, \
                              ssi_tx_rx_frame_sync_active active)
{
        ModifyRegister32(1<<ssi_frame_sync_invert_shift, \
                         active<<ssi_frame_sync_invert_shift, \
                         _reg_SSI_STCR(module));
}

/*!
 * This function controls the Frame Sync length (one word or one bit long) for the transmit section.
 *
 * @param        module        the module number
 * @param        length        the Frame Sync length
 */
void ssi_tx_frame_sync_length(ssi_mod module, \
                              ssi_tx_rx_frame_sync_length length)
{
        ModifyRegister32(1<<ssi_frame_sync_length_shift, \
                         length<<ssi_frame_sync_length_shift, \
                         _reg_SSI_STCR(module));
}

/*!
 * This function configures the time slot(s) to mask for the transmit section.
 *
 * @param        module        the module number
 * @param        mask          the mask to indicate the time slot(s) masked
 */
void ssi_tx_mask_time_slot(ssi_mod module, unsigned int mask)
{
        ModifyRegister32(0xFFFFFFFF, mask, _reg_SSI_STMSK(module));
}

/*!
 * This function configures the Prescale divider for the transmit section.
 *
 * @param        module        the module number
 * @param        divider       the divide ratio from 1 to  256
 * @return       This function returns the result of the operation (0 if successful, -1 otherwise).
 */
int ssi_tx_prescaler_modulus(ssi_mod module, unsigned int divider)
{
        int result;
        result = 0;
        
        if ((divider >= ssi_min_prescaler_modulus_ratio) && \
            (divider <= ssi_max_prescaler_modulus_ratio))
                ModifyRegister32(ssi_prescaler_modulus_mask<<\
                                 ssi_prescaler_modulus_shift, \
                                 (divider-1)<<ssi_prescaler_modulus_shift, \
                                 _reg_SSI_STCCR(module));
        else
                result = -1;
                
        return result;  
}

/*!
 * This function controls whether the MSB or LSB will be transmitted first in a sample.
 *
 * @param        module        the module number
 * @param        direction     the shift direction
 */
void ssi_tx_shift_direction(ssi_mod module, ssi_tx_rx_shift_direction direction)
{
        ModifyRegister32(1<<ssi_shift_direction_shift, \
                         direction<<ssi_shift_direction_shift, \
                         _reg_SSI_STCR(module) );
}

/*!
 * This function configures the Transmit word length.
 *
 * @param        module        the module number
 * @param        length        the word length
 */
void ssi_tx_word_length(ssi_mod module, ssi_word_length length)
{
        ModifyRegister32(ssi_word_length_mask<<ssi_word_length_shift, \
                         length<<ssi_word_length_shift, \
                         _reg_SSI_STCCR(module));
}


#ifdef SSI_TEST

static char ssi1_buf[BUF_SIZE];
static char ssi2_buf[BUF_SIZE];

struct ssi_priv_data{
	int ssi;
	int size;
	char* buf;
};

static struct ssi_priv_data ssi1_priv = {
	0,
	0,
	0
};

static struct ssi_priv_data ssi2_priv = {
	0,
	0,
	0
};

/*!
 * This function implements IOCTL controls on a SSI device.
 *
 * @param        inode       pointer on the node
 * @param        file        pointer on the file
 * @param        cmd         the command
 * @param        arg         the parameter
 * @return       This function returns 0 if successful.
 */
static int ssi_ioctl(struct inode *inode, \
                     struct file *file, \
                     unsigned int cmd, \
                     unsigned long arg)
{
        int result = 0;
	unsigned char prescaler;
	wave_configuration *wav_config;
        ssi_mod mod;

        if (_IOC_TYPE(cmd) != SSI_IOCTL)
                return -ENOTTY;

        switch (cmd) {
        case SSI_CONFIG_FOR_ATLAS:
                wav_config = (wave_configuration *)(arg);
		mod = wav_config->id_SSI;

	        /*
        	 * SPBA configuration for SSI2 - SDMA and MCU are set 
         	*/
        	spba_take_ownership(SPBA_SSI2, SPBA_MASTER_C|SPBA_MASTER_A);

#if defined(CONFIG_ARCH_SCMA11)
		/* assign USBPLL to be used by SSI1 */
		mxc_set_clocks_pll(SSI1_BAUD, USBPLL);
	
		/*set divider to 2*/
		mxc_set_clocks_div(SSI1_BAUD, 2);
	
		/* enable clock*/
		mxc_clks_enable(SSI1_BAUD);
	
		/* assign USBPLL to be used by SSI2 */
		mxc_set_clocks_pll(SSI2_BAUD, USBPLL);
	
		/*set divider to 2*/
		mxc_set_clocks_div(SSI2_BAUD, 2);
	
		/* enable clock*/
		mxc_clks_enable(SSI2_BAUD);  
#endif
	             
#if defined(CONFIG_ARCH_ARGONPLUS)
                ModifyRegister32(0x18000000,
                                 0x00000000,
                                 _reg_MCU_MCR);

                ModifyRegister32(0x0003FFFF,
                                 0x00000C06,
                                 _reg_MCU_MPDR1);
#endif
	        /****** SCR */
                /*! Reset SSI */
                ssi_enable(mod, false);
                /* Start by disabling all interrupts */
                ssi_interrupt_disable(mod, 0xFFFFFFFF);
                /* Disable System clock */
                ssi_system_clock(mod, false);
		ssi_clock_idle_state(mod, true);
                ssi_two_channel_mode(mod, true);
                /*! configure other registers */
                ssi_i2s_mode(mod, i2s_normal);
                ssi_synchronous_mode(mod, true);
                /*! Network mode config, enable Timeslots 0 and 1 */
                ssi_network_mode(mod, true);
                ssi_tx_fifo_enable(mod, ssi_fifo_0, true);
                ssi_tx_fifo_enable(mod, ssi_fifo_1, true);

	        /****** STCR */
                /*! transmit config */
                ssi_tx_early_frame_sync(mod, ssi_frame_sync_one_bit_before);
                ssi_tx_frame_sync_length(mod, ssi_frame_sync_one_bit);
                ssi_tx_frame_sync_active(mod, ssi_frame_sync_active_high);
                ssi_tx_clock_polarity(mod, ssi_clock_on_rising_edge);
                ssi_tx_shift_direction(mod, ssi_msb_first);
                if ((wav_config->mode_test == 3) && (mod == 1)) {
                        ssi_tx_frame_direction(mod, ssi_tx_rx_externally);
                        ssi_tx_clock_direction(mod, ssi_tx_rx_externally);
                } else {
                        ssi_tx_frame_direction(mod, ssi_tx_rx_internally);
                        ssi_tx_clock_direction(mod, ssi_tx_rx_internally);
		}
                ssi_tx_bit0(mod, true);

	        /****** STCCR */
                if (wav_config->bits_per_sample == 16) {
                        ssi_tx_word_length(mod, ssi_16_bits);		/* WL */
                } else {
                        ssi_tx_word_length(mod, ssi_8_bits);		/* WL */
                }
                ssi_tx_clock_divide_by_two(mod, false);		/* DIV2 */
                ssi_tx_frame_rate(mod, (unsigned char ) 4);	/* DC */ 
#if defined(CONFIG_ARCH_MX3)
                prescaler = (unsigned char)(44344444 / 
#endif
#if defined(CONFIG_ARCH_ARGONPLUS)
                prescaler = (unsigned char)(49887500 / 
#endif
#if defined(CONFIG_ARCH_SCMA11)
                prescaler = (unsigned char)(24000000 / 
#endif
                            (2 * wav_config->sample_rate * 
                            wav_config->bits_per_sample * 
                            wav_config->num_channels));
                ssi_tx_prescaler_modulus(mod, (unsigned char ) prescaler);/* PM */
                ssi_tx_clock_prescaler(mod, false);		/* PSR */
		
                /****** SRCR */
                /*! receive config */
                ssi_rx_early_frame_sync(mod, ssi_frame_sync_one_bit_before);
                ssi_rx_frame_sync_length(mod, ssi_frame_sync_one_bit);
                ssi_rx_frame_sync_active(mod, ssi_frame_sync_active_high);
                ssi_rx_clock_polarity(mod, ssi_clock_on_rising_edge);
                ssi_rx_shift_direction(mod, ssi_msb_first);
                ssi_rx_frame_direction(mod, ssi_tx_rx_externally);
                ssi_rx_clock_direction(mod, ssi_tx_rx_externally);
                ssi_rx_bit0(mod, true);

                /****** SRCCR */
                /*! receive clock control */
                if (wav_config->bits_per_sample == 16) {
                        ssi_rx_word_length(mod, ssi_16_bits);
		} else {
                        ssi_rx_word_length(mod, ssi_8_bits);
                }
                ssi_rx_clock_divide_by_two(mod, false);
                ssi_rx_frame_rate(mod, (unsigned char ) 4);
                ssi_rx_prescaler_modulus(mod, (unsigned char ) prescaler);
                ssi_rx_clock_prescaler(mod, false);

                /****** SFCSR */
                ssi_tx_fifo_empty_watermark(mod, \
                                            ssi_fifo_0, \
                                            (unsigned char ) 1);
                ssi_tx_fifo_empty_watermark(mod, \
                                            ssi_fifo_1, \
                                            (unsigned char ) 1);
                ssi_rx_fifo_full_watermark(mod, ssi_fifo_0, (unsigned char ) 8);
		ssi_rx_fifo_full_watermark(mod, ssi_fifo_1, (unsigned char ) 8);

                ssi_enable(mod, true);
                ssi_transmit_enable(mod, false);

                if (mod == SSI1) {
                        /****** STMSK */
                        ssi_tx_mask_time_slot(mod, 0xFFFFFFFC);
                        /****** SRMSK */
                        ssi_rx_mask_time_slot(mod, 0xFFFFFFFC);
                } else {
		        if (wav_config->mode_test == 2) {
                                /****** STMSK */
                                ssi_tx_mask_time_slot(mod, 0xFFFFFFFC);
                                /****** SRMSK */
                                ssi_rx_mask_time_slot(mod, 0xFFFFFFFC);
			} else {
                                /****** STMSK */
                                ssi_tx_mask_time_slot(mod, 0xFFFFFFF3);
                                /****** SRMSK */
                                ssi_rx_mask_time_slot(mod, 0xFFFFFFF3);
			}
                }

                break;

        case SSI_PLAY_MIX:
/*
                ssi_transmit_enable(0, true);
                ssi_transmit_enable(1, true);

                i = 0;
                j = 0;
                while ((data_size[0] >= 4) || (data_size[1] >= 4)) {
                        if (((_reg_SSI_SFCSR(0) & 0xF00) <= 0x600) && 
                             (data_size[0] >= 4)) {
                                left =(unsigned int)(((data_depeche[0][i+1]<<8)&0xFF00)|
                                                     (data_depeche[0][i]&0x00FF));
                                right =(unsigned int)(((data_depeche[0][i+3]<<8)&0xFF00)|
                                                     (data_depeche[0][i+2]&0x00FF));

                                ssi_set_data(0, ssi_fifo_0, left);
                                ssi_set_data(0, ssi_fifo_1, right);

                                data_size[0] -= 4;
                                i += 4;
			}

                        if (((_reg_SSI_SFCSR(1) & 0xF00) <= 0x600) && 
                             (data_size[1] >= 4)) {
                                left =(unsigned int)(((data_depeche[1][j+1]<<8)&0xFF00)|
                                                     (data_depeche[1][j]&0x00FF));
                                right =(unsigned int)(((data_depeche[1][j+3]<<8)&0xFF00)|
                                                      (data_depeche[1][j+2]&0x00FF));

                                ssi_set_data(1, ssi_fifo_0, left);
                                ssi_set_data(1, ssi_fifo_1, right);

                                data_size[1] -= 4;
                                j += 4;
			}
                }

                ssi_transmit_enable(0, true);
                ssi_transmit_enable(1, true);
*/
                break;

        default:
                break;
        }

        return result;
}

int write_fifo(struct ssi_priv_data* priv)
{
        int i, left, right;
        int data_size;

	i = 0;	
	data_size = priv->size;
        ssi_transmit_enable(priv->ssi, true);
        
        while(data_size >= 4){
	        if ((_reg_SSI_SFCSR(priv->ssi) & 0xF00) <= 0x600){
        	        left =(unsigned int)(((priv->buf[i + 1] << 8) & 0xFF00)|
                                                     (priv->buf[i] & 0x00FF));
                        right =(unsigned int)(((priv->buf[i + 3] << 8) & 0xFF00)|
                                                     (priv->buf[i + 2] & 0x00FF));
                        ssi_set_data(priv->ssi, ssi_fifo_0, left);
                        ssi_set_data(priv->ssi, ssi_fifo_1, right);
                        data_size -= 4;
                        i += 4;
		}
	}

        ssi_transmit_enable(priv->ssi, false);
        return priv->size;
}

/*!
 * This function implements the open method on a SSI device.
 *
 * @param        inode       pointer on the node
 * @param        file        pointer on the file
 * @return       This function returns 0.
 */
static int ssi_open(struct inode *inode, struct file *file)
{
	unsigned int minor = MINOR(inode->i_rdev);
	struct ssi_priv_data* priv;
	                
        priv = (minor == 0) ? &ssi1_priv : &ssi2_priv;
        priv->buf = (minor == 0) ? &ssi1_buf[0] : &ssi2_buf[0];
        	
        priv->ssi = minor;
        priv->size = 0;
        
        file->private_data = (void*)priv;
        
        return 0;
}

/*!
 * This function implements the release method on a SSI device.
 *
 * @param        inode       pointer on the node
 * @param        file        pointer on the file
 * @return       This function returns 0.
 */
static int ssi_free(struct inode *inode, struct file *file)
{
	struct ssi_priv_data* priv;
	
	priv = (struct ssi_priv_data*)file->private_data;
	file->private_data = NULL;
	
        return 0;
}

static ssize_t ssi_write(struct file* file, const char* buf, 
                               size_t bytes, loff_t* off)
{
	struct ssi_priv_data* priv;
	unsigned int minor;
	int res = 0;
	
	minor = MINOR(file->f_dentry->d_inode->i_rdev);
	priv = (struct ssi_priv_data*)file->private_data;
		
	res = copy_from_user(priv->buf, (void*)buf, bytes);
	if(res){
		printk("write: fault error when copying user data\n");
		return -EFAULT;
	}
	
	priv->size = bytes;
	return write_fifo(priv);
}

static ssize_t ssi_read(struct file* file, char* buf, 
                              size_t bytes, loff_t* off)
{
	struct ssi_priv_data* priv;
	unsigned int minor;
	
	minor = MINOR(file->f_dentry->d_inode->i_rdev);
	
	priv = (struct ssi_priv_data*)file->private_data;
	
	return 0;
}

/*!
 * This structure defines file operations for a SSI device.
 */
static struct file_operations ssi_fops = {
      /*!
       * the owner
       */
        .owner =        THIS_MODULE,
        
        .read  = 	ssi_read,
        
        .write = 	ssi_write,
        
      /*!
       * the ioctl operations
       */
        .ioctl =        ssi_ioctl,
        
      /*!
       * the open operation
       */
        .open =         ssi_open,
        
      /*!
       * the release operation
       */
        .release =      ssi_free,
};
#endif

/*!
 * This function implements the init function of the SSI device.
 * This function is called when the module is loaded.
 *
 * @return       This function returns 0.
 */
static int __init ssi_init(void)
{
#ifdef SSI_TEST
        printk(KERN_DEBUG "ssi : ssi_init(void) \n");

        major_ssi1 = register_chrdev(0,"ssi1", &ssi_fops);
        if (major_ssi1 < 0)
        {
                printk(KERN_WARNING "Unable to get a major for ssi1");
                return major_ssi1;
        }

        major_ssi2 = register_chrdev(0,"ssi2", &ssi_fops);

        if (major_ssi2 < 0)
        {
                printk(KERN_WARNING "Unable to get a major for ssi2");
                return major_ssi2;
        }

        printk("ssi : creating devfs entry for ssi1 \n");
        devfs_mk_cdev(MKDEV(major_ssi1,0), \
                      S_IFCHR | S_IRUGO | S_IWUSR, "ssi1" );
        printk("ssi : creating devfs entry for ssi2 \n");
        devfs_mk_cdev(MKDEV(major_ssi2,0), \
                      S_IFCHR | S_IRUGO | S_IWUSR, "ssi2" );

#endif
        return 0;
}

/*!
 * This function implements the exit function of the SPI device.
 * This function is called when the module is unloaded.
 *
 */
static void __exit ssi_exit(void)
{
#ifdef SSI_TEST
        devfs_remove("ssi1");
        unregister_chrdev(major_ssi1, "ssi1");

        devfs_remove("ssi2");
        unregister_chrdev(major_ssi2, "ssi2");

        printk(KERN_DEBUG "ssi : successfully unloaded\n");
#endif
}

module_init(ssi_init);
module_exit(ssi_exit);

MODULE_DESCRIPTION("SSI char device driver");
MODULE_AUTHOR("Freescale Semiconductor, Inc.");
MODULE_LICENSE("GPL");

