/*
*  File Name       : /drivers/img/mp200_img.h
*  Function        : IMG Driver
*  Release Version : Ver 1.00
*  Release Date    : 2005/05/25
*
*  Copyright (C) NEC Electronics Corporation 2005
*
*
*  This program is free software;you can redistribute it and/or modify it under the terms of 
*  the GNU General Public License as published by Free Softwere Foundation; either version 2 
*  of License, or (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
*  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
*  See the GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along with this program; 
*  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
*  MA 02111-1307, USA.
*
*/

#ifndef _MP200_IMG_H_
#define _MP200_IMG_H_


/****************************************************************************/
/* Device information structure                                             */
/****************************************************************************/
typedef struct mp200_img_dev {
    int                    flag;    /* status of device information structure */
                                    /*   FALSE(0):usable TRUE(1):used         */
    int vmas;                       /* active mappings                        */
    struct semaphore       sem_vm;  /* exclusive resource (for mmap)          */
    void                  *virt_addr;/* virtual address of last mmap          */
    unsigned long          phys_addr; /* physical address of last mmap        */
    struct semaphore       sem;       /* exclusive resource                   */
    wait_queue_head_t      wait_que;  /* wait queue                           */
    int                    sleep_mode;/* sleep status                         */
                                      /*   FALSE(0):wakeup TRUE(1):sleep      */
    int                    status;    /* status of IMG request process        */
                                      /*   IMG_STATUS_DONE(0)     : done      */
                                      /*   IMG_STATUS_WAIT(1)     : running   */
                                      /*   IMG_STATUS_NO_REQ(2)   : no request*/
                                      /*   IMG_STATUS_CANCELLED(3): cancelled */
    struct mp200_img_req  req;  /*IMG request(ATENTTION:It is variable length)*/
                                /*req.status(status of request)               
                                   <if status=0(IMG_STATUS_DONE)>             
                                     IMG_CALLBACK_SUCCESS      : IMG process is
                                                       completed (Success)    
                                     IMG_CALLBACK_SET_PARAM    : set values to
                                                      IMG registers (Success) 
                                     IMG_CALLBACK_CANCEL       : cancelled
                                                  request is done (Success)   
                                     IMG_CALLBACK_FAIL         : can't start
                                                    IMG process (Error)       
                                     IMG_CALLBACK_IMG_ERROR    : IMG ERROR
                                                  interrupt (Error)           
                                     IMG_CALLBACK_IMG_STOP     : IMG
                                            RETRY/SPLIT interrupt (Error)     
                                           <if status=1(IMG STATUS WAIT)>     
                                     IMG_REQ_STATUS_REGISTERED : registered
                                                   (before running)           
                                     IMG_REQ_STATUS_RUNNING    : running
                                                (running)                     
                                     IMG_REQ_STATUS_WAIT_CANCEL: waiting for
                                       cancelled request to be done (running) 
                                             req.callback (callback function) 
                                             set mp200_img_callback()         */
} MP200_IMG_Dev;


/* ------------------------------------------- */
/*   Public functions                          */
/* ------------------------------------------- */
void mp200_img_preferred_regist( struct mp200_img_req *p );
void mp200_img_regist( struct mp200_img_req *p );
int  mp200_img_cancel( struct mp200_img_req *p, int flag );
int  mp200_img_reschedule( void );
void mp200_img_guardtime( int ms );


#endif /* _MP200_IMG_H_ */

