/*
*  File Name       : /drivers/rot/mp200_rot.h
*  Function        : ROT Driver
*  Release Version : Ver 1.00
*  Release Date    : 2005/10/24
*
*  Copyright (C) NEC Electronics Corporation 2005
*
*
*  This program is free software;you can redistribute it and/or modify it under the terms of 
*  the GNU General Public License as published by Free Softwere Foundation; either version 2 
*  of License, or (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
*  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
*  See the GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along with this program; 
*  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
*  MA 02111-1307, USA.
*
*/

#ifndef _MP200_ROT_H_
#define _MP200_ROT_H_


#ifndef TRUE
#define TRUE 1
#endif /* TRUE */
#ifndef FALSE
#define FALSE 0
#endif /* FALSE */


/****************************************************************************/
/* Device information structure                                             */
/****************************************************************************/
typedef struct mp200_rot_dev {
    int                    flag;        /* status of device information structure */
                                        /*   FALSE(0):usable TRUE(1):used         */
    int vmas;                           /* active mappings                        */
    struct semaphore       sem_vm;      /* exclusive resource (for mmap)          */
    void                  *virt_addr;   /* virtual address of last mmap           */
    unsigned long          phys_addr;   /* physical address of last mmap          */
    struct semaphore       sem;         /* exclusive resource                     */
    wait_queue_head_t      wait_que;    /* wait queue                             */
    int                    sleep_mode;  /* sleep status                           */
                                        /*   FALSE(0):wakeup TRUE(1):sleep        */
    int                    status;      /* status of ROT request process          */
                                        /*   ROT_STATUS_DONE(0)     : done        */
                                        /*   ROT_STATUS_WAIT(1)     : running     */
                                        /*   ROT_STATUS_NO_REQ(2)   : no request  */
                                        /*   ROT_STATUS_CANCELLED(3): cancelled   */
    struct mp200_rot_req  req;  /* ROT request(ATENTTION : It is variable length)                        */
                                /*   req.status(status of request)                                       */
                                /*     <if status=0(ROT_STATUS_DONE)>                                    */
                                /*     ROT_CALLBACK_SUCCESS      : ROT process is completed (Success)    */
                                /*     ROT_CALLBACK_SET_PARAM    : set values to ROT registers (Success) */
                                /*     ROT_CALLBACK_CANCEL       : cancelled request is done (Success)   */
                                /*     ROT_CALLBACK_FAIL         : can't start ROT process (Error)       */
                                /*     ROT_CALLBACK_ROT_ERROR    : ROT ERROR interrupt (Error)           */
                                /*     ROT_CALLBACK_ROT_STOP     : ROT RETRY/SPLIT interrupt (Error)     */
                                /*     <if status=1(ROT STATUS WAIT)>                                    */
                                /*     ROT_REQ_STATUS_REGISTERED : registered (before running)           */
                                /*     ROT_REQ_STATUS_RUNNING    : running (running)                     */
                                /*     ROT_REQ_STATUS_WAIT_CANCEL: waiting for cancelled request to be done (running) */
                                /*   req.callback (callback function)                                    */
                                /*     set mp200_rot_callback()                                          */
} MP200_ROT_Dev;


/****************************************************************************/
/* Prototype declarations                                                   */
/****************************************************************************/
/* ------------------------------------------- */
/*   Public functions                          */
/* ------------------------------------------- */
void mp200_rot_regist( struct mp200_rot_req *p );
int  mp200_rot_cancel( struct mp200_rot_req *p, int flag );


#endif /* _MP200_ROT_H_ */

