/*
 * File Name       : /drivers/video/qfb.c
 * Function        : QVGA FraemBuffer Driver
 * Release Version : Ver 1.00
 * Release Date    : 2005/05/25
 *
 * Copyright (C) NEC Electronics Corporation 2005
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2 
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 *  MA 02111-1307, USA.
 *
 */

/*
 * Copyright 2006,2007 Sony Corporation. 
 */

/*
 *
 *  framebuffer driver for D77601 -- based on vfb.c
 *
 */

/*
 *  linux/drivers/video/vfb.c -- Virtual frame buffer device
 *
 *      Copyright (C) 2002 James Simmons
 *
 *	Copyright (C) 1997 Geert Uytterhoeven
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of this archive for
 *  more details.
 */

#define USE_INT_ROT 0
#define SHARE_INT_ROT 1
#define ENABLE_INT_ROT 1

#define TIMER_TABLE 1		/* interFb_timetable is available(not constant TIMER_CYCLE) */
#define TIMER_TABLE_IN_FB 1	/* interFb_timetable is in fb(not present from DSP/ROT) */
#define TIMER_CYCLE 6		/* timer default cycle by 10mS */

#define DPM_LCD_SUSPEND 1	/* ENABLE LCD power control in suspend */
#define DPM_LCD_RESUME  1	/* ENABLE LCD power control in resume  */
#define DPM_IMG_RESUME  1	/* ENABLE IMG initialaize   in resume  */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/tty.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <asm/uaccess.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <linux/timer.h>
#include <linux/sched.h>

#include <asm/arch/qfb.h>
#include <asm/arch/hardware.h>
#include <asm/arch/gpiocore.h>
#include <asm/arch/irqs.h>
#if UNRESET_LCD
#include <asm/arch/pmu.h>
#endif				/* UNRESET_LCD */

#if USE_INT_ROT
//#include "rot.h"
#include <asm/arch/rot.h>
#endif	/* USE_INT_ROT */
#define MP200_FB 1
#include <asm/arch/img.h>
#include "lcdc.h"
#include "fifo.h"
#include "xcommon.h"
#if ENABLE_3D
#include "gshark.h"
#include "glxcommon.h"
#endif	/* ENABLE_3D */
#if 0 // move to include/linux/mp200_mem.h
#include "dsp.h"
#endif

#define MASK_LCD_POWER_OFF		0x27
#define DATA_LCD_POWER_OFF		0x21	/* set LCDRST, off LCD_BL,
						   enable LCD_EN_POC, LCD
						   suspend on */

#ifdef CONFIG_DPM
#include <linux/device.h>

volatile bool DPM_suspend_flg = 0;

static int qfb_suspend(struct device *dev, u32 state, u32 level);
static int qfb_resume(struct device *dev, u32 level);

#endif	/* CONFIG_DPM */

    /*
     *  RAM we reserve for the frame buffer. This defines the maximum screen
     *  size
     *
     *  The default can be overridden if the driver is compiled as a module
     */

int FrontWidth = FRONT_WIDTH;
int FrontHeight = FRONT_HEIGHT;
int FrontWidthV = FRONT_WIDTH_V;
int FrontHeightV = FRONT_HEIGHT_V;
int BytesPerPixel = BYTES_PER_PIXEL;	/* for DispBuf (2D) */
int BitsPerPixel = BITS_PER_PIXEL;	/* for FrameBuf (LCDC) */

/* Smem */
static ulong Smem = SMEM_START;	/* paddr */
static ulong SmemLength = SMEM_LENGTH;
char *SmemV;			/* vaddr */

ulong DispBuf = SMEM_START + DISPBUF_OFFSET;	/* paddr */
char *DispBufV;			/* vaddr */
ulong DispBufLength = DISPBUF_LENGTH;
ulong FrameBufA = SMEM_START + FRAMEBUFA_OFFSET;	/* paddr */
ulong FrameBufB = SMEM_START + FRAMEBUFB_OFFSET;	/* paddr */
char *FrameBufAV;		/* vaddr */
char *FrameBufBV;		/* vaddr */
ulong FrameBufLength = FRAMEBUF_LENGTH;

#if ENABLE_3D
GLXWindowTable *pGLXWindowTable;	/* pointer to GLXWindowTable */
#endif	/* ENABLE_3D */
BLD_SCRN_LIST *pBldScreenList;	/* pointer to BLD_SCRN_LIST */
int *pXvImageUpdatedFlag;	/* pointer to Overlay image is updated
				   (ROT interrupt) flag */
int *pXvImageOnScrnFlag;	/* pointer to Overlay image is now on
				   screen flag */

#if ENABLE_3D
/* 3D Smem */
ulong GLXSmem = GSHARK_COMMON_BASE;	/* paddr */
ulong GLXSmemLength = GSHARK_COMMON_SIZE;
char *GLXSmemV;			/* vaddr */

/* GSHARK MMIO */
static ulong Mmio = GSHARK_BASE;	/* paddr */
static ulong MmioLength = GSHARK_SIZE;
char *MmioV;			/* vaddr */
#endif	/* ENABLE_3D */

#if USE_INT_ROT
/* ROT MMIO */
static ulong ROTMmio = ROTATOR_BASE;	/* paddr */
static ulong ROTMmioLength = ROTATOR_SIZE;
char *ROTMmioV;			/* vaddr */
#endif	/* USE_INT_ROT */

#ifndef CONFIG_MP200_IMG
/* IMG MMIO */
static ulong IMGMmio = IMAGEPROC_BASE;	/* paddr */
static ulong IMGMmioLength = IMAGEPROC_SIZE;
char *IMGMmioV;			/* vaddr */
#endif				/* !defined(CONFIG_MP200_IMG) */

/* LCDC MMIO */
static ulong LCDCMmio = LCDC_BASE;	/* paddr */
static ulong LCDCMmioLength = LCDC_SIZE;
char *LCDCMmioV;		/* vaddr */

/* FIFO MMIO */
static ulong FIFOMmio = FIFO_BASE;	/* paddr */
static ulong FIFOMmioLength = FIFO_SIZE;
char *FIFOMmioV;		/* vaddr */

/* DSP Smem */
static ulong DSPSmem = DSP_SMEM_START;	/* paddr */
static ulong DSPSmemLength = DSP_SMEM_LENGTH;
static char *DSPSmemV;		/* vaddr */

int index_BldScreenList = 0;	/* index of BLD_SCRN_LIST */
int override_BldScreenList = 1;	/* TRUE : possible to override
				   BLD_SCRN_LIST */
					/* FALSE: impossible to override
					 * BLD_SCRN_LIST */
Xv_Attr XvAttribute;		/* overlay parameter */
interFb_bufaddr *pinterFb_bufaddr = NULL;	/* pointer to interFb_bufaddr */
InterFb_BufInfo *pInterFb_BufInfo[] = { NULL, NULL, NULL };	/* pointer to
								   InterFb_BufInfo
								   for buffer 0-2 */

#if USE_INT_ROT
#ifdef SHARE_INT_ROT
/* IRQ handler */
static char *id = "qfb";
#endif
#endif				/* USE_INT_ROT */

/* Timer */
static struct timer_list qfb_timer;
static int jiffies_timerentry;

#ifndef TIMER_TABLE
static int cycle = TIMER_CYCLE;
#else				/* TIMER_TABLE */
typedef struct {
	int tablesize;		/* count of table - 0 origin */
	int adjusttime;		/* fb will adjust next timer, */
	/* DSP can reset adjust to clear this field. */
	int time[3];		/* timer tables(10mS) */
} interFb_timetable;

/* loccal version of interFb_timetable */
static interFb_timetable localFb_timetable = {
	2, 0, {7, 7, 6}
/*      int             tablesize=2;            count of table - 0 origin */
/*      int             adjusttime=0;           fb will adjust next timer */
/*      int             time[3]={7,7,6};        timer tables(10mS)        */
};

static interFb_timetable *pinterFb_timetable = &localFb_timetable;	/* pointer
									   to interFb_timetable */

/* QFB_XV_CUE immediate parameters(10mS) */
#define DEFAULT_SYNC_TIME   20
#define DEFAULT_MARGIN_TIME 2
static int sync_time = DEFAULT_SYNC_TIME;	/* QFB_XV_CUE to start to video. */
static int margin_time = DEFAULT_MARGIN_TIME;	/* timer-IRQ but hasn't
						   ROT-IRQ,then wait more
						   margin_time */

/* timetable indexes */
static int time_no = 0;		/* timer table index */
static int larger_time = 7;	/* if time[time_no]=larger_time,then make balance 10mS */
static bool adjustflag = 0;	/* if TRUE, waiting mode margin_time now. */
static int timeout_count = 0;	/* 040626 continuous time out(=DSP/ROT sleepin') -> enter over-load mode */
static int timeskip;
int ROT_readingframe = -1;
static int ROT_writtenframe = -1;	/* 040626 writtenframe loaded at end of ROT */
static int flag_XserverRunning = 0;
static unsigned int time_lastXserverCheck = 0;

static void QFB_XV_CUE(const int psync_time, const int pmargin_time,
		       const interFb_timetable * pTimer_table);
static int get_timer_value(void);
#endif				/* TIMER_TABLE */

#if ENABLE_3D
/* GLX timetable */
GLX_timetable *pGLX_timetable = NULL;	/* 3D timer table */
static int GLX_time_no = 0;	/* 3D timer table index */
int enable_GLX_timetable = 0;	/* TRUE: use GLX_timetable FALSE:not
				   use GLX_timetable */
#endif	/* ENABLE_3D */

/* sleep and wake up */
DECLARE_WAIT_QUEUE_HEAD(wait);
static int sleep_mode = 0;

#ifdef LOWER_ROT_SCREEN
unsigned short qfb_t_color;
#endif				/* LOWER_ROT_SCREEN */

static struct fb_var_screeninfo qfb_default __initdata = {
	.xres = 240,
	.yres = 320,
	.xres_virtual = 240,
	.yres_virtual = 320,
	.xoffset = 0,
	.yoffset = 0,
	.grayscale = 0,
	.bits_per_pixel = 16,
	.red = {11, 5, 0},
	.green = {5, 6, 0},
	.blue = {0, 5, 0},
	.transp = {0, 0, 0},
	.activate = 0,
	.height = -1,
	.width = -1,
	.pixclock = 166666,
	.left_margin = 0,
	.right_margin = 0,
	.upper_margin = 0,
	.lower_margin = 0,
	.hsync_len = 24,
	.vsync_len = 4,
	.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT,
	.vmode = FB_VMODE_NONINTERLACED,
};

static struct fb_fix_screeninfo qfb_fix __initdata = {
	.id = "MP200 FB",
	.type = FB_TYPE_PACKED_PIXELS,
	.visual = FB_VISUAL_TRUECOLOR,
	.xpanstep = 1,
	.ypanstep = 1,
	.ywrapstep = 1,
	.accel = FB_ACCEL_NONE,
};

static int qfb_enable __initdata = 0;	/* disabled by default */
MODULE_PARM(qfb_enable, "i");

    /*
     *  Interface used by the world
     */
int qfb_init(void);
int qfb_setup(char *);

static int qfb_open(struct fb_info *info, int user);
static int qfb_release(struct fb_info *info, int user);
static int qfb_check_var(struct fb_var_screeninfo *var, struct fb_info *info);
static int qfb_set_par(struct fb_info *info);
static int qfb_setcolreg(u_int regno, u_int red, u_int green, u_int blue,
			 u_int transp, struct fb_info *info);
static int qfb_pan_display(struct fb_var_screeninfo *var, struct fb_info *info);
static int qfb_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
		     unsigned long arg, struct fb_info *info);
static int qfb_mmap(struct fb_info *info, struct file *file,
		    struct vm_area_struct *vma);

static struct fb_ops qfb_ops = {
	.owner = THIS_MODULE,
	.fb_open = qfb_open,
	.fb_release = qfb_release,
	.fb_check_var = qfb_check_var,
	.fb_set_par = qfb_set_par,
	.fb_setcolreg = qfb_setcolreg,
	.fb_pan_display = qfb_pan_display,
	.fb_fillrect = cfb_fillrect,
	.fb_copyarea = cfb_copyarea,
	.fb_imageblit = cfb_imageblit,
	.fb_cursor = soft_cursor,
	.fb_ioctl = qfb_ioctl,
	.fb_mmap = qfb_mmap,
};

struct qfb_par {
	atomic_t use_count;
	u32 pseudo_palette[256];
};

    /*
     *  Internal routines
     */

#if USE_INT_ROT
/* ROT interrupt handler */
static irqreturn_t rot_irq_handler(int irq, void *dev_id, struct pt_regs *regs);
#endif	/* USE_INT_ROT */

#ifdef CONFIG_MP200_IMG
extern int mp200_img_init_module(void);
extern void mp200_img_cleanup_module(void);
/* IMG callback function */
void img_irq_handler(struct mp200_img_req *p, int flag);
extern void unexclude(void);
/* IMG DPM function */
extern int mp200_img_suspend(struct device *dev, u32 state, u32 level);
extern int mp200_img_resume(struct device *dev, u32 level);
#else				/* defined(CONFIG_MP200_IMG) */
/* IMG interrupt handler */
static irqreturn_t img_irq_handler(int irq, void *dev_id, struct pt_regs *regs);
#endif				/* defined(CONFIG_MP200_IMG) */

/*SONY Coporation: Backlight brightness control*/
extern int backlight_set_brightness(unsigned int brightness);

/* Timeout handler */
static void qfb_timedout(unsigned long data);

#if MIX_IMAGE_WITHOUT_X
static void makeBuildScreenList(void);
#endif				/* MIX_IMAGE_WITHOUT_X */

extern bool init_hw(void);

#ifndef CONFIG_MP200_IMG
#if DPM_IMG_RESUME
extern bool init_img(void);
#endif				/* DPM_IMG_RESUME */
#endif				/* !defined(CONFIG_MP200_IMG) */

extern bool mix_image(int pram1);

static int qfb_open(struct fb_info *info, int user)
{
	struct qfb_par *par = (struct qfb_par *)info->par;

	atomic_inc(&par->use_count);

	return 0;
}

static int qfb_release(struct fb_info *info, int user)
{
	struct qfb_par *par = (struct qfb_par *)info->par;
	u32 count;

	count = atomic_read(&par->use_count);
	if (count == 0)
		return -EINVAL;

	if (count == 1) {
		flag_XserverRunning = 0;
	}

	atomic_dec(&par->use_count);

	return 0;
}

static u_long get_line_length(int xres_virtual, int bpp)
{
	u_long length;

	length = xres_virtual * bpp;
	length = (length + 31) & ~31;
	length >>= 3;
	return (length);
}

    /*
     *  Setting the video mode has been split into two parts.
     *  First part, xxxfb_check_var, must not write anything
     *  to hardware, it should only verify and adjust var.
     *  This means it doesn't alter par but it does use hardware
     *  data from it to check this var. 
     */

static int qfb_check_var(struct fb_var_screeninfo *var, struct fb_info *info)
{
	u_long line_length;

	/*
	 *  FB_VMODE_CONUPDATE and FB_VMODE_SMOOTH_XPAN are equal!
	 *  as FB_VMODE_SMOOTH_XPAN is only used internally
	 */

	if (var->vmode & FB_VMODE_CONUPDATE) {
		var->vmode |= FB_VMODE_YWRAP;
		var->xoffset = info->var.xoffset;
		var->yoffset = info->var.yoffset;
	}

	/*
	 *  Some very basic checks
	 */
	if (!var->xres)
		var->xres = 1;
	if (!var->yres)
		var->yres = 1;
	if (var->xres > var->xres_virtual)
		var->xres_virtual = var->xres;
	if (var->yres > var->yres_virtual)
		var->yres_virtual = var->yres;
	if (var->bits_per_pixel <= 1)
		var->bits_per_pixel = 1;
	else if (var->bits_per_pixel <= 8)
		var->bits_per_pixel = 8;
	else if (var->bits_per_pixel <= 16)
		var->bits_per_pixel = 16;
	else if (var->bits_per_pixel <= 24)
		var->bits_per_pixel = 24;
	else if (var->bits_per_pixel <= 32)
		var->bits_per_pixel = 32;
	else
		return -EINVAL;

	if (var->xres_virtual < var->xoffset + var->xres)
		var->xres_virtual = var->xoffset + var->xres;
	if (var->yres_virtual < var->yoffset + var->yres)
		var->yres_virtual = var->yoffset + var->yres;

	/*
	 *  Memory limit
	 */
	line_length = get_line_length(var->xres_virtual, var->bits_per_pixel);
	if (line_length * var->yres_virtual > DispBufLength)
		return -ENOMEM;

	/*
	 * Now that we checked it we alter var. The reason being is that the video
	 * mode passed in might not work but slight changes to it might make it 
	 * work. This way we let the user know what is acceptable.
	 */
	switch (var->bits_per_pixel) {
	case 1:
	case 8:
		var->red.offset = 0;
		var->red.length = 8;
		var->green.offset = 0;
		var->green.length = 8;
		var->blue.offset = 0;
		var->blue.length = 8;
		var->transp.offset = 0;
		var->transp.length = 0;
		break;
	case 16:		/* RGBA 5551 */
		if (var->transp.length) {
			var->red.offset = 0;
			var->red.length = 5;
			var->green.offset = 5;
			var->green.length = 5;
			var->blue.offset = 10;
			var->blue.length = 5;
			var->transp.offset = 15;
			var->transp.length = 1;
		} else {	/* RGB 565 */
			var->red.offset = 11;
			var->red.length = 5;
			var->green.offset = 5;
			var->green.length = 6;
			var->blue.offset = 0;
			var->blue.length = 5;
			var->transp.offset = 0;
			var->transp.length = 0;
		}
		break;
	case 24:		/* RGB 888 */
		var->red.offset = 0;
		var->red.length = 8;
		var->green.offset = 8;
		var->green.length = 8;
		var->blue.offset = 16;
		var->blue.length = 8;
		var->transp.offset = 0;
		var->transp.length = 0;
		break;
	case 32:		/* RGBA 8888 */
		var->red.offset = 0;
		var->red.length = 8;
		var->green.offset = 8;
		var->green.length = 8;
		var->blue.offset = 16;
		var->blue.length = 8;
		var->transp.offset = 24;
		var->transp.length = 8;
		break;
	}
	var->red.msb_right = 0;
	var->green.msb_right = 0;
	var->blue.msb_right = 0;
	var->transp.msb_right = 0;

	return 0;
}

/* This routine actually sets the video mode. It's in here where we
 * the hardware state info->par and fix which can be affected by the 
 * change in par. For this driver it doesn't do much. 
 */
static int qfb_set_par(struct fb_info *info)
{
	info->fix.line_length = get_line_length(info->var.xres_virtual,
						info->var.bits_per_pixel);
	return 0;
}

    /*
     *  Set a single color register. The values supplied are already
     *  rounded down to the hardware's capabilities (according to the
     *  entries in the var structure). Return != 0 for invalid regno.
     */

static int qfb_setcolreg(u_int regno, u_int red, u_int green, u_int blue,
			 u_int transp, struct fb_info *info)
{
	if (regno >= 256)	/* no. of hw registers */
		return 1;
	/*
	 * Program hardware... do anything you want with transp
	 */

	/* grayscale works only partially under directcolor */
	if (info->var.grayscale) {
		/* grayscale = 0.30*R + 0.59*G + 0.11*B */
		red = green = blue = (red * 77 + green * 151 + blue * 28) >> 8;
	}

	/* Directcolor:
	 *   var->{color}.offset contains start of bitfield
	 *   var->{color}.length contains length of bitfield
	 *   {hardwarespecific} contains width of RAMDAC
	 *   cmap[X] is programmed to (X << red.offset) | (X << green.offset) | (X << blue.offset)
	 *   RAMDAC[X] is programmed to (red, green, blue)
	 * 
	 * Pseudocolor:
	 *    uses offset = 0 && length = RAMDAC register width.
	 *    var->{color}.offset is 0
	 *    var->{color}.length contains widht of DAC
	 *    cmap is not used
	 *    RAMDAC[X] is programmed to (red, green, blue)
	 * Truecolor:
	 *    does not use DAC. Usually 3 are present.
	 *    var->{color}.offset contains start of bitfield
	 *    var->{color}.length contains length of bitfield
	 *    cmap is programmed to (red << red.offset) | (green << green.offset) |
	 *                      (blue << blue.offset) | (transp << transp.offset)
	 *    RAMDAC does not exist
	 */
#define CNVT_TOHW(val,width) ((((val)<<(width))+0x7FFF-(val))>>16)
	switch (info->fix.visual) {
	case FB_VISUAL_TRUECOLOR:
	case FB_VISUAL_PSEUDOCOLOR:
		red = CNVT_TOHW(red, info->var.red.length);
		green = CNVT_TOHW(green, info->var.green.length);
		blue = CNVT_TOHW(blue, info->var.blue.length);
		transp = CNVT_TOHW(transp, info->var.transp.length);
		break;
	case FB_VISUAL_DIRECTCOLOR:
		red = CNVT_TOHW(red, 8);	/* expect 8 bit DAC */
		green = CNVT_TOHW(green, 8);
		blue = CNVT_TOHW(blue, 8);
		/* hey, there is bug in transp handling... */
		transp = CNVT_TOHW(transp, 8);
		break;
	}
#undef CNVT_TOHW
	/* Truecolor has hardware independent palette */
	if (info->fix.visual == FB_VISUAL_TRUECOLOR) {
		u32 v;

		if (regno >= 16)
			return 1;

		v = (red << info->var.red.offset) |
		    (green << info->var.green.offset) |
		    (blue << info->var.blue.offset) |
		    (transp << info->var.transp.offset);
		switch (info->var.bits_per_pixel) {
		case 8:
			break;
		case 16:
			((u32 *) (info->pseudo_palette))[regno] = v;
			break;
		case 24:
		case 32:
			((u32 *) (info->pseudo_palette))[regno] = v;
			break;
		}
		return 0;
	}
	return 0;
}

    /*
     *  Pan or Wrap the Display
     *
     *  This call looks only at xoffset, yoffset and the FB_VMODE_YWRAP flag
     */

static int qfb_pan_display(struct fb_var_screeninfo *var, struct fb_info *info)
{
	if (var->vmode & FB_VMODE_YWRAP) {
		if (var->yoffset < 0
		    || var->yoffset >= info->var.yres_virtual || var->xoffset)
			return -EINVAL;
	} else {
		if (var->xoffset + var->xres > info->var.xres_virtual ||
		    var->yoffset + var->yres > info->var.yres_virtual)
			return -EINVAL;
	}
	info->var.xoffset = var->xoffset;
	info->var.yoffset = var->yoffset;
	if (var->vmode & FB_VMODE_YWRAP)
		info->var.vmode |= FB_VMODE_YWRAP;
	else
		info->var.vmode &= ~FB_VMODE_YWRAP;
	return 0;
}

    /*
     *  ioctl function 
     */

static int qfb_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
		     unsigned long arg, struct fb_info *info)
{
	switch (cmd) {
	case QFB_CHK_SCRN:
		time_lastXserverCheck = jiffies;
		if (sleep_mode) {
			return -EFAULT;
		} else {
			do {
				sleep_mode = 1;
				interruptible_sleep_on(&wait);
				sleep_mode = 0;
			} while (!override_BldScreenList);
		}
		break;
	case QFB_BLD_SCRN:
		time_lastXserverCheck = jiffies;
		{
#if ENABLE_3D
			int i;
			bool image3Dflag = 0;	/* 3D-image find flag */
#endif	/* ENABLE_3D */
#ifdef CONFIG_DPM
			/* suspend now ? */
			if (DPM_suspend_flg)
				return -EFAULT;
#endif	/* CONFIG_DPM */
#if ENABLE_3D
			if (pGLX_timetable != NULL) {
				for (i = 0; i <= max_BLD_SCRN_LIST + 1; i++) {
					if (pBldScreenList[i].transfer ==
					    BLD_SCRN_TYPE_ENDMARK)
						break;
					if (pBldScreenList[i].image ==
					    BLD_SCRN_TYPE_IMAGE_3D) {
						image3Dflag = 1;	/* 3D-image
									   is found
									 */
						break;
					}
				}
				if (image3Dflag) {
					enable_GLX_timetable = 1;	/* use
									   GLX_timetable */
				} else {
					enable_GLX_timetable = 0;	/* not use
									   GLX_timetable */
				}
			}
#endif	/* ENABLE_3D */
			override_BldScreenList = 0;
			*pXvImageUpdatedFlag = 0;
			if (!mix_image(0))
				return -EFAULT;
			break;
		}
	case QFB_SET_XV_ATTR:
		if (copy_from_user
		    (&XvAttribute, (Xv_Attr *) arg, sizeof(XvAttribute)))
			return -EFAULT;
		pinterFb_bufaddr = (interFb_bufaddr *)
		    (DSPSmemV + (XvAttribute.paddr_interFb_bufaddr) - DSPSmem);
		pInterFb_BufInfo[0] = (InterFb_BufInfo *)
		    (DSPSmemV + (pinterFb_bufaddr->interFb_buf0) - DSPSmem);
		pInterFb_BufInfo[1] = (InterFb_BufInfo *)
		    (DSPSmemV +
		     (pinterFb_bufaddr->interFb_buf0 +
		      pinterFb_bufaddr->interFb_buf1_Offset) - DSPSmem);
		pInterFb_BufInfo[2] = (InterFb_BufInfo *)
		    (DSPSmemV +
		     (pinterFb_bufaddr->interFb_buf0 +
		      pinterFb_bufaddr->interFb_buf2_Offset) - DSPSmem);
#if PRINT_ROT_PARAMS
		printk(KERN_INFO
		       "QFB_SET_XV_ATTR: pinterFb_bufaddr = 0x%x\n "
		       "pinterFb_bufinfo = 0x%x\n\n",
		       pinterFb_bufaddr, pInterFb_BufInfo[0]);
#endif
		break;
	case QFB_XVFLIPROT:
		if (*pXvImageOnScrnFlag) {	/* 20041013 Add */
			*pXvImageUpdatedFlag = 1;
			ROT_writtenframe = pinterFb_bufaddr->writtenframe;	/* 
										   040626 pickup writtenframe from DSP at
										   end of ROT */
			timeout_count = 0;	/* 040626 exit from over-load mode */
		} /* 20041013 Add */
		else {		/* 20041013 Add */
			printk(KERN_INFO "*** QFB_XVFLIPROT: Error! "
			       "*pXvImageOnScrnFlag is FALSE.\n");
		}
		break;
#if ENABLE_3D
	case QFB_SET_GLX_TIMETABLE:
		{
			GLX_timetable glx_timetable;	/* timer table pointer */
			int size_GLX_timetable;
			copy_from_user(&glx_timetable, (GLX_timetable *) arg,
				       sizeof(GLX_timetable));
			size_GLX_timetable =
			    sizeof(GLX_timetable) +
			    sizeof(int) * glx_timetable.tablesize;
			if (pGLX_timetable != NULL) {
				vfree(pGLX_timetable);
			}
			pGLX_timetable =
			    (GLX_timetable *) vmalloc(size_GLX_timetable);
			copy_from_user(pGLX_timetable, (GLX_timetable *) arg,
				       size_GLX_timetable);
			GLX_time_no = 0;
			break;
		}
	case QFB_DEL_GLX_TIMETABLE:
		enable_GLX_timetable = 0;	/* not use GLX_timetable */
		if (pGLX_timetable != NULL) {
			vfree(pGLX_timetable);
		}
		pGLX_timetable = NULL;	/* 3D timer table NULL */
		break;
#endif	/* ENABLE_3D */
	case QFB_SET_XV_COLORKEY:
		{
			Xv_colorkey xv_colorkey;
			copy_from_user(&xv_colorkey, (Xv_colorkey *) arg,
				       sizeof(Xv_colorkey));
#ifdef LOWER_ROT_SCREEN
			qfb_t_color = (unsigned short)xv_colorkey.t_color;
#endif				/* LOWER_ROT_SCREEN */
			break;
		}
	case QFB_GET_XV_COLORKEY:
		{
			Xv_colorkey xv_colorkey;
#ifdef LOWER_ROT_SCREEN
			xv_colorkey.t_color = (int)qfb_t_color;
#else
			xv_colorkey.t_color = (int)0x0020;
#endif				/* LOWER_ROT_SCREEN */
			copy_to_user((Xv_colorkey *) arg, &xv_colorkey,
				     sizeof(Xv_colorkey));
			break;
		}
    /*20060531 SONY Corporation: Set backlight brightness */
    case QFB_SET_BL_BRIGHTNESS:
        {
            unsigned int ret,brightness;
            copy_from_user(&brightness, (unsigned int*)arg, 
                           sizeof(unsigned int));
        
            ret = backlight_set_brightness(brightness);
            if (ret != 0)
                printk(KERN_ERR "*** QFB_SET_BL_BRIGHTNESS: Error! ");
            break;
        }
	default:
		return -EINVAL;
	}
	return 0;
}

    /*
     *  Most drivers don't need their own mmap function 
     */

static int qfb_mmap(struct fb_info *info, struct file *file,
		    struct vm_area_struct *vma)
{
	unsigned long start;
	unsigned long off;
	u32 len;

	if (vma->vm_pgoff > (~0UL >> PAGE_SHIFT))
		return -EINVAL;

	off = vma->vm_pgoff << PAGE_SHIFT;

	if (off < MAX_OFFSET) {
		/*
		 * offset
		 */

		/* Xserver/fb common memory (includes frame buffer memory) */
		start = (unsigned long)Smem;
		len = PAGE_ALIGN((start & ~PAGE_MASK) + SmemLength);

#if ENABLE_3D
		if (off >= len) {
			/* 3D common memory */
			off -= len;
			start = (unsigned long)GLXSmem;
			len = PAGE_ALIGN((start & ~PAGE_MASK) + GLXSmemLength);
		}
#endif	/* ENABLE_3D */

		if (off >= len) {
			/* DSP common memory */
			off -= len;
			start = (unsigned long)DSPSmem;
			len = PAGE_ALIGN((start & ~PAGE_MASK) + DSPSmemLength);
		}

#if ENABLE_3D
		if (off >= len) {
			/* GSHARK MMIO */
			off -= len;
			start = (unsigned long)Mmio;
			len = PAGE_ALIGN((start & ~PAGE_MASK) + MmioLength);
		}
#endif	/* ENABLE_3D */
	} else {
		/*
		 * physical address
		 */

		/* Xserver/fb common memory (includes frame buffer memory) */
		off -= Smem;
		if ((signed)off < 0)
			return -EINVAL;
		start = (unsigned long)Smem;
		len = PAGE_ALIGN((start & ~PAGE_MASK) + SmemLength);

#if ENABLE_3D
		if (off >= len) {
			/* 3D common memory */
			off = off + Smem - GLXSmem;
			if ((signed)off < 0)
				return -EINVAL;
			start = (unsigned long)GLXSmem;
			len = PAGE_ALIGN((start & ~PAGE_MASK) + GLXSmemLength);
		}
#endif	/* ENABLE_3D */

		if (off >= len) {
			/* DSP common memory */
#if ENABLE_3D
			off = off + GLXSmem - DSPSmem;
#else	/* ENABLE_3D */
			off = off + Smem - DSPSmem;
#endif	/* ENABLE_3D */
			if ((signed)off < 0)
				return -EINVAL;
			start = (unsigned long)DSPSmem;
			len = PAGE_ALIGN((start & ~PAGE_MASK) + DSPSmemLength);
		}

#if ENABLE_3D
		if (off >= len) {
			/* GSHARK MMIO */
			off = off + DSPSmem - Mmio;
			if ((signed)off < 0)
				return -EINVAL;
			start = (unsigned long)Mmio;
			len = PAGE_ALIGN((start & ~PAGE_MASK) + MmioLength);
		}
#endif	/* ENABLE_3D */
	}

	start &= PAGE_MASK;
	if ((vma->vm_end - vma->vm_start + off) > len)
		return -EINVAL;

	off += start;
	vma->vm_pgoff = off >> PAGE_SHIFT;

	/*
	 * Accessing memory will be done non-cached.
	 */
	vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);

	/* To stop the swapper from even considering these pages */
	vma->vm_flags |= (VM_IO | VM_RESERVED);

	if (io_remap_page_range
	    (vma, vma->vm_start, off, vma->vm_end - vma->vm_start,
	     vma->vm_page_prot))
		return -EAGAIN;

	return 0;
}

int __init qfb_setup(char *options)
{
	char *this_opt;

	qfb_enable = 1;

	if (!options || !*options)
		return 1;

	while ((this_opt = strsep(&options, ",")) != NULL) {
		if (!*this_opt)
			continue;
		if (!strncmp(this_opt, "disable", 7))
			qfb_enable = 0;
	}
	return 1;
}

    /*
     *  Initialisation
     */

static void qfb_platform_release(struct device *device)
{
	// This is called when the reference count goes to zero.
}

static int __init qfb_probe(struct device *device)
{
	struct platform_device *dev = to_platform_device(device);
	struct fb_info *info;
	struct qfb_par *par = NULL;
	int retval = -ENOMEM;

#ifdef CONFIG_MP200_IMG
	printk("<1> @fb: qfb_init use IMG driver version.\n");
	/* initialize IMG driver */
	if (mp200_img_init_module() < 0) {
		retval = -EBUSY;
		goto err;
	}
#else
	printk("<1> @fb: qfb_init normal version.\n");
#endif

	/* Smem */
	if (!request_mem_region(Smem, SmemLength, "qfb")) {
		printk(KERN_WARNING
		       "*** qfb_init: cannot reserve smem %ld KB @ 0x%lx\n",
		       SmemLength >> 10, Smem);
		retval = -EBUSY;
		goto err1;
	}
	SmemV = ioremap_nocache(Smem, SmemLength);
	if (!SmemV) {
		release_mem_region(Smem, SmemLength);
		printk(KERN_WARNING
		       "*** qfb_init: cannot ioremap_nocache smem %ld KB @ 0x%lx\n",
		       SmemLength >> 10, Smem);
		retval = -EBUSY;
		goto err1;
	}

	printk(KERN_INFO "qfb_init: smem address= 0x%lx 0x%p\n", Smem, SmemV);	/* for info. */

#if ENABLE_3D
	/* 3D Smem */
	if (!request_mem_region(GLXSmem, GLXSmemLength, "qfb")) {
		printk(KERN_WARNING
		       "*** qfb_init: cannot reserve 3D smem %ld KB @ 0x%lx\n",
		       GLXSmemLength >> 10, GLXSmem);
		retval = -EBUSY;
		goto err2;
	}
	GLXSmemV = ioremap_nocache(GLXSmem, GLXSmemLength);
	if (!GLXSmemV) {
		release_mem_region(GLXSmem, GLXSmemLength);
		printk(KERN_WARNING
		       "*** qfb_init: cannot ioremap_nocache 3D smem %ld KB @ 0x%lx\n",
		       GLXSmemLength >> 10, GLXSmem);
		retval = -EBUSY;
		goto err2;
	}

	/* Mmio */
	if (!request_mem_region(Mmio, MmioLength, "qfb")) {
		printk(KERN_WARNING
		       "*** qfb_init: cannot reserve mmio %ld KB @ 0x%lx\n",
		       MmioLength >> 10, Mmio);
		retval = -EBUSY;
		goto err3;
	}
	MmioV = ioremap_nocache(Mmio, MmioLength);
	if (!MmioV) {
		release_mem_region(Mmio, MmioLength);
		printk(KERN_WARNING
		       "*** qfb_init: cannot ioremap_nocache mmio %ld KB @ 0x%lx\n",
		       MmioLength >> 10, Mmio);
		retval = -EBUSY;
		goto err3;
	}
#endif	/* ENABLE_3D */

#if USE_INT_ROT
	/* ROT Mmio */
	if (!request_mem_region(ROTMmio, ROTMmioLength, "qfb")) {
		printk(KERN_WARNING
		       "*** qfb_init: cannot reserve ROT mmio %ld KB @ 0x%lx\n",
		       ROTMmioLength >> 10, ROTMmio);
		retval = -EBUSY;
		goto err4;
	}
	ROTMmioV = ioremap_nocache(ROTMmio, ROTMmioLength);
	if (!ROTMmioV) {
		release_mem_region(ROTMmio, ROTMmioLength);
		printk(KERN_WARNING
		       "*** qfb_init: cannot ioremap_nocache ROT mmio %ld KB @ 0x%lx\n",
		       ROTMmioLength >> 10, ROTMmio);
		retval = -EBUSY;
		goto err4;
	}
#endif	/* USE_INT_ROT */
#ifndef CONFIG_MP200_IMG
	/* IMG Mmio */
	if (!request_mem_region(IMGMmio, IMGMmioLength, "qfb")) {
		printk(KERN_WARNING
		       "*** qfb_init: cannot reserve IMG mmio %ld KB @ 0x%lx\n",
		       IMGMmioLength >> 10, IMGMmio);
		retval = -EBUSY;
		goto err5;
	}
	IMGMmioV = ioremap_nocache(IMGMmio, IMGMmioLength);
	if (!IMGMmioV) {
		release_mem_region(IMGMmio, IMGMmioLength);
		printk(KERN_WARNING
		       "*** qfb_init: cannot ioremap_nocache IMG mmio "
		       "%ld KB @ 0x%lx\n", IMGMmioLength >> 10, IMGMmio);
		retval = -EBUSY;
		goto err5;
	}
#endif				/* !defined(CONFIG_MP200_IMG) */

	/* LCDC Mmio */
	if (!request_mem_region(LCDCMmio, LCDCMmioLength, "qfb")) {
		printk(KERN_WARNING
		       "*** qfb_init: cannot reserve LCDC mmio %ld KB "
		       "@ 0x%lx\n", LCDCMmioLength >> 10, LCDCMmio);
		retval = -EBUSY;
		goto err6;
	}
	LCDCMmioV = ioremap_nocache(LCDCMmio, LCDCMmioLength);
	if (!LCDCMmioV) {
		release_mem_region(LCDCMmio, LCDCMmioLength);
		printk(KERN_WARNING
		       "*** qfb_init: cannot ioremap_nocache LCDC mmio "
		       "%ld KB @ 0x%lx\n", LCDCMmioLength >> 10, LCDCMmio);
		retval = -EBUSY;
		goto err6;
	}

	/* FIFO Mmio */
	if (!request_mem_region(FIFOMmio, FIFOMmioLength, "qfb")) {
		printk(KERN_WARNING
		       "*** qfb_init: cannot reserve FIFO mmio %ld KB "
		       "@ 0x%lx\n", FIFOMmioLength >> 10, FIFOMmio);
		retval = -EBUSY;
		goto err7;
	}
	FIFOMmioV = ioremap_nocache(FIFOMmio, FIFOMmioLength);
	if (!FIFOMmioV) {
		release_mem_region(FIFOMmio, FIFOMmioLength);
		printk(KERN_WARNING
		       "*** qfb_init: cannot ioremap_nocache FIFO mmio %ld KB "
		       "@ 0x%lx\n", FIFOMmioLength >> 10, FIFOMmio);
		retval = -EBUSY;
		goto err7;
	}

	/* DSP Smem for interFb_bufaddr and InterFb_BufInfo */
	if (!request_mem_region(DSPSmem, DSPSmemLength, "qfb")) {
		printk(KERN_WARNING
		       "*** qfb_init: cannot reserve DSP smem %ld KB "
		       "@ 0x%lx\n", DSPSmemLength >> 10, DSPSmem);
		retval = -EBUSY;
		goto err8;
	}
	DSPSmemV = ioremap_nocache(DSPSmem, DSPSmemLength);
	if (!DSPSmemV) {
		release_mem_region(DSPSmem, DSPSmemLength);
		printk(KERN_WARNING
		       "*** qfb_init: cannot ioremap_nocache DSP smem %ld KB "
		       "@ 0x%lx\n", DSPSmemLength >> 10, DSPSmem);
		retval = -EBUSY;
		goto err8;
	}

	DispBufV = SmemV + DISPBUF_OFFSET;	/* pointer to 2D framebuffer */

	/* clear 2D framebuffer */
	memset_io((char __iomem *)DispBufV, 0, DispBufLength);

	FrameBufAV = SmemV + FRAMEBUFA_OFFSET;	/* pointer to LCDC framebuffer
						   A */
	FrameBufBV = SmemV + FRAMEBUFB_OFFSET;	/* pointer to LCDC framebuffer
						   B */
#if ENABLE_3D
	pGLXWindowTable = (GLXWindowTable *)
	    (SmemV + GLX_WINDOW_TABLE_OFFSET);	/* pointer to
						   GLXWindowTable
						 */
#endif	/* ENABLE_3D */
	pBldScreenList = (BLD_SCRN_LIST *)
	    (SmemV + BLD_SCRN_LIST_OFFSET);	/* pointer to
						   BLD_SCRN_LIST */
	pXvImageUpdatedFlag = (int *)
	    (SmemV + XV_IMAGE_UPDATED_FLAG_OFFSET);	/* pointer to
							   Overlay
							   image is
							   updated (ROT
							   interrupt)
							   flag */
	*pXvImageUpdatedFlag = 0;
	pXvImageOnScrnFlag = (int *)
	    (SmemV + XV_IMAGE_ONSCRN_FLAG_OFFSET);	/* pointer to
							   Overlay
							   image is now
							   on screen
							   flag */
	*pXvImageOnScrnFlag = 0;

#ifdef TIMER_TABLE
/*
	inter_module_register("QFB_XV_CUE", THIS_MODULE, QFB_XV_CUE);
*/
#endif

	/* setup irq handlers */
#if USE_INT_ROT
	writel(RO_DMASTOPMASK_BIT | RO_DMAERRMASK_BIT | RO_ROTENDMASK_BIT,
	       ROTMmioV + RO_INTENCLR);
#ifdef SHARE_INT_ROT
	if (request_irq(INT_ROT, rot_irq_handler, SA_SHIRQ, "qfb", id) < 0)
#else
	if (request_irq(INT_ROT, rot_irq_handler, 0, "qfb", 0) < 0)
#endif
	{
		printk(KERN_ERR "qfb_init: fail in request_irq(INT_ROT)\n");
		retval = -EBUSY;
		goto err9;
	}
#ifdef ENABLE_INT_ROT
	/* enable */
	writel(RO_ROTENDEN_BIT | RO_DMAERREN_BIT | RO_DMASTOPEN_BIT,
	       ROTMmioV + RO_INTENSET);
#endif
#endif				/* USE_INT_ROT */
#ifndef CONFIG_MP200_IMG
	writel(IMG_ENCLRSTAT_BIT | IMG_ENCLRERROR_BIT | IMG_ENCLRSTOP_BIT,
	       IMGMmioV + IMG_ENCLR);
	if (request_irq(INT_IMG, img_irq_handler, 0, "qfb", 0) < 0) {
		printk(KERN_ERR "qfb_init: fail in request_irq(INT_IMG)\n");
		retval = -EBUSY;
		goto err10;
	}
#endif				/* !defined(CONFIG_MP200_IMG) */

	info = framebuffer_alloc(sizeof(struct qfb_par), &dev->dev);
	if (!info)
		goto err11;

	par = (struct qfb_par *)info->par;
	info->screen_base = (char __iomem *)DispBufV;
	info->screen_size = DispBufLength;
	info->fbops = &qfb_ops;

	retval = fb_find_mode(&info->var, info, NULL, NULL, 0, NULL, 8);

	if (!retval || (retval == 4))
		info->var = qfb_default;
	info->fix = qfb_fix;
	info->fix.line_length =
	    info->var.xres_virtual * info->var.bits_per_pixel / 8;
	info->fix.smem_start = DispBuf;
	info->fix.smem_len = DispBufLength;
	info->pseudo_palette = par->pseudo_palette;
	info->flags = FBINFO_FLAG_DEFAULT;

	retval = fb_alloc_cmap(&info->cmap, 256, 0);
	if (retval < 0)
		goto err12;

	retval = register_framebuffer(info);
	if (retval < 0)
		goto err13;
	dev_set_drvdata(&dev->dev, info);

	if (!init_hw()) {
		retval = -EBUSY;
		goto err14;
	}

	printk(KERN_INFO
	       "fb%d: MP200 frame buffer device, using %ldK of video memory @ 0x%lx\n",
	       info->node, DispBufLength >> 10, DispBuf);

#ifdef LOWER_ROT_SCREEN
	qfb_t_color = 0x0020;
#endif				/* LOWER_ROT_SCREEN */

	init_timer(&qfb_timer);
	qfb_timer.function = qfb_timedout;
	qfb_timer.data = 0;
#ifdef TIMER_TABLE
	qfb_timer.expires = jiffies + get_timer_value();
	printk(KERN_INFO "qfb_init: init %ld\n", qfb_timer.expires);
#else
	qfb_timer.expires = jiffies + cycle;
	printk(KERN_INFO "qfb_init: const %ld\n", qfb_timer.expires);
#endif
	add_timer(&qfb_timer);

	return 0;
      err14:
	unregister_framebuffer(info);
      err13:
	fb_dealloc_cmap(&info->cmap);
      err12:
	framebuffer_release(info);
      err11:
#ifndef CONFIG_MP200_IMG
	free_irq(INT_IMG, 0);
      err10:
#endif				/* !defined(CONFIG_MP200_IMG) */
#if USE_INT_ROT
#ifdef ENABLE_INT_ROT
	/* disable */
	writel(RO_ROTENDMASK_BIT | RO_DMAERRMASK_BIT | RO_DMASTOPMASK_BIT,
	       ROTMmioV + RO_INTENCLR);
#endif				/* ENABLE_INT_ROT */
#ifdef SHARE_INT_ROT
	free_irq(INT_ROT, id);
#else				/*SHARE_INT_ROT */
	free_irq(INT_ROT, 0);
#endif				/* SHARE_INT_ROT */
      err9:
#endif				/* USE_INT_ROT */
#ifdef TIMER_TABLE
/*
	inter_module_unregister("QFB_XV_CUE");
*/
#endif
	iounmap(DSPSmemV);
	release_mem_region(DSPSmem, DSPSmemLength);
      err8:
	iounmap(FIFOMmioV);
	release_mem_region(FIFOMmio, FIFOMmioLength);
      err7:
	iounmap(LCDCMmioV);
	release_mem_region(LCDCMmio, LCDCMmioLength);
      err6:
#ifndef CONFIG_MP200_IMG
	iounmap(IMGMmioV);
	release_mem_region(IMGMmio, IMGMmioLength);
      err5:
#endif				/* !defined(CONFIG_MP200_IMG) */
#if USE_INT_ROT
	iounmap(ROTMmioV);
	release_mem_region(ROTMmio, ROTMmioLength);
      err4:
#endif	/* USE_INT_ROT */
#if ENABLE_3D
	iounmap(MmioV);
	release_mem_region(Mmio, MmioLength);
      err3:
	iounmap(GLXSmemV);
	release_mem_region(GLXSmem, GLXSmemLength);
      err2:
#endif	/* ENABLE_3D */
	iounmap(SmemV);
	release_mem_region(Smem, SmemLength);
      err1:
	mp200_img_cleanup_module();
      err:
	return retval;
}

static int qfb_remove(struct device *device)
{
	struct fb_info *info = dev_get_drvdata(device);
#ifdef CONFIG_MP201
	unsigned int lcd_write_data;
	unsigned int lcd_write_mask;
#endif				/* CONFIG_MP201 */

	del_timer(&qfb_timer);
#if ENABLE_3D
	if (pGLX_timetable != NULL) {
		vfree(pGLX_timetable);
	}
#endif	/* ENABLE_3D */

	/* power off LCD */
#ifdef CONFIG_MP201
	lcd_write_mask = (unsigned int)((1 << PWC_INT_LED_ON) | (1 << PWC_INT_LCD_OE));
	lcd_write_data = 0;
	gpiocore_pwc_write(GPIOCORE_PWC_IOOUT2, lcd_write_data, lcd_write_mask);
#else
	gpiocore_write(GPIOCORE_GP_O, GPIOCORE_GPIO_P5, 0);	/* BackLight */
	gpiocore_write(GPIOCORE_GP_O, GPIOCORE_GPIO_P3, 0);	/* Reset */
	gpiocore_write(GPIOCORE_GP_O, GPIOCORE_GPIO_P4, 0);	/* Power */
#endif				/* CONFIG_MP201 */

	if (info) {
		unregister_framebuffer(info);
		framebuffer_release(info);
	}
#ifndef CONFIG_MP200_IMG
	free_irq(INT_IMG, 0);
#endif				/* !defined(CONFIG_MP200_IMG) */
#if USE_INT_ROT
#ifdef ENABLE_INT_ROT
	/* disable */
	writel(RO_ROTENDMASK_BIT | RO_DMAERRMASK_BIT | RO_DMASTOPMASK_BIT,
	       ROTMmioV + RO_INTENCLR);
#endif				/*ENABLE_INT_ROT */
#ifdef SHARE_INT_ROT
	free_irq(INT_ROT, id);
#else				/* SHARE_INT_ROT */
	free_irq(INT_ROT, 0);
#endif				/* SHARE_INT_ROT */
#endif				/* USE_INT_ROT */
#ifdef TIMER_TABLE
/*
	inter_module_unregister("QFB_XV_CUE");
*/
#endif				/* TIMER_TABLE */
	iounmap(DSPSmemV);
	release_mem_region(DSPSmem, DSPSmemLength);
	iounmap(FIFOMmioV);
	release_mem_region(FIFOMmio, FIFOMmioLength);
	iounmap(LCDCMmioV);
	release_mem_region(LCDCMmio, LCDCMmioLength);
#ifndef CONFIG_MP200_IMG
	iounmap(IMGMmioV);
	release_mem_region(IMGMmio, IMGMmioLength);
#endif				/* !defined(CONFIG_MP200_IMG) */
#if USE_INT_ROT
	iounmap(ROTMmioV);
	release_mem_region(ROTMmio, ROTMmioLength);
#endif	/* USE_INT_ROT */
#if ENABLE_3D
	iounmap(MmioV);
	release_mem_region(Mmio, MmioLength);
	iounmap(GLXSmemV);
	release_mem_region(GLXSmem, GLXSmemLength);
#endif	/* ENABLE_3D */
	iounmap(SmemV);
	release_mem_region(Smem, SmemLength);

#if UNRESET_LCD
#ifndef CONFIG_MP200_IMG
	/* Reset IMG */
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_IMG);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_IMGPCLK);
	mp200_pmu_reset_device(MP200_RESETDEVICE_IMG);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_IMG_CLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_IMG_PCLK);
#endif				/* !defined(CONFIG_MP200_IMG) */

	/* Reset LCD */
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCD);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCDPCLK);
#ifdef CONFIG_MP201
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_TFDPCLK);
	mp200_pmu_reset_device(MP200_RESETDEVICE_TFD);
#endif				/* CONFIG_MP201 */
	mp200_pmu_reset_device(MP200_RESETDEVICE_LCD);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_CLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_PCLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_LCLK);
#ifndef CONFIG_MP201
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_MCMADS_CLK);
#endif				/* CONFIG_MP201 */
#ifdef CONFIG_MP201
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_TFD_PCLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_TFD_LCLK);
#endif				/* CONFIG_MP201 */
#endif				/* UNRESET_LCD */

	return 0;
}

#ifdef CONFIG_DPM
static struct device_driver qfb_driver = {
	.name = "mp200_fb",
	.bus = &platform_bus_type,
	.probe = qfb_probe,
	.suspend = qfb_suspend,
	.resume = qfb_resume,
	.remove = qfb_remove,
};
#else	/* CONFIG_DPM */
static struct device_driver qfb_driver = {
	.name = "mp200_fb",
	.bus = &platform_bus_type,
	.probe = qfb_probe,
	.remove = qfb_remove,
};
#endif	/* CONFIG_DPM */

static struct platform_device qfb_device = {
	.name = "qfb",
	.id = 0,
	.dev = {
		.release = qfb_platform_release,
		}
};

int __init qfb_init(void)
{
	int ret = 0;

#ifndef MODULE
	char *option = NULL;

	if (fb_get_options("qfb", &option))
		return -ENODEV;
	qfb_setup(option);
#endif

	if (!qfb_enable)
		return -ENXIO;

	ret = driver_register(&qfb_driver);

	if (!ret) {
		ret = platform_device_register(&qfb_device);
		if (ret)
			driver_unregister(&qfb_driver);
	}
	return ret;
}

module_init(qfb_init);

#ifdef MODULE
static void __exit qfb_exit(void)
{
	platform_device_unregister(&qfb_device);
	driver_unregister(&qfb_driver);
}

module_exit(qfb_exit);

MODULE_LICENSE("GPL");
#endif				/* MODULE */

#if USE_INT_ROT
/*
 *  ROT interrupt handler
 */
static irqreturn_t rot_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
{
	u32 val;

	val = readl(ROTMmioV + RO_INTSTATUS);
	if (val & RO_DMAERR_BIT) {
		writel(RO_DMAERRCLR_BIT, ROTMmioV + RO_INTFFCLR);
		printk(KERN_INFO "*** rot_irq_handler: ROTATOR AHB ERROR.\n");
	} else if (val & RO_DMASTOP_BIT) {
		writel(RO_DMASTOPCLR_BIT, ROTMmioV + RO_INTFFCLR);
		printk(KERN_INFO
		       "*** rot_irq_handler: ROTATOR AHB RETRY/SPLIT.\n");
	} else if (val & RO_ROTEND_BIT) {
		writel(RO_ROTENDCLR_BIT, ROTMmioV + RO_INTFFCLR);
		if (*pXvImageOnScrnFlag) {
			*pXvImageUpdatedFlag = 1;
			ROT_writtenframe = pinterFb_bufaddr->writtenframe;	/* 040626
										   pickup
										   writtenframe
										   from DSP at
										   end of ROT
										 */
			timeout_count = 0;	/* 040626 exit from over-load mode */
		} else {
			printk(KERN_INFO "*** rot_irq_handler: Error! "
			       "*pXvImageOnScrnFlag is FALSE.\n");
		}

/* Timer_table adjust - mix_image should be placed here */

	}
	return IRQ_HANDLED;
}
#endif	/* USE_INT_ROT */

    /*
     *  IMG interrupt handler
     */
#ifdef CONFIG_MP200_IMG
void img_irq_handler(struct mp200_img_req *p, int flag)
{
	if (flag >= IMG_CALLBACK_SUCCESS) {
		unexclude();
		mix_image(1);
	} else {
		index_BldScreenList = 0;
		override_BldScreenList = 1;
	}
}
#else				/* defined(CONFIG_MP200_IMG) */
static irqreturn_t img_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
{
	mix_image(1);
	return IRQ_HANDLED;
}
#endif				/* defined(CONFIG_MP200_IMG) */

#ifdef TIMER_TABLE

/*
 * external function to video CUE
 */
static void QFB_XV_CUE(const int psync_time, const int pmargin_time,
		       const interFb_timetable * pTimer_table)
{

	/* check pinterFb_bufaddr */
	if (pinterFb_bufaddr == NULL) {
		printk(KERN_INFO
		       "***QFB_XV_CUE : Error! pinterFb_bufaddr is NULL.\n");
		return;
	}

	if ((pTimer_table == NULL) || (psync_time <= 0) || (pmargin_time <= 0)) {
		/* fb built-in timertable */
		pinterFb_timetable = &localFb_timetable;
		sync_time = DEFAULT_SYNC_TIME;
		margin_time = DEFAULT_MARGIN_TIME;
	} else {
		pinterFb_timetable = (interFb_timetable *) pTimer_table;
		sync_time = psync_time;
		margin_time = pmargin_time;
		larger_time = pinterFb_timetable->time[0];	/* 040626
								   larger_time
								   initialize */
	}
	pinterFb_timetable->adjusttime = 0;
	time_no = 0;
	del_timer(&qfb_timer);	/* cansel current timer */
	if (!(*pXvImageOnScrnFlag)) {	/* 040826 add */
		*pXvImageOnScrnFlag = 1;
		ROT_writtenframe = -1;	/* 040626 no writtenframe from DSP/ROT */
		ROT_readingframe = -1;	/* 040626 no readingframe from DSP/ROT */
	}
	timeout_count = 0;

	/* 040626 adjust IMG's working time(12mS), and estimate VSYNC
	 * waiting(8mS). */
	qfb_timer.expires = jiffies + sync_time - 2;
	add_timer(&qfb_timer);	/* restart after sync_time */
}

EXPORT_SYMBOL(QFB_XV_CUE);

/*
 *  timer value adjustment
 */
static int get_timer_value(void)
{
	int retvalue;
	if (*pXvImageOnScrnFlag) {
		retvalue = pinterFb_timetable->time[time_no];
		if (larger_time < retvalue) {
			larger_time = retvalue;
		}		/* 040626 update actual largest time */
		if ((pinterFb_timetable->adjusttime > 0)
		    && (retvalue >= larger_time)) {
			retvalue--;
			pinterFb_timetable->adjusttime--;
		}
		time_no++;
		if (time_no > pinterFb_timetable->tablesize) {
			time_no = 0;
		}
#if ENABLE_3D
	} else if ((enable_GLX_timetable) && (pGLX_timetable != NULL)) {
		retvalue = pGLX_timetable->time[GLX_time_no];
		GLX_time_no++;
		if (GLX_time_no > pGLX_timetable->tablesize) {
			GLX_time_no = 0;
		}
#endif	/* ENABLE_3D */
	} else {
		retvalue = TIMER_CYCLE;
	}
	return retvalue;
}
#endif

    /*
     * Timeout handler
     */
static void qfb_timedout(unsigned long data)
{
	int timetemp;		/* 040626 temporary of get_timer_value() */

	jiffies_timerentry = jiffies;	/* snapshot of current time */
#ifdef TIMER_TABLE
	if (*pXvImageOnScrnFlag) {	/* DSP-movie frame */
		if (!adjustflag) {	/* not margin_time out */
			if (*pXvImageUpdatedFlag) {	/* ROT IRQ after last
							   screen */
				if (ROT_readingframe != ROT_writtenframe) {
					/* next frames are written/ready */
					if (ROT_readingframe >=
					    pinterFb_bufaddr->framecount) {
						/* take next frame */
						ROT_readingframe = 0;
					} else {
						ROT_readingframe++;
					}
				}
				timetemp = get_timer_value();	/* next frame
								   timer */
				qfb_timer.expires =
				    jiffies_timerentry + timetemp;
				/* next timer = 1 frame */
			} else {	/* no ROT IRQ -> wait more? */
				if (ROT_readingframe == ROT_writtenframe) {
					/* no more frames */
					if (timeout_count < 2) {	/* not yet
									   over-load
									   mode */
						qfb_timer.expires = jiffies_timerentry + margin_time;	/* wait
													   more
													   margin_time
													 */
						adjustflag = 1;
						pinterFb_timetable->
						    adjusttime += margin_time;
						add_timer(&qfb_timer);	/* wait 
									   more margin_time only
									   once */
						return;	/* (Don't mix_image at
							   this time) */
					} else {	/* in over-load mode */
						timetemp = get_timer_value();
						/* next frame timer */
						qfb_timer.expires = jiffies_timerentry + timetemp;	/* next timer
													   = 1 frame
													 */
					}
				} else {	/* next frames are written/ready */
					if (ROT_readingframe >=
					    pinterFb_bufaddr->framecount) {
						ROT_readingframe = 0;
					} else {
						ROT_readingframe++;
					}
					timetemp = get_timer_value();
					qfb_timer.expires =
					    jiffies_timerentry + timetemp;
				}
			}
		} else {	/* adjustflag==TRUE -> after margin_time waiting */
			if (*pXvImageUpdatedFlag) {	/* ROT IRQ has come */
				if (ROT_readingframe != ROT_writtenframe) {
					/* next frames are written/ready */
					if (ROT_readingframe >=
					    pinterFb_bufaddr->framecount) {
						/* take next frame */
						ROT_readingframe = 0;
					} else {
						ROT_readingframe++;
					}
				}
				timetemp = get_timer_value();
				qfb_timer.expires =
				    jiffies_timerentry + timetemp;
			} else {	/* ROT IRQ has't come */
				if (ROT_readingframe == ROT_writtenframe) {
					/* no more frames */
					timeout_count++;	/* count up for
								   over-load mode */
					if (timeout_count < 2) {	/* not yet
									   over-load
									   mode */
						timeskip =
						    pinterFb_timetable->
						    time[time_no];
						timetemp = get_timer_value();
						/* next frame timer */
						qfb_timer.expires = jiffies_timerentry + timetemp;	/* next timer
													   = 1 frame
													 */
						pinterFb_timetable->adjusttime += timeskip;	/* time out
												   ->
												   adjust
												   more 1
												   frame
												 */
					} else {	/* in over-load mode */
						pinterFb_timetable->adjusttime -= timeskip;	/* time
												   out ->
												   adjust
												   more 1
												   frame
												 */
						timetemp = get_timer_value();
						/* next frame timer */
						qfb_timer.expires = jiffies_timerentry + timetemp;	/* next timer
													   = 1 frame
													 */
					}
				} else {	/* next frames are written/ready */
					if (ROT_readingframe >=
					    pinterFb_bufaddr->framecount) {
						/* take next frame */
						ROT_readingframe = 0;
					} else {
						ROT_readingframe++;
					}
					timetemp = get_timer_value();
					qfb_timer.expires =
					    jiffies_timerentry + timetemp;
				}
			}
			adjustflag = 0;
		}
		pinterFb_bufaddr->readingframe = ROT_readingframe;
	} else {		/* else{} means no DSP movie */
		timetemp = get_timer_value();	/* next frame timer */
		qfb_timer.expires = jiffies_timerentry + timetemp;	/* next timer = 1 frame */
	}

#endif				/* TIMER_TABLE */

	if (sleep_mode) {
		flag_XserverRunning = 1;
		time_lastXserverCheck = jiffies;
		wake_up_interruptible(&wait);
	}
#if MIX_IMAGE_WITHOUT_X
	else {
		if (flag_XserverRunning) {
			if ((jiffies - time_lastXserverCheck) > WAIT_TIME_X_CHK) {
				printk(KERN_INFO
				       "@fb: reset flag_XserverRunning to FALSE\n");
				flag_XserverRunning = 0;
				time_lastXserverCheck = 0;
			}
		} else {
			if (override_BldScreenList) {
				override_BldScreenList = 0;
				makeBuildScreenList();
				*pXvImageUpdatedFlag = 0;
				mix_image(0);
			}
		}
	}
#endif				/* MIX_IMAGE_WITHOUT_X */
#ifndef TIMER_TABLE
	qfb_timer.expires = jiffies_timerentry + cycle;
#endif				/* TIMER_TABLE */
	if ((signed)(qfb_timer.expires - jiffies) <= 0) {
		printk(KERN_INFO
		       "too long time from Timer to end of screen update\n");
		qfb_timer.expires = jiffies + TIMER_CYCLE;	/* recovery */
	}
	add_timer(&qfb_timer);
}

#if MIX_IMAGE_WITHOUT_X
static void makeBuildScreenList(void)
{
	/* 240 x 320 from 2D buffer */
	pBldScreenList[0].transfer = BLD_SCRN_TYPE_TRANSFER;
	pBldScreenList[0].exclude = 0;
	pBldScreenList[0].unexclude = 0;
	pBldScreenList[0].image = BLD_SCRN_TYPE_IMAGE_2D;
	pBldScreenList[0].src_x = 0;
	pBldScreenList[0].src_y = 0;
	pBldScreenList[0].width = 240;
	pBldScreenList[0].height = 320;
	pBldScreenList[0].dest_x = 0;
	pBldScreenList[0].dest_y = 0;
	pBldScreenList[0].index_GLXWindowTable = 0;

	/* endmrk */
	pBldScreenList[1].transfer = BLD_SCRN_TYPE_ENDMARK;

	return;
}
#endif				/* MIX_IMAGE_WITHOUT_X */

#ifdef CONFIG_DPM
static int qfb_suspend(struct device *dev, u32 state, u32 level)
{
#if DPM_LCD_SUSPEND
#ifdef CONFIG_MP201
	unsigned int lcd_write_data;
	unsigned int lcd_write_mask;
#endif				/* CONFIG_MP201 */

    /* 2006.05.23 ish */
    printk(KERN_INFO "%s() (state=%d, level=%d) \n",
           __FUNCTION__, state, level);
    
#endif				/* DPM_LCD_SUSPEND */

	switch (level) {
	case SUSPEND_DISABLE:
		/* stop timer interrupt */
		del_timer(&qfb_timer);
		break;
	case SUSPEND_SAVE_STATE:
		break;
	case SUSPEND_POWER_DOWN:
#if USE_INT_ROT
		/* stop ROT interrupt */
		writel(RO_ROTENDMASK_BIT | RO_DMAERRMASK_BIT |
		       RO_DMASTOPMASK_BIT, ROTMmioV + RO_INTENCLR);
		writel(RO_ROTENDMASK_BIT | RO_DMAERRMASK_BIT |
		       RO_DMASTOPMASK_BIT, ROTMmioV + RO_INTFFCLR);
#endif	/* USE_INT_ROT */

#ifndef CONFIG_MP200_IMG
		/* stop IMG interrupt */
		writel(IMG_ENCLRSTAT_BIT | IMG_ENCLRERROR_BIT |
		       IMG_ENCLRSTOP_BIT, IMGMmioV + IMG_ENCLR);
		writel(IMG_FFCLRSTAT_BIT | IMG_FFCLRERROR_BIT |
		       IMG_FFCLRSTOP_BIT, IMGMmioV + IMG_FFCLR);

#if UNRESET_LCD
		/* Reset IMG */
		mp200_pmu_clkctrl_off(MP200_CLKCTRL_IMG);
		mp200_pmu_clkctrl_off(MP200_CLKCTRL_IMGPCLK);
		mp200_pmu_reset_device(MP200_RESETDEVICE_IMG);
		mp200_pmu_close_clockgate(MP200_CLOCKGATE_IMG_CLK);
		mp200_pmu_close_clockgate(MP200_CLOCKGATE_IMG_PCLK);
#endif				/* UNRESET_LCD */
#endif				/* !defined(CONFIG_MP200_IMG) */

		/* stop LCD interrupt */

#if DPM_LCD_SUSPEND
		/* stop LCD disp */
		writel(0, LCDCMmioV + LCD_LCDOUT);
		while (readl(LCDCMmioV + LCD_STATUS) & 0x00000001 != 0) ;

		/* stop FIFO cache */
		/*
		   for FIFO definition
		   bit
		   7:4  Master ID            d:LCDC
		   3    Master ID Enable     0:Disable 1:Enable
		   2    frame cache B Enable 0:Disable 1:Enable(need bit1 = 1)
		   1    frame cache A Enable 0:Disable 1:Enable(need bit1 = 1)
		   0    frame cache Enable   0:Disable 1:Enable
		 */
		writel(0x000000d8, FIFOMmioV + FIF_FCCTRL);


#ifdef CONFIG_SONY_ICX1087 /* 2006.05.23 ish */

        /* Set pwc gpio 14,15 to output pin */
        gpiocore_pwc_write(GPIOCORE_PWC_IOSEL2, 0xC0, 0xC0);

        /* BackLight off : GPIO33=Low */
        mp200_gio_write_outputs_high32(0x00000002, 0x00000000);

        /* LCD PCI : PWC_GPIO14=Low */
        gpiocore_pwc_write(GPIOCORE_PWC_IOOUT2, 0x00, 0x40);

        /* LCD PCI : GPIO35=Low  (NOTE: controls LED) */
        mp200_gio_write_outputs_high32(0x00000008, 0x00000000);
        
        msleep(20);

        /* LCD RST : PWC_GPIO15=Low */
        gpiocore_pwc_write(GPIOCORE_PWC_IOOUT2, 0x00, 0x80);

        /* LCD RST : GPIO36=Low (NOTE: controls LED) */
        mp200_gio_write_outputs_high32(0x00000010, 0x00000000);
#else        

		/* power off LCD */
#ifdef CONFIG_MP201
		lcd_write_mask = (unsigned int)((1 << PWC_INT_LED_ON) | (1 << PWC_INT_LCD_OE));
		lcd_write_data = 0;
		gpiocore_pwc_write(GPIOCORE_PWC_IOOUT2, lcd_write_data, lcd_write_mask);
#else
		gpiocore_write(GPIOCORE_GP_O, GPIOCORE_GPIO_P5, 0);	/* BackLight */
		gpiocore_write(GPIOCORE_GP_O, GPIOCORE_GPIO_P3, 0);	/* Reset */
		gpiocore_write(GPIOCORE_GP_O, GPIOCORE_GPIO_P4, 0);	/* Power */
#endif				/* CONFIG_MP201 */

#endif  /* CONFIG_SONY_ICX1087 */
        
        
#endif				/* DPM_LCD_SUSPEND */

		DPM_suspend_flg = 1;

		break;
	default:
		break;
	}

#ifdef CONFIG_MP200_IMG
	mp200_img_suspend(dev, state, level);	/* call IMG suspend function */
#endif				/* CONFIG_MP200_IMG */

	return 0;
}

static int qfb_resume(struct device *dev, u32 level)
{

#ifdef CONFIG_MP200_IMG
	mp200_img_resume(dev, level);	/* call IMG resume function */
#endif				/* CONFIG_MP200_IMG */

    /* 2006.05.23 ish */
    printk(KERN_INFO "%s() (level=%d) \n",
           __FUNCTION__, level);

	switch (level) {
	case RESUME_POWER_ON:

#if DPM_LCD_RESUME
		/* initialize hardware */
		init_hw();
#endif				/* DPM_LCD_RESUME */
#ifndef CONFIG_MP200_IMG
#if DPM_IMG_RESUME
		/* initialize img */
		init_img();
#endif
#endif				/* !defined(CONFIG_MP200_IMG) */

		/* start ROT interrupt */
#if USE_INT_ROT
		writel(RO_DMASTOPMASK_BIT | RO_DMAERRMASK_BIT |
		       RO_ROTENDMASK_BIT, ROTMmioV + RO_INTENCLR);
#ifdef ENABLE_INT_ROT
		writel(RO_ROTENDEN_BIT | RO_DMAERREN_BIT | RO_DMASTOPEN_BIT,
		       ROTMmioV + RO_INTENSET);
#endif
#endif				/* USE_INT_ROT */

		break;
	case RESUME_RESTORE_STATE:
		break;
	case RESUME_ENABLE:
		/* start timer */
#ifdef TIMER_TABLE
		qfb_timer.expires = jiffies + get_timer_value();
#else
		qfb_timer.expires = jiffies + cycle;
#endif
		add_timer(&qfb_timer);
		DPM_suspend_flg = 0;
		break;
	default:
		break;
	}
	return 0;
}
#endif	/* CONFIG_DPM */
