/*
 * File Name       : /drivers/video/mp200_v4l2/mp200_common.h
 * Function        : Common parameters for LCD Driver
 * Release Version : 
 * Release Date    : 
 *
 * Copyright (C) NEC Electronics Corporation 2005
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef _MP200_COMMON_H_
#define _MP200_COMMON_H_


/********************************************************
 *                                                      *
 *  Include Files                                       *
 *                                                      *
 *******************************************************/

#include <asm-arm/arch-mp200/fbcommon.h>


/********************************************************
 *                                                      *
 *  Definitions                                         *
 *                                                      *
 *******************************************************/

/* image infomation from V4L2 */
//YUV format
#define V4L2_YUV422_FORMAT	0
#define V4L2_YUV420_FORMAT	1
//endian
#define V4L2_BIG_ENDIAN		0
#define V4L2_LITTLE_ENDIAN	1

/* image infomation from fb driver */
#define FB_MASK_COLOR_DISP_OFF	TC_COLOR_DISABLE
#define FB_MASK_COLOR_DISP_ON	TC_COLOR_ENABLE

// mix_image_mode
#define ONLY_2D_MODE		0
#define DSP_AND_2D_MODE		1

/* fb data update ON/OFF */
#define FB_UPDATE_OFF			0
#define FB_UPDATE_ON			1
#define FB_ABSOLUTERY_UPDATE		2


/********************************************************
 *                                                      *
 *  Structure                                           *
 *                                                      *
 *******************************************************/

/* Image data (use I/F V4L2 -> LCD  or  fb -> LCD ) */
typedef struct _IMAGE_DATA {
	unsigned int  size;		/* address addition  */
	unsigned long yrgbaddr;		/* Y plane address   */
	unsigned long uvaddr;		/* UV plane address  */
	unsigned int  x;		/* x start address   */
	unsigned int  y;		/* y start address   */
	unsigned int  hsize;		/* image data width  */
	unsigned int  vsize;		/* image data height */
} IMAGE_DATA;

typedef struct _FRAME_DATA {
	void *dev;			/* mp200_v4l2_device               */
	void *buf;			/* videobuf_buffer                 */
} FRAME_DATA;

/* Image data from V4L2   (use I/F V4L2 -> LCD) */
typedef struct _V4L2_IMAGE_INFO {
	IMAGE_DATA    src_image_data;	/* input image data from v4l2 */
	FRAME_DATA    src_frame_data; /* v4l2 data delivery */
	unsigned int  yuvfmt;		/* input image data format    */
	unsigned int  dst_image_hsize;	/* output data width to LCD   */
	unsigned int  dst_image_vsize;	/* output data height to LCD  */
	unsigned long offsetx;		/* offset x                   */
	unsigned long offsety;		/* offset y                   */
	unsigned int  endian;		/* endian type                */
} V4L2_IMAGE_INFO;

/* Image data from fb driver  (use I/F fb -> LCD) */
typedef struct _FB_IMAGE_INFO {
	IMAGE_DATA    src_image_data;		/* input image data from fb driver */
	unsigned int  maskcolr;			/* maskcolor                       */
	unsigned int  maskcolrflg;		/* maskcolor flag                  */
	unsigned int  alpha;			/* alpha                           */
	unsigned int  invflg;			/* inverse mode flag               */
	unsigned int  mix_buf_page;		/* mix 2D buffer frame page        */
	unsigned int  update_flag;		/* update flag ON/Off              */
} FB_IMAGE_INFO;

#endif /* _MP200_COMMON_H_ */
