/*
 * File Name       : /drivers/video/mp200_v4l2/mp200_fb.c
 * Function        : QVGA FraemBuffer Driver corresponding to V4L2
 * Release Version :
 * Release Date    :
 *
 * Copyright (C) NEC Electronics Corporation 2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

/*
 *
 *  framebuffer driver for D77601 -- based on vfb.c
 *
 */

/*
 *  linux/drivers/video/vfb.c -- Virtual frame buffer device
 *
 *      Copyright (C) 2002 James Simmons
 *
 *	Copyright (C) 1997 Geert Uytterhoeven
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of this archive for
 *  more details.
 */

/*
 * Copyright 2006,2007 Sony Corporation. 
 */


/********************************************************
 *                                                      *
 *  Include Files                                       *
 *                                                      *
 *******************************************************/

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/tty.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <linux/timer.h>
#include <linux/sched.h>

#include <asm/uaccess.h>
#include <asm/arch/hardware.h>
#include <asm/arch/gpiocore.h>
#include <asm/arch/irqs.h>
#include <asm/arch/img.h>
#include <asm/arch/mp201_mem.h>
#include <asm/arch/fbcommon.h>

#ifdef CONFIG_SONY_ICX1087
#include <asm/arch/icx1087_backlight.h>
#include <asm/arch/pmu.h>
#include <asm/arch/pm.h>
#include <linux/spinlock.h>
#endif

#include "mp200_fb.h"
#include "mp200_common.h"

#ifdef CONFIG_SONY_ICX1087           /*2006.08.21 ita*/
#define QFB_DEFAULT_BRIGHT 94        /*2006.11.27 ita: same setting in mp200_lcd.c*/

#define QFB_MAX_BRIGHTNESS  100
#define QFB_MIN_BRIGHTNESS  0

#define QFB_GRADUAL_STEP 15          /*numbers of setting brightness */
#define QFB_BL_CTRL_DELAY 10         /*msec*/
#define QFB_BL_CTRL_DELAY_IN_SEQ 60  /*msec*/

#define QFB_BACKLIGHT_MIN 0
#define QFB_BACKLIGHT_MAX 5

/*value of backlight*/
static int backlight_table[] ={
    0,     /*0*/
    38,    /*1*/
    59,    /*2*/
    74,    /*3*/
    84,    /*4*/
    94     /*5*/
};


#endif
/********************************************************
 *                                                      *
 *  Variables                                           *
 *                                                      *
 *******************************************************/

// image infomation to LCD
        FB_IMAGE_INFO         fb_image_info;

// frame buffer A
static  FB_OTHER_IMAGE_INFO   fb_other_info_A;

// frame buffer B
static  FB_OTHER_IMAGE_INFO   fb_other_info_B;

// LCD backlight brightness   
#ifdef CONFIG_SONY_ICX1087
static  int                   iUsbSusMode = 0;
static  int                   iBkFlag = 0;
static  int                   iBkIndex = 5;
static  int                   iBkPeriod = 10;
static  int                   iBkStep = 1;
static  int                   iBkBrightness = QFB_DEFAULT_BRIGHT;  /*The Goal to go for FB.When iLcdDisable=1, 
                                                                     this is different from stBlSetting*/
static  int                   iGoalBrightness = QFB_DEFAULT_BRIGHT;   /*The Goal at one timer func ago*/
static  int                   iCurBkBrightness = QFB_DEFAULT_BRIGHT;  /*Current Brightness*/
static  backlight_setting     stBlSetting;                        /*Brightness set by Application via ioctl*/
static  int                   iStep2Goal[QFB_MAX_BRIGHTNESS];
static  int                   iCounter2Goal;
static  struct timer_list     bl_timer;
spinlock_t		bl_lock;
static int iFlgTimerOn = 0;
static int iBkOffSeq = 0;
static int iBkOnSeq = 0;
static int iLcdDisable=0;
static int qfb_init_module_called=0;
#endif /*CONFIG_SONY_ICX1087*/
// Smem
ulong   Smem       = SMEM_START;
ulong   SmemLength = SMEM_LENGTH;
char   *SmemV;

// Disp Buffer (2D)
        ulong   DispBufA        = SMEM_START + DISPBUF_A_OFFSET;
        ulong   DispBufB        = SMEM_START + DISPBUF_B_OFFSET;
static  ulong   DispBufLength   = DISPBUF_LENGTH;
        char   *DispBufBV;
        char   *DispBufAV;

// Timer
static  struct timer_list qfb_timer;
static  int               iTimeoutTime;
static         qfb_par    qfb_dev;
// flags
static  int               iFBStatus_A;
static  int               iFBStatus_B;
static  int               iNowDispPage;   /* now on display page */
static  int               iNextDispPage;  /* next display page   */
static  int               iMixModeFlg;
static  int               iUpdateTimerFlag;
static  int               iTimerStatusFlag;

extern  char  *FrameBufAV;  /* added by ita 2006.08.11*/
extern  char  *FrameBufBV;  /* added by ita 2006.08.11*/

/********************************************************
 *                                                      *
 *  Prototype declarations of function                  *
 *                                                      *
 *******************************************************/

// public function
int           qfb_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg, struct fb_info *info);
int           qfb_mmap(struct fb_info *info, struct file *file, struct vm_area_struct *vma);
void          qfb_callback(int iWrittenFBNo, int iMixModeStatus);
int   __init  qfb_init_module(void);

// private function
static  int            set_image_info_to_2DAPI(int iFBNo, scrn_modes *fb_scrn_modes, struct fb_info *info);
static  int            chk_fb_status(int iFBStatus);
static  int            chk_input_modes(scrn_modes fb_scrn_modes);
static  int            set_image_info_from_2DAPI(int iFBNo, scrn_modes *fb_scrn_modes, struct fb_info *info);
static  int            set_image_info_to_LCD(int iFBNo, struct fb_info *info);
static  int            chk_input_update_timer(update_timer fb_update_timer);
static  int            open(struct fb_info *info, int user);
static  int            release(struct fb_info *info, int user);
static  u_long         get_line_length(int xres_virtual, int bpp);
static  int            check_var(struct fb_var_screeninfo *var, struct fb_info *info);
static  int            set_par(struct fb_info *info);
static  int            setcolreg(u_int regno, u_int red, u_int green, u_int blue, u_int transp, struct fb_info *info);
static  int            pan_display(struct fb_var_screeninfo *var, struct fb_info *info);
static  int    __init  probe(struct device *device);
static  void           set_buffer_address(void);
static  void           set_info(struct fb_info *info, qfb_par **par);
static  void   __init  probe_error(int iErrorMode, struct fb_info *info);
static  void           set_buffer_address(void);
static  void           set_flag_init(void);
static  void           set_val_init(void);
static  void           timer_init(struct fb_info *info);
static  void           timeout_update( unsigned long data );
static  int            remove(struct device *device);
#ifdef CONFIG_DPM
static  int            qfb_suspend(struct device *dev, u32 state, u32 level);
static  int            qfb_resume(struct device *dev, u32 level);

#ifdef CONFIG_SONY_ICX1087
static  void            qfb_shutdown(struct device *dev);

int      qfb_prog_init(void);
int      qfb_prog_set_degree(int deg);
int      qfb_prog_get_degree(void);

int      qfb_set_lcd_disable(int flag);
int      qfb_get_lcd_disable(int *bl_bright);
#endif

#endif /* CONFIG_DPM */
// workqueue function
static void            timeout_update_bottom_half_do(void *num);


// extern LCD public function
extern  int   mp200_lcd_init_module(void);
extern  int   mp200_lcd_set_fb_image_data(FB_IMAGE_INFO *fb_image_info);
extern  void  mp200_lcd_remove(void);
extern int resume_lcdhw(int brightness);  /* 2006.08.02 ita*/
extern int suspend_lcdhw(int reg_log);  /* 2006.08.07 ita*/
extern void lcdhw_bl_off_seq(int stage);
extern int lcdhw_bl_on_seq(int brightness,int stage);

// extern IMG public function
extern  int   mp200_img_init_module(void);
#ifdef CONFIG_DPM
extern  int   mp200_lcd_suspend(struct device *dev, u32 state, u32 level);
extern  int   mp200_lcd_resume(struct device *dev, u32 level);
extern  int   mp200_img_suspend(struct device *dev, u32 state, u32 level);
extern  int   mp200_img_resume(struct device *dev, u32 level);
#endif /* CONFIG_DPM */


/********************************************************
 *                                                      *
 *  Function Structure                                  *
 *                                                      *
 *******************************************************/

// qfb ops
static struct fb_ops qfb_ops = {
	.owner          = THIS_MODULE,
	.fb_open        = open,
	.fb_release     = release,
	.fb_check_var   = check_var,
	.fb_set_par     = set_par,
	.fb_setcolreg   = setcolreg,
	.fb_pan_display = pan_display,
	.fb_fillrect    = cfb_fillrect,
	.fb_copyarea    = cfb_copyarea,
	.fb_imageblit   = cfb_imageblit,
	.fb_cursor      = soft_cursor,
	.fb_ioctl       = qfb_ioctl,
	.fb_mmap        = qfb_mmap,
};

// qfb driver
static struct device_driver qfb_driver = {
	.name = "mp200_fb",
	.bus = &platform_bus_type,
	.probe = probe,
#ifdef CONFIG_DPM
	.suspend = qfb_suspend,
	.resume = qfb_resume,

#ifdef CONFIG_SONY_ICX1087
    .shutdown = qfb_shutdown,
#endif
    
#endif	/* CONFIG_DPM */
	.remove = remove,
};

#ifdef CONFIG_SONY_ICX1087
/******************************************************************************
* FUNCTION : set the brightness to H/W. when setting brightness=0 or starting
*            from brightness=0, execute the stop or start sequence of LCD.
* RETURN   : 0...success
* NOTE     : 
* CREATE   : 2006.08.21 ita
* UPDATE   :                                                                   
******************************************************************************/
static int set_brightness(int brightness)
{
    unsigned int ret;

    if(brightness == 0){           /*when brightness=0,execute stop sequence */
        if(iCurBkBrightness !=0){
            if(iBkOffSeq == 0)                /* stop timer interrupt */
                del_timer(&qfb_timer);
            lcdhw_bl_off_seq(iBkOffSeq++);    /* execute lcd off sequence*/
            if(iBkOffSeq > 2){                /* when complete lcd off sequence*/
                iBkOffSeq = 0;
                iCurBkBrightness = brightness; 
            }
        }
    }else if(iCurBkBrightness == 0){ /*when start from brightness=0,execute start sequence */
        if(lcdhw_bl_on_seq(brightness,iBkOnSeq++) != 0){  /*execute lcd on sequence*/
            printk(KERN_ERR "*** FB Error in lcdhw_bl_on_seq()\n");
            return -1;
        }
        if(iBkOnSeq > 3){                   /* when complete lcd on sequence*/
            if((iUpdateTimerFlag == TIMER_ON) && (iMixModeFlg == ONLY_2D_MODE))
                add_timer(&qfb_timer);/* start timer interrupt */
            iBkOnSeq = 0;
            iCurBkBrightness = brightness;
        }
    }else{                                              /* when normal setting */
        ret = icx1087_set_bl_brightness(brightness);    /* set brightness to H/W.*/
        if (ret != 0) {
            printk(KERN_ERR "*** QFB_SET_BL_BRIGHTNESS: Error in icx1087_set_bl_brightness() ");
            return -1;
        }
        iCurBkBrightness = brightness;
    }
    return 0;
}

/******************************************************************************
* FUNCTION : calculate one step of brightness to set H/W.
*            iStep2Goal is the value which increase/decrease per one timer_func.
* RETURN   : none
* NOTE     : 
* CREATE   : 2006.08.21 ita
* UPDATE   :                                                                   
******************************************************************************/
static void calcStep2Goal(int step)
{
    int i;
    int sa,delta,moj;

    iCounter2Goal = 0;        /* when decide the steps, reset counter */

    sa = iCurBkBrightness - iGoalBrightness;
    delta = sa / step;
    moj = sa % step;

    for(i=0;i<step;i++){ 
        if(moj > 0){
            iStep2Goal[i] = delta + 1;
            moj--;
        }else if(moj < 0){
            iStep2Goal[i] = delta - 1;
            moj++;
        }else{
            iStep2Goal[i] = delta;
        }
    }
}

/******************************************************************************
* FUNCTION : calculate the brightness to set H/W.
* RETURN   : value of brightness to set H/W
* NOTE     : 
* CREATE   : 2006.08.21 ita
* UPDATE   :                                                                   
******************************************************************************/
static int calcBrightness2Set(void)
{
    int bl;

    bl = iCurBkBrightness - iStep2Goal[iCounter2Goal];

    if(((iStep2Goal[iCounter2Goal] < 0) && (bl > iGoalBrightness))
       || ((iStep2Goal[iCounter2Goal] > 0) && (bl < iGoalBrightness))){/*when exceed goal*/
        bl = iGoalBrightness;
    }
    iCounter2Goal++;
    return bl;
}

/******************************************************************************
* FUNCTION : timer function of backlight control 
*            when backlight brightness must be gradually changed.
* RETURN   : none                                                              
* NOTE     : 
* CREATE   : 2006.08.18 ita
* UPDATE   :                                                                   
******************************************************************************/
static void backlight_timer_func(unsigned long data)
{
    int bk,step,period;
    int set_br;

    spin_lock_bh(&bl_lock);
    bk = iBkBrightness;
    step = iBkStep;
    period = iBkPeriod;
    spin_unlock_bh(&bl_lock);

    if((iGoalBrightness != bk)  /*when setting was changed in ioctl(QFB_SET_BL_BRIGHTNESS:)*/
       && (iBkOffSeq == 0)      /*when not in the backlight off sequence*/
       &&(iBkOnSeq == 0)){      /*when not in the backlight on sequence*/
        iGoalBrightness = bk;   /* set brightness to get to*/
        calcStep2Goal(step);    /* calculate the value of increase/decrease at one timer_func*/
    }

    if(iBkOffSeq > 0)                 /* when in the backlight off sequence*/
        set_br = 0;                   /* set brightness=0 to hardware*/
    else if(iBkOnSeq > 0)             /* when in the backlight on sequence*/
        set_br = - iStep2Goal[0];       /* set fisrt step brightness to hardware*/
    else
        set_br = calcBrightness2Set();  /* calculate the value of brightness to set*/

    set_brightness(set_br);        /*set brightness to hardware*/
    /*
    printk("bk:%d, set:%d,counter:%d,step:%d,(num step:%d, period:%d)\n"
           ,bk,set_br,iCounter2Goal,iStep2Goal[iCounter2Goal-1],step,period);
    */
    if((iBkOffSeq != 0) || (iBkOnSeq != 0)){ /*if in backlight on or off sequence*/
        bl_timer.expires = jiffies + QFB_BL_CTRL_DELAY_IN_SEQ/10;
                                            /*after 30msec this func must be executed*/
        add_timer(&bl_timer);               /* keep on timer_func executing */
        return ;                            /* till  sequence complete. */
    }

    spin_lock_bh(&bl_lock);
    if(iBkBrightness == iCurBkBrightness){  /* when current brightness equals the setting*/
        iCounter2Goal = 0;                  /* reset to avoid exceeding 100 */
        iFlgTimerOn = 0;                    /*turn timer_func off*/
    }else{
        bl_timer.expires = jiffies + period/10;
        add_timer(&bl_timer);            /* keep on timer_func executing */
    }                                    /* till  current brightness equals the setting*/
    spin_unlock_bh(&bl_lock);

    return;
}

/******************************************************************************
* MODULE   : backlihgt timer_init                                                        
* FUNCTION : backlight control when backlight brightness must be gradually changed.
* RETURN   : none                                                              
* NOTE     : 
* CREATE   : 2006.08.18 ita
* UPDATE   :                                                                   
******************************************************************************/
static void backlight_control_init(void)
{
    int i;
    /*init timer*/
    init_timer(&bl_timer);
    bl_timer.data = 0;
    bl_timer.function = backlight_timer_func;
    /*spinlock*/
    spin_lock_init(&bl_lock);
    for(i=0;i<QFB_MAX_BRIGHTNESS;i++){
        iStep2Goal[i] = 0;
    }
    iCounter2Goal = 0;
	return;
}

#endif /*CONFIG_SONY_ICX1087*/

/********************************************************
 *                                                      *
 *  Function Definitions                                *
 *                                                      *
 *******************************************************/

/******************************************************************************
* MODULE   : qfb_set_brightness
* FUNCTION : set brightness of backlight
* RETURN   : FBSUCCESS(0)        : success
*          : CHKSCRN_TIMEOUT(-1) : timeout
*          :
* NOTE     : none
* CREATE   : 2006.05.26 H.Nagashima
* UPDATE   :
******************************************************************************/
static int qfb_set_brightness(int br, int step, int period)
{
    spin_lock_bh(&bl_lock);
    iBkBrightness = br;
    iBkStep = step;
    iBkPeriod = period;

    if(iFlgTimerOn == 0){ /*when timer off*/
        iFlgTimerOn = 1;
        bl_timer.expires = jiffies + QFB_BL_CTRL_DELAY/10;
        add_timer(&bl_timer);        /*wake up timer_function*/
    }
    spin_unlock_bh(&bl_lock);

    return 0;

}

/******************************************************************************
* MODULE   : qfb_ioctl
* FUNCTION : frame baffer driver ioctl
* RETURN   : FBSUCCESS(0)        : success
*          : CHKSCRN_TIMEOUT(-1) : timeout
*          :
* NOTE     : none
* CREATE   : 2006.05.26 H.Nagashima
* UPDATE   :
******************************************************************************/

int qfb_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg, struct fb_info *info)
{
	int iRet;

	switch (cmd)
	{
		/************************************/
		/* get frame buffer infomation      */
		/************************************/
		case QFB_GET_MODES:
		{
			scrn_modes    FBScrnModes;

			//set now on display infomation
			set_image_info_to_2DAPI(iNowDispPage, &FBScrnModes, info);

			//copy to user scrn mode infomation
			copy_to_user((scrn_modes *) arg, &FBScrnModes, sizeof(scrn_modes));
			break;
		}
		/************************************/
		/* check frame buffer status        */
		/************************************/
		case QFB_CHKSCRN:
		{
			int       iTotalTime = 0;
			scrn_mode FBScrnMode;

			// get scrn mode info
			copy_from_user(&FBScrnMode, (scrn_mode *) arg, sizeof(scrn_mode));

			// check frame buffer status
			FBScrnMode.page0 = chk_fb_status(iFBStatus_A);
			FBScrnMode.page1 = chk_fb_status(iFBStatus_B);

			// if arg->timeout=0, then break
			if(FBScrnMode.timeout <= 0)
			{
				copy_to_user((scrn_mode *) arg, &FBScrnMode, sizeof(scrn_mode));
				break;
			}

			// if both A and B states are "WAITING_FOR_ON_DISPLAY",
			// then waits during the time specified by "timeout"
			if( FBScrnMode.page0 == WAITING_FOR_ON_DISPLAY && FBScrnMode.page1 == WAITING_FOR_ON_DISPLAY )
			{
				// loop until timeout or enable write
				do
				{
					// sleep 10ms
					msleep(RECHK_TIME);
					iTotalTime += RECHK_TIME;

					// re-check frame buffer status
					FBScrnMode.page0 = chk_fb_status(iFBStatus_A);
					FBScrnMode.page1 = chk_fb_status(iFBStatus_B);

					// when over timeout time, return
					if(iTotalTime > FBScrnMode.timeout)
					{
						break;
					}
				}
				while(FBScrnMode.page0 == WAITING_FOR_ON_DISPLAY && FBScrnMode.page1 == WAITING_FOR_ON_DISPLAY);
			}

			// return scrn mode info
			copy_to_user((scrn_mode *) arg, &FBScrnMode, sizeof(scrn_mode));
			if (FBScrnMode.page0 == WAITING_FOR_ON_DISPLAY && FBScrnMode.page1 == WAITING_FOR_ON_DISPLAY)
			{
				return FBScrnMode.timeout;
			}
			break;
		}
		/*******************************************/
		/* Set or Update frame buffer infomation   */
		/*******************************************/
		case QFB_SET_MODES:
		case QFB_UPDATE_SCRN:
		{
			scrn_modes    FBSrcnModes;
			qfb_par      *par = (qfb_par *)info->par;

			// copy fb image info from 2D API
			copy_from_user(&FBSrcnModes, (scrn_modes *) arg, sizeof(scrn_modes));
            /*attention added by ita 2006.08.02 tentavely.*/
            /*To avoid the trouble of freezing kernel.        */
			FBSrcnModes.update = 0; /* commet out by ita 2006.08.24*/
            /*to here, attention added by ita 2006.08.02 tentavely.*/

			// check fb image infomation from 2D API
			iRet = chk_input_modes(FBSrcnModes);
			if(iRet != QFB_SUCCESS)
			{
				return -1;
			}

			down(&par->sem_fb_image_info);		/* lock fb_image_info */

			// set fb image infomation from 2D API
			set_image_info_from_2DAPI(FBSrcnModes.page, &FBSrcnModes, info);

			// set fb image infomaition to LCD
			set_image_info_to_LCD(FBSrcnModes.page, info);

			iNextDispPage = FBSrcnModes.page;

			// when cmd is QFB_SET_MODES, set fb image update flag off
			if(cmd == QFB_SET_MODES)
			{
				fb_image_info.update_flag = FB_UPDATE_OFF;
			}
			// when cmd is QFB_UPDATE_MODES,  set fb image update flag on
			else
			{
				// absolutely update data
				if(FBSrcnModes.update == UPDATE_FLAG_ON)
				{
					fb_image_info.update_flag = FB_ABSOLUTERY_UPDATE;
				}
				// when ONLY_2D_MODE, update data
				else
				{
					fb_image_info.update_flag = FB_UPDATE_ON;
				}
			}

			// set image infomation to LCD driver
			if(mp200_lcd_set_fb_image_data(&fb_image_info)){
				up(&par->sem_fb_image_info);	/* unlock fb_image_info */
				return -EBUSY;
			}

			up(&par->sem_fb_image_info);		/* unlock fb_image_info */

			if (FBSrcnModes.page == DISP_BUFA)
			{
				info->screen_base     = (char __iomem *)DispBufAV;
			}
			else
			{
				info->screen_base     = (char __iomem *)DispBufBV;
			}

			break;
		}
		/************************************/
		/* update timer infomation          */
		/************************************/
		case QFB_UPDATE_TIMER:
		{
			unsigned long flags;
			update_timer   FbUpdateTimer;

			// copy from update timer info from 2D API
			copy_from_user(&FbUpdateTimer, (update_timer *) arg, sizeof(update_timer));

			// fb timer stop
			spin_lock_irqsave(&qfb_dev.timer_lock, flags);
			del_timer(&qfb_timer);
			iTimerStatusFlag = FB_TIMER_STOP;
			spin_unlock_irqrestore(&qfb_dev.timer_lock, flags);

			// check input value
			iRet = chk_input_update_timer(FbUpdateTimer);
			if(iRet != QFB_SUCCESS)
			{
				return -1;
			}

			// set update structure info
			iUpdateTimerFlag = FbUpdateTimer.timerflag;
			iTimeoutTime     = FbUpdateTimer.timeout;

			// fb timer start
			if((iUpdateTimerFlag == TIMER_ON) && (iMixModeFlg == ONLY_2D_MODE))
			{
				spin_lock_irqsave(&qfb_dev.timer_lock, flags);
				if(iTimerStatusFlag == FB_TIMER_STOP)
				{
					qfb_timer.expires = jiffies + iTimeoutTime / 10;
					add_timer(&qfb_timer);
					iTimerStatusFlag = FB_TIMER_START;
				}
				spin_unlock_irqrestore(&qfb_dev.timer_lock, flags);
			}
			break;
		}

#ifdef CONFIG_SONY_ICX1087
        /*from here, 2006/10/17 new backlight command. ita*/
        case QFB_SET_BACKLIGHT:
        {
            bl_brightness bl;
            copy_from_user(&bl, (backlight_setting *)arg, 
                           sizeof(bl_brightness));
            if(((bl.bright <= QFB_BACKLIGHT_MAX) 
                && (bl.bright >= QFB_BACKLIGHT_MIN)) /*check brightness*/
               && ((bl.flag == 1) || (bl.flag == 0)))     /*check flag   */
            {
                iBkIndex = bl.bright;
                iBkFlag = bl.flag;
                stBlSetting.brightness = backlight_table[bl.bright];
                stBlSetting.period =  QFB_BL_CTRL_DELAY;
                if(bl.flag)
                    stBlSetting.step = QFB_GRADUAL_STEP;
                else
                    stBlSetting.step = 1;

                if(iLcdDisable == 0)
                    qfb_set_brightness(stBlSetting.brightness,
                                       stBlSetting.step,
                                       stBlSetting.period);
            }else{
                printk("*** QFB_SET_BACKLIGHT: bad value\n");
                return -1;
            }
            break;
        }
        case QFB_GET_BACKLIGHT:
        {
            bl_brightness bl;
            bl.bright = iBkIndex;
            bl.flag = iBkFlag;
            copy_to_user((bl_brightness *)arg, &bl, 
                           sizeof(bl_brightness));
            break;
        }
        /*added 20061211. Return "1" during backlight is changing*/
        case QFB_GET_BL_CHANGE:
        {
            copy_to_user((int *)arg, &iFlgTimerOn, sizeof(int));
            break;
        }
        /**/
        /*to here, 2006/10/17 new backlight command. ita*/
        /*20060531 SONY Corporation: Set backlight brightness */
        case QFB_SET_BL_BRIGHTNESS:
        {/*modified by ita 20060804*/
            backlight_setting bl_set;
            copy_from_user(&bl_set, (backlight_setting *)arg, 
                           sizeof(backlight_setting));
            if(((bl_set.brightness <= QFB_MAX_BRIGHTNESS) 
                && (bl_set.brightness >= QFB_MIN_BRIGHTNESS)) /*check brightness*/
               && ((bl_set.step <= QFB_MAX_BRIGHTNESS) 
                   && (bl_set.step > QFB_MIN_BRIGHTNESS))     /*check step   */
               && (bl_set.period >= 10))                       /*check period */
            {
                stBlSetting.brightness = bl_set.brightness;
                stBlSetting.step = bl_set.step;
                stBlSetting.period = bl_set.period;
                if(iLcdDisable == 0)
                    qfb_set_brightness(stBlSetting.brightness,
                                       stBlSetting.step,
                                       stBlSetting.period);
            }else{
                printk("*** QFB_SET_BL_BRIGHTNESS: bad value\n");
                return -1;
            }
            break;
        }
        case QFB_GET_BL_BRIGHTNESS:
        {/*modified by ita 20060804*/
            copy_to_user((backlight_setting *)arg, &stBlSetting, 
                           sizeof(backlight_setting));
            break;
        }
        case QFB_SET_USB_SUSPEND_MODE:
        {
            int flg;
            copy_from_user(&flg, (int *)arg, sizeof(int));
            if((flg == 0) || (flg == 1)){
                iUsbSusMode = flg;
            }else{
                printk("*** QFB_SET_USB_SUSPEND_MODE: bad value\n");
                return -1;
            }
            break;
        }
        case QFB_GET_USB_SUSPEND_MODE:
        {
            copy_to_user((int *)arg, &iUsbSusMode, sizeof(int));
            break;
        }
        /*from here. added for test by ita. 2006.10.03*/
        case QFB_INIT_PROG_BAR:
        {
            return qfb_prog_init();
            break;
        }
        case QFB_SET_PROG_BAR:
        {
            int deg;
            copy_from_user(&deg, (int *)arg, sizeof(int));
            return qfb_prog_set_degree(deg);
            break;
        }
        case QFB_GET_PROG_BAR:
        {
            int ret;
            ret = qfb_prog_get_degree();
            copy_to_user((int *)arg, &ret, sizeof(int));
            break;
        }
        case QFB_SET_LCD_DISABLE:
        {
            int flag;
            copy_from_user(&flag, (int *)arg, sizeof(int));
            return qfb_set_lcd_disable(flag);
            break;
        }
        case QFB_GET_LCD_DISABLE:
        {
            int ret,bl;
            ret = qfb_get_lcd_disable(&bl);
            copy_to_user((int *)arg, &ret, sizeof(int));
            break;
        }
        /*to here. added for test by ita. 2006.10.03*/
#endif /* CONFIG_SONY_ICX1087*/
        default:
        {
            break;
        }
	}
	return QFB_SUCCESS;
}


/******************************************************************************
* MODULE   : set_image_info_to_2DAPI
* FUNCTION : set other image infomation (frame bufferA/B) to 2D API
* RETURN   : QFB_SUCCESS(0)       : success
* NOTE     : none
* CREATE   : 2006.05.26 H.Nagashima
* UPDATE   :
******************************************************************************/

static int set_image_info_to_2DAPI(int iFBNo, scrn_modes *fb_scrn_modes, struct fb_info *info)
{
	qfb_par *par = (qfb_par *)info->par;

	//set fb A other image infomation
	if(iFBNo == FB_NO_A)
	{
		down(&par->sem_fb_other_info_A);	/* lock fb_other_info_A */
		fb_scrn_modes->tc_enable = fb_other_info_A.iMaskColFlg;
		fb_scrn_modes->t_color   = fb_other_info_A.iMaskCol;
		fb_scrn_modes->alpha     = fb_other_info_A.iAlpha;
		fb_scrn_modes->rot       = fb_other_info_A.iInvFlg;
		fb_scrn_modes->page      = fb_other_info_A.iPage;
		fb_scrn_modes->update    = fb_other_info_A.iUpDate;
		up(&par->sem_fb_other_info_A);		/* unlock fb_other_info_A */
	}
	//set fb B other image infomation
	else
	{
		down(&par->sem_fb_other_info_B);	/* lock fb_other_info_B */
		fb_scrn_modes->tc_enable = fb_other_info_B.iMaskColFlg;
		fb_scrn_modes->t_color   = fb_other_info_B.iMaskCol;
		fb_scrn_modes->alpha     = fb_other_info_B.iAlpha;
		fb_scrn_modes->rot       = fb_other_info_B.iInvFlg;
		fb_scrn_modes->page      = fb_other_info_B.iPage;
		fb_scrn_modes->update    = fb_other_info_B.iUpDate;
		up(&par->sem_fb_other_info_B);		/* unlock fb_other_info_B */
	}
	return QFB_SUCCESS;
}


/******************************************************************************
* MODULE   : chk_fb_status
* FUNCTION : check frame buffer (A/B) status
* RETURN   : OUT_OF_DISPLAY(0)         : update enable
             WAITING_FOR_ON_DISPLAY(1) : update disable
* NOTE     : none
* CREATE   : 2006.05.26 H.Nagashima
* UPDATE   :
******************************************************************************/

static int chk_fb_status(int iFBStatus)
{
	int iRet;

	//check frame buffer status
	if(iFBStatus == IS_NODATA || iFBStatus == ENABLE_WRITE)
	{
		iRet = OUT_OF_DISPLAY;
	}
	else
	{
		iRet = WAITING_FOR_ON_DISPLAY;
	}
	return iRet;
}


/******************************************************************************
* MODULE   : chk_input_modes
* FUNCTION : check input scrn_modes value from 2D API
* RETURN   : QFB_SUCCESS(0)       : correct
*            -EINVAL              : incorrect
* NOTE     : none
* CREATE   : 2006.06.05 H.Nagashima
* UPDATE   :
******************************************************************************/

static int chk_input_modes(scrn_modes fb_scrn_modes)
{
	// check mask color flag
	if((fb_scrn_modes.tc_enable != FB_MASK_COLOR_DISP_OFF) && (fb_scrn_modes.tc_enable != FB_MASK_COLOR_DISP_ON))
	{
		printk(KERN_WARNING " @mp200_fb: chk_input_modes:  mask color flag incorrect  tc_enable = %d\n", fb_scrn_modes.tc_enable);
		return -EINVAL;
	}

	// check mask color
	if((fb_scrn_modes.t_color < 0x00) || (0xFFFF < fb_scrn_modes.t_color))
	{
		printk(KERN_WARNING " @mp200_fb: chk_input_modes:  mask color incorrect  t_color = %d\n", fb_scrn_modes.t_color);
		return -EINVAL;
	}

	// check alpha
	if((fb_scrn_modes.alpha < 0x00) || (0xFF < fb_scrn_modes.alpha))
	{
		printk(KERN_WARNING " @mp200_fb: chk_input_modes:  alpha incorrect  alpha = %d\n", fb_scrn_modes.alpha);
		return -EINVAL;
	}

	// check rot
	if((fb_scrn_modes.rot != NO_INVERSE) && (fb_scrn_modes.rot != UDRL_INVERSE))
	{
		printk(KERN_WARNING " @mp200_fb: chk_input_modes:  rot incorrect  rot = %d\n", fb_scrn_modes.rot);
		return -EINVAL;
	}

	// check page
	if((fb_scrn_modes.page != FB_NO_A) && (fb_scrn_modes.page != FB_NO_B))
	{
		printk(KERN_WARNING " @mp200_fb: chk_input_modes:  page incorrect  page = %d\n", fb_scrn_modes.page);
		return -EINVAL;
	}

	// check update
	if((fb_scrn_modes.update != UPDATE_FLAG_OFF) && (fb_scrn_modes.update != UPDATE_FLAG_ON))
	{
		printk(KERN_WARNING " @mp200_fb: chk_input_modes:  update incorrect  update = %d\n", fb_scrn_modes.update);
		return -EINVAL;
	}

	return QFB_SUCCESS;
}


/******************************************************************************
* MODULE   : set_image_info_from_2DAPI
* FUNCTION : set image infomation from LCD to fb local
* RETURN   : QFB_SUCCESS(0)       : success
* NOTE     : none
* CREATE   : 2006.05.26 H.Nagashima
* UPDATE   :
******************************************************************************/

static int set_image_info_from_2DAPI(int iFBNo, scrn_modes *fb_scrn_modes, struct fb_info *info)
{
	qfb_par *par = (qfb_par *)info->par;

	//set FB_A image infomation from 2D API
	if(iFBNo == FB_NO_A)
	{
		iFBStatus_A = NOW_WRITING_TO_FB;

		down(&par->sem_fb_other_info_A);	/* lock fb_other_info_A */
		fb_other_info_A.iMaskColFlg = fb_scrn_modes->tc_enable;
		fb_other_info_A.iMaskCol    = fb_scrn_modes->t_color;
		fb_other_info_A.iAlpha      = fb_scrn_modes->alpha;
		fb_other_info_A.iInvFlg     = fb_scrn_modes->rot;
		fb_other_info_A.iPage       = fb_scrn_modes->page;
		fb_other_info_A.iUpDate     = fb_scrn_modes->update;
		up(&par->sem_fb_other_info_A);		/* unlock fb_other_info_A */
	}
	//set FB_B image infomation from 2D API
	else
	{
		iFBStatus_B = NOW_WRITING_TO_FB;

		down(&par->sem_fb_other_info_B);	/* lock fb_other_info_B */
		fb_other_info_B.iMaskColFlg = fb_scrn_modes->tc_enable;
		fb_other_info_B.iMaskCol    = fb_scrn_modes->t_color;
		fb_other_info_B.iAlpha      = fb_scrn_modes->alpha;
		fb_other_info_B.iInvFlg     = fb_scrn_modes->rot;
		fb_other_info_B.iPage       = fb_scrn_modes->page;
		fb_other_info_B.iUpDate     = fb_scrn_modes->update;
		up(&par->sem_fb_other_info_B);		/* unlock fb_other_info_B */
	}

	return QFB_SUCCESS;
}


/******************************************************************************
* MODULE   : set_image_info_to_LCD
* FUNCTION : set image infomation to LCD
* RETURN   : QFB_SUCCESS(0)       : success
* NOTE     : none
* CREATE   : 2006.05.26 H.Nagashima
* UPDATE   :
******************************************************************************/

static int set_image_info_to_LCD(int iFBNo, struct fb_info *info)
{
	qfb_par *par = (qfb_par *)info->par;

	//set fb A infomation
	if(iFBNo == DISP_BUFA)
	{
		fb_image_info.src_image_data.yrgbaddr = DispBufA;

		// set other image infomation
		down(&par->sem_fb_other_info_A);	/* lock fb_other_info_A */
		fb_image_info.maskcolr     = fb_other_info_A.iMaskCol;
		fb_image_info.maskcolrflg  = fb_other_info_A.iMaskColFlg;
		fb_image_info.alpha        = fb_other_info_A.iAlpha;
		fb_image_info.invflg       = fb_other_info_A.iInvFlg;
		fb_image_info.mix_buf_page = fb_other_info_A.iPage;
		up(&par->sem_fb_other_info_A);		/* unlock fb_other_info_A */
	}
	//set fb B infomation
	else
	{
		fb_image_info.src_image_data.yrgbaddr = DispBufB;

		// set other image infomation
		down(&par->sem_fb_other_info_B);	/* lock fb_other_info_B */
		fb_image_info.maskcolr     = fb_other_info_B.iMaskCol;
		fb_image_info.maskcolrflg  = fb_other_info_B.iMaskColFlg;
		fb_image_info.alpha        = fb_other_info_B.iAlpha;
		fb_image_info.invflg       = fb_other_info_B.iInvFlg;
		fb_image_info.mix_buf_page = fb_other_info_B.iPage;
		up(&par->sem_fb_other_info_B);		/* unlock fb_other_info_B */
	}

	return QFB_SUCCESS;
}


/******************************************************************************
* MODULE   : chk_input_update_timer
* FUNCTION : check update timer value from 2D API
* RETURN   : QFB_SUCCESS(0)       : success
           : -EINVAL              : input value error
* NOTE     : none
* CREATE   : 2006.05.26 H.Nagashima
* UPDATE   :
******************************************************************************/

static int chk_input_update_timer(update_timer fb_update_timer)
{

	// check update mode value
	if((fb_update_timer.timerflag != TIMER_OFF) && (fb_update_timer.timerflag != TIMER_ON))
	{
		printk(KERN_WARNING " @mp200_fb: chk_input_update_timer:  timerflag value incorrect  timerflag = %d\n", fb_update_timer.timerflag);
		return -EINVAL;
	}

	// check update mode value
	if(fb_update_timer.timeout < 33)
	{
		printk(KERN_WARNING " @mp200_fb: chk_input_update_timer:  timerout value incorrect  timerout = %d\n", fb_update_timer.timeout);
		return -EINVAL;
	}

	return QFB_SUCCESS;
}


/******************************************************************************
* MODULE   : qfb_mmap
* FUNCTION : mmap
* RETURN   : QFB_SUCCESS(0) : success
*          : -EINVAL        : input value error
*          : -EAGAIN        : run again
* NOTE     : none
* CREATE   :
* UPDATE   : 2006.05.27 H.Nagashima
******************************************************************************/

int qfb_mmap(struct fb_info *info, struct file *file, struct vm_area_struct *vma)
{
	unsigned long start;
	unsigned long off;
	u32 len;

	if (vma->vm_pgoff > (~0UL >> PAGE_SHIFT))
	{
		return -EINVAL;
	}

	off = vma->vm_pgoff << PAGE_SHIFT;
	if (off < SMEM_LENGTH)
	{
		start = (unsigned long)Smem;
		len = PAGE_ALIGN((start & ~PAGE_MASK) + SmemLength);
	}
	else
	{
		return -EINVAL;
	}

	start &= PAGE_MASK;
	if ((vma->vm_end - vma->vm_start + off) > len)
	{
		return -EINVAL;
	}

	off += start;
	vma->vm_pgoff = off >> PAGE_SHIFT;

	// Accessing memory will be done non-cached.
	vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);

	// To stop the swapper from even considering these pages
	vma->vm_flags |= (VM_IO | VM_RESERVED);

	if (io_remap_page_range(vma, vma->vm_start, off, vma->vm_end - vma->vm_start, vma->vm_page_prot))
	{
		return -EAGAIN;
	}

	return QFB_SUCCESS;
}


/******************************************************************************
* MODULE   : open
* FUNCTION : open qfb driver
* RETURN   : QFB_SUCCESS(0)  : success
* NOTE     : none
* CREATE   :
* UPDATE   : 2006.05.29 H.Nagashima
******************************************************************************/

static int open(struct fb_info *info, int user)
{
	qfb_par *par = (qfb_par *)info->par;

	atomic_inc(&par->use_count);

	return QFB_SUCCESS;
}


/******************************************************************************
* MODULE   : release
* FUNCTION : close qfb driver
* RETURN   : QFB_SUCCESS(0)  : success
* NOTE     : none
* CREATE   :
* UPDATE   : 2006.05.29 H.Nagashima
******************************************************************************/

static int release(struct fb_info *info, int user)
{
	qfb_par       *par = (qfb_par *)info->par;
	u32            count;

	count = atomic_read(&par->use_count);
	if (count == 0)
		return -EINVAL;

	if (count == 1) {
        set_flag_init();/*added by ita 2006.09.29. To chnage buffer of picture to default setteing*/
		iUpdateTimerFlag = DEFALT_UPDATE_TIMER;
		iTimeoutTime     = TIMEOUT_TIME;
		if(iMixModeFlg == ONLY_2D_MODE)
		{
			del_timer(&qfb_timer);
			qfb_timer.expires = jiffies + iTimeoutTime / 10;
			add_timer(&qfb_timer);
			iTimerStatusFlag = FB_TIMER_START;
		}
	}

	atomic_dec(&par->use_count);
	return QFB_SUCCESS;
}


/******************************************************************************
* MODULE   : get_line_length
* FUNCTION : get line length
* RETURN   : length
* NOTE     : none
* CREATE   :
* UPDATE   :
******************************************************************************/

static u_long get_line_length(int xres_virtual, int bpp)
{
	u_long length;

	length = xres_virtual * bpp;
	length = (length + 31) & ~31;
	length >>= 3;
	return length;
}


/******************************************************************************
* MODULE   : check_var
* FUNCTION : variable check
* RETURN   : QFB_SUCCESS(0) : success
*            -EINVAL        : input value incorrect
* NOTE     : none
* CREATE   :
* UPDATE   : 2006.05.29 H.Nagashima
******************************************************************************/

static int check_var(struct fb_var_screeninfo *var, struct fb_info *info)
{
	u_long line_length;

	if (var->vmode & FB_VMODE_CONUPDATE)
	{
		var->vmode |= FB_VMODE_YWRAP;
		var->xoffset = info->var.xoffset;
		var->yoffset = info->var.yoffset;
	}

	/* Some very basic checks */
	if (!var->xres)                     var->xres = 1;
	if (!var->yres)                     var->yres = 1;
	if (var->xres > var->xres_virtual)  var->xres_virtual = var->xres;
	if (var->yres > var->yres_virtual)  var->yres_virtual = var->yres;
	if (var->bits_per_pixel <= 1)       var->bits_per_pixel = 1;
	else if (var->bits_per_pixel <= 8)  var->bits_per_pixel = 8;
	else if (var->bits_per_pixel <= 16) var->bits_per_pixel = 16;
	else if (var->bits_per_pixel <= 24) var->bits_per_pixel = 24;
	else if (var->bits_per_pixel <= 32) var->bits_per_pixel = 32;
	else                                return -EINVAL;

	if (var->xres_virtual < var->xoffset + var->xres)
	{
		var->xres_virtual = var->xoffset + var->xres;
	}
	if (var->yres_virtual < var->yoffset + var->yres)
	{
		var->yres_virtual = var->yoffset + var->yres;
	}

	/* Memory limit */
	line_length = get_line_length(var->xres_virtual, var->bits_per_pixel);
	if (line_length * var->yres_virtual > DispBufLength)
	{
		return -ENOMEM;
	}

	/* check var */
	switch (var->bits_per_pixel)
	{
		case 1:
		case 8:
		{
			var->red.offset    = 0;
			var->red.length    = 8;
			var->green.offset  = 0;
			var->green.length  = 8;
			var->blue.offset   = 0;
			var->blue.length   = 8;
			var->transp.offset = 0;
			var->transp.length = 0;
			break;
		}
		case 16:		/* RGBA 5551 */
		{
			if (var->transp.length)
			{
				var->red.offset    = 0;
				var->red.length    = 5;
				var->green.offset  = 5;
				var->green.length  = 5;
				var->blue.offset   = 10;
				var->blue.length   = 5;
				var->transp.offset = 15;
				var->transp.length = 1;
			}
			else		/* RGB 565 */
			{
				var->red.offset    = 11;
				var->red.length    = 5;
				var->green.offset  = 5;
				var->green.length  = 6;
				var->blue.offset   = 0;
				var->blue.length   = 5;
				var->transp.offset = 0;
				var->transp.length = 0;
			}
			break;
		}
		case 24:		/* RGB 888 */
		{
			var->red.offset    = 0;
			var->red.length    = 8;
			var->green.offset  = 8;
			var->green.length  = 8;
			var->blue.offset   = 16;
			var->blue.length   = 8;
			var->transp.offset = 0;
			var->transp.length = 0;
			break;
		}
		case 32:		/* RGBA 8888 */
		{
			var->red.offset    = 0;
			var->red.length    = 8;
			var->green.offset  = 8;
			var->green.length  = 8;
			var->blue.offset   = 16;
			var->blue.length   = 8;
			var->transp.offset = 24;
			var->transp.length = 8;
			break;
		}
	}
	var->red.msb_right    = 0;
	var->green.msb_right  = 0;
	var->blue.msb_right   = 0;
	var->transp.msb_right = 0;

	return QFB_SUCCESS;
}

/******************************************************************************
* MODULE   : set_par
* FUNCTION : set qfb parameter
* RETURN   : QFB_SUCCESS(0) : success
* NOTE     : none
* CREATE   :
* UPDATE   : 2006.05.29 H.Nagashima
******************************************************************************/

static int set_par(struct fb_info *info)
{
	info->fix.line_length = get_line_length(info->var.xres_virtual, info->var.bits_per_pixel);
	return QFB_SUCCESS;
}


/******************************************************************************
* MODULE   : setcolreg
* FUNCTION : set qfb color
* RETURN   : QFB_SUCCESS(0) : success
* NOTE     : none
* CREATE   :
* UPDATE   : 2006.05.29 H.Nagashima
******************************************************************************/

static int setcolreg(u_int regno, u_int red, u_int green, u_int blue, u_int transp, struct fb_info *info)
{
	/* No. of hw registers */
	if (regno >= 256)
		return 1;

	/* grayscale works only partially under directcolor */
	if (info->var.grayscale)
	{
		/* grayscale = 0.30*R + 0.59*G + 0.11*B */
		red = green = blue = (red * 77 + green * 151 + blue * 28) >> 8;
	}

#define CNVT_TOHW(val,width) ((((val)<<(width))+0x7FFF-(val))>>16)
	switch (info->fix.visual)
	{
		case FB_VISUAL_TRUECOLOR:
		case FB_VISUAL_PSEUDOCOLOR:
		{
			red = CNVT_TOHW(red, info->var.red.length);
			green = CNVT_TOHW(green, info->var.green.length);
			blue = CNVT_TOHW(blue, info->var.blue.length);
			transp = CNVT_TOHW(transp, info->var.transp.length);
			break;
		}
		case FB_VISUAL_DIRECTCOLOR:
		{
			red = CNVT_TOHW(red, 8);	/* expect 8 bit DAC */
			green = CNVT_TOHW(green, 8);
			blue = CNVT_TOHW(blue, 8);
			/* hey, there is bug in transp handling... */
			transp = CNVT_TOHW(transp, 8);
			break;
		}
	}
#undef CNVT_TOHW
	/* Truecolor has hardware independent palette */
	if (info->fix.visual == FB_VISUAL_TRUECOLOR)
	{
		u32 v;

		if (regno >= 16)
		{
			return 1;
		}

		v = (red << info->var.red.offset)   | (green << info->var.green.offset)  |
		    (blue << info->var.blue.offset) | (transp << info->var.transp.offset);
		switch (info->var.bits_per_pixel)
		{
			case 16:
			case 24:
			case 32:
			{
#ifndef CONFIG_SONY_ICX1087
                ((u32 *) (info->pseudo_palette))[regno] = v;
#endif
				break;
			}
			case 8:
			default :
			{
				break;
			}
		}
	}
	return QFB_SUCCESS;
}

/******************************************************************************
* MODULE   : pan_display
* FUNCTION :
* RETURN   : QFB_SUCCESS(0) : success
*            -EINVAL()     : input value incorrect
* NOTE     : none
* CREATE   :
* UPDATE   : 2006.05.29 H.Nagashima
******************************************************************************/

static int pan_display(struct fb_var_screeninfo *var, struct fb_info *info)
{
	if (var->vmode & FB_VMODE_YWRAP)
	{
		if (var->yoffset < 0 || var->yoffset >= info->var.yres_virtual || var->xoffset)
		{
			return -EINVAL;
		}
	}
	else
	{
		if (var->xoffset + var->xres > info->var.xres_virtual ||
		    var->yoffset + var->yres > info->var.yres_virtual )
		{
			return -EINVAL;
		}
	}
	info->var.xoffset = var->xoffset;
	info->var.yoffset = var->yoffset;
	if (var->vmode & FB_VMODE_YWRAP)
	{
		info->var.vmode |= FB_VMODE_YWRAP;
	}
	else
	{
		info->var.vmode &= ~FB_VMODE_YWRAP;
	}

	return QFB_SUCCESS;
}

/******************************************************************************
* MODULE   : qfb_callback
* FUNCTION : call back from LCD driver
* RETURN   : none
* NOTE     : public function
* CREATE   : 2006.05.27 H.Nagashima
* UPDATE   :
******************************************************************************/

void qfb_callback(int iWrittenFBNo, int iMixModeStatus)
{
	iMixModeFlg = iMixModeStatus;

	// fb timer stop
	del_timer(&qfb_timer);
	iTimerStatusFlag = FB_TIMER_STOP;

	// change FB (A/B) status
	if(iWrittenFBNo == FB_NO_A)
	{
		iNowDispPage = FB_NO_A;
		iFBStatus_A  = ALREADY_DISPLAY;
		if (iNextDispPage != FB_NO_B) iFBStatus_B  = ENABLE_WRITE;
	}
	else
	{
		iNowDispPage = FB_NO_B;
		iFBStatus_B  = ALREADY_DISPLAY;
		if (iNextDispPage != FB_NO_A) iFBStatus_A  = ENABLE_WRITE;
	}

	// start timer
	if((iUpdateTimerFlag == TIMER_ON) && (iMixModeFlg == ONLY_2D_MODE))
	{
		qfb_timer.expires = jiffies + iTimeoutTime / 10;
		add_timer(&qfb_timer);
		iTimerStatusFlag = FB_TIMER_START;
	}

	return;
}


/********************************************************
 *                                                      *
 *  Init Function Definitions                           *
 *                                                      *
 *******************************************************/

/******************************************************************************
* MODULE   : qfb_init_module
* FUNCTION : frame buffer driver init function
* RETURN   : QFB_SUCCESS(0) : success
*            -ENXIO         : fail
* NOTE     : none
* CREATE   :
* UPDATE   : 2006.05.27 H.Nagashima
******************************************************************************/

int __init qfb_init_module(void)
{
	int   iRet   = 0;

    printk("qfb_init_module start - ");
#ifdef CONFIG_SONY_ICX1087
    stBlSetting.brightness = QFB_DEFAULT_BRIGHT;
    stBlSetting.step = iBkStep;
    stBlSetting.period = iBkPeriod;
    qfb_init_module_called = 1;
    printk("qfb_init_module done\n");
#endif /*CONFIG_SONY_ICX1087*/

	iRet = driver_register(&qfb_driver);
	if (!iRet)
	{
		return iRet;
	}


	return QFB_SUCCESS;
}


/******************************************************************************
* MODULE   : probe
* FUNCTION : frame buffer driver probe function
* RETURN   : QFB_SUCCESS(0) : success
*            -ENOMEM        : mem error
*            -EBUSY         : other driver init error
* NOTE     : none
* CREATE   :
* UPDATE   : 2006.05.27 H.Nagashima
******************************************************************************/

static int __init probe(struct device *device)
{
	struct platform_device *dev  = to_platform_device(device);
	struct fb_info         *info = NULL;
	qfb_par                *par  = NULL;
	int                     iRet = -ENOMEM;

	// reserve region Smem
	if (!request_mem_region(Smem, SmemLength, "qfb"))
	{
		printk(KERN_WARNING " @mp200_fb: probe: cannot reserve smem\n");
		probe_error(QFB_INIT_ERROR__SMEM_NOT_RESERVE, info);
		return -ENOMEM;
	}

	// remap SmemV
	SmemV = ioremap_nocache(Smem, SmemLength);
	if (!SmemV)
	{
		printk(KERN_WARNING " @mp200_fb: probe: cannot ioremap_nocache smem\n");
		probe_error(QFB_INIT_ERROR__SMEM_NOT_REMAP, info);
		return -ENOMEM;
	}
	printk(KERN_INFO " @mp200_fb: Smem address = 0x%lx 0x%p\n", Smem, SmemV);

	//initialize IMG driver
	if (mp200_img_init_module() < 0)
	{
		printk(KERN_WARNING " @mp200_fb: probe: cannot initialize IMG\n");
		probe_error(QFB_INIT_ERROR__IMG_NOT_INITIALIZE, info);
		return -EBUSY;
	}

	//initialize LCD driver
	if (mp200_lcd_init_module() != 0)
	{
		printk(KERN_WARNING " @mp200_fb: probe: cannot initialize LCD\n");
		probe_error(QFB_INIT_ERROR__LCD_NOT_INITIALIZE, info);
		return -EBUSY;
	}

	//set buffer address
	set_buffer_address();

	// alloc framebuffer
	info = framebuffer_alloc(sizeof(qfb_par), &dev->dev);
	if (!info)
	{
		printk(KERN_WARNING " @mp200_fb: probe: cannot framebuffer alloc\n");
		probe_error(QFB_INIT_ERROR__FB_NOT_ALLOC, info);
		return -ENOMEM;
	}

	//set infomation
	set_info(info, &par);

	// alloc cmap
	iRet = fb_alloc_cmap(&info->cmap, 256, 0);
	if (iRet < 0)
	{
		printk(KERN_WARNING " @mp200_fb: probe: cannot framebuffer alloc cmap\n");
		probe_error(QFB_INIT_ERROR__FB_CMAP_NOT_ALLOC, info);
		return -ENOMEM;
	}

	// regist frame buffer
	iRet = register_framebuffer(info);
	if (iRet < 0)
	{
		printk(KERN_WARNING " @mp200_fb: probe: cannot regist framebuffer\n");
		probe_error(QFB_INIT_ERROR__FB_NOT_REGIST, info);
		return -EBUSY;
	}

	// set driver data
	dev_set_drvdata(&dev->dev, info);

	/* semaphore initialize */
	sema_init(&par->sem_fb_image_info, 1);
	sema_init(&par->sem_fb_other_info_A, 1);
	sema_init(&par->sem_fb_other_info_B, 1);

	/* workqueue initialize */
	par->mp200_fb_workqueue = create_singlethread_workqueue("mp200_fb");
	INIT_WORK(&par->wk_timeout_update_bottom_half, timeout_update_bottom_half_do, (void *)info);

	// flag initialize
	set_flag_init();

	// variable initialize
	set_val_init();

	// timer init
	timer_init(info);

	backlight_control_init();

	return QFB_SUCCESS;
}


/******************************************************************************
* MODULE   : probe_error
* FUNCTION : When probe error, cleanup
* RETURN   : none
* NOTE     :
* CREATE   : 2006.05.27 H.Nagashima
* UPDATE   :
******************************************************************************/

static void __init probe_error(int iErrorMode, struct fb_info *info)
{
	switch(iErrorMode)
	{
		case    QFB_INIT_ERROR__FB_NOT_REGIST:		fb_dealloc_cmap(&info->cmap);
		case    QFB_INIT_ERROR__FB_CMAP_NOT_ALLOC:	framebuffer_release(info);
		case    QFB_INIT_ERROR__FB_NOT_ALLOC:
		case    QFB_INIT_ERROR__LCD_NOT_INITIALIZE:
		case    QFB_INIT_ERROR__IMG_NOT_INITIALIZE:	iounmap(SmemV);
		case    QFB_INIT_ERROR__SMEM_NOT_REMAP:		release_mem_region(Smem, SmemLength);
		case    QFB_INIT_ERROR__SMEM_NOT_RESERVE:
		default :									break;
	}

	return;
}


/******************************************************************************
* MODULE   : set_buffer_address
* FUNCTION : set frame buffer (A/B) address
* RETURN   : none
* NOTE     :
* CREATE   : 2006.05.27 H.Nagashima
* UPDATE   :
******************************************************************************/

static void set_buffer_address(void)
{
	//set & clean DispBufA(2D) address
	DispBufAV = SmemV + DISPBUF_A_OFFSET;

    /* modified to make initial picture drawn by u-boot alive. 2006.08.11*/
	/*memset_io((char __iomem *)DispBufAV, 0, DispBufLength); this is original */
	/* memcpy_fromio((char __iomem *)DispBufAV, (char __iomem *)FrameBufAV, DispBufLength);*/

    /* moreover modify. ita 2006.09.21 */
    /* DispBufAV has been already filled with RGB565 data of initial picture
     by U-BOOT.*/

	//set & clean DispBufB(2D) address
	DispBufBV = SmemV + DISPBUF_B_OFFSET;
	memset_io((char __iomem *)DispBufBV, 0, DispBufLength);

	return;
}


/******************************************************************************
* MODULE   : set_info
* FUNCTION : set frame buffer driver infomation
* RETURN   : none
* NOTE     :
* CREATE   : 2006.05.30 H.Nagashima
* UPDATE   :
******************************************************************************/

static void set_info(struct fb_info *info, qfb_par **par)
{
	int iRet;

	*par                  = (qfb_par *)info->par;
	info->screen_base     = (char __iomem *)DispBufAV;
	info->screen_size     = DispBufLength;
	info->fbops           = &qfb_ops;

	iRet = fb_find_mode(&info->var, info, NULL, NULL, 0, NULL, 8);

	if (!iRet || (iRet == 4))
	{
		info->var = qfb_default;
	}
	info->fix = qfb_fix;
	info->fix.line_length =info->var.xres_virtual * info->var.bits_per_pixel / 8;
	info->fix.smem_start  = Smem;
	info->fix.smem_len    = SmemLength;
	info->pseudo_palette  = (*par)->pseudo_palette;
	info->flags           = FBINFO_FLAG_DEFAULT;

	return;
}


/******************************************************************************
* MODULE   : set_flag_init
* FUNCTION : flag initialize
* RETURN   : none
* NOTE     :
* CREATE   : 2006.05.30 H.Nagashima
* UPDATE   :
******************************************************************************/

static void set_flag_init(void)
{
	iFBStatus_A      = IS_NODATA;
	iFBStatus_B      = IS_NODATA;
	iNowDispPage     = DISP_BUFA;
	iNextDispPage    = DISP_BUFA;
	iUpdateTimerFlag = DEFALT_UPDATE_TIMER;
	iUpdateTimerFlag = TIMER_ON;     /*2006.06.20 ita added this line. Initial TimeFlag should be always ON*/
	iMixModeFlg      = ONLY_2D_MODE;

	return;
}


/******************************************************************************
* MODULE   : set_val_init
* FUNCTION : variable initialize
* RETURN   : none
* NOTE     :
* CREATE   : 2006.06.05 H.Nagashima
* UPDATE   :
******************************************************************************/

static void set_val_init(void)
{
	// fb A val init
	fb_other_info_A.iMaskColFlg = TC_COLOR_DISABLE;
	fb_other_info_A.iMaskCol    = 0x0020;
	fb_other_info_A.iAlpha      = 0;//0xFF; 20060614 ita : for test
	fb_other_info_A.iInvFlg     = NO_INVERSE;
	fb_other_info_A.iPage       = DISP_BUFA;
	fb_other_info_A.iUpDate     = UPDATE_FLAG_OFF;

	// fb B val init
	fb_other_info_B.iMaskColFlg = TC_COLOR_DISABLE;
	fb_other_info_B.iMaskCol    = 0x0020;
	fb_other_info_B.iAlpha      = 0;//0xFF;20060614 ita : for test
	fb_other_info_B.iInvFlg     = NO_INVERSE;
	fb_other_info_B.iPage       = DISP_BUFB;
	fb_other_info_B.iUpDate     = UPDATE_FLAG_OFF;

	// fb data init (to LCD)
	fb_image_info.src_image_data.hsize    = FRONT_WIDTH;
	fb_image_info.src_image_data.vsize    = FRONT_HEIGHT;
	fb_image_info.src_image_data.size     = FRONT_WIDTH * BYTES_PER_PIXEL;
	fb_image_info.src_image_data.yrgbaddr = DispBufA;
	fb_image_info.src_image_data.uvaddr   = 0x00000000;
	fb_image_info.src_image_data.x        = 0;
	fb_image_info.src_image_data.y        = 0;

	// fb info init (to LCD)
	fb_image_info.maskcolrflg  = FB_MASK_COLOR_DISP_OFF;
	fb_image_info.maskcolr     = 0x0020;
	fb_image_info.alpha        = 0;//0xFF;20060614 ita : for test
	fb_image_info.invflg       = NO_INVERSE;
	fb_image_info.mix_buf_page = DISP_BUFA;
	fb_image_info.update_flag  = FB_UPDATE_OFF;

	// timer
	iTimeoutTime               = TIMEOUT_TIME;

	return;
}


/********************************************************
 *                                                      *
 *  Timer Function Definitions                          *
 *                                                      *
 *******************************************************/

/******************************************************************************
* MODULE   : timer_init
* FUNCTION : call back from LCD driver
* RETURN   : none
* NOTE     : public function
* CREATE   : 2006.06.07 H.Nagashima
* UPDATE   :
******************************************************************************/

static void timer_init(struct fb_info *info)
{

	init_timer(&qfb_timer);
	qfb_timer.function = timeout_update;
	qfb_timer.data = (unsigned long)info;
	if((iUpdateTimerFlag == TIMER_ON) && (iMixModeFlg == ONLY_2D_MODE))
	{
		qfb_timer.expires = jiffies + TIMEOUT_TIME / 10;
		add_timer(&qfb_timer);
	}
	return;
}

/******************************************************************************
* MODULE   : timeout_update
* FUNCTION : when Timer timeout, update execution
* RETURN   : none
* NOTE     : public function
* CREATE   : 2006.06.07 H.Nagashima
* UPDATE   :
******************************************************************************/

static void timeout_update(unsigned long data)
{
	unsigned long   flags;
	struct fb_info *info = (struct fb_info *)data;
	       qfb_par *par  = (qfb_par *)info->par;

	// fb timer stop
	spin_lock_irqsave(&qfb_dev.timer_lock, flags);
	del_timer(&qfb_timer);
	iTimerStatusFlag = FB_TIMER_STOP;
	spin_unlock_irqrestore(&qfb_dev.timer_lock, flags);

	queue_work(par->mp200_fb_workqueue, &par->wk_timeout_update_bottom_half);

	// start timer
	if((iUpdateTimerFlag == TIMER_ON) && (iMixModeFlg == ONLY_2D_MODE))
	{
		spin_lock_irqsave(&qfb_dev.timer_lock, flags);
		if(iTimerStatusFlag == FB_TIMER_STOP)
		{
			qfb_timer.expires = jiffies + iTimeoutTime / 10;
			add_timer(&qfb_timer);
			iTimerStatusFlag = FB_TIMER_START;
		}
		spin_unlock_irqrestore(&qfb_dev.timer_lock, flags);
	}

	return;
}


/******************************************************************************
* MODULE   : timeout_update_bottom_half_do
* FUNCTION : when Timer timeout, update execution
* RETURN   : none
* NOTE     : private function
* CREATE   : 2006.07.10 Y.Utsuhara
* UPDATE   :
******************************************************************************/

static void timeout_update_bottom_half_do(void *num)
{
	struct fb_info *info = (struct fb_info *)num;
	qfb_par *par = (qfb_par *)info->par;

	if(down_trylock(&par->sem_fb_image_info))		/* lock fb_image_info */
		return;

	// set image infomation to LCD
	set_image_info_to_LCD(iNextDispPage, info);

	// udpate on (normal QFB_UPDATE_SCRN exe)
	fb_image_info.update_flag = FB_UPDATE_ON;

	// call fb function
	mp200_lcd_set_fb_image_data(&fb_image_info);

	up(&par->sem_fb_image_info);		/* unlock fb_image_info */

	return;
}


/********************************************************
 *                                                      *
 *  Exit Function Definitions                           *
 *                                                      *
 *******************************************************/

/******************************************************************************
* MODULE   : qfb_remove
* FUNCTION : remove
* RETURN   : QFB_SUCCESS(0) : success
* NOTE     : none
* CREATE   :
* UPDATE   : 2006.05.27 H.Nagashima
******************************************************************************/

static int remove(struct device *device)
{
	struct fb_info *info = dev_get_drvdata(device);

	//remove frame buffer
	if (info)
	{
		unregister_framebuffer(info);
		fb_dealloc_cmap(&info->cmap);
		framebuffer_release(info);
	}

	// lcd remove
	mp200_lcd_remove();

	// remove Smem
	iounmap(SmemV);
	release_mem_region(Smem, SmemLength);

	return QFB_SUCCESS;
}


#ifdef CONFIG_DPM
/********************************************************
 *                                                      *
 *  Suspend/Resume Function Definitions                 *
 *                                                      *
 *******************************************************/

/******************************************************************************
* MODULE   : qfb_suspend
* FUNCTION : suspend frame buffer driver
* RETURN   : QFB_SUCCESS(0) : success
* NOTE     : none
* CREATE   :
* UPDATE   : 2006.05.29 H.Nagashima
******************************************************************************/
static int qfb_suspend(struct device *dev, u32 state, u32 level)
{
    int i;
    /* 2006.06.14 ita : debug */
    printk(KERN_INFO "%s() (state=%d, level=%d) \n",
           __FUNCTION__, state, level);

    /*-----------
      2006.11.17 counter measure of problem that LCD can not resume when HOLD SW
      turns on.
      In the case that suspend executes 
      during backlight control timer function is alive,
      LCDC register is not uninitiated and initiated normally.
    ------------*/
    //comment out by ita. 2006/12/21
    //while(iFlgTimerOn){ /*during backlight control timer function is alive.*/
    //  msleep(10);  /*wait 10msec*/
    //}
    for(i=0;i<100;i++){
        if(iFlgTimerOn == 0){
            break;
        }
        msleep(10);  /*wait 10msec*/
        if(i==99){
            printk(KERN_ERR"qfb_suspend iFlgTimerOn TIME OUT!\n");
            printk(KERN_ERR"stBlSetting.brightness=%d,.step=%d,.period=%d,iBkBrightness=%d,iBkStep=%d,iBkPeriod=%d,iGoalBrightness=%d,iCurBkBrightness=%d,iFlgTimerOn=%d,iBkOffSeq=%d,iBkOnSeq=%d,iLcdDisable=%d,qfb_init_module_called=%d,iCounter2Goal=%d\n",
                   stBlSetting.brightness,
                   stBlSetting.step,
                   stBlSetting.period,
                   iBkBrightness,
                   iBkStep,
                   iBkPeriod,
                   iGoalBrightness,
                   iCurBkBrightness,
                   iFlgTimerOn,
                   iBkOffSeq,
                   iBkOnSeq,
                   iLcdDisable,
                   qfb_init_module_called,
                   iCounter2Goal
                );
        }
    }
	switch (level)
	{
		case SUSPEND_DISABLE:
		{
			/* stop timer interrupt */
			del_timer(&qfb_timer);
#ifdef CONFIG_SONY_ICX1087 
            del_timer(&bl_timer);     /* stop backlight control timer func*/
            iFlgTimerOn = 0;  //timer off
#endif /*CONFIG_SONY_ICX1087*/ 
			break;
		}
		case SUSPEND_SAVE_STATE:
        {
            break;
        }
		case SUSPEND_POWER_DOWN:
        {
            break;
        }
		default:
		{
			break;
		}
	}

	// call LCD suspend function
	mp200_lcd_suspend(dev, state, level);
	// call IMG suspend function
	mp200_img_suspend(dev, state, level);

#ifdef CONFIG_SONY_ICX1087
	switch (level) {
    case SUSPEND_POWER_DOWN:
        /*2006.08.07 ita*/
        suspend_lcdhw(1);    /* NOTE: power off lcd and also change REGLOG */

        break;
	}
#endif /* 2006.06.14 ita:  to here*/
    
	return QFB_SUCCESS;
}


/******************************************************************************
* MODULE   : qfb_resume
* FUNCTION : resume frame buffer driver
* RETURN   : QFB_SUCCESS(0) : success
* NOTE     : none
* CREATE   :
* UPDATE   : 2006.05.29 H.Nagashima
******************************************************************************/

int qfb_resume(struct device *dev, u32 level)
{
    /* 2006.06.14 ita : debug */
    printk(KERN_INFO "%s() (level=%d) \n",
           __FUNCTION__, level);

	// call IMG resume function
	mp200_img_resume(dev, level);
	// call LCD resume function
	mp200_lcd_resume(dev, level);

	switch (level)
	{
		case RESUME_POWER_ON:
        {
#ifdef CONFIG_SONY_ICX1087  /* 2006.07.31 ita */
            /* when resume from USB SUSPEND */
            if((iUsbSusMode) && (mp200_pm_get_usb_suspend_state()))
                iBkBrightness = 0;   /* resume as backlihgt off*/

            /*resume setting of backlight*/
            iCurBkBrightness = iBkBrightness;
            iGoalBrightness = iBkBrightness;

            resume_lcdhw(iBkBrightness);  /* modified by ita 2006.07.31 */
#endif
            break;
        }
        case RESUME_RESTORE_STATE:
        {
            break;
        }
        case RESUME_ENABLE:
		{
            
			if((iUpdateTimerFlag == TIMER_ON) && (iMixModeFlg == ONLY_2D_MODE))
			{
				/* start timer interrupt */
				add_timer(&qfb_timer);
			}
			break;
		}
		default:
		{
			break;
		}
	}

	return QFB_SUCCESS;
}


#endif /* CONFIG_DPM */


#ifdef CONFIG_SONY_ICX1087
/**
 *
 */
static void qfb_shutdown(struct device *dev)
{
    printk("%s() \n", __FUNCTION__);

    /*2006.08.07 ita*/
    suspend_lcdhw(1);    /* NOTE: power off lcd and also change REGLOG */
    return;
    
}

/*Initial Progress Bar*/

#define PROG_LENGTH QFB_PROG_MAX
#define PROG_HEIGHT  8

#define PROG_INIT_POS_X   20
#define PROG_INIT_POS_Y   310

#define PROG_BACK_LENGTH  200
#define PROG_BACK_HEIGHT   8
#define PROG_BACK_POS_X   20
#define PROG_BACK_POS_Y   310

#define PROG_BACK_START_OFFSET (PROG_BACK_POS_Y * FRONT_WIDTH) + PROG_BACK_POS_X
#define PROG_START_OFFSET (PROG_INIT_POS_Y * FRONT_WIDTH) + PROG_INIT_POS_X

int iProgFlag = 0;   /*0...uninit, 1...init*/
int iProgDegree = 0;
int iProgDispPage = 0;
unsigned short pb_table[] = {
    0x747F,
    0x9D7F,
    0x437F,
    0x437F,
    0x5C3F,
    0x74FF,
    0x8DBF,
    0x9E5F
};
unsigned short bg_table[] = {
    0x18C4,
    0x18C4,
    0x20E5,
    0x2926,
    0x3167,
    0x39A8,
    0x40E9,
    0x420A
};

/******************************************************************************
* FUNCTION : initiate progress bar. set progress degree 0. 
*            display back background of progress bar.
* RETURN   : QFB_SUCCESS(0) : success
* NOTE     : none
* CREATE   :
* UPDATE   : 2006.10.3 ita
******************************************************************************/
int qfb_prog_init(void)
{
    unsigned short *fb_cur, *fb2write;
    unsigned short *bg;
    int w,h;

    /* copy whole frame buffer */
    if(iNowDispPage == FB_NO_A ){
        fb_cur = (unsigned short *)DispBufAV;
        fb2write = (unsigned short *)DispBufBV;
    }else{
        fb_cur = (unsigned short *)DispBufBV;
        fb2write = (unsigned short *)DispBufAV;
    }

    memcpy_fromio((char __iomem *)fb2write, (char __iomem *)fb_cur, DispBufLength);

    /* draw background */
    for(h=0;h<PROG_BACK_HEIGHT;h++){
        bg = (unsigned short *)(fb2write + PROG_BACK_START_OFFSET 
                                + (FRONT_WIDTH * h));
        for(w=0;w<PROG_BACK_LENGTH;w++){
            *bg++ = (unsigned short)bg_table[h];
        }
    }

    /* reset progress degree */
    iProgDegree = 0;
    iProgFlag = 1;

    /* Chnage display page */
    if(iNowDispPage == FB_NO_A )
        iNextDispPage = FB_NO_B;
    else 
        iNextDispPage = FB_NO_A;

    iProgDispPage = iNextDispPage;  /*Set page to draw progress bar.*/

    return 0;

}

/******************************************************************************
* FUNCTION : un-initiate progress bar. set progress degree 0. 
*            un-display back background of progress bar.
* RETURN   : QFB_SUCCESS(0) : success
* NOTE     : none
* CREATE   :
* UPDATE   : 2006.10.3 ita
******************************************************************************/
int qfb_prog_uninit(void)
{
    return 0;
}

/******************************************************************************
* FUNCTION : set progress degree(0-128)
*            display progress bar whose length is set by this command
* RETURN   : QFB_SUCCESS(0) : success
* NOTE     : none
* CREATE   :
* UPDATE   : 2006.10.3 ita
******************************************************************************/
int qfb_prog_set_degree(int deg)
{
    int w,h;
    unsigned short *fb2write, *bar;

    if(iProgFlag == 0){                          /*check if init or un-init*/
        printk("QFB error [prog has been not initiated yet.]\n");
        return -1;
    }

    if((deg < QFB_PROG_MIN) || (deg > QFB_PROG_MAX)){   /* check value             */
        printk("QFB error [prog degree is bad value]\n");
        return -1;
    }

    if(iProgDispPage == FB_NO_A )
        fb2write = (unsigned short *)DispBufAV;
    else
        fb2write = (unsigned short *)DispBufBV;
    
    /*draw progress bar*/
    for(w=0;w<(deg-iProgDegree);w++){
        bar = (unsigned short *)(fb2write + PROG_START_OFFSET 
                                 + w
                                 + (iProgDegree));
        for(h=0;h<PROG_HEIGHT;h++){
            bar[FRONT_WIDTH * h] = (unsigned short)pb_table[h];
        }
    }

    iProgDegree = deg;
    //printk("Deg:%d, addr:0x%x\n",iProgDegree,(int)fb2write);

    return 0;
}

/******************************************************************************
* FUNCTION : get progress degree(0-128)
* RETURN   : progress bar degree(0-128), -1...un-initiate
* NOTE     : none
* CREATE   :
* UPDATE   : 2006.10.3 ita
******************************************************************************/
int qfb_prog_get_degree(void)
{    
    if(iProgFlag == 0){   /*when un-init*/
        return -1;
    }
    
    return iProgDegree;
}

/******************************************************************************
* FUNCTION : when iLcdDisable=1, backlight is always off.
*            when 0, backlight turn on at setting brightness
* RETURN   : -1...fail
* INPUT    :  flag: 1...lcd disable, 0...enable
* NOTE     : bit0:1...disableLCD,  0...enableLCD
*          : bit4-7: brightness(0-5)
* CREATE   :
* UPDATE   : 2006.10.20 ita
******************************************************************************/
#define QFB_DISABLE_FLAG_MASK 0xff
#define QFB_DISABLE_BIT_MASK 0x1
#define QFB_DISABLE_KIDO_SHIFT 4
#define QFB_DISABLE_KIDO_MASK 0x70

int qfb_set_lcd_disable(int flag)
{
    int kido,tmp_flag;

    if((flag &  QFB_DISABLE_BIT_MASK) == 0){                  /*check value        */
        //printk("disable=%d,%d,%d\n",iLcdDisable,flag,qfb_init_module_called);
        if(iLcdDisable != flag){                        /*when change        */
            if(qfb_init_module_called == 0){/*when qfb has not been initialized yet.*/
                /*do nothing*/
            }else{                        /*when qfb has already initialized.*/
                qfb_set_brightness(stBlSetting.brightness,  
                                   QFB_GRADUAL_STEP,
                                   QFB_BL_CTRL_DELAY);      /* set h/w to turn on */
            }
            iLcdDisable = 0;                         /*set value           */
        }
    }else if((flag & QFB_DISABLE_BIT_MASK) == 1){             /*check value        */
        //printk("dis=%d,%d,%d\n",iLcdDisable,flag,qfb_init_module_called);
        if(iLcdDisable != flag){                      /*when change        */
            kido = (flag & QFB_DISABLE_FLAG_MASK) >> QFB_DISABLE_KIDO_SHIFT;
            if(kido > 5)
                kido = 5;
            tmp_flag = (kido << QFB_DISABLE_KIDO_SHIFT) | QFB_DISABLE_BIT_MASK;
            if(qfb_init_module_called == 0){/*when qfb has not been initialized yet.*/
                iBkBrightness=backlight_table[kido];            /*because fb will start with brightness=0*/
                iGoalBrightness = backlight_table[kido];
                iCurBkBrightness= backlight_table[kido];
            }else{                          /*when qfb has already initialized.*/
                qfb_set_brightness(backlight_table[kido],
                                   QFB_GRADUAL_STEP, 
                                   QFB_BL_CTRL_DELAY);   /* set h/w to turn off */
            }
            iLcdDisable = tmp_flag;                     /*set value           */
        }
    }else{
        printk(KERN_WARNING"QFB error in qfb_set_lcd_disabel [bad value]\n");
        return -1;
    }
    return 0;

}
int qfb_get_lcd_disable(int *bl_bright)
{
    *bl_bright = 
        backlight_table[(iLcdDisable & QFB_DISABLE_FLAG_MASK) >> QFB_DISABLE_KIDO_SHIFT];
    return iLcdDisable;

}

#endif /* CONFIG_SONY_ICX1087 */


/* function alias  */
module_init(qfb_init_module);

/* EXPORT_SYMBOL */
EXPORT_SYMBOL(qfb_callback);
EXPORT_SYMBOL(qfb_prog_get_degree);
EXPORT_SYMBOL(qfb_prog_set_degree);
EXPORT_SYMBOL(qfb_prog_init);
EXPORT_SYMBOL(qfb_set_lcd_disable);
EXPORT_SYMBOL(qfb_get_lcd_disable);
