/*
 * File Name       : /drivers/video/mp200_v4l2/mp200_fb.h
 * Function        : Frame Buffer Driver corresponding to V4L2  definitions
 * Release Version :
 * Release Date    :
 *
 * Copyright (C) NEC Electronics Corporation 2006
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef _MP200_FB_H_
#define _MP200_FB_H_


/********************************************************
 *                                                      *
 *  Definitions                                         *
 *                                                      *
 *******************************************************/

/*
 * Error type
 */

/* normal */
#define QFB_SUCCESS				0

/* init function */
#define QFB_INIT_ERROR__SMEM_NOT_RESERVE	-1
#define QFB_INIT_ERROR__SMEM_NOT_REMAP		-2
#define QFB_INIT_ERROR__LCD_NOT_INITIALIZE	-3
#define QFB_INIT_ERROR__IMG_NOT_INITIALIZE	-4
#define QFB_INIT_ERROR__FB_NOT_ALLOC		-5
#define QFB_INIT_ERROR__FB_CMAP_NOT_ALLOC	-6
#define QFB_INIT_ERROR__FB_NOT_REGIST		-7

/* frame buffer page */
#define FB_NO_A					0
#define FB_NO_B					1

/* frame buffer status */
#define IS_NODATA				0
#define NOW_WRITING_TO_FB			1
#define ALREADY_DISPLAY				2
#define ENABLE_WRITE				3

/* timer */
#define TIMER_OFF		0
#define TIMER_ON		1
#define TIMEOUT_TIME		60  //(ms)
#define FB_TIMER_STOP		0
#define FB_TIMER_START		1

/* CHK_SCRN timer */
#define RECHK_TIME		1  //(ms)

/* timer default setting */
#define DEFALT_UPDATE_TIMER	TIMER_ON

/********************************************************
 *                                                      *
 *  Structure                                           *
 *                                                      *
 *******************************************************/

// image infomation from 2D API
typedef struct fb_other_image_info{
	int  iMaskColFlg;
	int  iMaskCol;
	int  iAlpha;
	int  iInvFlg;
	int  iPage;
	int  iUpDate;
} FB_OTHER_IMAGE_INFO;

// image infomation from 2D API
typedef struct _qfb_par {
	atomic_t                 use_count;
	struct semaphore         sem_fb_image_info;		/* semafore for fb_image_info   */
	struct semaphore         sem_fb_other_info_A;		/* semafore for fb_other_info_A */
	struct semaphore         sem_fb_other_info_B;		/* semafore for fb_other_info_B */
	spinlock_t               timer_lock;			/* spin lock fb timer flag        */
	struct workqueue_struct *mp200_fb_workqueue;
	struct work_struct       wk_timeout_update_bottom_half;
	u32                      pseudo_palette[256];
} qfb_par;

// screen infomation default data
static struct fb_var_screeninfo qfb_default __initdata = 
{
	.xres           = 240,
	.yres           = 320,
	.xres_virtual   = 240,
	.yres_virtual   = 320,
	.xoffset        = 0,
	.yoffset        = 0,
	.grayscale      = 0,
	.bits_per_pixel = 16,
	.red            = {11, 5, 0},
	.green          = {5, 6, 0},
	.blue           = {0, 5, 0},
	.transp         = {0, 0, 0},
	.activate       = 0,
	.height         = -1,
	.width          = -1,
	.pixclock       = 166666,
	.left_margin    = 0,
	.right_margin   = 0,
	.upper_margin   = 0,
	.lower_margin   = 0,
	.hsync_len      = 24,
	.vsync_len      = 4,
	.sync           = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT,
	.vmode          = FB_VMODE_NONINTERLACED,
};

// screen infomation fix data
static struct fb_fix_screeninfo qfb_fix __initdata = 
{
	.id        = "MP200 FB",
	.type      = FB_TYPE_PACKED_PIXELS,
	.visual    = FB_VISUAL_TRUECOLOR,
	.xpanstep  = 1,
	.ypanstep  = 1,
	.ywrapstep = 1,
	.accel     = FB_ACCEL_NONE,
};
#endif /* _MP200_FB_H_ */
