/*
 * File Name       : /drivers/video/mp200_v4l2/mp200_lcd.c
 * Function        : LCD driver corresponding to V4L2
 * Release Version :
 * Release Date    :
 *
 * Copyright (C) NEC Electronics Corporation 2006
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

/*
 * Copyright 2006,2007 Sony Corporation. 
 */

/********************************************************
 *                                                      *
 *  Include Files                                       *
 *                                                      *
 *******************************************************/

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/tty.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <linux/timer.h>
#include <linux/sched.h>
#include <linux/device.h>

#include <asm/uaccess.h>
#include <asm/arch/hardware.h>
#include <asm/arch/gpiocore.h>
#include <asm/arch/irqs.h>
#include <asm/arch/pmu.h>
#include <asm/arch/img.h>
#include <asm/arch/mp201_mem.h>

#include "../mp200/lcdc.h"
#include "mp200_lcd.h"
#if ENABLE_FRAMECACHE
#include "../mp200/fifo.h"
#endif /* ENABLE_FRAMECACHE */
#include "mp200_common.h"

#define LCD_BL_DEFAULT_BRIGHT 94        /*2006.11.27 ita. same setting in mp200_fb.c*/

/********************************************************
 *                                                      *
 *  Variables                                           *
 *                                                      *
 *******************************************************/

/*
 * for IMG request
 */

// IMG_MODE
static  unsigned long ulImgMode;			/* img_mode */

/* input image data from v4l2 */
static  unsigned long ulSrcStarAddrYRGB;		/* (DSP) Y start address  */
static  unsigned long ulSrcStarAddrUV;			/* (DSP) UV start address */
static  unsigned int  uiSrcWidth;				/* (DSP) horizontal size  */
static  unsigned int  uiSrcHeight;				/* (DSP) Vertical size    */
static  unsigned int  uiSrcCoordinateX;			/* (DSP) X coordinate     */
static  unsigned int  uiSrcCoordinateY;			/* (DSP) Y coordinate     */
static  unsigned int  uiSrcScreenWidth;			/* (DSP) pitch size       */

/* input image data from fb driver */
static  unsigned long ulSrcStarAddrRGB_F;		/* (2D) fb image data RGB start address */
static  unsigned int  uiSrcWidth_F;				/* (2D) horizontal size                 */
static  unsigned int  uiSrcHeight_F;			/* (2D) Vertical size                   */
static  unsigned int  uiSrcCoordinateX_F;		/* (2D) X coordinate                    */
static  unsigned int  uiSrcCoordinateY_F;		/* (2D) Y coordinate                    */
static  unsigned int  uiSrcScreenWidth_F;		/* (2D) pitch size                      */
static  unsigned int  uiMixFrameBufferPageTmp;	/* (2D) name of mix frame buffer temp   */
static  unsigned int  uiMixFrameBufferPage;		/* (2D) name of mix frame buffer        */

/* output image data to LCD */
static  unsigned long ulDestStarAddrRGB;		/* (LCD) output data start address */
static  unsigned int  uiDestWidth;				/* (LCD) horizontal size           */
static  unsigned int  uiDestHeight;				/* (LCD) Vertical size             */
static  unsigned int  uiDestCoordinateX;		/* (LCD) X coordinate              */
static  unsigned int  uiDestCoordinateY;		/* (LCD) Y coordinate              */
static  unsigned int  uiDestScreenWidth;		/* (LCD) pitch size                */

/* other img infomation */
static  unsigned int  uiMaskColor;				/* mask color */
static  unsigned int  uiAlpha;					/* alpha      */
static  unsigned long ulOffsetX;				/* offset X   */
static  unsigned long ulOffsetY;				/* offset Y   */
static  unsigned long ulEndian;					/* endian     */
static  unsigned int  uiStepH;					/* h step     */
static  unsigned int  uiStepV;					/* v step     */
static  unsigned int  uiFoldH;					/* h fold     */
static  unsigned int  uiFoldV;					/* v fold     */

/* Flags */
static           int  iMixDSPFlg;				/* DSP mix ON/OFF            */
static           int  iMixDSPFlgToFB;			/* return FB DSP mix ON/OFF  */
static           int  iDSPPauseFlg;				/* DSP pause status          */
static  unsigned int  uiMaskColorFlag;			/* mask color ON/OFF Flag    */
static  unsigned int  uiInverseFlag;			/* inverse mode Flag         */
static  unsigned int  uiEndianFlag;				/* endian BIG/LITTLE Flag    */
static  unsigned int  uiYUVFormatFlag;			/* (DSP) YUV422/YUV420 flag  */
static  unsigned int  uiAbsolutelyUpFlag;		/* absolutely update flag    */
static           int  iCallbackV4L2Flg;			/* need to callback to V4L2  */
static           int  iMixImageMode;			/* mix image mode flag       */

/* LCD Disp page */
extern  enum framenum iDispPageNo;				/* now on display page      */
extern  enum framenum iChangePageNo;			/* change on display page   */

/* structure for IMG request */
struct mp200_img_req_lcd qIMG_req_lcd;			/* queue info for IMG */

/*
 * Smem
 */

/* Smem */
extern  ulong   Smem;
extern  ulong   SmemLength;
extern  char   *SmemV;

/* LCDC MMIO */
        ulong   LCDCMmio       = LCDC_BASE;
        ulong   LCDCMmioLength = LCDC_SIZE;
        char   *LCDCMmioV;

#if ENABLE_FRAMECACHE
/* FIFO MMIO */
ulong   FIFOMmio       = FIFO_BASE;
ulong   FIFOMmioLength = FIFO_SIZE;
char   *FIFOMmioV;
#endif /* ENABLE_FRAMECACHE */

/* FrameBuf(LCD) */
ulong   FrameBufA      = SMEM_START + FRAMEBUF_A_OFFSET;
ulong   FrameBufB      = SMEM_START + FRAMEBUF_B_OFFSET;
ulong   FrameBufLength = FRAMEBUF_LENGTH;
char   *FrameBufAV;
char   *FrameBufBV;

/*
 * LCD devices
 */

/* device major number */
static dev_t                dev_no    = 0;
static int                  dev_major = 0;
       char                *dev_name  = "lcd";
static char                *dev_file  = "lcd";
static struct class_simple *lcd_class;

/* LCD device status */
        MP200_LCD_DEV       lcd_dev;

/*
 * V4L2 dev data
 */

static  void *pvDevDataTmp;		/* mp200_v4l2_device temp          */
static	void *pvVideoBufTmp;	/* videobuf_buffer temp            */
static  void *pvDevData;		/* mp200_v4l2_device               */
static	void *pvVideoBuf;		/* videobuf_buffer                 */

/*
 * timer
 */
static  struct timer_list lcd_timer;
static  int               iTimerFlag;


/********************************************************
 *                                                      *
 *  Prototype declarations of function                  *
 *                                                      *
 *******************************************************/

/*
 * LCD Driver
 */
/* Public function */
int           mp200_lcd_set_v4l2_image_data(V4L2_IMAGE_INFO v4l2_image_info);
int           mp200_lcd_set_fb_image_data(FB_IMAGE_INFO *fb_image_info);
int  __init   mp200_lcd_init_module(void);
void          mp200_lcd_remove(void);
#ifdef MODULE
void __exit   mp200_lcd_exit_module(void);
#endif /* MODULE */
#ifdef CONFIG_DPM
int           mp200_lcd_suspend(struct device *dev, u32 state, u32 level);
int           mp200_lcd_resume(struct device *dev, u32 level);
#endif /* CONFIG_DPM */

/* Private function */
static  int         mix_image(int iMixMode, int iCallbackV4L2);
static  void        set_dest_image_data(void);
static  void        set_img_mode(int iMixMode);
static  void        regist_img_request(int iMixMode);
static  void        lcd_callback(MP200_IMG_REQ *pMP200_IMG_req, int iIMGRet);
        irqreturn_t lcd_irq_handler(int irq, void *dev_id, struct pt_regs *fp);	/* VSYNC interrupt handler */
static  void        set_val_init(void);
static  int         lcd_probe(void);
static  void        lcd_probe_error(int iErrorMode);
static  void        lcd_timer_init(void);
static  void        lcd_timeout(unsigned long data);


/*
 * extern FB Driver function
 */
extern  void  qfb_callback(int iWrittenFBNo, int iMixModeStatus);


/*
 * extern V4L2 Driver function
 */
extern  void  mp200_v4l2_core_lcd_callback(int iIMGRet, FRAME_DATA frame_data);


/*
 * extern IMG Driver function
 */
extern  int   mp200_img_init_module(void);
extern  void  mp200_img_preferred_regist(struct mp200_img_req_lcd *p);

/*
 * extern LCD H/W function
 */
extern  int   init_lcdhw(int brightness);
extern  int   change_frame(void);
extern  int   change_frame_when_lcdc_off(void);
extern  int   getLcdOn(void);
#ifdef MODULE
extern  void  stop_lcdhw(void);
#endif /* MODULE */
extern  void lcd_hw_int_enable(void);
extern  void lcd_hw_int_disable(void);
extern  void lcd_hw_save_reg(void);
extern  void lcd_hw_restore_reg(void);


/*
 * work queue
 */
struct workqueue_struct *mp200_lcd_workqueue;
struct work_struct       wk_timeout_bottom_half;
static void lcd_timeout_bottom_half_do(void *num);


/********************************************************
 *                                                      *
 *  Macros                                              *
 *                                                      *
 *******************************************************/

#define ImgRequest(DATA, REG)                               \
do{                                                         \
        qIMG_req_lcd.img_reg_info[index_req].reg  = (REG);  \
        qIMG_req_lcd.img_reg_info[index_req].data = (DATA); \
        index_req++;                                        \
}while(0)


/********************************************************
 *                                                      *
 *  Structure                                           *
 *                                                      *
 *******************************************************/

static struct file_operations mp200_lcd_fops = {
	.owner   = THIS_MODULE,
};

static struct cdev mp200_lcd_cdev = {
	.owner = THIS_MODULE,
	.ops = &mp200_lcd_fops,
};


/********************************************************
 *                                                      *
 *  Function Definitions                                *
 *                                                      *
 *******************************************************/

/******************************************************************************
* MODULE   : mp200_lcd_set_v4l2_image_data
* FUNCTION : set image data from v4l2 to LCD local
* RETURN   : LCD_SUCCESS(0)      : success
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/

int mp200_lcd_set_v4l2_image_data(V4L2_IMAGE_INFO v4l2_image_info)
{
	unsigned long flags;
	FRAME_DATA    frame_data;

	// Stop pause timer
	spin_lock_irqsave(&lcd_dev.timer_lock, flags);
	del_timer(&lcd_timer);
	iTimerFlag = LCD_TIMER_STOP;
	spin_unlock_irqrestore(&lcd_dev.timer_lock, flags);
	iDSPPauseFlg = PAUSE_DSP_OFF;

	//get v4l2 data semafore
	down(&lcd_dev.sem_image_data);

	// DSP stop
	if(v4l2_image_info.src_image_data.uvaddr == 0)
	{
		// movie off
		iMixDSPFlg = MIX_DSP_OFF;

		mix_image(ONLY_2D_MODE, CALLBACK_V4L2_OFF);

		return LCD_SUCCESS;
	}

	// movie on
	iMixDSPFlg   = MIX_DSP_ON;

	// DSP offset or size ERROR
	if(v4l2_image_info.dst_image_hsize == 0)
	{
		frame_data.dev = v4l2_image_info.src_frame_data.dev;
		frame_data.buf = v4l2_image_info.src_frame_data.buf;

		// call v4l2 callback function
		mp200_v4l2_core_lcd_callback(-1, frame_data);

		// release semafore
		up(&lcd_dev.sem_image_data);

		return LCD_SUCCESS;
	}

	// set input image data from v4l2
	uiSrcWidth        = v4l2_image_info.src_image_data.hsize;
	uiSrcHeight       = v4l2_image_info.src_image_data.vsize;
	ulSrcStarAddrYRGB = v4l2_image_info.src_image_data.yrgbaddr;
	ulSrcStarAddrUV   = v4l2_image_info.src_image_data.uvaddr;
	uiSrcCoordinateX  = v4l2_image_info.src_image_data.x;
	uiSrcCoordinateY  = v4l2_image_info.src_image_data.y;
	uiSrcScreenWidth  = v4l2_image_info.src_image_data.size;

	// set other infomation of input data from v4l2
	uiYUVFormatFlag   = v4l2_image_info.yuvfmt;
	uiDestWidth       = v4l2_image_info.dst_image_hsize;
	uiDestHeight      = v4l2_image_info.dst_image_vsize;
	ulOffsetX         = v4l2_image_info.offsetx;
	ulOffsetY         = v4l2_image_info.offsety;
	uiEndianFlag      = v4l2_image_info.endian;

	ulSrcStarAddrYRGB = ulSrcStarAddrYRGB + uiSrcCoordinateX + uiSrcCoordinateY * uiSrcScreenWidth;
	if (uiYUVFormatFlag == V4L2_YUV420_FORMAT)
	{
		ulSrcStarAddrUV   = ulSrcStarAddrUV + uiSrcCoordinateX + uiSrcCoordinateY * uiSrcScreenWidth / 2;
	}
	else // (uiYUVFormatFlag == V4L2_YUV422_FORMAT)
	{
		ulSrcStarAddrUV   = ulSrcStarAddrUV + uiSrcCoordinateX + uiSrcCoordinateY * uiSrcScreenWidth;
	}

	// set frame data from V4L2
	pvDevDataTmp      = v4l2_image_info.src_frame_data.dev;
	pvVideoBufTmp     = v4l2_image_info.src_frame_data.buf;

	//mix - image data from v4l2 and image data from fb driver (DSP and 2D)
	mix_image(DSP_AND_2D_MODE, CALLBACK_V4L2_ON);

	// start PAUSE timer
	spin_lock_irqsave(&lcd_dev.timer_lock, flags);
	if(iTimerFlag == LCD_TIMER_STOP)
	{
		lcd_timer.expires = jiffies + MIX_DSP_TIMEOUT;
		add_timer(&lcd_timer);
		iTimerFlag = LCD_TIMER_START;
	}
	spin_unlock_irqrestore(&lcd_dev.timer_lock, flags);

	return LCD_SUCCESS;
}

/******************************************************************************
* MODULE   : mp200_lcd_set_fb_image_data
* FUNCTION : set image data from Frame Buffer to LCD local
* RETURN   : LCD_SUCCESS(0)      : success
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/

int mp200_lcd_set_fb_image_data(FB_IMAGE_INFO *fb_image_info)
{
	//get data semafore
	if(down_interruptible(&lcd_dev.sem_image_data))
		return -1;

	//set input image data from fb driver
	uiSrcWidth_F         = fb_image_info->src_image_data.hsize;
	uiSrcHeight_F        = fb_image_info->src_image_data.vsize;
	ulSrcStarAddrRGB_F   = fb_image_info->src_image_data.yrgbaddr;
	uiSrcCoordinateX_F   = fb_image_info->src_image_data.x;
	uiSrcCoordinateY_F   = fb_image_info->src_image_data.y;
	uiSrcScreenWidth_F   = fb_image_info->src_image_data.size;
	ulSrcStarAddrRGB_F   = ulSrcStarAddrRGB_F +
	                       uiSrcCoordinateX_F * BITS_PER_PIXEL / 8 +
	                       uiSrcCoordinateY_F * uiSrcScreenWidth_F;

	//Set other infomation of input data from fb
	uiMaskColor             = fb_image_info->maskcolr;
	uiMaskColorFlag         = fb_image_info->maskcolrflg;
	uiAlpha                 = fb_image_info->alpha;
	uiInverseFlag           = fb_image_info->invflg;
	uiMixFrameBufferPageTmp = fb_image_info->mix_buf_page;
	uiAbsolutelyUpFlag      = fb_image_info->update_flag;

	// check update flag
	switch(uiAbsolutelyUpFlag)
	{
		case FB_UPDATE_ON:  //    ioctl(QFB_UPDATE_SCRN) & update = 0
		                    // or interval update
		{
			// movie on
			if(iMixDSPFlg == MIX_DSP_ON)
			{
				// pause status
				if (iDSPPauseFlg == PAUSE_DSP_ON)
				{
					mix_image(DSP_AND_2D_MODE, CALLBACK_V4L2_OFF);
				}
				// not pause status
				else
				{
					// release data semafore
					up(&lcd_dev.sem_image_data);
				}
			}
			// movie off
			else
			{
				mix_image(ONLY_2D_MODE, CALLBACK_V4L2_OFF);
			}
			break;
		}
		case FB_ABSOLUTERY_UPDATE:  // ioctl(QFB_UPDATE_SCRN) & update = 1
		{
			// movie on
			if(iMixDSPFlg == MIX_DSP_ON)
			{
				mix_image(DSP_AND_2D_MODE, CALLBACK_V4L2_OFF);
			}
			// movie off
			else
			{
				mix_image(ONLY_2D_MODE, CALLBACK_V4L2_OFF);
			}
			break;
		}
		case FB_UPDATE_OFF:  // ioctl(QFB_SET_MODES)
		default           :  // other
		{
			// release data semafore
			up(&lcd_dev.sem_image_data);
			break;
		}
	}
	return LCD_SUCCESS;
}


/******************************************************************************
* MODULE   : mix_image
* FUNCTION : By using IMG, mix 2D (and DSP) image
* RETURN   : LCD_SUCCESS(0)      : success
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/

static int mix_image(int iMixMode, int iCallbackV4L2)
{
	// get flag semafore
	down(&lcd_dev.sem_image_flag);

	// set flag and temp data
	iMixImageMode        = iMixMode;
	iCallbackV4L2Flg     = iCallbackV4L2;
	pvDevData            = pvDevDataTmp;
	pvVideoBuf           = pvVideoBufTmp;
	uiMixFrameBufferPage = uiMixFrameBufferPageTmp;
	iMixDSPFlgToFB       = iMixDSPFlg;

	//set dest image address
	set_dest_image_data();

	//set img_mode
	set_img_mode(iMixImageMode);

	//regist img request to IMG Driver
	regist_img_request(iMixImageMode);

	//preferred regist
	mp200_img_preferred_regist((struct mp200_img_req_lcd *)(&qIMG_req_lcd));

	// release data semafore
	up(&lcd_dev.sem_image_data);

	return LCD_SUCCESS;
}


/******************************************************************************
* MODULE   : set_dest_image_data
* FUNCTION : When DSP_AND_2D_MODE, set dest data
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/

void set_dest_image_data()
{
	//Display A -> set FrameBufB
	if(iDispPageNo == DISPLAY_FRAME_NO_A)
	{
		ulDestStarAddrRGB =  FrameBufB +
		                    (uiDestCoordinateX * BITS_PER_PIXEL / 8 ) +
		                    (uiDestCoordinateY * FRONT_WIDTH * BITS_PER_PIXEL / 8);
	}
	//Display B -> set FrameBufA
	else
	{
		ulDestStarAddrRGB =  FrameBufA +
		                    (uiDestCoordinateX * BITS_PER_PIXEL / 8 ) +
		                    (uiDestCoordinateY * FRONT_WIDTH * BITS_PER_PIXEL / 8);
	}

	return;
}

/******************************************************************************
* MODULE   : set_img_mode
* FUNCTION : set IMG_MODE
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/

void set_img_mode(int iMixMode)
{
    /*modified by itawaki 2006.09.14 for RGB666*/
    //ulImgMode = 0x00000000;
	ulImgMode = 0x00000030; /*PixelPack=1 & dither on YUV.*/
    //ulImgMode = 0x00000020; /*PixelPack=1 */
	switch(iMixMode)
	{
		case DSP_AND_2D_MODE:
		{
			// OP_MODE:18  FRONT:RGB565
			ulImgMode |= (IMG_OP_MODE_18_BIT | IMG_SRCFMT_F_BIT);

			if(uiYUVFormatFlag == V4L2_YUV420_FORMAT)
			{
				// YUV420
				ulImgMode |= IMG_SRCFMT_B_BIT;
			}
			break;
		}
		case ONLY_2D_MODE:
		{
			// OP_MODE:00  Back:RGB565
			ulImgMode |= (IMG_OP_MODE_00_BIT | IMG_SRCFMT_B_BIT);
			break;
		}
	}

	// DEST:RGB565
	ulImgMode |= IMG_DSTFMT_BIT;

	if(uiMaskColorFlag == FB_MASK_COLOR_DISP_OFF)
	{
		//vanish mask color
		ulImgMode |= IMG_MASK_COL_BIT;
	}

	if(uiInverseFlag == UDRL_INVERSE)
	{
		// up down right left inverse (180 rotate)
		ulImgMode |= (IMG_UDINV_BIT | IMG_RLINV_BIT );
	}

	return;
}


/******************************************************************************
* MODULE   : regist_img_request
* FUNCTION : regist data for IMG request
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/

void regist_img_request(int iMixMode)
{
	int index_req = 0;

	// IMG_MODE
	ImgRequest(ulImgMode,            IMG_MODE);

	if(iMixMode == DSP_AND_2D_MODE)
	{
		// input image data from v4l2
		ImgRequest(ulSrcStarAddrYRGB,   IMG_SRCYRGBADR_B);
		ImgRequest(ulSrcStarAddrUV,     IMG_SRCUVADR_B);
		ImgRequest(uiSrcScreenWidth,    IMG_SRCSIZE_B);
		ImgRequest(uiSrcWidth,          IMG_SRCHSIZE_B);
		ImgRequest(uiSrcHeight,         IMG_SRCVSIZE_B);

		// input image data from fb
		ImgRequest(ulSrcStarAddrRGB_F, IMG_SRCYRGBADR_F);
		ImgRequest(0,                  IMG_SRCUVADR_F);
		ImgRequest(uiSrcScreenWidth_F, IMG_SRCSIZE_F);
		ImgRequest(uiSrcWidth_F,       IMG_SRCHSIZE_F);
		ImgRequest(uiSrcHeight_F,      IMG_SRCVSIZE_F);
	}
	else // (iMixMode == ONLY_2D_MODE)
	{
		// input image data from fb
		ImgRequest(ulSrcStarAddrRGB_F, IMG_SRCYRGBADR_B);
		ImgRequest(0,                  IMG_SRCUVADR_B);
		ImgRequest(uiSrcScreenWidth_F, IMG_SRCSIZE_B);
		ImgRequest(uiSrcWidth_F,       IMG_SRCHSIZE_B);
		ImgRequest(uiSrcHeight_F,      IMG_SRCVSIZE_B);
	}

	// output image data to LCD
	ImgRequest(ulDestStarAddrRGB,    IMG_DSTYRGBADR);
	ImgRequest(0x00000000,           IMG_DSTUVADR);
	ImgRequest(uiDestScreenWidth,    IMG_DSTSIZE);

	if(iMixMode == DSP_AND_2D_MODE)
	{
		ImgRequest(uiDestWidth,  IMG_DSTHSIZE);
		ImgRequest(uiDestHeight, IMG_DSTVSIZE);


		// offset
		ImgRequest(ulOffsetX,    IMG_OFFSETX_F );
		ImgRequest(ulOffsetY,    IMG_OFFSETY_F );

		// maskcolor
		ImgRequest(uiMaskColor,  IMG_MASKCOLR_F );
		ImgRequest(uiAlpha,      IMG_ALPHA_F );

		// step
		uiStepH = uiSrcWidth   * 256 / uiDestWidth;
		uiStepV = uiSrcHeight  * 256 / uiDestHeight;
		ImgRequest(uiStepH,      IMG_HSTEP);
		ImgRequest(uiStepV,      IMG_VSTEP);

		// fold
		uiFoldH = uiDestWidth  * 64 / uiSrcWidth;
		uiFoldV = uiDestHeight * 64 / uiSrcHeight;
		ImgRequest(uiFoldH,      IMG_HFOLD);
		ImgRequest(uiFoldV,      IMG_VFOLD);

		// endian
		ImgRequest(ulEndian,     IMG_PEL_ENDIAN);
	}

	// IMG_REQ
	ImgRequest(IMG_REQ_BIT, IMG_REQ);

	// set lastLayer
	qIMG_req_lcd.lastLayer = 1;

	// regist  callback function
	qIMG_req_lcd.callback = (void *)lcd_callback;

	return;
}


/******************************************************************************
* MODULE   : lcd_callback
* FUNCTION : changing frame and callback to fb and v4l2
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.05.26 H.Nagashima
* UPDATE   :
******************************************************************************/
extern int icx1087_get_bl_brightness(void); /* 2006.08.07 itawaki*/

static void lcd_callback(MP200_IMG_REQ *pMP200_IMG_req, int iIMGRet)
{
	FRAME_DATA frame_data;
	unsigned long flags;
	spin_lock_irqsave(&lcd_dev.callback_lock, flags);

	// if IMG success, change frame execute
	if((iIMGRet >= IMG_CALLBACK_SUCCESS)
       && (getLcdOn()))   /* 2006.12.11 itawaki*/
        //&& (icx1087_get_bl_brightness() != 0)) /* 2006.08.07 itawaki*/
	{
		// enable LCDC VSYNC interrupt
		lcd_hw_int_enable();

		//frame change
		change_frame();
	}
	else
	{
		// call FB Driver function "qfb_callback"
		qfb_callback((int)uiMixFrameBufferPage, iMixDSPFlgToFB);

		// when kick mix_image by V4L2
		if(iCallbackV4L2Flg == CALLBACK_V4L2_ON)
		{
			iCallbackV4L2Flg = CALLBACK_V4L2_OFF;
			// set frame date to V4L2
			frame_data.dev = pvDevData;
			frame_data.buf = pvVideoBuf;

			//call v4l2 callback function
			mp200_v4l2_core_lcd_callback(iIMGRet, frame_data);
		}
		//frame change
		change_frame_when_lcdc_off(); /*2006.08.07 itawaki*/
		// release semafore
		up(&lcd_dev.sem_image_flag);
	}
	spin_unlock_irqrestore(&lcd_dev.callback_lock, flags);
	return;
}


/*****************************************************************************
* MODULE   : lcd_irq_handler
* FUNCTION : LCDC interrupt handler
* RETURN   : IRQ_HANDLED
* NOTE     : none
* CREATE   : 2006/07/14 Y.Utsuhara
* UPDATE   :
******************************************************************************/
irqreturn_t lcd_irq_handler(int irq, void *dev_id, struct pt_regs *fp)
{
	FRAME_DATA frame_data;

	/*  check Interuppt Status */
	unsigned long ulRegVal32 = readl(LCDCMmioV + LCD_INTSTATUS);

	if ((ulRegVal32 & LCD_LCDVS_BIT) == LCD_LCDVS_BIT)
	{
		// clear the cause of interrupt
		writel(LCD_LCDVSCLR_BIT, LCDCMmioV + LCD_INTFFCLR);

		// check AREASTATUS
		ulRegVal32 = readl(LCDCMmioV + LCD_FRAMESEL);

		if (((iChangePageNo == DISPLAY_FRAME_NO_A) && (((ulRegVal32 & LCD_AREASTATUS_BIT) >> 2) == LCD_FLAME_A)) ||
		    ((iChangePageNo == DISPLAY_FRAME_NO_B) && (((ulRegVal32 & LCD_AREASTATUS_BIT) >> 2) == LCD_FLAME_B)))
		{
			// disable LCDC interrupts
			lcd_hw_int_disable();

			// call FB Driver function "qfb_callback"
			qfb_callback((int)uiMixFrameBufferPage, iMixDSPFlgToFB);

			// when kick mix_image by V4L2
			if(iCallbackV4L2Flg == CALLBACK_V4L2_ON)
			{
				iCallbackV4L2Flg = CALLBACK_V4L2_OFF;
				// set frame date to V4L2
				frame_data.dev = pvDevData;
				frame_data.buf = pvVideoBuf;

				//call v4l2 callback function
				mp200_v4l2_core_lcd_callback(IMG_CALLBACK_SUCCESS, frame_data);
			}

			// release semafore
			up(&lcd_dev.sem_image_flag);
		}
	}

	return IRQ_HANDLED;
}


/********************************************************
 *                                                      *
 *  Init Function Definitions                           *
 *                                                      *
 *******************************************************/

/******************************************************************************
* MODULE   : mp200_lcd_init_module
* FUNCTION : initialize LCD Driver
* RETURN   : LCD_SUCCESS(0) : success
*            other          : fail
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/

int __init mp200_lcd_init_module(void)
{
	int            iRet;
    /* 2006.06.05 ish */
    printk(KERN_INFO " @lcd: lcd_init: called \n");
    
	// regist lcd device
	if (dev_major)
	{
		dev_no = MKDEV(dev_major, 0);
		iRet   = register_chrdev_region(dev_no, 1, dev_name);

	}
	else
	{
		iRet      = alloc_chrdev_region(&dev_no, 0, 1, dev_name);
		dev_major = MAJOR(dev_no);
	}
	if (iRet < 0)
	{
		printk(KERN_WARNING " @lcd: lcd_init: device registration error\n");
		return iRet;
	}

	// cdev initialize
	cdev_init(&mp200_lcd_cdev, &mp200_lcd_fops);

	// add cdev
	iRet = cdev_add(&mp200_lcd_cdev, dev_no, 1);
	if (iRet)
	{
		printk(KERN_WARNING " @lcd: lcd_init: device add error\n");
		unregister_chrdev_region(dev_no, 1);
		return iRet;
	}

	// create lcd class
	lcd_class = class_simple_create(THIS_MODULE, dev_file);
	if (IS_ERR(lcd_class))
	{
		printk(KERN_ERR " @lcd: lcd_init: creating lcd class error\n");
		cdev_del(&mp200_lcd_cdev);
		unregister_chrdev_region(dev_no, 1);
		return iRet;
	}
	class_simple_device_add(lcd_class, dev_no, NULL, dev_file);
	devfs_mk_cdev(dev_no, S_IFCHR | S_IRUGO | S_IWUGO, dev_file);

	// lcd device memset
	memset(&lcd_dev, 0, sizeof(MP200_LCD_DEV));

	// semafore initialize
	sema_init(&lcd_dev.sem_image_data, 1);
	sema_init(&lcd_dev.sem_image_flag, 1);

	// spin lock initialize
	spin_lock_init(&lcd_dev.timer_lock);
	spin_lock_init(&lcd_dev.callback_lock);

	//call lcd_probe
	iRet = lcd_probe();
	if(iRet != LCD_SUCCESS)
	{
		printk(KERN_ERR " @lcd: lcd_init: lcd probe error\n");
		lcd_probe_error(iRet);
		return iRet;
	}

	// variable initialized
	set_val_init();

	// timer initiarize
	lcd_timer_init();

	return LCD_SUCCESS;
}

/******************************************************************************
* MODULE   : lcd_probe
* FUNCTION : LCD Driver probe
* RETURN   : LCD_SUCCESS(0) : success
*            other          : fail
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/
#ifdef CONFIG_SONY_ICX1087
extern int qfb_get_lcd_disable(int *);
#endif /* CONFIG_SONY_ICX1087 */

static int lcd_probe()
{
#ifdef CONFIG_SONY_ICX1087
    int bl;
#endif
	int iRet;

	//reserve LCDC Mmio region
	if (!request_mem_region(LCDCMmio, LCDCMmioLength, "mp200_lcd"))
	{
		printk(KERN_WARNING " @lcd: lcd probe: cannnot request LCDC mem region\n");
		return LCD_INIT_ERROR__LCDC_MMIO_NOT_RESERVE;
	}
	LCDCMmioV = ioremap_nocache(LCDCMmio, LCDCMmioLength);
	if (!LCDCMmioV)
	{
		printk(KERN_WARNING " @lcd: lcd probe: cannot ioremap_nocache LCDC mmio\n");
		return LCD_INIT_ERROR__LCDC_MMIO_NOT_REMAP;
	}
	printk(KERN_WARNING " @lcd: LCDC mmio %ld KB  @ 0x%lx\n", LCDCMmioLength >> 10, LCDCMmio);

#if ENABLE_FRAMECACHE
	/* FIFO Mmio  reserve & remap*/
	if (!request_mem_region(FIFOMmio, FIFOMmioLength, "mp200_lcd"))
	{
		printk(KERN_WARNING " @lcd: lcd probe: cannot reserve FIFO mmio\n");
		return LCD_INIT_ERROR__FIFO_MMIO_NOT_RESERVE;
	}
	FIFOMmioV = ioremap_nocache(FIFOMmio, FIFOMmioLength);
	if (!FIFOMmioV)
	{
		printk(KERN_WARNING " @lcd: lcd probe: cannot ioremap_nocache FIFO mmio\n");
		return LCD_INIT_ERROR__FIFO_MMIO_NOT_REMAP;
	}
	printk(KERN_WARNING " @lcd: FIFO mmio %ld KB  @ 0x%lx\n", FIFOMmioLength >> 10, FIFOMmio);
#endif /* ENABLE_FRAMECACHE */

	// pointer to buffer(LCDC)
	FrameBufAV = SmemV + FRAMEBUF_A_OFFSET;
	FrameBufBV = SmemV + FRAMEBUF_B_OFFSET;

	// initiarize LCD H/W
    // with backlight brightness 100%
#ifdef CONFIG_SONY_ICX1087
    if(qfb_get_lcd_disable(&bl)) /*when lcd disable*/
        iRet = init_lcdhw(bl); /* turn off or on at command brightness*/
    else                      /* when lcd enable*/
        iRet = init_lcdhw(LCD_BL_DEFAULT_BRIGHT); /*turn on*/
#else
    iRet = init_lcdhw(LCD_BL_DEFAULT_BRIGHT);
#endif /* CONFIG_SONY_ICX1087 */

	if (iRet != LCD_SUCCESS)
	{
		printk(KERN_WARNING " @lcd: lcd probe: cannot initialize LCD H/W\n");
		return iRet;
	}

	// initiarize work queue
	mp200_lcd_workqueue = create_singlethread_workqueue("mp200_lcd");
	INIT_WORK(&wk_timeout_bottom_half, lcd_timeout_bottom_half_do, NULL);

	return LCD_SUCCESS;
}


/******************************************************************************
* MODULE   : lcd_probe_error
* FUNCTION : When fail in lcd_probe, remove lcd resource
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/

static void lcd_probe_error(int iErrorMode)
{
	switch(iErrorMode)
	{
		case    LCD_INIT_ERROR__LCD_NOT_POWER_ON:	free_irq(INT_LCD, NULL);
		case    LCD_INIT_ERROR__LCD_IRQ_NOT_REQUEST:	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCD);
								mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCDPCLK);
								mp200_pmu_clkctrl_off(MP200_CLKCTRL_TFDPCLK);
								mp200_pmu_reset_device(MP200_RESETDEVICE_TFD);
								mp200_pmu_reset_device(MP200_RESETDEVICE_LCD);
								mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_CLK);
								mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_PCLK);
								mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_LCLK);
								mp200_pmu_close_clockgate(MP200_CLOCKGATE_TFD_PCLK);
								mp200_pmu_close_clockgate(MP200_CLOCKGATE_TFD_LCLK);
#if ENABLE_FRAMECACHE
								iounmap(FIFOMmioV);
		case    LCD_INIT_ERROR__FIFO_MMIO_NOT_REMAP:	release_mem_region(FIFOMmio, FIFOMmioLength);
		case    LCD_INIT_ERROR__FIFO_MMIO_NOT_RESERVE:	iounmap(LCDCMmioV);
#else /* !(ENABLE_FRAMECACHE) */
								iounmap(LCDCMmioV);
#endif /* ENABLE_FRAMECACHE */
		case    LCD_INIT_ERROR__LCDC_MMIO_NOT_REMAP:	release_mem_region(LCDCMmio, LCDCMmioLength);
		case    LCD_INIT_ERROR__LCDC_MMIO_NOT_RESERVE:	devfs_remove(dev_file);
								class_simple_device_remove(dev_no);
								class_simple_destroy(lcd_class);
								cdev_del(&mp200_lcd_cdev);
								unregister_chrdev_region(dev_no, 1);
		default :					break;
	}

	return;
}


/******************************************************************************
* MODULE   : set_val_init
* FUNCTION : variable default set
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/

static void set_val_init(void)
{
	// IMG MODE
	ulImgMode            = 0x0000000;

	/* input image data from V4L2 driver */
	ulSrcStarAddrYRGB    = 0x00000000;
	ulSrcStarAddrUV      = 0x00000000;
	uiSrcWidth           = 240;
	uiSrcHeight          = 320;
	uiSrcCoordinateX     = 0;
	uiSrcCoordinateY     = 0;
	uiSrcScreenWidth     = 240;

	/* input image data from fb driver */
	ulSrcStarAddrRGB_F   = SMEM_START + DISPBUF_A_OFFSET;
	uiSrcWidth_F         = 240;
	uiSrcHeight_F        = 320;
	uiSrcCoordinateX_F   = 0;
	uiSrcCoordinateY_F   = 0;
	uiSrcScreenWidth_F   = 480;
	uiMixFrameBufferPage = DISPLAY_FRAME_NO_A;

	/* output image data to LCD */
	ulDestStarAddrRGB    = SMEM_START + FRAMEBUF_A_OFFSET;
	uiDestWidth          = 240;
	uiDestHeight         = 320;
	uiDestCoordinateX    = 0;
	uiDestCoordinateY    = 0;
    /*modified by itawaki 2006.09.14 for RGB666*/
	//uiDestScreenWidth    = FRONT_WIDTH * BITS_PER_PIXEL / 8;
    uiDestScreenWidth    = FRONT_WIDTH * BITS_PER_PIXEL_OUT / 8; 

	/* other img infomation */
	uiMaskColor          = 0x0020;
	uiAlpha              = 0xFF;
	ulOffsetX            = 0;
	ulOffsetY            = 0;
	ulEndian             = V4L2_LITTLE_ENDIAN;
	uiStepH              = 256;
	uiStepV              = 256;
	uiFoldH              = 64;
	uiFoldV              = 64;

	/* flags */
	iMixDSPFlg           = MIX_DSP_OFF;
	iMixDSPFlgToFB       = MIX_DSP_OFF;
	iDSPPauseFlg         = PAUSE_DSP_OFF;
	uiMaskColorFlag      = FB_MASK_COLOR_DISP_OFF;
	uiInverseFlag        = NO_INVERSE;
	uiEndianFlag         = V4L2_LITTLE_ENDIAN;
	uiYUVFormatFlag      = V4L2_YUV420_FORMAT;
	uiAbsolutelyUpFlag   = FB_UPDATE_OFF;
	iCallbackV4L2Flg     = CALLBACK_V4L2_OFF;

	/* frame data */
	pvDevData            = 0x00;
	pvVideoBuf           = 0x00;

	return;
}


/********************************************************
 *                                                      *
 *  Timer Function Definitions                          *
 *                                                      *
 *******************************************************/

/******************************************************************************
* MODULE   : lcd_timer_init
* FUNCTION : timer function init
* RETURN   : none
* NOTE     : private function
* CREATE   : 2006.06.07 H.Nagashima
* UPDATE   :
******************************************************************************/

static void lcd_timer_init(void)
{
	init_timer(&lcd_timer);
	lcd_timer.function = lcd_timeout;
	lcd_timer.data = 0;
	iTimerFlag = LCD_TIMER_STOP;

	return;
}


/******************************************************************************
* MODULE   : lcd_timeout
* FUNCTION : when call qfb_call_timer
* RETURN   : none
* NOTE     : public function
* CREATE   : 2006.06.07 H.Nagashimia
* UPDATE   :
******************************************************************************/

static void lcd_timeout(unsigned long data)
{
	// change LCD status to PAUSE
	iDSPPauseFlg = PAUSE_DSP_ON;

	queue_work(mp200_lcd_workqueue, &wk_timeout_bottom_half);
	return;
}


/******************************************************************************
* MODULE   : lcd_timeout_bottom_half_do
* FUNCTION : when call qfb_call_timer
* RETURN   : none
* NOTE     : public function
* CREATE   : 2006.07.21 M.Haguro
* UPDATE   :
******************************************************************************/

static void lcd_timeout_bottom_half_do(void *num)
{
	//get semafore
	if(down_trylock(&lcd_dev.sem_image_data))
		return ;

	// mix DSP and 2D
	mix_image(DSP_AND_2D_MODE, CALLBACK_V4L2_OFF);

	return;
}


/********************************************************
 *                                                      *
 *  Exit Function Definitions                           *
 *                                                      *
 *******************************************************/

#ifdef MODULE
/******************************************************************************
* MODULE   : mp200_lcd_exit_module
* FUNCTION : cleanup LCD module
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/

void __exit mp200_lcd_exit_module()
{
	stop_lcdhw();
	mp200_lcd_remove();
	platform_device_unregister(&lcd_device);
	driver_unregister(&lcd_driver);
}
#endif /* MODULE */


/******************************************************************************
* MODULE   : mp200_lcd_remove
* FUNCTION : release LCD resource
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.05.29 H.Nagashima
* UPDATE   :
******************************************************************************/

void mp200_lcd_remove()
{
	unsigned int lcd_write_data;
	unsigned int lcd_write_mask;

	// power off LCD
	lcd_write_mask = (unsigned int)((1 << PWC_INT_LED_ON) | (1 << PWC_INT_LCD_OE));
	lcd_write_data = 0;
	gpiocore_pwc_write(GPIOCORE_PWC_IOOUT2, lcd_write_data, lcd_write_mask);

	free_irq(INT_LCD, NULL);
#if ENABLE_FRAMECACHE
	iounmap(FIFOMmioV);
	release_mem_region(FIFOMmio, FIFOMmioLength);
#endif /* ENABLE_FRAMECACHE */
	iounmap(LCDCMmioV);
	release_mem_region(LCDCMmio, LCDCMmioLength);

	/* Reset LCD */
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCD);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCDPCLK);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_TFDPCLK);
	mp200_pmu_reset_device(MP200_RESETDEVICE_TFD);
	mp200_pmu_reset_device(MP200_RESETDEVICE_LCD);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_CLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_PCLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_LCLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_TFD_PCLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_TFD_LCLK);

	// release resource
	devfs_remove(dev_file);
	class_simple_device_remove(dev_no);
	class_simple_destroy(lcd_class);
	cdev_del(&mp200_lcd_cdev);
	unregister_chrdev_region(dev_no, 1);

	return;
}

#ifdef CONFIG_DPM
/********************************************************
 *                                                      *
 *  Suspend/Resume Function Definitions                 *
 *                                                      *
 *******************************************************/

/******************************************************************************
* MODULE   : mp200_lcd_suspend
* FUNCTION : suspend LCD driver
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.08.15 H.Nagashima
* UPDATE   :
******************************************************************************/

int mp200_lcd_suspend(struct device *dev, u32 state, u32 level)
{
/*2006.09.08 ita */
#ifndef CONFIG_SONY_ICX1087
	unsigned int   lcd_write_data;
	unsigned int   lcd_write_mask;
#endif /*CONFIG_SONY_ICX1087*/

	switch (level)
	{
		case SUSPEND_DISABLE:
		{
/*2006.09.08 ita*/
#ifndef CONFIG_SONY_ICX1087
			// LCD timer stop
			del_timer(&lcd_timer);

			// BackLight OFF
			lcd_write_mask = (unsigned int)(1 << PWC_INT_LED_ON);
			lcd_write_data = 0;
			gpiocore_pwc_write(GPIOCORE_PWC_IOOUT2, lcd_write_data, lcd_write_mask);
#endif /*CONFIG_SONY_ICX1087*/

			break;
		}
		case SUSPEND_SAVE_STATE:
		{
			// Save LCD H/W register data
			/*lcd_hw_save_reg(); comment out by ita. 2006.08.25*/
			
			// wait for LCD disp off
			mdelay(20);

			break;
		}
		case SUSPEND_POWER_DOWN:
		{
/*2006.09.08 ita*/
#ifndef CONFIG_SONY_ICX1087
			// LCD POWER OFF
			lcd_write_mask = (unsigned int)(1 << PWC_INT_LCD_OE);
			lcd_write_data = 0;
			gpiocore_pwc_write(GPIOCORE_PWC_IOOUT2, lcd_write_data, lcd_write_mask);

			/* Reset LCD */
			mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCD);
			mp200_pmu_clkctrl_off(MP200_CLKCTRL_TFDPCLK);
			mp200_pmu_reset_device(MP200_RESETDEVICE_TFD);
			mp200_pmu_reset_device(MP200_RESETDEVICE_LCD);
			mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_CLK);
			mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_PCLK);
			mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_LCLK);
			mp200_pmu_close_clockgate(MP200_CLOCKGATE_TFD_PCLK);
			mp200_pmu_close_clockgate(MP200_CLOCKGATE_TFD_LCLK);
#endif /*CONFIG_SONY_ICX1087*/

			break;
		}
		default:
		{
			break;
		}
	}

	return LCD_SUCCESS;
}


/******************************************************************************
* MODULE   : mp200_lcd_resume
* FUNCTION : suspend LCD driver
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.08.15 H.Nagashima
* UPDATE   :
******************************************************************************/

int mp200_lcd_resume(struct device *dev, u32 level)
{
/*2006.09.08 ita*/
#ifndef CONFIG_SONY_ICX1087
	int          iRet;

	unsigned int lcd_write_data;
	unsigned int lcd_write_mask;
#endif /*CONFIG_SONY_ICX1087*/

	switch (level)
	{
		case RESUME_POWER_ON:
		{
/*2006.09.08 ita*/
#ifndef CONFIG_SONY_ICX1087
			// LCD POWER ON
			lcd_write_mask = (unsigned int)(1 << PWC_INT_LCD_OE);
			lcd_write_data = lcd_write_mask;
			iRet = gpiocore_pwc_write(GPIOCORE_PWC_IOOUT2, lcd_write_data, lcd_write_mask);
			if (iRet < 0)
			{
				printk(KERN_ERR " @lcd: init_lcdhw: cannot use gpiocore:%x\n", iRet);
				return LCD_INIT_ERROR__LCD_NOT_POWER_ON;
			}
#endif /*CONFIG_SONY_ICX1087*/
			break;
		}
		case RESUME_RESTORE_STATE:
		{
			// Restore LCD H/W register data
			/*lcd_hw_restore_reg(); comment out by itawaki.2006.08.25*/

			break;
		}
		case RESUME_ENABLE:
		{
/*2006.09.08 ita*/
#ifndef CONFIG_SONY_ICX1087
			// BackLight ON
			lcd_write_mask = (unsigned int)(1 << PWC_INT_LED_ON);
			lcd_write_data = lcd_write_mask;
			iRet = gpiocore_pwc_write(GPIOCORE_PWC_IOOUT2, lcd_write_data, lcd_write_mask);
			if (iRet < 0)
			{
				printk(KERN_ERR " @lcd: init_lcdhw: cannot use gpiocore:%x\n", iRet);
				return LCD_INIT_ERROR__LCD_NOT_POWER_ON;
			}

			// LCD timer start
			if((iTimerFlag == LCD_TIMER_START) && (iMixDSPFlg == MIX_DSP_ON))
			{
				lcd_timer.expires = jiffies + MIX_DSP_TIMEOUT;
				add_timer(&lcd_timer);
			}

#endif /*CONFIG_SONY_ICX1087*/
			break;
		}
		default:
		{
			break;
		}
	}

	return LCD_SUCCESS;
}


#endif /* CONFIG_DPM */

/* module alias */
#ifdef MODULE
module_init(mp200_lcd_init_module);
module_exit(mp200_lcd_exit_module);
#else
#ifndef CONFIG_FB_MP200_V4L2
/* 2006.06.06 ish : module_init() should not be called 
module_init(mp200_lcd_init_module);
*/
#endif
#endif

/* EXPORT_SYMBOL */
EXPORT_SYMBOL(mp200_lcd_set_v4l2_image_data);
EXPORT_SYMBOL(mp200_lcd_set_fb_image_data);
EXPORT_SYMBOL(mp200_lcd_init_module);
#ifdef CONFIG_FB_MP200_V4L2
EXPORT_SYMBOL(mp200_lcd_remove);
#endif
#ifdef MODULE
EXPORT_SYMBOL(mp200_lcd_exit_module);
#endif /* MODULE */
#ifdef CONFIG_DPM
EXPORT_SYMBOL(mp200_lcd_suspend);
EXPORT_SYMBOL(mp200_lcd_resume);
#endif /* CONFIG_DPM */
