/*
 * File Name       : /drivers/video/mp200_v4l2/mp200_lcd.h
 * Function        : LCD Driver corresponding to V4L2 only definitions
 * Release Version :
 * Release Date    :
 *
 * Copyright (C) NEC Electronics Corporation 2006
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

/*
 * Copyright 2006,2007 Sony Corporation. 
 */

#ifndef _MP200_LCD_H_
#define _MP200_LCD_H_

/********************************************************
 *                                                      *
 *  Definitions                                         *
 *                                                      *
 *******************************************************/

/*
 * Flags
 */
// DSP mix ON/OFF
#define MIX_DSP_OFF			0
#define MIX_DSP_ON			1
// DSP pause status
#define PAUSE_DSP_OFF		0
#define PAUSE_DSP_ON		1
// callback to V4L2 status
#define CALLBACK_V4L2_OFF	0
#define CALLBACK_V4L2_ON	1
// display frame
#define DISPLAY_FRAME_NO_A	0
#define DISPLAY_FRAME_NO_B	1

/*
 * FIFO
 */
#if ENABLE_FRAMECACHE
#define FIF_FCCTRL_VAL		0x0000009f
#endif /* ENABLE_FRAMECACHE */


/*
 * Error type
 */
#define LCD_SUCCESS								0
#define LCD_INIT_ERROR__LCDC_MMIO_NOT_RESERVE	-1
#define LCD_INIT_ERROR__LCDC_MMIO_NOT_REMAP		-2
#define LCD_INIT_ERROR__FIFO_MMIO_NOT_RESERVE	-3
#define LCD_INIT_ERROR__FIFO_MMIO_NOT_REMAP		-4
#define LCD_INIT_ERROR__LCD_IRQ_NOT_REQUEST		-5
#define LCD_INIT_ERROR__LCD_NOT_POWER_ON		-6


/*
 * LCD H/W define
 */

#ifdef CONFIG_SONY_ICX1087  /* 2006.05.15 ish */

#if ENABLE_FRAMECACHE
#define LCD_CONTROL_VAL				0x00000040
#else				/* ENABLE_FRAMECACHE */
/*modified by ita 2006.09.14 for RGB666*/
//#define LCD_CONTROL_VAL			0x00000140
#define LCD_CONTROL_VAL				0x00000100
#endif

#else /*CONFIG_SONY_ICX1087*/

#if ENABLE_FRAMECACHE
#define LCD_CONTROL_VAL				0x00000046
#else /* !(ENABLE_FRAMECACHE) */
#define LCD_CONTROL_VAL				0x00000146
#endif /* ENABLE_FRAMECACHE */
#endif /*CONFIG_SONY_ICX1087*/

#define LCD_FLAME_A				0x00000001
#define LCD_FLAME_B				0x00000002


#ifdef CONFIG_SONY_ICX1087  /* 2006.05.15 ish */

#define LCD_HAREA_VAL   FRONT_WIDTH
#define LCD_HBLANK_VAL 12
#define LCD_VAREA_VAL  (FRONT_HEIGHT+1)
#define LCD_VBLANK_VAL 12

#else

#define LCD_HAREA_VAL				FRONT_WIDTH
#define LCD_HBLANK_VAL				24
#define LCD_VAREA_VAL				FRONT_HEIGHT
#define LCD_VBLANK_VAL				4

#endif /*CONFIG_SONY_ICX1087*/

#define MAX_SAVE_REG_COUNT			30	/* save lcd_hw register data count */



/*
 * timer
 */
#define MIX_DSP_TIMEOUT		10 		/* (MIX_DSP_TIMEOUT * 10ms) is waittime */
#define LCD_TIMER_STOP		0
#define LCD_TIMER_START		1

/*
 * Vsync
 */
#define VSYNC_BIT_OFF		0
#define VSYNC_BIT_ON		1


/********************************************************
 *                                                      *
 *  Structure                                           *
 *                                                      *
 *******************************************************/

// structure sending IMG reqest from lcd
typedef struct mp200_img_req_lcd {
	struct mp200_img_req_lcd *next;
	struct mp200_img_req_lcd *prev;
	int status;
	img_callback_func callback;
	int lastLayer;
	unsigned long data;
	MP200_IMG_REG_INFO img_reg_info[IMG_REG_INFO_MAX];
} MP200_IMG_REQ_LCD;

// structure lcd device status
typedef struct mp200_lcd_dev {
	struct semaphore      sem_image_data;		/* semafore v4l2/fb/img local data */
	struct semaphore      sem_image_flag;		/* semafore v4l2/fb/img local flag */
	spinlock_t	      timer_lock;		/* spin lock LCD timer flag        */
	spinlock_t	      callback_lock;		/* spin lock callback              */
} MP200_LCD_DEV;


#endif /* _MP200_LCD_H_ */
