/*
 * File Name       : /drivers/video/mp200_v4l2/mp200_lcdhw.c
 * Function        : LCD Driver corresponding to V4L2 (H/W Control)
 * Release Version :
 * Release Date    :
 *
 * Copyright (C) NEC Electronics Corporation 2006
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

/*
 * Copyright 2006,2007 Sony Corporation. 
 */


/********************************************************
 *                                                      *
 *  Include Files                                       *
 *                                                      *
 *******************************************************/

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <asm/io.h>
#include <asm/arch/gpiocore.h>
#include <asm/arch/irqs.h>
#include <asm/arch/pmu.h>
#include <asm/arch/img.h>
#include <asm/arch/mp201_mem.h>
#include <linux/delay.h>   /*added by ita 2006.08.03 for mdelay()*/

#include <asm/arch/smu.h>  /*added by ita 2006.08.01 for stop_lcdc()*/

#include "../mp200/lcdc.h"
#include "mp200_lcd.h"
#include "mp200_common.h"
#if ENABLE_FRAMECACHE
#include "../mp200/fifo.h"
#endif /* ENABLE_FRAMECACHE */

#ifdef CONFIG_SONY_ICX1087  /* 2006.07.21 ishi */
#include <asm/arch/icx1087_backlight.h>
#endif


/********************************************************
 *                                                      *
 *  Variables                                           *
 *                                                      *
 *******************************************************/
       enum     framenum   { DISPLAY_FRAME_A, DISPLAY_FRAME_B };
       enum     framenum   iDispPageNo = DISPLAY_FRAME_A;
       enum     framenum   iChangePageNo = DISPLAY_FRAME_A;
static int                 iVsyncBit = VSYNC_BIT_OFF;
       unsigned long       ulLcdhwRegVal32[MAX_SAVE_REG_COUNT];

#ifdef CONFIG_SONY_ICX1087  /* 2006.05.15 ishi */
void start_lcd_clock(void);
static int nRegLog = 0; /*2006.08.07 ita*/
static unsigned int _nLsiVer = 0;
static int iFlgLcdOn = 0;
extern MP200_LCD_DEV       lcd_dev;
#endif

/*
 * extern
 */
extern  ulong  FrameBufA;		/* paddr */
extern  ulong  FrameBufB;		/* paddr */
extern  char  *FrameBufAV;		/* vaddr */
extern  char  *FrameBufBV;		/* vaddr */
extern  ulong  FrameBufLength;
extern  char  *LCDCMmioV;		/* vaddr */
#if ENABLE_FRAMECACHE
extern  char  *FIFOMmioV;		/* vaddr */
#endif

extern  char  *dev_name;


/********************************************************
 *                                                      *
 *  Prototype declarations of function                  *
 *                                                      *
 *******************************************************/

/*
 * LCD Driver (H/W)
 */

/* Private function */
              int   change_frame(void);
              int   init_lcdhw(int brightness);
              int getLcdOn(void);
#if ENABLE_FRAMECACHE
static        void  init_fifo(void);
#endif /* ENABLE_FRAMECACHE */
static inline void  set_size_lcd(void);
static inline void  start_lcdhw(void);
static inline void  start_sync_lcdhw(void);
#ifdef MODULE
              void  stop_lcdhw(void);
#endif /* MODULE */
              void lcd_hw_int_enable(void);
              void lcd_hw_int_disable(void);
              void lcd_hw_save_reg(void);
              void lcd_hw_restore_reg(void);

int suspend_lcdhw(int reg_log);
/*
 * extern LCD function
 */
extern irqreturn_t lcd_irq_handler(int irq, void *dev_id, struct pt_regs *fp);


/********************************************************
 *                                                      *
 *  Function Definitions                                *
 *                                                      *
 *******************************************************/

/******************************************************************************
* MODULE   : change_frame
* FUNCTION : change LCD display page
* RETURN   : LCD_SUCCESS(0)      : success
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/

int change_frame()
{
	/* frame change B -> A */
	if (iDispPageNo == DISPLAY_FRAME_B)
	{
		writel(LCD_FLAME_A, LCDCMmioV + LCD_FRAMESEL);
		iDispPageNo = DISPLAY_FRAME_A;
	}
	/* frame change A -> B */
	else
	{
		writel(LCD_FLAME_B, LCDCMmioV + LCD_FRAMESEL);
		iDispPageNo = DISPLAY_FRAME_B;
	}
	return LCD_SUCCESS;
}

/*from here, added by ita 2006.08.07*/
int change_frame_when_lcdc_off(void)
{
    /*printk(KERN_INFO"change_frame_when_lcdc_off\n");*/
	/* frame change B -> A */
	if (iDispPageNo == DISPLAY_FRAME_B)
	{
		iDispPageNo = DISPLAY_FRAME_A;
	}
	/* frame change A -> B */
	else
	{
		iDispPageNo = DISPLAY_FRAME_B;
	}
    /*printk("ch_off\n");*/
	return LCD_SUCCESS;
}
/*to here, added by ita 2006.08.07*/

/*from here, added by ita 2006.12.11*/
/*
 * return: 1...LCD on, 0...off
*/
int getLcdOn(void)
{
    return iFlgLcdOn;

}
/*to here, added by ita 2006.12.11*/

/********************************************************
 *                                                      *
 *  Init Function Definitions                           *
 *                                                      *
 *******************************************************/

/******************************************************************************
* MODULE   : init_lcdhw
* FUNCTION : LCD(H/W) initialized
* RETURN   : 0     : success
*            other : fail
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/
int init_lcdhw(int brightness)
{
#ifndef CONFIG_SONY_ICX1087  /* 2006.08.02 */
	int iRet = 0;
	unsigned int lcd_write_data;
	unsigned int lcd_write_mask;
#endif

#if ENABLE_FRAMECACHE
	// frame cache settings
	init_fifo();
#endif /* ENABLE_FRAMECACHE */

#ifdef CONFIG_SONY_ICX1087  /* 2006.05.15 ish */
    pwc_read(PWC_LSIVER, &_nLsiVer);  /*read version of this chip*/

    /* NOTE: power on lcd but and also change REGLOG */
    icx1087_power_on_lcd(1,1);
    nRegLog = 1;
#else
	/* Unreset LCD */
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCD);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCDPCLK);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_TFDPCLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_TFD_PCLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_TFD_LCLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_CLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_PCLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_LCLK);
	mp200_pmu_unreset_device(MP200_RESETDEVICE_LCD);
	mp200_pmu_unreset_device(MP200_RESETDEVICE_TFD);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_TFDPCLK);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_LCD);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_LCDPCLK);

#endif    

    /* comment out to make initial picture drawn by u-boot alive. 2006.08.11*/
	// frame buffer clear
    /*memset_io((char __iomem *)FrameBufAV, 0, FrameBufLength);
      memset_io((char __iomem *)FrameBufBV, 0, FrameBufLength);*/

	// display frame set
	writel(LCD_CONTROL_VAL,  LCDCMmioV + LCD_CONTROL);
    //writel(FRONT_WIDTH * BITS_PER_PIXEL_OUT / 8,    LCDCMmioV + LCD_HOFFSET);
	writel(LCD_FLAME_A,      LCDCMmioV + LCD_FRAMESEL);

	writel(FrameBufA,        LCDCMmioV + LCD_AREAADR_A);
	writel(FrameBufB,        LCDCMmioV + LCD_AREAADR_B);

#if ENABLE_FRAMECACHE
	writel(FRAMECACHE_START, LCDCMmioV + LCD_AREAADR_C);
#endif /* ENABLE_FRAMECACHE */
	// lcd display size set
	set_size_lcd();

	// disable LCDC interrupts
	lcd_hw_int_disable();

	// install LCDC irq handler
	if (request_irq(INT_LCD, lcd_irq_handler, 0, dev_name, NULL))
	{
		printk(KERN_ERR " @lcd: lcd_probe: fail in request_irq(INT_LCD)\n");
		return LCD_INIT_ERROR__LCD_IRQ_NOT_REQUEST;
	}

	// lcd start
	start_lcdhw();

#ifdef CONFIG_SONY_ICX1087  /* 2006.05.15 ish */
    if (brightness > 0) {
        icx1087_set_bl_brightness(brightness);
    }else{     /*when brightness equals 0*/
        suspend_lcdhw(0);  /*do the same as supend. not change reg_log*/
    }
    return 0;
#else

    
	// Power & BackLight
	lcd_write_mask = (unsigned int)((1 << PWC_INT_LED_ON) | (1 << PWC_INT_LCD_OE));
	lcd_write_data = lcd_write_mask;
	iRet = gpiocore_pwc_write(GPIOCORE_PWC_IOOUT2, lcd_write_data, lcd_write_mask);
	if (iRet < 0)
	{
		printk(KERN_ERR " @lcd: init_lcdhw: cannot use gpiocore:%x\n", iRet);
		return LCD_INIT_ERROR__LCD_NOT_POWER_ON;
	}

	return LCD_SUCCESS;

#endif
    
}


/*from here, added by ita 2006.07.31 */
#ifdef CONFIG_SONY_ICX1087  /* 2006.07.31 ita */
/******************************************************************************
* FUNCTION : sequencial execete of turning on LCD.
* RETURN   : none
* NOTE     : none                                                              
* CREATE   : 2006.08.21 ita                               
* UPDATE   :                                                                   
******************************************************************************/
void lcdhw_bl_off_seq(int stage)
{
    /* Backlight off (code changed) */
    if(stage == 0){
        iFlgLcdOn = 0;  /*added 20061212*/
        icx1087_set_bl_brightness(0);
        start_sync_lcdhw(); /*stop data*/
    }
    icx1087_power_off_lcd_sequencial(stage);
    return;
}

/******************************************************************************
* FUNCTION : sequencial execete of turning off LCD.
* RETURN   : 0... success
* NOTE     : none                                                              
* CREATE   : 2006.08.21 ita                               
* UPDATE   :                                                                   
******************************************************************************/
int lcdhw_bl_on_seq(int brightness,int stage)
{
	unsigned long flags;
    icx1087_power_on_lcd_sequencial(brightness,stage);
    if(stage == 0){/*initialize LCDC register and start sync signal*/
        start_lcd_clock();  /**20061202 move from icx1087_power_on_lcd_sequencial*/
        writel(LCD_CONTROL_VAL,  LCDCMmioV + LCD_CONTROL);

        if (iDispPageNo == DISPLAY_FRAME_A)
            writel(LCD_FLAME_A, LCDCMmioV + LCD_FRAMESEL);
        else
            writel(LCD_FLAME_B, LCDCMmioV + LCD_FRAMESEL);

        writel(FrameBufA,        LCDCMmioV + LCD_AREAADR_A);
        writel(FrameBufB,        LCDCMmioV + LCD_AREAADR_B);

#if ENABLE_FRAMECACHE
        writel(FRAMECACHE_START, LCDCMmioV + LCD_AREAADR_C);
#endif /* ENABLE_FRAMECACHE */

        // lcd display size set
        set_size_lcd();

        // disable LCDC interrupts
        lcd_hw_int_disable();

        free_irq(INT_LCD,NULL);

        // install LCDC irq handler
        if (request_irq(INT_LCD, lcd_irq_handler, 0, dev_name, NULL))
        {
            printk(KERN_ERR " @lcd: lcd_probe: fail in request_irq(INT_LCD)\n");
            return LCD_INIT_ERROR__LCD_IRQ_NOT_REQUEST;
        }

        start_sync_lcdhw();
    }else if(stage == 2){
        /*this setting is needed, before lcd start.*/
        spin_lock_irqsave(&lcd_dev.callback_lock, flags);  /*added by ita 20061211*/
        if (iDispPageNo == DISPLAY_FRAME_A)
            writel(LCD_FLAME_A, LCDCMmioV + LCD_FRAMESEL);
        else
            writel(LCD_FLAME_B, LCDCMmioV + LCD_FRAMESEL);
        /* lcd start*/
        start_lcdhw();
        spin_unlock_irqrestore(&lcd_dev.callback_lock, flags);  /**/
    }
    return 0;
}
/*from here, added by ita 2006.08.07 */
int suspend_lcdhw(int reg_log)
{
    /* Backlight off (code changed) */
    icx1087_set_bl_brightness(0);
    if(reg_log){/* power off lcd and also change REGLOG */
        if(nRegLog == 1){
            icx1087_power_off_lcd(1,1);
            nRegLog = 0;
        }else{
            icx1087_power_off_lcd(0,1);
        }
    }else{
        icx1087_power_off_lcd(0,1);
    }
    iFlgLcdOn = 0;  /*added 20061212*/
    //printk(KERN_INFO"nRegLog:%d\n",nRegLog);
    return 0;
}

/*from here, added by ita 2006.07.31 */
int resume_lcdhw(int brightness)
{
	unsigned long flags;
#if ENABLE_FRAMECACHE
	// frame cache settings
	init_fifo();
#endif /* ENABLE_FRAMECACHE */

    if(brightness == 0){/*modified by ita 2006.08.06*/
        /* NOTE: keep lcd off but also change REGLOG */
        if(nRegLog == 0){
            icx1087_power_on_lcd(1,0);
            nRegLog = 1;
        }
    }else{ /*modified by ita 2006.08.06*/
        /* NOTE: power on lcd and also change REGLOG */
        if(nRegLog == 0){
            icx1087_power_on_lcd(1,1);
            nRegLog = 1;
        }else{
            icx1087_power_on_lcd(0,1);
        }

        start_lcd_clock();  /*20061202 move from icx1087_power_on_lcd*/

        // display frame set
        writel(LCD_CONTROL_VAL,  LCDCMmioV + LCD_CONTROL);

        spin_lock_irqsave(&lcd_dev.timer_lock, flags);  /*added by ita 20061211*/
        if (iDispPageNo == DISPLAY_FRAME_A)
        {
            writel(LCD_FLAME_A, LCDCMmioV + LCD_FRAMESEL);
        }
        else
        {
            writel(LCD_FLAME_B, LCDCMmioV + LCD_FRAMESEL);
        }
        spin_unlock_irqrestore(&lcd_dev.timer_lock, flags);  /**/

        writel(FrameBufA,        LCDCMmioV + LCD_AREAADR_A);
        writel(FrameBufB,        LCDCMmioV + LCD_AREAADR_B);

#if ENABLE_FRAMECACHE
        writel(FRAMECACHE_START, LCDCMmioV + LCD_AREAADR_C);
#endif /* ENABLE_FRAMECACHE */

        // lcd display size set
        set_size_lcd();

        // disable LCDC interrupts
        lcd_hw_int_disable();

        free_irq(INT_LCD,NULL);

        // install LCDC irq handler
        if (request_irq(INT_LCD, lcd_irq_handler, 0, dev_name, NULL))
        {
            printk(KERN_ERR " @lcd: lcd_probe: fail in request_irq(INT_LCD)\n");
            return LCD_INIT_ERROR__LCD_IRQ_NOT_REQUEST;
        }

        // lcd start
        start_lcdhw();

        //icx1087_set_bl_brightness(brightness);
        lcdhw_bl_on_seq(brightness,2); /*PWM ON*/
        msleep(40);  /*wait PWM stable*/
        lcdhw_bl_on_seq(brightness,3); /*Backlight ON*/
    }
    //printk(KERN_INFO"nRegLog:%d\n",nRegLog);
    return 0;
}
#endif //CONFIG_SONY_ICX1087  /* 2006.07.31 ita */
/*to here, added by ita 2006.07.31 */


/*******************************************************************************
* MODULE   : lcd_hw_int_enable
* FUNCTION : enable LCDC VSYNC interrupt
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.07.14 Y.Utsuhara
* UPDATE   :
********************************************************************************/
void lcd_hw_int_enable(void)
{
	/* Display A -> set FrameBufB */
	if(iDispPageNo == DISPLAY_FRAME_NO_A)
	{
		iChangePageNo = DISPLAY_FRAME_NO_B;
	}
	/* Display B -> set FrameBufA */
	else
	{
		iChangePageNo = DISPLAY_FRAME_NO_A;
	}

	/*  Interuppt Enable Set */
	writel(LCD_LCDVSEN_BIT, LCDCMmioV + LCD_INTENSET);
	iVsyncBit = VSYNC_BIT_ON;

	return;
}

/*******************************************************************************
* MODULE   : lcd_hw_int_disable
* FUNCTION : disable LCDC interrupts
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.07.14 Y.Utsuhara
* UPDATE   :
********************************************************************************/
void lcd_hw_int_disable(void)
{
	// Interrupt Enable Clear
	writel(LCD_LCDSTOPMASK_BIT | LCD_UNDERRUNMASK_BIT |
	       LCD_LCDVSMASK_BIT   | LCD_DMAERRMASK_BIT   | LCD_DMASTOPMASK_BIT,
	       LCDCMmioV + LCD_INTENCLR);
	// Interrupt Status Clear
	writel(LCD_LCDSTOPCLR_BIT | LCD_UNDERRUNCLR_BIT |
	       LCD_DMASTOPCLR_BIT | LCD_DMAERRCLR_BIT   | LCD_LCDVSCLR_BIT,
	       LCDCMmioV + LCD_INTFFCLR);

	iVsyncBit = VSYNC_BIT_OFF;

	return;
}

#if ENABLE_FRAMECACHE

/******************************************************************************
* MODULE   : init_fifo
* FUNCTION : fifo initialize
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/

static void init_fifo()
{
	writel(FrameBufA, FIFOMmioV + FIF_FRAMEAADDR);
	writel(FrameBufB, FIFOMmioV + FIF_FRAMEBADDR);
	writel(FRAMECACHE_LENGTH, FIFOMmioV + FIF_FRAMESIZE);
	writel(FRAMECACHE_START, FIFOMmioV + FIF_FCBASEADDR);
	writel(FIF_FCCTRL_VAL, FIFOMmioV + FIF_FCCTRL);

	return;
}
#endif /* ENABLE_FRAMECACHE */


/******************************************************************************
* MODULE   : set_size_lcd
* FUNCTION : LCD(H/W) size initialize
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/

static inline void set_size_lcd()
{
#ifdef CONFIG_SONY_ICX1087  /* 2006.05.15 ishi */

    writel(LCD_HAREA_VAL + LCD_HBLANK_VAL + 12, LCDCMmioV + LCD_HTOTAL);
    writel(LCD_HAREA_VAL, LCDCMmioV + LCD_HAREA);
    writel(8, LCDCMmioV + LCD_HEDGE1);
	writel(LCD_HBLANK_VAL, LCDCMmioV + LCD_HEDGE2);
	writel(LCD_VAREA_VAL + LCD_VBLANK_VAL, LCDCMmioV + LCD_VTOTAL);
	writel(LCD_VAREA_VAL, LCDCMmioV + LCD_VAREA);
    writel(8, LCDCMmioV + LCD_VEDGE1);
	writel(10, LCDCMmioV + LCD_VEDGE2);
#else
    
	writel(LCD_HAREA_VAL + LCD_HBLANK_VAL, LCDCMmioV + LCD_HTOTAL);
	writel(LCD_HAREA_VAL,                  LCDCMmioV + LCD_HAREA);
	writel(LCD_HAREA_VAL,                  LCDCMmioV + LCD_HEDGE1);
	writel(LCD_HAREA_VAL + LCD_HBLANK_VAL, LCDCMmioV + LCD_HEDGE2);
	writel(LCD_VAREA_VAL + LCD_VBLANK_VAL, LCDCMmioV + LCD_VTOTAL);
	writel(LCD_VAREA_VAL,                  LCDCMmioV + LCD_VAREA);
	writel(LCD_VAREA_VAL,                  LCDCMmioV + LCD_VEDGE1);
	writel(LCD_VAREA_VAL + LCD_VBLANK_VAL, LCDCMmioV + LCD_VEDGE2);
#endif

    /*modified by ita 2006.09.14 for RGB666*/
	//writel(LCD_HAREA_VAL / 16 * 32,        LCDCMmioV + LCD_HOFFSET);
    writel(FRONT_WIDTH * BITS_PER_PIXEL_OUT / 8,    LCDCMmioV + LCD_HOFFSET);
	return;
}


/******************************************************************************
* MODULE   : start_lcdhw
* FUNCTION : start LCD(H/W)
* RETURN   : none
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/

static inline void start_lcdhw()
{
    iFlgLcdOn = 1;  /*added 20061212*/
	writel(LCD_BUSSEL_AHB, LCDCMmioV + LCD_BUSSEL);
	writel(LCD_LCDOUT_BIT, LCDCMmioV + LCD_LCDOUT);

	return;
}

static inline void start_sync_lcdhw()
{
	writel(LCD_LCDOUT_SYNC, LCDCMmioV + LCD_LCDOUT);
	return;
}


#ifdef MODULE
/********************************************************
 *                                                      *
 *  Exit Function Definitions                           *
 *                                                      *
 *******************************************************/

/******************************************************************************
* MODULE   : stop_lcdhw
* FUNCTION : stop LCD(H/W)
* RETURN   : 0     : success
* NOTE     : none
* CREATE   : 2006.05.25 H.Nagashima
* UPDATE   :
******************************************************************************/

int stop_lcdhw(void)
{
	writel(0, LCDCMmioV + LCD_LCDOUT);
	//return 0;
    return ;
}

#endif /* MODULE */

/* 2006.07.27 ishi */
#ifdef CONFIG_SONY_ICX1087

void stop_lcd_out(void)
{
    
}


/*
 *
 */
void stop_lcd_clock(void)
{
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCD);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCDPCLK);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_TFDPCLK);
	mp200_pmu_reset_device(MP200_RESETDEVICE_TFD);
	mp200_pmu_reset_device(MP200_RESETDEVICE_LCD);

    /*mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_CLK);*/

	mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_PCLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_LCD_LCLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_TFD_PCLK);
	mp200_pmu_close_clockgate(MP200_CLOCKGATE_TFD_LCLK);
    
}


/**
 *
 */
void start_lcd_clock(void)
{
	/* Unreset LCD */
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCD);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_LCDPCLK);
	mp200_pmu_clkctrl_off(MP200_CLKCTRL_TFDPCLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_TFD_PCLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_TFD_LCLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_CLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_PCLK);
	mp200_pmu_open_clockgate(MP200_CLOCKGATE_LCD_LCLK);
	mp200_pmu_unreset_device(MP200_RESETDEVICE_LCD);
	mp200_pmu_unreset_device(MP200_RESETDEVICE_TFD);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_TFDPCLK);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_LCD);
	mp200_pmu_clkctrl_on(MP200_CLKCTRL_LCDPCLK);
    
}
#endif /* CONFIG_SONY_ICX1087 */
    


/********************************************************
 *                                                      *
 *  save/restore Function Definitions                   *
 *                                                      *
 *******************************************************/

/******************************************************************************
* MODULE   : lcd_hw_save_reg
* FUNCTION : save LCD(H/W) register data
* RETURN   : 0     : success
* NOTE     : none
* CREATE   : 2006.08.17 H.Nagashima
* UPDATE   :
******************************************************************************/

void lcd_hw_save_reg(void)
{
	// LCD display off
	writel(0, LCDCMmioV + LCD_LCDOUT);

	return;
}


/******************************************************************************
* MODULE   : lcd_hw_restore_reg
* FUNCTION : restore LCD(H/W) register data
* RETURN   : 0     : success
* NOTE     : none
* CREATE   : 2006.08.17 H.Nagashima
* UPDATE   :
******************************************************************************/

void lcd_hw_restore_reg(void)
{
	/* init fifo */
#if ENABLE_FRAMECACHE
	init_fifo();
#endif /* ENABLE_FRAMECACHE */

	/* restore LCD framebuffer address */
	writel(LCD_CONTROL_VAL,   LCDCMmioV + LCD_CONTROL);

	if(iDispPageNo == DISPLAY_FRAME_A)
		writel(LCD_FLAME_A,   LCDCMmioV + LCD_FRAMESEL);
	else
		writel(LCD_FLAME_B,   LCDCMmioV + LCD_FRAMESEL);
	writel(FrameBufA,         LCDCMmioV + LCD_AREAADR_A);
	writel(FrameBufB,         LCDCMmioV + LCD_AREAADR_B);

#if ENABLE_FRAMECACHE
	writel(FRAMECACHE_START,  LCDCMmioV + LCD_AREAADR_C);
#endif /* ENABLE_FRAMECACHE */

	if(iVsyncBit == VSYNC_BIT_ON)
		writel(LCD_LCDVSEN_BIT, LCDCMmioV + LCD_INTENSET);

	/* init LCD size */
	set_size_lcd();

	/* LCD display on */
	start_lcdhw();

	return;
}
