/*
 * Copyright 2004-2005 Freescale Semiconductor, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


/*
 * @file    mxcfb.h
 *
 * @brief Global header file for the MXC Frame buffer 
 * 
 * @ingroup Framebuffer
 */
#ifndef _INCLUDE_MXCFB_H_
#define _INCLUDE_MXCFB_H_

/*
 * Bits per pixel defines
 */
#ifdef CONFIG_FB_MXC_16BPP
#define MXCFB_BPP     16
#elif CONFIG_FB_MXC_24BPP
#define MXCFB_BPP     24
#elif CONFIG_FB_MXC_32BPP
#define MXCFB_BPP     32
#endif

struct mxcfb_gbl_alpha {
        int enable;
        int alpha;
};

struct mxcfb_color_key {
        int enable;
        __u32 color_key;
};

#define MXCFB_SET_GBL_ALPHA     _IOW('M', 0, struct mxcfb_gbl_alpha)
#define MXCFB_SET_CLR_KEY       _IOW('M', 1, struct mxcfb_color_key)
#define MXCFB_WAIT_FOR_VSYNC    _IO('M', 2)
#define MXCFB_SET_BRIGHTNESS    _IOW('M', 3, __u8)

#ifdef __KERNEL__

#include "../drivers/mxc/ipu/ipu.h"

/*
 * Driver name
 */
#define MXCFB_NAME      "MXCFB"

struct panel_info {
    char * name;
    ipu_panel_t type;
    uint32_t pixel_fmt;
    int width;
    int height;
    ipu_di_signal_cfg_t sig_pol;
    int vSyncWidth;
    int vStartWidth;
    int vEndWidth;
    int hSyncWidth;
    int hStartWidth;
    int hEndWidth;
};

/*
 * Structure containing the generic framebuffer information.
 */
struct mxcfb_info {
        int32_t                 open_count;
        ipu_channel_t           ipu_ch;
        uint32_t                ipu_ch_irq;
        uint32_t                cur_pixel_fmt;
        uint32_t                cur_ipu_buf;

	u32			pseudo_palette[17];

        struct semaphore        flip_sem;
        spinlock_t              fb_lock;
};

#endif /* __KERNEL__ */
#endif
