/*
 * Copyright 2005 Freescale Semiconductor, Inc. All Rights Reserved.
 */

/* 
 * The code contained herein is licensed under the GNU General Public
 * License. You may obtain a copy of the GNU General Public License
 * Version 2 or later at the following locations:
 * 
 * http://www.opensource.org/licenses/gpl-license.html 
 * http://www.gnu.org/copyleft/gpl.html
 */

#ifndef __BOARD_H__

#define __BOARD_H__

/*!
 * @defgroup BRDCFG Board Configuration Options
 * @ingroup MSL
 */

/*! 
 * @file board.h
 *
 * @brief This file contains all the board level configuration options.
 *
 * It currently hold the options defined for Argon Plus Platform.
 * 
 * @ingroup BRDCFG
 */

/*
 * Include Files
 */
#include <linux/config.h>
#ifdef CONFIG_ARCH_ARGONPLUSEVB
#include "board-evb.h"
#endif
#ifdef CONFIG_ARCH_ARGONPLUSODYSSEY
#include "board-odyssey.h"
#endif

/*!
 * @name Configurable Graphics/IPU parameters
 */
/*! @{ */

/*!
 * Size of IPU buffer memory
 * Note: Currently the last 4MB in the 64MB range is used.
 */
#define MXCIPU_MEM_SIZE         SZ_4M

/*!
 * Screen width and height
 */
#if defined(CONFIG_FB_MXC_VGA_PANEL) 
#define MXCFB_SCREEN_WIDTH      640
#define MXCFB_SCREEN_HEIGHT     480
#elif defined(CONFIG_FB_MXC_EPSON_PANEL)
#define MXCFB_SCREEN_WIDTH      176
#define MXCFB_SCREEN_HEIGHT     220
#else
#define MXCFB_SCREEN_WIDTH      240
#define MXCFB_SCREEN_HEIGHT     320
#endif

/*!
 * Sensor screen width
 */
#define MXC_SENSOR_SCREEN_WIDTH         240
/*!
 * Sensor screen height 
 */
#define MXC_SENSOR_SCREEN_HEIGHT        320
/*! @} End configurable parameters */

/*! @{*/
/*! This is System IRQ used by CS8900A for interrupt generation taken from platform.h */
#define CS8900AIRQ              INT_SIRF_EXT_INT5_OR    
/*! @} */

#endif /* __BOARD_H__ */
