/*
 * Copyright 2005 Freescale Semiconductor, Inc. All Rights Reserved.
 */

/* 
 * The code contained herein is licensed under the GNU General Public
 * License. You may obtain a copy of the GNU General Public License
 * Version 2 or later at the following locations:
 * 
 * http://www.opensource.org/licenses/gpl-license.html 
 * http://www.gnu.org/copyleft/gpl.html
 */

#ifndef __CLOCK_H

#define __CLOCK_H

/*!
 * @defgroup CLOCKS Clocking Setup and Retrieving
 * @ingroup MSL
 */

/*! 
 * @file clock.h
 *
 * @brief API for setting up and retrieving clocks.
 *
 * This file contains API for setting up and retrieving clocks.
 *
 * @ingroup CLOCKS
 */

#include <asm/arch/platform.h>

/* Register offsets */
#define MXC_CCM_MCR         0x0
#define MXC_CCM_MPDR0       0x4
#define MXC_CCM_MPDR1       0x8
#define MXC_CCM_MPCTL       0x10
#define MXC_CCM_UPCTL       0x14
#define MXC_CCM_MCGR0       0x1C
#define MXC_CCM_MCGR1       0x20
#define MXC_CCM_MCGR2       0x24

/* Register bit definitions */
#define MXC_CCM_MCR_SSIS1                   0x10000000
#define MXC_CCM_MCR_SSIS2                   0x8000000
#define MXC_CCM_MCR_FIRS                    0x800

#define MXC_CCM_MPDR0_CSI_PDF_OFFSET        23
#define MXC_CCM_MPDR0_CSI_PDF_MASK          (0x1FF << 23)
#define MXC_CCM_MPDR0_NFC_PDF_OFFSET        8
#define MXC_CCM_MPDR0_NFC_PDF_MASK          (0x7 << 8)
#define MXC_CCM_MPDR0_IPG_PDF_OFFSET        6
#define MXC_CCM_MPDR0_IPG_PDF_MASK          (0x3 << 6)
#define MXC_CCM_MPDR0_MAX_PDF_OFFSET        3
#define MXC_CCM_MPDR0_MAX_PDF_MASK          (0x7 << 3)
#define MXC_CCM_MPDR0_BRMM_OFFSET           0
#define MXC_CCM_MPDR0_BRMM_MASK             0x7

#define MXC_CCM_MPDR1_USB_PDF_OFFSET        27
#define MXC_CCM_MPDR1_USB_PDF_MASK          (0x7 << 27)
#define MXC_CCM_MPDR1_FIRI_PREPDF_OFFSET    24
#define MXC_CCM_MPDR1_FIRI_PREPDF_MASK      (0x7 << 24)
#define MXC_CCM_MPDR1_FIRI_PDF_OFFSET       19
#define MXC_CCM_MPDR1_FIRI_PDF_MASK         (0x1F << 19)
#define MXC_CCM_MPDR1_SSI2_PREPDF_OFFSET    15
#define MXC_CCM_MPDR1_SSI2_PREPDF_MASK      (0x7 << 15)
#define MXC_CCM_MPDR1_SSI2_PDF_OFFSET       10
#define MXC_CCM_MPDR1_SSI2_PDF_MASK         (0x1F << 10)
#define MXC_CCM_MPDR1_SSI1_PREPDF_OFFSET    6
#define MXC_CCM_MPDR1_SSI1_PREPDF_MASK      (0x7 << 6)
#define MXC_CCM_MPDR1_SSI1_PDF_OFFSET       1
#define MXC_CCM_MPDR1_SSI1_PDF_MASK         (0x1F << 1)

/* Bit definitions for both MCU and USB PLL control registers */
#define MXC_CCM_PCTL_BRM                    0x80000000
#define MXC_CCM_PCTL_PDF_OFFSET             26
#define MXC_CCM_PCTL_PDF_MASK               (0xF << 26)
#define MXC_CCM_PCTL_MFD_OFFSET             16
#define MXC_CCM_PCTL_MFD_MASK               (0x3FF << 16)
#define MXC_CCM_MCUPCTL_MFI_OFFSET          11
#define MXC_CCM_MCUPCTL_MFI_MASK            (0xF << 11)
#define MXC_CCM_MCUPCTL_MFN_OFFSET          0
#define MXC_CCM_MCUPCTL_MFN_MASK            0x7FF
#define MXC_CCM_USBPCTL_MFI_OFFSET          10
#define MXC_CCM_USBPCTL_MFI_MASK            (0xF << 10)
#define MXC_CCM_USBPCTL_MFN_OFFSET          0
#define MXC_CCM_USBPCTL_MFN_MASK            0x3FF

#define MXC_CCM_MCGR0_SSI1                  (0x3 << 2)
#define MXC_CCM_MCGR0_UART1                 (0x3 << 4)   
#define MXC_CCM_MCGR0_UART2                 (0x3 << 6)   
#define MXC_CCM_MCGR0_CSPI1                 (0x3 << 8)
#define MXC_CCM_MCGR0_FIRI                  (0x3 << 10)
#define MXC_CCM_MCGR0_GPTMCU                (0x3 << 12)
#define MXC_CCM_MCGR0_RTC                   (0x3 << 14)
#define MXC_CCM_MCGR0_EPIT1MCU              (0x3 << 16)
#define MXC_CCM_MCGR0_EPIT2MCU              (0x3 << 18)
#define MXC_CCM_MCGR0_EDIO                  (0x3 << 20)      
#define MXC_CCM_MCGR0_WDOGMCU               (0x3 << 22)
#define MXC_CCM_MCGR0_SIRF                  (0x3 << 24)
#define MXC_CCM_MCGR0_PWMMCU                (0x3 << 26)
#define MXC_CCM_MCGR0_OWIRE                 (0x3 << 28)
#define MXC_CCM_MCGR0_I2C                   (0x3 << 30)

#define MXC_CCM_MCGR1_IPU                    0x3
#define MXC_CCM_MCGR1_MPEG4                 (0x3 << 2)
#define MXC_CCM_MCGR1_EMI                   (0x3 << 4)
#define MXC_CCM_MCGR1_IIM                   (0x3 << 6)
#define MXC_CCM_MCGR1_SIM1                  (0x3 << 8)
#define MXC_CCM_MCGR1_SIM2                  (0x3 << 10)
#define MXC_CCM_MCGR1_HAC                   (0x3 << 12)
#define MXC_CCM_MCGR1_GEM                   (0x3 << 14)
#define MXC_CCM_MCGR1_USBOTG                (0x3 << 16)
#define MXC_CCM_MCGR1_CSPI2                 (0x3 << 20)
#define MXC_CCM_MCGR1_UART4                 (0x3 << 22)
#define MXC_CCM_MCGR1_UART3                 (0x3 << 24)
#define MXC_CCM_MCGR1_SDHC2                 (0x3 << 26)
#define MXC_CCM_MCGR1_SDHC1                 (0x3 << 28)
#define MXC_CCM_MCGR1_SSI2                  (0x3 << 30)

#define MXC_CCM_MCGR2_SDMA                   0x3
#define MXC_CCM_MCGR2_RTRMCU                (0x3 << 2)
#define MXC_CCM_MCGR2_RNG                   (0x3 << 4)
#define MXC_CCM_MCGR2_KPP                   (0x3 << 6)
#define MXC_CCM_MCGR2_MU                    (0x3 << 8)
#define MXC_CCM_MCGR2_ECT                   (0x3 << 10)
#define MXC_CCM_MCGR2_SMC                   (0x3 << 12)
#define MXC_CCM_MCGR2_RTIC                  (0x3 << 14)
#define MXC_CCM_MCGR2_SPBA                  (0x3 << 16)

#define MXC_PLL_REF_CLK                     26000000

/*!
 * Enumerations for plls
 */
enum plls {
        MCUPLL, // MCU PLL
        USBPLL, // USB PLL
};

/*!
 * Enumerations for MXC clocks
 */
enum mxc_clocks {
        CPU_CLK,
        AHB_CLK,
        IPG_CLK,
        NFC_CLK,
        USB_CLK,
        UART1_BAUD,
        UART2_BAUD,
        UART3_BAUD,
        UART4_BAUD,
        SSI1_BAUD,
        SSI2_BAUD,
        CSI_BAUD,
        FIRI_BAUD,
        I2C_CLK,
        CSPI1_CLK,
        CSPI2_CLK,
        GPT_CLK,
        RTC_CLK,
        EPIT1_CLK,
        EPIT2_CLK,
        EDIO_CLK,
        WDOG_CLK,
        PWM_CLK,
        IPU_CLK,
        MPEG4_CLK,
        SIM1_CLK,
        SIM2_CLK,
        HAC_CLK,
        GEM_CLK,
        SDHC2_CLK,
        SDHC1_CLK,
        SDMA_CLK,
        RNG_CLK,
        KPP_CLK,
        MU_CLK,
        SPBA_CLK, 
};

/*!
 * This function returns the PLL output value in Hz based on pll.
 * @param       pll     PLL as defined in enum plls
 * @return      PLL value in Hz.
 */
unsigned long mxc_pll_clock(enum plls pll); 

/*!
 * This function returns the various MXC clock value in Hz
 * 
 * @param       clk     as defined in enum mxc_clocks
 *
 * @return      clock value in Hz
 */
unsigned long mxc_get_clocks(enum mxc_clocks clk);

/*!
 * This function sets the PLL source for a clock.
 *
 * @param clk     as defined in enum mxc_clocks
 * @param pll_num the PLL that you wish to use as source for this clock
 */
void mxc_set_clocks_pll(enum mxc_clocks clk, enum plls pll_num);

/*!
 * This function sets the division factor for a clock.
 *
 * @param clk as defined in enum mxc_clocks
 * @param div the division factor to be used for the clock (For SSI, pass in
 *            2 times the expected division value to account for FP vals)
 */
void mxc_set_clocks_div(enum mxc_clocks clk, unsigned int div);

/*!
 * This function is called to gate off the individual module clocks
 *
 * @param       clk     as defined in enum mxc_clocks
 */
void mxc_clks_disable(enum mxc_clocks clk);
                                                                                
/*!
 * This function is called to enable the individual module clocks
 *
 * @param       clk     as defined in enum mxc_clocks
 */
void mxc_clks_enable(enum mxc_clocks clk);
#endif /* __CLOCK_H */
