/*
 *  File Name	    : linux/include/asm-arm/arch-mp200/dma.h
 *  Function	    : dmac
 *  Release Version : Ver 1.01
 *  Release Date    : 2006/02/01
 *
 *  Copyright (C) NEC Electronics Corporation 2005-2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#ifndef __ASM_ARM_ARCH_MP200_DMA_H
#define __ASM_ARM_ARCH_MP200_DMA_H

#define MAX_DMA_ADDRESS 		0xffffffff
#define MAX_DMA_CHANNELS		0	/* don't use standard DMA I/F */

#ifdef CONFIG_MP201
#define MP200_DMAC_MAX_P_CHANNELS	3
#define MP200_DMAC_MAX_L_CHANNELS	( MP200_DMAC_P0_MAX_L_CHANNELS + \
					  MP200_DMAC_P1_MAX_L_CHANNELS + \
					  MP200_DMAC_P2_MAX_L_CHANNELS )
#else
#define MP200_DMAC_MAX_P_CHANNELS	4
#define MP200_DMAC_MAX_L_CHANNELS	( MP200_DMAC_P0_MAX_L_CHANNELS + \
					  MP200_DMAC_P1_MAX_L_CHANNELS + \
					  MP200_DMAC_P2_MAX_L_CHANNELS + \
					  MP200_DMAC_P3_MAX_L_CHANNELS )
#endif				/* CONFIG_MP201 */

/* DMAC logical channel ID */
/* P0 */
#ifdef CONFIG_MP201
#define MP200_DMAC_ARM_LCH0		0x00	/* Memory to Memory (for ARM) */
#define MP200_DMAC_ARM_LCH1		0x01	/* Memory to Memory (for ARM) */
#define MP200_DMAC_ARM_LCH2		0x02	/* Memory to Memory (for ARM) */
#define MP200_DMAC_ARM_LCH3		0x03	/* Memory to Memory (for ARM) */
#define MP200_DMAC_P0_MAX_L_CHANNELS	4
#else
#define MP200_DMAC_ARM_LCH0		0x00	/* Memory to Memory (for ARM) */
#define MP200_DMAC_ARM_LCH1		0x01	/* Memory to Memory (for ARM) */
#define MP200_DMAC_ARM_LCH2		0x02	/* Memory to Memory (for ARM) */
#define MP200_DMAC_P0_MAX_L_CHANNELS	3
#endif				/* CONFIG_MP201 */
/* P1 */
#ifdef CONFIG_MP201
#define MP200_DMAC_M2P_LCH0		0x10	/* Memory to Peripheral (UART) */
#define MP200_DMAC_M2P_LCH1		0x11	/* Memory to Peripheral (SPI0) */
#define MP200_DMAC_M2P_LCH2		0x12	/* Memory to Peripheral (SPI1/MSP) */
#define MP200_DMAC_M2P_LCH3		0x13	/* Memory to Peripheral (MSP) */
#define MP200_DMAC_M2P_LCH4		0x14	/* Memory to Peripheral (PCM0) */
#define MP200_DMAC_M2P_LCH5		0x15	/* Memory to Peripheral (PCM1) */
#define MP200_DMAC_M2P_LCH6		0x16	/* Memory to Peripheral (ASYNC) */
#define MP200_DMAC_M2P_LCH7		0x17	/* Memory to Peripheral (SD/MMM) */
#define MP200_DMAC_P1_MAX_L_CHANNELS	8
#else
#define MP200_DMAC_K6SC_LCH0		0x20	/* Memory to Memory (for K6SC) */
#define MP200_DMAC_K6SC_LCH1		0x21	/* Memory to Memory (for K6SC) */
#define MP200_DMAC_K6SC_LCH2		0x22	/* Memory to Memory (for K6SC) */
#define MP200_DMAC_K6SC_LCH3		0x23	/* Memory to Memory (for K6SC) */
#define MP200_DMAC_K6SC_LCH4		0x24	/* Memory to Memory (for K6SC) */
#define MP200_DMAC_K6SC_LCH5		0x25	/* Memory to Memory (for K6SC) */
#define MP200_DMAC_P1_MAX_L_CHANNELS	6
#endif				/* CONFIG_MP201 */
/* P2 */
#ifdef CONFIG_MP201
#define MP200_DMAC_P2M_LCH0		0x20	/* Peripheral to Memory (UART) */
#define MP200_DMAC_P2M_LCH1		0x21	/* Peripheral to Memory (SPI0) */
#define MP200_DMAC_P2M_LCH2		0x22	/* Peripheral to Memory (SPI1/MSP) */
#define MP200_DMAC_P2M_LCH3		0x23	/* Peripheral to Memory (MSP) */
#define MP200_DMAC_P2M_LCH4		0x24	/* Peripheral to Memory (PCM0) */
#define MP200_DMAC_P2M_LCH5		0x25	/* Peripheral to Memory (PCM1) */
#define MP200_DMAC_P2M_LCH6		0x26	/* Peripheral to Memory (ASYNC) */
#define MP200_DMAC_P2M_LCH7		0x27	/* Peripheral to Memory (SD/MMM) */
#define MP200_DMAC_P2_MAX_L_CHANNELS	8
#else
#define MP200_DMAC_M2P_LCH0		0x40	/* Memory to Peripheral (U70) */
#define MP200_DMAC_M2P_LCH1		0x41	/* Memory to Peripheral (U71) */
#define MP200_DMAC_M2P_LCH2		0x42	/* Memory to Peripheral (U72) */
#define MP200_DMAC_M2P_LCH3		0x43	/* Memory to Peripheral (U73) */
#define MP200_DMAC_M2P_LCH4		0x44	/* Memory to Peripheral (MOD1/MOD0 not secure) */
#define MP200_DMAC_M2P_LCH5		0x45	/* Memory to Peripheral (MMM/MOD0 secure) */
#define MP200_DMAC_M2P_LCH6		0x46	/* Memory to Peripheral (SI0) */
#define MP200_DMAC_M2P_LCH7		0x47	/* Memory to Peripheral (SI1) */
#define MP200_DMAC_M2P_LCH8		0x48	/* Memory to Peripheral (USC) */
#define MP200_DMAC_M2P_LCH9		0x49	/* Memory to Peripheral (PM0) */
#define MP200_DMAC_M2P_LCH10		0x4a	/* Memory to Peripheral (PM1) */
#define MP200_DMAC_M2P_LCH11		0x4b	/* Memory to Peripheral (PM2) */
#define MP200_DMAC_M2P_LCH12		0x4c	/* Memory to Peripheral (SP0) */
#define MP200_DMAC_M2P_LCH13		0x4d	/* Memory to Peripheral (SP1) */
#define MP200_DMAC_M2P_LCH14		0x4e	/* Memory to Peripheral (SP2) */
#define MP200_DMAC_P2_MAX_L_CHANNELS	15
#endif				/* CONFIG_MP201 */
/* P3 */
#ifndef CONFIG_MP201
#define MP200_DMAC_P2M_LCH0		0x60	/* Peripheral to Memory (U70) */
#define MP200_DMAC_P2M_LCH1		0x61	/* Peripheral to Memory (U71) */
#define MP200_DMAC_P2M_LCH2		0x62	/* Peripheral to Memory (U72) */
#define MP200_DMAC_P2M_LCH3		0x63	/* Peripheral to Memory (U73) */
#define MP200_DMAC_P2M_LCH4		0x64	/* Peripheral to Memory (MOD1/MOD0 not secure) */
#define MP200_DMAC_P2M_LCH5		0x65	/* Peripheral to Memory (MMM/MOD0 secure) */
#define MP200_DMAC_P2M_LCH6		0x66	/* Peripheral to Memory (SI0) */
#define MP200_DMAC_P2M_LCH7		0x67	/* Peripheral to Memory (SI1) */
#define MP200_DMAC_P2M_LCH8		0x68	/* Peripheral to Memory (USC) */
#define MP200_DMAC_P2M_LCH9		0x69	/* Peripheral to Memory (PM0) */
#define MP200_DMAC_P2M_LCH10		0x6a	/* Peripheral to Memory (PM1) */
#define MP200_DMAC_P2M_LCH11		0x6b	/* Peripheral to Memory (PM2) */
#define MP200_DMAC_P2M_LCH12		0x6c	/* Peripheral to Memory (SP0) */
#define MP200_DMAC_P2M_LCH13		0x6d	/* Peripheral to Memory (SP1) */
#define MP200_DMAC_P2M_LCH14		0x6e	/* Peripheral to Memory (SP2) */
#define MP200_DMAC_P3_MAX_L_CHANNELS	15
#endif				/* CONFIG_MP201 */

typedef struct {
	volatile u32 aadd;	/* +0x00 source (start) address */
	volatile u32 aadp;	/* +0x04 source address pointer */
	volatile u32 aoff;	/* +0x08 source address offset (lower 16bit) */
	volatile u32 asize;	/* +0x0c source block size (lower 16bit) */
	volatile u32 asize_count;	/* +0x10 source block count (lower 4bit) */
	volatile u32 rfu0;	/* +0x14 */
	volatile u32 rfu1;	/* +0x18 */
	volatile u32 rfu2;	/* +0x1c */
	volatile u32 badd;	/* +0x20 destination (start) address */
	volatile u32 badp;	/* +0x24 destination address pointer */
	volatile u32 boff;	/* +0x28 destination address offset (lower 16bit) */
	volatile u32 bsize;	/* +0x2c destination block size (lower 16bit) */
	volatile u32 bsize_count;	/* +0x30 destination block count (lower 4bit) */
	volatile u32 rfu3;	/* +0x34 */
	volatile u32 rfu4;	/* +0x38 */
	volatile u32 rfu5;	/* +0x3c */
	volatile u32 leng;	/* +0x40 length (lower 24bit) */
	volatile u32 rcount;	/* +0x44 read length count (lower 24bit) */
	volatile u32 wcount;	/* +0x48 write length count (lower 24bit) */
	volatile u32 size;	/* +0x4c block size (lower 16bit) */
	volatile u32 mode;	/* +0x50 mode (lower 2bit and bit8) */
	volatile u32 time;	/* +0x54 timer (lower 24bit) */
	volatile u32 time_count;	/* +0x58 timer count (lower 24bit) */
	volatile u32 rfu6[64 - 23];	/* +0x5c fill up to offset of 0xff */
} dma_regs_t;

/* for `aoff' `boff' setting */
#ifdef CONFIG_MP201
#define MP200_DMAC_ASIGN_NORMAL		0x00000000
#define MP200_DMAC_ASIGN_REVERSE	0x00010000
#define MP200_DMAC_BSIGN_NORMAL		0x00000000
#define MP200_DMAC_BSIGN_REVERSE	0x00010000
#else
#define MP200_DMAC_OFFSET_ADD		0x00000000
#define MP200_DMAC_OFFSET_SUB		0x00010000
#endif				/* CONFIG_MP201 */

/* for `mode' setting */

#ifdef CONFIG_MP201
#define MP200_DMAC_DEFMODE_32BIT	( MP200_DMAC_AMODE_BIT32 |	\
					  MP200_DMAC_ENDI_R3210 |	\
					  MP200_DMAC_ENDI_W3210 )
#define MP200_DMAC_DEFMODE_16BIT	( MP200_DMAC_AMODE_BIT16 |	\
					  MP200_DMAC_ENDI_R10 |		\
					  MP200_DMAC_ENDI_W10 )
#define MP200_DMAC_DEFMODE_8BIT		MP200_DMAC_AMODE_BIT8
#endif				/* CONFIG_MP201 */

#define MP200_DMAC_BMODE_REPEAT 	0x100
#define MP200_DMAC_AMODE_BIT32		(0x0 << 4)
#define MP200_DMAC_AMODE_BIT16		(0x1 << 4)
#define MP200_DMAC_AMODE_BIT8		(0x2 << 4)
#define MP200_DMAC_AMODE_TIME		0x4
#ifndef CONFIG_MP201
#define MP200_DMAC_AMODE_ENDI		0x2
#endif				/* CONFIG_MP201 */
#define MP200_DMAC_AMODE_REPEAT 	0x1

#ifdef CONFIG_MP201
#define MP200_DMAC_ENDI_R		16
#define MP200_DMAC_ENDI_W		24
#define MP200_DMAC_ENDI(HH, HL, LH, LL, RW)	\
		((((HH) << 6) | ((HL) << 4) | ((LH) << 2) | (LL)) << (RW))

/* Endian(Read) for 32bit */
#define MP200_DMAC_ENDI_R0123		MP200_DMAC_ENDI(0, 1, 2, 3, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R0132		MP200_DMAC_ENDI(0, 1, 3, 2, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R0213		MP200_DMAC_ENDI(0, 2, 1, 3, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R0231		MP200_DMAC_ENDI(0, 2, 3, 1, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R0312		MP200_DMAC_ENDI(0, 3, 1, 2, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R0321		MP200_DMAC_ENDI(0, 3, 2, 1, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R1023		MP200_DMAC_ENDI(1, 0, 2, 3, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R1032		MP200_DMAC_ENDI(1, 0, 3, 2, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R1203		MP200_DMAC_ENDI(1, 2, 0, 3, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R1230		MP200_DMAC_ENDI(1, 2, 3, 0, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R1302		MP200_DMAC_ENDI(1, 3, 0, 2, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R1320		MP200_DMAC_ENDI(1, 3, 2, 0, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R2013		MP200_DMAC_ENDI(2, 0, 1, 3, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R2031		MP200_DMAC_ENDI(2, 0, 3, 1, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R2103		MP200_DMAC_ENDI(2, 1, 0, 3, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R2130		MP200_DMAC_ENDI(2, 1, 3, 0, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R2301		MP200_DMAC_ENDI(2, 3, 0, 1, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R2310		MP200_DMAC_ENDI(2, 3, 1, 0, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R3012		MP200_DMAC_ENDI(3, 0, 1, 2, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R3021		MP200_DMAC_ENDI(3, 0, 2, 1, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R3102		MP200_DMAC_ENDI(3, 1, 0, 2, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R3120		MP200_DMAC_ENDI(3, 1, 2, 0, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R3201		MP200_DMAC_ENDI(3, 2, 0, 1, MP200_DMAC_ENDI_R)
#define MP200_DMAC_ENDI_R3210		MP200_DMAC_ENDI(3, 2, 1, 0, MP200_DMAC_ENDI_R)

/* Endian(Write) for 32bit */
#define MP200_DMAC_ENDI_W0123		MP200_DMAC_ENDI(0, 1, 2, 3, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W0132		MP200_DMAC_ENDI(0, 1, 3, 2, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W0213		MP200_DMAC_ENDI(0, 2, 1, 3, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W0231		MP200_DMAC_ENDI(0, 2, 3, 1, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W0312		MP200_DMAC_ENDI(0, 3, 1, 2, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W0321		MP200_DMAC_ENDI(0, 3, 2, 1, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W1023		MP200_DMAC_ENDI(1, 0, 2, 3, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W1032		MP200_DMAC_ENDI(1, 0, 3, 2, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W1203		MP200_DMAC_ENDI(1, 2, 0, 3, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W1230		MP200_DMAC_ENDI(1, 2, 3, 0, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W1302		MP200_DMAC_ENDI(1, 3, 0, 2, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W1320		MP200_DMAC_ENDI(1, 3, 2, 0, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W2013		MP200_DMAC_ENDI(2, 0, 1, 3, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W2031		MP200_DMAC_ENDI(2, 0, 3, 1, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W2103		MP200_DMAC_ENDI(2, 1, 0, 3, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W2130		MP200_DMAC_ENDI(2, 1, 3, 0, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W2301		MP200_DMAC_ENDI(2, 3, 0, 1, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W2310		MP200_DMAC_ENDI(2, 3, 1, 0, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W3012		MP200_DMAC_ENDI(3, 0, 1, 2, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W3021		MP200_DMAC_ENDI(3, 0, 2, 1, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W3102		MP200_DMAC_ENDI(3, 1, 0, 2, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W3120		MP200_DMAC_ENDI(3, 1, 2, 0, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W3201		MP200_DMAC_ENDI(3, 2, 0, 1, MP200_DMAC_ENDI_W)
#define MP200_DMAC_ENDI_W3210		MP200_DMAC_ENDI(3, 2, 1, 0, MP200_DMAC_ENDI_W)

/* Endian(Read) for 16bit */
#define MP200_DMAC_ENDI_R01		MP200_DMAC_ENDI_R2301
#define MP200_DMAC_ENDI_R10		MP200_DMAC_ENDI_R3210

/* Endian(Write) for 16bit */
#define MP200_DMAC_ENDI_W01		MP200_DMAC_ENDI_W2301
#define MP200_DMAC_ENDI_W10		MP200_DMAC_ENDI_W3210
#endif				/* CONFIG_MP201 */

/* for checking interrupt status */
#define MP200_DMAC_INT_TIME_WR		0x80
#define MP200_DMAC_INT_ERROR_WR 	0x40
#define MP200_DMAC_INT_BLOCK_WR 	0x20
#define MP200_DMAC_INT_LENG_WR		0x10
#define MP200_DMAC_INT_TIME_RD		0x08
#define MP200_DMAC_INT_ERROR_RD 	0x04
#define MP200_DMAC_INT_BLOCK_RD 	0x02
#define MP200_DMAC_INT_LENG_RD		0x01

/* for setting interrupt mask */
#define MP200_DMAC_INT_TIME_EN		0x08
#define MP200_DMAC_INT_ERROR_EN 	0x04
#define MP200_DMAC_INT_BLOCK_EN 	0x02
#define MP200_DMAC_INT_LENG_EN		0x01

typedef void (*dma_callback_t) (void *data, int intsts, int intrawsts);

extern int mp200_request_dma(int channel, const char *device_id,
			     dma_callback_t callback, void *data,
			     dma_regs_t ** dma_regs);
extern void mp200_free_dma(int channel);
extern int mp200_start_dma(int channel, dma_addr_t src_ptr, u_int size,
			   dma_addr_t dst_ptr, int intmask);
extern int mp200_dma_status(int channel);
extern int mp200_dma_busy(void);
extern void mp200_clear_dma(int channel);
extern void mp200_reset_dma(int channel);
extern dma_addr_t mp200_get_dma_pos(int channel);
extern void mp200_stop_dma(int channel);
extern int mp200_dma_writable_flag(int channel);

#endif				/* __ASM_ARM_ARCH_MP200_DMA_H */
