#ifndef ICX1087_BEEP_H
#define ICX1087_BEEP_H
/*
 * icx1087_beep.h -- Sony ICX1087 platform beep interface
 *
 * Copyright 2006 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <asm/ioctl.h>

/* Set/Get beep volume (0 to 100) */
#define BEEP_SET_VOLUME		_IOW('b', 0x01, int)
#define BEEP_GET_VOLUME		_IOR('b', 0x02, int)
	#define BEEP_DEFAULT_VOLUME		(4)

/* play beep pattern */
#define BEEP_PLAY			_IOW('b', 0x03, beep_play_t)

/* ex. 1kHz 120ms beep on
  beep_time_t	ptn = { .on_time = 120, .off_time = 0 };
  beep_play_t	param = {
    freq:1000, pri:BEEP_PRI_NORMAL, repeat:0, bytes:sizeof(ptn), pattern:&ptn
  };
  ioctl(fd, BEEP_PLAY, &param);
*/

typedef enum {
	BEEP_PRI_MIN,
	BEEP_PRI_LOW,
	BEEP_PRI_NORMAL,
	BEEP_PRI_HIGH,
	BEEP_PRI_MAX
} beep_pri_t;

#define MAX_BEEP_PATTERN		(20)
#define MAX_BEEP_PATTERN_BYTES	(MAX_BEEP_PATTERN * sizeof(beep_time_t))

typedef struct {
	unsigned short	on_time;	/* beep on time (ms) */
	unsigned short	off_time;	/* beep off time (ms) */
} beep_time_t;

typedef struct {
	unsigned			freq;		/* frequency (Hz) */
	beep_pri_t			pri;		/* priority. same pri is a post-winning */
	int					repeat;		/* pattern repeat count. -1 is infinity */
	unsigned			bytes;		/* pattern length */
	const beep_time_t	*pattern;	/* point to beep pattern array */
} beep_play_t;

/* stop beep */
#define BEEP_STOP			_IOW('b', 0x04, int)

/* get beep status */
#define BEEP_STATUS			_IOWR('b', 0x05, int)
	#define BEEP_STATUS_STOP		(0)
	#define BEEP_STATUS_INTERVAL	(1)
	#define BEEP_STATUS_PLAY_ON		(2)
	#define BEEP_STATUS_PLAY_OFF	(3)

/* Set/Get beep interval time (ms) */
#define BEEP_SET_INTERVAL	_IOW('b', 0x06, int)
#define BEEP_GET_INTERVAL	_IOR('b', 0x07, int)
	#define BEEP_DEFAULT_INTERVAL	(50)

/* synchronization for play finish */
#define BEEP_SYNC			_IO('b', 0x08)

#endif	/* ICX1087_BEEP_H */
