/*
 * icx1087_power.h -- ICX1087 Power Path driver interface
 *
 * Copyright 2006 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef ICX1087_POWER_H
#define ICX1087_POWER_H

#include <asm/ioctl.h>


#define ICX1087_POWER_IOC_MAGIC  'P'

#define ICX1087_POWER_GET_STATUS        _IOR(ICX1087_POWER_IOC_MAGIC, 0, int)
#define ICX1087_POWER_SET_VBUS_LIMIT    _IOW(ICX1087_POWER_IOC_MAGIC, 1, int)
#define ICX1087_POWER_GET_BAT_ADVAL     _IOR(ICX1087_POWER_IOC_MAGIC, 2, int)
#define ICX1087_POWER_GET_BAT_GAUGE     _IOR(ICX1087_POWER_IOC_MAGIC, 3, int)
#define ICX1087_POWER_ENABLE_CHARGER    _IOW(ICX1087_POWER_IOC_MAGIC, 4, int)
#define ICX1087_POWER_ENABLE_DEBUG_PORT _IOW(ICX1087_POWER_IOC_MAGIC, 5, int)
#define ICX1087_POWER_IS_FULLY_CHARGED  _IOR(ICX1087_POWER_IOC_MAGIC, 6, int)


#define ICX1087_POWER_STATUS_CHARGE   0x80
#define ICX1087_POWER_STATUS_DETUSB   0x20
#define ICX1087_POWER_STATUS_DETADP   0x10
#define ICX1087_POWER_STATUS_DETBAT   0x08   /* should not be used */

#define ICX1087_POWER_BAT_LOWBAT       0     /* lowbat */
#define ICX1087_POWER_BAT_BLINK        1     /* blink  */
#define ICX1087_POWER_BAT_GAUGE1       2     /* O      */
#define ICX1087_POWER_BAT_GAUGE2       3     /* OO     */
#define ICX1087_POWER_BAT_GAUGE3       4     /* OOO    */
#define ICX1087_POWER_BAT_GAUGE4       5     /* OOOO   */


int icx1087_set_vbus_limit(int nLimit, int nUSB_AC_Adatpro);
int icx1087_get_bat_voltage(int nMode);
unsigned int icx1087_get_power_status(void);
void icx1087_vbus_changed(void);
int icx1087_check_power_state_for_bl(void);
int icx1087_get_power_state_for_bl(void);


#endif /* ICX1087_POWER_H */

