/*
 * File Name       : /include/asm-arm/arch-mp200/img.h
 * Function        : IMG MMIO definitions, and Driver - Driver I/F of IMG driver
 * Release Version : Ver 1.00
 * Release Date    : 2005/05/25
 *
 * Copyright (C) NEC Electronics Corporation 2005
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef _IMG_H_
#define _IMG_H_

#if !defined(MP200_FB) || defined(CONFIG_MP200_IMG) || defined(CONFIG_MP200_IMG_MODULE)
/*===============================================================*/
/* IMG Driver Definitions                                        */
/*===============================================================*/

#define IMG_GUARDTIME_DEFAULT      70	/* default waiting time to cancel status of mixing images */

/* Parameters of mp200_img_cancel (cancel request) */
#define IMG_CANCEL_SYNC             1	/* SYNC flag for cancel request               */

/* Statuses of IMG request */
#define IMG_REQ_STATUS_REGISTERED   1	/* registered                                 */
#define IMG_REQ_STATUS_RUNNING      2	/* running                                    */
#define IMG_REQ_STATUS_WAIT_CANCEL  3	/* waiting for cancelled request to be done   */
#define IMG_REQ_STATUS_CANCELLED    4	/* cancelled                                  */
#define IMG_REQ_STATUS_DONE         5	/* done                                       */
#define IMG_REQ_STATUS_ERROR        6	/* failed                                     */

/* Returns of mp200_img_cancel (cancel request) */
#define IMG_CANCEL_DONE             0	/* cancelled                                  */
#define IMG_CANCEL_WAIT             1	/* waiting for cancelled request to be done   */
#define IMG_CANCEL_NO_REQ           2	/* target request does not exist              */

/* Returns of mp200_img_reschedule (run Normal Resuest) */
#define IMG_RESCHEDULE_START        0	/* started                                    */
#define IMG_RESCHEDULE_NO_REQ      -1	/* no request                                 */
#define IMG_RESCHEDULE_BUSY        -2	/* IMG busy                                   */

/* Parameters of callback function */
#define IMG_CALLBACK_SUCCESS        0	/* IMG process is completed (Success)         */
#define IMG_CALLBACK_SET_PARAM      1	/* set values to IMG registers (Success)      */
#define IMG_CALLBACK_CANCEL         2	/* cancelled request is done (Success)        */
#define IMG_CALLBACK_FAIL          -1	/* can't start IMG process (Error)            */
#define IMG_CALLBACK_IMG_ERROR     -2	/* IMG ERROR interrupt (Error)                */
#define IMG_CALLBACK_IMG_STOP      -3	/* IMG RETRY/SPLIT interrupt (Error)          */

struct mp200_img_req;		/* Declaration of structure tag (incomplete type) */

/*===============================================================*/
/* Callback function type                                        */
/*===============================================================*/
typedef void (*img_callback_func) (struct mp200_img_req * p, int flag);

/*===============================================================*/
/* IMG register setting information structure                    */
/*===============================================================*/
typedef struct mp200_img_reg_info {
	unsigned long reg;	/* offset to IMG register               */
	unsigned long data;	/* data to set                          */
} MP200_IMG_REG_INFO;

/*===============================================================*/
/* IMG request information structure                             */
/*===============================================================*/
typedef struct mp200_img_req {
	struct mp200_img_req *next;	/* pointer to next IMG requst (used by IMG
					   driver)                 */
	struct mp200_img_req *prev;	/* pointer to previous IMG requst (used by
					   IMG driver)             */
	int status;		/* Status of IMG request (used by IMG
				   driver)                      */
	img_callback_func callback;	/* callback function
					 */
	int lastLayer;		/* 1:last layer of a series of mix images
				   (only used with Preferred Request) */
	unsigned long data;	/* optional data (used by caller)
				 */
	struct mp200_img_reg_info img_reg_info[];	/* IMG register setting
							   informations */
	/* repeat until end condition (max 50 pairs)
	 *   end condition :
	 *   
	 *         set 0x00000001 to IMG_REQ (request to start IMG process)
	 *         reg=IMG_REQ data=0x00000001,
	 *         or reg=-1(0xFFFFFFFF)
	 */
} MP200_IMG_REQ;
#define IMG_REG_INFO_MAX  50	/* max number of IMG register setting informations                 */

#endif				/* !defined(MP200_FB) || defined(CONFIG_MP200_IMG) || defined(CONFIG_MP200_IMG_MODULE) */

/*===============================================================*/
/* IMG MMIO definitions   for uPD77601                           */
/*===============================================================*/
#include <asm/arch/hardware.h>
#define IMAGEPROC_OFFSET    MP200_IMAGEPROC_OFFSET
#define IMAGEPROC_BASE      MP200_IMAGEPROC_BASE
#define IMAGEPROC_SIZE      MP200_IMAGEPROC_SIZE

/*===============================================================*/
/* IMG register definitions   for uPD77601                       */
/*===============================================================*/
//                                                          Bit    R/W    Reset
// Functions                                               18:0    R/W    0x0
#define IMG_MODE            (0x00)

// Request for Processing                                   0:0    R/W    0x0
#define IMG_REQ             (0x04)

// Current Processing Status                                0:0    R      0x0
#define IMG_ACK             (0x08)

// Interuppt Status                                         2:0    R      0x0
#define IMG_STATUS          (0x0C)

// Interuppt Raw Status                                     2:0    R      0x0
#define IMG_RAWSTATUS       (0x10)

// Interuppt Enable Set                                     2:0    R/W    0x0
#define IMG_ENSET           (0x14)

// Interuppt Enable Clear                                   2:0    W      0x0
#define IMG_ENCLR           (0x18)

// Interuppt FF Clear                                       2:0    W      0x0
#define IMG_FFCLR           (0x1C)

// Address increment value for the source image            12:0    R/W    0x000
#define IMG_SRCSIZE_B       (0x20)

// Address increment value for the source image            12:0    R/W    0x000
#define IMG_SRCSIZE_F       (0x24)

// Address increment value for the destination image       12:0    R/W    0x000
#define IMG_DSTSIZE         (0x28)

// Y/RGB Plane Address for the source image                31:0    R/W    0x0000_0000
#define IMG_SRCYRGBADR_B    (0x30)

// Y/RGB Plane Address for the source image                31:0    R/W    0x0000_0000
#define IMG_SRCYRGBADR_F    (0x34)

// Y/RGB Plane Address for the destination image           31:0    R/W    0x0000_0000
#define IMG_DSTYRGBADR      (0x38)

// UV Plane Address for the source image                   31:0    R/W    0x0000_0000
#define IMG_SRCUVADR_B      (0x40)

// UV Plane Address for the source image                   31:0    R/W    0x0000_0000
#define IMG_SRCUVADR_F      (0x44)

// UV Plane Address for the destination image              31:0    R/W    0x0000_0000
#define IMG_DSTUVADR        (0x48)

// Horizontal size for the source image                    10:0    R/W    0x008
#define IMG_SRCHSIZE_B      (0x50)

// Horizontal size for the source image                    10:0    R/W    0x008
#define IMG_SRCHSIZE_F      (0x54)

// Vartical size of the source image                       10:0    R/W    0x008
#define IMG_SRCVSIZE_B      (0x58)

// Vartical size of the source image                       10:0    R/W    0x008
#define IMG_SRCVSIZE_F      (0x5C)

// Horizontal location offset for front image              10:0    R/W    0x000
#define IMG_OFFSETX_F       (0x60)

// Vertical location offset for front image                10:0    R/W    0x000
#define IMG_OFFSETY_F       (0x64)

// Front Image Mask Color                                  15:0    R/W    0x0000
#define IMG_MASKCOLR_F      (0x68)

// Front Image Transpearency                                7:0    R/W    0xFF
#define IMG_ALPHA_F         (0x6C)

// Horizontal size for output image                        10:0    R/W    0x008
#define IMG_DSTHSIZE        (0x70)

// Vartical size for output image                          10:0    R/W    0x008
#define IMG_DSTVSIZE        (0x74)

// Horizontal Enlargement and Reduction steps for resizing 13:0    R/W    0x100
#define IMG_HSTEP           (0x78)

// Vertical Enlargement and Reducation steps for resizing  13:0    R/W    0x100
#define IMG_VSTEP           (0x7C)

// Horizontal Enlargement and Reduction rate for resizing   8:0    R/W    0x40
#define IMG_HFOLD           (0x80)

// Vertical Enlargement and Reducation rate for resizing    8:0    R/W    0x40
#define IMG_VFOLD           (0x84)

// Kernel Matrix 00 Component                              13:0    R/W    0x100
#define IMG_FLTMTX_00       (0x88)

// Kernel Matrix 01 Component                              13:0    R/W    0x0
#define IMG_FLTMTX_01       (0x8C)

// Kernel Matrix 02 Component                              13:0    R/W    0x0
#define IMG_FLTMTX_02       (0x90)

// Kernel Matrix 10 Component                              13:0    R/W    0x0
#define IMG_FLTMTX_10       (0x94)

// Kernel Matrix 11 Component                              13:0    R/W    0x100
#define IMG_FLTMTX_11       (0x98)

// Kernel Matrix 12 Component                              13:0    R/W    0x0
#define IMG_FLTMTX_12       (0x9C)

// Kernel Matrix 20 Component                              13:0    R/W    0x0
#define IMG_FLTMTX_20       (0xA0)

// Kernel Matrix 21 Component                              13:0    R/W    0x0
#define IMG_FLTMTX_21       (0xA4)

// Kernel Matrix 22 Component                              13:0    R/W    0x100
#define IMG_FLTMTX_22       (0xA8)

// Pixel Endian Component                                   1:0    R/W    0x0
#define IMG_PEL_ENDIAN      (0xAC)

// HADDR for debuging                                      31:0    R/W    0x0
#define IMG_DBG_HADDR       (0xB0)

// Regfile testing address                                  7:0    R/W    0x0
#define IMG_TEST_ADR        (0xB4)

// Regfile testing data                                    31:0    R/W    0x0
#define IMG_TEST_DATA       (0xB8)

/*===============================================================*/
/* IMG register bit assigns                                      */
/*===============================================================*/

/*------------------------------*/
/* IMG_MODE                     */
/*------------------------------*/
#define IMG_MASK_COL_BIT        0x00080000
#define IMG_MASK_COL_SFT        0x13

#define IMG_RLINV_BIT           0x00040000
#define IMG_RLINV_SFT           0x12

#define IMG_UDINV_BIT           0x00020000
#define IMG_UDINV_SFT           0x11

#if 1
#define IMG_OP_MODE_00_BIT      0x00000000
#define IMG_OP_MODE_01_BIT      0x00001000
#define IMG_OP_MODE_02_BIT      0x00002000
#define IMG_OP_MODE_03_BIT      0x00003000
#define IMG_OP_MODE_04_BIT      0x00004000
#define IMG_OP_MODE_05_BIT      0x00005000
#define IMG_OP_MODE_06_BIT      0x00006000
#define IMG_OP_MODE_07_BIT      0x00007000
#define IMG_OP_MODE_08_BIT      0x00010200
#define IMG_OP_MODE_09_BIT      0x00010500
#define IMG_OP_MODE_10_BIT      0x00011300
#define IMG_OP_MODE_11_BIT      0x00011400
#define IMG_OP_MODE_12_BIT      0x00012000
#define IMG_OP_MODE_13_BIT      0x00012700
#define IMG_OP_MODE_14_BIT      0x00013100
#define IMG_OP_MODE_15_BIT      0x00013600
#define IMG_OP_MODE_16_BIT      0x00014100
#define IMG_OP_MODE_17_BIT      0x00014600
#define IMG_OP_MODE_18_BIT      0x00015000
#define IMG_OP_MODE_19_BIT      0x00015700
#define IMG_OP_MODE_20_BIT      0x00016300
#define IMG_OP_MODE_21_BIT      0x00016400
#define IMG_OP_MODE_22_BIT      0x00017200
#define IMG_OP_MODE_23_BIT      0x00017500
#define IMG_OP_MODE_24_BIT      0x00008000
#define IMG_OP_MODE_25_BIT      0x00009000
#define IMG_OP_MODE_26_BIT      0x0000a000
#else
#define IMG_OVERLAY_BIT         0x00010000
#define IMG_OVERLAY_SHT         0x10

#define IMG_OPB_BIT             0x0000F000
#define IMG_OPB_SFT             0xC

#define IMG_OPF_BIT             0x00000F00
#define IMG_OPF_SFT             0x8
#endif

#define IMG_RFILTER_B_BIT       0x00000080
#define IMG_RFILTER_B_SFT       0x7

#define IMG_RFILTER_F_BIT       0x00000040
#define IMG_RFILTER_F_SFT       0x6

#define IMG_PELPACK_BIT         0x00000020
#define IMG_PELPACK_SFT         0x5

#define IMG_DITHER_B_BIT        0x00000010
#define IMG_DITHER_B_SFT        0x4

#define IMG_DITHER_F_BIT        0x00000008
#define IMG_DITHER_F_SFT        0x3

#define IMG_DSTFMT_BIT          0x00000004
#define IMG_DSTFMT_SFT          0x2

#define IMG_SRCFMT_B_BIT        0x00000002
#define IMG_SRCFMT_B_SFT        0x1

#define IMG_SRCFMT_F_BIT        0x00000001
#define IMG_SRCFMT_F_SFT        0x0

/*------------------------------*/
/* IMG_REQ                      */
/*------------------------------*/
#define IMG_REQ_BIT             0x00000001
#define IMG_REQ_SFT             0x0

/*------------------------------*/
/* IMG_ACK                      */
/*------------------------------*/
#define IMG_ACK_BIT             0x00000001
#define IMG_ACK_SFT             0x0

/*------------------------------*/
/* IMG_STATUS                   */
/*------------------------------*/
#define IMG_STOP_BIT            0x00000004
#define IMG_STOP_SFT            0x2

#define IMG_ERROR_BIT           0x00000002
#define IMG_ERROR_SFT           0x1

#define IMG_STAT_BIT            0x00000001
#define IMG_STAT_SFT            0x0

/*------------------------------*/
/* IMG_RAWSTATUS                */
/*------------------------------*/
#define IMG_RAWSTOP_BIT         0x00000004
#define IMG_RAWSTOP_SFT         0x2

#define IMG_RAWERROR_BIT        0x00000002
#define IMG_RAWERROR_SFT        0x1

#define IMG_RAWSTAT_BIT         0x00000001
#define IMG_RAWSTAT_SFT         0x0

/*------------------------------*/
/* IMG_ENSET                    */
/*------------------------------*/
#define IMG_ENSETSTOP_BIT       0x00000004
#define IMG_ENSETSTOP_SFT       0x2

#define IMG_ENSETERROR_BIT      0x00000002
#define IMG_ENSETERROR_SFT      0x1

#define IMG_ENSETSTAT_BIT       0x00000001
#define IMG_ENSETSTAT_SFT       0x0

/*------------------------------*/
/* IMG_ENCLR                    */
/*------------------------------*/
#define IMG_ENCLRSTOP_BIT       0x00000004
#define IMG_ENCLRSTOP_SFT       0x2

#define IMG_ENCLRERROR_BIT      0x00000002
#define IMG_ENCLRERROR_SFT      0x1

#define IMG_ENCLRSTAT_BIT       0x00000001
#define IMG_ENCLRSTAT_SFT       0x0

/*------------------------------*/
/* IMG_FFCLR                    */
/*------------------------------*/
#define IMG_FFCLRSTOP_BIT       0x00000004
#define IMG_FFCLRSTOP_SFT       0x2

#define IMG_FFCLRERROR_BIT      0x00000002
#define IMG_FFCLRERROR_SFT      0x1

#define IMG_FFCLRSTAT_BIT       0x00000001
#define IMG_FFCLRSTAT_SFT       0x0

/*------------------------------*/
/* IMG_SRCSIZE_B                */
/*------------------------------*/
#define IMG_SRCSIZE_B_BIT       0x00001FFF
#define IMG_SRCSIZE_B_SFT       0x0

/*------------------------------*/
/* IMG_SRCSIZE_F                */
/*------------------------------*/
#define IMG_SRCSIZE_F_BIT       0x00001FFF
#define IMG_SRCSIZE_F_SFT       0x0

/*------------------------------*/
/* IMG_DSTSIZE                  */
/*------------------------------*/
#define IMG_DSTSIZE_BIT         0x00001FFF
#define IMG_DSTSIZE_SFT         0x0

/*------------------------------*/
/* IMG_SRCYRGBADR_B             */
/*------------------------------*/
#define IMG_SRCYRGBADR_B_BIT    0xFFFFFFFF
#define IMG_SRCYRGBADR_B_SFT    0x0

/*------------------------------*/
/* IMG_SRCYRGBADR_F             */
/*------------------------------*/
#define IMG_SRCYRGBADR_F_BIT    0xFFFFFFFF
#define IMG_SRCYRGBADR_F_SFT    0x0

/*------------------------------*/
/* IMG_DSTYRGBADR               */
/*------------------------------*/
#define IMG_DSTYRGBADR_BIT      0xFFFFFFFF
#define IMG_DSTYRGBADR_SFT      0x0

/*------------------------------*/
/* IMG_SRCUVADR_B               */
/*------------------------------*/
#define IMG_SRCUVADR_B_BIT      0xFFFFFFFF
#define IMG_SRCUVADR_B_SFT      0x0

/*------------------------------*/
/* IMG_SRCUVADR_F               */
/*------------------------------*/
#define IMG_SRCUVADR_F_BIT      0xFFFFFFFF
#define IMG_SRCUVADR_F_SFT      0x0

/*------------------------------*/
/* IMG_DSTUVADR                 */
/*------------------------------*/
#define IMG_DSTUVADR_BIT        0xFFFFFFFF
#define IMG_DSTUVADR_SFT        0x0

/*------------------------------*/
/* IMG_SRCHSIZE_B               */
/*------------------------------*/
#define IMG_SRCHSIZE_B_BIT      0x000007FF
#define IMG_SRCHSIZE_B_SFT      0x0

/*------------------------------*/
/* IMG_SRCHSIZE_F               */
/*------------------------------*/
#define IMG_SRCHSIZE_F_BIT      0x000007FF
#define IMG_SRCHSIZE_F_SFT      0x0

/*------------------------------*/
/* IMG_SRCHSIZE_B               */
/*------------------------------*/
#define IMG_SRCVSIZE_B_BIT      0x000007FF
#define IMG_SRCVSIZE_B_SFT      0x0

/*------------------------------*/
/* IMG_SRCVSIZE_F               */
/*------------------------------*/
#define IMG_SRCVSIZE_F_BIT      0x000007FF
#define IMG_SRCVSIZE_F_SFT      0x0

/*------------------------------*/
/* IMG_OFFSETX_F                */
/*------------------------------*/
#define IMG_OFFSETX_F_BIT       0x000007FF
#define IMG_OFFSETX_F_SFT       0x0

/*------------------------------*/
/* IMG_OFFSETY_F                */
/*------------------------------*/
#define IMG_OFFSETY_F_BIT       0x000007FF
#define IMG_OFFSETY_F_SFT       0x0

/*------------------------------*/
/* IMG_MASKCOLR_F               */
/*------------------------------*/
#define IMG_MASKCOLR_F_BIT      0x0000FFFF
#define IMG_MASKCOLR_F_SFT      0x0

/*------------------------------*/
/* IMG_ALPHA_F                  */
/*------------------------------*/
#define IMG_ALPHA_F_BIT         0x000000FF
#define IMG_ALPHA_F_SFT         0x0

/*------------------------------*/
/* IMG_DSTHSIZE                 */
/*------------------------------*/
#define IMG_DSTHSIZE_BIT        0x000007FF
#define IMG_DSTHSIZE_SFT        0x0

/*------------------------------*/
/* IMG_DSTVSIZE                 */
/*------------------------------*/
#define IMG_DSTVSIZE_BIT        0x000007FF
#define IMG_DSTVSIZE_SFT        0x0

/*------------------------------*/
/* IMG_HSTEP                    */
/*------------------------------*/
#define IMG_HSTEP_BIT           0x00003FFF
#define IMG_HSTEP_SFT           0x0

/*------------------------------*/
/* IMG_VSTEP                    */
/*------------------------------*/
#define IMG_VSTEP_BIT           0x00003FFF
#define IMG_VSTEP_SFT           0x0

/*------------------------------*/
/* IMG_HFOLD                    */
/*------------------------------*/
#define IMG_HFOLD_BIT           0x000001FF
#define IMG_HFOLD_SFT           0x0

/*------------------------------*/
/* IMG_VFOLD                    */
/*------------------------------*/
#define IMG_VFOLD_BIT           0x000001FF
#define IMG_VFOLD_SFT           0x0

/*------------------------------*/
/* IMG_FLTMTX_00                */
/*------------------------------*/
#define IMG_FLTMTX_00_BIT       0x00001FFF
#define IMG_FLTMTX_00_SFT       0x0

/*------------------------------*/
/* IMG_FLTMTX_01                */
/*------------------------------*/
#define IMG_FLTMTX_01_BIT       0x00001FFF
#define IMG_FLTMTX_01_SFT       0x0

/*------------------------------*/
/* IMG_FLTMTX_02                */
/*------------------------------*/
#define IMG_FLTMTX_02_BIT       0x00001FFF
#define IMG_FLTMTX_02_SFT       0x0

/*------------------------------*/
/* IMG_FLTMTX_10                */
/*------------------------------*/
#define IMG_FLTMTX_10_BIT       0x00001FFF
#define IMG_FLTMTX_10_SFT       0x0

/*------------------------------*/
/* IMG_FLTMTX_11                */
/*------------------------------*/
#define IMG_FLTMTX_11_BIT       0x00001FFF
#define IMG_FLTMTX_11_SFT       0x0

/*------------------------------*/
/* IMG_FLTMTX_12                */
/*------------------------------*/
#define IMG_FLTMTX_12_BIT       0x00001FFF
#define IMG_FLTMTX_12_SFT       0x0

/*------------------------------*/
/* IMG_FLTMTX_20                */
/*------------------------------*/
#define IMG_FLTMTX_20_BIT       0x00001FFF
#define IMG_FLTMTX_20_SFT       0x0

/*------------------------------*/
/* IMG_FLTMTX_21                */
/*------------------------------*/
#define IMG_FLTMTX_21_BIT       0x00001FFF
#define IMG_FLTMTX_21_SFT       0x0

/*------------------------------*/
/* IMG_FLTMTX_22                */
/*------------------------------*/
#define IMG_FLTMTX_22_BIT       0x00001FFF
#define IMG_FLTMTX_22_SFT       0x0

/*------------------------------*/
/* IMG_PEL_ENDIAN                */
/*------------------------------*/
#define IMG_PEL_ENDIAN_BIT      0x00000003
#define IMG_PEL_ENDIAN_SFT      0x0

/*------------------------------*/
/* IMG_PEL_ENDIAN               */
/*------------------------------*/
#define IMG_DBG_HADDR_BIT       0xFFFFFFFF
#define IMG_DBG_HADDR_SFT       0x0

/*------------------------------*/
/* IMG_TEST_ADR                 */
/*------------------------------*/
#define IMG_TEST_ADR_BIT        0x000000FF
#define IMG_TEST_ADR_SFT        0x0

/*------------------------------*/
/* IMG_TEST_DATA                */
/*------------------------------*/
#define IMG_TEST_DATA_BIT       0xFFFFFFFF
#define IMG_TEST_DATA_SFT       0x0


/*******************************************************************/
/************ Inter module communication definitions ***************/
/*******************************************************************/
#define MP200_IMG_DEV_NAME	"mp200_img"
#define MP200_IMG_DEV_FILE_NAME	"img"

#endif				/* _IMG_H_ */
