/*
 * File Name       : /include/asm-arm/arch-mp200/IMG_user.h
 * Function        : User - Driver I/F of IMG driver
 * Release Version : Ver 1.00
 * Release Date    : 2005/05/25
 *
 * Copyright (C) NEC Electronics Corporation 2005
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef _IMG_USER_H_
#define _IMG_USER_H_

#include <asm/ioctl.h>

#include "img.h"


/*============================================================================
* IOCTL    : IMGIOGET_PHYS_ADDR
* FUNCTION : get physical address of mmaped IO buffer
* INPUT    : user area addres
* OUTPUT   : physical address
*            Return
*              0 : Success
*             -1 : Fail
*============================================================================*/
typedef struct tagIMGIOGET_PHYS_ADDR{
    void *virt_addr;                /* user area virtual address (input) */
    long  phys_addr;                /* physical address (output) */
}_IMGIOGET_PHYS_ADDR;
#define SIZE_IMGIOGET_PHYS_ADDR 8

#define MAX_SIZE_IMGIOCTL SIZE_IMGIOGET_PHYS_ADDR

/*===============================================================*/
/* Write request information structure                           */
/*===============================================================*/
typedef struct tagIMG_DRV_REQ{
    long            request_id;     /* request ID              */
    size_t          request_size;   /* request size            */
    long            result_data;    /* result (output)         */
}_IMG_DRV_REQ;
#define SIZE_IMG_DRV_REQ 12

/*
 *INFORMATION for '_IMG_DRV_REQ' which is used by write() of IMG driver *
  request_id:
        command ID to request ex.)regist IMG request IMGWRITE_mp200_img_regist
  request_size:
        data size if data pointer or something like is in the structure.
        ignored by IMG driver when data pointer does not exist.  ex.)size of
        IMG request information
  result_data:
        status or something like to return.
*/


/*=============================================================================
* WRITE    : IMGWRITE_MP200_IMG_REGIST
* FUNCTION : register IMG request in Normal Request's queue.
*            can't register a new request before registered request is done.
*            open() without O_NONBLOCK : register the IMG request, wait for the
*            request to be done (BLOCK), and return.  open() with O_NONBLOCK :
*            register the IMG request, and return immediately (non BLOCK). not
*            wait for the request to be done.
* INPUT    : pointer to IMG request information
* OUTPUT   : Returns
*              IMG_REQ_REGISTERED(0) : the request is registered
*              IMG_REQ_COMPLETE(1)   : the request is registered and done
*              IMG_REQ_CANCELLED(2)  : the request is registered and cancelled
*              -EBUSY                : fail to register the request
*              result_data(Status of IMG process)
*              <without O_NONBLOCK and return=1(the request is registered and
*              done)>
*              IMG_CALLBACK_SUCCESS       : IMG process is completed (Success)
*              IMG_CALLBACK_SET_PARAM     : set values to IMG registers (Success)
*              IMG_CALLBACK_CANCEL        : cancelled request is done (Success)
*              IMG_CALLBACK_FAIL          : can't start IMG process (Error)
*              IMG_CALLBACK_IMG_ERROR     : IMG ERROR interrupt (Error)
*              IMG_CALLBACK_IMG_STOP      : IMG RETRY/SPLIT interrupt (Error)
*=============================================================================*/
typedef struct tagIMGWRITE_MP200_IMG_REGIST{
    _IMG_DRV_REQ    request_data;   /* Write request information
                                       request_data.request_id  :
                                       IMGWRITE_MP200_IMG_REGIST
                                       request_data.request_size: size of data
                                       pointed by img_request                 */
    MP200_IMG_REQ *img_request;     /* pointer to IMG request information     */
}_IMGWRITE_MP200_IMG_REGIST;
#define SIZE_IMGWRITE_MP200_IMG_REGIST (SIZE_IMG_DRV_REQ + 4)

#define IMG_REQ_REGISTERED 0        /* the request is registered              */
#define IMG_REQ_COMPLETE   1        /* the request is registered and done     */
#define IMG_REQ_CANCELLED  2        /* the request is registered and cancelled*/


/*=============================================================================
* WRITE    : IMGWRITE_MP200_IMG_CALLBACK
* FUNCTION : wait for the registered IMG request to be done, or get status of
*             the request.
*            open() without O_NONBLOCK :
*              wait for the request to be done (BLOCK), and return.
*            open() with O_NONBLOCK :
*              return immediately (non BLOCK). not wait for the request to be
*              done.
* INPUT    : pointer to IMG request information
* OUTPUT   : Returns
*              IMG_STATUS_DONE(0)         : IMG process is completed
*              IMG_STATUS_WAIT(1)         : IMG process is running (already
*                                           registered)
*              IMG_STATUS_NO_REQ(2)       : target request does not exist
*              IMG_STATUS_CANCELLED(3)    : target request is cancelled
*                                           result_data(status of IMG process)
*                                           <return=0(IMG request had been
*                                           done)>
*              IMG_CALLBACK_SUCCESS       : IMG process is completed (Success)
*              IMG_CALLBACK_SET_PARAM     : set values to IMG registers (Success)
*              IMG_CALLBACK_CANCEL        : cancelled request is done (Success)
*              IMG_CALLBACK_FAIL          : can't start IMG process (Error)
*              IMG_CALLBACK_IMG_ERROR     : IMG ERROR interrupt (Error)
*              IMG_CALLBACK_IMG_STOP      : IMG RETRY/SPLIT interrupt (Error)
*                                           <return=1(IMG request is running)>
*              IMG_REQ_STATUS_REGISTERED  : registered (before running)
*              IMG_REQ_STATUS_RUNNING     : running (running)
*              IMG_REQ_STATUS_WAIT_CANCEL : waiting for cancelled request to be
*                                           done (running)
*=============================================================================*/
typedef struct tagIMGWRITE_MP200_IMG_CALLBACK{
    _IMG_DRV_REQ    request_data;   /* Write request information
                                         request_data.request_id  :
                                              IMGWRITE_MP200_IMG_CALLBACK
                                         request_data.request_size: size of
                                              data pointed by img_request */
    MP200_IMG_REQ *img_request;     /* pointer to IMG request information                               */
}_IMGWRITE_MP200_IMG_CALLBACK;
#define SIZE_IMGWRITE_MP200_IMG_CALLBACK (SIZE_IMG_DRV_REQ + 4)

#define IMG_STATUS_DONE       0     /* IMG process is completed               */
#define IMG_STATUS_WAIT       1     /* IMG process is running (already
                                       registered) */
#define IMG_STATUS_NO_REQ     2     /* target request does not exist          */
#define IMG_STATUS_CANCELLED  3     /* target request is cancelled            */


/*=============================================================================
* WRITE    : IMGWRITE_MP200_IMG_CANCEL
* FUNCTION : cancel registered IMG request
*            target IMG request has not been started :
*                remove the target request from the queue.
*            target IMG request has been started (running) :
*              without IMG_CANCEL_SYNC :
*                set IMG_CANCEL_WAIT to the status of the request,
*                and wait for the request to be done.
*                Then set IMG_CALLBACK_CANCEL to result_data
*                as return of IMGWRITE_MP200_IMG_CALLBACK.
*              with IMG_CANCEL_SYNC    :
*                remove the target request from the queue immediately.
*                and skip waiting for the request to be done
*                (IMGWRITE_MP200_IMG_CALLBACK)
* INPUT    : pointer to IMG request information
*            SYNC flag
* OUTPUT   : Returns
*              IMG_CANCEL_DONE(0)   : cancelled
*              IMG_CANCEL_WAIT(1)   : waiting for cancelled request to be done
*              IMG_CANCEL_NO_REQ(2) : target request does not exist
*=============================================================================*/
typedef struct tagIMGWRITE_MP200_IMG_CANCEL{
    _IMG_DRV_REQ    request_data; /* Write request information                */
                                  /* request_data.request_id :
                                        IMGWRITE_MP200_IMG_CANCEL
                                        request_data.request_size: size of data
                                        pointed by img_request                */
    MP200_IMG_REQ *img_request;   /* pointer to IMG request information       */
    int             flag;           /* SYNC flag (IMG_CANCEL_SYNC)            */
}_IMGWRITE_MP200_IMG_CANCEL;

#define SIZE_IMGWRITE_MP200_IMG_CANCEL (SIZE_IMG_DRV_REQ + 8)

#define MAX_SIZE_IMGWRITE SIZE_IMGWRITE_MP200_IMG_CANCEL

#define IOC_IMG_MAGIC		('N')

/*===============================================================*/
/* IOCTLs to IMG driver. 0x4E is 'N' for NEC.                    */
/*===============================================================*/
#define IMGIOGET_PHYS_ADDR	_IOWR(IOC_IMG_MAGIC, 10, \
                sizeof(_IMGIOGET_PHYS_ADDR))

/*===============================================================*/
/* WRITE to IMG driver. 0x4E is 'N' for NEC.                     */
/*===============================================================*/
#define IMGWRITE_MP200_IMG_REGIST   0x4E12  /* regist IMG request            */
#define IMGWRITE_MP200_IMG_CALLBACK 0x4E13  /* waiting IMG request to finish */
#define IMGWRITE_MP200_IMG_CANCEL   0x4E14  /* cancel IMG request            */



#endif /* _IMG_USER_H_ */
