/* -*- mode: c; c-basic-offset: 8; comment-column: 40; -*- */

/*
 * linux/arch/arm/mach-mp200/dsp.c
 */

/*
 * InterDSP Driver
 *
 * Copyright (C) NEC Electronics Corporation 2004,2005,2006
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#ifndef INTER_DSP_H
#define INTER_DSP_H

#include "inter_dsp_ioctl.h"

/* ̿ͥ */
#define INTERDSP_CH_NUM		3	/* Channel  */

/* Channel ̵ѤΥޡINTERDSP_CH_NUM 礭Ǥʤ
    */
#define INTERDSP_CH_INVALID	(INTERDSP_CH_NUM+1)

/* SPA/SPX ޥɥեޥå */
union interdsp_spa_header {
	uint32_t header[2];	/* Accessor */

	/* Control Command Format */
	struct {
		unsigned issue:8;
		unsigned taskid:4;
		unsigned reserve1:2;
		unsigned buffer:2; /* 00 */
		unsigned reserve2:13;
		unsigned buffer_no:3;	/* 000 */
		unsigned length:16;
		unsigned block_num:16;
	} spa_ccf;

	/* Transfer Command Format */
	struct {
		unsigned issue:8;
		unsigned taskid:4;
		unsigned reserve1:2;
		unsigned buffer:2; /* 1x */
		unsigned reserve2:13;
		unsigned buffer_no:3;
		unsigned length:16;
		unsigned block_num:16;
	} spa_tcf;

	/* Response Format */
	struct {
		unsigned issue:8;
		unsigned taskid:4;
		unsigned reserve1:2;
		unsigned eor:1;
		unsigned rrf:1;	/* 0 */
		unsigned code:15;
		unsigned error:1;
		unsigned length:16;
		unsigned block_num:16;
	} spa_res;

	/* Request Format */
	struct {
		unsigned reserve:8;
		unsigned taskid:4;
		unsigned reserve1:3;
		unsigned rrf:1;	/* 1 */
		unsigned code:16;
		unsigned length:16;
		unsigned block_num:16;
	} spa_req;

	/* SPA Command Format */
	struct {
		unsigned issue:8;
		unsigned taskid:4; /* 0 */
		unsigned reserve1:4;
		unsigned code:8;
		unsigned reserve2:8;
		unsigned length:16;
		unsigned block_num:16;
	} spa_cmd;
};

/*  */
struct arm_queue_ent;
struct list_head;
struct interdsp_pending_resreq;
struct file_operations;

/* ؿץȥ */
int interdsp_read (int file_ch, int nonblock, struct dsp_cmd_rw *rwcmd,
		   int (*read_callback)(int, int, size_t, void *buf, size_t));

int interdsp_writebuf (int ch, int num, int nonblock,
		       struct dsp_cmd_rw *rwcmd, size_t size);

int interdsp_wait_ack (int ch, struct arm_queue_ent *sq);

int interdsp_sendhdr (int ch, int num, int nonblock, int ack_sync,
		      struct dsp_cmd_rw *rwcmd);

int interdsp_get_taskid_channel (uint32_t taskid);
int interdsp_set_taskid_channel (uint32_t taskid, uint32_t ch);

int interdsp_get_chinfo (int ind, int ch, struct interdsp_chinfo *chinfo);
int interdsp_set_chinfo (int ind, int ch, struct interdsp_chinfo *chinfo);

int interdsp_checkch (struct dsp_cmd_ch_status *cmd);
int interdsp_waitch (struct dsp_cmd_ch_status *cmd);

int interdsp_download (struct dsp_cmd_download *dl);

void interdsp_send_ack (int ch);

/*
 * ARM channel buffer
 */

int interdsp_arm_write_buffer (int ch, int num, size_t offset,
			       const void *ubuf, size_t sz);
int interdsp_arm_write_buffer_internal (int ch, int num, size_t offset,
					const void *kbuf, size_t sz);
int interdsp_arm_read_buffer_internal (int ch, int num, size_t offset,
				       void *kbuf, size_t sz);

/*
 * DSP channel buffer
 */
int interdsp_dsp_read_buffer (int ch, int num, size_t offset,
			      void *ubuf, size_t sz);

int interdsp_dsp_read_buffer_internal (int ch, int num, size_t offset,
				       void *kbuf, size_t sz);

/*
 * fs operations
 */

int interdsp_ioctl (struct inode *inode, struct file *file,
		    unsigned int cmd, unsigned long arg);

int interdsp_open_core (struct inode *inode, struct file *file);

int interdsp_release (struct inode *inode, struct file *file);

int interdsp_release_core (struct inode *inode, struct file *file);

int interdsp_dsp_release_core (int ch, struct inode *inode, struct file *file);

unsigned int interdsp_poll (struct file *file, poll_table *wait);
unsigned int interdsp_arm_poll_core (int ch,
				     struct file *file, poll_table *wait);
unsigned int interdsp_dsp_poll_core (int ch,
				     struct file *file, poll_table *wait);

int interdsp_fsync (struct file *file, struct dentry *dentry, int datasync);
int interdsp_arm_fsync_core (int ch, struct file *file, struct dentry *dentry,
			     int datasync);

int interdsp_fasync (int fd, struct file *file, int mode);
int interdsp_dsp_fasync_core (int ch, int fd, struct file *file, int mode);

/*
 * extend ioctl
 */

int interdsp_push_ioctl (int (*) (struct inode *, struct file *,
				  unsigned int, unsigned long, int *),
			 int post);
void interdsp_remove_ioctl (int (*) (struct inode *, struct file *,
				     unsigned int, unsigned long, int *),
			    int post);


/*
 * set/clear error bit
 */
void interdsp_set_armch_errorbit (int bitnr);
void interdsp_clear_armch_errorbit (int bitnr);
int interdsp_test_and_clear_armch_errorbit (int bitnr);
void interdsp_set_dspch_errorbit (int bitnr);
void interdsp_clear_dspch_errorbit (int bitnr);
int interdsp_test_and_clear_dspch_errorbit (int bitnr);

/*
 * entry of send queue
 */

/* block lock/unlock */
int interdsp_lock_block_sq_entry (struct arm_queue_ent *sq, int nonblock);
void interdsp_unlock_block_sq_entry (struct arm_queue_ent *sq);

/* free/new/send */
void interdsp_free_sq_entry (struct arm_queue_ent *sq);
int interdsp_new_sq_entry (struct arm_queue_ent **sqp,
			   int ch, int num, int ack_sync,
			   struct dsp_cmd_rw *rwcmd,
			   void (*ack_callback) (int ch,
						 union interdsp_spa_header,
						 int, unsigned long),
			   unsigned long cb_data);

/* prepare send/discard */
int interdsp_prepare_send_sq_entry (struct arm_queue_ent *sq, int nonblock);
void interdsp_put_sq_entry (struct arm_queue_ent *sq);

/* send */
void interdsp_send_sq_entry (struct arm_queue_ent *sq);

/* get command header */
union interdsp_spa_header interdsp_get_hdr_sq_entry (struct arm_queue_ent *sq);
int interdsp_rewrite_sq_entry (struct arm_queue_ent *sq,
			       union interdsp_spa_header,
			       void *buf, size_t size);

/*
 * snatch res/req
 */

int interdsp_push_req_handler (int (*handler) (int,
					       union interdsp_spa_header *,
					       void *, size_t));
void interdsp_remove_req_handler (int (*handler) (int,
						  union interdsp_spa_header *,
						  void *, size_t));

/*
 * release notify
 */
int interdsp_push_release_notify (void (*notify) (struct inode *,
						  struct file *));
void interdsp_remove_release_notify (void (*notify) (struct inode *,
						     struct file *));

/*
 * insert/delete DSP Response/Request
 */
struct interdsp_pending_resreq *
interdsp_push_resreq (uint32_t fdata[2], void *kdata, size_t len,
		      void (*cb) (unsigned long cbdata),
		      unsigned long cbdata, int nonblock, int wake);
void interdsp_pop_resreq (struct interdsp_pending_resreq *resreq);

/*
 * add/remove minor device
 */
int interdsp_add_minor_device (int minor,
			       char *name, struct file_operations *fops);
int interdsp_remove_minor_device (int minor, struct file_operations *fops);

/*
 * force initializing shared memory
 */
int interdsp_shmem_init (void);

#endif /* INTER_DSP_H */
