/* -*- mode: c; c-basic-offset: 8; comment-column: 32; -*- */

/*
 * InterDSP Driver
 *
 * Copyright (C) NEC Electronics Corporation 2004,2005,2006
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

/*
 * 	CPU-DSP̿ɥ饤 : inter_dsp_ioctl.h
 */

#ifndef INTER_DSP_IOCTL_H
#define INTER_DSP_IOCTL_H

#ifdef __KERNEL__
#include <linux/types.h>
#include <linux/ioctl.h>
#else
#include <inttypes.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#endif

/*
 *	ioctlѹ¤
 */

/*
 * IOCTL struct
 */

struct interdsp_chinfo {
	uint32_t header_offset;	/* إåΰؤΥեå */
	uint32_t buffer_offset;	/* ХåեΰؤΥեå */
	uint32_t buffer_blknum;	/* ХåեΥ֥å */
	uint32_t buffer_blksize; /* Хåե֥åΥ(32bitñ) */
};

/* INTERDSP_READ/INTERDSP_WRITE */

/* struct dsp_cmd_rw  flags  */
#define INTERDSP_WRITEBUF_OK	0x0001 /* Хåե */
#define INTERDSP_SENDHDR_OK	0x0002 /* إå */

struct dsp_cmd_rw {
	uint32_t fdata[2];	/* إåǡ*/
	uint8_t *bdata;		/* 桼Υԡ */
	size_t bsize;		/* 桼Υԡ֥(byte) */
	size_t offset;		/* ֥߰եå */

	/* ʲϥɥ饤Ф */
        uint32_t flags;		/* nonblocking ξ  */
};

/* ch info */
struct dsp_cmd_ch {
	uint32_t dsparm;	/* 0(ARM->DSP), 1(DSP->ARM) */
	uint32_t chnum;		/* ֹͥ(0, 1, 2) */
	struct interdsp_chinfo chinfo; /* ͥ (all 0: invalid) */
};

/* TaskID Channel */
struct dsp_cmd_task_ch {
	uint32_t taskid;	/* TaskID */
	int32_t chnum;		/* ֹͥ(0,1,2), -1 ̵򼨤 */
};  

/* Interval Get/Set */
struct dsp_cmd_ch_interval {
	int32_t chnum;		/* ֹͥ */
	int32_t intq_num;	/* 󥿡Х륭塼ֹ */
	int32_t interval;	/* ñ: ms, -1 ̵򼨤 */
	int32_t repeat;	 /* ֲ(0[̵ꡦ̵ѹ], -1[̵]) */
	int32_t missfire; /* ߲(0[̵ꡦ̵ѹ], -1[̵]) */
};

/* Channel Status 2 */
struct dsp_cmd_ch_status {
	uint32_t armchmask;	/* Ch. No. Mask (1<<0,1<<1,1<<2) */
	uint32_t dspchmask;	/* Ch. No. Mask (1<<0,1<<1,1<<2) */

	/* ARM CH0, 1, 2 ΥåѤΥ֥åֹ(0,1,2...)ӥåȥޥ
	   פˤ֤1 å0 ̤å */
	unsigned long armch_blockbitmap[3];
};


/*
 * Program DownLoad
 */
/* control flags */
#define INTERDSP_DL_PROG	0x01 /*  */
				     /* 0x02 unused (INTERDSP_DL_POWER) */
#define INTERDSP_DL_RESET	0x04 /* ꥻå */
#define INTERDSP_DL_CLOCK	0x08 /* å */
#define INTERDSP_DL_CLKCTRL	0x10 /* ưå */

/*
 * Download `offset' examples
 *
 *   memtype  target-addr     offset
 *   0        0xb0001234  =>  0x00001234    [target - 0xb0000000]    
 *   1        0xb0108000  =>  0x00008000    [target - 0xb0100000]
 *   2        0xa0000100  =>  0x00000100    [target - 0xa0000000]
 *   3        0x33a04000  =>  0x03a04000    [target - 0x30000000]
 */

/* DSP Control */
struct dsp_cmd_download {
	uint32_t control;	/* control flags */

	/* INTERDSP_DL_PROG */
	void *data;		/* Program/Data Address */
	uint32_t memtype;	/* 0:InstSRAM  1:DataSRAM  2:ExtSRAM 3:SDRAM */
	uint32_t offset;	/* Write Offset in SRAM/SDRAM */
	uint32_t size;		/* Copy Size */

	/* INTERDSP_DL_RESET */
	uint32_t reset;		/* 0:clear 1:set 2:(first set, last clear) */

	uint32_t power;		/* unused (backward compatibility) */

	/* INTERDSP_DL_CLOCK */
	uint32_t clock;		/* 1:start, 0:stop */

	/* INTERDSP_DL_CLKCTRL */
	uint32_t clkctrl;	/* 1:on, 0:off (auto clock control) */
};

/*
 * Errot Bit Number (INTERDSP_GET_ERROR)
 */

#define INTERDSP_ERROR_ARMCH_ACK_TIMEOUT(CH)	(CH)

/* Υӥåֹ wrapper ˻Ȥäɤ */
#define INTERDSP_ERROR_WRAPPER_NR		8

/*
 * DCV (AC1/MP201)
 */

/* DCV 쥸 */
struct dcv_reg {
	uint16_t bank_no;	/* BANKֹ 0-15 */
	union {
		struct {
			unsigned bank_offset:12; /* DCV_BANKn_OFFSET */
			unsigned bank_set:4; /* DCV_BANKn_SET */
		} bank_reg;
		uint16_t bank_addr; /* ()16bit */
	} bank_val;
};

/* DCV /¤ */
struct dsp_cmd_dcv_set {
	unsigned int count;	/* set/get number */
	struct dcv_reg *dcv_regs; /* pointer to `struct dcv_reg' */
};

/*
 *	ioctlޥ
 */

#define INTERDSP_READ			_IOR('D', 0x01, struct dsp_cmd_rw)
#define INTERDSP_WRITE			_IOW('D', 0x02, struct dsp_cmd_rw)

#define INTERDSP_WRITEBUF		_IOWR('D', 0x08, struct dsp_cmd_rw)
#define INTERDSP_SENDHDR		_IOWR('D', 0x09, struct dsp_cmd_rw)

#define INTERDSP_GET_CHINFO	_IOR('D', 0x10, struct dsp_cmd_ch)
#define INTERDSP_SET_CHINFO	_IOW('D', 0x11, struct dsp_cmd_ch)
#define INTERDSP_GET_TASKCH	_IOR('D', 0x12, struct dsp_cmd_task_ch)
#define INTERDSP_SET_TASKCH	_IOWR('D', 0x13, struct dsp_cmd_task_ch)
#define INTERDSP_SHMEM_INIT	_IO('D', 0x18)

#define INTERDSP_CHECKCH	_IOWR('D', 0x22, struct dsp_cmd_ch_status)
#define INTERDSP_WAITCH		_IOWR('D', 0x23, struct dsp_cmd_ch_status)

#define INTERDSP_DOWNLOAD	_IOWR('D', 0x30, struct dsp_cmd_download)

#define INTERDSP_GET_ERROR	_IOR('D', 0x40, unsigned long)

#define INTERDSP_SET_DCV_REGS	_IOW('D', 0x50, struct dsp_cmd_dcv_set)
#define INTERDSP_GET_DCV_REGS	_IOW('D', 0x51, struct dsp_cmd_dcv_set)


#endif /* INTER_DSP_IOCTL_H */
