/*
 *  File Name	    : linux/include/asm-arm/arch-mp200/mp201_mem.h
 *  Function	    : memory map difinition
 *  Release Version : Ver 1.01
 *  Release Date    : 2006/09/14
 *
 *  Copyright (C) NEC Electronics Corporation 2005-2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

/*
 * Copyright 2006,2007 Sony Corporation. 
 */


#ifndef __MP201_MEM_H
#define __MP201_MEM_H

/******* SDRAM ********/

#define INTERDSP_SHARED_ADDR			0x31F80000 /* CPU<->DSP command buffer */
#define INTERDSP_SHARED_SIZE			0x00080000 /*   512KB */

#define INTERDSP_DL_TOP_ADDR			0x31900000 /* DSP-FW downlaod range (top) */
#define INTERDSP_DL_BOTTOM_ADDR			0x31F7FFFF /*                       (end) */

#define OMF_MC_SDRAM_BASE   			0x31C80000 /* shared memory for OMF */
#define OMF_MC_AUDIO_SHARED_MEMORY_SIZE 0x00018000 /*   for audio (98KB) */
#define OMF_MC_VIDEO_SHARED_MEMORY_SIZE 0x0029EA00 /*   for video (2.7MB) */
#define OMF_MC_ACOUSTIC_PARAM_MEM_SIZE  0x00001000 /*   for param (4KB)  */

#define FB_FRAME_BUFFER_ADDR			0x31700000 /* frame buffer , fb driver work */
#define FB_FRAME_BUFFER_SIZE			0x00100000 /*   1.0MB  */
#ifdef CONFIG_FB_MP200      /*not use V4L2 driver*/
#define FB_VIDEO_WORK_ADDR				0x31A00000 /* FB driver work for video */
#define FB_VIDEO_WORK_SIZE				0x00600000 /*   6.0MB */
#endif
#ifdef CONFIG_VIDEO_MP200	/* V4L2 driver */
#define ROT_SHARED_ADDR					0x31800000       /* buffer for rotator                */
#define ROT_SHARED_SIZE					(0x00054600 * 2) /*   for YUV420 480x480 (337.5KB) *2 */

#endif


/* SDRAM difinition for OMF/kenrel drivers  */

#define SHARED_MEM_ADDRESS 		INTERDSP_SHARED_ADDR	/* CPU<->DSP command buffer start address*/
#define SHARED_MEM_SIZE 		INTERDSP_SHARED_SIZE
#define DOWNLOAD_RANGE_START 	INTERDSP_DL_TOP_ADDR
#define DOWNLOAD_RANGE_END 		INTERDSP_DL_BOTTOM_ADDR

#define OMF_MC_AUDIO_SHARED_MEMORY_ADDR (OMF_MC_SDRAM_BASE + 0x00000000)
#define OMF_MC_VIDEO_SHARED_MEMORY_ADDR (OMF_MC_SDRAM_BASE + 0x0001C000)
#define OMF_MC_ACOUSTIC_PARAM_MEM_ADDR (OMF_MC_SDRAM_BASE  + 0x00018000)

#define FRONT_WIDTH		240	/* for FrameBuf (LCDC) Width  */
#define FRONT_HEIGHT		320	/* for FrameBuf (LCDC) Height */
#define FRONT_WIDTH_V		240	/* for DispBuf  (2D)   Width  */
#define FRONT_HEIGHT_V		320	/* for DispBuf  (2D)   Height */
#define BYTES_PER_PIXEL		2	/* for DispBuf  (2D)   - 16bpp */
#define BITS_PER_PIXEL		16	/* for FrameBuf (LCDC) - 16bpp */
#define BITS_PER_PIXEL_OUT	18	/* for FrameBuf (LCDC) - 18bpp 
                                   added by ita 2006.09.14 for RGB666*/

#ifdef CONFIG_FB_MP200      /*not use V4L2 driver*/
#define DSP_SMEM_START		FB_VIDEO_WORK_ADDR	/* for moving image (common with DSP) */
#define DSP_SMEM_LENGTH		FB_VIDEO_WORK_SIZE	/* for moving image (common with DSP) */
#else                      /* use V4L2 driver */
#define DSP_SMEM_START		0x31A00000		/* for moving image (common with DSP) */
#define DSP_SMEM_LENGTH		0x00600000		/* for moving image (common with DSP) */
#endif
#ifdef CONFIG_VIDEO_MP200	/* V4L2 driver */
#define ROT_SMEM_START		ROT_SHARED_ADDR		/* for rotate image (common with DSP) */
#define ROT_SMEM_LENGTH		ROT_SHARED_SIZE		/* YUV420 QVGA (240 * 320 * 1.5) */
#endif
#define SMEM_START		FB_FRAME_BUFFER_ADDR	/* frame buffer, X/fb driver common, etc */
#define SMEM_LENGTH		FB_FRAME_BUFFER_SIZE

#ifdef CONFIG_FB_MP200
#define DISPBUF_OFFSET		0x00000000	/* SMEM_START + 0x00000000 */
#endif
#ifdef CONFIG_FB_MP200_V4L2	/* V4L2 fb driver */
#define DISPBUF_A_OFFSET	0x00000000	/* SMEM_START + 0x00000000 */
#define DISPBUF_B_OFFSET	0x00040000	/* SMEM_START + 0x00040000 */
#endif
#define DISPBUF_LENGTH		(FRONT_WIDTH_V * FRONT_HEIGHT_V * BYTES_PER_PIXEL)
#ifdef CONFIG_FB_MP200
#define FRAMEBUFA_OFFSET	0x00040000	/* SMEM_START + 0x00040000 */
#define FRAMEBUFB_OFFSET	0x00080000	/* SMEM_START + 0x00080000 */
#endif
#ifdef CONFIG_FB_MP200_V4L2	/* V4L2 fb driver */
#define FRAMEBUF_A_OFFSET	0x00080000	/* SMEM_START + 0x00080000 */
#define FRAMEBUF_B_OFFSET	0x000C0000	/* SMEM_START + 0x000C0000 */
#endif
#define FRAMEBUF_LENGTH		(FRONT_WIDTH * FRONT_HEIGHT * BITS_PER_PIXEL / 8)
#ifdef CONFIG_FB_MP200
#define GLX_WINDOW_TABLE_OFFSET	0x000C0000		/* SMEM_START + 0x000C0000 */
#define BLD_SCRN_LIST_OFFSET	0x000C1000		/* SMEM_START + 0x000C1000 */
#define XV_IMAGE_UPDATED_FLAG_OFFSET 0x000C2000	/* SMEM_START + 0x000C2000 */
#define XV_IMAGE_UPDATED_FLAG_LENGTH (sizeof(int))
#define XV_IMAGE_ONSCRN_FLAG_OFFSET 0x000C2010	/* SMEM_START + 0x000C2010 */
#define XV_IMAGE_ONSCRN_FLAG_LENGTH (sizeof(int))
#endif


/******* Internal SRAM ********/

/* Frame Cache */

#define ENABLE_FRAMECACHE 0				/* 1:use frame cache, 0:not use frame cache */
#define FRAMECACHE_START	0xA0000000	/* start address */
#define FRAMECACHE_LENGTH	((FRAMEBUF_LENGTH + 0x3FFF) & ~0x3FFF)  /* depend of frame buffer */

/* Warm boot code & save area for CPU context (0xA003F000-0xA003FFFF: bottom 4KB of ISRAM) */
#define WARMBOOT_ADDR			0xEA03F800	/* resume code virtual address */
#define HANDLER_SIZE			0x00000800	/* max size of resume code */

/* OMF Work area */
#define OMF_DSP_ISRAM_WORK_ADDR		0xA0030000
#define OMF_DSP_ISRAM_WORK_LENGTH	0x00000D60	/* 3.3KB */

#endif /* __MP201_MEM_H */

