/*
 *  File Name	    : linux/include/asm-arm/arch-mp200/pcm_ioctl.h
 *  Function	    : Common audio interface for the MP200
 *  Release Version : Ver 1.01
 *  Release Date    : 2006/03/22
 *
 *  Copyright (C) NEC Electronics Corporation 2005-2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#ifndef __ASM_ARM_ARCH_MP200_PCM_IOCTL_H
#define __ASM_ARM_ARCH_MP200_PCM_IOCTL_H

#include <asm/ioctl.h>

/* mode_sel : Operation mode */
#define PCM_MODE_0		0	/* mode 0 */
#define PCM_MODE_1		1	/* mode 1 */
#define PCM_MODE_2		2	/* mode 2 (I2S Format) */
#define PCM_MODE_3		3	/* mode 3 (MSB First)  */
#define PCM_MODE_4		4	/* mode 4 (LSB First)  */
#define PCM_MODE_5		5	/* mode 5 (Multi Channel Mode) */
#define PCM_MODE_6		6	/* mode 6 (Multi Channel Mode) */

/* m_s : Master/Slave */
#define PCM_MASTER_MODE		0	/* Master */
#define PCM_SLAVE_MODE		1	/* Slave  */

/* tx_tim : Transmission timing */
#define PCM_TX_30_WORD		0	/* 30word */
#define PCM_TX_16_WORD		1	/* 16word */
#define PCM_TX_08_WORD		2	/*  8word */
#define PCM_TX_04_WORD		3	/*  4word */

/* rx_pd : Data padding (RX) */
/* tx_pd : Data padding (TX) */
#define PCM_PADDING_OFF		0	/* Data padding non */
#define PCM_PADDING_ON		1	/* Data padding */

#ifdef CONFIG_MP201
/* INT_MODE */
#define PCM_INT_CPU_MODE	0
#define PCM_INT_DSP_MODE	1
#endif

typedef struct pcm_ctrl {
	struct func_sel {
		unsigned char mode_sel;	/* Operation mode */
		unsigned char m_s;	/* Master/Slave */
		unsigned char tx_tim;	/* Transmission timing */
	} func;
	struct cycle {
		unsigned char cyc_val;	/* Frame length */
		unsigned char sib;	/* Data bit length (PMx_SI) */
		unsigned char rx_pd;	/* Data padding (RX) */
		unsigned char sob;	/* Data bit length (PMx_SO) */
		unsigned char tx_pd;	/* Data padding (TX) */
	} cyc;
	struct cycle2 {
		unsigned char cyc_val2;	/* Frame length */
		unsigned char sib2;	/* Data bit length (PMx_SI) */
		unsigned char sob2;	/* Data bit length (PMx_SO) */
	} cyc2;
	unsigned int formats_caps;	/* Audio data formats */
} pcm_ctrl_t;

#define PCM_CODEC_SET		_IOW( 'D',0x01, unsigned int )
#define PCM_SET_CTRL		_IOW( 'D',0x02, pcm_ctrl_t* )
#define PCM_GET_CTRL		_IOR( 'D',0x03, pcm_ctrl_t* )

#ifdef CONFIG_MP201
#define PCM_DMA_INT_SET		_IOW( 'D',0x04, int )
#endif

#endif				/* __ASM_ARM_ARCH_MP200_PCM_IOCTL_H */
