/*
 * File Name       : /drivers/video/qfb.h
 * Function        : QVGA FraemBuffer Driver IOCTL Interface
 * Release Version : Ver 1.00
 * Release Date    : 2005/05/25
 *
 * Copyright (C) NEC Electronics Corporation 2005
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */
/*
 *      qfb.h  --  NEC D77601 Framebuffer Driver
 */

/*
 * Copyright 2006,2007 Sony Corporation. 
 */

#ifndef __LINUX_QFB_H__
#define __LINUX_QFB_H__

#include <asm/ioctl.h>
#include <asm/types.h>
#ifdef CONFIG_MP201
#include <asm/arch/mp201_mem.h>
#endif

/* Structure argument for QFB_SET_XV_ATTR ioctl  */
typedef struct {
	unsigned long paddr_interFb_bufaddr;	/* physical address of interFb_bufaddr */
	short src_x;
	short src_y;
	short drw_x;
	short drw_y;
	short drw_w;
	short drw_h;
	short width;
	short height;
} Xv_Attr;

/* Structure argument for QFB_SET_GLX_TIMETABLE ioctl  */
typedef struct {
	int tablesize;		/* count of table - 0 origin */
	int time[1];		/* timer tables(10mS) */
} GLX_timetable;

/* Structure argument for QFB_SET_XV_COLORKEY / QFB_GET_XV_COLORKEY ioctl  */
typedef struct {
	int t_color;		/* transparent color (16bpp) */
} Xv_colorkey;

/*
 * IOCTLs to D77601 fb driver. 0x4E is 'N' for NEC.
 */

#define IOC_QFB_MAGIC		('N')

#define QFB_CHK_SCRN		_IO (IOC_QFB_MAGIC, 0)
#define QFB_BLD_SCRN		_IO (IOC_QFB_MAGIC, 1)
#define QFB_SET_XV_ATTR		_IOW(IOC_QFB_MAGIC, 2, sizeof(Xv_Attr))
#define QFB_XVFLIPROT		_IO (IOC_QFB_MAGIC, 3)
#define QFB_SET_GLX_TIMETABLE	_IOW(IOC_QFB_MAGIC, 4, sizeof(GLX_timetable))
#define QFB_DEL_GLX_TIMETABLE	_IO (IOC_QFB_MAGIC, 5)
#define QFB_SET_XV_COLORKEY	_IOW(IOC_QFB_MAGIC, 6, sizeof(Xv_colorkey))
#define QFB_GET_XV_COLORKEY	_IOR(IOC_QFB_MAGIC, 7, sizeof(Xv_colorkey))
/*20060531 SONY Corporation: Set backlight brightness */
/*#define QFB_SET_BL_BRIGHTNESS _IOW(IOC_QFB_MAGIC, 8, sizeof(unsigned int))*/
/*******************************************************************/
/*********************** module parameters *************************/
/*******************************************************************/

#define ENABLE_3D 0
#define MIX_IMAGE_WITHOUT_X 1
#define UNRESET_LCD 1		/* Unreset LCD not in bootloader but in fb driver */
#define LOWER_ROT_SCREEN 1
#define WAIT_TIME_X_CHK 500	/* tick count 5000msec */

/* MMAP parameter */
#define SMEM_OFFSET	0
#ifndef CONFIG_MP201
#define SMEM_LENGTH	0x00180000
#endif
#if ENABLE_3D
#define GLX_SMEM_OFFSET	SMEM_LENGTH
#define GLX_SMEM_LENGTH	0x00400000
#define DSP_SMEM_OFFSET	(SMEM_LENGTH + GLX_SMEM_LENGTH)
#ifndef CONFIG_MP201
#define DSP_SMEM_LENGTH	0x00600000
#endif
#define MMIO_OFFSET	(SMEM_LENGTH + GLX_SMEM_LENGTH + DSP_SMEM_LENGTH)
#define MMIO_LENGTH	0x00010000
#define MAX_OFFSET	(SMEM_LENGTH + GLX_SMEM_LENGTH + DSP_SMEM_LENGTH + MMIO_LENGTH)
#else	/* !ENABLE_3D */
#define DSP_SMEM_OFFSET	(SMEM_LENGTH)
#ifndef CONFIG_MP201
#define DSP_SMEM_LENGTH	0x00600000
#endif
#define MAX_OFFSET	(SMEM_LENGTH + DSP_SMEM_LENGTH)
#endif

#endif				/* __LINUX_QFB_H__ */
