/*
*  File Name       : /include/asm-arm/arch-mp200/rot.h
*  Function        : ROT MMIO definitions, and Driver - Driver I/F of ROT driver
*  Release Version : Ver 1.00
*  Release Date    : 2005/08/23
*
*  Copyright (C) NEC Electronics Corporation 2005
*
*
*  This program is free software;you can redistribute it and/or modify it under the terms of 
*  the GNU General Public License as published by Free Softwere Foundation; either version 2 
*  of License, or (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
*  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
*  See the GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along with this program; 
*  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
*  MA 02111-1307, USA.
*
*/


#ifndef _ROT_H_
#define _ROT_H_


/*===============================================================*/
/* ROT Driver Definitions                                        */
/*===============================================================*/

/* Parameters of mp200_rot_cancel (cancel request) */
#define ROT_CANCEL_SYNC             1    /* SYNC flag for cancel request               */

/* Statuses of ROT request */
#define ROT_REQ_STATUS_REGISTERED   1    /* registered                                 */
#define ROT_REQ_STATUS_RUNNING      2    /* running                                    */
#define ROT_REQ_STATUS_WAIT_CANCEL  3    /* waiting for cancelled request to be done   */
#define ROT_REQ_STATUS_CANCELLED    4    /* cancelled                                  */
#define ROT_REQ_STATUS_DONE         5    /* done                                       */
#define ROT_REQ_STATUS_ERROR        6    /* failed                                     */

/* Returns of mp200_rot_cancel (cancel request) */
#define ROT_CANCEL_DONE             0    /* cancelled                                  */
#define ROT_CANCEL_WAIT             1    /* waiting for cancelled request to be done   */
#define ROT_CANCEL_NO_REQ           2    /* target request does not exist              */

/* Returns of mp200_rot_reschedule (run Normal Resuest) */
#define ROT_RESCHEDULE_START        0    /* started                                    */
#define ROT_RESCHEDULE_NO_REQ      -1    /* no request                                 */
#define ROT_RESCHEDULE_BUSY        -2    /* ROT busy                                   */

/* Parameters of callback function */
#define ROT_CALLBACK_SUCCESS        0    /* ROT process is completed (Success)         */
#define ROT_CALLBACK_SET_PARAM      1    /* set values to ROT registers (Success)      */
#define ROT_CALLBACK_CANCEL         2    /* cancelled request is done (Success)        */
#define ROT_CALLBACK_FAIL          -1    /* can't start ROT process (Error)            */
#define ROT_CALLBACK_ROT_ERROR     -2    /* ROT ERROR interrupt (Error)                */
#define ROT_CALLBACK_ROT_STOP      -3    /* ROT RETRY/SPLIT interrupt (Error)          */


struct mp200_rot_req;                    /* Declaration of structure tag (incomplete type) */


/*===============================================================*/
/* Callback function type                                        */
/*===============================================================*/
typedef void (*rot_callback_func)(struct mp200_rot_req *p, int flag);


/*===============================================================*/
/* ROT register setting information structure                    */
/*===============================================================*/
typedef struct mp200_rot_reg_info {
    unsigned long reg;                   /* offset to ROT register               */
    unsigned long data;                  /* data to set                          */
} MP200_ROT_REG_INFO;


/*===============================================================*/
/* ROT request information structure                             */
/*===============================================================*/
typedef struct mp200_rot_req {
    struct mp200_rot_req *next;     /* pointer to next ROT requst (used by ROT driver)                 */
    struct mp200_rot_req *prev;     /* pointer to previous ROT requst (used by ROT driver)             */
    int                 status;     /* Status of ROT request (used by ROT driver)                      */
    rot_callback_func   callback;   /* callback function                                               */
    unsigned long       data;       /* optional data (used by caller)                                  */
    struct mp200_rot_reg_info rot_reg_info[];  /* ROT register setting informations                    */
    /* repeat until end condition (max 50 pairs)                                                       */
    /*   end condition :                                                                               */
    /*         set 0x00000001 to ROT_REQ (request to start ROT process) reg=ROT_REQ data=0x00000001,   */
    /*         or reg=-1(0xFFFFFFFF)                                                                   */
} MP200_ROT_REQ;
#define ROT_REG_INFO_MAX  50        /* max number of ROT register setting informations                 */


/*===============================================================*/
/* ROT MMIO definitions   for uPD77601                           */
/*===============================================================*/
#include <asm/arch/hardware.h>
#define ROTATOR_OFFSET    MP200_ROTATOR_OFFSET
#define ROTATOR_BASE      MP200_ROTATOR_BASE
#define ROTATOR_SIZE      MP200_ROTATOR_SIZE


/*===============================================================*/
/* ROT register definitions   for uPD77601                       */
/*===============================================================*/
//                                                           R/W   Bit  Reset
// Mode for ROTATOR                                          R/W   4:0  0x00
#define RO_MODE                 (0x00)

// Frame Select                                              R/W   3:0  0x5
#define RO_FRAME                (0x04)

// Rotate Request                                            R/W   0:0  0x0
#define RO_REQ                  (0x08)

// Status for ROTATOR                                         R    0:0  0x0
#define RO_STATUS               (0x0C)

// Original Address Size                                     R/W  10:2  0x000
#define RO_SRCSIZE              (0x20)

// Rotate Address Size                                       R/W  10:2  0x000
#define RO_DSTSIZE              (0x24)

// Original Y Address for A Frame                            R/W  31:2  0x0000_0000
#define RO_SRCYADR_A            (0x30)

// Original Y Address for B Frame                            R/W  31:2  0x0000_0000
#define RO_SRCYADR_B            (0x34)

// Original Y Address for C Frame                            R/W  31:2  0x0000_0000
#define RO_SRCYADR_C            (0x38)

// Rotate Y Address for A Frame                              R/W  31:2  0x0000_0000
#define RO_DSTYADR_A            (0x40)

// Rotate Y Address for B Frame                              R/W  31:2  0x0000_0000
#define RO_DSTYADR_B            (0x44)

// Rotate Y Address for C Frame                              R/W  31:2  0x0000_0000
#define RO_DSTYADR_C            (0x48)

// Original UV Address for A Frame                           R/W  31:2  0x0000_0000
#define RO_SRCUVADR_A           (0x60)

// Original UV Address for B Frame                           R/W  31:2  0x0000_0000
#define RO_SRCUVADR_B           (0x64)

// Original UV Address for C Frame                           R/W  31:2  0x0000_0000
#define RO_SRCUVADR_C           (0x68)

// Rotate UV Address for A Frame                             R/W  31:2  0x0000_0000
#define RO_DSTUVADR_A           (0x70)

// Rotate UV Address for B Frame                             R/W  31:2  0x0000_0000
#define RO_DSTUVADR_B           (0x74)

// Rotate UV Address for C Frame                             R/W  31:2  0x0000_0000
#define RO_DSTUVADR_C           (0x78)

// Horizontal Size                                           R/W  10:2  0x008
#define RO_SRCHSIZE             (0x90)

// Vertical Size                                             R/W  10:2  0x008
#define RO_SRCVSIZE             (0xA0)

// Interrupt Status                                           R    2:0  0x0
#define RO_INTSTATUS            (0xB0)

// Interrupt Raw Status                                       R    2:0  0x0
#define RO_INTRAWSTATUS         (0xB4)

// Interrupt Enable Set                                      R/W   2:0  0x0
#define RO_INTENSET             (0xB8)

// Interrupt Enable Clear                                     W    2:0  0x0
#define RO_INTENCLR             (0xBC)

// Interrupt Status Clear                                     W    2:0  0x0
#define RO_INTFFCLR             (0xC0)

// DMA Error Respons Address                                 R/W  31:0  0x0
#define RO_ERRORADR             (0xC4)


/*===============================================================*/
/* ROT register bit assigns                                      */
/*===============================================================*/


/*------------------------------*/
/* RO_MODE                      */
/*------------------------------*/
#define SRCENDIAN_BIT           0x00000010
#define SRCENDIAN_SFT           0x4
#define SRCENDIAN_BIG           0x00000000
#define SRCENDIAN_LITTLE        0x00000010

#define DSTENDIAN_BIT           0x00000008
#define DSTENDIAN_SFT           0x3
#define DSTENDIAN_BIG           0x00000000
#define DSTENDIAN_LITTLE        0x00000008

#define SRCYUV_BIT              0x00000004
#define SRCYUV_SFT              0x2
#define SRCYUV_YUV422           0x00000000
#define SRCYUV_YUV420           0x00000004

#define ROT_MOD_BIT             0x00000003
#define ROT_MOD_SFT             0x0
#define ROT_MOD_0               0x00000000
#define ROT_MOD_90              0x00000001
#define ROT_MOD_180             0x00000002
#define ROT_MOD_270             0x00000003

/*------------------------------*/
/* RO_FRAME                     */
/*------------------------------*/
#define DSTSEL_BIT              0x0000000C
#define DSTSEL_SFT              0x2
#define DSTSEL_FRAME_A          0x00000004
#define DSTSEL_FRAME_B          0x00000008
#define DSTSEL_FRAME_C          0x0000000C

#define SRCSEL_BIT              0x00000003
#define SRCSEL_SFT              0x0
#define SRCSEL_FRAME_A          0x00000001
#define SRCSEL_FRAME_B          0x00000002
#define SRCSEL_FRAME_C          0x00000003

/*------------------------------*/
/* RO_REQ                       */
/*------------------------------*/
#define RO_REQ_BIT              0x00000001
#define RO_REQ_SFT              0x0


/*------------------------------*/
/* RO_STATUS                    */
/*------------------------------*/
#define RO_STATUS_BIT           0x00000001
#define RO_STATUS_SFT           0x0


/*------------------------------*/
/* RO_INTSTATUS                 */
/*------------------------------*/
#define RO_DMASTOP_BIT		0x00000004
#define RO_DMASTOP_SFT		0x02

#define RO_DMAERR_BIT		0x00000002
#define RO_DMAERR_SFT		0x01

#define RO_ROTEND_BIT		0x00000001
#define RO_ROTEND_SFT		0x00


/*------------------------------*/
/* RO_INTRAWSTATUS              */
/*------------------------------*/
#define RO_DMASTOPRAW_BIT	0x00000004
#define RO_DMASTOPRAW_SFT	0x02

#define RO_DMAERRRAW_BIT	0x00000002
#define RO_DMAERRRAW_SFT	0x01

#define RO_ROTENDRAW_BIT	0x00000001
#define RO_ROTENDRAW_SFT	0x00


/*------------------------------*/
/* RO_INTENSET                  */
/*------------------------------*/
#define RO_DMASTOPEN_BIT	0x00000004
#define RO_DMASTOPEN_SFT	0x02

#define RO_DMAERREN_BIT		0x00000002
#define RO_DMAERREN_SFT		0x01

#define RO_ROTENDEN_BIT		0x00000001
#define RO_ROTENDEN_SFT		0x00


/*------------------------------*/
/* RO_INTENCLR                  */
/*------------------------------*/
#define RO_DMASTOPMASK_BIT	0x00000004
#define RO_DMASTOPMASK_SFT	0x02

#define RO_DMAERRMASK_BIT	0x00000002
#define RO_DMAERRMASK_SFT	0x01

#define RO_ROTENDMASK_BIT	0x00000001
#define RO_ROTENDMASK_SFT	0x00


/*------------------------------*/
/* RO_INTFFCLR                  */
/*------------------------------*/
#define RO_DMASTOPCLR_BIT	0x00000004
#define RO_DMASTOPCLR_SFT	0x02

#define RO_DMAERRCLR_BIT	0x00000002
#define RO_DMAERRCLR_SFT	0x01

#define RO_ROTENDCLR_BIT	0x00000001
#define RO_ROTENDCLR_SFT	0x00

/*******************************************************************/
/************ Inter module communication definitions ***************/
/*******************************************************************/
#define MP200_ROT_DEV_NAME	"mp200_rotator"
#define MP200_ROT_DEV_FILE_NAME	"rot"

#endif /* _ROT_H_ */
