/*
*  File Name       : /include/asm-arm/arch-mp200/rot_user.h
*  Function        : User - Driver I/F of ROT driver
*  Release Version : Ver 1.00
*  Release Date    : 2005/08/23
*
*  Copyright (C) NEC Electronics Corporation 2005
*
*
*  This program is free software;you can redistribute it and/or modify it under the terms of 
*  the GNU General Public License as published by Free Softwere Foundation; either version 2 
*  of License, or (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
*  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
*  See the GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License along with this program; 
*  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
*  MA 02111-1307, USA.
*
*/


#ifndef _ROT_USER_H_
#define _ROT_USER_H_

#include "rot.h"


/*===================================================================================
* IOCTL    : ROTIOGET_PHYS_ADDR
* FUNCTION : get physical address of mmaped IO buffer
* INPUT    : user area addres
* OUTPUT   : physical address
*            Return
*              0 : Success
*             -1 : Fail
*====================================================================================*/
typedef struct tagROTIOGET_PHYS_ADDR{
    void *virt_addr;                /* user area virtual address (input) */
    long  phys_addr;                /* physical address (output) */
}_ROTIOGET_PHYS_ADDR;
#define SIZE_ROTIOGET_PHYS_ADDR 8

/*===================================================================================
* IOCTL    : ROTIO_ROTATION
* FUNCTION : rotate image with ROT
* INPUT    : input data address, width, height, format, endian, rotation angle,
*            output data address, endian
* OUTPUT   : 
*            Return
*              ROT_COMPLETE(0) : Success
*              ROT_ERROR(-1)   : Fail (ROT busy or parameter error)
*====================================================================================*/
typedef struct tagROTIO_ROTATION{
    unsigned long input_data_addr_y;    /* input Y data start address (physical)   */
    unsigned long input_data_addr_uv;   /* input UV data start address (physical)  */
    unsigned int input_data_width;      /* input data width                     */
    unsigned int input_data_height;     /* input data height                    */
    unsigned int input_data_format;     /* input data format                    */
                                        /*   ROT_FORM_YUV422Pl2:YUV422Pl2-De    */
                                        /*   ROT_FORM_YUV420Pl2:YUV420Pl2-De    */
    unsigned int input_data_endian;     /* input data endian                    */
                                        /*   ROT_ENDIAN_BIG   :big endian       */
                                        /*   ROT_ENDIAN_LITTLE:little endian    */
    unsigned int rotation_angle;        /* rotation angle                       */
                                        /*   ROT_ANGLE_0   :  0 degrees         */
                                        /*   ROT_ANGLE_90  : 90 degrees         */
                                        /*   ROT_ANGLE_180 :180 degrees         */
                                        /*   ROT_ANGLE_270 :270 degrees         */
    unsigned long output_data_addr_y;   /* output Y data start address (physical)  */
    unsigned long output_data_addr_uv;  /* output UV data start address (physical) */
    unsigned int output_data_endian;    /* output data endian                   */
                                        /*   ROT_ENDIAN_BIG   :big endian       */
                                        /*   ROT_ENDIAN_LITTLE:little endian    */
}_ROTIO_ROTATION;
#define SIZE_ROTIO_ROTATION 40

#define ROT_FORM_YUV422Pl2 0
#define ROT_FORM_YUV420Pl2 1
#define ROT_ENDIAN_BIG     0
#define ROT_ENDIAN_LITTLE  1
#define ROT_ANGLE_0        0
#define ROT_ANGLE_90       1
#define ROT_ANGLE_180      2
#define ROT_ANGLE_270      3

#define ROT_COMPLETE 0
#define ROT_ERROR    -1

#define MAX_SIZE_ROTIOCTL SIZE_ROTIO_ROTATION

/*===============================================================*/
/* Write request information structure                           */
/*===============================================================*/
typedef struct tagROT_DRV_REQ{
    long            request_id;     /* request ID              */
    size_t          request_size;   /* request size            */
    long            result_data;    /* result (output)         */
}_ROT_DRV_REQ;
#define SIZE_ROT_DRV_REQ 12

/*
******* INFORMATION for '_ROT_DRV_REQ' which is used by write() of ROT driver *******
  request_id:
    command ID to request
     ex.)regist ROT request
        ROTWRITE_mp200_rot_regist
  request_size:
    data size if data pointer or something like is in the structure.
    ignored by ROT driver when data pointer does not exist.
     ex.)size of ROT request information
  result_data:
    status or something like to return.
*/


/*===================================================================================
* WRITE    : ROTWRITE_MP200_ROT_REGIST
* FUNCTION : register ROT request in Normal Request's queue.
*            can't register a new request before registered request is done.
*            open() without O_NONBLOCK :
*              register the ROT request, wait for the request to be done (BLOCK), and return.
*            open() with O_NONBLOCK :
*              register the ROT request, and return immediately (non BLOCK). not wait for the request to be done.
* INPUT    : pointer to ROT request information
* OUTPUT   : Returns
*              ROT_REQ_REGISTERED(0) : the request is registered
*              ROT_REQ_COMPLETE(1)   : the request is registered and done
*              ROT_REQ_CANCELLED(2)  : the request is registered and cancelled
*              -EBUSY                : fail to register the request
*            result_data(Status of ROT process)
*              <without O_NONBLOCK and return=1(the request is registered and done)>
*              ROT_CALLBACK_SUCCESS       : ROT process is completed (Success)
*              ROT_CALLBACK_SET_PARAM     : set values to ROT registers (Success)
*              ROT_CALLBACK_CANCEL        : cancelled request is done (Success)
*              ROT_CALLBACK_FAIL          : can't start ROT process (Error)
*              ROT_CALLBACK_ROT_ERROR     : ROT ERROR interrupt (Error)
*              ROT_CALLBACK_ROT_STOP      : ROT RETRY/SPLIT interrupt (Error)
*====================================================================================*/
typedef struct tagROTWRITE_MP200_ROT_REGIST{
    _ROT_DRV_REQ    request_data;   /* Write request information                                        */
                                    /*   request_data.request_id  : ROTWRITE_MP200_ROT_REGIST           */
                                    /*   request_data.request_size: size of data pointed by rot_request */
    MP200_ROT_REQ *rot_request;     /* pointer to ROT request information                               */
}_ROTWRITE_MP200_ROT_REGIST;
#define SIZE_ROTWRITE_MP200_ROT_REGIST (SIZE_ROT_DRV_REQ + 4)

#define ROT_REQ_REGISTERED 0        /* the request is registered               */
#define ROT_REQ_COMPLETE   1        /* the request is registered and done      */
#define ROT_REQ_CANCELLED  2        /* the request is registered and cancelled */


/*===================================================================================
* WRITE    : ROTWRITE_MP200_ROT_CALLBACK
* FUNCTION : wait for the registered ROT request to be done, or get status of the request.
*            open() without O_NONBLOCK :
*              wait for the request to be done (BLOCK), and return.
*            open() with O_NONBLOCK :
*              return immediately (non BLOCK). not wait for the request to be done.
* INPUT    : pointer to ROT request information
* OUTPUT   : Returns
*              ROT_STATUS_DONE(0)         : ROT process is completed
*              ROT_STATUS_WAIT(1)         : ROT process is running (already registered)
*              ROT_STATUS_NO_REQ(2)       : target request does not exist
*              ROT_STATUS_CANCELLED(3)    : target request is cancelled
*            result_data(status of ROT process)
*              <return=0(ROT request had been done)>
*              ROT_CALLBACK_SUCCESS       : ROT process is completed (Success)
*              ROT_CALLBACK_SET_PARAM     : set values to ROT registers (Success)
*              ROT_CALLBACK_CANCEL        : cancelled request is done (Success)
*              ROT_CALLBACK_FAIL          : can't start ROT process (Error)
*              ROT_CALLBACK_ROT_ERROR     : ROT ERROR interrupt (Error)
*              ROT_CALLBACK_ROT_STOP      : ROT RETRY/SPLIT interrupt (Error)
*              <return=1(ROT request is running)>
*              ROT_REQ_STATUS_REGISTERED  : registered (before running)
*              ROT_REQ_STATUS_RUNNING     : running (running)
*              ROT_REQ_STATUS_WAIT_CANCEL : waiting for cancelled request to be done (running)
*====================================================================================*/
typedef struct tagROTWRITE_MP200_ROT_CALLBACK{
    _ROT_DRV_REQ    request_data;   /* Write request information                                        */
                                    /*   request_data.request_id  : ROTWRITE_MP200_ROT_CALLBACK         */
                                    /*   request_data.request_size: size of data pointed by rot_request */
    MP200_ROT_REQ *rot_request;     /* pointer to ROT request information                               */
}_ROTWRITE_MP200_ROT_CALLBACK;
#define SIZE_ROTWRITE_MP200_ROT_CALLBACK (SIZE_ROT_DRV_REQ + 4)

#define ROT_STATUS_DONE    0        /* ROT process is completed                    */
#define ROT_STATUS_WAIT    1        /* ROT process is running (already registered) */
#define ROT_STATUS_NO_REQ  2        /* target request does not exist               */
#define ROT_STATUS_CANCELLED 3      /* target request is cancelled                 */


/*===================================================================================
* WRITE    : ROTWRITE_MP200_ROT_CANCEL
* FUNCTION : cancel registered ROT request
*            target ROT request has not been started :
*                remove the target request from the queue.
*            target ROT request has been started (running) :
*              without ROT_CANCEL_SYNC :
*                set ROT_CANCEL_WAIT to the status of the request,
*                and wait for the request to be done.
*                Then set ROT_CALLBACK_CANCEL to result_data
*                as return of ROTWRITE_MP200_ROT_CALLBACK.
*              with ROT_CANCEL_SYNC    :
*                remove the target request from the queue immediately.
*                and skip waiting for the request to be done (ROTWRITE_MP200_ROT_CALLBACK)
* INPUT    : pointer to ROT request information
*            SYNC flag
* OUTPUT   : Returns
*              ROT_CANCEL_DONE(0)   : cancelled
*              ROT_CANCEL_WAIT(1)   : waiting for cancelled request to be done
*              ROT_CANCEL_NO_REQ(2) : target request does not exist
*====================================================================================*/
typedef struct tagROTWRITE_MP200_ROT_CANCEL{
    _ROT_DRV_REQ    request_data;   /* Write request information                                        */
                                    /*   request_data.request_id  :ROTWRITE_MP200_ROT_CANCEL            */
                                    /*   request_data.request_size: size of data pointed by rot_request */
    MP200_ROT_REQ *rot_request;     /* pointer to ROT request information                               */
    int             flag;           /* SYNC flag (ROT_CANCEL_SYNC)                                      */
}_ROTWRITE_MP200_ROT_CANCEL;
#define SIZE_ROTWRITE_MP200_ROT_CANCEL (SIZE_ROT_DRV_REQ + 8)

#define MAX_SIZE_ROTWRITE SIZE_ROTWRITE_MP200_ROT_CANCEL


#define IOC_ROT_MAGIC		('N')

/*===============================================================*/
/* IOCTLs to ROT driver. 0x4E is 'N' for NEC.                    */
/*===============================================================*/
#define ROTIOGET_PHYS_ADDR	_IOWR(IOC_ROT_MAGIC, 20, \
                sizeof(_ROTIOGET_PHYS_ADDR))/* get physical address       */
#define ROTIO_ROTATION		_IOW (IOC_ROT_MAGIC, 21, \
                sizeof(_ROTIO_ROTATION))    /* rotate image with ROT      */

/*===============================================================*/
/* WRITE to ROT driver. 0x4E is 'N' for NEC.                     */
/*===============================================================*/
#define ROTWRITE_MP200_ROT_REGIST   0x4E12  /* regist ROT request            */
#define ROTWRITE_MP200_ROT_CALLBACK 0x4E13  /* waiting ROT request to finish */
#define ROTWRITE_MP200_ROT_CANCEL   0x4E14  /* cancel ROT request            */


#endif /* _ROT_USER_H_ */
