/*
 *  File Name       : linux/include/asm-arm/arch-mp200/spi.h
 *  Function        : MP200 SPI interface
 *  Release Version : Ver 1.01
 *  Release Date    : 2006/03/06
 *
 *  Copyright (C) NEC Electronics Corporation 2005-2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

/*
 * Copyright 2006,2007 Sony Corporation. 
 */


#ifndef __ASM_ARM_ARCH_MP200_SPI_H
#define __ASM_ARM_ARCH_MP200_SPI_H

/*
 * spi mode info
 */
typedef struct {
	unsigned int nb;	/* bit length */
	unsigned int cs_sel;	/* chip select */
	unsigned int m_s;	/* master/slave */
	unsigned int dma;	/* dma on/off */
	unsigned int tx_tim;	/* transfer beginning cycle */
	unsigned int blksize;	/* block size */
	unsigned int sclk;	/* clock */
} spi_mode_t;


#ifdef CONFIG_MP201
/**  ADD FOR spi_write_without_intr **/
typedef struct {
	unsigned char dev;
	unsigned int  size;
	unsigned int  mode;
	unsigned int  pol;
	unsigned int  sclk;
} SPI_CONFIG;

/* SPI_CONFIG.mode */
#define SPI_MODE_NB_8BIT	(0x07 << 8)
#define SPI_MODE_NB_16BIT	(0x0F << 8)
#define SPI_MODE_NB_32BIT	(0x1F << 8)

#define SPI_MODE_CS_SEL_CS0	(0x00 << 4)
#define SPI_MODE_CS_SEL_CS1	(0x01 << 4)
#define SPI_MODE_CS_SEL_CS2	(0x02 << 4)
#define SPI_MODE_CS_SEL_CS3	(0x03 << 4)

#define SPI_MODE_M_S_MASTER	(0x00 << 1)
#define SPI_MODE_M_S_SLAVE	(0x01 << 1)

#define SPI_MODE_DMA_OFF	(0x00 << 0)
#define SPI_MODE_DMA_ON		(0x01 << 0)

/* SPI_CONFIG.pol */
#define SPI_POL_CSW_8SCLK	(0x07 << 12)
#define SPI_POL_CSW_16SCLK	(0x0F << 12)
#define SPI_POL_CSW_8SCLK	(0x07 << 12)
#define SPI_POL_CSW_4SCLK	(0x03 << 12)
#define SPI_POL_CSW_2SCLK	(0x01 << 12)

#define SPI_POL_CK3_DLY_ON	(0x01 << 11)
#define SPI_POL_CK3_DLY_OFF	(0x00 << 11)
#define SPI_POL_CK3_POL_NEG	(0x01 << 10)
#define SPI_POL_CK3_POL_POS	(0x00 << 10)
#define SPI_POL_CS3_POL_NEG	(0x01 << 9)
#define SPI_POL_CS3_POL_POS	(0x00 << 9)

#define SPI_POL_CK2_DLY_ON	(0x01 << 8)
#define SPI_POL_CK2_DLY_OFF	(0x00 << 8)
#define SPI_POL_CK2_POL_NEG	(0x01 << 7)
#define SPI_POL_CK2_POL_POS	(0x00 << 7)
#define SPI_POL_CS2_POL_NEG	(0x01 << 6)
#define SPI_POL_CS2_POL_POS	(0x00 << 6)

#define SPI_POL_CK1_DLY_ON	(0x01 << 5)
#define SPI_POL_CK1_DLY_OFF	(0x00 << 5)
#define SPI_POL_CK1_POL_NEG	(0x01 << 4)
#define SPI_POL_CK1_POL_POS	(0x00 << 4)
#define SPI_POL_CS1_POL_NEG	(0x01 << 3)
#define SPI_POL_CS1_POL_POS	(0x00 << 3)

#define SPI_POL_CK0_DLY_ON	(0x01 << 2)
#define SPI_POL_CK0_DLY_OFF	(0x00 << 2)
#define SPI_POL_CK0_POL_NEG	(0x01 << 1)
#define SPI_POL_CK0_POL_POS	(0x00 << 1)
#define SPI_POL_CS0_POL_NEG	(0x01 << 0)
#define SPI_POL_CS0_POL_POS	(0x00 << 0)

/* SPI_POL: default value */
#define SPI_POL_VAL_SPI0	(SPI_POL_CSW_8SCLK | \
	SPI_POL_CK3_DLY_OFF | SPI_POL_CK3_POL_POS | SPI_POL_CS3_POL_POS | \
	SPI_POL_CK2_DLY_OFF | SPI_POL_CK2_POL_NEG | SPI_POL_CS2_POL_NEG | \
	SPI_POL_CK1_DLY_OFF | SPI_POL_CK1_POL_NEG | SPI_POL_CS1_POL_NEG | \
	SPI_POL_CK0_DLY_OFF | SPI_POL_CK0_POL_NEG | SPI_POL_CS0_POL_NEG)

#define SPI_POL_VAL_SPI1	(SPI_POL_CSW_8SCLK | \
	SPI_POL_CK3_DLY_OFF | SPI_POL_CK3_POL_POS | SPI_POL_CS3_POL_POS | \
	SPI_POL_CK2_DLY_OFF | SPI_POL_CK2_POL_POS | SPI_POL_CS2_POL_POS | \
	SPI_POL_CK1_DLY_OFF | SPI_POL_CK1_POL_POS | SPI_POL_CS1_POL_POS | \
	SPI_POL_CK0_DLY_OFF | SPI_POL_CK0_POL_POS | SPI_POL_CS0_POL_POS)

/* rst_flag */
#define SPI_RESET_OFF		0
#define SPI_RESET_ON		1

/**  ADD FOR spi_write_without_intr END **/
#endif

/*
 * bit length
 */
#define SPI_NB_8BIT		8
#define SPI_NB_16BIT		16
#define SPI_NB_32BIT		32

/*
 * chip select
 */
#define SPI_CS_SEL_CS0		0
#define SPI_CS_SEL_CS1		1
#define SPI_CS_SEL_CS2		2
#define SPI_CS_SEL_CS3		3

/*
 * master/slave
 */
#define SPI_M_S_MASTER		0
#define SPI_M_S_SLAVE		1

/*
 * dma
 */
#define SPI_DMA_OFF		0
#define SPI_DMA_ON		1

/*
 * transfer beginning cycle
 */
#define SPI_TX_TIM_30WORD	0
#define SPI_TX_TIM_16WORD	1
#define SPI_TX_TIM_8WORD	2
#define SPI_TX_TIM_1WORD	3

/*
 * clock
 */
/* SPI_CONFIG.pol */
/*zha 20060624*/
#define FM_TUNER_CLOCK
#ifdef FM_TUNER_CLOCK
#define SPI_SCLK_400KHZ     0
#define SPI_SCLK_1500KHZ	1
#define SPI_SCLK_3MHZ		2
#define SPI_SCLK_6MHZ		3
#define SPI_SCLK_12MHZ		4
#define SPI_SCLK_24MHZ		5
#define SPI_SCLK_48MHZ		6
#else
#define SPI_SCLK_1500KHZ	0
#define SPI_SCLK_3MHZ		1
#define SPI_SCLK_6MHZ		2
#define SPI_SCLK_12MHZ		3
#define SPI_SCLK_24MHZ		4
#define SPI_SCLK_48MHZ		5
#endif

#ifdef CONFIG_MP201
/* device number */
/* SPI_CONFIG.pol */
#define SPI_DEV_SPI0      0
#define SPI_DEV_SPI1      1

/* bloking mode*/
#define SPI_BLOCK     0
#define SPI_NONBLOCK  1

/* read/write mode*/
#define SPI_READ_MODE      1
#define SPI_WRITE_MODE     2
#endif
/*
 * operation command
 */
#define SPI_CMD_RX_START	_IO('s', 0x01)
#define SPI_CMD_RX_STOP		_IO('s', 0x02)

/*
 * setting command
 */
#define SPI_CMD_GET_MODE	_IOR('s', 0x11, spi_mode_t)
#define SPI_CMD_SET_MODE	_IOW('s', 0x12, spi_mode_t)

#ifdef CONFIG_MP201
extern int mp201_spi_read(unsigned char dev, char *buf, unsigned int count, unsigned char block_mode);

extern int mp201_spi_write(unsigned char dev, const char *buf,  unsigned int count, unsigned char block_mode);

extern int mp201_spi_init(unsigned char dev, unsigned char rw_mode);

extern int mp201_spi_end(unsigned char dev);

extern int mp201_spi_setmode(unsigned char dev, spi_mode_t *mode);

/**  ADD FOR spi_write_without_intr **/
extern int
spi_write_without_intr(const SPI_CONFIG *config, const char *buf, int rst_flag);
/**  ADD FOR spi_write_without_intr END **/

#endif
#endif				/* __ASM_ARM_ARCH_MP200_SPI_H */
