/*
 * Copyright 2005 Freescale Semiconductor, Inc. All Rights Reserved.
 */

/* 
 * The code contained herein is licensed under the GNU General Public
 * License. You may obtain a copy of the GNU General Public License
 * Version 2 or later at the following locations:
 * 
 * http://www.opensource.org/licenses/gpl-license.html 
 * http://www.gnu.org/copyleft/gpl.html
 */

#ifndef __BOARD_H

#define __BOARD_H

/*!
 * @defgroup BRDCFG Board Configuration Options
 * @ingroup MSL
 */

/*! 
 * @file board.h
 *
 * @brief This file contains all the board level configuration options.
 *
 * It currently hold the options defined for MX3 ADS Platform.
 * 
 * @ingroup BRDCFG
 */

/*
 * Include Files
 */
#include <linux/config.h>

/*
 * The modes of the UART ports
 */
#define MODE_DTE                0
#define MODE_DCE                1
/*
 * Is the UART configured to be a IR port
 */
#define IRDA                    0
#define NO_IRDA                 1

/*!
 * @name MXC UART EVB board level configurations
 */
/*! @{ */
/*!
 * Specify the max baudrate for the MXC UARTs for your board, do not specify a max
 * baudrate greater than 1500000. This is used while specifying the UART Power
 * management constraints.
 */
#define MAX_UART_BAUDRATE       1500000
/*!
 * Specifies if the Irda transmit path is inverting
 */
#define MXC_IRDA_TX_INV         MXC_UARTUCR3_INVT
/*!
 * Specifies if the Irda receive path is inverting
 */
#define MXC_IRDA_RX_INV         0
/* UART 1 configuration */
/*!
 * This define specifies if the UART port is configured to be in DTE or 
 * DCE mode. There exists a define like this for each UART port. Valid 
 * values that can be used are \b MODE_DTE or \b MODE_DCE.
 */
#define UART1_MODE              MODE_DCE
/*!
 * This define specifies if the UART is to be used for IRDA. There exists a 
 * define like this for each UART port. Valid values that can be used are
 * \b IRDA or \b NO_IRDA. 
 */
#define UART1_IR                NO_IRDA
/*!
 * This define is used to enable or disable a particular UART port. If 
 * disabled, the UART will not be registered in the file system and the user
 * will not be able to access it. There exists a define like this for each UART
 * port. Specify a value of 1 to enable the UART and 0 to disable it.
 */
#define UART1_ENABLED           1
/*! @} */
/* UART 2 configuration */
#define UART2_MODE              MODE_DCE
#define UART2_IR                NO_IRDA
#define UART2_ENABLED           1
/* UART 3 configuration */
#define UART3_MODE              MODE_DCE
#define UART3_IR                NO_IRDA
#define UART3_ENABLED           1
/* UART 4 configuration */
#define UART4_MODE              MODE_DCE
#define UART4_IR                IRDA
#define UART4_ENABLED           1
/* UART 5 configuration */
#define UART5_MODE              MODE_DCE
#define UART5_IR                NO_IRDA
#define UART5_ENABLED           1

/*!
 * @name Configurable Graphics/IPU parameters
 */
/*! @{ */
/*!
 * Physical address start of IPU buffer memory
 */
#define MXCIPU_MEM_ADDRESS      0x83C00000

#ifdef CONFIG_VIRTIO_SUPPORT  
#define CONFIG_DUMB_SENSOR      1
#else
#undef CONFIG_DUMB_SENSOR
#endif

/*!
 * Size of IPU buffer memory
 * Note: Currently the last 4MB in the 64MB range is used.
 */
#define MXCIPU_MEM_SIZE         SZ_4M

/*!
 * Screen width and height
 */
#if defined(CONFIG_FB_MXC_VGA_PANEL) 
#define MXCFB_SCREEN_WIDTH      640
#define MXCFB_SCREEN_HEIGHT     480
#elif defined(CONFIG_FB_MXC_EPSON_PANEL)
#define MXCFB_SCREEN_WIDTH      176
#define MXCFB_SCREEN_HEIGHT     220
#else
#define MXCFB_SCREEN_WIDTH      240
#define MXCFB_SCREEN_HEIGHT     320
#endif

/*!
 * Sensor screen width
 */
#define MXC_SENSOR_SCREEN_WIDTH         240
/*!
 * Sensor screen height 
 */
#define MXC_SENSOR_SCREEN_HEIGHT        320
/*! @} End configurable parameters */

/*!
 * @name Keypad Configurations
 */
/*! @{ */
/*!
 * Maximum number of rows (0 to 7)
 */
#define MAXROW                          8
/*!
 * Maximum number of columns (0 to 7)
 */
#define MAXCOL                          8
/*! @} */

/*!
 * @name PBC Controller parameters
 */
/*! @{ */
/*!
 * Base address of PBC controller
 */
#define PBC_BASE_ADDRESS        IO_ADDRESS(CS4_BASE_ADDR)
/* Offsets for the PBC Controller register */
/*!
 * TODO: This define should be removed when Virtio updates their implementation
 * IRQSTAT register offset within PBC. IRQSTAT of PBC register will indicate
 * the current state of the Irq(1:0) interrupts.
 */
#define PBC_IRQSTAT             0x000002
/*!
 * PBC Board status register offset
 */
#define PBC_BSTAT               0x000002
/*!
 * PBC Board control register 1 set address.
 */
#define PBC_BCTRL1_SET          0x000004
/*!
 * PBC Board control register 1 clear address.
 */
#define PBC_BCTRL1_CLEAR        0x000006
/*!
 * PBC Board control register 2 set address.
 */
#define PBC_BCTRL2_SET          0x000008
/*!
 * PBC Board control register 2 clear address.
 */
#define PBC_BCTRL2_CLEAR        0x00000A
/*!
 * PBC Board control register 3 set address.
 */
#define PBC_BCTRL3_SET          0x00000C
/*!
 * PBC Board control register 3 clear address.
 */
#define PBC_BCTRL3_CLEAR        0x00000E
/*!
 * PBC Board control register 3 set address.
 */
#define PBC_BCTRL4_SET          0x000010
/*!
 * PBC Board control register 3 clear address.
 */
#define PBC_BCTRL4_CLEAR        0x000012
/*!
 * PBC Interrupt mask register set address.
 */
#define PBC_INTMASK_SET         0x00001A
/*!
 * PBC Interrupt mask register clear address.
 */
#define PBC_INTMASK_CLEAR       0x00001C

/*!
 * External UART A.
 */
#define PBC_SC16C652_UARTA      0x010000
/*!
 * External UART B.
 */
#define PBC_SC16C652_UARTB      0x010010
/*!
 * Ethernet Controller IO base address.
 */
#define PBC_CS8900A_IOBASE      0x020000
/*!
 * Ethernet Controller Memory base address.
 */
#define PBC_CS8900A_MEMBASE     0x021000
/*!
 * Ethernet Controller DMA base address.
 */
#define PBC_CS8900A_DMABASE     0x022000
/*!
 * External chip select 0.
 */
#define PBC_XCS0                0x040000
/*!
 * LCD Display enable.
 */
#define PBC_LCD_EN_B            0x060000
/*!
 * Code test debug enable.
 */
#define PBC_CODE_B              0x070000
/*!
 * PSRAM memory select.
 */
#define PBC_PSRAM_B             0x5000000

/* PBC Board Control Register 1 bit definitions */
#define PBC_BCTRL1_ERST         0x0001 /* Ethernet Reset */
#define PBC_BCTRL1_URST         0x0002 /* Reset External UART controller */
#define PBC_BCTRL1_UENA         0x0004 /* Enable UART A transceiver */
#define PBC_BCTRL1_UENB         0x0008 /* Enable UART B transceiver */
#define PBC_BCTRL1_UENCE        0x0010 /* Enable UART CE transceiver */
#define PBC_BCTRL1_IREN         0x0020 /* Enable the IRDA transmitter */
#define PBC_BCTRL1_LED0         0x0040 /* Used to control LED 0 (green) */
#define PBC_BCTRL1_LED1         0x0080 /* Used to control LED 1 (yellow) */
#define PBC_BCTRL1_SENSOR1_ON	0x0600 /* Enable Sensor 1 */
#define PBC_BCTRL1_SENSOR2_ON	0x3000 /* Enable Sensor 2 */
#define PBC_BCTRL1_BEND         0x4000 /* Big Endian Select */
#define PBC_BCTRL1_LCDON        0x8000 /* Enable the LCD */

/* PBC Board Control Register 2 bit definitions */
#define PBC_BCTRL2_USELA	0x0001 /* UART A Select, 0 = UART1, 1 = UART5 */
#define PBC_BCTRL2_USELB	0x0002 /* UART B Select, 0 = UART3, 1 = UART5 */
#define PBC_BCTRL2_USELC	0x0004 /* UART C Select, 0 = UART2, 1 = UART1 */
#define PBC_BCTRL2_UMODENA	0x0008 /* UART A Modem Signals Enable, 0 = enabled */
#define PBC_BCTRL2_UMODENC	0x0008 /* UART C Modem Signals Enable, 0 = enabled */
#define PBC_BCTRL2_CSI_EN	0x0020 /* Enable the CSI interface, 0 = enabled */
#define PBC_BCTRL2_ATA_EN	0x0040 /* Enable the ATA interface, 0 = enabled */
#define PBC_BCTRL2_ATA_SEL	0x0080 /* ATA Select, 0 = group A, 1 = group B */
#define PBC_BCTRL2_IRDA_MOD	0x0100 /* IRDA Mode (see CPLD spec) */
#define PBC_BCTRL2_LDC_RST0	0x0200 /* LCD 0 Reset, 1 = reset signal asserted */
#define PBC_BCTRL2_LDC_RST1	0x0400 /* LCD 1 Reset, 1 = reset signal asserted */ 
#define PBC_BCTRL2_LDC_RST2	0x0800 /* LCD 2 Reset, 1 = reset signal asserted */ 
#define PBC_BCTRL2_LDCIO_EN	0x1000 /* LCD GPIO Enable, 0 = enabled */
#define PBC_BCTRL2_CT_CS	0x2000 /* Code Test Chip Select, = Code Test selected */
#define PBC_BCTRL2_VPP_EN	0x4000 /* PCMCIA VPP Enable, 1 = power on */
#define PBC_BCTRL2_VCC_EN	0x8000 /* PCMCIA VCC Enable, 1 = power on */

/* PBC Board Control Register 3 bit definitions */
#define PBC_BCTRL3_OTG_FS_SEL	0x0001 /* USB OTG Full Speed Select, 0 = ATLAS, 1 = CPU */
#define PBC_BCTRL3_OTG_FS_EN	0x0002 /* USB OTG Full Speed Enable, 0 = enabled */
#define PBC_BCTRL3_FSH_SEL	0x0004 /* USB Full Speed Host Select, 0 = Group A, 1 = Group B */
#define PBC_BCTRL3_FSH_EN	0x0008 /* USB Full Speed Host Enable, 0 = enabled */
#define PBC_BCTRL3_HSH_SEL	0x0010 /* USB High Speed Host Select, 0 = Group A, 1 = Group B */
#define PBC_BCTRL3_HSH_EN	0x0020 /* USB High Speed Host Enable, 0 = enabled */
#define PBC_BCTRL3_FSH_MOD	0x0040 /* USB Full Speed Host Mode, 0 = Differential, 1 = Single ended */
#define PBC_BCTRL3_OTG_HS_EN	0x0080 /* USB OTG High Speed Enable, 0 = enabled */
#define PBC_BCTRL3_OTG_VBUS_EN	0x0100 /* USB OTG VBUS Regulator Enable, 0 = enabled */
#define PBC_BCTRL3_FSH_VBUS_EN	0x0200 /* USB Full Speed Host VBUS Regulator Enable, 0 = enabled */
#define PBC_BCTRL3_CARD1_SEL	0x0400 /* Card1 Select, 0 = SD1, 1 = MS1 */
#define PBC_BCTRL3_CARD2_SEL	0x0800 /* Card2 Select, 0 = PCMCIA & SD2, 1 = MS2 */
#define PBC_BCTRL3_SYNTH_RST	0x1000 /* Audio Synthesizer Reset, 0 = reset asserted */
#define PBC_BCTRL3_VSIM_EN	0x2000 /* VSIM Regulator Enable, 1 = enabled */
#define PBC_BCTRL3_VESIM_EN	0x4000 /* VESIM Regulator Enable, 1 = enabled */
#define PBC_BCTRL3_SPI3_RESET	0x8000 /* CSPI3 Connector Reset, 0 = reset asserted */

/* PBC Board Control Register 4 bit definitions */
#define PBC_BCTRL4_CSI_MSB_EN	0x0001 /* CSI MSB Enable, 0 = CSI_Data[3:0] enabled */
#define PBC_BCTRL4_REGEN_SEL	0x0002 /* Regulator Enable Select, 0 = enabled */
#define PBC_BCTRL4_USER_OFF	0x0004 /* User Off Indication, 1 = user off confirmation */
#define PBC_BCTRL4_VIB_EN	0x0008 /* Vibrator Enable, 1 = enabled */
#define PBC_BCTRL4_PCMCIA_EN	0x0010 /* PCMCIA Enable, 0 = buffer enabled */

#define PBC_INT_CS8900A         4
/*! @} */


/*!
 * @name  Defines Base address and IRQ used for CS8900A Ethernet Controller on MXC Boards
 */
/*! @{*/
/*! This is System IRQ used by CS8900A for interrupt generation taken from platform.h */
#define CS8900AIRQ              PBC_INT_CS8900A    
/*! This is I/O Base address used to access registers of CS8900A on MXC ADS */
#define CS8900A_BASE_ADDRESS    (PBC_BASE_ADDRESS + PBC_CS8900A_IOBASE + 0x300)
/*! @} */

#endif /* __BOARD_H */
