/*
 * Copyright 2005 Freescale Semiconductor, Inc. All Rights Reserved.
 */

/* 
 * The code contained herein is licensed under the GNU General Public
 * License. You may obtain a copy of the GNU General Public License
 * Version 2 or later at the following locations:
 * 
 * http://www.opensource.org/licenses/gpl-license.html 
 * http://www.gnu.org/copyleft/gpl.html
 */

#ifndef __CLOCK_H

#define __CLOCK_H

/*!
 * @defgroup CLOCKS Clocking Setup and Retrieving
 * @ingroup MSL
 */

/*! 
 * @file clock.h
 *
 * @brief API for setting up and retrieving clocks.
 *
 * This file contains API for setting up and retrieving clocks.
 *
 * @ingroup CLOCKS
 */

#include <asm/arch/platform.h>

/* Register offsets */
#define MXC_CCM_CCMR            0x0
#define MXC_CCM_PDR0            0x4
#define MXC_CCM_PDR1            0x8
#define MXC_CCM_RCSR            0xC
#define MXC_CCM_MPCTL           0x10
#define MXC_CCM_UPCTL           0x14
#define MXC_CCM_SRPCTL          0x18
#define MXC_CCM_COSR            0x1C
#define MXC_CCM_CGR0            0x20
#define MXC_CCM_CGR1            0x24
#define MXC_CCM_CGR2            0x28
#define MXC_CCM_WIMR            0x2C
#define MXC_CCM_LDC             0x30
#define MXC_CCM_DCVR0           0x34
#define MXC_CCM_DCVR1           0x38
#define MXC_CCM_DCVR2           0x3C
#define MXC_CCM_DCVR3           0x40
#define MXC_CCM_LTR0            0x44
#define MXC_CCM_LTR1            0x48
#define MXC_CCM_LTR2            0x4C
#define MXC_CCM_LTR3            0x50
#define MXC_CCM_LTBR0           0x54
#define MXC_CCM_LTBR1           0x58
#define MXC_CCM_PMCR0           0x5C
#define MXC_CCM_PMCR1           0x60
#define MXC_CCM_PDR2            0x64

/* Register bit definitions */
#define MXC_CCM_CCMR_CSCS                       0x02000000
#define MXC_CCM_CCMR_PERCS                      0x01000000
#define MXC_CCM_CCMR_SSI1S_OFFSET               18
#define MXC_CCM_CCMR_SSI1S_MASK                 (0x3 << 18)
#define MXC_CCM_CCMR_SSI2S_OFFSET               21
#define MXC_CCM_CCMR_SSI2S_MASK                 (0x3 << 21)
#define MXC_CCM_CCMR_FIRS_OFFSET                11
#define MXC_CCM_CCMR_FIRS_MASK                  (0x3 << 11)
#define MXC_CCM_CCMR_MDS                        0x80
#define MXC_CCM_CCMR_MPE                        0x8
#define MXC_CCM_CCMR_PRCS_OFFSET                1
#define MXC_CCM_CCMR_PRCS_MASK                  (0x3 << 1)

#define MXC_CCM_PDR0_CSI_PODF_OFFSET            23
#define MXC_CCM_PDR0_CSI_PODF_MASK              (0x1FF << 23)
#define MXC_CCM_PDR0_PER_PODF_OFFSET            16
#define MXC_CCM_PDR0_PER_PODF_MASK              (0x1F << 16)
#define MXC_CCM_PDR0_HSP_PODF_OFFSET            11
#define MXC_CCM_PDR0_HSP_PODF_MASK              (0x7 << 11)
#define MXC_CCM_PDR0_NFC_PODF_OFFSET            8
#define MXC_CCM_PDR0_NFC_PODF_MASK              (0x7 << 8)
#define MXC_CCM_PDR0_IPG_PODF_OFFSET            6
#define MXC_CCM_PDR0_IPG_PODF_MASK              (0x3 << 6)
#define MXC_CCM_PDR0_MAX_PODF_OFFSET            3
#define MXC_CCM_PDR0_MAX_PODF_MASK              (0x7 << 3)
#define MXC_CCM_PDR0_MCU_PODF_OFFSET            0
#define MXC_CCM_PDR0_MCU_PODF_MASK              0x7

#define MXC_CCM_PDR1_USB_PRDF_OFFSET            30
#define MXC_CCM_PDR1_USB_PRDF_MASK              (0x3 << 30)
#define MXC_CCM_PDR1_USB_PODF_OFFSET            27
#define MXC_CCM_PDR1_USB_PODF_MASK              (0x7 << 27)
#define MXC_CCM_PDR1_FIRI_PRE_PODF_OFFSET       24
#define MXC_CCM_PDR1_FIRI_PRE_PODF_MASK         (0x7 << 24)
#define MXC_CCM_PDR1_FIRI_PODF_OFFSET           18
#define MXC_CCM_PDR1_FIRI_PODF_MASK             (0x3F << 18)
#define MXC_CCM_PDR1_SSI2_PRE_PODF_OFFSET       15
#define MXC_CCM_PDR1_SSI2_PRE_PODF_MASK         (0x7 << 15)
#define MXC_CCM_PDR1_SSI2_PODF_OFFSET           9
#define MXC_CCM_PDR1_SSI2_PODF_MASK             (0x3F << 9)
#define MXC_CCM_PDR1_SSI1_PRE_PODF_OFFSET       6
#define MXC_CCM_PDR1_SSI1_PRE_PODF_MASK         (0x7 << 6)
#define MXC_CCM_PDR1_SSI1_PODF_OFFSET           0
#define MXC_CCM_PDR1_SSI1_PODF_MASK             0x3F

/* Bit definitions for both MCU, USB and SR PLL control registers */
#define MXC_CCM_PCTL_BRM                        0x80000000
#define MXC_CCM_PCTL_PD_OFFSET                  26
#define MXC_CCM_PCTL_PD_MASK                    (0xF << 26)
#define MXC_CCM_PCTL_MFD_OFFSET                 16
#define MXC_CCM_PCTL_MFD_MASK                   (0x3FF << 16)
#define MXC_CCM_PCTL_MFI_OFFSET                 10
#define MXC_CCM_PCTL_MFI_MASK                   (0xF << 10)
#define MXC_CCM_PCTL_MFN_OFFSET                 0
#define MXC_CCM_PCTL_MFN_MASK                   0x3FF

#define MXC_CCM_CGR0_SD_MMC1                    0x3
#define MXC_CCM_CGR0_SD_MMC2                    (0x3 << 2)
#define MXC_CCM_CGR0_GPT                        (0x3 << 4)   
#define MXC_CCM_CGR0_EPIT1                      (0x3 << 6)   
#define MXC_CCM_CGR0_EPIT2                      (0x3 << 8)
#define MXC_CCM_CGR0_IIM                        (0x3 << 10)
#define MXC_CCM_CGR0_ATA                        (0x3 << 12)
#define MXC_CCM_CGR0_SDMA                       (0x3 << 14)
#define MXC_CCM_CGR0_CSPI3                      (0x3 << 16)
#define MXC_CCM_CGR0_RNG                         (0x3 << 18)
#define MXC_CCM_CGR0_UART1                      (0x3 << 20)      
#define MXC_CCM_CGR0_UART2                      (0x3 << 22)
#define MXC_CCM_CGR0_SSI1                       (0x3 << 24)
#define MXC_CCM_CGR0_I2C1                       (0x3 << 26)
#define MXC_CCM_CGR0_I2C2                       (0x3 << 28)
#define MXC_CCM_CGR0_I2C3                       (0x3 << 30)

#define MXC_CCM_CGR1_HANTRO                     0x3
#define MXC_CCM_CGR1_MEMSTICK1                  (0x3 << 2)
#define MXC_CCM_CGR1_MEMSTICK2                  (0x3 << 4)
#define MXC_CCM_CGR1_CSI                        (0x3 << 6)
#define MXC_CCM_CGR1_RTC                        (0x3 << 8)
#define MXC_CCM_CGR1_WDOG                       (0x3 << 10)
#define MXC_CCM_CGR1_PWM                        (0x3 << 12)
#define MXC_CCM_CGR1_SIM                        (0x3 << 14)
#define MXC_CCM_CGR1_ECT                        (0x3 << 16)
#define MXC_CCM_CGR1_USBOTG                     (0x3 << 18)
#define MXC_CCM_CGR1_KPP                        (0x3 << 20)
#define MXC_CCM_CGR1_IPU                        (0x3 << 22)
#define MXC_CCM_CGR1_UART3                      (0x3 << 24)
#define MXC_CCM_CGR1_UART4                      (0x3 << 26)
#define MXC_CCM_CGR1_UART5                      (0x3 << 28)
#define MXC_CCM_CGR1_OWIRE                      (0x3 << 30)

#define MXC_CCM_CGR2_SSI2                       0x3
#define MXC_CCM_CGR2_CSPI1                      (0x3 << 2)
#define MXC_CCM_CGR2_CSPI2                      (0x3 << 4)
#define MXC_CCM_CGR2_GACC                       (0x3 << 6)
#define MXC_CCM_CGR2_EMI                        (0x3 << 8)
#define MXC_CCM_CGR2_RTIC                       (0x3 << 10)
#define MXC_CCM_CGR2_FIRI                       (0x3 << 12)
#define MXC_CCM_CGR2_IPMUX1                     (0x3 << 14)
#define MXC_CCM_CGR2_IPMUX2                     (0x3 << 16)

#define MXC_CCM_PMCR0_DFSUP1                    0x80000000
#define MXC_CCM_PMCR0_DFSUP0                    0x40000000
#define MXC_CCM_PMCR0_UDSC                      0x08000000
#define MXC_CCM_PMCR0_DVFEV                     0x00800000
#define MXC_CCM_PMCR0_DVFIS                     0x00400000
#define MXC_CCM_PMCR0_LBMI                      0x00200000
#define MXC_CCM_PMCR0_LBFL                      0x00100000
#define MXC_CCM_PMCR0_PTVIS                     0x00020000
#define MXC_CCM_PMCR0_UPDTEN                    0x00010000
#define MXC_CCM_PMCR0_FSVAIM                    0x00008000
#define MXC_CCM_PMCR0_DPVCR                     0x00001000
#define MXC_CCM_PMCR0_DPVV                      0x00000800
#define MXC_CCM_PMCR0_WFIM                      0x00000400
#define MXC_CCM_PMCR0_DRCE3                     0x00000200
#define MXC_CCM_PMCR0_DRCE2                     0x00000100
#define MXC_CCM_PMCR0_DRCE1                     0x00000080
#define MXC_CCM_PMCR0_DRCE0                     0x00000040
#define MXC_CCM_PMCR0_DCR                       0x00000020
#define MXC_CCM_PMCR0_DVFEN                     0x00000010
#define MXC_CCM_PMCR0_PTVAIM                    0x00000008
#define MXC_CCM_PMCR0_DPTEN                     0x00000001

#define MXC_CCM_PDR2_MST2_PDF_MASK              (0x3F << 7)
#define MXC_CCM_PDR2_MST2_PDF_OFFSET            7
#define MXC_CCM_PDR2_MST1_PDF_MASK              0x3F 
#define MXC_CCM_PDR2_MST1_PDF_OFFSET            0

#define CKIH_CLK_FREQ                           26000000
#define CKIL_CLK_FREQ                           32768

enum plls {
        MCUPLL,
        USBPLL,
        SERIALPLL,
        PLL_REF_CLK,
};

enum mxc_clocks {
        CPU_CLK,
        AHB_CLK,
        IPG_CLK,
        NFC_CLK,
        USB_CLK,
        UART1_BAUD,
        UART2_BAUD,
        UART3_BAUD,
        UART4_BAUD,
        UART5_BAUD,
        I2C_CLK,
        I2C2_CLK,
        I2C3_CLK,
        SSI1_BAUD,
        SSI2_BAUD,
        CSI_BAUD,
        FIRI_BAUD,
        IPU_CLK,
        SDHC1_CLK,
        SDHC2_CLK,
        KPP_CLK,
        RTIC_CLK,
        RNG_CLK,
        CSPI1_CLK,
        CSPI2_CLK,
        CSPI3_CLK,
        GPT_CLK,
        PWM_CLK,
        EPIT1_CLK,
        EPIT2_CLK,
        SDMA_CLK,
        ATA_CLK,
        MBX_CLK,
        HANTRO_CLK,
        MSTICK1_BAUD,
        MSTICK2_BAUD,
};

/*!
 * This function returns the PLL output value in Hz based on pll.
 * @param       pll     PLL as defined in enum plls
 * @return      PLL value in Hz.
 */
unsigned long mxc_pll_clock(enum plls pll);

/*!
 * This function returns the various MXC clock value in Hz
 *
 * @param       clk     as defined in enum mxc_clocks
 *
 * @return      clock value in Hz
 */
unsigned long mxc_get_clocks(enum mxc_clocks clk);

/*!
 * This function sets the PLL source for a clock.
 *
 * @param clk     as defined in enum mxc_clocks
 * @param pll_num the PLL that you wish to use as source for this clock
 */
void mxc_set_clocks_pll(enum mxc_clocks clk, enum plls pll_num);

/*!
 * This function sets the division factor for a clock.
 *
 * @param clk as defined in enum mxc_clocks
 * @param div the division factor to be used for the clock (For SSI, pass in
 *            2 times the expected division value to account for FP vals)
 */
void mxc_set_clocks_div(enum mxc_clocks clk, unsigned int div);

/*!
 * This function is called to gate off the individual module clocks
 *
 * @param       clk     as defined in enum mxc_clocks
 */
void mxc_clks_disable(enum mxc_clocks clk);

/*!
 * This function is called to enable the individual module clocks
 *
 * @param       clk     as defined in enum mxc_clocks
 */
void mxc_clks_enable(enum mxc_clocks clk);

/*!
 * This function is called to read the contents of a CCM register
 *
 * @param reg_offset the CCM register that will read
 *
 * @return the register contents
 */
volatile unsigned long mxc_ccm_get_reg(unsigned int reg_offset);

/*!
 * This function is called to modify the contents of a CCM register
 *
 * @param reg_offset the CCM register that will read
 * @param mask       the mask to be used to clear the bits that are to be modified
 * @param data       the data that should be written to the register
 */
void mxc_ccm_modify_reg(unsigned int reg_offset, unsigned int mask, unsigned int data);
/*!
 * This function is used to modify PLL registers to generate the required
 * frequency.
 *
 * @param  pll      the PLL that you wish to modify
 * @param  mfi      multiplication factor integer part
 * @param  pdf      pre-division factor
 * @param  mfd      multiplication factor denominator
 * @param  mfn      multiplication factor numerator
 */
void mxc_pll_set(enum plls pll, unsigned int mfi, unsigned int pdf,
                 unsigned int mfd, unsigned int mfn);

#endif /* __CLOCK_H */
