/*
 *  Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 *    - Platform specific register memory map
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef __PLATFORM_H__
#define __PLATFORM_H__

#include <asm/arch/hardware.h>

/*!
 * @file platform.h
 * @brief This file contains the register definitions and physical
 * to virtual address translations for peripherals.
 *
 * @ingroup System
 */

/*
 * The memory map is very fragmented. We define the memory size and
 * location of each module and map it to kernel space (in kernel space we
 * collect the memory fragments to a homogeneous memory block)
 */

/*
 * IRAM
 */
#define     IRAM_BASE_ADDR          0x1FFC0000       /* internal ram */
#define     IRAM_BASE_ADDR_VIRT     0xD0000000
#define     IRAM_SIZE               SZ_16K

/*
 * L2CC
 */
#define     L2CC_BASE_ADDR          0x30000000
#define     L2CC_BASE_ADDR_VIRT     0xD1000000
#define     L2CC_SIZE               SZ_64K

/*
 * AIPS 1
 */
#define     AIPS1_BASE_ADDR         0x43F00000
#define     AIPS1_BASE_ADDR_VIRT    0xD4000000
#define     AIPS1_SIZE              SZ_1M
#define     MAX_BASE_ADDR           0x43F04000
#define     EVTMON_BASE_ADDR        0x43F08000
#define     CLKCTL_BASE_ADDR        0x43F0C000
#define     ETB_SLOT4_BASE_ADDR     0x43F10000
#define     ETB_SLOT5_BASE_ADDR     0x43F14000
#define     ECT_CTIO_BASE_ADDR      0x43F18000
#define     I2C_BASE_ADDR           0x43F80000
#define     I2C3_BASE_ADDR          0x43F84000
#define     OTG_BASE_ADDR           0x43F88000
#define     ATA_BASE_ADDR           0x43F8C000
#define     UART1_BASE_ADDR         0x43F90000
#define     UART2_BASE_ADDR         0x43F94000
#define     I2C2_BASE_ADDR          0x43F98000
#define     OWIRE_BASE_ADDR         0x43F9C000
#define     SSI1_BASE_ADDR          0x43FA0000
#define     CSPI1_BASE_ADDR         0x43FA4000
#define     KPP_BASE_ADDR           0x43FA8000
#define     IOMUXC_BASE_ADDR        0x43FAC000
#define     UART4_BASE_ADDR         0x43FB0000
#define     UART5_BASE_ADDR         0x43FB4000
#define     ECT_IP1_BASE_ADDR       0x43FB8000
#define     ECT_IP2_BASE_ADDR       0x43FBC000

/*
 * SPBA
 */
#define     SPBA_BASE_ADDR          0x50000000
#define     SPBA_BASE_ADDR_VIRT     0xD4100000
#define     SPBA_SIZE               SZ_1M
#define     MMC_SDHC1_BASE_ADDR     0x50004000
#define     MMC_SDHC2_BASE_ADDR     0x50008000
#define     UART3_BASE_ADDR         0x5000C000
#define     CSPI2_BASE_ADDR         0x50010000
#define     SSI2_BASE_ADDR          0x50014000
#define     SIM_BASE_ADDR           0x50018000
#define     IIM_BASE_ADDR           0x5001C000
#define     ATA_DMA_BASE_ADDR       0x50020000
#define     MSHC1_BASE_ADDR         0x50024000
#define     MSHC2_BASE_ADDR         0x50024000
#define     SPBA_CTRL_BASE_ADDR     0x5003C000

/*
 * AIPS 2
 */
#define     AIPS2_BASE_ADDR         0x53F00000
#define     AIPS2_BASE_ADDR_VIRT    0xD4200000
#define     AIPS2_SIZE              SZ_1M
#define     CCM_BASE_ADDR           0x53F80000
#define     CSPI3_BASE_ADDR         0x53F84000
#define     FIRI_BASE_ADDR          0x53F8C000
#define     GPT1_BASE_ADDR          0x53F90000
#define     EPIT1_BASE_ADDR         0x53F94000
#define     EPIT2_BASE_ADDR         0x53F98000
#define     GPIO3_BASE_ADDR         0x53FA4000
#define     SCC_BASE_ADDR           0x53FAC000
#define     SCM_BASE_ADDR           0x53FAE000
#define     SMN_BASE_ADDR           0x53FAF000
#define     RNGA_BASE_ADDR          0x53FB0000
#define     IPU_CTRL_BASE_ADDR      0x53FC0000
#define     AUDMUX_BASE_ADDR        0x53FC4000
#define     MPEG4_ENC_BASE_ADDR     0x53FC8000
#define     GPIO1_BASE_ADDR         0x53FCC000
#define     GPIO2_BASE_ADDR         0x53FD0000
#define     SDMA_BASE_ADDR          0x53FD4000
#define     RTC_BASE_ADDR           0x53FD8000
#define     WDOG_BASE_ADDR          0x53FDC000
#define     PWM_BASE_ADDR           0x53FE0000
#define     RTIC_BASE_ADDR          0x53FEC000

/*
 * Misc
 */
#define     ROMP_BASE_ADDR          0x60000000
#define     ROMP_BASE_ADDR_VIRT     0xD4300000
#define     ROMP_SIZE               SZ_64K

#define     AVIC_BASE_ADDR          0x68000000
#define     AVIC_BASE_ADDR_VIRT     0xD4310000
#define     AVIC_SIZE               SZ_64K

/*
 * NAND, SDRAM, WEIM, M3IF, EMI controllers
 */
#define     X_MEMC_BASE_ADDR        0xB8000000
#define     X_MEMC_BASE_ADDR_VIRT   0xD4320000
#define     X_MEMC_SIZE             SZ_64K

#define     NFC_BASE_ADDR           X_MEMC_BASE_ADDR
#define     ESDCTL_BASE_ADDR        0xB8001000
#define     WEIM_BASE_ADDR          0xB8002000
#define     M3IF_BASE_ADDR          0xB8003000
#define     PCMCIA_CTL_BASE_ADDR    0xB8004000

/*
 * Memory regions and CS
 */
/* Mx3 ADS SDRAM is from 0x80000000, 64M */
#define     SDRAM_BASE_ADDR         0x80000000
#define     IPU_MEM_BASE_ADDR       0x70000000
#define     CSD1_BASE_ADDR          0x90000000
#define     CS0_BASE_ADDR           0xA0000000
#define     CS1_BASE_ADDR           0xA8000000
#define     CS2_BASE_ADDR           0xB0000000
#define     CS3_BASE_ADDR           0xB2000000
#define     CS4_BASE_ADDR           0xB4000000
#define     CS4_BASE_ADDR_VIRT      0xEB000000
#define     CS4_SIZE                SZ_16M
#define     CS5_BASE_ADDR           0xB6000000
#define     PCMCIA_MEM_BASE_ADDR    0xBC000000

/*!
 * This macro defines the physical to virtual address mapping for all the
 * peripheral modules. It is used by passing in the physical address as x
 * and returning the virtual address. If the physical address is not mapped,
 * it returns 0xDEADBEEF
 */
#define IO_ADDRESS(x)   \
        (((x >= IRAM_BASE_ADDR) && (x < (IRAM_BASE_ADDR + IRAM_SIZE))) ? IRAM_IO_ADDRESS(x):\
        ((x >= L2CC_BASE_ADDR) && (x < (L2CC_BASE_ADDR + L2CC_SIZE))) ? L2CC_IO_ADDRESS(x):\
        ((x >= AIPS1_BASE_ADDR) && (x < (AIPS1_BASE_ADDR + AIPS1_SIZE))) ? AIPS1_IO_ADDRESS(x):\
        ((x >= SPBA_BASE_ADDR) && (x < (SPBA_BASE_ADDR + SPBA_SIZE))) ? SPBA_IO_ADDRESS(x):\
        ((x >= AIPS2_BASE_ADDR) && (x < (AIPS2_BASE_ADDR + AIPS2_SIZE))) ? AIPS2_IO_ADDRESS(x):\
        ((x >= ROMP_BASE_ADDR) && (x < (ROMP_BASE_ADDR + ROMP_SIZE))) ? ROMP_IO_ADDRESS(x):\
        ((x >= AVIC_BASE_ADDR) && (x < (AVIC_BASE_ADDR + AVIC_SIZE))) ? AVIC_IO_ADDRESS(x):\
        ((x >= CS4_BASE_ADDR) && (x < (CS4_BASE_ADDR + CS4_SIZE))) ? CS4_IO_ADDRESS(x):\
        ((x >= X_MEMC_BASE_ADDR) && (x < (X_MEMC_BASE_ADDR + X_MEMC_SIZE))) ? X_MEMC_IO_ADDRESS(x):\
        0xDEADBEEF)

/*
 * define the address mapping macros: in physical address order
 */

#define IRAM_IO_ADDRESS(x)  \
        (((x) - IRAM_BASE_ADDR) + IRAM_BASE_ADDR_VIRT)

#define L2CC_IO_ADDRESS(x)  \
        (((x) - L2CC_BASE_ADDR) + L2CC_BASE_ADDR_VIRT)

#define     AIPS1_IO_ADDRESS(x)  \
        (((x) - AIPS1_BASE_ADDR) + AIPS1_BASE_ADDR_VIRT)

#define     SPBA_IO_ADDRESS(x)  \
        (((x) - SPBA_BASE_ADDR) + SPBA_BASE_ADDR_VIRT)

#define     AIPS2_IO_ADDRESS(x)  \
        (((x) - AIPS2_BASE_ADDR) + AIPS2_BASE_ADDR_VIRT)

#define     ROMP_IO_ADDRESS(x)  \
        (((x) - ROMP_BASE_ADDR) + ROMP_BASE_ADDR_VIRT)

#define     AVIC_IO_ADDRESS(x)  \
        (((x) - AVIC_BASE_ADDR) + AVIC_BASE_ADDR_VIRT)

#define     CS4_IO_ADDRESS(x)  \
        (((x) - CS4_BASE_ADDR) + CS4_BASE_ADDR_VIRT)

#define     X_MEMC_IO_ADDRESS(x)  \
        (((x) - X_MEMC_BASE_ADDR) + X_MEMC_BASE_ADDR_VIRT)

/*
 * Interrupt numbers
 *
 * TODO: Interrupt numbers for first virtio release.
 * To be finalized once board schematic/Virtio ready.
 */
#define INT_PEN_ADS7843         0
#define INT_RESV1               1
#define INT_CS8900A             2
#define INT_I2C3                3
#define INT_I2C2                4
#define INT_MPEG4_ENCODER       5
#define INT_RTIC                6
#define INT_FIRI                7
#define INT_MMC_SDHC2           8
#define INT_MMC_SDHC1           9
#define INT_I2C                 10
#define INT_SSI2                11
#define INT_SSI1                12
#define INT_CSPI2               13
#define INT_CSPI1               14
#define INT_ATA                 15
#define INT_MBX                 16
#define INT_CSPI3               17
#define INT_UART3               18
#define INT_IIM                 19
#define INT_SIM2                20
#define INT_SIM1                21
#define INT_RNGA                22
#define INT_EVTMON              23
#define INT_KPP                 24
#define INT_RTC                 25
#define INT_PWM                 26
#define INT_EPIT2               27
#define INT_EPIT1               28
#define INT_GPT                 29
#define INT_RESV30              30
#define INT_RESV31              31
#define INT_UART2               32
#define INT_NANDFC              33
#define INT_SDMA                34
#define INT_USB1                35
#define INT_USB2                36
#define INT_USB3                37
#define INT_USB4                38
#define INT_MSHC1               39
#define INT_MSHC2               40
#define INT_IPU_ERR             41
#define INT_IPU_SYN             42
#define INT_RESV43              43
#define INT_RESV44              44
#define INT_UART1               45
#define INT_UART4               46
#define INT_UART5               47
#define INT_ECT                 48
#define INT_SCC_SCM             49
#define INT_SCC_SMN             50
#define INT_GPIO2               51
#define INT_GPIO1               52
#define INT_CCM                 53
#define INT_PCMCIA              54
#define INT_WDOG                55
#define INT_GPIO3               56
#define INT_RESV57              57
#define INT_EXT_POWER           58
#define INT_EXT_TEMPER          59
#define INT_EXT_SENSOR60        60
#define INT_EXT_SENSOR61        61
#define INT_EXT_WDOG            62
#define INT_EXT_TV              63

#define MAXIRQNUM               63
#define MAXFIQNUM               63
#define MAXSWINUM               63

/*
 *  These are useconds NOT ticks.
 *
 */
#define mSEC_1                  1000
#define mSEC_5                  (mSEC_1 * 5)
#define mSEC_10                 (mSEC_1 * 10)
#define mSEC_25                 (mSEC_1 * 25)
#define SEC_1                   (mSEC_1 * 1000)

/*
 *****************************************
 * EPIT1  Register definitions           *
 *****************************************
 */
#define _reg_EPIT1_EPITCR    ((volatile __u32 *)(IO_ADDRESS(EPIT1_BASE_ADDR + 0x00)))
#define _reg_EPIT1_EPITSR    ((volatile __u32 *)(IO_ADDRESS(EPIT1_BASE_ADDR + 0x04)))
#define _reg_EPIT1_EPITLR    ((volatile __u32 *)(IO_ADDRESS(EPIT1_BASE_ADDR + 0x08)))
#define _reg_EPIT1_EPITCMPR  ((volatile __u32 *)(IO_ADDRESS(EPIT1_BASE_ADDR + 0x0C)))
#define _reg_EPIT1_EPITCNR   ((volatile __u32 *)(IO_ADDRESS(EPIT1_BASE_ADDR + 0x10)))

/*
 *****************************************
 * EPIT2  Register definitions           *
 *****************************************
 */
#define _reg_EPIT2_EPITCR    ((volatile __u32 *)(IO_ADDRESS(EPIT2_BASE_ADDR + 0x00)))
#define _reg_EPIT2_EPITSR    ((volatile __u32 *)(IO_ADDRESS(EPIT2_BASE_ADDR + 0x04)))
#define _reg_EPIT2_EPITLR    ((volatile __u32 *)(IO_ADDRESS(EPIT2_BASE_ADDR + 0x08)))
#define _reg_EPIT2_EPITCMPR  ((volatile __u32 *)(IO_ADDRESS(EPIT2_BASE_ADDR + 0x0C)))
#define _reg_EPIT2_EPITCNR   ((volatile __u32 *)(IO_ADDRESS(EPIT2_BASE_ADDR + 0x10)))

/*
 *****************************************
 * GPT  Register definitions             *
 *****************************************
 */

#define _reg_GPT_GPTCR    ((volatile __u32 *)(IO_ADDRESS(GPT1_BASE_ADDR + 0x00)))
#define _reg_GPT_GPTPR    ((volatile __u32 *)(IO_ADDRESS(GPT1_BASE_ADDR + 0x04)))
#define _reg_GPT_GPTSR    ((volatile __u32 *)(IO_ADDRESS(GPT1_BASE_ADDR + 0x08)))
#define _reg_GPT_GPTIR    ((volatile __u32 *)(IO_ADDRESS(GPT1_BASE_ADDR + 0x0C)))
#define _reg_GPT_GPTOCR1  ((volatile __u32 *)(IO_ADDRESS(GPT1_BASE_ADDR + 0x10)))
#define _reg_GPT_GPTOCR2  ((volatile __u32 *)(IO_ADDRESS(GPT1_BASE_ADDR + 0x14)))
#define _reg_GPT_GPTOCR3  ((volatile __u32 *)(IO_ADDRESS(GPT1_BASE_ADDR + 0x18)))
#define _reg_GPT_GPTICR1  ((volatile __u32 *)(IO_ADDRESS(GPT1_BASE_ADDR + 0x1C)))
#define _reg_GPT_GPTICR2  ((volatile __u32 *)(IO_ADDRESS(GPT1_BASE_ADDR + 0x20)))
#define _reg_GPT_GPTCNT   ((volatile __u32 *)(IO_ADDRESS(GPT1_BASE_ADDR + 0x24)))

/* This is used to turn on/off debugging */
#define MXC_TRACE
#ifdef MXC_TRACE
/*!
 * This is used for error checking in debugging mode.
 */
#define MXC_ERR_CHK(a) \
        do { \
                if ((a)) { \
                        printk("Error at line %d in function %s in file %s", \
                                __LINE__, __FUNCTION__, __FILE__); \
                        BUG(); \
                } \
        } \
        while (0)
#else
#define MXC_ERR_CHK(a)
#endif

#endif  /* __PLATFORM_H__ */
