/*
 * Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __GPIO_H__
#define __GPIO_H__

/*!
 * @defgroup GPIO General Purpose Input Output (GPIO)
 * @ingroup MSL
 */

/*!
 * @file gpio.h
 * @brief This file contains the GPIO and IOMUX configuration details.
 *
 * @ingroup GPIO
 */

#include <asm/sizes.h>
#include <asm/arch/platform.h>
#include <linux/interrupt.h>

/*!
 * @name IOMUX/PAD Bit field definitions
 */

/*! @{ */

/*!
 * This enumeration is constructed based on the section 3 "IOMUX Control"
 * of the SCM-A11 IC Spec.
 */
enum iomux_pins {
        /* IOMUX for AP side */
        AP_CLE = 0,
        AP_ALE,
        AP_CE_B,
        AP_RE_B,
        AP_WE_B,
        AP_WP_B,
        AP_BSY_B,
        AP_U1_TXD,
        AP_U1_RXD,
        AP_U1_RTS_B,
        AP_U1_CTS_B,
        AP_AD1_TXD,
        AP_AD1_RXD,
        AP_AD1_TXC,
        AP_AD1_TXFS,
        AP_AD2_TXD,
        AP_AD2_RXD,
        AP_AD2_TXC,
        AP_AD2_TXFS,
        AP_OWDAT,
        AP_IPU_LD17,
        AP_IPU_D3_VSYNC,
        AP_IPU_D3_HSYNC,
        AP_IPU_D3_CLK,
        AP_IPU_D3_DRDY,
        AP_IPU_D3_CONTR,
        AP_IPU_D0_CS,
        AP_IPU_LD16,
        AP_IPU_D2_CS,
        AP_IPU_PAR_RS,
        AP_IPU_D3_PS,
        AP_IPU_D3_CLS,
        AP_IPU_RD,
        AP_IPU_WR,
        AP_IPU_LD0,
        AP_IPU_LD1,
        AP_IPU_LD2,
        AP_IPU_LD3,
        AP_IPU_LD4,
        AP_IPU_LD5,
        AP_IPU_LD6,
        AP_IPU_LD7,
        AP_IPU_LD8,
        AP_IPU_LD9,
        AP_IPU_LD10,
        AP_IPU_LD11,
        AP_IPU_LD12,
        AP_IPU_LD13,
        AP_IPU_LD14,
        AP_IPU_LD15,
        AP_KPROW4,
        AP_KPROW5,
        AP_GPIO_AP_B17,
        AP_GPIO_AP_B18,
        AP_KPCOL3,
        AP_KPCOL4,
        AP_KPCOL5,
        AP_GPIO_AP_B22,
        AP_GPIO_AP_B23,
        AP_CSI_D0,
        AP_CSI_D1,
        AP_CSI_D2,
        AP_CSI_D3,
        AP_CSI_D4,
        AP_CSI_D5,
        AP_CSI_D6,
        AP_CSI_D7,
        AP_CSI_D8,
        AP_CSI_D9,
        AP_CSI_MCLK,
        AP_CSI_VSYNC,
        AP_CSI_HSYNC,
        AP_CSI_PIXCLK,
        AP_I2CLK,
        AP_I2DAT,
        AP_GPIO_AP_C8,
        AP_GPIO_AP_C9,
        AP_GPIO_AP_C10,
        AP_GPIO_AP_C11,
        AP_GPIO_AP_C12,
        AP_GPIO_AP_C13,
        AP_GPIO_AP_C14,
        AP_GPIO_AP_C15,
        AP_GPIO_AP_C16,
        AP_GPIO_AP_C17,
        AP_ED_INT0,
        AP_ED_INT1,
        AP_ED_INT2,
        AP_ED_INT3,
        AP_ED_INT4,
        AP_ED_INT5,
        AP_ED_INT6,
        AP_ED_INT7,
        AP_U2_DSR_B,
        AP_U2_RI_B,
        AP_U2_CTS_B,
        AP_U2_DTR_B,
        AP_KPROW0,
        AP_KPROW1,
        AP_KPROW2,
        AP_KPROW3,
        AP_KPCOL0,
        AP_KPCOL1,
        AP_KPCOL2,

        /* IOMUX shared between AP and BP */
        SP_U3_TXD,
        SP_U3_RXD,
        SP_U3_RTS_B,
        SP_U3_CTS_B,
        SP_USB_TXOE_B,
        SP_USB_DAT_VP,
        SP_USB_SE0_VM,
        SP_USB_RXD,
        SP_UH2_TXOE_B,
        SP_UH2_SPEED,
        SP_UH2_SUSPEND,
        SP_UH2_TXDP,
        SP_UH2_RXDP,
        SP_UH2_RXDM,
        SP_UH2_OVR,
        SP_UH2_PWR,
        SP_SD1_DAT0,
        SP_SD1_DAT1,
        SP_SD1_DAT2,
        SP_SD1_DAT3,
        SP_SD1_CMD,
        SP_SD1_CLK,
        SP_SD2_DAT0,
        SP_SD2_DAT1,
        SP_SD2_DAT2,
        SP_SD2_DAT3,
        SP_GPIO_Shared26,
        SP_SPI1_CLK,
        SP_SPI1_MOSI,
        SP_SPI1_MISO,
        SP_SPI1_SS0,
        SP_SPI1_SS1,
        SP_SD2_CMD,
        SP_SD2_CLK,
        SP_SIM1_RST_B,
        SP_SIM1_SVEN,
        SP_SIM1_CLK,
        SP_SIM1_TRXD,
        SP_SIM1_PD,
        SP_UH2_TXDM,
        SP_UH2_RXD,

        AP_MAX_PIN = AP_KPCOL2,
};

/*! @} End IOMUX Bit field definitions */

/*!
 * IOMUX PAD Groups
 */
enum iopad_group {
        IOPAD_GROUP1,
        IOPAD_GROUP2,
        IOPAD_GROUP3,
        IOPAD_GROUP4,
        IOPAD_GROUP5,
        IOPAD_GROUP6,
        IOPAD_GROUP7,
        IOPAD_GROUP8,
        IOPAD_GROUP9,
        IOPAD_GROUP10,
        IOPAD_GROUP11,
        IOPAD_GROUP12,
        IOPAD_GROUP13,
        IOPAD_GROUP14,
        IOPAD_GROUP15,
        IOPAD_GROUP16,
        IOPAD_GROUP17,
        IOPAD_GROUP18,
        IOPAD_GROUP19,
        IOPAD_GROUP20,
        IOPAD_GROUP21,
        IOPAD_GROUP22,
        IOPAD_GROUP23,
        IOPAD_GROUP24,
        IOPAD_GROUP25,
        IOPAD_GROUP26,
        IOPAD_GROUP27,
        IOPAD_GROUP28,
};

/*!
 * various IOMUX output functions
 */
enum iomux_output_config {
        OUTPUTCONFIG_DEFAULT = 0,
        OUTPUTCONFIG_FUNC1,
        OUTPUTCONFIG_FUNC2,
        OUTPUTCONFIG_FUNC3,
        OUTPUTCONFIG_FUNC4,
        OUTPUTCONFIG_FUNC5,
        OUTPUTCONFIG_FUNC6,
        OUTPUTCONFIG_FUNC7,
};

/*!
 * various IOMUX input functions
 */
enum iomux_input_config {
        INPUTCONFIG_NONE    = 0,
        INPUTCONFIG_DEFAULT = 1 << 0,
        INPUTCONFIG_FUNC1   = 1 << 1,
        INPUTCONFIG_FUNC2   = 1 << 2,
        INPUTCONFIG_FUNC3   = 1 << 3,
};

/*!
 * various IOMUX pad functions
 */
enum iomux_pad_config {
        SRE_FAST        = 0x1 << 0,
        SRE_SLOW        = 0x0 << 0,
        DSE_NORMAL      = 0x0 << 1,
        DSE_HIGH        = 0x1 << 1,
        DSE_MAX         = 0x2 << 1,
        DSE_MIN         = 0x3 << 1,
        ODE_CMOS        = 0x0 << 3,
        ODE_OPEN_DRAIN  = 0x1 << 3,
        PKE_ENABLE      = 0x1 << 7,
        PKE_DISABLE     = 0x0 << 7,
        PUE_ENABLE      = 0x1 << 4,
        PUE_KEEPER      = 0x0 << 4,
        HYS_SCHMITZ     = 0x1 << 8,
        HYS_CMOS        = 0x0 << 8,
        DDR_MODE_DDR    = 0x1 << 9,
        DDR_MODE_CMOS   = 0x0 << 9,
        DDR_INPUT_DDR   = 0x1 << 10,
        DDR_INPUT_CMOS  = 0x0 << 10,
};

/*!
 * This function is used to configure a pin through the IOMUX module.
 *
 * @param  pin          pin number as defined in \b #iomux_pins
 * @param  out          output function as defined in \b #iomux_output_config
 * @param  in           input function as defined in \b #iomux_input_config
 */
void iomux_config_mux(enum iomux_pins pin, 
                      enum iomux_output_config out, 
                      enum iomux_input_config in);

/*!
 * This function returns the pad value for a IOMUX group.
 *
 * @param  grp          a group number as defined in \b enum \b #iopad_group
 *
 * @return A group pad value
 */
__u16 iomux_get_pad(enum iopad_group grp);

/*!
 * This function configures the pad value for a IOMUX group.
 *
 * @param  grp          a group number as defined in \b enum \b #iopad_group
 * @param  config       the ORed value of elements defined in \b enum \b #iomux_pad_config
 */
void iomux_set_pad(enum iopad_group grp, __u32 config);

/* gpio related defines */

/*!
 * There are two queues for registered GPIO ISRs. One is for high priority and
 * the other is for low priority. The ISRs in the high priority queue will be
 * called first before the low priority queue if more than one GPIO interrupt
 * occurs at the same time.
 */
enum gpio_prio {
        GPIO_HIGH_PRIO  = 0,     /*!< high priority queue */
        GPIO_LOW_PRIO            /*!< low priority queue */
};

/*!
 * This enumeration data type defines various different ways for interrupting 
 * the ARM core from GPIO signals. The way to interrupt the core is dictated 
 * by the external hardware.
 */
enum gpio_int_cfg {
        GPIO_INT_LOW_LEV        = 0x0,    /*!< low level sensitive */
        GPIO_INT_HIGH_LEV       = 0x1,    /*!< high level sensitive */
        GPIO_INT_RISE_EDGE      = 0x2,    /*!< rising edge sensitive */
        GPIO_INT_FALL_EDGE      = 0x3,    /*!< falling edge sensitive */
        GPIO_INT_NONE           = 0x4     /*!< No interrupt */
};

/*!
 * This enumeration data type defines various different ways for interrupting 
 * the ARM core from EDIO signals. The way to interrupt the core is dictated 
 * by the external hardware.
 */
enum edio_int_cfg {
        EDIO_INT_LOW_LEV        = 0x0,    /*!< low level sensitive */
        EDIO_INT_RISE_EDGE      = 0x1,    /*!< rise edge sensitive */
        EDIO_INT_FALL_EDGE      = 0x2,    /*!< falling edge sensitive */
        EDIO_INT_BOTH_EDGE      = 0x3,    /*!< both edge sensitive */
        EDIO_INT_NONE           = 0x4     /*!< No interrupt */
};

/*!
 * This enumeration data type defines various signals for interrupting the
 * ARM core from EDIO signals. 
 */
enum edio_sig_no {
        ED_INT0			= 0,		    
        ED_INT1			= 1,
        ED_INT2			= 2,
        ED_INT3			= 3,
        ED_INT4			= 4,
        ED_INT5			= 5,
        ED_INT6			= 6,
        ED_INT7			= 7,
};

typedef irqreturn_t (*gpio_irq_handler)(int, void *, struct pt_regs *);

/*!
 * This function configures the GPIO signal to be either input or output. For
 * input signals used for generating interrupts for the ARM core, how the
 * interrupts being triggered is also passed in via \a icr. For output signals,
 * the \a icr value doesn't matter.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based)
 * @param  out          #true for output, #false for input
 * @param  icr          value defined in \b #gpio_int_cfg
 */
void gpio_config(__u32 port, __u32 sig_no, bool out, enum gpio_int_cfg icr);

/*!
 * This function sets a GPIO signal value.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based)
 * @param  data         value to be set (only 0 or 1 is valid)
 */
void gpio_set_data(__u32 port, __u32 sig_no, __u32 data);

/*!
 * This function returns the value of the GPIO signal.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based)
 *
 * @return Value of the GPIO signal
 */
__u32 gpio_get_data(__u32 port, __u32 sig_no);

/*!
 * This function returns the value of the GPIO signal pad value.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based)
 *
 * @return Pad value of the GPIO signal
 */
__u32 gpio_get_pad(__u32 port, __u32 sig_no);

/*!
 * This function enables/disables a GPIO signal's interrupt.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based)
 * @param  en           \b true for enabling the interrupt; \b false for disabling the interrupt
 */
void gpio_config_int_en(__u32 port, __u32 sig_no, bool en);

/*!
 * This function checks to see if the GPIO interrupt is enabled for a particular signal.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based)
 *
 * @return \b #true if the interrupt is enabled for the GPIO signal; \b #false otherwise.
 */
bool gpio_int_enabled(__u32 port, __u32 sig_no);

/*!
 * This function clears the GPIO interrupt for a signal on a port.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based) to clear
 */
void gpio_clear_int(__u32 port, __u32 sig_no);

/*!
 * This function is responsible for registering a GPIO signal's ISR.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based)
 * @param  prio         priority as defined in \b enum \b #gpio_prio
 * @param  handler      GPIO ISR function pointer for the GPIO signal
 * @param  irq_flags    irq flags (not used)
 * @param  devname      device name associated with the interrupt
 * @param  dev_id       some unique information for the ISR
 *
 * @return 0 if successful; non-zero otherwise.
 */
int gpio_request_irq(__u32 port, __u32 sig_no, enum gpio_prio prio,
                     gpio_irq_handler handler, __u32 irq_flags, 
                     const char * devname, void *dev_id);

/*!
 * This function un-registers an ISR with the GPIO interrupt module.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based)
 * @param  prio         priority as defined in \b enum \b #gpio_prio
 */
void gpio_free_irq(__u32 port, __u32 sig_no, enum gpio_prio prio);

/*!
 * This function configures the EDIO signal to be either input or output. For
 * input signals used for generating interrupts for the ARM core, how the
 * interrupts being triggered is also passed in via \a icr. For output signals
 * the \a icr value doesn't matter.
 *
 * @param  sig_no       EDIO signal (0 based) as defined in \b #edio_sig_no
 * @param  out          #true for output, #false for input
 * @param  icr          value defined in \b #edio_int_cfg
 */
void edio_config(enum edio_sig_no sig_no, bool out, enum edio_int_cfg icr);

/*!
 * This function sets a EDIO signal value.
 *
 * @param  sig_no       EDIO signal (0 based) as defined in \b #edio_sig_no
 * @param  data         value to be set (only 0 or 1 is valid)
 */
void edio_set_data(enum edio_sig_no sig_no, __u32 data);

/*!
 * This function returns the value of the EDIO signal.
 *
 * @param  sig_no       EDIO signal (0 based) as defined in \b #edio_sig_no
 *
 * @return Value of the EDIO signal (either 0 or 1)
 */
__u32 edio_get_data(enum edio_sig_no sig_no);

/*!
 * This function returns the EDIO edge-trigged interrupt for a signal.
 *
 * @param  sig_no       EDIO signal (0 based) as defined in \b #edio_sig_no 
 */
__u32 edio_get_int(enum edio_sig_no sig_no);

/*!
 * This function clears the EDIO edge-trigged interrupt for a signal.
 *
 * @param  sig_no       EDIO signal (0 based) as defined in \b #edio_sig_no
 */
void edio_clear_int(enum edio_sig_no sig_no);

#endif /* __GPIO_H__ */
