/*
 * Copyright 2005 Freescale Semiconductor, Inc. All Rights Reserved.
 */

/*
 * The code contained herein is licensed under the GNU General Public
 * License. You may obtain a copy of the GNU General Public License
 * Version 2 or later at the following locations:
 *
 * http://www.opensource.org/licenses/gpl-license.html
 * http://www.gnu.org/copyleft/gpl.html
 */

#ifndef __CLOCK_H

#define __CLOCK_H

/*!
 * @defgroup CLOCKS Clocking Setup and Retrieving
 * @ingroup MSL
 */

/*!
 * @file clock.h
 *
 * @brief API for setting up and retrieving clocks.
 *
 * This file contains API for setting up and retrieving clocks.
 *
 * @ingroup CLOCKS
 */

#include <asm/arch/platform.h>

/* CRM_AP Register Offsets */
#define     CRM_AP_ASCSR            0x00
#define     CRM_AP_ACDR             0x04
#define     CRM_AP_ACDER1           0x08
#define     CRM_AP_ACDER2           0x0C
#define     CRM_AP_ACGCR            0x10
#define     CRM_AP_ACCGCR           0x14
#define     CRM_AP_AMLPMRA          0x18
#define     CRM_AP_AMLPMRB          0x1C
#define     CRM_AP_AMLPMRC          0x20
#define     CRM_AP_AMLPMRD          0x24
#define     CRM_AP_AMLPMRE1         0x28
#define     CRM_AP_AMLPMRE2         0x2C
#define     CRM_AP_AMLPMRF          0x30
#define     CRM_AP_AMLPMRG          0x34
#define     CRM_AP_APGCR            0x38
#define     CRM_AP_ACSR             0x3C
#define     CRM_AP_ADCR             0x40
#define     CRM_AP_ACR              0x44
#define     CRM_AP_AMCR             0x48
#define     CRM_AP_APCR             0x4C
#define     CRM_AP_AMORA            0x50
#define     CRM_AP_AMORB            0x54
#define     CRM_AP_AGPR             0x58
#define     CRM_AP_APRA             0x5C
#define     CRM_AP_APRB             0x60
#define     CRM_AP_APOR             0x64

/* CRM_COM Register Offsets */
#define     CRM_COM_CBMR            0x00
#define     CRM_COM_CRSRBP          0x04
#define     CRM_COM_CCRCR           0x08
#define     CRM_COM_CSCR            0x0C
#define     CRM_COM_CCCR            0x10
#define     CRM_COM_CRSRAP          0x14

/* DPLL Register Offsets */
#define     DPLL_DP_CTL             0x00
#define     DPLL_DP_CONFIG          0x04
#define     DPLL_DP_OP              0x08
#define     DPLL_DP_MFD             0x0C
#define     DPLL_DP_MFN             0x10
#define     DPLL_DP_MFNMINUS        0x14
#define     DPLL_DP_MFNPLUS         0x18
#define     DPLL_DP_HFS_OP          0x1C
#define     DPLL_DP_HFS_MFD         0x20
#define     DPLL_DP_HFS_MFN         0x24
#define     DPLL_DP_TOGC            0x28
#define     DPLL_DP_DESTAT          0x2C

/* 
 * CRM AP Register Bit Field Positions (left shift)
 */
#define ASCSR_AP_ISEL_BFS           0
#define ASCSR_APSEL_BFS             3
#define ASCSR_SSI1SEL_BFS           5
#define ASCSR_SSI2SEL_BFS           7
#define ASCSR_FIRISEL_BFS           9
#define ASCSR_CSSEL_BFS             11
#define ASCSR_USBSEL_BFS            13
#define ASCSR_AP_PAT_REF_DIV_BFS    15
#define ASCSR_CRS_BFS               16

#define ACDR_IP_DIV_BFS             0
#define ACDR_AHB_DIV_BFS            4
#define ACDR_ARM_DIV_BFS            8

#define ACDER1_SSI1_DIV_BFS         0
#define ACDER1_SSI1_EN_BFS          6
#define ACDER1_SSI2_DIV_BFS         8
#define ACDER1_SSI2_EN_BFS          14
#define ACDER1_FIRI_DIV_BFS         16
#define ACDER1_FIRI_EN_BFS          22
#define ACDER1_CS_DIV_BFS           24
#define ACDER1_CS_EN_BFS            30

#define ACDER2_BAUD_DIV_BFS         0
#define ACDER2_BAUD_ISEL_BFS        5
#define ACDER2_USB_DIV_BFS          8
#define ACDER2_USB_EN_BFS           12
#define ACDER2_NFC_DIV_BFS          16
#define ACDER2_NFC_EN_BFS           20

#define ACGCR_ACG0_BFS              0
#define ACGCR_ACG1_BFS              3
#define ACGCR_ACG2_BFS              6
#define ACGCR_ACG3_BFS              9
#define ACGCR_ACG4_BFS              12

#define ACCGCR_ACCG0_BFS            0
#define ACCGCR_ACCG1_BFS            3
#define ACCGCR_ACCG2_BFS            6

// Field definitions apply to all AMLPRMx
#define AMLPMR_MLPM0_BFS            0
#define AMLPMR_MLPM1_BFS            3
#define AMLPMR_MLPM2_BFS            6
#define AMLPMR_MLPM4_BFS            12
#define AMLPMR_MLPM5_BFS            16
#define AMLPMR_MLPM6_BFS            19
#define AMLPMR_MLPM7_BFS            22

#define APGCR_PG0_BFS               0
#define APGCR_PG1_BFS               2

#define ACSR_ACS_BFS                0
#define ACSR_WPS_BFS                1
#define ACSR_PDS_BFS                2
#define ACSR_SMD_BFS                3
#define ACSR_DI_BFS                 7
#define ACSR_ADS_BFS                8

#define ADCR_DIV_BYP_BFS            1
#define ADCR_VSTAT_BFS              3
#define ADCR_TSTAT_BFS              4
#define ADCR_DFS_DIV_EN_BFS         5
#define ADCR_CLK_ON_BFS             6
#define ADCR_ALT_PLL_BFS            7
#define ADCR_LFDF_BFS               8
#define ADCR_AP_DELAY_BFS           16

#define ACR_CKOS_BFS                4
#define ACR_CKOD_BFS                7
#define ACR_CKOHDIV_BFS             8
#define ACR_CKOHS_BFS               12
#define ACR_CKOHD_BFS               15

#define AMCR_WB_EN_BFS              0
#define AMCR_FA_BFS                 3
#define AMCR_SPA_BFS                4
#define AMCR_L2C_SAFE_EN_BFS        8
#define AMCR_L2C_PG_BFS             9
#define AMCR_SW_AP_BFS              14
#define AMCR_CSB_BFS                15
#define AMCR_SDMA_ACK_BYP_BFS       16
#define AMCR_IPU_ACK_BYP_BFS        17
#define AMCR_MAX_ACK_BYP_BFS        18
#define AMCR_USBOTG_ACK_BYP_BFS     20
#define AMCR_IPU_REQ_DIS_BFS        22

#define APCR_PC_BFS                 0

#define AMORA_MOA_BFS               0

#define AMORB_MOB_BFS               0

#define AGPR_GPB_BFS                0 

#define APRA_UART1_EN_BFS           0
#define APRA_UART2_EN_BFS           8
#define APRA_UART3_EN_BFS           16
#define APRA_UART3_DIV_BFS          17
#define APRA_SIM_EN_BFS             18

#define APRB_SDHC1_EN_BFS           0
#define APRB_SDHC1_DIV_BFS          1
#define APRB_SDHC1_ISEL_BFS         5
#define APRB_SDHC2_EN_BFS           8
#define APRB_SDHC2_DIV_BFS          9
#define APRB_SDHC2_ISEL_BFS         13
#define APRB_ECT_EN_BFS             16
    
#define APOR_POB_BFS                0

/* 
 * CRM AP Register Bit Field Widths
 */
#define ASCSR_AP_ISEL_BFW           2
#define ASCSR_APSEL_BFW             2
#define ASCSR_SSI1SEL_BFW           2
#define ASCSR_SSI2SEL_BFW           2
#define ASCSR_FIRISEL_BFW           2
#define ASCSR_CSSEL_BFW             2
#define ASCSR_USBSEL_BFW            2
#define ASCSR_AP_PAT_REF_DIV_BFW    1
#define ASCSR_CRS_BFW               1

#define ACDR_IP_DIV_BFW             4
#define ACDR_AHB_DIV_BFW            4
#define ACDR_ARM_DIV_BFW            4

#define ACDER1_SSI1_DIV_BFW         6
#define ACDER1_SSI1_EN_BFW          1
#define ACDER1_SSI2_DIV_BFW         6
#define ACDER1_SSI2_EN_BFW          1
#define ACDER1_FIRI_DIV_BFW         5
#define ACDER1_FIRI_EN_BFW          1
#define ACDER1_CS_DIV_BFW           6
#define ACDER1_CS_EN_BFW            1

#define ACDER2_BAUD_DIV_BFW         4
#define ACDER2_BAUD_ISEL_BFW        2
#define ACDER2_USB_DIV_BFW          4
#define ACDER2_USB_EN_BFW           1
#define ACDER2_NFC_DIV_BFW          4
#define ACDER2_NFC_EN_BFW           1

#define ACGCR_ACG0_BFW              3
#define ACGCR_ACG1_BFW              3
#define ACGCR_ACG2_BFW              3
#define ACGCR_ACG3_BFW              3
#define ACGCR_ACG4_BFW              3

#define ACCGCR_ACCG0_BFW            3
#define ACCGCR_ACCG1_BFW            3
#define ACCGCR_ACCG2_BFW            3

// Field definitions apply to all AMLPRMx
#define AMLPMR_MLPM0_BFW            3
#define AMLPMR_MLPM1_BFW            3
#define AMLPMR_MLPM2_BFW            3
#define AMLPMR_MLPM3_BFW            3
#define AMLPMR_MLPM4_BFW            3
#define AMLPMR_MLPM5_BFW            3
#define AMLPMR_MLPM6_BFW            3
#define AMLPMR_MLPM7_BFW            3

#define APGCR_PG0_BFW               2
#define APGCR_PG1_BFW               2

#define ACSR_ACS_BFW                1
#define ACSR_WPS_BFW                1
#define ACSR_PDS_BFW                1
#define ACSR_SMD_BFW                1
#define ACSR_DI_BFW                 1
#define ACSR_ADS_BFW                1

#define ADCR_DIV_BYP_BFW            1
#define ADCR_VSTAT_BFW              1
#define ADCR_TSTAT_BFW              1
#define ADCR_DFS_DIV_EN_BFW         1
#define ADCR_CLK_ON_BFW             1
#define ADCR_ALT_PLL_BFW            1
#define ADCR_LFDF_BFW               2
#define ADCR_AP_DELAY_BFW           16

#define ACR_CKOS_BFW                3
#define ACR_CKOD_BFW                1
#define ACR_CKODIV_BFW              4
#define ACR_CKOHS_BFW               3
#define ACR_CKOHD_BFW               1

#define AMCR_WB_EN_BFW              1
#define AMCR_FA_BFW                 1
#define AMCR_SPA_BFW                2
#define AMCR_L2C_SAFE_EN_BFW        1
#define AMCR_L2C_PG_BFW             1
#define AMCR_SW_AP_BFW              1
#define AMCR_CSB_BFW                1
#define AMCR_SDMA_ACK_BYP_BFW       1
#define AMCR_IPU_ACK_BYP_BFW        1
#define AMCR_MAX_ACK_BYP_BFW        1
#define AMCR_USBOTG_ACK_BYP_BFW     1    
#define AMCR_IPU_REQ_DIS_BFW        1

#define APCR_PC_BFW                 32

#define AMORA_MOA_BFW               32

#define AMORB_MOB_BFW               28

#define AGPR_GPB_BFW                32 

#define APRA_UART1_EN_BFW           1
#define APRA_UART2_EN_BFW           1
#define APRA_UART3_EN_BFW           1
#define APRA_UART3_DIV_BFW          4
#define APRA_SIM_EN_BFW             1

#define APRB_SDHC1_EN_BFW           1
#define APRB_SDHC1_DIV_BFW          4
#define APRB_SDHC1_ISEL_BFW         3
#define APRB_SDHC2_EN_BFW           1
#define APRB_SDHC2_DIV_BFW          4
#define APRB_SDHC2_ISEL_BFW         3
#define APRB_ECT_EN_BFW             1
    
#define APOR_POB_BFW                12


/* 
 * CRM AP Register Bit Field Values
 */
#define ASCSR_AP_ISEL_CKIH          0
#define ASCSR_AP_ISEL_DIGRF         1
        
#define ASCSR_APSEL_ADPLL           0
#define ASCSR_APSEL_BDPLL           1
#define ASCSR_APSEL_UDPLL           2
#define ASCSR_APSEL_MRCG0           3
    
#define ASCSR_SSI1SEL_ADPLL         0
#define ASCSR_SSI1SEL_BDPLL         1
#define ASCSR_SSI1SEL_UDPLL         2
#define ASCSR_SSI1SEL_MRCG2         3
    
#define ASCSR_SSI2SEL_ADPLL         0
#define ASCSR_SSI2SEL_BDPLL         1
#define ASCSR_SSI2SEL_UDPLL         2
#define ASCSR_SSI2SEL_MRCG2         3

#define ASCSR_FIRISEL_ADPLL         0
#define ASCSR_FIRISEL_BDPLL         1
#define ASCSR_FIRISEL_UDPLL         2
#define ASCSR_FIRISEL_MRCG2         3

#define ASCSR_CSSEL_ADPLL           0
#define ASCSR_CSSEL_BDPLL           1
#define ASCSR_CSSEL_UDPLL           2
#define ASCSR_CSSEL_MRCG0           3

#define ASCSR_USBSEL_ADPLL          0
#define ASCSR_USBSEL_BDPLL          1
#define ASCSR_USBSEL_UDPLL          2
#define ASCSR_USBSEL_MRCG2          3

#define ASCSR_AP_PAT_REF_DIV_1      0
#define ASCSR_AP_PAT_REF_DIV_2      1

#define ASCSR_CRS_UNCORRECTED       0
#define ASCSR_CRS_CORRECTED         1

#define ACDR_ARM_DIV_1              0x8
#define ACDR_ARM_DIV_2              0x0
#define ACDR_ARM_DIV_3              0x1
#define ACDR_ARM_DIV_4              0x2
#define ACDR_ARM_DIV_5              0x3
#define ACDR_ARM_DIV_6              0x4
#define ACDR_ARM_DIV_8              0x5
#define ACDR_ARM_DIV_10             0x6
#define ACDR_ARM_DIV_12             0x7

#define ACDER2_BAUD_ISEL_CKIH       0x0
#define ACDER2_BAUD_ISEL_CKIH_X2    0x1
#define ACDER2_BAUD_ISEL_DIGRF      0x2
#define ACDER2_BAUD_ISEL_DIGRF_X2   0x3
    
#define ACDER2_BAUD_DIV_1           0x8
#define ACDER2_BAUD_DIV_2           0x0
#define ACDER2_BAUD_DIV_3           0x1
#define ACDER2_BAUD_DIV_4           0x2
#define ACDER2_BAUD_DIV_5           0x3
#define ACDER2_BAUD_DIV_6           0x4
#define ACDER2_BAUD_DIV_8           0x5
#define ACDER2_BAUD_DIV_10          0x6
#define ACDER2_BAUD_DIV_12          0x7

#define ACGCR_ACG_ALWAYS_ON         0x0
#define ACGCR_ACG_ON_RUN            0x1
#define ACGCR_ACG_ON_RUN_8          0x2
#define ACGCR_ACG_ON_RUN_WAIT       0x3
#define ACGCR_ACG_ALWAYS_OFF        0x4

#define APGCR_PGC_ALWAYS_OFF        0x0
#define APGCR_PGC_ON_RUN            0x1
#define APGCR_PGC_ON_RUN_WAIT       0x2
#define APGCR_PGC_ALWAYS_ON         0x3

#define AMLPMR_MLPM_ALWAYS_OFF      0x0
#define AMLPMR_MLPM_ON_RUN          0x1
#define AMLPMR_MLPM_ON_RUN_8        0x2
#define AMLPMR_MLPM_ON_RUN_WAIT     0x3
#define AMLPMR_MLPM_ALWAYS_ON       0x4




#define ACSR_ACS_REF_CORE_CLK       0x0
#define ACSR_ACS_PLL_CLK            0x1

#define ACSR_WPS_PLL_CLK            0x0
#define ACSR_WPS_PAT_REF            0x1
    
#define ACSR_PDS_PLL_ON_IN_STOP     0x0
#define ACSR_PDS_PLL_OFF_IN_STOP    0x1

#define ACSR_SMD_SYNC_MUX_ENABLE    0x0
#define ACSR_SMD_SYNC_MUX_DISABLE   0x1

#define ACSR_DI_IGNORE_DSM_INT      0x0
#define ACSR_DI_USE_DSM_INT         0x1

#define ACSR_ADS_NO_DOUBLER         0x0
#define ACSR_ADS_DOUBLER            0x1

#define ADCR_DIV_BYP_USED           0x0
#define ADCR_DIV_BYP_NOT_USED       0x1

#define ADCR_VSTAT_HIGH_VOLTAGE     0x0
#define ADCR_VSTAT_LOW_VOLTAGE      0x1

#define ADCR_TSTAT_TIMED_OUT        0x0
#define ADCR_TSTAT_RUNNING          0x1

#define ADCR_DFS_DIV_EN_NON_INT_DIV 0x0
#define ADCR_DFS_DIV_EN_INT_DIV     0x1

#define ADCR_CLK_ON_DVFS_NO_CLOCK   0x0
#define ADCR_CLK_ON_DVFS_PAT_REF    0x1

#define ADCR_ALT_PLL_NO_ALT         0x0
#define ADCR_ALT_PLL_USE_ALT        0x1

#define ADCR_LFDF_NO_DIV            0x0
#define ADCR_LFDF_DIV2              0x1
#define ADCR_LFDF_DIV4              0x2
#define ADCR_LFDF_DIV8              0x8

#define ACR_CKOS_CKIL               0x0
#define ACR_CKOS_AP_PAT_REF_CLK     0x1
#define ACR_CKOS_AP_REF_X2_CLK      0x2
#define ACR_CKOS_SSI1_CLK           0x3
#define ACR_CKOS_SSI2_CLK           0x4
#define ACR_CKOS_CS_CLK             0x5
#define ACR_CKOS_FIRI_CLK           0x6
#define ACR_CKOS_AP_UC_PAT_REF_CLK  0x7

#define ACR_CKOD_CKO_ENABLE         0x0
#define ACR_CKOD_CKO_DISABLE        0x1

#define ACR_CKOHDIV_1               0x8
#define ACR_CKOHDIV_2               0x0
#define ACR_CKOHDIV_3               0x1
#define ACR_CKOHDIV_4               0x2
#define ACR_CKOHDIV_5               0x3
#define ACR_CKOHDIV_6               0x4
#define ACR_CKOHDIV_8               0x5
#define ACR_CKOHDIV_10              0x6
#define ACR_CKOHDIV_12              0x7

#define ACR_CKOHS_AP_UC_PAT_REF_CLK 0x0
#define ACR_CKOHS_AP_CLK            0x1
#define ACR_CKOHS_AP_AHB_CLK        0x2
#define ACR_CKOHS_AP_PCLK           0x3
#define ACR_CKOHS_USB_CLK           0x4
#define ACR_CKOHS_AP_PERCLK         0x5
#define ACR_CKOHS_AP_CKIL_CLK       0x6
#define ACR_CKOHS_AP_PAT_REF_CLK    0x7

#define ACR_CKOHD_CKOH_ENABLED      0x0
#define ACR_CKOHD_CKOH_DISABLED     0x1

#define AMCR_WB_EN_WELL_BIAS_OFF    0x0
#define AMCR_WB_EN_WELL_BIAS_ON     0x1

#define AMCR_FA_SLOW                0x0
#define AMCR_FA_NOMINAL             0x1

#define AMCR_L2C_SAFE_EN_UNSAFE     0
#define AMCR_L2C_SAFE_EN_SAFE       1

#define AMCR_L2C_PG_NO_PWR_GATE     0x0
#define AMCR_L2C_PG_PWR_GATE        0x1

#define AMCR_SW_AP_WARM_RESET       0x0
#define AMCR_SW_AP_NO_RESET         0x1

#define AMCR_CSB_CKIL_SYNC          0x0
#define AMCR_CSB_NO_CKIL_SYNC       0x1

#define AMCR_SDMA_ACK_BYP_WAIT      0
#define AMCR_SDMA_ACK_BYP_NO_WAIT   1

#define AMCR_IPU_ACK_BYP_WAIT       0
#define AMCR_IPU_ACK_BYP_NO_WAIT    1

#define AMCR_MAX_ACK_BYP_WAIT       0
#define AMCR_MAX_ACK_BYP_NO_WAIT    1

#define AMCR_USBOTG_ACK_BYP_WAIT    0
#define AMCR_USBOTG_ACK_BYP_NO_WAIT 1

#define AMCR_IPU_REQ_DIS_REQUEST    0
#define AMCR_IPU_REQ_DIS_NO_REQUEST 1

#define APRB_SDHC_ISEL_CKIH         0
#define APRB_SDHC_ISEL_CKIH_X2      1
#define APRB_SDHC_ISEL_DIGRF        2
#define APRB_SDHC_ISEL_DIGRF_X2     3
#define APRB_SDHC_ISEL_USB_CLK      4
#define APRB_SDHC_ISEL_MRCG2_CLK    5


/* 
 * DPLL Register Bit Field Positions (left shift)
 */
#define DP_CTL_LRF_BFS              0
#define DP_CTL_BRMO_BFS             1
#define DP_CTL_PLM_BFS              2
#define DP_CTL_RCP_BFS              3
#define DP_CTL_RST_BFS              4
#define DP_CTL_UPEN_BFS             5
#define DP_CTL_PRE_BFS              6
#define DP_CTL_HFSM_BFS             7
#define DP_CTL_REF_CLK_SEL_BFS      8
#define DP_CTL_REF_CLK_DIV_BFS      10
#define DP_CTL_ADE_BFS              11
#define DP_CTL_DPDCK0_2_EN_BFS      12

#define DP_CONFIG_LDREQ_BFS         0
#define DP_CONFIG_AREN_BFS          1
#define DP_CONFIG_BIST_CE_BFS       3

#define DP_OP_PDF_BFS               0
#define DP_OP_MFI_BFS               4

#define DP_MFD_MFD_BFS              0

#define DP_MFN_MFN_BFS              0

#define DP_TOGC_TOF_MFN_CNT_BFS     0
#define DP_TOGC_TOG_EN_BFS          16
#define DP_TOGC_TOG_DIS_BFS         17

#define DP_DESTAT_TOG_MFN_BFS       0
#define DP_DESTAT_TOG_SEL_BFS       31

//------------------------------------------------------------------------------
// REGISTER BIT FIELD WIDTHS
//------------------------------------------------------------------------------
#define DP_CTL_LRF_BFW              1
#define DP_CTL_BRMO_BFW             1
#define DP_CTL_PLM_BFW              1
#define DP_CTL_RCP_BFW              1
#define DP_CTL_RST_BFW              1
#define DP_CTL_UPEN_BFW             1
#define DP_CTL_PRE_BFW              1
#define DP_CTL_HFSM_BFW             1
#define DP_CTL_REF_CLK_SEL_BFW      2
#define DP_CTL_REF_CLK_DIV_BFW      1
#define DP_CTL_ADE_BFW              1
#define DP_CTL_DPDCK0_2_EN_BFW      1

#define DP_CONFIG_LDREQ_BFW         1
#define DP_CONFIG_AREN_BFW          1
#define DP_CONFIG_BIST_CE_BFW       1

#define DP_OP_PDF_BFW               4
#define DP_OP_MFI_BFW               4

#define DP_MFD_MFD_BFW              27

#define DP_MFN_MFN_BFW              27

#define DP_TOGC_TOF_MFN_CNT_BFW     16
#define DP_TOGC_TOG_EN_BFW          1
#define DP_TOGC_TOG_DIS_BFW         1

#define DP_DESTAT_TOG_MFN_BFW       27
#define DP_DESTAT_TOG_SEL_BFW       1

//------------------------------------------------------------------------------
// REGISTER BIT WRITE VALUES
//------------------------------------------------------------------------------

#define DP_CTL_LRF_LOCKED           0x0
#define DP_CTL_LRF_UNLOCKED         0x1

#define DP_CTL_BRMO_FIRST_ORDER     0x0
#define DP_CTL_BRMO_SECOND_ORDER    0x1

#define DP_CTL_PLM_FREQ_ONLY        0x0
#define DP_CTL_PLM_FREQ_PHASE       0x1

#define DP_CTL_RCP_POS_EDGE         0x0
#define DP_CTL_RCP_NEG_EDGE         0x1

#define DP_CTL_RST_NO_RESTART       0x0
#define DP_CTL_RST_RESTART          0x1

#define DP_CTL_UPEN_OFF             0x0
#define DP_CTL_UPEN_ON              0x1

#define DP_CTL_PRE_NO_RESET         0x0
#define DP_CTL_PRE_RESET            0x1

#define DP_CTL_HFSM_NORMAL_MODE     0x0
#define DP_CTL_HFSM_HFS_MODE        0x1

#define DP_CTL_REF_CLK_SEL_CKIH     0x0
#define DP_CTL_REF_CLK_SEL_DIGRF    0x1
#define DP_CTL_REF_CLK_SEL_CKIH_X2  0x2
#define DP_CTL_REF_CLK_SEL_DIGRF_X2 0x3

#define DP_CTL_REF_CLK_DIV_1        0x0
#define DP_CTL_REF_CLK_DIV_2        0x1

#define DP_CONFIG_LDREQ_DONE        0x0
#define DP_CONFIG_LDREQ_UPDATE      0x1

#define DP_CONFIG_AREN_NO_AUTO_RESTART 0x0
#define DP_CONFIG_AREN_AUTO_RESTART    0x1

#define DP_CONFIG_BIST_CE_NORMAL    0x0

#define DP_TOGC_TOG_DIS_ENABLED     0x0
#define DP_TOGC_TOD_DIS_DISABLED    0x1

#define DP_TOGC_TOD_EN_DISABLED     0x0
#define DP_TOGC_TOD_EN_ENABLED      0x1

#define DP_DESTAT_TOG_SEL_INACTIVE  0x0
#define DP_DESTAT_TOG_SEL_ACTIVE    0x1


/*
 * Define reference clock inputs
 */
#define CKIL_CLK_FREQ               32768
#define CKIH_CLK_FREQ               16800000
#define DIGRF_CLK_FREQ              26000000


/*!
 * Enumerations for plls
 */
enum plls {
        MCUPLL = 0,   // MCU PLL
        USBPLL = 2,   // USB PLL
        DSPPLL = 1,   // DSP PLL
        MRCGPLL = 3,  // MRCG PLL
};

/*!
 * Enumerations for MXC clocks
 */
enum mxc_clocks {
        CPU_CLK,
        AHB_CLK,
        IPG_CLK,
        NFC_CLK,
        USB_CLK,
        UART1_BAUD,
        UART2_BAUD,
        UART3_BAUD,
        SSI1_BAUD,
        SSI2_BAUD,
        CSI_BAUD,
        FIRI_BAUD,        
        I2C_CLK,
        CSPI1_CLK,
        CSPI2_CLK,
        GPT_CLK,
        RTC_CLK,
        EPIT1_CLK,
        EPIT2_CLK,
        EDIO_CLK,
        WDOG_CLK,
        PWM_CLK,
        IPU_CLK,
        SIM1_CLK,
        HAC_CLK,
        SDHC2_CLK,
        SDHC1_CLK,
        SDMA_CLK,
        RNG_CLK,
        KPP_CLK,
        MU_CLK,
        SCC_CLK,
        SPBA_CLK, 
        DSM_CLK,
        OWIRE_CLK,
        CKIH_CLK,
        CKIL_CLK,
        DIGRF_CLK,
};

/*!
 * This function is used to modify PLL registers to generate the required
 * frequency.
 *
 * @param  pll      the PLL that you wish to modify
 * @param  mfi      multiplication factor integer part
 * @param  pdf      pre-division factor
 * @param  mfd      multiplication factor denominator
 * @param  mfn      multiplication factor numerator
 */
void mxc_pll_set(enum plls pll, unsigned int mfi, unsigned int pdf,
                 unsigned int mfd, unsigned int mfn);

/*!
 * This function is used to get PLL registers values used generate the clock
 * frequency.
 *
 * @param  pll      the PLL that you wish to access
 * @param  mfi      pointer that holds multiplication factor integer part
 * @param  pdf      pointer that holds pre-division factor
 * @param  mfd      pointer that holds multiplication factor denominator
 * @param  mfn      pointer that holds multiplication factor numerator
 */
void mxc_pll_get(enum plls pll, unsigned int *mfi, unsigned int *pdf,
                 unsigned int *mfd, unsigned int *mfn);

/*!
 * This function returns the PLL output value in Hz based on pll.
 *
 * @param       pll     PLL as defined in enum plls
 *
 * @return      PLL value in Hz.
 */
unsigned long mxc_pll_clock(enum plls pll);

/*!
 * This function returns the clock value in Hz for various MXC modules.
 *
 * @param       clk     as defined in enum mxc_clocks
 *
 * @return      clock value in Hz
 */
unsigned long mxc_get_clocks(enum mxc_clocks clk);

/*!
 * This function returns the divider value for a clock.
 *
 * @param       clk as defined in enum mxc_clocks
 *
 * @return      divider value
 */
unsigned long mxc_get_clocks_div(enum mxc_clocks clk);

/*!
 * This function sets the PLL source for a clock.
 *
 * @param clk     as defined in enum mxc_clocks
 * @param pll_num the PLL that you wish to use as source for this clock
 */
void mxc_set_clocks_pll(enum mxc_clocks clk, enum plls pll_num);
                                                                                               
/*!
 * This function sets the division factor for a clock.
 *
 * @param clk as defined in enum mxc_clocks
 * @param div the division factor to be used for the clock (For SSI & CSI, pass 
 *            in 2 times the expected division value to account for FP vals)
 */
void mxc_set_clocks_div(enum mxc_clocks clk, unsigned int div);

/*!
 * This function returns the peripheral clock dividers.
 * Note that for SSI divider, in order to maintain the accuracy, the returned
 * divider is doubled.
 *
 * @param       clk     peripheral clock as defined in enum mxc_clocks
 *
 * @return      divider value
 */
unsigned long mxc_peri_clock_divider(enum mxc_clocks clk);

/*!
 * This function returns the main clock dividers.
 *
 * @param       clk     peripheral clock as defined in enum mxc_clocks
 *
 * @return      divider value
 */
unsigned long mxc_main_clock_divider(enum mxc_clocks clk);

/*!
 * This function returns the DFS block divider - LFDF value
 *
 * @return      Low Voltage frequency Divider Factor value
 */
unsigned int mxc_get_lfdf_value(void);

/*!
 * This function is called to gate off the individual module clocks
 *
 * @param clks     as defined in enum mxc_clocks
 */
void mxc_clks_disable(enum mxc_clocks clks);

/*!
 * This function is called to enable the individual module clocks
 *
 * @param clks     as defined in enum mxc_clocks
 */
void mxc_clks_enable(enum mxc_clocks clks);
#endif /* __CLOCK_H */
