/*
 * Copyright 2004 Freescale Semiconductor, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __GPIO_H__
#define __GPIO_H__

/*!
 * @defgroup GPIO General Purpose Input Output (GPIO)
 * @ingroup MSL
 */

/*!
 * @file gpio.h
 * @brief This file contains the GPIO and IOMUX configuration details.
 *
 * @ingroup GPIO
 */

#include <asm/sizes.h>
#include <asm/arch/platform.h>
#include <linux/interrupt.h>

/*!
 * @name IOMUX/PAD Bit field definitions
 */

/*! @{ */

/*!
 * This enumeration is constructed based on the IOMUX chapter of the Zeus
 * Reference Manual.
 */
enum iomux_pins {
    /* IOMUX_AP pins */
    AD1_RXD_PIN = 0,
    AD1_TXC_PIN,
    AD1_TXD_PIN,
    AD1_TXFS_PIN,
    AD2_RXD_PIN,
    AD2_TXC_PIN,
    AD2_TXD_PIN,
    AD2_TXFS_PIN,
    AD3_RXC_PIN,
    AD3_RXD_PIN,
    AD3_RXFS_PIN,
    AD3_TXC_PIN,
    AD3_TXD_PIN,
    AD3_TXFS_PIN,
    ALE_PIN,
    BMOD0_PIN,
    BMOD1_PIN,
    BMOD2_PIN,
    BSY_B_PIN,
    CE_B_PIN,
    CLE_PIN,
    CSI_D0_PIN,
    CSI_D1_PIN,
    CSI_D2_PIN,
    CSI_D3_PIN,
    CSI_D4_PIN,
    CSI_D5_PIN,
    CSI_D6_PIN,
    CSI_D7_PIN,
    CSI_D8_PIN,
    CSI_D9_PIN,
    CSI_HSYNC_PIN,
    CSI_PIXCLK_PIN,
    CSI_VSYNC_PIN,
    ED_INT0_PIN,
    ED_INT1_PIN,
    ED_INT2_PIN,
    ED_INT3_PIN,
    ED_INT4_PIN,
    ED_INT5_PIN,
    ED_INT6_PIN,
    ED_INT7_PIN,
    U1_TXD_PIN,
    U1_RXD_PIN,
    IPU_LD17_D0_VSYNC_PIN,
    IPU_D3_CLK_CONTR_PIN,
    IPU_D3_DRDY_SP_PIN,
    IPU_SER_RS_PIN,
    IPU_D2_CS_PIN,
    U1_RTS_B_PIN,
    IPU_REV_D1_CS_PIN,
    IPU_SD_D_PIN,
    IPU_SD_CLK_PIN,
    U1_CTS_B_PIN,
    U2_TXD_PIN,
    U2_RXD_PIN,
    U2_CTS_B_PIN,
    U2_RTS_B_PIN,
    SPI1_SS1_PIN,
    GP_AP_C0_PIN,
    GP_AP_C1_PIN,
    GP_AP_C10_PIN,
    GP_AP_C11_PIN,
    ICAP1_PIN,
    ICK_AP_PIN,
    OC1_PIN,
    OC2_PIN,
    OC3_PIN,
    ICAP2_PIN,
    GP_AP_C2_PIN,
    CSI_MCLK_PIN,
    IPU_D3_VSYNC_BE0_PIN,
    IPU_D3_HSYNC_BE1_PIN,
    IPU_LD16_D0_CS_PIN,
    IPU_D3_CONTR_PAR_RS_PIN,
    GP_AP_C3_PIN,
    IPU_D3_SPL_RD_PIN,
    IPU_D3_CLS_WR_PIN,
    GP_AP_C4_PIN,
    GP_AP_C5_PIN,
    GP_AP_C6_PIN,
    GP_AP_C7_PIN,
    GP_AP_C8_PIN,
    GP_AP_C9_PIN,
    I2CLK_PIN,
    I2DAT_PIN,
    IPU_LD0_PIN,
    IPU_LD1_PIN,
    IPU_LD10_PIN,
    IPU_LD11_PIN,
    IPU_LD12_PIN,
    IPU_LD13_PIN,
    IPU_LD14_PIN,
    IPU_LD15_PIN,
    IPU_LD2_PIN,
    IPU_LD3_PIN,
    IPU_LD4_PIN,
    IPU_LD5_PIN,
    IPU_LD6_PIN,
    IPU_LD7_PIN,
    IPU_LD8_PIN,
    IPU_LD9_PIN,
    KPCOL0_PIN,
    KPCOL1_PIN,
    KPCOL2_PIN,
    KPCOL3_PIN,
    KPCOL4_PIN,
    KPCOL5_PIN,
    KPCOL6_PIN,
    KPCOL7_PIN,
    KPROW0_PIN,
    KPROW1_PIN,
    KPROW2_PIN,
    KPROW3_PIN,
    KPROW4_PIN,
    KPROW5_PIN,
    KPROW6_PIN,
    KPROW7_PIN,
    OWDAT_PIN,
    RE_B_PIN,
    SPI1_CLK_PIN,
    SPI1_MISO_PIN,
    SPI1_MOSI_PIN,
    SPI1_SS0_PIN,
    WE_B_PIN,
    WP_B_PIN,

    /* IOMUX_COM pins */
    USB_DAT_VP_PIN,
    USB_SE0_VM_PIN,
    UH2_OVR_PIN,
    UH2_PWR_PIN,
    UH1_TXOE_B_PIN,
    UH1_SPEED_PIN,
    UH1_SUSPEND_PIN,
    UH1_TXDP_PIN,
    UH1_TXDM_PIN,
    USB_TXOE_B_PIN,
    UH1_RXD_PIN,
    UH1_RXDP_PIN,
    SD1_CMD_PIN,
    SD1_CLK_PIN,
    SD1_MMC_PU_CTRL_PIN,
    UH1_RXDM_PIN,
    SD2_CMD_PIN,
    SD2_CLK_PIN,
    SD2_MMC_PU_CTRL_PIN,
    SPI2_CLK_PIN,
    SPI2_MOSI_PIN,
    SPI2_MISO_PIN,
    SPI2_SS0_PIN,
    UH1_OVR_PIN,
    SPI2_SS1_PIN,
    UH1_PWR_PIN,
    U3CE_DSR_B_PIN,
    U3CE_RI_B_PIN,
    U3CE_DCD_B_PIN,
    U3CE_DTR_B_PIN,
    MDO0_PIN,
    MDO1_PIN,
    MDO10_PIN,
    MDO11_PIN,
    MDO12_PIN,
    MDO13_PIN,
    MDO14_PIN,
    MDO15_PIN,
    MDO2_PIN,
    MDO3_PIN,
    MDO4_PIN,
    MDO5_PIN,
    MDO6_PIN,
    MDO7_PIN,
    MDO8_PIN,
    MDO9_PIN,
    SD1_DAT0_PIN,
    SD1_DAT1_PIN,
    SD1_DAT2_PIN,
    SD1_DAT3_PIN,
    SD2_DAT0_PIN,
    SD2_DAT1_PIN,
    SD2_DAT2_PIN,
    SD2_DAT3_PIN,
    SIM_CLK_PIN,
    SIM_PD_PIN,
    SIM_RST_B_PIN,
    SIM_SVEN_PIN,
    SIM_TRXD_PIN,
    U3CE_CTS_B_PIN,
    USB_RXD_PIN,
 
    AP_MAX_PIN = WP_B_PIN,
};

/*! @} End IOMUX Bit field definitions */

/*!
 * IOMUX PAD Groups
 */
enum iopad_group {

    /* IOMUX_AP pad groups */
    PAD_GROUP10 = 0,
    PAD_GROUP11,
    PAD_GROUP12,
    PAD_GROUP13,
    PAD_GROUP14,
    PAD_GROUP15,
    PAD_GROUP16,
    PAD_GROUP17,
    PAD_GROUP18,
    PAD_GROUP19,
    PAD_GROUP20,
    PAD_GROUP21,
    PAD_GROUP22,
    PAD_GROUP23,
    PAD_GROUP24,
    PAD_GROUP25,
    PAD_GROUP26,
    PAD_GROUP27,
    PAD_GROUP28,
    PAD_GROUP29,
    PAD_GROUP30,
    PAD_GROUP31,
    PAD_GROUP32,
    PAD_GROUP33,
    PAD_GROUP34,
    PAD_GROUP35,
    PAD_GROUP36,
    PAD_GROUP37,
    PAD_GROUP38,
    PAD_GROUP39,
    PAD_GROUP40,
    PAD_GROUP41,
    PAD_GROUP42,
    PAD_GROUP43,
    PAD_GROUP44,
    PAD_GROUP45,
    PAD_GROUP46,
    PAD_GROUP47,
    PAD_GROUP48,
    PAD_GROUP49,
    PAD_GROUP50,
    PAD_GROUP51,
    PAD_GROUP52,
    PAD_GROUP53,
    PAD_GROUP54,
    PAD_GROUP55,
    PAD_GROUP56,
    PAD_GROUP57,
    PAD_GROUP58,
    
    /* IOMUX_COM pad groups */    
    PAD_GROUP1,
    PAD_GROUP2,
    PAD_GROUP3,
    PAD_GROUP4,
    PAD_GROUP5,
    PAD_GROUP6,
    PAD_GROUP7,
    PAD_GROUP8,
    PAD_GROUP9,
    PAD_GROUP59,
    PAD_GROUP60,
    PAD_GROUP61,
    PAD_GROUP62,
    PAD_GROUP63,
    PAD_GROUP64,
    PAD_GROUP65,
    PAD_GROUP66,
    PAD_GROUP67,
    PAD_GROUP68,
    PAD_GROUP69,
    PAD_GROUP70,
    PAD_GROUP71,
    PAD_GROUP73,
    PAD_GROUP74,
    PAD_GROUP75,
    PAD_GROUP99,
    PAD_GROUP100,
    PAD_GROUP101,
    PAD_GROUP102,
    PAD_GROUP103,

    AP_MAX_PAD_GROUP = PAD_GROUP58,
};

/*!
 * various IOMUX output functions
 */
enum iomux_output_config {
        GPIO_MUX1_OUT = (0 << 4),
        MUX0_OUT =      (1 << 4),
        MUX2_OUT =      (2 << 4),
        MUX3_OUT =      (3 << 4),
        MUX4_OUT =      (4 << 4),
        MUX5_OUT =      (5 << 4),
        MUX6_OUT =      (6 << 4),
        MUX7_OUT =      (7 << 4),
};

/*!
 * various IOMUX input functions
 */
enum iomux_input_config {
        NONE_IN =       0,
        GPIO_MUX1_IN =  (1 << 0),
        MUX0_IN =       (1 << 1),
        MUX2_IN =       (1 << 2),
        MUX3_IN =       (1 << 3),
};

/*!
 * various IOMUX pad functions
 */
enum iomux_pad_config {
        SRE_FAST        = 0x1 << 0,
        SRE_SLOW        = 0x0 << 0,
        DSE_NORMAL      = 0x0 << 1,
        DSE_HIGH        = 0x1 << 1,
        DSE_MAX         = 0x2 << 1,
        DSE_MIN         = 0x3 << 1,
        ODE_CMOS        = 0x0 << 3,
        ODE_OPEN_DRAIN  = 0x1 << 3,
        PKE_ENABLE      = 0x1 << 7,
        PKE_DISABLE     = 0x0 << 7,
        PUE_ENABLE      = 0x1 << 4,
        PUE_KEEPER      = 0x0 << 4,
        HYS_SCHMITZ     = 0x1 << 8,
        HYS_CMOS        = 0x0 << 8,
        DDR_MODE_DDR    = 0x1 << 9,
        DDR_MODE_CMOS   = 0x0 << 9,
        DDR_INPUT_DDR   = 0x1 << 10,
        DDR_INPUT_CMOS  = 0x0 << 10,
};

/*!
 * This function is used to configure a pin through the IOMUX module.
 *
 * @param  pin          pin number as defined in \b #iomux_pins
 * @param  out          output function as defined in \b #iomux_output_config
 * @param  in           input function as defined in \b #iomux_input_config
 */
void iomux_config_mux(enum iomux_pins pin, 
                      enum iomux_output_config out, 
                      enum iomux_input_config in);

/*!
 * This function returns the pad value for a IOMUX group.
 *
 * @param  grp          a group number as defined in \b enum \b #iopad_group
 *
 * @return A group pad value
 */
__u16 iomux_get_pad(enum iopad_group grp);

/*!
 * This function configures the pad value for a IOMUX group.
 *
 * @param  grp          a group number as defined in \b enum \b #iopad_group
 * @param  config       the ORed value of elements defined in \b enum \b #iomux_pad_config
 */
void iomux_set_pad(enum iopad_group grp, __u16 config);

/* gpio related defines */

/*!
 * There are two queues for registered GPIO ISRs. One is for high priority and
 * the other is for low priority. The ISRs in the high priority queue will be
 * called first before the low priority queue if more than one GPIO interrupt
 * occurs at the same time.
 */
enum gpio_prio {
        GPIO_HIGH_PRIO  = 0,     /*!< high priority queue */
        GPIO_LOW_PRIO            /*!< low priority queue */
};

/*!
 * This enumeration data type defines various different ways for interrupting 
 * the ARM core from GPIO signals. The way to interrupt the core is dictated 
 * by the external hardware.
 */
enum gpio_int_cfg {
        GPIO_INT_LOW_LEV        = 0x0,    /*!< low level sensitive */
        GPIO_INT_HIGH_LEV       = 0x1,    /*!< high level sensitive */
        GPIO_INT_RISE_EDGE      = 0x2,    /*!< rising edge sensitive */
        GPIO_INT_FALL_EDGE      = 0x3,    /*!< falling edge sensitive */
        GPIO_INT_NONE           = 0x4     /*!< No interrupt */
};

/*!
 * This enumeration data type defines various different ways for interrupting 
 * the ARM core from EDIO signals. The way to interrupt the core is dictated 
 * by the external hardware.
 */
enum edio_int_cfg {
        EDIO_INT_LOW_LEV        = 0x0,    /*!< low level sensitive */
        EDIO_INT_RISE_EDGE      = 0x1,    /*!< rise edge sensitive */
        EDIO_INT_FALL_EDGE      = 0x2,    /*!< falling edge sensitive */
        EDIO_INT_BOTH_EDGE      = 0x3,    /*!< both edge sensitive */
        EDIO_INT_NONE           = 0x4     /*!< No interrupt */
};

/*!
 * This enumeration data type defines various signals for interrupting the
 * ARM core from EDIO signals. 
 */
enum edio_sig_no {
        ED_INT0 = 0,
        ED_INT1 = 1,
        ED_INT2 = 2,
        ED_INT3 = 3,
        ED_INT4 = 4,
        ED_INT5 = 5,
        ED_INT6 = 6,
        ED_INT7 = 7,
};

typedef irqreturn_t (*gpio_irq_handler)(int, void *, struct pt_regs *);

/*!
 * This function configures the GPIO signal to be either input or output. For
 * input signals used for generating interrupts for the ARM core, how the
 * interrupts being triggered is also passed in via \a icr. For output signals,
 * the \a icr value doesn't matter.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based)
 * @param  out          #true for output, #false for input
 * @param  icr          value defined in \b #gpio_int_cfg
 */
void gpio_config(__u32 port, __u32 sig_no, bool out, enum gpio_int_cfg icr);

/*!
 * This function sets a GPIO signal value.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based)
 * @param  data         value to be set (only 0 or 1 is valid)
 */
void gpio_set_data(__u32 port, __u32 sig_no, __u32 data);

/*!
 * This function returns the value of the GPIO signal.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based)
 *
 * @return Value of the GPIO signal
 */
__u32 gpio_get_data(__u32 port, __u32 sig_no);

/*!
 * This function returns the value of the GPIO signal pad value.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based)
 *
 * @return Pad value of the GPIO signal
 */
__u32 gpio_get_pad(__u32 port, __u32 sig_no);

/*!
 * This function enables/disables a GPIO signal's interrupt.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based)
 * @param  en           \b true for enabling the interrupt; \b false for disabling the interrupt
 */
void gpio_config_int_en(__u32 port, __u32 sig_no, bool en);

/*!
 * This function checks to see if the GPIO interrupt is enabled for a particular signal.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based)
 *
 * @return \b #true if the interrupt is enabled for the GPIO signal; \b #false otherwise.
 */
bool gpio_int_enabled(__u32 port, __u32 sig_no);

/*!
 * This function clears the GPIO interrupt for a signal on a port.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based) to clear
 */
void gpio_clear_int(__u32 port, __u32 sig_no);

/*!
 * This function is responsible for registering a GPIO signal's ISR.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based)
 * @param  prio         priority as defined in \b enum \b #gpio_prio
 * @param  handler      GPIO ISR function pointer for the GPIO signal
 * @param  irq_flags    irq flags (not used)
 * @param  devname      device name associated with the interrupt
 * @param  dev_id       some unique information for the ISR
 *
 * @return 0 if successful; non-zero otherwise.
 */
int gpio_request_irq(__u32 port, __u32 sig_no, enum gpio_prio prio,
                     gpio_irq_handler handler, __u32 irq_flags, 
                     const char * devname, void *dev_id);

/*!
 * This function un-registers an ISR with the GPIO interrupt module.
 *
 * @param  port         specified port with 0-GPIO port 1; 1-GPIO port 2
 * @param  sig_no       specified GPIO signal (0 based)
 * @param  prio         priority as defined in \b enum \b #gpio_prio
 */
void gpio_free_irq(__u32 port, __u32 sig_no, enum gpio_prio prio);

/*!
 * This function configures the EDIO signal to be either input or output. For
 * input signals used for generating interrupts for the ARM core, how the
 * interrupts being triggered is also passed in via \a icr. For output signals
 * the \a icr value doesn't matter.
 *
 * @param  sig_no       EDIO signal (0 based) as defined in \b #edio_sig_no
 * @param  out          #true for output, #false for input
 * @param  icr          value defined in \b #edio_int_cfg
 */
void edio_config(enum edio_sig_no sig_no, bool out, enum edio_int_cfg icr);

/*!
 * This function sets a EDIO signal value.
 *
 * @param  sig_no       EDIO signal (0 based) as defined in \b #edio_sig_no
 * @param  data         value to be set (only 0 or 1 is valid)
 */
void edio_set_data(enum edio_sig_no sig_no, __u32 data);

/*!
 * This function returns the value of the EDIO signal.
 *
 * @param  sig_no       EDIO signal (0 based) as defined in \b #edio_sig_no
 *
 * @return Value of the EDIO signal (either 0 or 1)
 */
__u32 edio_get_data(enum edio_sig_no sig_no);

/*!
 * This function returns the EDIO edge-trigged interrupt for a signal.
 *
 * @param  sig_no       EDIO signal (0 based) as defined in \b #edio_sig_no 
 */
__u32 edio_get_int(enum edio_sig_no sig_no);

/*!
 * This function clears the EDIO edge-trigged interrupt for a signal.
 *
 * @param  sig_no       EDIO signal (0 based) as defined in \b #edio_sig_no
 */
void edio_clear_int(enum edio_sig_no sig_no);

#endif /* __GPIO_H__ */
