/*
 * Copyright 2005 Freescale Semiconductor, Inc. All Rights Reserved.
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA
 *
 */

#ifndef __PMIC_EXTERNAL_H__
#define __PMIC_EXTERNAL_H__

/*!
 * @defgroup PMIC_RR_DRVRS Roadrunner PMIC Drivers
 */

/*!
 * @defgroup PMIC_RR_CORE Roadrunner Protocol Driver
 * @ingroup PMIC_RR_DRVRS
 */

/*!
 * @file pmic_external.h
 * @brief This file contains interface of Roadrunner protocol driver.
 *
 * @ingroup PMIC_RR_CORE
 */

#include <asm/ioctl.h>

/*!
 * This is the enumeration of versions of PMIC
 */
typedef enum {
        PMIC_ATLAS      = 1,      /*!< Atlas */
        PMIC_ROADRUNNER = 2       /*!< Roadrunner */
} t_pmic_version;

/*!
 * @name IOCTL definations for Roadrunner core driver
 */
/*! @{ */
/*! Read a PMIC register */
#define PMIC_READ_REG          _IOWR('P', 0xa0, void*)
/*! Write a PMIC register */
#define PMIC_WRITE_REG         _IOWR('P', 0xa1, void*)
/*! Subscribe a PMIC interrupt event */
#define PMIC_SUBSCRIBE         _IOWR('P', 0xa2, type_event_notification)
/*! Unscribe a PMIC interrupt event */
#define PMIC_UNSUBSCRIBE       _IOWR('P', 0xa3, type_event_notification)
/*! @} */

/*!
 * This is PMIC registers valid bits
 */
#define PMIC_ALL_BITS          0xFFFFFF


/*!
 * This is the enumeration of register names of Roadrunner
 */
typedef enum {
        /*!
        * REG_ISR
        */
        REG_ISR = 0,
        /*!
        * REG_IMR
        */
        REG_IMR,
        /*!
        * REG_PSTAT
        */
        REG_PSTAT,
        /*!
        * REG_INT_SEL
        */
        REG_INT_SEL,
        /*!
        * REG_SWCTRL
        */
        REG_SWCTRL,
        /*!
        * REG_VREG
        */
        REG_VREG,
        /*!
        * REG_IRQ_TEST
        */
        REG_IRQ_TEST,
        /*!
        * REG_TMOD_TRIM_3
        */
        REG_TMOD_TRIM_3,
        /*!
        * REG_BATT_DAC
        */
        REG_BATT_DAC,
        /*!
        * REG_ADC1
        */
        REG_ADC1,
        /*!
        * REG_ADC2
        */
        REG_ADC2,
        /*!
        * REG_AUD_CODEC
        */
        REG_AUD_CODEC,
        /*!
        * REG_RX_AUD_AMPS
        */
        REG_RX_AUD_AMPS,
        /*!
        * REG_ST_DAC
        */
        REG_ST_DAC,
        /*!
        * REG_RTC_TOD
        */
        REG_RTC_TOD,
        /*!
        * REG_RTC_TODA
        */
        REG_RTC_TODA,
        /*!
        * REG_RTC_DAY
        */
        REG_RTC_DAY,
        /*!
        * REG_RTC_DAYA
        */
        REG_RTC_DAYA,
        /*!
        * REG_RTC_CAL
        */
        REG_RTC_CAL,
        /*!
        * REG_PWRCTRL
        */
        REG_PWRCTRL,
        /*!
        * REG_BUSCTRL
        */
        REG_BUSCTRL,
        /*!
        * REG_BACKLIGHT_1
        */
        REG_BACKLIGHT_1,
        /*!
        * REG_BACKLIGHT_2
        */
        REG_BACKLIGHT_2,
        /*!
        * REG_TC_CONTROL1
        */
        REG_TC_CONTROL1,
        /*!
        * REG_TC_CONTROL2
        */
        REG_TC_CONTROL2,
        /*!
        * REG_ARB_REG
        */
        REG_ARB_REG,
        /*!
        * REG_TX_AUD_AMPS
        */
        REG_TX_AUD_AMPS,
        /*!
        * REG_GP_REG
        */
        REG_GP_REG,
        /*!
        * REG_TEST
        */
        REG_TEST,
        /*!
        * REG_TMOD_CTRL_1
        */
        REG_TMOD_CTRL_1,
        /*!
        * REG_TMOD_CTRL_2
        */
        REG_TMOD_CTRL_2,
        /*!
        * REG_TMOD_CTRL_3
        */
        REG_TMOD_CTRL_3,
        /*!
        * REG_RX_STATIC1
        */
        REG_RX_STATIC1,
        /*!
        * REG_RX_STATIC2
        */
        REG_RX_STATIC2,
        /*!
        * REG_RX_STATIC3
        */
        REG_RX_STATIC3,
        /*!
        * REG_RX_STATIC4
        */
        REG_RX_STATIC4,
        /*!
        * REG_RX_DYNAMIC
        */
        REG_RX_DYNAMIC,
        /*!
        * REG_TX_DYNAMIC
        */
        REG_TX_DYNAMIC,
        /*!
        * REG_TX_DYNAMIC2
        */
        REG_TX_DYNAMIC2,
        /*!
        * REG_TX_STATIC1
        */
        REG_TX_STATIC1,
        /*!
        * REG_TX_STATIC2
        */
        REG_TX_STATIC2,
        /*!
        * REG_TX_STATIC3
        */
        REG_TX_STATIC3,
        /*!
        * REG_TX_STATIC4
        */
        REG_TX_STATIC4,
        /*!
        * REG_TX_STATIC5
        */
        REG_TX_STATIC5,
        /*!
        * REG_TX_STATIC6
        */
        REG_TX_STATIC6,
        /*!
        * REG_TX_STATIC7
        */
        REG_TX_STATIC7,
        /*!
        * REG_TX_STATIC8
        */
        REG_TX_STATIC8,
        /*!
        * REG_TX_STATIC9
        */
        REG_TX_STATIC9,
        /*!
        * REG_TX_STATIC10
        */
        REG_TX_STATIC10,
        /*!
        * REG_TX_STATIC11
        */
        REG_TX_STATIC11,
        /*!
        * REG_TX_STATIC12
        */
        REG_TX_STATIC12,
        /*!
        * REG_TX_STATIC13
        */
        REG_TX_STATIC13,
        /*!
        * REG_TX_STATIC14
        */
        REG_TX_STATIC14,
        /*!
        * REG_TX_STATIC15
        */
        REG_TX_STATIC15,
        /*!
        * REG_TX_STATIC16
        */
        REG_TX_STATIC16,
        /*!
        * REG_TX_STATIC17
        */
        REG_TX_STATIC17,
        /*!
        * REG_TX_STATIC18
        */
        REG_TX_STATIC18,
        /*!
        * REG_TONE_GEN
        */
        REG_TONE_GEN,
        /*!
        * REG_GPS_REG
        */
        REG_GPS_REG,
        /*!
        * REG_TMODE_TRIM_1
        */
        REG_TMODE_TRIM_1,
        /*!
        * REG_TMODE_TRIM_2
        */
        REG_TMODE_TRIM_2,
        /*!
        * REG_T_RD_ERROR
        */
        REG_T_RD_ERROR,
        /*!
        * REG_T_RD_NPCOUNT
        */
        REG_T_RD_NPCOUNT,
        /*!
        * REG_ADC2_ALL
        */
        REG_ADC2_ALL,
        /*!
        * REG_NB
        */
        REG_NB
} roadrunner_reg;

/*!
 * Generic PMIC register enumeration type.
 */
typedef roadrunner_reg pmic_reg;

/*!
 * This is event list of Roadrunner interrupt
 */

typedef enum {
         /*!
         * completion of the 8 programmed A/D conversions in standard
         * operation
         */
         EVENT_ADCDONEI = 0,
         /*!
         * touchscreen press
         */
         EVENT_TSI = 1,
         /*!
         * interrupt is from the 1Hz output.
         */
         EVENT_1HZI = 2,
         /*!
         * A/D word read in ADC digital comparison mode exceeding the
         * WHIGH[5:0] word.
         */
         EVENT_WHI = 3,
         /*!
         * A/D word read in ADC digital comparison mode reading below
         * the WLOW[5:0] word.
         */
         EVENT_WLI = 4,
         /*!
         * RTC_TOD = RTC_TODA; RTC_DAY = RTC_DAYA
         */
         EVENT_TODAI = 5,
         /*!
         * occurs on rising and falling debounced edges of USBDET_4.4V.
         */
         EVENT_USB_44VI = 6,
         /*!
         * ON/OFF button was pressed.
         */
         EVENT_ONOFFI = 7,
         /*!
         * ON/OFF2 button was pressed.
         */
         EVENT_ONOFF2I = 8,
         /*!
         * interrupt occurs on rising and falling edges of USBDET_0.8V
         */
         EVENT_USB_08VI = 9,
         /*!
         * interrupt occurs on rising and falling debounced edges of
         * MOBPORTB (EXT B+).
         */
         EVENT_MOBPORTI = 10,
         /*!
         * Interrupt linked to PTT_DET , to be debounced on both edges.
         */
         EVENT_PTTI = 11,
         /*!
         * Triggered on debounced transition of A1_INT
         */
         EVENT_A1I = 12,
         /*!
         * Power Cut transition occurred when PCEN=1 and B+ was
         * re-applied before the Power Cut timer expired
         */
         EVENT_PCI = 14,
         /*!
         * warm start to the MCU
         */
         EVENT_WARMI = 15,
         /*!
         * End of Life (low battery shut off)
         */
         EVENT_EOLI = 16,
         /*!
         * positive or negative edge of CLK_STAT
         */
         EVENT_CLKI = 17,
         /*!
         * interrupt occurs on rising and falling debounced edges of
         * USBDET_2.0V.
         */
         EVENT_USB_20VI = 18,
         /*!
         * Interrupt generated from the de-bounced output of the USB
         * ID-detect comparator.
         */
         EVENT_AB_DETI = 19,
         /*!
         * completion of the 7 programmed A/D conversions in standard
         * operation.
         */
         EVENT_ADCDONE2I = 20,
         /*!
         * Will be set (1) only if SYS_RST_MODE bits = 10 and (2) the
         * BATT_DET_IN/SYS_RESTART input was asserted for the minimum
         * debounce time.
         */
         EVENT_SOFT_RESETI = 21,
         /*!
         * Number of events.
         */
         EVENT_NB,
} type_event;

/*!
 * This enumeration defines all priorities of atlas request.
 */
typedef enum {
        /*!
         * PMIC internal access priority
         */
        PRIO_CORE=0,
        /*!
         * PMIC ADC management priority
         */
        PRIO_ADC,
        /*!
         * PMIC Power management priority
         */
        PRIO_PWM,
        /*!
         * PMIC Battery management priority
         */
        PRIO_BATTERY,
        /*!
         * PMIC Audio priority
         */
        PRIO_AUDIO,
        /*!
         * PMIC Connectivity priority
         */
        PRIO_CONN,
        /*!
         * PMIC Light priority
         */
        PRIO_LIGHT,
        /*!
         * PMIC GPS priority
         */
        PRIO_GPS,
} t_prio;

/*!
 * This structure is used to define an event.
 */
typedef struct  {
        /*!
         * event type
         */
        type_event event;
        /*!
         * call back function with parameter, called when event occur
         */
        void (*callback)(void*);
        /*!
         * call back parameter
         */
        void* param;
} type_event_notification;

/*!
 * This structure is used with IOCTL.
 * It defines register, register value, register mask and event number
 */
typedef struct {
        /*!
         * register number
         */
        int reg;
        /*!
         * value of register
         */
        unsigned int reg_value;
        /*!
         * mask of bits, only used with PMIC_WRITE_REG
         */
        unsigned int reg_mask;
        /*!
         * event number
         */
        unsigned int event;
} register_info;

/*!
 * Define bool type if it is not defined.
 */
#ifndef bool
#define bool int
#endif

/*!
 * This enumeration all senses of Roadrunner.
 */
typedef enum {
        /*!
         * Status of USB 4.4V Comparator
         */
        SENSOR_USBDET_44V = 0,
        /*!
         * The logic output of ONOFFSNS
         */
        SENSOR_ONOFFSNS,
        /*!
         * The logic output of ONOFFSNS2
         */
        SENSOR_ONOFFSNS2,
        /*!
         * Status of USB 0.8V Comparator
         */
        SENSOR_USBDET_08V,
        /*!
         * The power up state of the radio
         */
        SENSOR_MOBSNSB,
        /*!
         * Status of PTT_DET pin
         */
        SENSOR_PTTSNS,
        /*!
         * Status of A1_INT pin
         */
        SENSOR_A1SNS,
        /*!
         * Status of USB 2.0V Comparator
         */
        SENSOR_USBDET_20V,
        /*!
         * Current state of EOL comparator output
         */
        SENSOR_EOL_STAT,
        /*!
         * 32 kHz external oscillation
         */
        SENSOR_CLK_STAT,
        /*!
         * System reset
         */
        SENSOR_SYS_RST,
        /*!
         * Warm system reset
         */
        SENSOR_WARM_SYS_RST,
        /*!
         * State of the BATT_DET_IN/SYS_RST input
         */
        SENSOR_BATT_DET_IN_SNS,
} t_sensor;


/*!
 * This structure is used to read all sense bits of Roadrunner.
 */
typedef struct  {
        /*!
         * Status of USB 4.4V Comparator
         */
        bool usbdet_44v;
        /*!
         * The logic output of ONOFFSNS
         */
        bool onoffsns;
        /*!
         * The logic output of ONOFFSNS2
         */
        bool onoffsns2;
        /*!
         * Status of USB 0.8V Comparator
         */
        bool usbdet_08v;
        /*!
         * The power up state of the radio
         */
        bool mobsnsb;
        /*!
         * Status of PTT_DET pin
         */
        bool pttsns;
        /*!
         * Status of A1_INT pin
         */
        bool a1sns;
        /*!
         * Status of USB 2.0V Comparator
         */
        bool usbdet_20v;
        /*!
         * Current state of EOL comparator output
         */
        bool eol_stat;
        /*!
         * 32 kHz external oscillation
         */
        bool clk_stat;
        /*!
         * System reset
         */
        bool sys_rst;
        /*!
         * Warm system reset
         */
        bool warm_sys_rst;
        /*!
         * State of the BATT_DET_IN/SYS_RST input
         */
        bool batt_det_in_sns;
} t_sensor_bits;

/* EXPORTED FUNCTIONS */

/*!
 * This function returns the PMIC version in system.
 *
 * @return       This function returns PMIC version.
 */
t_pmic_version pmic_get_version(void);

/*!
 * This function is called by PMIC clients to read a register on PMIC.
 *
 * @param        priority   priority of access
 * @param        reg        number of register
 * @param        reg_value   return value of register
 *
 * @return       This function returns PMIC_SUCCESS if successful.
 */
PMIC_STATUS pmic_read_reg(t_prio priority, int reg, unsigned int *reg_value);

/*!
 * This function is called by PMIC clients to write a register on Roadrunner.
 *
 * @param        priority   priority of access
 * @param        reg        number of register
 * @param        reg_value  New value of register
 * @param        reg_mask   Bitmap mask indicating which bits to modify
 *
 * @return       This function returns PMIC_SUCCESS if successful.
 */
PMIC_STATUS pmic_write_reg(t_prio priority,int reg, unsigned int reg_value, unsigned int reg_mask);

/*!
 * This function is called by PMIC clients to subscribe on an event.
 *
 * @param        event_sub   structure of event, it contains type of event and callback
 *
 * @return       This function returns PMIC_SUCCESS if successful.
 */
PMIC_STATUS pmic_event_subscribe(type_event_notification event_sub);

/*!
* This function is called by PMIC clients to un-subscribe on an event.
*
* @param        event_unsub   structure of event, it contains type of event and callback
*
* @return       This function returns PMIC_SUCCESS if successful.
*/
PMIC_STATUS pmic_event_unsubscribe(type_event_notification event_unsub);

/*!
* This function is called to initialize an event.
*
* @param        event  structure of event.
*
* @return       This function returns PMIC_SUCCESS if successful.
*/
PMIC_STATUS pmic_event_init(type_event_notification *event);

/*!
* This function is called to read all sensor bits of PMIC.
*
* @param        sensor    Sensor to be checked.
*
* @return       This function returns true if the sensor bit is high;
*               or returns false if the sensor bit is low.
*/
bool pmic_check_sensor(t_sensor sensor);

/*!
* This function checks one sensor of PMIC.
*
* @param        sensor_bits  structure of all sensor bits.
*
* @return       This function returns PMIC_SUCCESS if successful.
*/
PMIC_STATUS pmic_get_sensors(t_sensor_bits *sensor_bits);

#endif  /* __PMIC_EXTERNAL_H__ */
