#ifndef KSYMBOL_H
#define KSYMBOL_H
/*
 * ksymbol.h -- Set/Get a value to/from kernel symbol.
 *
 * Copyright 2006 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Should you need to contact me, the author, you can do so either by
 * e-mail - mail your message to <vojtech@suse.cz>, or by paper mail:
 * Vojtech Pavlik, Ucitelska 1576, Prague 8, 182 00 Czech Republic
 */

#include <asm/ioctl.h>

/* ex. get a "loops_per_jiffy" symbol value
  unsigned	lpj = 0;
  ksymbol_io_t	param = {
	.symbol = "loops_per_jiffy",
	.offset = 0,
	.buf = &lpj,
	.bufsize = sizeof(lpj)
  };
  ioctl(fd, KSYMBOL_GET, &param);
  printf("loops_per_jiffy = %u\n", lpj);
*/

/* Set/Get a value to kernel symbol */
#define KSYMBOL_SET		_IOW('k', 0x01, ksymbol_io_t)
#define KSYMBOL_GET		_IOR('k', 0x02, ksymbol_io_t)

typedef struct {
	const char	*symbol;	/* kernel or driver symbol name */
	size_t		offset;		/* offset from symbol */
	void		*buf;		/* read or write buffer */
	size_t		bufsize;	/* buffer size */
} ksymbol_io_t;

#define KSYMBOL_GET_JIFFIES		_IOR('k', 0x03, unsigned)

#endif	/* KSYMBOL_H */
