/*
 *  File Name       : linux/sound/oss/mp200-audio.h
 *  Function        : Common audio handling for the MP200
 *  Release Version : Ver 1.01
 *  Release Date    : 2006/03/29
 *
 *  Copyright (C) NEC Electronics Corporation 2005-2006
 *
 *
 *  This program is free software;you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by Free Softwere Foundation; either version 2
 *  of License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warrnty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with this program;
 *  If not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 02111-1307, USA.
 *
 */

#ifndef __SOUND_OSS_MP200_AUDIO_H
#define __SOUND_OSS_MP200_AUDIO_H

#define MP201_DSP_CPU 1
#define MP201_FIFO_STOP 1
#define MP201_CODEC_32 1
#define MP201_PADDING_ON 1
#define MP200_AUDIO_SINGLE_TRANSFER 1
#define MP200_AUDIO_SINGLE_TRANSFER_2 1
#define MP200_AUDIO_SINGLE_TRANSFER_3 1
#define MP200_AUDIO_SINGLE_TRANSFER_4 1
#define MP201_SINGLE_SPECIAL 1
#define MP201_SINGLE_SPECIAL_2 1
#define MP200_AUDIO_PCM_NONERR 1
#define MP200_AUDIO_PCM_NONFRE 1

#include <asm/dma.h>
#include <asm/arch/pcm_ioctl.h>

#define PCM0_ADDR		IO_ADDRESS(MP200_PCM0_BASE)
#define PCM1_ADDR		IO_ADDRESS(MP200_PCM1_BASE)
#define PCM2_ADDR		IO_ADDRESS(MP200_PCM2_BASE)

#define MP200_PCM_REG(PCM, OFF) \
		(*(unsigned int volatile *)((unsigned char *)(PCM) + OFF))

#define PMx_FUNC_SEL(ADDR)	MP200_PCM_REG(ADDR, 0x0000)	/* R/W */
#define PMx_TXRX_EN(ADDR)	MP200_PCM_REG(ADDR, 0x0004)	/* W */
#define PMx_TXRX_DIS(ADDR)	MP200_PCM_REG(ADDR, 0x0008)	/* W */
#define PMx_TXRX_MON(ADDR)	MP200_PCM_REG(ADDR, 0x0004)	/* R */
#define PMx_CYCLE(ADDR)		MP200_PCM_REG(ADDR, 0x000c)	/* R/W */
#define PMx_RAW(ADDR)		MP200_PCM_REG(ADDR, 0x0010)	/* R */
#define PMx_STATUS(ADDR)	MP200_PCM_REG(ADDR, 0x0014)	/* R */
#define PMx_ENSET(ADDR)		MP200_PCM_REG(ADDR, 0x0018)	/* W */
#define PMx_ENCLR(ADDR)		MP200_PCM_REG(ADDR, 0x001c)	/* W */
#define PMx_ENMON(ADDR)		MP200_PCM_REG(ADDR, 0x0018)	/* R */
#define PMx_CLEAR(ADDR)		MP200_PCM_REG(ADDR, 0x0020)	/* W */
#define PMx_TXQ(ADDR)		MP200_PCM_REG(ADDR, 0x0024)	/* R/W */
#define PMx_RXQ(ADDR)		MP200_PCM_REG(ADDR, 0x0028)	/* R */
#ifdef MP201_SINGLE_SPECIAL
#define PMx_FIFO_P(ADDR)	MP200_PCM_REG(ADDR, 0x002c)	/* R */
#endif
#define PMx_CYCLE2(ADDR)	MP200_PCM_REG(ADDR, 0x0030)	/* R/W */

/* PMx_FUNC_SEL Masks */
#define PCM_FUNC_30WORD		0x00000000
#define PCM_FUNC_16WORD		0x00000020
#define PCM_FUNC_08WORD		0x00000040
#define PCM_FUNC_04WORD		0x00000060
#define PCM_FUNC_MASTER		0x00000008
#define PCM_FUNC_SLAVE		0x00000010
#define PCM_FUNC_MODE0		0x00000000
#define PCM_FUNC_MODE1		0x00000001
#define PCM_FUNC_MODE2		0x00000002
#define PCM_FUNC_MODE3		0x00000003
#define PCM_FUNC_MODE4		0x00000004
#define PCM_FUNC_MODE5		0x00000005
#define PCM_FUNC_MODE6		0x00000006

/* PMx_TXRX_EN Masks */
#define PCM_TX_EN		0x01
#define PCM_RX_EN		0x02

/* PMx_INT Masks */
#define PCM_INT_TX_FRE		0x01
#define PCM_INT_TX_URE		0x02
#define PCM_INT_TX_ORE		0x04
#define PCM_INT_TX_WEN		0x08
#define PCM_INT_RX_FRE		0x10
#define PCM_INT_RX_URE		0x20
#define PCM_INT_RX_ORE		0x40
#define PCM_INT_RX_REN		0x80

#define AUDIO_NAME		"audio"
#define AUDIO_NBFRAGS_DEFAULT	16
#define AUDIO_FRAGSIZE_DEFAULT	8192

#define AUDIO_INIT_DATA		0x00
#define AUDIO_NBFRAGS_READWAIT	4
#define AUDIO_OUT		0
#define AUDIO_IN		1

#ifdef CONFIG_MP201
#define PCM_DMA_M2P_INT_SEL	(IO_ADDRESS (MP200_DMAC_BASE) + 0x9100)
#define PCM_DMA_P2M_INT_SEL	(IO_ADDRESS (MP200_DMAC_BASE) + 0xA100)

#define PCM_LCH4_INT_SEL	0x00000300
#endif

/* for rd_ref/wr_ref in audio_state_t */
#define AUDIO_REF_EN		0x00000001	/* open reference */
#define AUDIO_REF_PW		0x00000002	/* power on codec */
#define AUDIO_REF_ERR		0x00000010	/* errer */


/*
 * Buffer Management
 */

typedef struct {
	int offset;		/* current offset */
	char *data;		/* points to actual buffer */
	dma_addr_t dma_addr;	/* physical buffer address */
	int dma_ref;		/* DMA refcount */
} audio_buf_t;

typedef struct pcm_regs {
	unsigned int volatile *const func_sel;	/* R/W */
	unsigned int volatile *const cycle;	/* R/W */
	unsigned int volatile *const txrx_en;	/* W */
	unsigned int volatile *const txrx_dis;	/* W */
	unsigned int volatile *const txrx_mon;	/* R */
	unsigned int volatile *const enset;	/* W */
	unsigned int volatile *const enclr;	/* W */
	unsigned int volatile *const enmon;	/* R */
	unsigned int volatile *const clear;	/* R */
	unsigned int volatile *const status;	/* R */
	unsigned int volatile *const raw;	/* R */
#ifdef MP201_SINGLE_SPECIAL
	unsigned int volatile *const txq;	/* R/W */
	unsigned int volatile *const fifo_p;	/* R */
#endif
	unsigned int volatile *const cycle2;	/* R/W */
} pcm_regs_t;

typedef struct pcm_ctrls {
	pcm_ctrl_t *sett;
	pcm_regs_t *regs;
} pcm_ctrls_t;

typedef struct {
	char *id;		/* identification string */
	audio_buf_t *buffers;	/* pointer to audio buffer structures */
	int dmabufsize;		/* size of dma buffer allocation */
	dma_addr_t dma_top;	/* physical buffer top address */
	u_int usr_head;		/* user fragment index */
	u_int dma_head;		/* DMA fragment index to go */
	u_int dma_tail;		/* DMA fragment index to complete */
	u_int fragsize;		/* fragment i.e. buffer size */
	u_int nbfrags;		/* nbr of fragments i.e. buffers */
	u_int pending_frags;	/* Fragments sent to DMA */
	int dma_channel;	/* DMA channel */
	dma_regs_t *dma_regs;	/* points to our DMA registers */
	spinlock_t pcm_lock;	/* for PCM interrupt */
	unsigned int volatile pcm_intr;	/* Interrupt counter */
	unsigned int volatile pcm_status;	/* Interrupt status */
	pcm_regs_t *pcm_regs;	/* points to our PCM registers */
	unsigned int irq_bit;	/* PCM interruption mask */
	unsigned int cl_bit;	/* PCM interruption mask */
	unsigned int en_bit;	/* PCM interruption mask */
	unsigned int const phys_xq;	/* PCM Data buffer address */
	unsigned int int_dma;	/* DMA interruption mask */
	int bytecount;		/* nbr of processed bytes */
	int fragcount;		/* nbr of fragment transitions */
	struct semaphore sem;	/* account for fragment usage */
	wait_queue_head_t wq;	/* for poll */
	int mapped;		/* mmap()'ed buffers */
	int active;		/* actually in progress */
	int stopped;		/* might be active but stopped */
	unsigned int format;	/* Audio data format */
	unsigned int stereo;	/* 0:mono 1:stereo */
	unsigned int type;	/* Type of this structure (IN/OUT) */
} audio_stream_t;

/*
 * State structure for one instance
 */

typedef struct {
	audio_stream_t *output_stream;
	audio_stream_t *input_stream;
	int rd_ref;		/* open reference for recording */
	int wr_ref;		/* open reference for playback */
	unsigned int pcm_irq;
	void *data;
	void (*hw_init) (audio_stream_t *, audio_stream_t *);
	void (*hw_shutdown) (audio_stream_t *, audio_stream_t *);
	int (*client_ioctl) (struct inode *, struct file *, uint, ulong);
	pcm_ctrls_t *ctrls;
	struct semaphore sem;	/* to protect against races in attach() */
#ifdef MP201_DSP_CPU
	int int_mode;           /* interrupt mode  */
#endif
} audio_state_t;

typedef struct {
	unsigned int func_sel;
	unsigned int cycle;
	unsigned int txrx_mon;
	unsigned int enmon;
	unsigned int cycle2;
} pcm_regs_save_t;

/*
 * Functions exported by this module
 */
extern int mp200_audio_attach(struct inode *inode, struct file *file,
			      audio_state_t * state);
extern int audio_pcm_set_ctrl(audio_state_t * s);
#ifdef MP201_DSP_CPU
extern int audio_dsp_set_ctrl(int val, audio_state_t * s);
#endif

#endif				/* __SOUND_OSS_MP200_AUDIO_H */
