/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Copyright (C) 2004 Freescale Inc Ltd
 *
 * History :
 *        04/06/29 : Creation
 */
  
 /*!
  * @defgroup SOUND_DRV Atlas Sound Driver
  * @ingroup ATLASDRVRS
  */

 /*!
  * @file mxc-ctrls.h
  * @brief external definitions for the mxc sound driver
  * @ingroup SOUND_DRV
  */
  
#ifndef _MXC_CTRLS_H
#define _MXC_CTRLS_H

/*!
 * Allows the user to add extra IOCTL to get access to MXC registers
 * For debug only
 */
#include "mxc-ctrls-dbg.h"

/*!
 * To set the adder configuration, use the audio control 
 * SNDCTL_ATLAS_WRITE_OUT_MIXER.\n 
 * Possible arguments are :
 * @see ATLAS_AUDIO_ADDER_STEREO
 * @see ATLAS_AUDIO_ADDER_STEREO_OPPOSITE
 * @see ATLAS_AUDIO_ADDER_MONO
 * @see ATLAS_AUDIO_ADDER_MONO_OPPOSITE \n 
 * \n To get the adder configuration use :
 * @see SNDCTL_ATLAS_READ_OUT_ADDER
*/
#define SNDCTL_ATLAS_WRITE_OUT_ADDER     _SIOWR('Z', 7, int)

/*!
 * To get the adder configuration, use the audio control 
 * SNDCTL_ATLAS_READ_OUT_MIXER.\n 
 * Possible returned values are :
 * @see ATLAS_AUDIO_ADDER_STEREO
 * @see ATLAS_AUDIO_ADDER_STEREO_OPPOSITE
 * @see ATLAS_AUDIO_ADDER_MONO
 * @see ATLAS_AUDIO_ADDER_MONO_OPPOSITE \n 
 * \n To set the adder configuration use :
 * @see SNDCTL_ATLAS_WRITE_OUT_ADDER
*/
#define SNDCTL_ATLAS_READ_OUT_ADDER        _SIOR('Z', 6, int)

/*! 
 * Argument for the Atlas adder configuration
 * @see SNDCTL_ATLAS_WRITE_OUT_ADDER
 * @see SNDCTL_ATLAS_READ_OUT_ADDER
 */
#define ATLAS_AUDIO_ADDER_STEREO                1
/*! 
 * Argument for the Atlas adder configuration
 * @see SNDCTL_ATLAS_WRITE_OUT_ADDER
 * @see SNDCTL_ATLAS_READ_OUT_ADDER
 */
#define ATLAS_AUDIO_ADDER_STEREO_OPPOSITE       2
/*! 
 * Argument for the Atlas adder configuration
 * @see SNDCTL_ATLAS_WRITE_OUT_ADDER
 * @see SNDCTL_ATLAS_READ_OUT_ADDER
 */
#define ATLAS_AUDIO_ADDER_MONO                  4
/*! 
 * Argument for the Atlas adder configuration
 * @see SNDCTL_ATLAS_WRITE_OUT_ADDER
 * @see SNDCTL_ATLAS_READ_OUT_ADDER
 */
#define ATLAS_AUDIO_ADDER_MONO_OPPOSITE         8

/*!
 * To get the codec balance configuration, use the audio control 
 * SNDCTL_ATLAS_READ_OUT_BALANCE.\n 
 * Range is 0 (-21 dB left) to 100 (-21 dB right), linear, 3dB step ; 
 * 50 is no balance.
 * \n    Examples:
 * \n      0 : -21dB left   50 : balance deactivated   100 : -21 dB right
 * @see SNDCTL_ATLAS_WRITE_OUT_BALANCE 
*/
#define SNDCTL_ATLAS_READ_OUT_BALANCE      _SIOR('Z', 8, int)

/*!
 * To set the codec balance configuration, use the audio control 
 * SNDCTL_ATLAS_WRITE_OUT_BALANCE.\n 
 * Range is 0 (-21 dB left) to 100 (-21 dB right), linear, 3dB step ; 
 * 50 is no balance.
 * \n    Examples:
 * \n      0 : -21dB left   50 : balance deactivated   100 : -21 dB right
 * @see SNDCTL_ATLAS_READ_OUT_BALANCE
 */
#define SNDCTL_ATLAS_WRITE_OUT_BALANCE      _SIOWR('Z', 9, int)

/*!
 * To get the codec bias configuration, use the audio control 
 * SNDCTL_ATLAS_READ_IN_BIAS.\n 
 * Possible returned values are :
 * \n         0 : bias disabled
 * \n         1 : bias enabled
 * @see SNDCTL_ATLAS_WRITE_IN_BIAS
 */
#define SNDCTL_ATLAS_READ_IN_BIAS           _SIOR('Z', 10, int)

/*!
 * To set the codec bias configuration, use the audio control :
 * SNDCTL_ATLAS_WRITE_IN_BIAS.\n 
 * Possible arguments are :
 * \n         0 : to disable the bias
 * \n         1 : to enable the bias
 * @see SNDCTL_ATLAS_READ_IN_BIAS
 */
#define SNDCTL_ATLAS_WRITE_IN_BIAS           _SIOWR('Z', 11, int)

/*!
 * To set the codec filter configuration, use the audio control 
 * SNDCTL_ATLAS_WRITE_CODEC_FILTER.
 * The new configuration replaces the old one.\n 
 * Possible arguments are :
 * @see ATLAS_CODEC_FILTER_DISABLE
 * @see ATLAS_CODEC_FILTER_HIGH_PASS_IN
 * @see ATLAS_CODEC_FILTER_HIGH_PASS_OUT
 * @see ATLAS_CODEC_FILTER_DITHERING \n 
 * \n To get the codec filter configuration, use :
 * @see SNDCTL_ATLAS_READ_CODEC_FILTER
 */
#define SNDCTL_ATLAS_WRITE_CODEC_FILTER      _SIOWR('Z', 20, int)

/*!
 * To get the codec filter configuration, use the audio control :
 * SNDCTL_ATLAS_READ_CODEC_FILTER.
 * The new configuration replaces the old one.\n 
 * Possible returned values are :
 * @see ATLAS_CODEC_FILTER_DISABLE
 * @see ATLAS_CODEC_FILTER_HIGH_PASS_IN
 * @see ATLAS_CODEC_FILTER_HIGH_PASS_OUT
 * @see ATLAS_CODEC_FILTER_DITHERING \n 
 * \n To set the codec filter configuration, use :
 * @see SNDCTL_ATLAS_WRITE_CODEC_FILTER
 */
#define SNDCTL_ATLAS_READ_CODEC_FILTER       _SIOR('Z', 21, int)

/*!
 * Argument for the Atlas codec filter configuration
 * @see SNDCTL_ATLAS_WRITE_CODEC_FILTER 
 * @see SNDCTL_ATLAS_READ_CODEC_FILTER 
 */
#define ATLAS_CODEC_FILTER_DISABLE              0
/*!
 * Argument for the Atlas codec filter configuration
 * @see SNDCTL_ATLAS_WRITE_CODEC_FILTER 
 * @see SNDCTL_ATLAS_READ_CODEC_FILTER 
 */
#define ATLAS_CODEC_FILTER_HIGH_PASS_IN         1
/*!
 * Argument for the Atlas codec filter configuration
 * @see SNDCTL_ATLAS_WRITE_CODEC_FILTER 
 * @see SNDCTL_ATLAS_READ_CODEC_FILTER 
 */
#define ATLAS_CODEC_FILTER_HIGH_PASS_OUT        2
/*!
 * Argument for the Atlas codec filter configuration
 * @see SNDCTL_ATLAS_WRITE_CODEC_FILTER 
 * @see SNDCTL_ATLAS_READ_CODEC_FILTER 
 */
#define ATLAS_CODEC_FILTER_DITHERING            4

/*!
 * Argument for the system audio clocking selection
 * @see MXC_AUDIO_CLOCKING_MCU_MASTER
 * @see SNDCTL_CLK_SET_MASTER
 */
#define MXC_AUDIO_CLOCKING_ATLAS_MASTER         0

/*!
 * Argument for the system audio clocking selection
 * @see MXC_AUDIO_CLOCKING_ATLAS_MASTER
 * @see SNDCTL_CLK_SET_MASTER
 */
#define MXC_AUDIO_CLOCKING_MCU_MASTER           1

/*
 * To set the clock configuration, use the audio control 
 * SNDCTL_CLK_SET_MASTER.\n 
 * Possible arguments are :
 * \n         1 : to MCU master
 * \n         2 : to ATLAS master
 * Possible returned values are :
 * \n         1 : MCU master
 * \n         2 : MCU master
 */
#define SNDCTL_CLK_SET_MASTER                   _SIOR('Z', 30, int)

/*!
 * To set the output port, use the audio control 
 * SNDCTL_DAM_SET_OUT_PORT.\n 
 * Possible returned values are :
 * \n         1 : to port 4
 * \n         2 : to port 5
 * Possible returned values are :
 * \n         1 : port 4
 * \n         2 : port 5
 */
#define SNDCTL_DAM_SET_OUT_PORT                 _SIOR('Z', 31, int)

/*!
 * Argument for the DAM output port selection
 * @see SNDCTL_DAM_SET_OUT_PORT
 * @see MXC_DAM_OUT_PORT_AD2
 */
#define MXC_DAM_OUT_PORT_AD1                    0

/*!
 * Argument for the DAM output port selection
 * @see SNDCTL_DAM_SET_OUT_PORT
 * @see MXC_DAM_OUT_PORT_AD1
 */
#define MXC_DAM_OUT_PORT_AD2                    1

#endif
