/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Copyright (C) 2004 Freescale Inc Ltd
 *
 * History :
 *      04/05/18: Removed the dmabuf structure.
 *      04/08/12: Added dam destination port as a configuration parameter
 *      04/11/19: TLSbo44269: mixer rework
 *      04/11/26: TLSbo45095: Add word_size in the SDMA configuration
 *      05/03/03: TLSbo48139: HW configuration bring up
 */
                                                                                
 /*!
  * @file mxc-defs.h
  * @brief common definitions for the mxc sound driver
  * @ingroup SOUND_DRV
  */
  
#ifndef _MXC_DEFS_H
#define _MXC_DEFS_H

#include <asm/atomic.h>
#include <linux/soundcard.h>
#include "mxc-ctrls.h"
#include <atlas_audio.h>
#include <ssi.h>
#include <dam.h>

/*! This define is necessary when using Virtio models 
#define DRV_FOR_VIRTIO_MODEL */
/*
#define DEBUG 1
*/
/*
 * The flags below can be activated to add traces to dmesg 
 * Allow only one at a time
 */
#ifdef DEBUG
#define DPRINTK(fmt, args...) printk("%s: " fmt, __FUNCTION__, ## args)
#define FUNC_START
#define FUNC_END
#define FUNC_ERR              printk("Operation Failed ! (%s)\n", __FUNCTION__);
#else        /* DEBUG */
#ifdef DEBUG_ALL
#define DPRINTK(fmt, args...) printk("%s: " fmt, __FUNCTION__, ## args)
#define FUNC_START            DPRINTK(KERN_ERR"start\n");
#define FUNC_END              DPRINTK(KERN_ERR"end\n");
#define FUNC_ERR              printk("Operation Failed ! (%s)\n", __FUNCTION__);
#else        /* DEBUG_ALL */
#define FUNC_START
#define FUNC_END
#define FUNC_ERR
#define DPRINTK(fmt, args...)
#endif        /* DEBUG_ALL */
#endif        /*DEBUG*/

#ifdef MORE_DEBUG_CHECKING
#define DBG_INFO(fmt, args...) printk("%s: " fmt, __FUNCTION__, ## args)
#define CHECK_IF_0(x, str) \
        do { \
                if ( (x)!=0 ) DBG_INFO("Error: %s=%i, should be 0\n", str, x);\
        } while (0);
#else
#define DBG_INFO(fmt, args...) 
#define CHECK_IF_0(x, str) 
#endif

/*! This macro checks that the returned value by specified operation is zero
    If not, then goto _err */
#define TRY(a) \
        do { \
                if ( (a) != 0 ) goto _err; \
        } while (0);

/*! This macro checks that the returned value by specified operation is 
    the one specified as an arg. If not, then goto _err */
#define TRYEQ(a, b) \
        do { \
                if ( (a) != (b) ) goto _err; \
        } while (0);

/*! This macro checks that the returned value by specified operation is 
    not the one specified as an arg. If not, then goto _err */
#define TRYNEQ(a, b) \
        do { \
                if ( (a) == (b) ) goto _err; \
        } while (0);

/*! This macro checks that the returned value by specified operation is 
    accepted by the hardware (returned value is zero).
    If yes, then break, else, continue */
#define TRYOK(a) \
        do { \
                if ( (a) == 0 ) goto _next; \
        } while (0);

/*! this macro is accompaigning the TRY macro */
#define CATCH _err

/*
 * The hardware has 2 SSI buses in full-duplex mode to carry audio
 * Capabilities are limitied to 2 play + 1 rec
 */
/*! Number of SSIs connected to MCU */
#define NR_HW_CH                        2
/*! Number of possible record at a time */
#define NR_HW_CH_RX                     1
/*! Number of possible play at a time */
#define NR_HW_CH_TX                     2

/*! Use the DAM port_4 as the default output port */
#define DAM_DEFAULT_OUT_PORT            port_4

/*! Use the SSI fifo #0 as the default fifo */
#define SSI_DEFAULT_FIFO_NB             ssi_fifo_0

/*! sound devices defines for converters (can/cna) */
#define ATLAS_DEVICE_OUT_NONE           0x0
#define ATLAS_DEVICE_IN_NONE            0x0
#define ATLAS_DEVICE_OUT_STDAC          0x1
#define ATLAS_DEVICE_OUT_CODEC          0x2
#define ATLAS_DEVICE_OUT_LINE_IN        0x4
#define ATLAS_DEVICE_IN_CODEC           0x8

/*! Defines for "connected_device" */
#define ATLAS_PHYSICAL_DEVICE_NONE      0x0
#define ATLAS_PHYSICAL_DEVICE_STDAC     0x1
#define ATLAS_PHYSICAL_DEVICE_CODEC     0x2

/* OSS defines to be integrated in soundcard.h - 
    not implemented yet by Atlas drv
#define SOUND_MIXER_IBIAS 25
#define SOUND_MIXER_WRITE_IBIAS         MIXER_WRITE(SOUND_MIXER_IBIAS)
#define SOUND_MIXER_READ_IBIAS          MIXER_READ(SOUND_MIXER_IBIAS) */

#define MXC_STEREO_IN_SRC_MASK   (SOUND_MASK_PHONEIN)
#define MXC_MONO_IN_SRC_MASK     (SOUND_MASK_LINE | SOUND_MASK_MIC)
#define MXC_RECORDING_SRC_MASK   \
        (MXC_STEREO_IN_SRC_MASK | MXC_MONO_IN_SRC_MASK)
#define MXC_STEREO_OUT_SRC_MASK  (SOUND_MASK_VOLUME | SOUND_MASK_PCM)
#define MXC_MONO_OUT_SRC_MASK    (SOUND_MASK_PHONEOUT | SOUND_MASK_SPEAKER)
#define MXC_PLAYING_SRC_MASK     \
        (MXC_STEREO_OUT_SRC_MASK | MXC_MONO_OUT_SRC_MASK)

/*! sound devices defines for Sources (lines) */
#define ATLAS_SRC_OUT_NONE              0
#define ATLAS_SRC_IN_NONE               0
#define ATLAS_SRC_OUT_EARPIECE          SOUND_MASK_PHONEOUT        
#define ATLAS_SRC_OUT_HANDSFREE         SOUND_MASK_SPEAKER
#define ATLAS_SRC_OUT_HEADSET           SOUND_MASK_VOLUME
#define ATLAS_SRC_OUT_LINE_OUT          SOUND_MASK_PCM
#define ATLAS_SRC_IN_HANDSET            SOUND_MASK_PHONEIN
#define ATLAS_SRC_IN_LINE_IN            SOUND_MASK_LINE
#define ATLAS_SRC_IN_HEADSET            SOUND_MASK_MIC

/*! the default sources at driver opening */
#ifdef DRV_FOR_VIRTIO_MODEL
#define ATLAS_DEFAULT_SRC_IN            ATLAS_SRC_IN_LINE_IN
#define ATLAS_DEFAULT_SRC_OUT           ATLAS_SRC_OUT_LINE_OUT
#else
#define ATLAS_DEFAULT_SRC_IN            ATLAS_SRC_IN_HEADSET
#define ATLAS_DEFAULT_SRC_OUT           ATLAS_SRC_OUT_HEADSET
#endif

/*! the default master for audio clocking (MCU or SSI) */
#define CLK_DEFAULT_MASTER              MXC_AUDIO_CLOCKING_MCU_MASTER

/*! Volume coding in Atlas */
#define ATLAS_IN_VOLUME_MIN             0x0    /* -8dB*/
#define ATLAS_OUT_VOLUME_MIN            0x0    /* -39dB */
/*! Volume coding in Atlas */
#define ATLAS_IN_VOLUME_MAX             0x1F   /* +23dB */
#define ATLAS_OUT_VOLUME_MAX            0xF    /* +6dB */
/*! Volume coding in OSS */
#define ATLAS_IN_VOL_CODE_MIN           0      /* -8dB*/
#define ATLAS_OUT_VOL_CODE_MIN          0      /* -39dB */
/*! Volume coding in OSS */
#define ATLAS_IN_VOL_CODE_MAX           100    /* +23dB */
#define ATLAS_OUT_VOL_CODE_MAX          100    /* +6dB */

/*! Balance coding in Atlas */
#define ATLAS_OUT_BAL_MIN               0x0    /* -21dB */
/*! Balance coding in Atlas */
#define ATLAS_OUT_BAL_MAX               0x7    /* 0dB*/
/*! Balance coding in OSS */
#define ATLAS_OUT_BAL_CODE_MIN          0      /* -21dB left*/
/*! Balance coding in OSS */
#define ATLAS_OUT_NO_BAL_CODE           50     /* 0dB*/
/*! Balance coding in OSS */
#define ATLAS_OUT_BAL_CODE_MAX          100    /* -21dB right*/

/* Volume to balance conversion */
#define ATLAS_VOL_TO_BAL_RATIO          ((6+39)/(21+21))

/*! 0<=vol<=100; output: ATLAS_OUT_VOLUME_MIN<=x<=ATLAS_OUT_VOLUME_MAX
    left vol = right vol */
#define CONVERT_OUT_VOL_OSS_TO_HW(vol) \
        ( ((vol)*(ATLAS_OUT_VOLUME_MAX-ATLAS_OUT_VOLUME_MIN)/100) \
        -ATLAS_OUT_VOLUME_MIN )
#define CONVERT_OUT_VOL_HW_TO_OSS(vol) \
        ( ((vol)-ATLAS_OUT_VOLUME_MIN)* \
        (100/(ATLAS_OUT_VOLUME_MAX-ATLAS_OUT_VOLUME_MIN)) )
#define CONVERT_IN_VOL_OSS_TO_HW(vol) \
        ( ((vol)*(ATLAS_IN_VOLUME_MAX-ATLAS_IN_VOLUME_MIN)/100) \
        -ATLAS_IN_VOLUME_MIN )
#define CONVERT_IN_VOL_HW_TO_OSS(vol) \
        ( ((vol)-ATLAS_IN_VOLUME_MIN)* \
        (100/(ATLAS_IN_VOLUME_MAX-ATLAS_IN_VOLUME_MIN)) )

/*! 0<=bal<=100; output: ATLAS_OUT_BAL_MIN<=x<=ATLAS_OUT_BAL_MAX
    0->50 left bal; 50->100 right bal */
#define CONVERT_BAL_OSS_TO_HW(bal) \
        (((bal)*(ATLAS_OUT_BAL_MAX-ATLAS_OUT_BAL_MIN)/50)-ATLAS_OUT_BAL_MIN)
#define CONVERT_BAL_HW_TO_OSS(bal) \
        (((bal)-ATLAS_OUT_BAL_MIN)*(50/(ATLAS_OUT_BAL_MAX-ATLAS_OUT_BAL_MIN)))

/*
 * ARGONPLUS specific definitions
 */
#ifdef CONFIG_ARCH_ARGONPLUS
/* Prescaler modulus base definition for A+ */
#define MXC_SYSTEM_CLK                   49887500

#define _reg_MCU_MCR   (*((volatile unsigned long *) \
                              (IO_ADDRESS(CRM_MCU_BASE_ADDR + 0x00))))
#define _reg_MCU_MPDR1      (*((volatile unsigned long *) \
                              (IO_ADDRESS(CRM_MCU_BASE_ADDR + 0x08))))
#endif /* CONFIG_ARCH_ARGONPLUS */

/*
 * SCMA11 specific definitions
 */
#ifdef CONFIG_ARCH_SCMA11
/* Prescaler modulus base definition for SCMA11 */
#define MXC_SYSTEM_CLK                   24000000

#define _reg_CRM_AP_ASCSR   (*((volatile unsigned long *) \
                              (IO_ADDRESS(CRM_AP_BASE_ADDR + 0x00))))
#define _reg_CRM_AP_ACDER1      (*((volatile unsigned long *) \
                              (IO_ADDRESS(CRM_AP_BASE_ADDR + 0x08))))
#endif /* CONFIG_ARCH_SCMA11 */

/*
 * MX3 specific definitions
 */
#ifdef CONFIG_ARCH_MX3
/* Prescaler modulus base definition for MX3 */
#define MXC_SYSTEM_CLK                   49887500
#endif /* CONFIG_ARCH_MX3 */

/* Prescaler modulus for SCMA11 (DIV2=0, PSR=0) */
#define MXC_PM_BASE_MONO_16BITS         (MXC_SYSTEM_CLK/16/4)

/* Prescaler modulus definitions for Mono/16 bits */
#define MXC_PM_8000                     (MXC_PM_BASE_MONO_16BITS/8000)
#define MXC_PM_11025                    (MXC_PM_BASE_MONO_16BITS/11025)
#define MXC_PM_12000                    (MXC_PM_BASE_MONO_16BITS/12000)
#define MXC_PM_16000                    (MXC_PM_BASE_MONO_16BITS/16000)
#define MXC_PM_22050                    (MXC_PM_BASE_MONO_16BITS/22050)
#define MXC_PM_24000                    (MXC_PM_BASE_MONO_16BITS/24000)
#define MXC_PM_32000                    (MXC_PM_BASE_MONO_16BITS/32000)
#define MXC_PM_44100                    (MXC_PM_BASE_MONO_16BITS/44100)
#define MXC_PM_48000                    (MXC_PM_BASE_MONO_16BITS/48000)
#define MXC_PM_64000                    (MXC_PM_BASE_MONO_16BITS/64000)
#define MXC_PM_96000                    (MXC_PM_BASE_MONO_16BITS/96000)

/* SSI */
#define MXC_SSI_INT_DISABLE_ALL        0xFFFFFFFF

/* Watermark levels */
/*! FIFO SIZE for SSI */
#define SSI_FIFO_SIZE                   8
/*! number of empty slots for Tx Watermark level */
#define TX_FIFO_EMPTY_WML               4
/* number of data for Rx Watermark level */
#define RX_FIFO_FULL_WML                6

/* definitions for other platform configuration */
#define ModifyRegister(a,b,c)   ((c) = ( ( (c)&(~(a)) ) | (b) ))

#define _reg_SSI2_SPBA      (*((volatile unsigned long *) \
                              (IO_ADDRESS(SPBA_CTRL_BASE_ADDR + SPBA_SSI2))))

/*! 
 * @brief holds the information about the Atlas chip 
 *
 */
struct _atlas_state {
        /*! selected device for output (StDAC, CODEC).
            Only one at a time (use of only SSI #1) */
        int out_cna;
        
        /*! selected device for input (CODEC).
            Only one at a time (use of only SSI #1) */
        int in_can;
        
        /*! selected source for output (headset, handset...) */
        output_device out_line;
        
        /*! selected source for input */
        input_device in_line;
        
        /* sound device output parameters */
        int out_volume;                 /*!< output volume. L+R coded */
        int out_balance;                /*!< output balance */
        int out_adder_mode;             /*!< mono adder cfg */

        /*! sound device input parameters (if applicable) */
        int in_volume;                  /*!< input volume. L+R coded */
        int in_bias;                    /*!< input bias */
        
        /*! Atlas config for in and out configuration */
        int in_out_filter;              /*!< codec filtering */
        int in_out_sr;                  /*!< sample rate (8kHz to 96 kHz) */
};

struct _mxc_card;

/*!
 * @brief holds all the information used by a driver instance
 *
 */
struct _mxc_state {
        /*! Card info */
        struct _mxc_card *card;

        /*! soundcore stuff */
        int dev_audio;
        
        /*! Audio device availability - set when playing/recording */
        int device_busy;

        /*! SSI availability - Set when the instance is used */
        int ssi_busy;

        /*! ssi channel number */
        ssi_mod ssi_index;

        /*! used ssi fifo number */
        fifo_nb ssi_fifo_nb;
        
        /*! the stream channel number (same for input and output) */
        int in_out_channel_num;

        /*! the stream bit depth: 8 or 16 bits allowed */
        int in_out_bits_per_sample;
        
        /*! Indicates whether the can is used or not */
        int using_can;

        /*! Indicates whether the cna is used or not */
        int using_cna;

        /*! the base index for the (adjacent, if stereo) ssi timeslot (TX) */
        int ssi_tx_timeslot;
        
        /*! the base index for the (adjacent, if stereo) ssi timeslot (RX) */
        int ssi_rx_timeslot;
        
        /*! dam port destination */
        dam_port dam_dest;

        /*!  */
        int judge[2];
#define J_PLAY 0
#define J_REC 1

        /* single open lock mechanism, only used for recording */
        wait_queue_head_t open_wait;

        /*! file opening mode */
        int open_mode;

        /*! DMA channels information */
        int dma_playbuf;
        int dma_recbuf;

        /*! Audio clocking mastership */
        int clock_master;
        
        /*! device index */
        int dev_index;
};

/*!
 * @brief holds all the information used by all sound driver instances
 *
 */
struct _mxc_card {
        /*! instance data of opened /dev/dsp */
        struct _mxc_state *state[NR_HW_CH];

        /*! drv instance, play and rec counters */
        int open_cnt, play_cnt, rec_cnt;
        
        /*! The physical device connected to ARM core */
        int connected_device;

        /*! mixer stuff */
        int dev_mixer;

        /*! Audio chip info */
        struct _atlas_state audio_chip;

        /*! inter-instance race cond. */
        spinlock_t drv_inst_lock;

        /*! Open/Close reentrance management */
        struct semaphore open_close_sema;

        /*! */
        struct semaphore mutex;
        
        /*! This variable holds whether we are using one or 
         *two SSI's simultaneously
         */
        int ssi_ongoing;
};

typedef struct _mxc_card mxc_card_t;
typedef struct _mxc_state mxc_state_t;
typedef struct _atlas_state atlas_state_t;

/*!
 * @brief gives the direction of an audio channel
 *
*/
typedef enum {
        eDIR_TX_PLAY=0, /*!< 0=out */
        eDIR_RX_REC,    /*!< 1= in */
        eDIR_BOTH       /*!< full-duplex */
} e_transfer_dir_t;

#endif
